#pragma once

#include "td/tl/TlObject.h"

#include <cstdint>
#include <utility>
#include <vector>

#include <jni.h>

#include <string>

namespace td {
class TlStorerToString;

namespace td_api {

using int32 = std::int32_t;
using int53 = std::int64_t;
using int64 = std::int64_t;

using string = std::string;

using bytes = std::string;

template <class Type>
using array = std::vector<Type>;

class Object;
using BaseObject = Object;

template <class Type>
using object_ptr = ::td::tl_object_ptr<Type>;

template <class Type, class... Args>
object_ptr<Type> make_object(Args &&... args) {
  return object_ptr<Type>(new Type(std::forward<Args>(args)...));
}

template <class ToType, class FromType>
object_ptr<ToType> move_object_as(FromType &&from) {
  return object_ptr<ToType>(static_cast<ToType *>(from.release()));
}

std::string to_string(const BaseObject &value);

template <class T>
std::string to_string(const object_ptr<T> &value) {
  if (value == nullptr) {
    return "null";
  }

  return to_string(*value);
}

template <class T>
std::string to_string(const std::vector<object_ptr<T>> &values) {
  std::string result = "{\n";
  for (const auto &value : values) {
    if (value == nullptr) {
      result += "null\n";
    } else {
      result += to_string(*value);
    }
  }
  result += "}\n";
  return result;
}

const char *&get_package_name_ref();
const char *get_git_commit_hash();

class Object {
 public:
  virtual ~Object() {
  }

  virtual void store(JNIEnv *env, jobject &s) const {
  }

  virtual void store(TlStorerToString &s, const char *field_name) const = 0;

  static jclass Class;

  virtual std::int32_t get_id() const = 0;

  static object_ptr<Object> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class Function: public Object {
 public:
  static jclass Class;

  static object_ptr<Function> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class accentColor final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int32 built_in_accent_color_id_;
  static jfieldID built_in_accent_color_id_fieldID;
  array<int32> light_theme_colors_;
  static jfieldID light_theme_colors_fieldID;
  array<int32> dark_theme_colors_;
  static jfieldID dark_theme_colors_fieldID;
  int32 min_channel_chat_boost_level_;
  static jfieldID min_channel_chat_boost_level_fieldID;

  accentColor();

  accentColor(int32 id_, int32 built_in_accent_color_id_, array<int32> &&light_theme_colors_, array<int32> &&dark_theme_colors_, int32 min_channel_chat_boost_level_);

  static const std::int32_t ID = -496870680;

  static object_ptr<accentColor> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class acceptedGiftTypes final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool unlimited_gifts_;
  static jfieldID unlimited_gifts_fieldID;
  bool limited_gifts_;
  static jfieldID limited_gifts_fieldID;
  bool upgraded_gifts_;
  static jfieldID upgraded_gifts_fieldID;
  bool premium_subscription_;
  static jfieldID premium_subscription_fieldID;

  acceptedGiftTypes();

  acceptedGiftTypes(bool unlimited_gifts_, bool limited_gifts_, bool upgraded_gifts_, bool premium_subscription_);

  static const std::int32_t ID = 1783521300;

  static object_ptr<acceptedGiftTypes> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class accountInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 registration_month_;
  static jfieldID registration_month_fieldID;
  int32 registration_year_;
  static jfieldID registration_year_fieldID;
  string phone_number_country_code_;
  static jfieldID phone_number_country_code_fieldID;
  int32 last_name_change_date_;
  static jfieldID last_name_change_date_fieldID;
  int32 last_photo_change_date_;
  static jfieldID last_photo_change_date_fieldID;

  accountInfo();

  accountInfo(int32 registration_month_, int32 registration_year_, string const &phone_number_country_code_, int32 last_name_change_date_, int32 last_photo_change_date_);

  static const std::int32_t ID = 1803492711;

  static object_ptr<accountInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class accountTtl final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 days_;
  static jfieldID days_fieldID;

  accountTtl();

  explicit accountTtl(int32 days_);

  static const std::int32_t ID = 1324495492;

  static object_ptr<accountTtl> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ReactionType;

class addedReaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> type_;
  static jfieldID type_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  addedReaction();

  addedReaction(object_ptr<ReactionType> &&type_, object_ptr<MessageSender> &&sender_id_, bool is_outgoing_, int32 date_);

  static const std::int32_t ID = 1258586525;

  static object_ptr<addedReaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class addedReaction;

class addedReactions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<addedReaction>> reactions_;
  static jfieldID reactions_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  addedReactions();

  addedReactions(int32 total_count_, array<object_ptr<addedReaction>> &&reactions_, string const &next_offset_);

  static const std::int32_t ID = 226352304;

  static object_ptr<addedReactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class address final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string country_code_;
  static jfieldID country_code_fieldID;
  string state_;
  static jfieldID state_fieldID;
  string city_;
  static jfieldID city_fieldID;
  string street_line1_;
  static jfieldID street_line1_fieldID;
  string street_line2_;
  static jfieldID street_line2_fieldID;
  string postal_code_;
  static jfieldID postal_code_fieldID;

  address();

  address(string const &country_code_, string const &state_, string const &city_, string const &street_line1_, string const &street_line2_, string const &postal_code_);

  static const std::int32_t ID = -2043654342;

  static object_ptr<address> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starAmount;

class affiliateInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 commission_per_mille_;
  static jfieldID commission_per_mille_fieldID;
  int53 affiliate_chat_id_;
  static jfieldID affiliate_chat_id_fieldID;
  object_ptr<starAmount> star_amount_;
  static jfieldID star_amount_fieldID;

  affiliateInfo();

  affiliateInfo(int32 commission_per_mille_, int53 affiliate_chat_id_, object_ptr<starAmount> &&star_amount_);

  static const std::int32_t ID = -1312695046;

  static object_ptr<affiliateInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramParameters;

class starAmount;

class affiliateProgramInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<affiliateProgramParameters> parameters_;
  static jfieldID parameters_fieldID;
  int32 end_date_;
  static jfieldID end_date_fieldID;
  object_ptr<starAmount> daily_revenue_per_user_amount_;
  static jfieldID daily_revenue_per_user_amount_fieldID;

  affiliateProgramInfo();

  affiliateProgramInfo(object_ptr<affiliateProgramParameters> &&parameters_, int32 end_date_, object_ptr<starAmount> &&daily_revenue_per_user_amount_);

  static const std::int32_t ID = -1761810251;

  static object_ptr<affiliateProgramInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 commission_per_mille_;
  static jfieldID commission_per_mille_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;

  affiliateProgramParameters();

  affiliateProgramParameters(int32 commission_per_mille_, int32 month_count_);

  static const std::int32_t ID = 1642662996;

  static object_ptr<affiliateProgramParameters> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateProgramSortOrder: public Object {
 public:
  static jclass Class;

  static object_ptr<AffiliateProgramSortOrder> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramSortOrderProfitability final : public AffiliateProgramSortOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  affiliateProgramSortOrderProfitability();

  static const std::int32_t ID = -1963282585;

  static object_ptr<AffiliateProgramSortOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramSortOrderCreationDate final : public AffiliateProgramSortOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  affiliateProgramSortOrderCreationDate();

  static const std::int32_t ID = -1558628083;

  static object_ptr<AffiliateProgramSortOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramSortOrderRevenue final : public AffiliateProgramSortOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  affiliateProgramSortOrderRevenue();

  static const std::int32_t ID = 1923269304;

  static object_ptr<AffiliateProgramSortOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateType: public Object {
 public:
  static jclass Class;

  static object_ptr<AffiliateType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class affiliateTypeCurrentUser final : public AffiliateType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  affiliateTypeCurrentUser();

  static const std::int32_t ID = 1453785589;

  static object_ptr<AffiliateType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateTypeBot final : public AffiliateType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  affiliateTypeBot();

  explicit affiliateTypeBot(int53 user_id_);

  static const std::int32_t ID = -1032587200;

  static object_ptr<AffiliateType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateTypeChannel final : public AffiliateType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  affiliateTypeChannel();

  explicit affiliateTypeChannel(int53 chat_id_);

  static const std::int32_t ID = -683939735;

  static object_ptr<AffiliateType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class alternativeVideo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  string codec_;
  static jfieldID codec_fieldID;
  object_ptr<file> hls_file_;
  static jfieldID hls_file_fieldID;
  object_ptr<file> video_;
  static jfieldID video_fieldID;

  alternativeVideo();

  alternativeVideo(int64 id_, int32 width_, int32 height_, string const &codec_, object_ptr<file> &&hls_file_, object_ptr<file> &&video_);

  static const std::int32_t ID = 483379470;

  static object_ptr<alternativeVideo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class animatedChatPhoto final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 length_;
  static jfieldID length_fieldID;
  object_ptr<file> file_;
  static jfieldID file_fieldID;
  double main_frame_timestamp_;
  static jfieldID main_frame_timestamp_fieldID;

  animatedChatPhoto();

  animatedChatPhoto(int32 length_, object_ptr<file> &&file_, double main_frame_timestamp_);

  static const std::int32_t ID = 191994926;

  static object_ptr<animatedChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class sticker;

class animatedEmoji final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  int32 sticker_width_;
  static jfieldID sticker_width_fieldID;
  int32 sticker_height_;
  static jfieldID sticker_height_fieldID;
  int32 fitzpatrick_type_;
  static jfieldID fitzpatrick_type_fieldID;
  object_ptr<file> sound_;
  static jfieldID sound_fieldID;

  animatedEmoji();

  animatedEmoji(object_ptr<sticker> &&sticker_, int32 sticker_width_, int32 sticker_height_, int32 fitzpatrick_type_, object_ptr<file> &&sound_);

  static const std::int32_t ID = 1378918079;

  static object_ptr<animatedEmoji> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class thumbnail;

class animation final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  string file_name_;
  static jfieldID file_name_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  bool has_stickers_;
  static jfieldID has_stickers_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<file> animation_;
  static jfieldID animation_fieldID;

  animation();

  animation(int32 duration_, int32 width_, int32 height_, string const &file_name_, string const &mime_type_, bool has_stickers_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&animation_);

  static const std::int32_t ID = -872359106;

  static object_ptr<animation> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class animation;

class animations final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<animation>> animations_;
  static jfieldID animations_fieldID;

  animations();

  explicit animations(array<object_ptr<animation>> &&animations_);

  static const std::int32_t ID = 344216945;

  static object_ptr<animations> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class archiveChatListSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool archive_and_mute_new_chats_from_unknown_users_;
  static jfieldID archive_and_mute_new_chats_from_unknown_users_fieldID;
  bool keep_unmuted_chats_archived_;
  static jfieldID keep_unmuted_chats_archived_fieldID;
  bool keep_chats_from_folders_archived_;
  static jfieldID keep_chats_from_folders_archived_fieldID;

  archiveChatListSettings();

  archiveChatListSettings(bool archive_and_mute_new_chats_from_unknown_users_, bool keep_unmuted_chats_archived_, bool keep_chats_from_folders_archived_);

  static const std::int32_t ID = 1058499236;

  static object_ptr<archiveChatListSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class attachmentMenuBotColor;

class file;

class attachmentMenuBot final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  bool supports_self_chat_;
  static jfieldID supports_self_chat_fieldID;
  bool supports_user_chats_;
  static jfieldID supports_user_chats_fieldID;
  bool supports_bot_chats_;
  static jfieldID supports_bot_chats_fieldID;
  bool supports_group_chats_;
  static jfieldID supports_group_chats_fieldID;
  bool supports_channel_chats_;
  static jfieldID supports_channel_chats_fieldID;
  bool request_write_access_;
  static jfieldID request_write_access_fieldID;
  bool is_added_;
  static jfieldID is_added_fieldID;
  bool show_in_attachment_menu_;
  static jfieldID show_in_attachment_menu_fieldID;
  bool show_in_side_menu_;
  static jfieldID show_in_side_menu_fieldID;
  bool show_disclaimer_in_side_menu_;
  static jfieldID show_disclaimer_in_side_menu_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<attachmentMenuBotColor> name_color_;
  static jfieldID name_color_fieldID;
  object_ptr<file> default_icon_;
  static jfieldID default_icon_fieldID;
  object_ptr<file> ios_static_icon_;
  static jfieldID ios_static_icon_fieldID;
  object_ptr<file> ios_animated_icon_;
  static jfieldID ios_animated_icon_fieldID;
  object_ptr<file> ios_side_menu_icon_;
  static jfieldID ios_side_menu_icon_fieldID;
  object_ptr<file> android_icon_;
  static jfieldID android_icon_fieldID;
  object_ptr<file> android_side_menu_icon_;
  static jfieldID android_side_menu_icon_fieldID;
  object_ptr<file> macos_icon_;
  static jfieldID macos_icon_fieldID;
  object_ptr<file> macos_side_menu_icon_;
  static jfieldID macos_side_menu_icon_fieldID;
  object_ptr<attachmentMenuBotColor> icon_color_;
  static jfieldID icon_color_fieldID;
  object_ptr<file> web_app_placeholder_;
  static jfieldID web_app_placeholder_fieldID;

  attachmentMenuBot();

  attachmentMenuBot(int53 bot_user_id_, bool supports_self_chat_, bool supports_user_chats_, bool supports_bot_chats_, bool supports_group_chats_, bool supports_channel_chats_, bool request_write_access_, bool is_added_, bool show_in_attachment_menu_, bool show_in_side_menu_, bool show_disclaimer_in_side_menu_, string const &name_, object_ptr<attachmentMenuBotColor> &&name_color_, object_ptr<file> &&default_icon_, object_ptr<file> &&ios_static_icon_, object_ptr<file> &&ios_animated_icon_, object_ptr<file> &&ios_side_menu_icon_, object_ptr<file> &&android_icon_, object_ptr<file> &&android_side_menu_icon_, object_ptr<file> &&macos_icon_, object_ptr<file> &&macos_side_menu_icon_, object_ptr<attachmentMenuBotColor> &&icon_color_, object_ptr<file> &&web_app_placeholder_);

  static const std::int32_t ID = -1183966273;

  static object_ptr<attachmentMenuBot> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class attachmentMenuBotColor final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 light_color_;
  static jfieldID light_color_fieldID;
  int32 dark_color_;
  static jfieldID dark_color_fieldID;

  attachmentMenuBotColor();

  attachmentMenuBotColor(int32 light_color_, int32 dark_color_);

  static const std::int32_t ID = 1680039612;

  static object_ptr<attachmentMenuBotColor> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class thumbnail;

class audio final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string performer_;
  static jfieldID performer_fieldID;
  string file_name_;
  static jfieldID file_name_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  object_ptr<minithumbnail> album_cover_minithumbnail_;
  static jfieldID album_cover_minithumbnail_fieldID;
  object_ptr<thumbnail> album_cover_thumbnail_;
  static jfieldID album_cover_thumbnail_fieldID;
  array<object_ptr<thumbnail>> external_album_covers_;
  static jfieldID external_album_covers_fieldID;
  object_ptr<file> audio_;
  static jfieldID audio_fieldID;

  audio();

  audio(int32 duration_, string const &title_, string const &performer_, string const &file_name_, string const &mime_type_, object_ptr<minithumbnail> &&album_cover_minithumbnail_, object_ptr<thumbnail> &&album_cover_thumbnail_, array<object_ptr<thumbnail>> &&external_album_covers_, object_ptr<file> &&audio_);

  static const std::int32_t ID = -166398841;

  static object_ptr<audio> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class AuthenticationCodeType;

class authenticationCodeInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  object_ptr<AuthenticationCodeType> type_;
  static jfieldID type_fieldID;
  object_ptr<AuthenticationCodeType> next_type_;
  static jfieldID next_type_fieldID;
  int32 timeout_;
  static jfieldID timeout_fieldID;

  authenticationCodeInfo();

  authenticationCodeInfo(string const &phone_number_, object_ptr<AuthenticationCodeType> &&type_, object_ptr<AuthenticationCodeType> &&next_type_, int32 timeout_);

  static const std::int32_t ID = -860345416;

  static object_ptr<authenticationCodeInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FirebaseDeviceVerificationParameters;

class AuthenticationCodeType: public Object {
 public:
  static jclass Class;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeTelegramMessage final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeTelegramMessage();

  explicit authenticationCodeTypeTelegramMessage(int32 length_);

  static const std::int32_t ID = 2079628074;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeSms final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeSms();

  explicit authenticationCodeTypeSms(int32 length_);

  static const std::int32_t ID = 962650760;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeSmsWord final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string first_letter_;
  static jfieldID first_letter_fieldID;

  authenticationCodeTypeSmsWord();

  explicit authenticationCodeTypeSmsWord(string const &first_letter_);

  static const std::int32_t ID = -1509540765;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeSmsPhrase final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string first_word_;
  static jfieldID first_word_fieldID;

  authenticationCodeTypeSmsPhrase();

  explicit authenticationCodeTypeSmsPhrase(string const &first_word_);

  static const std::int32_t ID = 784108753;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeCall final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeCall();

  explicit authenticationCodeTypeCall(int32 length_);

  static const std::int32_t ID = 1636265063;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeFlashCall final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string pattern_;
  static jfieldID pattern_fieldID;

  authenticationCodeTypeFlashCall();

  explicit authenticationCodeTypeFlashCall(string const &pattern_);

  static const std::int32_t ID = 1395882402;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeMissedCall final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_prefix_;
  static jfieldID phone_number_prefix_fieldID;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeMissedCall();

  authenticationCodeTypeMissedCall(string const &phone_number_prefix_, int32 length_);

  static const std::int32_t ID = 700123783;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeFragment final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeFragment();

  authenticationCodeTypeFragment(string const &url_, int32 length_);

  static const std::int32_t ID = -2129693491;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeFirebaseAndroid final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<FirebaseDeviceVerificationParameters> device_verification_parameters_;
  static jfieldID device_verification_parameters_fieldID;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeFirebaseAndroid();

  authenticationCodeTypeFirebaseAndroid(object_ptr<FirebaseDeviceVerificationParameters> &&device_verification_parameters_, int32 length_);

  static const std::int32_t ID = 1872475422;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authenticationCodeTypeFirebaseIos final : public AuthenticationCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string receipt_;
  static jfieldID receipt_fieldID;
  int32 push_timeout_;
  static jfieldID push_timeout_fieldID;
  int32 length_;
  static jfieldID length_fieldID;

  authenticationCodeTypeFirebaseIos();

  authenticationCodeTypeFirebaseIos(string const &receipt_, int32 push_timeout_, int32 length_);

  static const std::int32_t ID = -11162989;

  static object_ptr<AuthenticationCodeType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmailAddressResetState;

class authenticationCodeInfo;

class emailAddressAuthenticationCodeInfo;

class termsOfService;

class AuthorizationState: public Object {
 public:
  static jclass Class;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitTdlibParameters final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateWaitTdlibParameters();

  static const std::int32_t ID = 904720988;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitPhoneNumber final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateWaitPhoneNumber();

  static const std::int32_t ID = 306402531;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitPremiumPurchase final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;

  authorizationStateWaitPremiumPurchase();

  explicit authorizationStateWaitPremiumPurchase(string const &store_product_id_);

  static const std::int32_t ID = -862487743;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitEmailAddress final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_apple_id_;
  static jfieldID allow_apple_id_fieldID;
  bool allow_google_id_;
  static jfieldID allow_google_id_fieldID;

  authorizationStateWaitEmailAddress();

  authorizationStateWaitEmailAddress(bool allow_apple_id_, bool allow_google_id_);

  static const std::int32_t ID = 1040478663;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitEmailCode final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_apple_id_;
  static jfieldID allow_apple_id_fieldID;
  bool allow_google_id_;
  static jfieldID allow_google_id_fieldID;
  object_ptr<emailAddressAuthenticationCodeInfo> code_info_;
  static jfieldID code_info_fieldID;
  object_ptr<EmailAddressResetState> email_address_reset_state_;
  static jfieldID email_address_reset_state_fieldID;

  authorizationStateWaitEmailCode();

  authorizationStateWaitEmailCode(bool allow_apple_id_, bool allow_google_id_, object_ptr<emailAddressAuthenticationCodeInfo> &&code_info_, object_ptr<EmailAddressResetState> &&email_address_reset_state_);

  static const std::int32_t ID = -1868627365;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitCode final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<authenticationCodeInfo> code_info_;
  static jfieldID code_info_fieldID;

  authorizationStateWaitCode();

  explicit authorizationStateWaitCode(object_ptr<authenticationCodeInfo> &&code_info_);

  static const std::int32_t ID = 52643073;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitOtherDeviceConfirmation final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  authorizationStateWaitOtherDeviceConfirmation();

  explicit authorizationStateWaitOtherDeviceConfirmation(string const &link_);

  static const std::int32_t ID = 860166378;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitRegistration final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<termsOfService> terms_of_service_;
  static jfieldID terms_of_service_fieldID;

  authorizationStateWaitRegistration();

  explicit authorizationStateWaitRegistration(object_ptr<termsOfService> &&terms_of_service_);

  static const std::int32_t ID = 550350511;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateWaitPassword final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_hint_;
  static jfieldID password_hint_fieldID;
  bool has_recovery_email_address_;
  static jfieldID has_recovery_email_address_fieldID;
  bool has_passport_data_;
  static jfieldID has_passport_data_fieldID;
  string recovery_email_address_pattern_;
  static jfieldID recovery_email_address_pattern_fieldID;

  authorizationStateWaitPassword();

  authorizationStateWaitPassword(string const &password_hint_, bool has_recovery_email_address_, bool has_passport_data_, string const &recovery_email_address_pattern_);

  static const std::int32_t ID = 112238030;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateReady final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateReady();

  static const std::int32_t ID = -1834871737;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateLoggingOut final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateLoggingOut();

  static const std::int32_t ID = 154449270;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateClosing final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateClosing();

  static const std::int32_t ID = 445855311;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class authorizationStateClosed final : public AuthorizationState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  authorizationStateClosed();

  static const std::int32_t ID = 1526047584;

  static object_ptr<AuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autoDownloadSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_auto_download_enabled_;
  static jfieldID is_auto_download_enabled_fieldID;
  int32 max_photo_file_size_;
  static jfieldID max_photo_file_size_fieldID;
  int53 max_video_file_size_;
  static jfieldID max_video_file_size_fieldID;
  int53 max_other_file_size_;
  static jfieldID max_other_file_size_fieldID;
  int32 video_upload_bitrate_;
  static jfieldID video_upload_bitrate_fieldID;
  bool preload_large_videos_;
  static jfieldID preload_large_videos_fieldID;
  bool preload_next_audio_;
  static jfieldID preload_next_audio_fieldID;
  bool preload_stories_;
  static jfieldID preload_stories_fieldID;
  bool use_less_data_for_calls_;
  static jfieldID use_less_data_for_calls_fieldID;

  autoDownloadSettings();

  autoDownloadSettings(bool is_auto_download_enabled_, int32 max_photo_file_size_, int53 max_video_file_size_, int53 max_other_file_size_, int32 video_upload_bitrate_, bool preload_large_videos_, bool preload_next_audio_, bool preload_stories_, bool use_less_data_for_calls_);

  static const std::int32_t ID = 991433696;

  static object_ptr<autoDownloadSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autoDownloadSettings;

class autoDownloadSettingsPresets final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<autoDownloadSettings> low_;
  static jfieldID low_fieldID;
  object_ptr<autoDownloadSettings> medium_;
  static jfieldID medium_fieldID;
  object_ptr<autoDownloadSettings> high_;
  static jfieldID high_fieldID;

  autoDownloadSettingsPresets();

  autoDownloadSettingsPresets(object_ptr<autoDownloadSettings> &&low_, object_ptr<autoDownloadSettings> &&medium_, object_ptr<autoDownloadSettings> &&high_);

  static const std::int32_t ID = -782099166;

  static object_ptr<autoDownloadSettingsPresets> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettingsException;

class scopeAutosaveSettings;

class autosaveSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<scopeAutosaveSettings> private_chat_settings_;
  static jfieldID private_chat_settings_fieldID;
  object_ptr<scopeAutosaveSettings> group_settings_;
  static jfieldID group_settings_fieldID;
  object_ptr<scopeAutosaveSettings> channel_settings_;
  static jfieldID channel_settings_fieldID;
  array<object_ptr<autosaveSettingsException>> exceptions_;
  static jfieldID exceptions_fieldID;

  autosaveSettings();

  autosaveSettings(object_ptr<scopeAutosaveSettings> &&private_chat_settings_, object_ptr<scopeAutosaveSettings> &&group_settings_, object_ptr<scopeAutosaveSettings> &&channel_settings_, array<object_ptr<autosaveSettingsException>> &&exceptions_);

  static const std::int32_t ID = 1629412502;

  static object_ptr<autosaveSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class scopeAutosaveSettings;

class autosaveSettingsException final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<scopeAutosaveSettings> settings_;
  static jfieldID settings_fieldID;

  autosaveSettingsException();

  autosaveSettingsException(int53 chat_id_, object_ptr<scopeAutosaveSettings> &&settings_);

  static const std::int32_t ID = 1483470280;

  static object_ptr<autosaveSettingsException> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class AutosaveSettingsScope: public Object {
 public:
  static jclass Class;

  static object_ptr<AutosaveSettingsScope> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettingsScopePrivateChats final : public AutosaveSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  autosaveSettingsScopePrivateChats();

  static const std::int32_t ID = 1395227007;

  static object_ptr<AutosaveSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettingsScopeGroupChats final : public AutosaveSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  autosaveSettingsScopeGroupChats();

  static const std::int32_t ID = 853544526;

  static object_ptr<AutosaveSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettingsScopeChannelChats final : public AutosaveSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  autosaveSettingsScopeChannelChats();

  static const std::int32_t ID = -499572783;

  static object_ptr<AutosaveSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettingsScopeChat final : public AutosaveSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  autosaveSettingsScopeChat();

  explicit autosaveSettingsScopeChat(int53 chat_id_);

  static const std::int32_t ID = -1632255255;

  static object_ptr<AutosaveSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class gift;

class availableGift final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;
  int32 resale_count_;
  static jfieldID resale_count_fieldID;
  int53 min_resale_star_count_;
  static jfieldID min_resale_star_count_fieldID;
  string title_;
  static jfieldID title_fieldID;

  availableGift();

  availableGift(object_ptr<gift> &&gift_, int32 resale_count_, int53 min_resale_star_count_, string const &title_);

  static const std::int32_t ID = 850530502;

  static object_ptr<availableGift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class availableGift;

class availableGifts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<availableGift>> gifts_;
  static jfieldID gifts_fieldID;

  availableGifts();

  explicit availableGifts(array<object_ptr<availableGift>> &&gifts_);

  static const std::int32_t ID = -1450341886;

  static object_ptr<availableGifts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class availableReaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> type_;
  static jfieldID type_fieldID;
  bool needs_premium_;
  static jfieldID needs_premium_fieldID;

  availableReaction();

  availableReaction(object_ptr<ReactionType> &&type_, bool needs_premium_);

  static const std::int32_t ID = -117292153;

  static object_ptr<availableReaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionUnavailabilityReason;

class availableReaction;

class availableReactions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<availableReaction>> top_reactions_;
  static jfieldID top_reactions_fieldID;
  array<object_ptr<availableReaction>> recent_reactions_;
  static jfieldID recent_reactions_fieldID;
  array<object_ptr<availableReaction>> popular_reactions_;
  static jfieldID popular_reactions_fieldID;
  bool allow_custom_emoji_;
  static jfieldID allow_custom_emoji_fieldID;
  bool are_tags_;
  static jfieldID are_tags_fieldID;
  object_ptr<ReactionUnavailabilityReason> unavailability_reason_;
  static jfieldID unavailability_reason_fieldID;

  availableReactions();

  availableReactions(array<object_ptr<availableReaction>> &&top_reactions_, array<object_ptr<availableReaction>> &&recent_reactions_, array<object_ptr<availableReaction>> &&popular_reactions_, bool allow_custom_emoji_, bool are_tags_, object_ptr<ReactionUnavailabilityReason> &&unavailability_reason_);

  static const std::int32_t ID = 912529522;

  static object_ptr<availableReactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundType;

class document;

class background final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  bool is_default_;
  static jfieldID is_default_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  object_ptr<BackgroundType> type_;
  static jfieldID type_fieldID;

  background();

  background(int64 id_, bool is_default_, bool is_dark_, string const &name_, object_ptr<document> &&document_, object_ptr<BackgroundType> &&type_);

  static const std::int32_t ID = -429971172;

  static object_ptr<background> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundFill: public Object {
 public:
  static jclass Class;

  static object_ptr<BackgroundFill> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class backgroundFillSolid final : public BackgroundFill {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 color_;
  static jfieldID color_fieldID;

  backgroundFillSolid();

  explicit backgroundFillSolid(int32 color_);

  static const std::int32_t ID = 1010678813;

  static object_ptr<BackgroundFill> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class backgroundFillGradient final : public BackgroundFill {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 top_color_;
  static jfieldID top_color_fieldID;
  int32 bottom_color_;
  static jfieldID bottom_color_fieldID;
  int32 rotation_angle_;
  static jfieldID rotation_angle_fieldID;

  backgroundFillGradient();

  backgroundFillGradient(int32 top_color_, int32 bottom_color_, int32 rotation_angle_);

  static const std::int32_t ID = -1839206017;

  static object_ptr<BackgroundFill> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class backgroundFillFreeformGradient final : public BackgroundFill {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> colors_;
  static jfieldID colors_fieldID;

  backgroundFillFreeformGradient();

  explicit backgroundFillFreeformGradient(array<int32> &&colors_);

  static const std::int32_t ID = -1145469255;

  static object_ptr<BackgroundFill> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundFill;

class BackgroundType: public Object {
 public:
  static jclass Class;

  static object_ptr<BackgroundType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class backgroundTypeWallpaper final : public BackgroundType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_blurred_;
  static jfieldID is_blurred_fieldID;
  bool is_moving_;
  static jfieldID is_moving_fieldID;

  backgroundTypeWallpaper();

  backgroundTypeWallpaper(bool is_blurred_, bool is_moving_);

  static const std::int32_t ID = 1972128891;

  static object_ptr<BackgroundType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class backgroundTypePattern final : public BackgroundType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BackgroundFill> fill_;
  static jfieldID fill_fieldID;
  int32 intensity_;
  static jfieldID intensity_fieldID;
  bool is_inverted_;
  static jfieldID is_inverted_fieldID;
  bool is_moving_;
  static jfieldID is_moving_fieldID;

  backgroundTypePattern();

  backgroundTypePattern(object_ptr<BackgroundFill> &&fill_, int32 intensity_, bool is_inverted_, bool is_moving_);

  static const std::int32_t ID = 1290213117;

  static object_ptr<BackgroundType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class backgroundTypeFill final : public BackgroundType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BackgroundFill> fill_;
  static jfieldID fill_fieldID;

  backgroundTypeFill();

  explicit backgroundTypeFill(object_ptr<BackgroundFill> &&fill_);

  static const std::int32_t ID = 993008684;

  static object_ptr<BackgroundType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class backgroundTypeChatTheme final : public BackgroundType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  backgroundTypeChatTheme();

  explicit backgroundTypeChatTheme(string const &theme_name_);

  static const std::int32_t ID = 1299879762;

  static object_ptr<BackgroundType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class background;

class backgrounds final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<background>> backgrounds_;
  static jfieldID backgrounds_fieldID;

  backgrounds();

  explicit backgrounds(array<object_ptr<background>> &&backgrounds_);

  static const std::int32_t ID = 724728704;

  static object_ptr<backgrounds> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class bankCardActionOpenUrl final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  string url_;
  static jfieldID url_fieldID;

  bankCardActionOpenUrl();

  bankCardActionOpenUrl(string const &text_, string const &url_);

  static const std::int32_t ID = -196454267;

  static object_ptr<bankCardActionOpenUrl> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class bankCardActionOpenUrl;

class bankCardInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  array<object_ptr<bankCardActionOpenUrl>> actions_;
  static jfieldID actions_fieldID;

  bankCardInfo();

  bankCardInfo(string const &title_, array<object_ptr<bankCardActionOpenUrl>> &&actions_);

  static const std::int32_t ID = -2116647730;

  static object_ptr<bankCardInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatMemberStatus;

class basicGroup final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  int32 member_count_;
  static jfieldID member_count_fieldID;
  object_ptr<ChatMemberStatus> status_;
  static jfieldID status_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;
  int53 upgraded_to_supergroup_id_;
  static jfieldID upgraded_to_supergroup_id_fieldID;

  basicGroup();

  basicGroup(int53 id_, int32 member_count_, object_ptr<ChatMemberStatus> &&status_, bool is_active_, int53 upgraded_to_supergroup_id_);

  static const std::int32_t ID = -194767217;

  static object_ptr<basicGroup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommands;

class chatInviteLink;

class chatMember;

class chatPhoto;

class basicGroupFullInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  string description_;
  static jfieldID description_fieldID;
  int53 creator_user_id_;
  static jfieldID creator_user_id_fieldID;
  array<object_ptr<chatMember>> members_;
  static jfieldID members_fieldID;
  bool can_hide_members_;
  static jfieldID can_hide_members_fieldID;
  bool can_toggle_aggressive_anti_spam_;
  static jfieldID can_toggle_aggressive_anti_spam_fieldID;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;
  array<object_ptr<botCommands>> bot_commands_;
  static jfieldID bot_commands_fieldID;

  basicGroupFullInfo();

  basicGroupFullInfo(object_ptr<chatPhoto> &&photo_, string const &description_, int53 creator_user_id_, array<object_ptr<chatMember>> &&members_, bool can_hide_members_, bool can_toggle_aggressive_anti_spam_, object_ptr<chatInviteLink> &&invite_link_, array<object_ptr<botCommands>> &&bot_commands_);

  static const std::int32_t ID = -1879035520;

  static object_ptr<basicGroupFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class birthdate final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 day_;
  static jfieldID day_fieldID;
  int32 month_;
  static jfieldID month_fieldID;
  int32 year_;
  static jfieldID year_fieldID;

  birthdate();

  birthdate(int32 day_, int32 month_, int32 year_);

  static const std::int32_t ID = 1644064030;

  static object_ptr<birthdate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BlockList: public Object {
 public:
  static jclass Class;

  static object_ptr<BlockList> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class blockListMain final : public BlockList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  blockListMain();

  static const std::int32_t ID = 1352930172;

  static object_ptr<BlockList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class blockListStories final : public BlockList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  blockListStories();

  static const std::int32_t ID = 103323228;

  static object_ptr<BlockList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommand final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string command_;
  static jfieldID command_fieldID;
  string description_;
  static jfieldID description_fieldID;

  botCommand();

  botCommand(string const &command_, string const &description_);

  static const std::int32_t ID = -1032140601;

  static object_ptr<botCommand> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BotCommandScope: public Object {
 public:
  static jclass Class;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeDefault final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botCommandScopeDefault();

  static const std::int32_t ID = 795652779;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeAllPrivateChats final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botCommandScopeAllPrivateChats();

  static const std::int32_t ID = -344889543;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeAllGroupChats final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botCommandScopeAllGroupChats();

  static const std::int32_t ID = -981088162;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeAllChatAdministrators final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botCommandScopeAllChatAdministrators();

  static const std::int32_t ID = 1998329169;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeChat final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  botCommandScopeChat();

  explicit botCommandScopeChat(int53 chat_id_);

  static const std::int32_t ID = -430234971;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeChatAdministrators final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  botCommandScopeChatAdministrators();

  explicit botCommandScopeChatAdministrators(int53 chat_id_);

  static const std::int32_t ID = 1119682126;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommandScopeChatMember final : public BotCommandScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  botCommandScopeChatMember();

  botCommandScopeChatMember(int53 chat_id_, int53 user_id_);

  static const std::int32_t ID = -211380494;

  static object_ptr<BotCommandScope> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommand;

class botCommands final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  array<object_ptr<botCommand>> commands_;
  static jfieldID commands_fieldID;

  botCommands();

  botCommands(int53 bot_user_id_, array<object_ptr<botCommand>> &&commands_);

  static const std::int32_t ID = 1741364468;

  static object_ptr<botCommands> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class affiliateProgramInfo;

class animation;

class botCommand;

class botMenuButton;

class botVerificationParameters;

class chatAdministratorRights;

class photo;

class botInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string short_description_;
  static jfieldID short_description_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;
  object_ptr<botMenuButton> menu_button_;
  static jfieldID menu_button_fieldID;
  array<object_ptr<botCommand>> commands_;
  static jfieldID commands_fieldID;
  string privacy_policy_url_;
  static jfieldID privacy_policy_url_fieldID;
  object_ptr<chatAdministratorRights> default_group_administrator_rights_;
  static jfieldID default_group_administrator_rights_fieldID;
  object_ptr<chatAdministratorRights> default_channel_administrator_rights_;
  static jfieldID default_channel_administrator_rights_fieldID;
  object_ptr<affiliateProgramInfo> affiliate_program_;
  static jfieldID affiliate_program_fieldID;
  int32 web_app_background_light_color_;
  static jfieldID web_app_background_light_color_fieldID;
  int32 web_app_background_dark_color_;
  static jfieldID web_app_background_dark_color_fieldID;
  int32 web_app_header_light_color_;
  static jfieldID web_app_header_light_color_fieldID;
  int32 web_app_header_dark_color_;
  static jfieldID web_app_header_dark_color_fieldID;
  object_ptr<botVerificationParameters> verification_parameters_;
  static jfieldID verification_parameters_fieldID;
  bool can_get_revenue_statistics_;
  static jfieldID can_get_revenue_statistics_fieldID;
  bool can_manage_emoji_status_;
  static jfieldID can_manage_emoji_status_fieldID;
  bool has_media_previews_;
  static jfieldID has_media_previews_fieldID;
  object_ptr<InternalLinkType> edit_commands_link_;
  static jfieldID edit_commands_link_fieldID;
  object_ptr<InternalLinkType> edit_description_link_;
  static jfieldID edit_description_link_fieldID;
  object_ptr<InternalLinkType> edit_description_media_link_;
  static jfieldID edit_description_media_link_fieldID;
  object_ptr<InternalLinkType> edit_settings_link_;
  static jfieldID edit_settings_link_fieldID;

  botInfo();

  botInfo(string const &short_description_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_, object_ptr<botMenuButton> &&menu_button_, array<object_ptr<botCommand>> &&commands_, string const &privacy_policy_url_, object_ptr<chatAdministratorRights> &&default_group_administrator_rights_, object_ptr<chatAdministratorRights> &&default_channel_administrator_rights_, object_ptr<affiliateProgramInfo> &&affiliate_program_, int32 web_app_background_light_color_, int32 web_app_background_dark_color_, int32 web_app_header_light_color_, int32 web_app_header_dark_color_, object_ptr<botVerificationParameters> &&verification_parameters_, bool can_get_revenue_statistics_, bool can_manage_emoji_status_, bool has_media_previews_, object_ptr<InternalLinkType> &&edit_commands_link_, object_ptr<InternalLinkType> &&edit_description_link_, object_ptr<InternalLinkType> &&edit_description_media_link_, object_ptr<InternalLinkType> &&edit_settings_link_);

  static const std::int32_t ID = 1771886272;

  static object_ptr<botInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryContent;

class botMediaPreview final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<StoryContent> content_;
  static jfieldID content_fieldID;

  botMediaPreview();

  botMediaPreview(int32 date_, object_ptr<StoryContent> &&content_);

  static const std::int32_t ID = -1632264984;

  static object_ptr<botMediaPreview> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botMediaPreview;

class botMediaPreviewInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<botMediaPreview>> previews_;
  static jfieldID previews_fieldID;
  array<string> language_codes_;
  static jfieldID language_codes_fieldID;

  botMediaPreviewInfo();

  botMediaPreviewInfo(array<object_ptr<botMediaPreview>> &&previews_, array<string> &&language_codes_);

  static const std::int32_t ID = -284783012;

  static object_ptr<botMediaPreviewInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botMediaPreview;

class botMediaPreviews final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<botMediaPreview>> previews_;
  static jfieldID previews_fieldID;

  botMediaPreviews();

  explicit botMediaPreviews(array<object_ptr<botMediaPreview>> &&previews_);

  static const std::int32_t ID = -1787720586;

  static object_ptr<botMediaPreviews> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botMenuButton final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  string url_;
  static jfieldID url_fieldID;

  botMenuButton();

  botMenuButton(string const &text_, string const &url_);

  static const std::int32_t ID = -944407322;

  static object_ptr<botMenuButton> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class botVerification final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  int64 icon_custom_emoji_id_;
  static jfieldID icon_custom_emoji_id_fieldID;
  object_ptr<formattedText> custom_description_;
  static jfieldID custom_description_fieldID;

  botVerification();

  botVerification(int53 bot_user_id_, int64 icon_custom_emoji_id_, object_ptr<formattedText> &&custom_description_);

  static const std::int32_t ID = -1319061774;

  static object_ptr<botVerification> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class botVerificationParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 icon_custom_emoji_id_;
  static jfieldID icon_custom_emoji_id_fieldID;
  string organization_name_;
  static jfieldID organization_name_fieldID;
  object_ptr<formattedText> default_custom_description_;
  static jfieldID default_custom_description_fieldID;
  bool can_set_custom_description_;
  static jfieldID can_set_custom_description_fieldID;

  botVerificationParameters();

  botVerificationParameters(int64 icon_custom_emoji_id_, string const &organization_name_, object_ptr<formattedText> &&default_custom_description_, bool can_set_custom_description_);

  static const std::int32_t ID = -723737249;

  static object_ptr<botVerificationParameters> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class webApp;

class BotWriteAccessAllowReason: public Object {
 public:
  static jclass Class;

  static object_ptr<BotWriteAccessAllowReason> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botWriteAccessAllowReasonConnectedWebsite final : public BotWriteAccessAllowReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string domain_name_;
  static jfieldID domain_name_fieldID;

  botWriteAccessAllowReasonConnectedWebsite();

  explicit botWriteAccessAllowReasonConnectedWebsite(string const &domain_name_);

  static const std::int32_t ID = 2016325603;

  static object_ptr<BotWriteAccessAllowReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botWriteAccessAllowReasonAddedToAttachmentMenu final : public BotWriteAccessAllowReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botWriteAccessAllowReasonAddedToAttachmentMenu();

  static const std::int32_t ID = -2104795235;

  static object_ptr<BotWriteAccessAllowReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botWriteAccessAllowReasonLaunchedWebApp final : public BotWriteAccessAllowReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<webApp> web_app_;
  static jfieldID web_app_fieldID;

  botWriteAccessAllowReasonLaunchedWebApp();

  explicit botWriteAccessAllowReasonLaunchedWebApp(object_ptr<webApp> &&web_app_);

  static const std::int32_t ID = -240843561;

  static object_ptr<BotWriteAccessAllowReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botWriteAccessAllowReasonAcceptedRequest final : public BotWriteAccessAllowReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  botWriteAccessAllowReasonAcceptedRequest();

  static const std::int32_t ID = -1983497220;

  static object_ptr<BotWriteAccessAllowReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessAwayMessageSchedule: public Object {
 public:
  static jclass Class;

  static object_ptr<BusinessAwayMessageSchedule> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessAwayMessageScheduleAlways final : public BusinessAwayMessageSchedule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessAwayMessageScheduleAlways();

  static const std::int32_t ID = -910564679;

  static object_ptr<BusinessAwayMessageSchedule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessAwayMessageScheduleOutsideOfOpeningHours final : public BusinessAwayMessageSchedule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessAwayMessageScheduleOutsideOfOpeningHours();

  static const std::int32_t ID = -968630506;

  static object_ptr<BusinessAwayMessageSchedule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessAwayMessageScheduleCustom final : public BusinessAwayMessageSchedule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  int32 end_date_;
  static jfieldID end_date_fieldID;

  businessAwayMessageScheduleCustom();

  businessAwayMessageScheduleCustom(int32 start_date_, int32 end_date_);

  static const std::int32_t ID = -1967108654;

  static object_ptr<BusinessAwayMessageSchedule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessAwayMessageSchedule;

class businessRecipients;

class businessAwayMessageSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  object_ptr<businessRecipients> recipients_;
  static jfieldID recipients_fieldID;
  object_ptr<BusinessAwayMessageSchedule> schedule_;
  static jfieldID schedule_fieldID;
  bool offline_only_;
  static jfieldID offline_only_fieldID;

  businessAwayMessageSettings();

  businessAwayMessageSettings(int32 shortcut_id_, object_ptr<businessRecipients> &&recipients_, object_ptr<BusinessAwayMessageSchedule> &&schedule_, bool offline_only_);

  static const std::int32_t ID = 353084137;

  static object_ptr<businessAwayMessageSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessBotManageBar final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string manage_url_;
  static jfieldID manage_url_fieldID;
  bool is_bot_paused_;
  static jfieldID is_bot_paused_fieldID;
  bool can_bot_reply_;
  static jfieldID can_bot_reply_fieldID;

  businessBotManageBar();

  businessBotManageBar(int53 bot_user_id_, string const &manage_url_, bool is_bot_paused_, bool can_bot_reply_);

  static const std::int32_t ID = -311399806;

  static object_ptr<businessBotManageBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessBotRights final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_reply_;
  static jfieldID can_reply_fieldID;
  bool can_read_messages_;
  static jfieldID can_read_messages_fieldID;
  bool can_delete_sent_messages_;
  static jfieldID can_delete_sent_messages_fieldID;
  bool can_delete_all_messages_;
  static jfieldID can_delete_all_messages_fieldID;
  bool can_edit_name_;
  static jfieldID can_edit_name_fieldID;
  bool can_edit_bio_;
  static jfieldID can_edit_bio_fieldID;
  bool can_edit_profile_photo_;
  static jfieldID can_edit_profile_photo_fieldID;
  bool can_edit_username_;
  static jfieldID can_edit_username_fieldID;
  bool can_view_gifts_and_stars_;
  static jfieldID can_view_gifts_and_stars_fieldID;
  bool can_sell_gifts_;
  static jfieldID can_sell_gifts_fieldID;
  bool can_change_gift_settings_;
  static jfieldID can_change_gift_settings_fieldID;
  bool can_transfer_and_upgrade_gifts_;
  static jfieldID can_transfer_and_upgrade_gifts_fieldID;
  bool can_transfer_stars_;
  static jfieldID can_transfer_stars_fieldID;
  bool can_manage_stories_;
  static jfieldID can_manage_stories_fieldID;

  businessBotRights();

  businessBotRights(bool can_reply_, bool can_read_messages_, bool can_delete_sent_messages_, bool can_delete_all_messages_, bool can_edit_name_, bool can_edit_bio_, bool can_edit_profile_photo_, bool can_edit_username_, bool can_view_gifts_and_stars_, bool can_sell_gifts_, bool can_change_gift_settings_, bool can_transfer_and_upgrade_gifts_, bool can_transfer_stars_, bool can_manage_stories_);

  static const std::int32_t ID = 1224839038;

  static object_ptr<businessBotRights> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class businessChatLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string title_;
  static jfieldID title_fieldID;
  int32 view_count_;
  static jfieldID view_count_fieldID;

  businessChatLink();

  businessChatLink(string const &link_, object_ptr<formattedText> &&text_, string const &title_, int32 view_count_);

  static const std::int32_t ID = -1902539901;

  static object_ptr<businessChatLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class businessChatLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  businessChatLinkInfo();

  businessChatLinkInfo(int53 chat_id_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -864865105;

  static object_ptr<businessChatLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessChatLink;

class businessChatLinks final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<businessChatLink>> links_;
  static jfieldID links_fieldID;

  businessChatLinks();

  explicit businessChatLinks(array<object_ptr<businessChatLink>> &&links_);

  static const std::int32_t ID = 79067036;

  static object_ptr<businessChatLinks> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessBotRights;

class businessRecipients;

class businessConnectedBot final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<businessRecipients> recipients_;
  static jfieldID recipients_fieldID;
  object_ptr<businessBotRights> rights_;
  static jfieldID rights_fieldID;

  businessConnectedBot();

  businessConnectedBot(int53 bot_user_id_, object_ptr<businessRecipients> &&recipients_, object_ptr<businessBotRights> &&rights_);

  static const std::int32_t ID = -1815439021;

  static object_ptr<businessConnectedBot> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessBotRights;

class businessConnection final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int53 user_chat_id_;
  static jfieldID user_chat_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<businessBotRights> rights_;
  static jfieldID rights_fieldID;
  bool is_enabled_;
  static jfieldID is_enabled_fieldID;

  businessConnection();

  businessConnection(string const &id_, int53 user_id_, int53 user_chat_id_, int32 date_, object_ptr<businessBotRights> &&rights_, bool is_enabled_);

  static const std::int32_t ID = -995703933;

  static object_ptr<businessConnection> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessFeature: public Object {
 public:
  static jclass Class;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureLocation final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureLocation();

  static const std::int32_t ID = -1064304004;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureOpeningHours final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureOpeningHours();

  static const std::int32_t ID = 461054701;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureQuickReplies final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureQuickReplies();

  static const std::int32_t ID = -1674048894;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureGreetingMessage final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureGreetingMessage();

  static const std::int32_t ID = 1789424756;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureAwayMessage final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureAwayMessage();

  static const std::int32_t ID = 1090119901;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureAccountLinks final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureAccountLinks();

  static const std::int32_t ID = 1878693646;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureStartPage final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureStartPage();

  static const std::int32_t ID = 401471457;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureBots final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureBots();

  static const std::int32_t ID = 275084773;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureEmojiStatus final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureEmojiStatus();

  static const std::int32_t ID = -846282523;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureChatFolderTags final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureChatFolderTags();

  static const std::int32_t ID = -543880918;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeatureUpgradedStories final : public BusinessFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  businessFeatureUpgradedStories();

  static const std::int32_t ID = -1812245550;

  static object_ptr<BusinessFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessFeature;

class animation;

class businessFeaturePromotionAnimation final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BusinessFeature> feature_;
  static jfieldID feature_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;

  businessFeaturePromotionAnimation();

  businessFeaturePromotionAnimation(object_ptr<BusinessFeature> &&feature_, object_ptr<animation> &&animation_);

  static const std::int32_t ID = 2047174666;

  static object_ptr<businessFeaturePromotionAnimation> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessFeature;

class businessFeatures final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<BusinessFeature>> features_;
  static jfieldID features_fieldID;

  businessFeatures();

  explicit businessFeatures(array<object_ptr<BusinessFeature>> &&features_);

  static const std::int32_t ID = -1532468184;

  static object_ptr<businessFeatures> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessRecipients;

class businessGreetingMessageSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  object_ptr<businessRecipients> recipients_;
  static jfieldID recipients_fieldID;
  int32 inactivity_days_;
  static jfieldID inactivity_days_fieldID;

  businessGreetingMessageSettings();

  businessGreetingMessageSettings(int32 shortcut_id_, object_ptr<businessRecipients> &&recipients_, int32 inactivity_days_);

  static const std::int32_t ID = 1689140754;

  static object_ptr<businessGreetingMessageSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessAwayMessageSettings;

class businessGreetingMessageSettings;

class businessLocation;

class businessOpeningHours;

class businessStartPage;

class businessInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessLocation> location_;
  static jfieldID location_fieldID;
  object_ptr<businessOpeningHours> opening_hours_;
  static jfieldID opening_hours_fieldID;
  object_ptr<businessOpeningHours> local_opening_hours_;
  static jfieldID local_opening_hours_fieldID;
  int32 next_open_in_;
  static jfieldID next_open_in_fieldID;
  int32 next_close_in_;
  static jfieldID next_close_in_fieldID;
  object_ptr<businessGreetingMessageSettings> greeting_message_settings_;
  static jfieldID greeting_message_settings_fieldID;
  object_ptr<businessAwayMessageSettings> away_message_settings_;
  static jfieldID away_message_settings_fieldID;
  object_ptr<businessStartPage> start_page_;
  static jfieldID start_page_fieldID;

  businessInfo();

  businessInfo(object_ptr<businessLocation> &&location_, object_ptr<businessOpeningHours> &&opening_hours_, object_ptr<businessOpeningHours> &&local_opening_hours_, int32 next_open_in_, int32 next_close_in_, object_ptr<businessGreetingMessageSettings> &&greeting_message_settings_, object_ptr<businessAwayMessageSettings> &&away_message_settings_, object_ptr<businessStartPage> &&start_page_);

  static const std::int32_t ID = 1428179342;

  static object_ptr<businessInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class location;

class businessLocation final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  string address_;
  static jfieldID address_fieldID;

  businessLocation();

  businessLocation(object_ptr<location> &&location_, string const &address_);

  static const std::int32_t ID = -1084969126;

  static object_ptr<businessLocation> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class businessMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  object_ptr<message> reply_to_message_;
  static jfieldID reply_to_message_fieldID;

  businessMessage();

  businessMessage(object_ptr<message> &&message_, object_ptr<message> &&reply_to_message_);

  static const std::int32_t ID = -94353850;

  static object_ptr<businessMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessMessage;

class businessMessages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<businessMessage>> messages_;
  static jfieldID messages_fieldID;

  businessMessages();

  explicit businessMessages(array<object_ptr<businessMessage>> &&messages_);

  static const std::int32_t ID = -764562473;

  static object_ptr<businessMessages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessOpeningHoursInterval;

class businessOpeningHours final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string time_zone_id_;
  static jfieldID time_zone_id_fieldID;
  array<object_ptr<businessOpeningHoursInterval>> opening_hours_;
  static jfieldID opening_hours_fieldID;

  businessOpeningHours();

  businessOpeningHours(string const &time_zone_id_, array<object_ptr<businessOpeningHoursInterval>> &&opening_hours_);

  static const std::int32_t ID = 816603700;

  static object_ptr<businessOpeningHours> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessOpeningHoursInterval final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 start_minute_;
  static jfieldID start_minute_fieldID;
  int32 end_minute_;
  static jfieldID end_minute_fieldID;

  businessOpeningHoursInterval();

  businessOpeningHoursInterval(int32 start_minute_, int32 end_minute_);

  static const std::int32_t ID = -1108322732;

  static object_ptr<businessOpeningHoursInterval> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessRecipients final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;
  array<int53> excluded_chat_ids_;
  static jfieldID excluded_chat_ids_fieldID;
  bool select_existing_chats_;
  static jfieldID select_existing_chats_fieldID;
  bool select_new_chats_;
  static jfieldID select_new_chats_fieldID;
  bool select_contacts_;
  static jfieldID select_contacts_fieldID;
  bool select_non_contacts_;
  static jfieldID select_non_contacts_fieldID;
  bool exclude_selected_;
  static jfieldID exclude_selected_fieldID;

  businessRecipients();

  businessRecipients(array<int53> &&chat_ids_, array<int53> &&excluded_chat_ids_, bool select_existing_chats_, bool select_new_chats_, bool select_contacts_, bool select_non_contacts_, bool exclude_selected_);

  static const std::int32_t ID = 868656909;

  static object_ptr<businessRecipients> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class businessStartPage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  string message_;
  static jfieldID message_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  businessStartPage();

  businessStartPage(string const &title_, string const &message_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -1616709681;

  static object_ptr<businessStartPage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallState;

class call final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;
  bool is_video_;
  static jfieldID is_video_fieldID;
  object_ptr<CallState> state_;
  static jfieldID state_fieldID;

  call();

  call(int32 id_, int53 user_id_, bool is_outgoing_, bool is_video_, object_ptr<CallState> &&state_);

  static const std::int32_t ID = 920360804;

  static object_ptr<call> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallDiscardReason: public Object {
 public:
  static jclass Class;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonEmpty final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callDiscardReasonEmpty();

  static const std::int32_t ID = -1258917949;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonMissed final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callDiscardReasonMissed();

  static const std::int32_t ID = 1680358012;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonDeclined final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callDiscardReasonDeclined();

  static const std::int32_t ID = -1729926094;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonDisconnected final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callDiscardReasonDisconnected();

  static const std::int32_t ID = -1342872670;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonHungUp final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callDiscardReasonHungUp();

  static const std::int32_t ID = 438216166;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callDiscardReasonUpgradeToGroupCall final : public CallDiscardReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  callDiscardReasonUpgradeToGroupCall();

  explicit callDiscardReasonUpgradeToGroupCall(string const &invite_link_);

  static const std::int32_t ID = 1254509319;

  static object_ptr<CallDiscardReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callId final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;

  callId();

  explicit callId(int32 id_);

  static const std::int32_t ID = 65717769;

  static object_ptr<callId> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallProblem: public Object {
 public:
  static jclass Class;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callProblemEcho final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemEcho();

  static const std::int32_t ID = 801116548;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemNoise final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemNoise();

  static const std::int32_t ID = 1053065359;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemInterruptions final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemInterruptions();

  static const std::int32_t ID = 1119493218;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemDistortedSpeech final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemDistortedSpeech();

  static const std::int32_t ID = 379960581;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemSilentLocal final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemSilentLocal();

  static const std::int32_t ID = 253652790;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemSilentRemote final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemSilentRemote();

  static const std::int32_t ID = 573634714;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemDropped final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemDropped();

  static const std::int32_t ID = -1207311487;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemDistortedVideo final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemDistortedVideo();

  static const std::int32_t ID = 385245706;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProblemPixelatedVideo final : public CallProblem {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callProblemPixelatedVideo();

  static const std::int32_t ID = 2115315411;

  static object_ptr<CallProblem> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProtocol final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool udp_p2p_;
  static jfieldID udp_p2p_fieldID;
  bool udp_reflector_;
  static jfieldID udp_reflector_fieldID;
  int32 min_layer_;
  static jfieldID min_layer_fieldID;
  int32 max_layer_;
  static jfieldID max_layer_fieldID;
  array<string> library_versions_;
  static jfieldID library_versions_fieldID;

  callProtocol();

  callProtocol(bool udp_p2p_, bool udp_reflector_, int32 min_layer_, int32 max_layer_, array<string> &&library_versions_);

  static const std::int32_t ID = -1075562897;

  static object_ptr<callProtocol> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallServerType;

class callServer final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string ip_address_;
  static jfieldID ip_address_fieldID;
  string ipv6_address_;
  static jfieldID ipv6_address_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  object_ptr<CallServerType> type_;
  static jfieldID type_fieldID;

  callServer();

  callServer(int64 id_, string const &ip_address_, string const &ipv6_address_, int32 port_, object_ptr<CallServerType> &&type_);

  static const std::int32_t ID = 1865932695;

  static object_ptr<callServer> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallServerType: public Object {
 public:
  static jclass Class;

  static object_ptr<CallServerType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callServerTypeTelegramReflector final : public CallServerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes peer_tag_;
  static jfieldID peer_tag_fieldID;
  bool is_tcp_;
  static jfieldID is_tcp_fieldID;

  callServerTypeTelegramReflector();

  callServerTypeTelegramReflector(bytes const &peer_tag_, bool is_tcp_);

  static const std::int32_t ID = 850343189;

  static object_ptr<CallServerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callServerTypeWebrtc final : public CallServerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  string password_;
  static jfieldID password_fieldID;
  bool supports_turn_;
  static jfieldID supports_turn_fieldID;
  bool supports_stun_;
  static jfieldID supports_stun_fieldID;

  callServerTypeWebrtc();

  callServerTypeWebrtc(string const &username_, string const &password_, bool supports_turn_, bool supports_stun_);

  static const std::int32_t ID = 1250622821;

  static object_ptr<CallServerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallDiscardReason;

class callProtocol;

class callServer;

class error;

class CallState: public Object {
 public:
  static jclass Class;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callStatePending final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_created_;
  static jfieldID is_created_fieldID;
  bool is_received_;
  static jfieldID is_received_fieldID;

  callStatePending();

  callStatePending(bool is_created_, bool is_received_);

  static const std::int32_t ID = 1073048620;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callStateExchangingKeys final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callStateExchangingKeys();

  static const std::int32_t ID = -1848149403;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callStateReady final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<callProtocol> protocol_;
  static jfieldID protocol_fieldID;
  array<object_ptr<callServer>> servers_;
  static jfieldID servers_fieldID;
  string config_;
  static jfieldID config_fieldID;
  bytes encryption_key_;
  static jfieldID encryption_key_fieldID;
  array<string> emojis_;
  static jfieldID emojis_fieldID;
  bool allow_p2p_;
  static jfieldID allow_p2p_fieldID;
  bool is_group_call_supported_;
  static jfieldID is_group_call_supported_fieldID;
  string custom_parameters_;
  static jfieldID custom_parameters_fieldID;

  callStateReady();

  callStateReady(object_ptr<callProtocol> &&protocol_, array<object_ptr<callServer>> &&servers_, string const &config_, bytes const &encryption_key_, array<string> &&emojis_, bool allow_p2p_, bool is_group_call_supported_, string const &custom_parameters_);

  static const std::int32_t ID = -281776921;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callStateHangingUp final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  callStateHangingUp();

  static const std::int32_t ID = -2133790038;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callStateDiscarded final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<CallDiscardReason> reason_;
  static jfieldID reason_fieldID;
  bool need_rating_;
  static jfieldID need_rating_fieldID;
  bool need_debug_information_;
  static jfieldID need_debug_information_fieldID;
  bool need_log_;
  static jfieldID need_log_fieldID;

  callStateDiscarded();

  callStateDiscarded(object_ptr<CallDiscardReason> &&reason_, bool need_rating_, bool need_debug_information_, bool need_log_);

  static const std::int32_t ID = 1394310213;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callStateError final : public CallState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<error> error_;
  static jfieldID error_fieldID;

  callStateError();

  explicit callStateError(object_ptr<error> &&error_);

  static const std::int32_t ID = -975215467;

  static object_ptr<CallState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callbackQueryAnswer final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  bool show_alert_;
  static jfieldID show_alert_fieldID;
  string url_;
  static jfieldID url_fieldID;

  callbackQueryAnswer();

  callbackQueryAnswer(string const &text_, bool show_alert_, string const &url_);

  static const std::int32_t ID = 360867933;

  static object_ptr<callbackQueryAnswer> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CallbackQueryPayload: public Object {
 public:
  static jclass Class;

  static object_ptr<CallbackQueryPayload> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callbackQueryPayloadData final : public CallbackQueryPayload {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes data_;
  static jfieldID data_fieldID;

  callbackQueryPayloadData();

  explicit callbackQueryPayloadData(bytes const &data_);

  static const std::int32_t ID = -1977729946;

  static object_ptr<CallbackQueryPayload> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callbackQueryPayloadDataWithPassword final : public CallbackQueryPayload {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  callbackQueryPayloadDataWithPassword();

  callbackQueryPayloadDataWithPassword(string const &password_, bytes const &data_);

  static const std::int32_t ID = 1340266738;

  static object_ptr<CallbackQueryPayload> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callbackQueryPayloadGame final : public CallbackQueryPayload {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string game_short_name_;
  static jfieldID game_short_name_fieldID;

  callbackQueryPayloadGame();

  explicit callbackQueryPayloadGame(string const &game_short_name_);

  static const std::int32_t ID = 1303571512;

  static object_ptr<CallbackQueryPayload> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CanPostStoryResult: public Object {
 public:
  static jclass Class;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultOk final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 story_count_;
  static jfieldID story_count_fieldID;

  canPostStoryResultOk();

  explicit canPostStoryResultOk(int32 story_count_);

  static const std::int32_t ID = 2083205610;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultPremiumNeeded final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canPostStoryResultPremiumNeeded();

  static const std::int32_t ID = 935130501;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultBoostNeeded final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canPostStoryResultBoostNeeded();

  static const std::int32_t ID = 80246195;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultActiveStoryLimitExceeded final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canPostStoryResultActiveStoryLimitExceeded();

  static const std::int32_t ID = -1640759002;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultWeeklyLimitExceeded final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 retry_after_;
  static jfieldID retry_after_fieldID;

  canPostStoryResultWeeklyLimitExceeded();

  explicit canPostStoryResultWeeklyLimitExceeded(int32 retry_after_);

  static const std::int32_t ID = 552858605;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canPostStoryResultMonthlyLimitExceeded final : public CanPostStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 retry_after_;
  static jfieldID retry_after_fieldID;

  canPostStoryResultMonthlyLimitExceeded();

  explicit canPostStoryResultMonthlyLimitExceeded(int32 retry_after_);

  static const std::int32_t ID = -784208562;

  static object_ptr<CanPostStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CanSendMessageToUserResult: public Object {
 public:
  static jclass Class;

  static object_ptr<CanSendMessageToUserResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class canSendMessageToUserResultOk final : public CanSendMessageToUserResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canSendMessageToUserResultOk();

  static const std::int32_t ID = 1530583042;

  static object_ptr<CanSendMessageToUserResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canSendMessageToUserResultUserHasPaidMessages final : public CanSendMessageToUserResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 outgoing_paid_message_star_count_;
  static jfieldID outgoing_paid_message_star_count_fieldID;

  canSendMessageToUserResultUserHasPaidMessages();

  explicit canSendMessageToUserResultUserHasPaidMessages(int53 outgoing_paid_message_star_count_);

  static const std::int32_t ID = -1346487602;

  static object_ptr<CanSendMessageToUserResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canSendMessageToUserResultUserIsDeleted final : public CanSendMessageToUserResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canSendMessageToUserResultUserIsDeleted();

  static const std::int32_t ID = -1944639903;

  static object_ptr<CanSendMessageToUserResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canSendMessageToUserResultUserRestrictsNewChats final : public CanSendMessageToUserResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canSendMessageToUserResultUserRestrictsNewChats();

  static const std::int32_t ID = 1929699797;

  static object_ptr<CanSendMessageToUserResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CanTransferOwnershipResult: public Object {
 public:
  static jclass Class;

  static object_ptr<CanTransferOwnershipResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class canTransferOwnershipResultOk final : public CanTransferOwnershipResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canTransferOwnershipResultOk();

  static const std::int32_t ID = -89881021;

  static object_ptr<CanTransferOwnershipResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canTransferOwnershipResultPasswordNeeded final : public CanTransferOwnershipResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canTransferOwnershipResultPasswordNeeded();

  static const std::int32_t ID = 1548372703;

  static object_ptr<CanTransferOwnershipResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canTransferOwnershipResultPasswordTooFresh final : public CanTransferOwnershipResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 retry_after_;
  static jfieldID retry_after_fieldID;

  canTransferOwnershipResultPasswordTooFresh();

  explicit canTransferOwnershipResultPasswordTooFresh(int32 retry_after_);

  static const std::int32_t ID = 811440913;

  static object_ptr<CanTransferOwnershipResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class canTransferOwnershipResultSessionTooFresh final : public CanTransferOwnershipResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 retry_after_;
  static jfieldID retry_after_fieldID;

  canTransferOwnershipResultSessionTooFresh();

  explicit canTransferOwnershipResultSessionTooFresh(int32 retry_after_);

  static const std::int32_t ID = 984664289;

  static object_ptr<CanTransferOwnershipResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BlockList;

class ChatActionBar;

class ChatAvailableReactions;

class ChatList;

class ChatType;

class MessageSender;

class businessBotManageBar;

class chatBackground;

class chatJoinRequestsInfo;

class chatNotificationSettings;

class chatPermissions;

class chatPhotoInfo;

class chatPosition;

class draftMessage;

class emojiStatus;

class message;

class videoChat;

class chat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  object_ptr<ChatType> type_;
  static jfieldID type_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<chatPhotoInfo> photo_;
  static jfieldID photo_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;
  int32 profile_accent_color_id_;
  static jfieldID profile_accent_color_id_fieldID;
  int64 profile_background_custom_emoji_id_;
  static jfieldID profile_background_custom_emoji_id_fieldID;
  object_ptr<chatPermissions> permissions_;
  static jfieldID permissions_fieldID;
  object_ptr<message> last_message_;
  static jfieldID last_message_fieldID;
  array<object_ptr<chatPosition>> positions_;
  static jfieldID positions_fieldID;
  array<object_ptr<ChatList>> chat_lists_;
  static jfieldID chat_lists_fieldID;
  object_ptr<MessageSender> message_sender_id_;
  static jfieldID message_sender_id_fieldID;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;
  bool has_protected_content_;
  static jfieldID has_protected_content_fieldID;
  bool is_translatable_;
  static jfieldID is_translatable_fieldID;
  bool is_marked_as_unread_;
  static jfieldID is_marked_as_unread_fieldID;
  bool view_as_topics_;
  static jfieldID view_as_topics_fieldID;
  bool has_scheduled_messages_;
  static jfieldID has_scheduled_messages_fieldID;
  bool can_be_deleted_only_for_self_;
  static jfieldID can_be_deleted_only_for_self_fieldID;
  bool can_be_deleted_for_all_users_;
  static jfieldID can_be_deleted_for_all_users_fieldID;
  bool can_be_reported_;
  static jfieldID can_be_reported_fieldID;
  bool default_disable_notification_;
  static jfieldID default_disable_notification_fieldID;
  int32 unread_count_;
  static jfieldID unread_count_fieldID;
  int53 last_read_inbox_message_id_;
  static jfieldID last_read_inbox_message_id_fieldID;
  int53 last_read_outbox_message_id_;
  static jfieldID last_read_outbox_message_id_fieldID;
  int32 unread_mention_count_;
  static jfieldID unread_mention_count_fieldID;
  int32 unread_reaction_count_;
  static jfieldID unread_reaction_count_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;
  object_ptr<ChatAvailableReactions> available_reactions_;
  static jfieldID available_reactions_fieldID;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;
  object_ptr<chatBackground> background_;
  static jfieldID background_fieldID;
  string theme_name_;
  static jfieldID theme_name_fieldID;
  object_ptr<ChatActionBar> action_bar_;
  static jfieldID action_bar_fieldID;
  object_ptr<businessBotManageBar> business_bot_manage_bar_;
  static jfieldID business_bot_manage_bar_fieldID;
  object_ptr<videoChat> video_chat_;
  static jfieldID video_chat_fieldID;
  object_ptr<chatJoinRequestsInfo> pending_join_requests_;
  static jfieldID pending_join_requests_fieldID;
  int53 reply_markup_message_id_;
  static jfieldID reply_markup_message_id_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;
  string client_data_;
  static jfieldID client_data_fieldID;

  chat();

  chat(int53 id_, object_ptr<ChatType> &&type_, string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_, object_ptr<chatPermissions> &&permissions_, object_ptr<message> &&last_message_, array<object_ptr<chatPosition>> &&positions_, array<object_ptr<ChatList>> &&chat_lists_, object_ptr<MessageSender> &&message_sender_id_, object_ptr<BlockList> &&block_list_, bool has_protected_content_, bool is_translatable_, bool is_marked_as_unread_, bool view_as_topics_, bool has_scheduled_messages_, bool can_be_deleted_only_for_self_, bool can_be_deleted_for_all_users_, bool can_be_reported_, bool default_disable_notification_, int32 unread_count_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int32 unread_mention_count_, int32 unread_reaction_count_, object_ptr<chatNotificationSettings> &&notification_settings_, object_ptr<ChatAvailableReactions> &&available_reactions_, int32 message_auto_delete_time_, object_ptr<emojiStatus> &&emoji_status_, object_ptr<chatBackground> &&background_, string const &theme_name_, object_ptr<ChatActionBar> &&action_bar_, object_ptr<businessBotManageBar> &&business_bot_manage_bar_, object_ptr<videoChat> &&video_chat_, object_ptr<chatJoinRequestsInfo> &&pending_join_requests_, int53 reply_markup_message_id_, object_ptr<draftMessage> &&draft_message_, string const &client_data_);

  static const std::int32_t ID = 830601369;

  static object_ptr<chat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatAction: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatActionTyping final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionTyping();

  static const std::int32_t ID = 380122167;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionRecordingVideo final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionRecordingVideo();

  static const std::int32_t ID = 216553362;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionUploadingVideo final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 progress_;
  static jfieldID progress_fieldID;

  chatActionUploadingVideo();

  explicit chatActionUploadingVideo(int32 progress_);

  static const std::int32_t ID = 1234185270;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionRecordingVoiceNote final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionRecordingVoiceNote();

  static const std::int32_t ID = -808850058;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionUploadingVoiceNote final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 progress_;
  static jfieldID progress_fieldID;

  chatActionUploadingVoiceNote();

  explicit chatActionUploadingVoiceNote(int32 progress_);

  static const std::int32_t ID = -613643666;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionUploadingPhoto final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 progress_;
  static jfieldID progress_fieldID;

  chatActionUploadingPhoto();

  explicit chatActionUploadingPhoto(int32 progress_);

  static const std::int32_t ID = 654240583;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionUploadingDocument final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 progress_;
  static jfieldID progress_fieldID;

  chatActionUploadingDocument();

  explicit chatActionUploadingDocument(int32 progress_);

  static const std::int32_t ID = 167884362;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionChoosingSticker final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionChoosingSticker();

  static const std::int32_t ID = 372753697;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionChoosingLocation final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionChoosingLocation();

  static const std::int32_t ID = -2017893596;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionChoosingContact final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionChoosingContact();

  static const std::int32_t ID = -1222507496;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionStartPlayingGame final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionStartPlayingGame();

  static const std::int32_t ID = -865884164;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionRecordingVideoNote final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionRecordingVideoNote();

  static const std::int32_t ID = 16523393;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionUploadingVideoNote final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 progress_;
  static jfieldID progress_fieldID;

  chatActionUploadingVideoNote();

  explicit chatActionUploadingVideoNote(int32 progress_);

  static const std::int32_t ID = 1172364918;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionWatchingAnimations final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;

  chatActionWatchingAnimations();

  explicit chatActionWatchingAnimations(string const &emoji_);

  static const std::int32_t ID = 2052990641;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionCancel final : public ChatAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionCancel();

  static const std::int32_t ID = 1160523958;

  static object_ptr<ChatAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class accountInfo;

class ChatActionBar: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarReportSpam final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_unarchive_;
  static jfieldID can_unarchive_fieldID;

  chatActionBarReportSpam();

  explicit chatActionBarReportSpam(bool can_unarchive_);

  static const std::int32_t ID = -1312758246;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarInviteMembers final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionBarInviteMembers();

  static const std::int32_t ID = 1985313904;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarReportAddBlock final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_unarchive_;
  static jfieldID can_unarchive_fieldID;
  object_ptr<accountInfo> account_info_;
  static jfieldID account_info_fieldID;

  chatActionBarReportAddBlock();

  chatActionBarReportAddBlock(bool can_unarchive_, object_ptr<accountInfo> &&account_info_);

  static const std::int32_t ID = -1476817269;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarAddContact final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionBarAddContact();

  static const std::int32_t ID = -733325295;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarSharePhoneNumber final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatActionBarSharePhoneNumber();

  static const std::int32_t ID = 35188697;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatActionBarJoinRequest final : public ChatActionBar {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  bool is_channel_;
  static jfieldID is_channel_fieldID;
  int32 request_date_;
  static jfieldID request_date_fieldID;

  chatActionBarJoinRequest();

  chatActionBarJoinRequest(string const &title_, bool is_channel_, int32 request_date_);

  static const std::int32_t ID = 1037140744;

  static object_ptr<ChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryList;

class storyInfo;

class chatActiveStories final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<StoryList> list_;
  static jfieldID list_fieldID;
  int53 order_;
  static jfieldID order_fieldID;
  int32 max_read_story_id_;
  static jfieldID max_read_story_id_fieldID;
  array<object_ptr<storyInfo>> stories_;
  static jfieldID stories_fieldID;

  chatActiveStories();

  chatActiveStories(int53 chat_id_, object_ptr<StoryList> &&list_, int53 order_, int32 max_read_story_id_, array<object_ptr<storyInfo>> &&stories_);

  static const std::int32_t ID = -1398869529;

  static object_ptr<chatActiveStories> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministrator final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string custom_title_;
  static jfieldID custom_title_fieldID;
  bool is_owner_;
  static jfieldID is_owner_fieldID;

  chatAdministrator();

  chatAdministrator(int53 user_id_, string const &custom_title_, bool is_owner_);

  static const std::int32_t ID = 1920449836;

  static object_ptr<chatAdministrator> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministratorRights final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_manage_chat_;
  static jfieldID can_manage_chat_fieldID;
  bool can_change_info_;
  static jfieldID can_change_info_fieldID;
  bool can_post_messages_;
  static jfieldID can_post_messages_fieldID;
  bool can_edit_messages_;
  static jfieldID can_edit_messages_fieldID;
  bool can_delete_messages_;
  static jfieldID can_delete_messages_fieldID;
  bool can_invite_users_;
  static jfieldID can_invite_users_fieldID;
  bool can_restrict_members_;
  static jfieldID can_restrict_members_fieldID;
  bool can_pin_messages_;
  static jfieldID can_pin_messages_fieldID;
  bool can_manage_topics_;
  static jfieldID can_manage_topics_fieldID;
  bool can_promote_members_;
  static jfieldID can_promote_members_fieldID;
  bool can_manage_video_chats_;
  static jfieldID can_manage_video_chats_fieldID;
  bool can_post_stories_;
  static jfieldID can_post_stories_fieldID;
  bool can_edit_stories_;
  static jfieldID can_edit_stories_fieldID;
  bool can_delete_stories_;
  static jfieldID can_delete_stories_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;

  chatAdministratorRights();

  chatAdministratorRights(bool can_manage_chat_, bool can_change_info_, bool can_post_messages_, bool can_edit_messages_, bool can_delete_messages_, bool can_invite_users_, bool can_restrict_members_, bool can_pin_messages_, bool can_manage_topics_, bool can_promote_members_, bool can_manage_video_chats_, bool can_post_stories_, bool can_edit_stories_, bool can_delete_stories_, bool is_anonymous_);

  static const std::int32_t ID = 1599049796;

  static object_ptr<chatAdministratorRights> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministrator;

class chatAdministrators final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatAdministrator>> administrators_;
  static jfieldID administrators_fieldID;

  chatAdministrators();

  explicit chatAdministrators(array<object_ptr<chatAdministrator>> &&administrators_);

  static const std::int32_t ID = -2126186435;

  static object_ptr<chatAdministrators> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ChatAvailableReactions: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatAvailableReactions> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatAvailableReactionsAll final : public ChatAvailableReactions {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 max_reaction_count_;
  static jfieldID max_reaction_count_fieldID;

  chatAvailableReactionsAll();

  explicit chatAvailableReactionsAll(int32 max_reaction_count_);

  static const std::int32_t ID = 694160279;

  static object_ptr<ChatAvailableReactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAvailableReactionsSome final : public ChatAvailableReactions {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<ReactionType>> reactions_;
  static jfieldID reactions_fieldID;
  int32 max_reaction_count_;
  static jfieldID max_reaction_count_fieldID;

  chatAvailableReactionsSome();

  chatAvailableReactionsSome(array<object_ptr<ReactionType>> &&reactions_, int32 max_reaction_count_);

  static const std::int32_t ID = 152513153;

  static object_ptr<ChatAvailableReactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class background;

class chatBackground final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<background> background_;
  static jfieldID background_fieldID;
  int32 dark_theme_dimming_;
  static jfieldID dark_theme_dimming_fieldID;

  chatBackground();

  chatBackground(object_ptr<background> &&background_, int32 dark_theme_dimming_);

  static const std::int32_t ID = 1653152104;

  static object_ptr<chatBackground> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatBoostSource;

class chatBoost final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  int32 count_;
  static jfieldID count_fieldID;
  object_ptr<ChatBoostSource> source_;
  static jfieldID source_fieldID;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;

  chatBoost();

  chatBoost(string const &id_, int32 count_, object_ptr<ChatBoostSource> &&source_, int32 start_date_, int32 expiration_date_);

  static const std::int32_t ID = -1765815118;

  static object_ptr<chatBoost> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLevelFeatures;

class chatBoostFeatures final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatBoostLevelFeatures>> features_;
  static jfieldID features_fieldID;
  int32 min_profile_background_custom_emoji_boost_level_;
  static jfieldID min_profile_background_custom_emoji_boost_level_fieldID;
  int32 min_background_custom_emoji_boost_level_;
  static jfieldID min_background_custom_emoji_boost_level_fieldID;
  int32 min_emoji_status_boost_level_;
  static jfieldID min_emoji_status_boost_level_fieldID;
  int32 min_chat_theme_background_boost_level_;
  static jfieldID min_chat_theme_background_boost_level_fieldID;
  int32 min_custom_background_boost_level_;
  static jfieldID min_custom_background_boost_level_fieldID;
  int32 min_custom_emoji_sticker_set_boost_level_;
  static jfieldID min_custom_emoji_sticker_set_boost_level_fieldID;
  int32 min_automatic_translation_boost_level_;
  static jfieldID min_automatic_translation_boost_level_fieldID;
  int32 min_speech_recognition_boost_level_;
  static jfieldID min_speech_recognition_boost_level_fieldID;
  int32 min_sponsored_message_disable_boost_level_;
  static jfieldID min_sponsored_message_disable_boost_level_fieldID;

  chatBoostFeatures();

  chatBoostFeatures(array<object_ptr<chatBoostLevelFeatures>> &&features_, int32 min_profile_background_custom_emoji_boost_level_, int32 min_background_custom_emoji_boost_level_, int32 min_emoji_status_boost_level_, int32 min_chat_theme_background_boost_level_, int32 min_custom_background_boost_level_, int32 min_custom_emoji_sticker_set_boost_level_, int32 min_automatic_translation_boost_level_, int32 min_speech_recognition_boost_level_, int32 min_sponsored_message_disable_boost_level_);

  static const std::int32_t ID = -940531367;

  static object_ptr<chatBoostFeatures> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLevelFeatures final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 level_;
  static jfieldID level_fieldID;
  int32 story_per_day_count_;
  static jfieldID story_per_day_count_fieldID;
  int32 custom_emoji_reaction_count_;
  static jfieldID custom_emoji_reaction_count_fieldID;
  int32 title_color_count_;
  static jfieldID title_color_count_fieldID;
  int32 profile_accent_color_count_;
  static jfieldID profile_accent_color_count_fieldID;
  bool can_set_profile_background_custom_emoji_;
  static jfieldID can_set_profile_background_custom_emoji_fieldID;
  int32 accent_color_count_;
  static jfieldID accent_color_count_fieldID;
  bool can_set_background_custom_emoji_;
  static jfieldID can_set_background_custom_emoji_fieldID;
  bool can_set_emoji_status_;
  static jfieldID can_set_emoji_status_fieldID;
  int32 chat_theme_background_count_;
  static jfieldID chat_theme_background_count_fieldID;
  bool can_set_custom_background_;
  static jfieldID can_set_custom_background_fieldID;
  bool can_set_custom_emoji_sticker_set_;
  static jfieldID can_set_custom_emoji_sticker_set_fieldID;
  bool can_enable_automatic_translation_;
  static jfieldID can_enable_automatic_translation_fieldID;
  bool can_recognize_speech_;
  static jfieldID can_recognize_speech_fieldID;
  bool can_disable_sponsored_messages_;
  static jfieldID can_disable_sponsored_messages_fieldID;

  chatBoostLevelFeatures();

  chatBoostLevelFeatures(int32 level_, int32 story_per_day_count_, int32 custom_emoji_reaction_count_, int32 title_color_count_, int32 profile_accent_color_count_, bool can_set_profile_background_custom_emoji_, int32 accent_color_count_, bool can_set_background_custom_emoji_, bool can_set_emoji_status_, int32 chat_theme_background_count_, bool can_set_custom_background_, bool can_set_custom_emoji_sticker_set_, bool can_enable_automatic_translation_, bool can_recognize_speech_, bool can_disable_sponsored_messages_);

  static const std::int32_t ID = 975439470;

  static object_ptr<chatBoostLevelFeatures> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;
  bool is_public_;
  static jfieldID is_public_fieldID;

  chatBoostLink();

  chatBoostLink(string const &link_, bool is_public_);

  static const std::int32_t ID = -1253999503;

  static object_ptr<chatBoostLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_public_;
  static jfieldID is_public_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  chatBoostLinkInfo();

  chatBoostLinkInfo(bool is_public_, int53 chat_id_);

  static const std::int32_t ID = -602785660;

  static object_ptr<chatBoostLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSlot final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 slot_id_;
  static jfieldID slot_id_fieldID;
  int53 currently_boosted_chat_id_;
  static jfieldID currently_boosted_chat_id_fieldID;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;
  int32 cooldown_until_date_;
  static jfieldID cooldown_until_date_fieldID;

  chatBoostSlot();

  chatBoostSlot(int32 slot_id_, int53 currently_boosted_chat_id_, int32 start_date_, int32 expiration_date_, int32 cooldown_until_date_);

  static const std::int32_t ID = 123206343;

  static object_ptr<chatBoostSlot> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSlot;

class chatBoostSlots final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatBoostSlot>> slots_;
  static jfieldID slots_fieldID;

  chatBoostSlots();

  explicit chatBoostSlots(array<object_ptr<chatBoostSlot>> &&slots_);

  static const std::int32_t ID = 1014966293;

  static object_ptr<chatBoostSlots> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatBoostSource: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatBoostSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSourceGiftCode final : public ChatBoostSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string gift_code_;
  static jfieldID gift_code_fieldID;

  chatBoostSourceGiftCode();

  chatBoostSourceGiftCode(int53 user_id_, string const &gift_code_);

  static const std::int32_t ID = -98299206;

  static object_ptr<ChatBoostSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSourceGiveaway final : public ChatBoostSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string gift_code_;
  static jfieldID gift_code_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;
  bool is_unclaimed_;
  static jfieldID is_unclaimed_fieldID;

  chatBoostSourceGiveaway();

  chatBoostSourceGiveaway(int53 user_id_, string const &gift_code_, int53 star_count_, int53 giveaway_message_id_, bool is_unclaimed_);

  static const std::int32_t ID = 1918145690;

  static object_ptr<ChatBoostSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSourcePremium final : public ChatBoostSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  chatBoostSourcePremium();

  explicit chatBoostSourcePremium(int53 user_id_);

  static const std::int32_t ID = 972011;

  static object_ptr<ChatBoostSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class prepaidGiveaway;

class chatBoostStatus final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string boost_url_;
  static jfieldID boost_url_fieldID;
  array<int32> applied_slot_ids_;
  static jfieldID applied_slot_ids_fieldID;
  int32 level_;
  static jfieldID level_fieldID;
  int32 gift_code_boost_count_;
  static jfieldID gift_code_boost_count_fieldID;
  int32 boost_count_;
  static jfieldID boost_count_fieldID;
  int32 current_level_boost_count_;
  static jfieldID current_level_boost_count_fieldID;
  int32 next_level_boost_count_;
  static jfieldID next_level_boost_count_fieldID;
  int32 premium_member_count_;
  static jfieldID premium_member_count_fieldID;
  double premium_member_percentage_;
  static jfieldID premium_member_percentage_fieldID;
  array<object_ptr<prepaidGiveaway>> prepaid_giveaways_;
  static jfieldID prepaid_giveaways_fieldID;

  chatBoostStatus();

  chatBoostStatus(string const &boost_url_, array<int32> &&applied_slot_ids_, int32 level_, int32 gift_code_boost_count_, int32 boost_count_, int32 current_level_boost_count_, int32 next_level_boost_count_, int32 premium_member_count_, double premium_member_percentage_, array<object_ptr<prepaidGiveaway>> &&prepaid_giveaways_);

  static const std::int32_t ID = -1050332618;

  static object_ptr<chatBoostStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatEventAction;

class MessageSender;

class chatEvent final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;
  object_ptr<ChatEventAction> action_;
  static jfieldID action_fieldID;

  chatEvent();

  chatEvent(int64 id_, int32 date_, object_ptr<MessageSender> &&member_id_, object_ptr<ChatEventAction> &&action_);

  static const std::int32_t ID = -652102704;

  static object_ptr<chatEvent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatAvailableReactions;

class ChatMemberStatus;

class MessageSender;

class chatBackground;

class chatInviteLink;

class chatLocation;

class chatPermissions;

class chatPhoto;

class emojiStatus;

class forumTopicInfo;

class message;

class ChatEventAction: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMessageEdited final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> old_message_;
  static jfieldID old_message_fieldID;
  object_ptr<message> new_message_;
  static jfieldID new_message_fieldID;

  chatEventMessageEdited();

  chatEventMessageEdited(object_ptr<message> &&old_message_, object_ptr<message> &&new_message_);

  static const std::int32_t ID = -430967304;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMessageDeleted final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  bool can_report_anti_spam_false_positive_;
  static jfieldID can_report_anti_spam_false_positive_fieldID;

  chatEventMessageDeleted();

  chatEventMessageDeleted(object_ptr<message> &&message_, bool can_report_anti_spam_false_positive_);

  static const std::int32_t ID = 935316851;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMessagePinned final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  chatEventMessagePinned();

  explicit chatEventMessagePinned(object_ptr<message> &&message_);

  static const std::int32_t ID = 438742298;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMessageUnpinned final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  chatEventMessageUnpinned();

  explicit chatEventMessageUnpinned(object_ptr<message> &&message_);

  static const std::int32_t ID = -376161513;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventPollStopped final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  chatEventPollStopped();

  explicit chatEventPollStopped(object_ptr<message> &&message_);

  static const std::int32_t ID = 2009893861;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberJoined final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatEventMemberJoined();

  static const std::int32_t ID = -235468508;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberJoinedByInviteLink final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;
  bool via_chat_folder_invite_link_;
  static jfieldID via_chat_folder_invite_link_fieldID;

  chatEventMemberJoinedByInviteLink();

  chatEventMemberJoinedByInviteLink(object_ptr<chatInviteLink> &&invite_link_, bool via_chat_folder_invite_link_);

  static const std::int32_t ID = -1445536390;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberJoinedByRequest final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 approver_user_id_;
  static jfieldID approver_user_id_fieldID;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;

  chatEventMemberJoinedByRequest();

  chatEventMemberJoinedByRequest(int53 approver_user_id_, object_ptr<chatInviteLink> &&invite_link_);

  static const std::int32_t ID = -1647804865;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberInvited final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<ChatMemberStatus> status_;
  static jfieldID status_fieldID;

  chatEventMemberInvited();

  chatEventMemberInvited(int53 user_id_, object_ptr<ChatMemberStatus> &&status_);

  static const std::int32_t ID = 953663433;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberLeft final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatEventMemberLeft();

  static const std::int32_t ID = -948420593;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberPromoted final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<ChatMemberStatus> old_status_;
  static jfieldID old_status_fieldID;
  object_ptr<ChatMemberStatus> new_status_;
  static jfieldID new_status_fieldID;

  chatEventMemberPromoted();

  chatEventMemberPromoted(int53 user_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_);

  static const std::int32_t ID = 525297761;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberRestricted final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;
  object_ptr<ChatMemberStatus> old_status_;
  static jfieldID old_status_fieldID;
  object_ptr<ChatMemberStatus> new_status_;
  static jfieldID new_status_fieldID;

  chatEventMemberRestricted();

  chatEventMemberRestricted(object_ptr<MessageSender> &&member_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_);

  static const std::int32_t ID = 1603608069;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMemberSubscriptionExtended final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<ChatMemberStatus> old_status_;
  static jfieldID old_status_fieldID;
  object_ptr<ChatMemberStatus> new_status_;
  static jfieldID new_status_fieldID;

  chatEventMemberSubscriptionExtended();

  chatEventMemberSubscriptionExtended(int53 user_id_, object_ptr<ChatMemberStatus> &&old_status_, object_ptr<ChatMemberStatus> &&new_status_);

  static const std::int32_t ID = -1141198846;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventAvailableReactionsChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatAvailableReactions> old_available_reactions_;
  static jfieldID old_available_reactions_fieldID;
  object_ptr<ChatAvailableReactions> new_available_reactions_;
  static jfieldID new_available_reactions_fieldID;

  chatEventAvailableReactionsChanged();

  chatEventAvailableReactionsChanged(object_ptr<ChatAvailableReactions> &&old_available_reactions_, object_ptr<ChatAvailableReactions> &&new_available_reactions_);

  static const std::int32_t ID = -1749491521;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventBackgroundChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatBackground> old_background_;
  static jfieldID old_background_fieldID;
  object_ptr<chatBackground> new_background_;
  static jfieldID new_background_fieldID;

  chatEventBackgroundChanged();

  chatEventBackgroundChanged(object_ptr<chatBackground> &&old_background_, object_ptr<chatBackground> &&new_background_);

  static const std::int32_t ID = -1225953992;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventDescriptionChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string old_description_;
  static jfieldID old_description_fieldID;
  string new_description_;
  static jfieldID new_description_fieldID;

  chatEventDescriptionChanged();

  chatEventDescriptionChanged(string const &old_description_, string const &new_description_);

  static const std::int32_t ID = 39112478;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventEmojiStatusChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<emojiStatus> old_emoji_status_;
  static jfieldID old_emoji_status_fieldID;
  object_ptr<emojiStatus> new_emoji_status_;
  static jfieldID new_emoji_status_fieldID;

  chatEventEmojiStatusChanged();

  chatEventEmojiStatusChanged(object_ptr<emojiStatus> &&old_emoji_status_, object_ptr<emojiStatus> &&new_emoji_status_);

  static const std::int32_t ID = -2081850594;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventLinkedChatChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 old_linked_chat_id_;
  static jfieldID old_linked_chat_id_fieldID;
  int53 new_linked_chat_id_;
  static jfieldID new_linked_chat_id_fieldID;

  chatEventLinkedChatChanged();

  chatEventLinkedChatChanged(int53 old_linked_chat_id_, int53 new_linked_chat_id_);

  static const std::int32_t ID = 1797419439;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventLocationChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatLocation> old_location_;
  static jfieldID old_location_fieldID;
  object_ptr<chatLocation> new_location_;
  static jfieldID new_location_fieldID;

  chatEventLocationChanged();

  chatEventLocationChanged(object_ptr<chatLocation> &&old_location_, object_ptr<chatLocation> &&new_location_);

  static const std::int32_t ID = -405930674;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventMessageAutoDeleteTimeChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 old_message_auto_delete_time_;
  static jfieldID old_message_auto_delete_time_fieldID;
  int32 new_message_auto_delete_time_;
  static jfieldID new_message_auto_delete_time_fieldID;

  chatEventMessageAutoDeleteTimeChanged();

  chatEventMessageAutoDeleteTimeChanged(int32 old_message_auto_delete_time_, int32 new_message_auto_delete_time_);

  static const std::int32_t ID = 17317668;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventPermissionsChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPermissions> old_permissions_;
  static jfieldID old_permissions_fieldID;
  object_ptr<chatPermissions> new_permissions_;
  static jfieldID new_permissions_fieldID;

  chatEventPermissionsChanged();

  chatEventPermissionsChanged(object_ptr<chatPermissions> &&old_permissions_, object_ptr<chatPermissions> &&new_permissions_);

  static const std::int32_t ID = -1311557720;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventPhotoChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> old_photo_;
  static jfieldID old_photo_fieldID;
  object_ptr<chatPhoto> new_photo_;
  static jfieldID new_photo_fieldID;

  chatEventPhotoChanged();

  chatEventPhotoChanged(object_ptr<chatPhoto> &&old_photo_, object_ptr<chatPhoto> &&new_photo_);

  static const std::int32_t ID = -811572541;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventSlowModeDelayChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 old_slow_mode_delay_;
  static jfieldID old_slow_mode_delay_fieldID;
  int32 new_slow_mode_delay_;
  static jfieldID new_slow_mode_delay_fieldID;

  chatEventSlowModeDelayChanged();

  chatEventSlowModeDelayChanged(int32 old_slow_mode_delay_, int32 new_slow_mode_delay_);

  static const std::int32_t ID = -1653195765;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventStickerSetChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 old_sticker_set_id_;
  static jfieldID old_sticker_set_id_fieldID;
  int64 new_sticker_set_id_;
  static jfieldID new_sticker_set_id_fieldID;

  chatEventStickerSetChanged();

  chatEventStickerSetChanged(int64 old_sticker_set_id_, int64 new_sticker_set_id_);

  static const std::int32_t ID = -1243130481;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventCustomEmojiStickerSetChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 old_sticker_set_id_;
  static jfieldID old_sticker_set_id_fieldID;
  int64 new_sticker_set_id_;
  static jfieldID new_sticker_set_id_fieldID;

  chatEventCustomEmojiStickerSetChanged();

  chatEventCustomEmojiStickerSetChanged(int64 old_sticker_set_id_, int64 new_sticker_set_id_);

  static const std::int32_t ID = 118244123;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventTitleChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string old_title_;
  static jfieldID old_title_fieldID;
  string new_title_;
  static jfieldID new_title_fieldID;

  chatEventTitleChanged();

  chatEventTitleChanged(string const &old_title_, string const &new_title_);

  static const std::int32_t ID = 1134103250;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventUsernameChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string old_username_;
  static jfieldID old_username_fieldID;
  string new_username_;
  static jfieldID new_username_fieldID;

  chatEventUsernameChanged();

  chatEventUsernameChanged(string const &old_username_, string const &new_username_);

  static const std::int32_t ID = 1728558443;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventActiveUsernamesChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> old_usernames_;
  static jfieldID old_usernames_fieldID;
  array<string> new_usernames_;
  static jfieldID new_usernames_fieldID;

  chatEventActiveUsernamesChanged();

  chatEventActiveUsernamesChanged(array<string> &&old_usernames_, array<string> &&new_usernames_);

  static const std::int32_t ID = -1508790810;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventAccentColorChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 old_accent_color_id_;
  static jfieldID old_accent_color_id_fieldID;
  int64 old_background_custom_emoji_id_;
  static jfieldID old_background_custom_emoji_id_fieldID;
  int32 new_accent_color_id_;
  static jfieldID new_accent_color_id_fieldID;
  int64 new_background_custom_emoji_id_;
  static jfieldID new_background_custom_emoji_id_fieldID;

  chatEventAccentColorChanged();

  chatEventAccentColorChanged(int32 old_accent_color_id_, int64 old_background_custom_emoji_id_, int32 new_accent_color_id_, int64 new_background_custom_emoji_id_);

  static const std::int32_t ID = -427591885;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventProfileAccentColorChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 old_profile_accent_color_id_;
  static jfieldID old_profile_accent_color_id_fieldID;
  int64 old_profile_background_custom_emoji_id_;
  static jfieldID old_profile_background_custom_emoji_id_fieldID;
  int32 new_profile_accent_color_id_;
  static jfieldID new_profile_accent_color_id_fieldID;
  int64 new_profile_background_custom_emoji_id_;
  static jfieldID new_profile_background_custom_emoji_id_fieldID;

  chatEventProfileAccentColorChanged();

  chatEventProfileAccentColorChanged(int32 old_profile_accent_color_id_, int64 old_profile_background_custom_emoji_id_, int32 new_profile_accent_color_id_, int64 new_profile_background_custom_emoji_id_);

  static const std::int32_t ID = -1514612124;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventHasProtectedContentToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_protected_content_;
  static jfieldID has_protected_content_fieldID;

  chatEventHasProtectedContentToggled();

  explicit chatEventHasProtectedContentToggled(bool has_protected_content_);

  static const std::int32_t ID = -184270335;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventInvitesToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_invite_users_;
  static jfieldID can_invite_users_fieldID;

  chatEventInvitesToggled();

  explicit chatEventInvitesToggled(bool can_invite_users_);

  static const std::int32_t ID = -62548373;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventIsAllHistoryAvailableToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_all_history_available_;
  static jfieldID is_all_history_available_fieldID;

  chatEventIsAllHistoryAvailableToggled();

  explicit chatEventIsAllHistoryAvailableToggled(bool is_all_history_available_);

  static const std::int32_t ID = -1599063019;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventHasAggressiveAntiSpamEnabledToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_aggressive_anti_spam_enabled_;
  static jfieldID has_aggressive_anti_spam_enabled_fieldID;

  chatEventHasAggressiveAntiSpamEnabledToggled();

  explicit chatEventHasAggressiveAntiSpamEnabledToggled(bool has_aggressive_anti_spam_enabled_);

  static const std::int32_t ID = -125348094;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventSignMessagesToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool sign_messages_;
  static jfieldID sign_messages_fieldID;

  chatEventSignMessagesToggled();

  explicit chatEventSignMessagesToggled(bool sign_messages_);

  static const std::int32_t ID = -1313265634;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventShowMessageSenderToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool show_message_sender_;
  static jfieldID show_message_sender_fieldID;

  chatEventShowMessageSenderToggled();

  explicit chatEventShowMessageSenderToggled(bool show_message_sender_);

  static const std::int32_t ID = -794343453;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventAutomaticTranslationToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_automatic_translation_;
  static jfieldID has_automatic_translation_fieldID;

  chatEventAutomaticTranslationToggled();

  explicit chatEventAutomaticTranslationToggled(bool has_automatic_translation_);

  static const std::int32_t ID = 194147926;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventInviteLinkEdited final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatInviteLink> old_invite_link_;
  static jfieldID old_invite_link_fieldID;
  object_ptr<chatInviteLink> new_invite_link_;
  static jfieldID new_invite_link_fieldID;

  chatEventInviteLinkEdited();

  chatEventInviteLinkEdited(object_ptr<chatInviteLink> &&old_invite_link_, object_ptr<chatInviteLink> &&new_invite_link_);

  static const std::int32_t ID = -460190366;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventInviteLinkRevoked final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;

  chatEventInviteLinkRevoked();

  explicit chatEventInviteLinkRevoked(object_ptr<chatInviteLink> &&invite_link_);

  static const std::int32_t ID = -1579417629;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventInviteLinkDeleted final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;

  chatEventInviteLinkDeleted();

  explicit chatEventInviteLinkDeleted(object_ptr<chatInviteLink> &&invite_link_);

  static const std::int32_t ID = -1394974361;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventVideoChatCreated final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  chatEventVideoChatCreated();

  explicit chatEventVideoChatCreated(int32 group_call_id_);

  static const std::int32_t ID = 1822853755;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventVideoChatEnded final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  chatEventVideoChatEnded();

  explicit chatEventVideoChatEnded(int32 group_call_id_);

  static const std::int32_t ID = 1630039112;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventVideoChatMuteNewParticipantsToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool mute_new_participants_;
  static jfieldID mute_new_participants_fieldID;

  chatEventVideoChatMuteNewParticipantsToggled();

  explicit chatEventVideoChatMuteNewParticipantsToggled(bool mute_new_participants_);

  static const std::int32_t ID = -126547970;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventVideoChatParticipantIsMutedToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  bool is_muted_;
  static jfieldID is_muted_fieldID;

  chatEventVideoChatParticipantIsMutedToggled();

  chatEventVideoChatParticipantIsMutedToggled(object_ptr<MessageSender> &&participant_id_, bool is_muted_);

  static const std::int32_t ID = 521165047;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventVideoChatParticipantVolumeLevelChanged final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  int32 volume_level_;
  static jfieldID volume_level_fieldID;

  chatEventVideoChatParticipantVolumeLevelChanged();

  chatEventVideoChatParticipantVolumeLevelChanged(object_ptr<MessageSender> &&participant_id_, int32 volume_level_);

  static const std::int32_t ID = 1131385534;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventIsForumToggled final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_forum_;
  static jfieldID is_forum_fieldID;

  chatEventIsForumToggled();

  explicit chatEventIsForumToggled(bool is_forum_);

  static const std::int32_t ID = 1516491033;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicCreated final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> topic_info_;
  static jfieldID topic_info_fieldID;

  chatEventForumTopicCreated();

  explicit chatEventForumTopicCreated(object_ptr<forumTopicInfo> &&topic_info_);

  static const std::int32_t ID = 2005269314;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicEdited final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> old_topic_info_;
  static jfieldID old_topic_info_fieldID;
  object_ptr<forumTopicInfo> new_topic_info_;
  static jfieldID new_topic_info_fieldID;

  chatEventForumTopicEdited();

  chatEventForumTopicEdited(object_ptr<forumTopicInfo> &&old_topic_info_, object_ptr<forumTopicInfo> &&new_topic_info_);

  static const std::int32_t ID = 1624910860;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicToggleIsClosed final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> topic_info_;
  static jfieldID topic_info_fieldID;

  chatEventForumTopicToggleIsClosed();

  explicit chatEventForumTopicToggleIsClosed(object_ptr<forumTopicInfo> &&topic_info_);

  static const std::int32_t ID = -962704070;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicToggleIsHidden final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> topic_info_;
  static jfieldID topic_info_fieldID;

  chatEventForumTopicToggleIsHidden();

  explicit chatEventForumTopicToggleIsHidden(object_ptr<forumTopicInfo> &&topic_info_);

  static const std::int32_t ID = -1609175250;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicDeleted final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> topic_info_;
  static jfieldID topic_info_fieldID;

  chatEventForumTopicDeleted();

  explicit chatEventForumTopicDeleted(object_ptr<forumTopicInfo> &&topic_info_);

  static const std::int32_t ID = -1332795123;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventForumTopicPinned final : public ChatEventAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> old_topic_info_;
  static jfieldID old_topic_info_fieldID;
  object_ptr<forumTopicInfo> new_topic_info_;
  static jfieldID new_topic_info_fieldID;

  chatEventForumTopicPinned();

  chatEventForumTopicPinned(object_ptr<forumTopicInfo> &&old_topic_info_, object_ptr<forumTopicInfo> &&new_topic_info_);

  static const std::int32_t ID = 2143626222;

  static object_ptr<ChatEventAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEventLogFilters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool message_edits_;
  static jfieldID message_edits_fieldID;
  bool message_deletions_;
  static jfieldID message_deletions_fieldID;
  bool message_pins_;
  static jfieldID message_pins_fieldID;
  bool member_joins_;
  static jfieldID member_joins_fieldID;
  bool member_leaves_;
  static jfieldID member_leaves_fieldID;
  bool member_invites_;
  static jfieldID member_invites_fieldID;
  bool member_promotions_;
  static jfieldID member_promotions_fieldID;
  bool member_restrictions_;
  static jfieldID member_restrictions_fieldID;
  bool info_changes_;
  static jfieldID info_changes_fieldID;
  bool setting_changes_;
  static jfieldID setting_changes_fieldID;
  bool invite_link_changes_;
  static jfieldID invite_link_changes_fieldID;
  bool video_chat_changes_;
  static jfieldID video_chat_changes_fieldID;
  bool forum_changes_;
  static jfieldID forum_changes_fieldID;
  bool subscription_extensions_;
  static jfieldID subscription_extensions_fieldID;

  chatEventLogFilters();

  chatEventLogFilters(bool message_edits_, bool message_deletions_, bool message_pins_, bool member_joins_, bool member_leaves_, bool member_invites_, bool member_promotions_, bool member_restrictions_, bool info_changes_, bool setting_changes_, bool invite_link_changes_, bool video_chat_changes_, bool forum_changes_, bool subscription_extensions_);

  static const std::int32_t ID = -1032965711;

  static object_ptr<chatEventLogFilters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatEvent;

class chatEvents final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatEvent>> events_;
  static jfieldID events_fieldID;

  chatEvents();

  explicit chatEvents(array<object_ptr<chatEvent>> &&events_);

  static const std::int32_t ID = -585329664;

  static object_ptr<chatEvents> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderIcon;

class chatFolderName;

class chatFolder final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolderName> name_;
  static jfieldID name_fieldID;
  object_ptr<chatFolderIcon> icon_;
  static jfieldID icon_fieldID;
  int32 color_id_;
  static jfieldID color_id_fieldID;
  bool is_shareable_;
  static jfieldID is_shareable_fieldID;
  array<int53> pinned_chat_ids_;
  static jfieldID pinned_chat_ids_fieldID;
  array<int53> included_chat_ids_;
  static jfieldID included_chat_ids_fieldID;
  array<int53> excluded_chat_ids_;
  static jfieldID excluded_chat_ids_fieldID;
  bool exclude_muted_;
  static jfieldID exclude_muted_fieldID;
  bool exclude_read_;
  static jfieldID exclude_read_fieldID;
  bool exclude_archived_;
  static jfieldID exclude_archived_fieldID;
  bool include_contacts_;
  static jfieldID include_contacts_fieldID;
  bool include_non_contacts_;
  static jfieldID include_non_contacts_fieldID;
  bool include_bots_;
  static jfieldID include_bots_fieldID;
  bool include_groups_;
  static jfieldID include_groups_fieldID;
  bool include_channels_;
  static jfieldID include_channels_fieldID;

  chatFolder();

  chatFolder(object_ptr<chatFolderName> &&name_, object_ptr<chatFolderIcon> &&icon_, int32 color_id_, bool is_shareable_, array<int53> &&pinned_chat_ids_, array<int53> &&included_chat_ids_, array<int53> &&excluded_chat_ids_, bool exclude_muted_, bool exclude_read_, bool exclude_archived_, bool include_contacts_, bool include_non_contacts_, bool include_bots_, bool include_groups_, bool include_channels_);

  static const std::int32_t ID = 1596164696;

  static object_ptr<chatFolder> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderIcon final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  chatFolderIcon();

  explicit chatFolderIcon(string const &name_);

  static const std::int32_t ID = -146104090;

  static object_ptr<chatFolderIcon> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderIcon;

class chatFolderName;

class chatFolderInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  object_ptr<chatFolderName> name_;
  static jfieldID name_fieldID;
  object_ptr<chatFolderIcon> icon_;
  static jfieldID icon_fieldID;
  int32 color_id_;
  static jfieldID color_id_fieldID;
  bool is_shareable_;
  static jfieldID is_shareable_fieldID;
  bool has_my_invite_links_;
  static jfieldID has_my_invite_links_fieldID;

  chatFolderInfo();

  chatFolderInfo(int32 id_, object_ptr<chatFolderName> &&name_, object_ptr<chatFolderIcon> &&icon_, int32 color_id_, bool is_shareable_, bool has_my_invite_links_);

  static const std::int32_t ID = 815535117;

  static object_ptr<chatFolderInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string name_;
  static jfieldID name_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  chatFolderInviteLink();

  chatFolderInviteLink(string const &invite_link_, string const &name_, array<int53> &&chat_ids_);

  static const std::int32_t ID = 493969661;

  static object_ptr<chatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInfo;

class chatFolderInviteLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolderInfo> chat_folder_info_;
  static jfieldID chat_folder_info_fieldID;
  array<int53> missing_chat_ids_;
  static jfieldID missing_chat_ids_fieldID;
  array<int53> added_chat_ids_;
  static jfieldID added_chat_ids_fieldID;

  chatFolderInviteLinkInfo();

  chatFolderInviteLinkInfo(object_ptr<chatFolderInfo> &&chat_folder_info_, array<int53> &&missing_chat_ids_, array<int53> &&added_chat_ids_);

  static const std::int32_t ID = 1119450395;

  static object_ptr<chatFolderInviteLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLink;

class chatFolderInviteLinks final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatFolderInviteLink>> invite_links_;
  static jfieldID invite_links_fieldID;

  chatFolderInviteLinks();

  explicit chatFolderInviteLinks(array<object_ptr<chatFolderInviteLink>> &&invite_links_);

  static const std::int32_t ID = 1853351525;

  static object_ptr<chatFolderInviteLinks> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class chatFolderName final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool animate_custom_emoji_;
  static jfieldID animate_custom_emoji_fieldID;

  chatFolderName();

  chatFolderName(object_ptr<formattedText> &&text_, bool animate_custom_emoji_);

  static const std::int32_t ID = -330482274;

  static object_ptr<chatFolderName> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptionPricing;

class chatInviteLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string name_;
  static jfieldID name_fieldID;
  int53 creator_user_id_;
  static jfieldID creator_user_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  int32 edit_date_;
  static jfieldID edit_date_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;
  object_ptr<starSubscriptionPricing> subscription_pricing_;
  static jfieldID subscription_pricing_fieldID;
  int32 member_limit_;
  static jfieldID member_limit_fieldID;
  int32 member_count_;
  static jfieldID member_count_fieldID;
  int32 expired_member_count_;
  static jfieldID expired_member_count_fieldID;
  int32 pending_join_request_count_;
  static jfieldID pending_join_request_count_fieldID;
  bool creates_join_request_;
  static jfieldID creates_join_request_fieldID;
  bool is_primary_;
  static jfieldID is_primary_fieldID;
  bool is_revoked_;
  static jfieldID is_revoked_fieldID;

  chatInviteLink();

  chatInviteLink(string const &invite_link_, string const &name_, int53 creator_user_id_, int32 date_, int32 edit_date_, int32 expiration_date_, object_ptr<starSubscriptionPricing> &&subscription_pricing_, int32 member_limit_, int32 member_count_, int32 expired_member_count_, int32 pending_join_request_count_, bool creates_join_request_, bool is_primary_, bool is_revoked_);

  static const std::int32_t ID = -957651664;

  static object_ptr<chatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkCount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 invite_link_count_;
  static jfieldID invite_link_count_fieldID;
  int32 revoked_invite_link_count_;
  static jfieldID revoked_invite_link_count_fieldID;

  chatInviteLinkCount();

  chatInviteLinkCount(int53 user_id_, int32 invite_link_count_, int32 revoked_invite_link_count_);

  static const std::int32_t ID = -1021999210;

  static object_ptr<chatInviteLinkCount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkCount;

class chatInviteLinkCounts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatInviteLinkCount>> invite_link_counts_;
  static jfieldID invite_link_counts_fieldID;

  chatInviteLinkCounts();

  explicit chatInviteLinkCounts(array<object_ptr<chatInviteLinkCount>> &&invite_link_counts_);

  static const std::int32_t ID = 920326637;

  static object_ptr<chatInviteLinkCounts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InviteLinkChatType;

class chatInviteLinkSubscriptionInfo;

class chatPhotoInfo;

class verificationStatus;

class chatInviteLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 accessible_for_;
  static jfieldID accessible_for_fieldID;
  object_ptr<InviteLinkChatType> type_;
  static jfieldID type_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<chatPhotoInfo> photo_;
  static jfieldID photo_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  string description_;
  static jfieldID description_fieldID;
  int32 member_count_;
  static jfieldID member_count_fieldID;
  array<int53> member_user_ids_;
  static jfieldID member_user_ids_fieldID;
  object_ptr<chatInviteLinkSubscriptionInfo> subscription_info_;
  static jfieldID subscription_info_fieldID;
  bool creates_join_request_;
  static jfieldID creates_join_request_fieldID;
  bool is_public_;
  static jfieldID is_public_fieldID;
  object_ptr<verificationStatus> verification_status_;
  static jfieldID verification_status_fieldID;

  chatInviteLinkInfo();

  chatInviteLinkInfo(int53 chat_id_, int32 accessible_for_, object_ptr<InviteLinkChatType> &&type_, string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, string const &description_, int32 member_count_, array<int53> &&member_user_ids_, object_ptr<chatInviteLinkSubscriptionInfo> &&subscription_info_, bool creates_join_request_, bool is_public_, object_ptr<verificationStatus> &&verification_status_);

  static const std::int32_t ID = -1145310535;

  static object_ptr<chatInviteLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkMember final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 joined_chat_date_;
  static jfieldID joined_chat_date_fieldID;
  bool via_chat_folder_invite_link_;
  static jfieldID via_chat_folder_invite_link_fieldID;
  int53 approver_user_id_;
  static jfieldID approver_user_id_fieldID;

  chatInviteLinkMember();

  chatInviteLinkMember(int53 user_id_, int32 joined_chat_date_, bool via_chat_folder_invite_link_, int53 approver_user_id_);

  static const std::int32_t ID = 29156795;

  static object_ptr<chatInviteLinkMember> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkMember;

class chatInviteLinkMembers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatInviteLinkMember>> members_;
  static jfieldID members_fieldID;

  chatInviteLinkMembers();

  chatInviteLinkMembers(int32 total_count_, array<object_ptr<chatInviteLinkMember>> &&members_);

  static const std::int32_t ID = 315635051;

  static object_ptr<chatInviteLinkMembers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptionPricing;

class chatInviteLinkSubscriptionInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starSubscriptionPricing> pricing_;
  static jfieldID pricing_fieldID;
  bool can_reuse_;
  static jfieldID can_reuse_fieldID;
  int64 form_id_;
  static jfieldID form_id_fieldID;

  chatInviteLinkSubscriptionInfo();

  chatInviteLinkSubscriptionInfo(object_ptr<starSubscriptionPricing> &&pricing_, bool can_reuse_, int64 form_id_);

  static const std::int32_t ID = 953119592;

  static object_ptr<chatInviteLinkSubscriptionInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class chatInviteLinks final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatInviteLink>> invite_links_;
  static jfieldID invite_links_fieldID;

  chatInviteLinks();

  chatInviteLinks(int32 total_count_, array<object_ptr<chatInviteLink>> &&invite_links_);

  static const std::int32_t ID = 112891427;

  static object_ptr<chatInviteLinks> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatJoinRequest final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  string bio_;
  static jfieldID bio_fieldID;

  chatJoinRequest();

  chatJoinRequest(int53 user_id_, int32 date_, string const &bio_);

  static const std::int32_t ID = 59341416;

  static object_ptr<chatJoinRequest> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatJoinRequest;

class chatJoinRequests final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatJoinRequest>> requests_;
  static jfieldID requests_fieldID;

  chatJoinRequests();

  chatJoinRequests(int32 total_count_, array<object_ptr<chatJoinRequest>> &&requests_);

  static const std::int32_t ID = 1291680519;

  static object_ptr<chatJoinRequests> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatJoinRequestsInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  chatJoinRequestsInfo();

  chatJoinRequestsInfo(int32 total_count_, array<int53> &&user_ids_);

  static const std::int32_t ID = 888534463;

  static object_ptr<chatJoinRequestsInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatList: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatList> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatListMain final : public ChatList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatListMain();

  static const std::int32_t ID = -400991316;

  static object_ptr<ChatList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatListArchive final : public ChatList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatListArchive();

  static const std::int32_t ID = 362770115;

  static object_ptr<ChatList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatListFolder final : public ChatList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  chatListFolder();

  explicit chatListFolder(int32 chat_folder_id_);

  static const std::int32_t ID = 385760856;

  static object_ptr<ChatList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class chatLists final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<ChatList>> chat_lists_;
  static jfieldID chat_lists_fieldID;

  chatLists();

  explicit chatLists(array<object_ptr<ChatList>> &&chat_lists_);

  static const std::int32_t ID = -258292771;

  static object_ptr<chatLists> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class location;

class chatLocation final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  string address_;
  static jfieldID address_fieldID;

  chatLocation();

  chatLocation(object_ptr<location> &&location_, string const &address_);

  static const std::int32_t ID = -1566863583;

  static object_ptr<chatLocation> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatMemberStatus;

class MessageSender;

class chatMember final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;
  int53 inviter_user_id_;
  static jfieldID inviter_user_id_fieldID;
  int32 joined_chat_date_;
  static jfieldID joined_chat_date_fieldID;
  object_ptr<ChatMemberStatus> status_;
  static jfieldID status_fieldID;

  chatMember();

  chatMember(object_ptr<MessageSender> &&member_id_, int53 inviter_user_id_, int32 joined_chat_date_, object_ptr<ChatMemberStatus> &&status_);

  static const std::int32_t ID = 1829953909;

  static object_ptr<chatMember> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministratorRights;

class chatPermissions;

class ChatMemberStatus: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusCreator final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string custom_title_;
  static jfieldID custom_title_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;
  bool is_member_;
  static jfieldID is_member_fieldID;

  chatMemberStatusCreator();

  chatMemberStatusCreator(string const &custom_title_, bool is_anonymous_, bool is_member_);

  static const std::int32_t ID = -160019714;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusAdministrator final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string custom_title_;
  static jfieldID custom_title_fieldID;
  bool can_be_edited_;
  static jfieldID can_be_edited_fieldID;
  object_ptr<chatAdministratorRights> rights_;
  static jfieldID rights_fieldID;

  chatMemberStatusAdministrator();

  chatMemberStatusAdministrator(string const &custom_title_, bool can_be_edited_, object_ptr<chatAdministratorRights> &&rights_);

  static const std::int32_t ID = -70024163;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusMember final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 member_until_date_;
  static jfieldID member_until_date_fieldID;

  chatMemberStatusMember();

  explicit chatMemberStatusMember(int32 member_until_date_);

  static const std::int32_t ID = -32707562;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusRestricted final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_member_;
  static jfieldID is_member_fieldID;
  int32 restricted_until_date_;
  static jfieldID restricted_until_date_fieldID;
  object_ptr<chatPermissions> permissions_;
  static jfieldID permissions_fieldID;

  chatMemberStatusRestricted();

  chatMemberStatusRestricted(bool is_member_, int32 restricted_until_date_, object_ptr<chatPermissions> &&permissions_);

  static const std::int32_t ID = 1661432998;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusLeft final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMemberStatusLeft();

  static const std::int32_t ID = -5815259;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMemberStatusBanned final : public ChatMemberStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 banned_until_date_;
  static jfieldID banned_until_date_fieldID;

  chatMemberStatusBanned();

  explicit chatMemberStatusBanned(int32 banned_until_date_);

  static const std::int32_t ID = -1653518666;

  static object_ptr<ChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMember;

class chatMembers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatMember>> members_;
  static jfieldID members_fieldID;

  chatMembers();

  chatMembers(int32 total_count_, array<object_ptr<chatMember>> &&members_);

  static const std::int32_t ID = -497558622;

  static object_ptr<chatMembers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatMembersFilter: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterContacts final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterContacts();

  static const std::int32_t ID = 1774485671;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterAdministrators final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterAdministrators();

  static const std::int32_t ID = -1266893796;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterMembers final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterMembers();

  static const std::int32_t ID = 670504342;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterMention final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  chatMembersFilterMention();

  explicit chatMembersFilterMention(int53 message_thread_id_);

  static const std::int32_t ID = 856419831;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterRestricted final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterRestricted();

  static const std::int32_t ID = 1256282813;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterBanned final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterBanned();

  static const std::int32_t ID = -1863102648;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMembersFilterBots final : public ChatMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatMembersFilterBots();

  static const std::int32_t ID = -1422567288;

  static object_ptr<ChatMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class chatMessageSender final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> sender_;
  static jfieldID sender_fieldID;
  bool needs_premium_;
  static jfieldID needs_premium_fieldID;

  chatMessageSender();

  chatMessageSender(object_ptr<MessageSender> &&sender_, bool needs_premium_);

  static const std::int32_t ID = 760590010;

  static object_ptr<chatMessageSender> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatMessageSender;

class chatMessageSenders final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatMessageSender>> senders_;
  static jfieldID senders_fieldID;

  chatMessageSenders();

  explicit chatMessageSenders(array<object_ptr<chatMessageSender>> &&senders_);

  static const std::int32_t ID = -1866230970;

  static object_ptr<chatMessageSenders> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatNotificationSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool use_default_mute_for_;
  static jfieldID use_default_mute_for_fieldID;
  int32 mute_for_;
  static jfieldID mute_for_fieldID;
  bool use_default_sound_;
  static jfieldID use_default_sound_fieldID;
  int64 sound_id_;
  static jfieldID sound_id_fieldID;
  bool use_default_show_preview_;
  static jfieldID use_default_show_preview_fieldID;
  bool show_preview_;
  static jfieldID show_preview_fieldID;
  bool use_default_mute_stories_;
  static jfieldID use_default_mute_stories_fieldID;
  bool mute_stories_;
  static jfieldID mute_stories_fieldID;
  bool use_default_story_sound_;
  static jfieldID use_default_story_sound_fieldID;
  int64 story_sound_id_;
  static jfieldID story_sound_id_fieldID;
  bool use_default_show_story_poster_;
  static jfieldID use_default_show_story_poster_fieldID;
  bool show_story_poster_;
  static jfieldID show_story_poster_fieldID;
  bool use_default_disable_pinned_message_notifications_;
  static jfieldID use_default_disable_pinned_message_notifications_fieldID;
  bool disable_pinned_message_notifications_;
  static jfieldID disable_pinned_message_notifications_fieldID;
  bool use_default_disable_mention_notifications_;
  static jfieldID use_default_disable_mention_notifications_fieldID;
  bool disable_mention_notifications_;
  static jfieldID disable_mention_notifications_fieldID;

  chatNotificationSettings();

  chatNotificationSettings(bool use_default_mute_for_, int32 mute_for_, bool use_default_sound_, int64 sound_id_, bool use_default_show_preview_, bool show_preview_, bool use_default_mute_stories_, bool mute_stories_, bool use_default_story_sound_, int64 story_sound_id_, bool use_default_show_story_poster_, bool show_story_poster_, bool use_default_disable_pinned_message_notifications_, bool disable_pinned_message_notifications_, bool use_default_disable_mention_notifications_, bool disable_mention_notifications_);

  static const std::int32_t ID = 1459533846;

  static object_ptr<chatNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatPermissions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_send_basic_messages_;
  static jfieldID can_send_basic_messages_fieldID;
  bool can_send_audios_;
  static jfieldID can_send_audios_fieldID;
  bool can_send_documents_;
  static jfieldID can_send_documents_fieldID;
  bool can_send_photos_;
  static jfieldID can_send_photos_fieldID;
  bool can_send_videos_;
  static jfieldID can_send_videos_fieldID;
  bool can_send_video_notes_;
  static jfieldID can_send_video_notes_fieldID;
  bool can_send_voice_notes_;
  static jfieldID can_send_voice_notes_fieldID;
  bool can_send_polls_;
  static jfieldID can_send_polls_fieldID;
  bool can_send_other_messages_;
  static jfieldID can_send_other_messages_fieldID;
  bool can_add_link_previews_;
  static jfieldID can_add_link_previews_fieldID;
  bool can_change_info_;
  static jfieldID can_change_info_fieldID;
  bool can_invite_users_;
  static jfieldID can_invite_users_fieldID;
  bool can_pin_messages_;
  static jfieldID can_pin_messages_fieldID;
  bool can_create_topics_;
  static jfieldID can_create_topics_fieldID;

  chatPermissions();

  chatPermissions(bool can_send_basic_messages_, bool can_send_audios_, bool can_send_documents_, bool can_send_photos_, bool can_send_videos_, bool can_send_video_notes_, bool can_send_voice_notes_, bool can_send_polls_, bool can_send_other_messages_, bool can_add_link_previews_, bool can_change_info_, bool can_invite_users_, bool can_pin_messages_, bool can_create_topics_);

  static const std::int32_t ID = -118334855;

  static object_ptr<chatPermissions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class animatedChatPhoto;

class chatPhotoSticker;

class minithumbnail;

class photoSize;

class chatPhoto final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 added_date_;
  static jfieldID added_date_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  array<object_ptr<photoSize>> sizes_;
  static jfieldID sizes_fieldID;
  object_ptr<animatedChatPhoto> animation_;
  static jfieldID animation_fieldID;
  object_ptr<animatedChatPhoto> small_animation_;
  static jfieldID small_animation_fieldID;
  object_ptr<chatPhotoSticker> sticker_;
  static jfieldID sticker_fieldID;

  chatPhoto();

  chatPhoto(int64 id_, int32 added_date_, object_ptr<minithumbnail> &&minithumbnail_, array<object_ptr<photoSize>> &&sizes_, object_ptr<animatedChatPhoto> &&animation_, object_ptr<animatedChatPhoto> &&small_animation_, object_ptr<chatPhotoSticker> &&sticker_);

  static const std::int32_t ID = -1430870201;

  static object_ptr<chatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class chatPhotoInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<file> small_;
  static jfieldID small_fieldID;
  object_ptr<file> big_;
  static jfieldID big_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  bool has_animation_;
  static jfieldID has_animation_fieldID;
  bool is_personal_;
  static jfieldID is_personal_fieldID;

  chatPhotoInfo();

  chatPhotoInfo(object_ptr<file> &&small_, object_ptr<file> &&big_, object_ptr<minithumbnail> &&minithumbnail_, bool has_animation_, bool is_personal_);

  static const std::int32_t ID = 281195686;

  static object_ptr<chatPhotoInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundFill;

class ChatPhotoStickerType;

class chatPhotoSticker final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatPhotoStickerType> type_;
  static jfieldID type_fieldID;
  object_ptr<BackgroundFill> background_fill_;
  static jfieldID background_fill_fieldID;

  chatPhotoSticker();

  chatPhotoSticker(object_ptr<ChatPhotoStickerType> &&type_, object_ptr<BackgroundFill> &&background_fill_);

  static const std::int32_t ID = -1459387485;

  static object_ptr<chatPhotoSticker> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatPhotoStickerType: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatPhotoStickerType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatPhotoStickerTypeRegularOrMask final : public ChatPhotoStickerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 sticker_set_id_;
  static jfieldID sticker_set_id_fieldID;
  int64 sticker_id_;
  static jfieldID sticker_id_fieldID;

  chatPhotoStickerTypeRegularOrMask();

  chatPhotoStickerTypeRegularOrMask(int64 sticker_set_id_, int64 sticker_id_);

  static const std::int32_t ID = -415147620;

  static object_ptr<ChatPhotoStickerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatPhotoStickerTypeCustomEmoji final : public ChatPhotoStickerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  chatPhotoStickerTypeCustomEmoji();

  explicit chatPhotoStickerTypeCustomEmoji(int64 custom_emoji_id_);

  static const std::int32_t ID = -266224943;

  static object_ptr<ChatPhotoStickerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatPhoto;

class chatPhotos final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatPhoto>> photos_;
  static jfieldID photos_fieldID;

  chatPhotos();

  chatPhotos(int32 total_count_, array<object_ptr<chatPhoto>> &&photos_);

  static const std::int32_t ID = -1510699180;

  static object_ptr<chatPhotos> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ChatSource;

class chatPosition final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> list_;
  static jfieldID list_fieldID;
  int64 order_;
  static jfieldID order_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  object_ptr<ChatSource> source_;
  static jfieldID source_fieldID;

  chatPosition();

  chatPosition(object_ptr<ChatList> &&list_, int64 order_, bool is_pinned_, object_ptr<ChatSource> &&source_);

  static const std::int32_t ID = -622557355;

  static object_ptr<chatPosition> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueAmount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 total_amount_;
  static jfieldID total_amount_fieldID;
  int64 balance_amount_;
  static jfieldID balance_amount_fieldID;
  int64 available_amount_;
  static jfieldID available_amount_fieldID;
  bool withdrawal_enabled_;
  static jfieldID withdrawal_enabled_fieldID;

  chatRevenueAmount();

  chatRevenueAmount(string const &cryptocurrency_, int64 total_amount_, int64 balance_amount_, int64 available_amount_, bool withdrawal_enabled_);

  static const std::int32_t ID = -1505178024;

  static object_ptr<chatRevenueAmount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class chatRevenueAmount;

class chatRevenueStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StatisticalGraph> revenue_by_hour_graph_;
  static jfieldID revenue_by_hour_graph_fieldID;
  object_ptr<StatisticalGraph> revenue_graph_;
  static jfieldID revenue_graph_fieldID;
  object_ptr<chatRevenueAmount> revenue_amount_;
  static jfieldID revenue_amount_fieldID;
  double usd_rate_;
  static jfieldID usd_rate_fieldID;

  chatRevenueStatistics();

  chatRevenueStatistics(object_ptr<StatisticalGraph> &&revenue_by_hour_graph_, object_ptr<StatisticalGraph> &&revenue_graph_, object_ptr<chatRevenueAmount> &&revenue_amount_, double usd_rate_);

  static const std::int32_t ID = 1667438779;

  static object_ptr<chatRevenueStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatRevenueTransactionType;

class chatRevenueTransaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 cryptocurrency_amount_;
  static jfieldID cryptocurrency_amount_fieldID;
  object_ptr<ChatRevenueTransactionType> type_;
  static jfieldID type_fieldID;

  chatRevenueTransaction();

  chatRevenueTransaction(string const &cryptocurrency_, int64 cryptocurrency_amount_, object_ptr<ChatRevenueTransactionType> &&type_);

  static const std::int32_t ID = 80192767;

  static object_ptr<chatRevenueTransaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class RevenueWithdrawalState;

class ChatRevenueTransactionType: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatRevenueTransactionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueTransactionTypeEarnings final : public ChatRevenueTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  int32 end_date_;
  static jfieldID end_date_fieldID;

  chatRevenueTransactionTypeEarnings();

  chatRevenueTransactionTypeEarnings(int32 start_date_, int32 end_date_);

  static const std::int32_t ID = -400776056;

  static object_ptr<ChatRevenueTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueTransactionTypeWithdrawal final : public ChatRevenueTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 withdrawal_date_;
  static jfieldID withdrawal_date_fieldID;
  string provider_;
  static jfieldID provider_fieldID;
  object_ptr<RevenueWithdrawalState> state_;
  static jfieldID state_fieldID;

  chatRevenueTransactionTypeWithdrawal();

  chatRevenueTransactionTypeWithdrawal(int32 withdrawal_date_, string const &provider_, object_ptr<RevenueWithdrawalState> &&state_);

  static const std::int32_t ID = 252939755;

  static object_ptr<ChatRevenueTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueTransactionTypeRefund final : public ChatRevenueTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 refund_date_;
  static jfieldID refund_date_fieldID;
  string provider_;
  static jfieldID provider_fieldID;

  chatRevenueTransactionTypeRefund();

  chatRevenueTransactionTypeRefund(int32 refund_date_, string const &provider_);

  static const std::int32_t ID = 302430279;

  static object_ptr<ChatRevenueTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueTransaction;

class chatRevenueTransactions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatRevenueTransaction>> transactions_;
  static jfieldID transactions_fieldID;

  chatRevenueTransactions();

  chatRevenueTransactions(int32 total_count_, array<object_ptr<chatRevenueTransaction>> &&transactions_);

  static const std::int32_t ID = -553258171;

  static object_ptr<chatRevenueTransactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatSource: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatSourceMtprotoProxy final : public ChatSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  chatSourceMtprotoProxy();

  static const std::int32_t ID = 394074115;

  static object_ptr<ChatSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatSourcePublicServiceAnnouncement final : public ChatSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string type_;
  static jfieldID type_fieldID;
  string text_;
  static jfieldID text_fieldID;

  chatSourcePublicServiceAnnouncement();

  chatSourcePublicServiceAnnouncement(string const &type_, string const &text_);

  static const std::int32_t ID = -328571244;

  static object_ptr<ChatSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class chatStatisticsAdministratorActionsInfo;

class chatStatisticsInteractionInfo;

class chatStatisticsInviterInfo;

class chatStatisticsMessageSenderInfo;

class dateRange;

class statisticalValue;

class ChatStatistics: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatStatistics> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsSupergroup final : public ChatStatistics {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<dateRange> period_;
  static jfieldID period_fieldID;
  object_ptr<statisticalValue> member_count_;
  static jfieldID member_count_fieldID;
  object_ptr<statisticalValue> message_count_;
  static jfieldID message_count_fieldID;
  object_ptr<statisticalValue> viewer_count_;
  static jfieldID viewer_count_fieldID;
  object_ptr<statisticalValue> sender_count_;
  static jfieldID sender_count_fieldID;
  object_ptr<StatisticalGraph> member_count_graph_;
  static jfieldID member_count_graph_fieldID;
  object_ptr<StatisticalGraph> join_graph_;
  static jfieldID join_graph_fieldID;
  object_ptr<StatisticalGraph> join_by_source_graph_;
  static jfieldID join_by_source_graph_fieldID;
  object_ptr<StatisticalGraph> language_graph_;
  static jfieldID language_graph_fieldID;
  object_ptr<StatisticalGraph> message_content_graph_;
  static jfieldID message_content_graph_fieldID;
  object_ptr<StatisticalGraph> action_graph_;
  static jfieldID action_graph_fieldID;
  object_ptr<StatisticalGraph> day_graph_;
  static jfieldID day_graph_fieldID;
  object_ptr<StatisticalGraph> week_graph_;
  static jfieldID week_graph_fieldID;
  array<object_ptr<chatStatisticsMessageSenderInfo>> top_senders_;
  static jfieldID top_senders_fieldID;
  array<object_ptr<chatStatisticsAdministratorActionsInfo>> top_administrators_;
  static jfieldID top_administrators_fieldID;
  array<object_ptr<chatStatisticsInviterInfo>> top_inviters_;
  static jfieldID top_inviters_fieldID;

  chatStatisticsSupergroup();

  chatStatisticsSupergroup(object_ptr<dateRange> &&period_, object_ptr<statisticalValue> &&member_count_, object_ptr<statisticalValue> &&message_count_, object_ptr<statisticalValue> &&viewer_count_, object_ptr<statisticalValue> &&sender_count_, object_ptr<StatisticalGraph> &&member_count_graph_, object_ptr<StatisticalGraph> &&join_graph_, object_ptr<StatisticalGraph> &&join_by_source_graph_, object_ptr<StatisticalGraph> &&language_graph_, object_ptr<StatisticalGraph> &&message_content_graph_, object_ptr<StatisticalGraph> &&action_graph_, object_ptr<StatisticalGraph> &&day_graph_, object_ptr<StatisticalGraph> &&week_graph_, array<object_ptr<chatStatisticsMessageSenderInfo>> &&top_senders_, array<object_ptr<chatStatisticsAdministratorActionsInfo>> &&top_administrators_, array<object_ptr<chatStatisticsInviterInfo>> &&top_inviters_);

  static const std::int32_t ID = -17244633;

  static object_ptr<ChatStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsChannel final : public ChatStatistics {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<dateRange> period_;
  static jfieldID period_fieldID;
  object_ptr<statisticalValue> member_count_;
  static jfieldID member_count_fieldID;
  object_ptr<statisticalValue> mean_message_view_count_;
  static jfieldID mean_message_view_count_fieldID;
  object_ptr<statisticalValue> mean_message_share_count_;
  static jfieldID mean_message_share_count_fieldID;
  object_ptr<statisticalValue> mean_message_reaction_count_;
  static jfieldID mean_message_reaction_count_fieldID;
  object_ptr<statisticalValue> mean_story_view_count_;
  static jfieldID mean_story_view_count_fieldID;
  object_ptr<statisticalValue> mean_story_share_count_;
  static jfieldID mean_story_share_count_fieldID;
  object_ptr<statisticalValue> mean_story_reaction_count_;
  static jfieldID mean_story_reaction_count_fieldID;
  double enabled_notifications_percentage_;
  static jfieldID enabled_notifications_percentage_fieldID;
  object_ptr<StatisticalGraph> member_count_graph_;
  static jfieldID member_count_graph_fieldID;
  object_ptr<StatisticalGraph> join_graph_;
  static jfieldID join_graph_fieldID;
  object_ptr<StatisticalGraph> mute_graph_;
  static jfieldID mute_graph_fieldID;
  object_ptr<StatisticalGraph> view_count_by_hour_graph_;
  static jfieldID view_count_by_hour_graph_fieldID;
  object_ptr<StatisticalGraph> view_count_by_source_graph_;
  static jfieldID view_count_by_source_graph_fieldID;
  object_ptr<StatisticalGraph> join_by_source_graph_;
  static jfieldID join_by_source_graph_fieldID;
  object_ptr<StatisticalGraph> language_graph_;
  static jfieldID language_graph_fieldID;
  object_ptr<StatisticalGraph> message_interaction_graph_;
  static jfieldID message_interaction_graph_fieldID;
  object_ptr<StatisticalGraph> message_reaction_graph_;
  static jfieldID message_reaction_graph_fieldID;
  object_ptr<StatisticalGraph> story_interaction_graph_;
  static jfieldID story_interaction_graph_fieldID;
  object_ptr<StatisticalGraph> story_reaction_graph_;
  static jfieldID story_reaction_graph_fieldID;
  object_ptr<StatisticalGraph> instant_view_interaction_graph_;
  static jfieldID instant_view_interaction_graph_fieldID;
  array<object_ptr<chatStatisticsInteractionInfo>> recent_interactions_;
  static jfieldID recent_interactions_fieldID;

  chatStatisticsChannel();

  chatStatisticsChannel(object_ptr<dateRange> &&period_, object_ptr<statisticalValue> &&member_count_, object_ptr<statisticalValue> &&mean_message_view_count_, object_ptr<statisticalValue> &&mean_message_share_count_, object_ptr<statisticalValue> &&mean_message_reaction_count_, object_ptr<statisticalValue> &&mean_story_view_count_, object_ptr<statisticalValue> &&mean_story_share_count_, object_ptr<statisticalValue> &&mean_story_reaction_count_, double enabled_notifications_percentage_, object_ptr<StatisticalGraph> &&member_count_graph_, object_ptr<StatisticalGraph> &&join_graph_, object_ptr<StatisticalGraph> &&mute_graph_, object_ptr<StatisticalGraph> &&view_count_by_hour_graph_, object_ptr<StatisticalGraph> &&view_count_by_source_graph_, object_ptr<StatisticalGraph> &&join_by_source_graph_, object_ptr<StatisticalGraph> &&language_graph_, object_ptr<StatisticalGraph> &&message_interaction_graph_, object_ptr<StatisticalGraph> &&message_reaction_graph_, object_ptr<StatisticalGraph> &&story_interaction_graph_, object_ptr<StatisticalGraph> &&story_reaction_graph_, object_ptr<StatisticalGraph> &&instant_view_interaction_graph_, array<object_ptr<chatStatisticsInteractionInfo>> &&recent_interactions_);

  static const std::int32_t ID = -1375151660;

  static object_ptr<ChatStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsAdministratorActionsInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 deleted_message_count_;
  static jfieldID deleted_message_count_fieldID;
  int32 banned_user_count_;
  static jfieldID banned_user_count_fieldID;
  int32 restricted_user_count_;
  static jfieldID restricted_user_count_fieldID;

  chatStatisticsAdministratorActionsInfo();

  chatStatisticsAdministratorActionsInfo(int53 user_id_, int32 deleted_message_count_, int32 banned_user_count_, int32 restricted_user_count_);

  static const std::int32_t ID = -406467202;

  static object_ptr<chatStatisticsAdministratorActionsInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatStatisticsObjectType;

class chatStatisticsInteractionInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatStatisticsObjectType> object_type_;
  static jfieldID object_type_fieldID;
  int32 view_count_;
  static jfieldID view_count_fieldID;
  int32 forward_count_;
  static jfieldID forward_count_fieldID;
  int32 reaction_count_;
  static jfieldID reaction_count_fieldID;

  chatStatisticsInteractionInfo();

  chatStatisticsInteractionInfo(object_ptr<ChatStatisticsObjectType> &&object_type_, int32 view_count_, int32 forward_count_, int32 reaction_count_);

  static const std::int32_t ID = 1766496909;

  static object_ptr<chatStatisticsInteractionInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsInviterInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 added_member_count_;
  static jfieldID added_member_count_fieldID;

  chatStatisticsInviterInfo();

  chatStatisticsInviterInfo(int53 user_id_, int32 added_member_count_);

  static const std::int32_t ID = 629396619;

  static object_ptr<chatStatisticsInviterInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsMessageSenderInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 sent_message_count_;
  static jfieldID sent_message_count_fieldID;
  int32 average_character_count_;
  static jfieldID average_character_count_fieldID;

  chatStatisticsMessageSenderInfo();

  chatStatisticsMessageSenderInfo(int53 user_id_, int32 sent_message_count_, int32 average_character_count_);

  static const std::int32_t ID = 1762295371;

  static object_ptr<chatStatisticsMessageSenderInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatStatisticsObjectType: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatStatisticsObjectType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsObjectTypeMessage final : public ChatStatisticsObjectType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  chatStatisticsObjectTypeMessage();

  explicit chatStatisticsObjectTypeMessage(int53 message_id_);

  static const std::int32_t ID = 1872700662;

  static object_ptr<ChatStatisticsObjectType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatStatisticsObjectTypeStory final : public ChatStatisticsObjectType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  chatStatisticsObjectTypeStory();

  explicit chatStatisticsObjectTypeStory(int32 story_id_);

  static const std::int32_t ID = 364575152;

  static object_ptr<ChatStatisticsObjectType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class themeSettings;

class chatTheme final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<themeSettings> light_settings_;
  static jfieldID light_settings_fieldID;
  object_ptr<themeSettings> dark_settings_;
  static jfieldID dark_settings_fieldID;

  chatTheme();

  chatTheme(string const &name_, object_ptr<themeSettings> &&light_settings_, object_ptr<themeSettings> &&dark_settings_);

  static const std::int32_t ID = -113218503;

  static object_ptr<chatTheme> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatType: public Object {
 public:
  static jclass Class;

  static object_ptr<ChatType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatTypePrivate final : public ChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  chatTypePrivate();

  explicit chatTypePrivate(int53 user_id_);

  static const std::int32_t ID = 1579049844;

  static object_ptr<ChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatTypeBasicGroup final : public ChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;

  chatTypeBasicGroup();

  explicit chatTypeBasicGroup(int53 basic_group_id_);

  static const std::int32_t ID = 973884508;

  static object_ptr<ChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatTypeSupergroup final : public ChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool is_channel_;
  static jfieldID is_channel_fieldID;

  chatTypeSupergroup();

  chatTypeSupergroup(int53 supergroup_id_, bool is_channel_);

  static const std::int32_t ID = -1472570774;

  static object_ptr<ChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatTypeSecret final : public ChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 secret_chat_id_;
  static jfieldID secret_chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  chatTypeSecret();

  chatTypeSecret(int32 secret_chat_id_, int53 user_id_);

  static const std::int32_t ID = 862366513;

  static object_ptr<ChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chats final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  chats();

  chats(int32 total_count_, array<int53> &&chat_ids_);

  static const std::int32_t ID = 1809654812;

  static object_ptr<chats> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CheckChatUsernameResult: public Object {
 public:
  static jclass Class;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultOk final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultOk();

  static const std::int32_t ID = -1498956964;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultUsernameInvalid final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultUsernameInvalid();

  static const std::int32_t ID = -636979370;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultUsernameOccupied final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultUsernameOccupied();

  static const std::int32_t ID = 1320892201;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultUsernamePurchasable final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultUsernamePurchasable();

  static const std::int32_t ID = 5885529;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultPublicChatsTooMany final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultPublicChatsTooMany();

  static const std::int32_t ID = -659264388;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkChatUsernameResultPublicGroupsUnavailable final : public CheckChatUsernameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkChatUsernameResultPublicGroupsUnavailable();

  static const std::int32_t ID = -51833641;

  static object_ptr<CheckChatUsernameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CheckStickerSetNameResult: public Object {
 public:
  static jclass Class;

  static object_ptr<CheckStickerSetNameResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class checkStickerSetNameResultOk final : public CheckStickerSetNameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkStickerSetNameResultOk();

  static const std::int32_t ID = -1404308904;

  static object_ptr<CheckStickerSetNameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkStickerSetNameResultNameInvalid final : public CheckStickerSetNameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkStickerSetNameResultNameInvalid();

  static const std::int32_t ID = 177992244;

  static object_ptr<CheckStickerSetNameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class checkStickerSetNameResultNameOccupied final : public CheckStickerSetNameResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  checkStickerSetNameResultNameOccupied();

  static const std::int32_t ID = 1012980872;

  static object_ptr<CheckStickerSetNameResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class birthdate;

class closeBirthdayUser final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<birthdate> birthdate_;
  static jfieldID birthdate_fieldID;

  closeBirthdayUser();

  closeBirthdayUser(int53 user_id_, object_ptr<birthdate> &&birthdate_);

  static const std::int32_t ID = -2147067410;

  static object_ptr<closeBirthdayUser> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class VectorPathCommand;

class closedVectorPath final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<VectorPathCommand>> commands_;
  static jfieldID commands_fieldID;

  closedVectorPath();

  explicit closedVectorPath(array<object_ptr<VectorPathCommand>> &&commands_);

  static const std::int32_t ID = 589951657;

  static object_ptr<closedVectorPath> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class collectibleItemInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 purchase_date_;
  static jfieldID purchase_date_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 cryptocurrency_amount_;
  static jfieldID cryptocurrency_amount_fieldID;
  string url_;
  static jfieldID url_fieldID;

  collectibleItemInfo();

  collectibleItemInfo(int32 purchase_date_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, string const &url_);

  static const std::int32_t ID = 1460640717;

  static object_ptr<collectibleItemInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class CollectibleItemType: public Object {
 public:
  static jclass Class;

  static object_ptr<CollectibleItemType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class collectibleItemTypeUsername final : public CollectibleItemType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;

  collectibleItemTypeUsername();

  explicit collectibleItemTypeUsername(string const &username_);

  static const std::int32_t ID = 458680273;

  static object_ptr<CollectibleItemType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class collectibleItemTypePhoneNumber final : public CollectibleItemType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;

  collectibleItemTypePhoneNumber();

  explicit collectibleItemTypePhoneNumber(string const &phone_number_);

  static const std::int32_t ID = 1256251714;

  static object_ptr<CollectibleItemType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramParameters;

class connectedAffiliateProgram final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<affiliateProgramParameters> parameters_;
  static jfieldID parameters_fieldID;
  int32 connection_date_;
  static jfieldID connection_date_fieldID;
  bool is_disconnected_;
  static jfieldID is_disconnected_fieldID;
  int64 user_count_;
  static jfieldID user_count_fieldID;
  int64 revenue_star_count_;
  static jfieldID revenue_star_count_fieldID;

  connectedAffiliateProgram();

  connectedAffiliateProgram(string const &url_, int53 bot_user_id_, object_ptr<affiliateProgramParameters> &&parameters_, int32 connection_date_, bool is_disconnected_, int64 user_count_, int64 revenue_star_count_);

  static const std::int32_t ID = 1488942101;

  static object_ptr<connectedAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectedAffiliateProgram;

class connectedAffiliatePrograms final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<connectedAffiliateProgram>> programs_;
  static jfieldID programs_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  connectedAffiliatePrograms();

  connectedAffiliatePrograms(int32 total_count_, array<object_ptr<connectedAffiliateProgram>> &&programs_, string const &next_offset_);

  static const std::int32_t ID = 1505880847;

  static object_ptr<connectedAffiliatePrograms> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectedWebsite final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string domain_name_;
  static jfieldID domain_name_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string browser_;
  static jfieldID browser_fieldID;
  string platform_;
  static jfieldID platform_fieldID;
  int32 log_in_date_;
  static jfieldID log_in_date_fieldID;
  int32 last_active_date_;
  static jfieldID last_active_date_fieldID;
  string ip_address_;
  static jfieldID ip_address_fieldID;
  string location_;
  static jfieldID location_fieldID;

  connectedWebsite();

  connectedWebsite(int64 id_, string const &domain_name_, int53 bot_user_id_, string const &browser_, string const &platform_, int32 log_in_date_, int32 last_active_date_, string const &ip_address_, string const &location_);

  static const std::int32_t ID = 1978115978;

  static object_ptr<connectedWebsite> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectedWebsite;

class connectedWebsites final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<connectedWebsite>> websites_;
  static jfieldID websites_fieldID;

  connectedWebsites();

  explicit connectedWebsites(array<object_ptr<connectedWebsite>> &&websites_);

  static const std::int32_t ID = -1727949694;

  static object_ptr<connectedWebsites> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ConnectionState: public Object {
 public:
  static jclass Class;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class connectionStateWaitingForNetwork final : public ConnectionState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  connectionStateWaitingForNetwork();

  static const std::int32_t ID = 1695405912;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectionStateConnectingToProxy final : public ConnectionState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  connectionStateConnectingToProxy();

  static const std::int32_t ID = -93187239;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectionStateConnecting final : public ConnectionState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  connectionStateConnecting();

  static const std::int32_t ID = -1298400670;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectionStateUpdating final : public ConnectionState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  connectionStateUpdating();

  static const std::int32_t ID = -188104009;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class connectionStateReady final : public ConnectionState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  connectionStateReady();

  static const std::int32_t ID = 48608492;

  static object_ptr<ConnectionState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class contact final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;
  string vcard_;
  static jfieldID vcard_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  contact();

  contact(string const &phone_number_, string const &first_name_, string const &last_name_, string const &vcard_, int53 user_id_);

  static const std::int32_t ID = -1993844876;

  static object_ptr<contact> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class count final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 count_;
  static jfieldID count_fieldID;

  count();

  explicit count(int32 count_);

  static const std::int32_t ID = 1295577348;

  static object_ptr<count> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class countryInfo;

class countries final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<countryInfo>> countries_;
  static jfieldID countries_fieldID;

  countries();

  explicit countries(array<object_ptr<countryInfo>> &&countries_);

  static const std::int32_t ID = 1854211813;

  static object_ptr<countries> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class countryInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string country_code_;
  static jfieldID country_code_fieldID;
  string name_;
  static jfieldID name_fieldID;
  string english_name_;
  static jfieldID english_name_fieldID;
  bool is_hidden_;
  static jfieldID is_hidden_fieldID;
  array<string> calling_codes_;
  static jfieldID calling_codes_fieldID;

  countryInfo();

  countryInfo(string const &country_code_, string const &name_, string const &english_name_, bool is_hidden_, array<string> &&calling_codes_);

  static const std::int32_t ID = 1617195722;

  static object_ptr<countryInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class failedToAddMembers;

class createdBasicGroupChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<failedToAddMembers> failed_to_add_members_;
  static jfieldID failed_to_add_members_fieldID;

  createdBasicGroupChat();

  createdBasicGroupChat(int53 chat_id_, object_ptr<failedToAddMembers> &&failed_to_add_members_);

  static const std::int32_t ID = -20417068;

  static object_ptr<createdBasicGroupChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class currentWeather final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double temperature_;
  static jfieldID temperature_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;

  currentWeather();

  currentWeather(double temperature_, string const &emoji_);

  static const std::int32_t ID = -355555136;

  static object_ptr<currentWeather> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class customRequestResult final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string result_;
  static jfieldID result_fieldID;

  customRequestResult();

  explicit customRequestResult(string const &result_);

  static const std::int32_t ID = -2009960452;

  static object_ptr<customRequestResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class data final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes data_;
  static jfieldID data_fieldID;

  data();

  explicit data(bytes const &data_);

  static const std::int32_t ID = 221197337;

  static object_ptr<data> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class databaseStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string statistics_;
  static jfieldID statistics_fieldID;

  databaseStatistics();

  explicit databaseStatistics(string const &statistics_);

  static const std::int32_t ID = -1123912880;

  static object_ptr<databaseStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class date final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 day_;
  static jfieldID day_fieldID;
  int32 month_;
  static jfieldID month_fieldID;
  int32 year_;
  static jfieldID year_fieldID;

  date();

  date(int32 day_, int32 month_, int32 year_);

  static const std::int32_t ID = -277956960;

  static object_ptr<date> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class dateRange final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  int32 end_date_;
  static jfieldID end_date_fieldID;

  dateRange();

  dateRange(int32 start_date_, int32 end_date_);

  static const std::int32_t ID = 1360333926;

  static object_ptr<dateRange> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class datedFile final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<file> file_;
  static jfieldID file_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  datedFile();

  datedFile(object_ptr<file> &&file_, int32 date_);

  static const std::int32_t ID = -1840795491;

  static object_ptr<datedFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class deepLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool need_update_application_;
  static jfieldID need_update_application_fieldID;

  deepLinkInfo();

  deepLinkInfo(object_ptr<formattedText> &&text_, bool need_update_application_);

  static const std::int32_t ID = 1864081662;

  static object_ptr<deepLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class DeviceToken: public Object {
 public:
  static jclass Class;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenFirebaseCloudMessaging final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;
  bool encrypt_;
  static jfieldID encrypt_fieldID;

  deviceTokenFirebaseCloudMessaging();

  deviceTokenFirebaseCloudMessaging(string const &token_, bool encrypt_);

  static const std::int32_t ID = -797881849;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenApplePush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string device_token_;
  static jfieldID device_token_fieldID;
  bool is_app_sandbox_;
  static jfieldID is_app_sandbox_fieldID;

  deviceTokenApplePush();

  deviceTokenApplePush(string const &device_token_, bool is_app_sandbox_);

  static const std::int32_t ID = 387541955;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenApplePushVoIP final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string device_token_;
  static jfieldID device_token_fieldID;
  bool is_app_sandbox_;
  static jfieldID is_app_sandbox_fieldID;
  bool encrypt_;
  static jfieldID encrypt_fieldID;

  deviceTokenApplePushVoIP();

  deviceTokenApplePushVoIP(string const &device_token_, bool is_app_sandbox_, bool encrypt_);

  static const std::int32_t ID = 804275689;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenWindowsPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string access_token_;
  static jfieldID access_token_fieldID;

  deviceTokenWindowsPush();

  explicit deviceTokenWindowsPush(string const &access_token_);

  static const std::int32_t ID = -1410514289;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenMicrosoftPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string channel_uri_;
  static jfieldID channel_uri_fieldID;

  deviceTokenMicrosoftPush();

  explicit deviceTokenMicrosoftPush(string const &channel_uri_);

  static const std::int32_t ID = 1224269900;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenMicrosoftPushVoIP final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string channel_uri_;
  static jfieldID channel_uri_fieldID;

  deviceTokenMicrosoftPushVoIP();

  explicit deviceTokenMicrosoftPushVoIP(string const &channel_uri_);

  static const std::int32_t ID = -785603759;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenWebPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string endpoint_;
  static jfieldID endpoint_fieldID;
  string p256dh_base64url_;
  static jfieldID p256dh_base64url_fieldID;
  string auth_base64url_;
  static jfieldID auth_base64url_fieldID;

  deviceTokenWebPush();

  deviceTokenWebPush(string const &endpoint_, string const &p256dh_base64url_, string const &auth_base64url_);

  static const std::int32_t ID = -1694507273;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenSimplePush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string endpoint_;
  static jfieldID endpoint_fieldID;

  deviceTokenSimplePush();

  explicit deviceTokenSimplePush(string const &endpoint_);

  static const std::int32_t ID = 49584736;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenUbuntuPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  deviceTokenUbuntuPush();

  explicit deviceTokenUbuntuPush(string const &token_);

  static const std::int32_t ID = 1782320422;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenBlackBerryPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  deviceTokenBlackBerryPush();

  explicit deviceTokenBlackBerryPush(string const &token_);

  static const std::int32_t ID = 1559167234;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenTizenPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string reg_id_;
  static jfieldID reg_id_fieldID;

  deviceTokenTizenPush();

  explicit deviceTokenTizenPush(string const &reg_id_);

  static const std::int32_t ID = -1359947213;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class deviceTokenHuaweiPush final : public DeviceToken {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;
  bool encrypt_;
  static jfieldID encrypt_fieldID;

  deviceTokenHuaweiPush();

  deviceTokenHuaweiPush(string const &token_, bool encrypt_);

  static const std::int32_t ID = 1989103142;

  static object_ptr<DeviceToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class DiceStickers: public Object {
 public:
  static jclass Class;

  static object_ptr<DiceStickers> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class diceStickersRegular final : public DiceStickers {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  diceStickersRegular();

  explicit diceStickersRegular(object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -740299570;

  static object_ptr<DiceStickers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class diceStickersSlotMachine final : public DiceStickers {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> background_;
  static jfieldID background_fieldID;
  object_ptr<sticker> lever_;
  static jfieldID lever_fieldID;
  object_ptr<sticker> left_reel_;
  static jfieldID left_reel_fieldID;
  object_ptr<sticker> center_reel_;
  static jfieldID center_reel_fieldID;
  object_ptr<sticker> right_reel_;
  static jfieldID right_reel_fieldID;

  diceStickersSlotMachine();

  diceStickersSlotMachine(object_ptr<sticker> &&background_, object_ptr<sticker> &&lever_, object_ptr<sticker> &&left_reel_, object_ptr<sticker> &&center_reel_, object_ptr<sticker> &&right_reel_);

  static const std::int32_t ID = -375223124;

  static object_ptr<DiceStickers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class thumbnail;

class document final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string file_name_;
  static jfieldID file_name_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<file> document_;
  static jfieldID document_fieldID;

  document();

  document(string const &file_name_, string const &mime_type_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&document_);

  static const std::int32_t ID = -1357271080;

  static object_ptr<document> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class downloadedFileCounts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 active_count_;
  static jfieldID active_count_fieldID;
  int32 paused_count_;
  static jfieldID paused_count_fieldID;
  int32 completed_count_;
  static jfieldID completed_count_fieldID;

  downloadedFileCounts();

  downloadedFileCounts(int32 active_count_, int32 paused_count_, int32 completed_count_);

  static const std::int32_t ID = -1973999550;

  static object_ptr<downloadedFileCounts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class draftMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<InputMessageContent> input_message_text_;
  static jfieldID input_message_text_fieldID;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;

  draftMessage();

  draftMessage(object_ptr<InputMessageReplyTo> &&reply_to_, int32 date_, object_ptr<InputMessageContent> &&input_message_text_, int64 effect_id_);

  static const std::int32_t ID = 1125328749;

  static object_ptr<draftMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmailAddressAuthentication: public Object {
 public:
  static jclass Class;

  static object_ptr<EmailAddressAuthentication> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCode final : public EmailAddressAuthentication {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  emailAddressAuthenticationCode();

  explicit emailAddressAuthenticationCode(string const &code_);

  static const std::int32_t ID = -993257022;

  static object_ptr<EmailAddressAuthentication> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationAppleId final : public EmailAddressAuthentication {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  emailAddressAuthenticationAppleId();

  explicit emailAddressAuthenticationAppleId(string const &token_);

  static const std::int32_t ID = 633948265;

  static object_ptr<EmailAddressAuthentication> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationGoogleId final : public EmailAddressAuthentication {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  emailAddressAuthenticationGoogleId();

  explicit emailAddressAuthenticationGoogleId(string const &token_);

  static const std::int32_t ID = -19142846;

  static object_ptr<EmailAddressAuthentication> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string email_address_pattern_;
  static jfieldID email_address_pattern_fieldID;
  int32 length_;
  static jfieldID length_fieldID;

  emailAddressAuthenticationCodeInfo();

  emailAddressAuthenticationCodeInfo(string const &email_address_pattern_, int32 length_);

  static const std::int32_t ID = 1151066659;

  static object_ptr<emailAddressAuthenticationCodeInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmailAddressResetState: public Object {
 public:
  static jclass Class;

  static object_ptr<EmailAddressResetState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressResetStateAvailable final : public EmailAddressResetState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 wait_period_;
  static jfieldID wait_period_fieldID;

  emailAddressResetStateAvailable();

  explicit emailAddressResetStateAvailable(int32 wait_period_);

  static const std::int32_t ID = -1917177600;

  static object_ptr<EmailAddressResetState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressResetStatePending final : public EmailAddressResetState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 reset_in_;
  static jfieldID reset_in_fieldID;

  emailAddressResetStatePending();

  explicit emailAddressResetStatePending(int32 reset_in_);

  static const std::int32_t ID = -1885966805;

  static object_ptr<EmailAddressResetState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategory;

class emojiCategories final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<emojiCategory>> categories_;
  static jfieldID categories_fieldID;

  emojiCategories();

  explicit emojiCategories(array<object_ptr<emojiCategory>> &&categories_);

  static const std::int32_t ID = -1455387824;

  static object_ptr<emojiCategories> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmojiCategorySource;

class sticker;

class emojiCategory final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<sticker> icon_;
  static jfieldID icon_fieldID;
  object_ptr<EmojiCategorySource> source_;
  static jfieldID source_fieldID;
  bool is_greeting_;
  static jfieldID is_greeting_fieldID;

  emojiCategory();

  emojiCategory(string const &name_, object_ptr<sticker> &&icon_, object_ptr<EmojiCategorySource> &&source_, bool is_greeting_);

  static const std::int32_t ID = 571335919;

  static object_ptr<emojiCategory> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmojiCategorySource: public Object {
 public:
  static jclass Class;

  static object_ptr<EmojiCategorySource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategorySourceSearch final : public EmojiCategorySource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  emojiCategorySourceSearch();

  explicit emojiCategorySourceSearch(array<string> &&emojis_);

  static const std::int32_t ID = -453260262;

  static object_ptr<EmojiCategorySource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategorySourcePremium final : public EmojiCategorySource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  emojiCategorySourcePremium();

  static const std::int32_t ID = -1932358388;

  static object_ptr<EmojiCategorySource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmojiCategoryType: public Object {
 public:
  static jclass Class;

  static object_ptr<EmojiCategoryType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategoryTypeDefault final : public EmojiCategoryType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  emojiCategoryTypeDefault();

  static const std::int32_t ID = 1188782699;

  static object_ptr<EmojiCategoryType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategoryTypeRegularStickers final : public EmojiCategoryType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  emojiCategoryTypeRegularStickers();

  static const std::int32_t ID = -1337484846;

  static object_ptr<EmojiCategoryType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategoryTypeEmojiStatus final : public EmojiCategoryType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  emojiCategoryTypeEmojiStatus();

  static const std::int32_t ID = 1381282631;

  static object_ptr<EmojiCategoryType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiCategoryTypeChatPhoto final : public EmojiCategoryType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  emojiCategoryTypeChatPhoto();

  static const std::int32_t ID = 1059063081;

  static object_ptr<EmojiCategoryType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiKeyword final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;
  string keyword_;
  static jfieldID keyword_fieldID;

  emojiKeyword();

  emojiKeyword(string const &emoji_, string const &keyword_);

  static const std::int32_t ID = -2112285985;

  static object_ptr<emojiKeyword> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiKeyword;

class emojiKeywords final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<emojiKeyword>> emoji_keywords_;
  static jfieldID emoji_keywords_fieldID;

  emojiKeywords();

  explicit emojiKeywords(array<object_ptr<emojiKeyword>> &&emoji_keywords_);

  static const std::int32_t ID = 689723339;

  static object_ptr<emojiKeywords> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class emojiReaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;
  string title_;
  static jfieldID title_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;
  object_ptr<sticker> static_icon_;
  static jfieldID static_icon_fieldID;
  object_ptr<sticker> appear_animation_;
  static jfieldID appear_animation_fieldID;
  object_ptr<sticker> select_animation_;
  static jfieldID select_animation_fieldID;
  object_ptr<sticker> activate_animation_;
  static jfieldID activate_animation_fieldID;
  object_ptr<sticker> effect_animation_;
  static jfieldID effect_animation_fieldID;
  object_ptr<sticker> around_animation_;
  static jfieldID around_animation_fieldID;
  object_ptr<sticker> center_animation_;
  static jfieldID center_animation_fieldID;

  emojiReaction();

  emojiReaction(string const &emoji_, string const &title_, bool is_active_, object_ptr<sticker> &&static_icon_, object_ptr<sticker> &&appear_animation_, object_ptr<sticker> &&select_animation_, object_ptr<sticker> &&activate_animation_, object_ptr<sticker> &&effect_animation_, object_ptr<sticker> &&around_animation_, object_ptr<sticker> &&center_animation_);

  static const std::int32_t ID = 1616063583;

  static object_ptr<emojiReaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class EmojiStatusType;

class emojiStatus final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<EmojiStatusType> type_;
  static jfieldID type_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;

  emojiStatus();

  emojiStatus(object_ptr<EmojiStatusType> &&type_, int32 expiration_date_);

  static const std::int32_t ID = 973424912;

  static object_ptr<emojiStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int64> custom_emoji_ids_;
  static jfieldID custom_emoji_ids_fieldID;

  emojiStatusCustomEmojis();

  explicit emojiStatusCustomEmojis(array<int64> &&custom_emoji_ids_);

  static const std::int32_t ID = 917123337;

  static object_ptr<emojiStatusCustomEmojis> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftBackdropColors;

class EmojiStatusType: public Object {
 public:
  static jclass Class;

  static object_ptr<EmojiStatusType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusTypeCustomEmoji final : public EmojiStatusType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  emojiStatusTypeCustomEmoji();

  explicit emojiStatusTypeCustomEmoji(int64 custom_emoji_id_);

  static const std::int32_t ID = -1666780939;

  static object_ptr<EmojiStatusType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusTypeUpgradedGift final : public EmojiStatusType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 upgraded_gift_id_;
  static jfieldID upgraded_gift_id_fieldID;
  string gift_title_;
  static jfieldID gift_title_fieldID;
  string gift_name_;
  static jfieldID gift_name_fieldID;
  int64 model_custom_emoji_id_;
  static jfieldID model_custom_emoji_id_fieldID;
  int64 symbol_custom_emoji_id_;
  static jfieldID symbol_custom_emoji_id_fieldID;
  object_ptr<upgradedGiftBackdropColors> backdrop_colors_;
  static jfieldID backdrop_colors_fieldID;

  emojiStatusTypeUpgradedGift();

  emojiStatusTypeUpgradedGift(int64 upgraded_gift_id_, string const &gift_title_, string const &gift_name_, int64 model_custom_emoji_id_, int64 symbol_custom_emoji_id_, object_ptr<upgradedGiftBackdropColors> &&backdrop_colors_);

  static const std::int32_t ID = -837921804;

  static object_ptr<EmojiStatusType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatus;

class emojiStatuses final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<emojiStatus>> emoji_statuses_;
  static jfieldID emoji_statuses_fieldID;

  emojiStatuses();

  explicit emojiStatuses(array<object_ptr<emojiStatus>> &&emoji_statuses_);

  static const std::int32_t ID = 1186104146;

  static object_ptr<emojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emojis final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  emojis();

  explicit emojis(array<string> &&emojis_);

  static const std::int32_t ID = 950339552;

  static object_ptr<emojis> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class encryptedCredentials final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes data_;
  static jfieldID data_fieldID;
  bytes hash_;
  static jfieldID hash_fieldID;
  bytes secret_;
  static jfieldID secret_fieldID;

  encryptedCredentials();

  encryptedCredentials(bytes const &data_, bytes const &hash_, bytes const &secret_);

  static const std::int32_t ID = 1331106766;

  static object_ptr<encryptedCredentials> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementType;

class datedFile;

class encryptedPassportElement final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;
  bytes data_;
  static jfieldID data_fieldID;
  object_ptr<datedFile> front_side_;
  static jfieldID front_side_fieldID;
  object_ptr<datedFile> reverse_side_;
  static jfieldID reverse_side_fieldID;
  object_ptr<datedFile> selfie_;
  static jfieldID selfie_fieldID;
  array<object_ptr<datedFile>> translation_;
  static jfieldID translation_fieldID;
  array<object_ptr<datedFile>> files_;
  static jfieldID files_fieldID;
  string value_;
  static jfieldID value_fieldID;
  string hash_;
  static jfieldID hash_fieldID;

  encryptedPassportElement();

  encryptedPassportElement(object_ptr<PassportElementType> &&type_, bytes const &data_, object_ptr<datedFile> &&front_side_, object_ptr<datedFile> &&reverse_side_, object_ptr<datedFile> &&selfie_, array<object_ptr<datedFile>> &&translation_, array<object_ptr<datedFile>> &&files_, string const &value_, string const &hash_);

  static const std::int32_t ID = 2002386193;

  static object_ptr<encryptedPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class error final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 code_;
  static jfieldID code_fieldID;
  string message_;
  static jfieldID message_fieldID;

  error();

  error(int32 code_, string const &message_);

  static const std::int32_t ID = -1679978726;

  static object_ptr<error> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class factCheck final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string country_code_;
  static jfieldID country_code_fieldID;

  factCheck();

  factCheck(object_ptr<formattedText> &&text_, string const &country_code_);

  static const std::int32_t ID = -1048184552;

  static object_ptr<factCheck> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class failedToAddMember final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool premium_would_allow_invite_;
  static jfieldID premium_would_allow_invite_fieldID;
  bool premium_required_to_send_messages_;
  static jfieldID premium_required_to_send_messages_fieldID;

  failedToAddMember();

  failedToAddMember(int53 user_id_, bool premium_would_allow_invite_, bool premium_required_to_send_messages_);

  static const std::int32_t ID = -282891070;

  static object_ptr<failedToAddMember> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class failedToAddMember;

class failedToAddMembers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<failedToAddMember>> failed_to_add_members_;
  static jfieldID failed_to_add_members_fieldID;

  failedToAddMembers();

  explicit failedToAddMembers(array<object_ptr<failedToAddMember>> &&failed_to_add_members_);

  static const std::int32_t ID = -272587152;

  static object_ptr<failedToAddMembers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class localFile;

class remoteFile;

class file final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int53 size_;
  static jfieldID size_fieldID;
  int53 expected_size_;
  static jfieldID expected_size_fieldID;
  object_ptr<localFile> local_;
  static jfieldID local_fieldID;
  object_ptr<remoteFile> remote_;
  static jfieldID remote_fieldID;

  file();

  file(int32 id_, int53 size_, int53 expected_size_, object_ptr<localFile> &&local_, object_ptr<remoteFile> &&remote_);

  static const std::int32_t ID = 1263291956;

  static object_ptr<file> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class fileDownload final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  int32 add_date_;
  static jfieldID add_date_fieldID;
  int32 complete_date_;
  static jfieldID complete_date_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;

  fileDownload();

  fileDownload(int32 file_id_, object_ptr<message> &&message_, int32 add_date_, int32 complete_date_, bool is_paused_);

  static const std::int32_t ID = -2092100780;

  static object_ptr<fileDownload> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileDownloadedPrefixSize final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 size_;
  static jfieldID size_fieldID;

  fileDownloadedPrefixSize();

  explicit fileDownloadedPrefixSize(int53 size_);

  static const std::int32_t ID = -2015205381;

  static object_ptr<fileDownloadedPrefixSize> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FileType: public Object {
 public:
  static jclass Class;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeNone final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeNone();

  static const std::int32_t ID = 2003009189;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeAnimation final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeAnimation();

  static const std::int32_t ID = -290816582;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeAudio final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeAudio();

  static const std::int32_t ID = -709112160;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeDocument final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeDocument();

  static const std::int32_t ID = -564722929;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeNotificationSound final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeNotificationSound();

  static const std::int32_t ID = -1020289271;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypePhoto final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypePhoto();

  static const std::int32_t ID = -1718914651;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypePhotoStory final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypePhotoStory();

  static const std::int32_t ID = 2018995956;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeProfilePhoto final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeProfilePhoto();

  static const std::int32_t ID = 1795089315;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSecret final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSecret();

  static const std::int32_t ID = -1871899401;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSecretThumbnail final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSecretThumbnail();

  static const std::int32_t ID = -1401326026;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSecure final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSecure();

  static const std::int32_t ID = -1419133146;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSelfDestructingPhoto final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSelfDestructingPhoto();

  static const std::int32_t ID = 2077176475;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSelfDestructingVideo final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSelfDestructingVideo();

  static const std::int32_t ID = -1223900123;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSelfDestructingVideoNote final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSelfDestructingVideoNote();

  static const std::int32_t ID = 1495274177;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSelfDestructingVoiceNote final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSelfDestructingVoiceNote();

  static const std::int32_t ID = 1691409181;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeSticker final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeSticker();

  static const std::int32_t ID = 475233385;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeThumbnail final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeThumbnail();

  static const std::int32_t ID = -12443298;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeUnknown final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeUnknown();

  static const std::int32_t ID = -2011566768;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeVideo final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeVideo();

  static const std::int32_t ID = 1430816539;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeVideoNote final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeVideoNote();

  static const std::int32_t ID = -518412385;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeVideoStory final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeVideoStory();

  static const std::int32_t ID = -2146754143;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeVoiceNote final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeVoiceNote();

  static const std::int32_t ID = -588681661;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class fileTypeWallpaper final : public FileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  fileTypeWallpaper();

  static const std::int32_t ID = 1854930076;

  static object_ptr<FileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FirebaseAuthenticationSettings: public Object {
 public:
  static jclass Class;

  static object_ptr<FirebaseAuthenticationSettings> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class firebaseAuthenticationSettingsAndroid final : public FirebaseAuthenticationSettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  firebaseAuthenticationSettingsAndroid();

  static const std::int32_t ID = -1771112932;

  static object_ptr<FirebaseAuthenticationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class firebaseAuthenticationSettingsIos final : public FirebaseAuthenticationSettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string device_token_;
  static jfieldID device_token_fieldID;
  bool is_app_sandbox_;
  static jfieldID is_app_sandbox_fieldID;

  firebaseAuthenticationSettingsIos();

  firebaseAuthenticationSettingsIos(string const &device_token_, bool is_app_sandbox_);

  static const std::int32_t ID = 222930116;

  static object_ptr<FirebaseAuthenticationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FirebaseDeviceVerificationParameters: public Object {
 public:
  static jclass Class;

  static object_ptr<FirebaseDeviceVerificationParameters> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class firebaseDeviceVerificationParametersSafetyNet final : public FirebaseDeviceVerificationParameters {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes nonce_;
  static jfieldID nonce_fieldID;

  firebaseDeviceVerificationParametersSafetyNet();

  explicit firebaseDeviceVerificationParametersSafetyNet(bytes const &nonce_);

  static const std::int32_t ID = 731296497;

  static object_ptr<FirebaseDeviceVerificationParameters> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class firebaseDeviceVerificationParametersPlayIntegrity final : public FirebaseDeviceVerificationParameters {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string nonce_;
  static jfieldID nonce_fieldID;
  int64 cloud_project_number_;
  static jfieldID cloud_project_number_fieldID;

  firebaseDeviceVerificationParametersPlayIntegrity();

  firebaseDeviceVerificationParametersPlayIntegrity(string const &nonce_, int64 cloud_project_number_);

  static const std::int32_t ID = -889936502;

  static object_ptr<FirebaseDeviceVerificationParameters> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntity;

class formattedText final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  array<object_ptr<textEntity>> entities_;
  static jfieldID entities_fieldID;

  formattedText();

  formattedText(string const &text_, array<object_ptr<textEntity>> &&entities_);

  static const std::int32_t ID = -252624564;

  static object_ptr<formattedText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatNotificationSettings;

class draftMessage;

class forumTopicInfo;

class message;

class forumTopic final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> info_;
  static jfieldID info_fieldID;
  object_ptr<message> last_message_;
  static jfieldID last_message_fieldID;
  int64 order_;
  static jfieldID order_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  int32 unread_count_;
  static jfieldID unread_count_fieldID;
  int53 last_read_inbox_message_id_;
  static jfieldID last_read_inbox_message_id_fieldID;
  int53 last_read_outbox_message_id_;
  static jfieldID last_read_outbox_message_id_fieldID;
  int32 unread_mention_count_;
  static jfieldID unread_mention_count_fieldID;
  int32 unread_reaction_count_;
  static jfieldID unread_reaction_count_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;

  forumTopic();

  forumTopic(object_ptr<forumTopicInfo> &&info_, object_ptr<message> &&last_message_, int64 order_, bool is_pinned_, int32 unread_count_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int32 unread_mention_count_, int32 unread_reaction_count_, object_ptr<chatNotificationSettings> &&notification_settings_, object_ptr<draftMessage> &&draft_message_);

  static const std::int32_t ID = -2094608976;

  static object_ptr<forumTopic> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class forumTopicIcon final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 color_;
  static jfieldID color_fieldID;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  forumTopicIcon();

  forumTopicIcon(int32 color_, int64 custom_emoji_id_);

  static const std::int32_t ID = -818765421;

  static object_ptr<forumTopicIcon> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class forumTopicIcon;

class forumTopicInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<forumTopicIcon> icon_;
  static jfieldID icon_fieldID;
  int32 creation_date_;
  static jfieldID creation_date_fieldID;
  object_ptr<MessageSender> creator_id_;
  static jfieldID creator_id_fieldID;
  bool is_general_;
  static jfieldID is_general_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;
  bool is_closed_;
  static jfieldID is_closed_fieldID;
  bool is_hidden_;
  static jfieldID is_hidden_fieldID;

  forumTopicInfo();

  forumTopicInfo(int53 chat_id_, int53 message_thread_id_, string const &name_, object_ptr<forumTopicIcon> &&icon_, int32 creation_date_, object_ptr<MessageSender> &&creator_id_, bool is_general_, bool is_outgoing_, bool is_closed_, bool is_hidden_);

  static const std::int32_t ID = -654857606;

  static object_ptr<forumTopicInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class forumTopic;

class forumTopics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<forumTopic>> topics_;
  static jfieldID topics_fieldID;
  int32 next_offset_date_;
  static jfieldID next_offset_date_fieldID;
  int53 next_offset_message_id_;
  static jfieldID next_offset_message_id_fieldID;
  int53 next_offset_message_thread_id_;
  static jfieldID next_offset_message_thread_id_fieldID;

  forumTopics();

  forumTopics(int32 total_count_, array<object_ptr<forumTopic>> &&topics_, int32 next_offset_date_, int53 next_offset_message_id_, int53 next_offset_message_thread_id_);

  static const std::int32_t ID = 732819537;

  static object_ptr<forumTopics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class forwardSource final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  string sender_name_;
  static jfieldID sender_name_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;

  forwardSource();

  forwardSource(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&sender_id_, string const &sender_name_, int32 date_, bool is_outgoing_);

  static const std::int32_t ID = 1795337929;

  static object_ptr<forwardSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramInfo;

class foundAffiliateProgram final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<affiliateProgramInfo> info_;
  static jfieldID info_fieldID;

  foundAffiliateProgram();

  foundAffiliateProgram(int53 bot_user_id_, object_ptr<affiliateProgramInfo> &&info_);

  static const std::int32_t ID = -966565242;

  static object_ptr<foundAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class foundAffiliateProgram;

class foundAffiliatePrograms final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<foundAffiliateProgram>> programs_;
  static jfieldID programs_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundAffiliatePrograms();

  foundAffiliatePrograms(int32 total_count_, array<object_ptr<foundAffiliateProgram>> &&programs_, string const &next_offset_);

  static const std::int32_t ID = 186317057;

  static object_ptr<foundAffiliatePrograms> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatBoost;

class foundChatBoosts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<chatBoost>> boosts_;
  static jfieldID boosts_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundChatBoosts();

  foundChatBoosts(int32 total_count_, array<object_ptr<chatBoost>> &&boosts_, string const &next_offset_);

  static const std::int32_t ID = 51457680;

  static object_ptr<foundChatBoosts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class foundChatMessages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<message>> messages_;
  static jfieldID messages_fieldID;
  int53 next_from_message_id_;
  static jfieldID next_from_message_id_fieldID;

  foundChatMessages();

  foundChatMessages(int32 total_count_, array<object_ptr<message>> &&messages_, int53 next_from_message_id_);

  static const std::int32_t ID = 427484196;

  static object_ptr<foundChatMessages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class downloadedFileCounts;

class fileDownload;

class foundFileDownloads final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<downloadedFileCounts> total_counts_;
  static jfieldID total_counts_fieldID;
  array<object_ptr<fileDownload>> files_;
  static jfieldID files_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundFileDownloads();

  foundFileDownloads(object_ptr<downloadedFileCounts> &&total_counts_, array<object_ptr<fileDownload>> &&files_, string const &next_offset_);

  static const std::int32_t ID = 1395890392;

  static object_ptr<foundFileDownloads> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class foundMessages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<message>> messages_;
  static jfieldID messages_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundMessages();

  foundMessages(int32 total_count_, array<object_ptr<message>> &&messages_, string const &next_offset_);

  static const std::int32_t ID = -529809608;

  static object_ptr<foundMessages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class foundPosition final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 position_;
  static jfieldID position_fieldID;

  foundPosition();

  explicit foundPosition(int32 position_);

  static const std::int32_t ID = -1886724216;

  static object_ptr<foundPosition> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class foundPositions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<int32> positions_;
  static jfieldID positions_fieldID;

  foundPositions();

  foundPositions(int32 total_count_, array<int32> &&positions_);

  static const std::int32_t ID = -80518368;

  static object_ptr<foundPositions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class story;

class foundStories final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<story>> stories_;
  static jfieldID stories_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundStories();

  foundStories(int32 total_count_, array<object_ptr<story>> &&stories_, string const &next_offset_);

  static const std::int32_t ID = 1678513512;

  static object_ptr<foundStories> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class foundUsers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  foundUsers();

  foundUsers(array<int53> &&user_ids_, string const &next_offset_);

  static const std::int32_t ID = 1150570075;

  static object_ptr<foundUsers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class webApp;

class foundWebApp final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<webApp> web_app_;
  static jfieldID web_app_fieldID;
  bool request_write_access_;
  static jfieldID request_write_access_fieldID;
  bool skip_confirmation_;
  static jfieldID skip_confirmation_fieldID;

  foundWebApp();

  foundWebApp(object_ptr<webApp> &&web_app_, bool request_write_access_, bool skip_confirmation_);

  static const std::int32_t ID = -290926562;

  static object_ptr<foundWebApp> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class animation;

class formattedText;

class photo;

class game final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string short_name_;
  static jfieldID short_name_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;

  game();

  game(int64 id_, string const &short_name_, string const &title_, object_ptr<formattedText> &&text_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_);

  static const std::int32_t ID = -1565597752;

  static object_ptr<game> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class gameHighScore final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 position_;
  static jfieldID position_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 score_;
  static jfieldID score_fieldID;

  gameHighScore();

  gameHighScore(int32 position_, int53 user_id_, int32 score_);

  static const std::int32_t ID = 342871838;

  static object_ptr<gameHighScore> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class gameHighScore;

class gameHighScores final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<gameHighScore>> scores_;
  static jfieldID scores_fieldID;

  gameHighScores();

  explicit gameHighScores(array<object_ptr<gameHighScore>> &&scores_);

  static const std::int32_t ID = -725770727;

  static object_ptr<gameHighScores> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class gift final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  int53 default_sell_star_count_;
  static jfieldID default_sell_star_count_fieldID;
  int53 upgrade_star_count_;
  static jfieldID upgrade_star_count_fieldID;
  bool is_for_birthday_;
  static jfieldID is_for_birthday_fieldID;
  int32 remaining_count_;
  static jfieldID remaining_count_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  int32 first_send_date_;
  static jfieldID first_send_date_fieldID;
  int32 last_send_date_;
  static jfieldID last_send_date_fieldID;

  gift();

  gift(int64 id_, object_ptr<sticker> &&sticker_, int53 star_count_, int53 default_sell_star_count_, int53 upgrade_star_count_, bool is_for_birthday_, int32 remaining_count_, int32 total_count_, int32 first_send_date_, int32 last_send_date_);

  static const std::int32_t ID = 2096710701;

  static object_ptr<gift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGift;

class giftForResale final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;

  giftForResale();

  giftForResale(object_ptr<upgradedGift> &&gift_, string const &received_gift_id_);

  static const std::int32_t ID = 1130990515;

  static object_ptr<giftForResale> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiftForResaleOrder: public Object {
 public:
  static jclass Class;

  static object_ptr<GiftForResaleOrder> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giftForResaleOrderPrice final : public GiftForResaleOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  giftForResaleOrderPrice();

  static const std::int32_t ID = 1371740258;

  static object_ptr<GiftForResaleOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giftForResaleOrderPriceChangeDate final : public GiftForResaleOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  giftForResaleOrderPriceChangeDate();

  static const std::int32_t ID = -1694144054;

  static object_ptr<GiftForResaleOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giftForResaleOrderNumber final : public GiftForResaleOrder {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  giftForResaleOrderNumber();

  static const std::int32_t ID = -1301157632;

  static object_ptr<GiftForResaleOrder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class acceptedGiftTypes;

class giftSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool show_gift_button_;
  static jfieldID show_gift_button_fieldID;
  object_ptr<acceptedGiftTypes> accepted_gift_types_;
  static jfieldID accepted_gift_types_fieldID;

  giftSettings();

  giftSettings(bool show_gift_button_, object_ptr<acceptedGiftTypes> &&accepted_gift_types_);

  static const std::int32_t ID = 45783168;

  static object_ptr<giftSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftBackdrop;

class upgradedGiftModel;

class upgradedGiftSymbol;

class giftUpgradePreview final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<upgradedGiftModel>> models_;
  static jfieldID models_fieldID;
  array<object_ptr<upgradedGiftSymbol>> symbols_;
  static jfieldID symbols_fieldID;
  array<object_ptr<upgradedGiftBackdrop>> backdrops_;
  static jfieldID backdrops_fieldID;

  giftUpgradePreview();

  giftUpgradePreview(array<object_ptr<upgradedGiftModel>> &&models_, array<object_ptr<upgradedGiftSymbol>> &&symbols_, array<object_ptr<upgradedGiftBackdrop>> &&backdrops_);

  static const std::int32_t ID = 729908218;

  static object_ptr<giftUpgradePreview> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giftForResale;

class upgradedGiftBackdropCount;

class upgradedGiftModelCount;

class upgradedGiftSymbolCount;

class giftsForResale final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<giftForResale>> gifts_;
  static jfieldID gifts_fieldID;
  array<object_ptr<upgradedGiftModelCount>> models_;
  static jfieldID models_fieldID;
  array<object_ptr<upgradedGiftSymbolCount>> symbols_;
  static jfieldID symbols_fieldID;
  array<object_ptr<upgradedGiftBackdropCount>> backdrops_;
  static jfieldID backdrops_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  giftsForResale();

  giftsForResale(int32 total_count_, array<object_ptr<giftForResale>> &&gifts_, array<object_ptr<upgradedGiftModelCount>> &&models_, array<object_ptr<upgradedGiftSymbolCount>> &&symbols_, array<object_ptr<upgradedGiftBackdropCount>> &&backdrops_, string const &next_offset_);

  static const std::int32_t ID = 35082425;

  static object_ptr<giftsForResale> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayParticipantStatus;

class GiveawayInfo: public Object {
 public:
  static jclass Class;

  static object_ptr<GiveawayInfo> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giveawayInfoOngoing final : public GiveawayInfo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 creation_date_;
  static jfieldID creation_date_fieldID;
  object_ptr<GiveawayParticipantStatus> status_;
  static jfieldID status_fieldID;
  bool is_ended_;
  static jfieldID is_ended_fieldID;

  giveawayInfoOngoing();

  giveawayInfoOngoing(int32 creation_date_, object_ptr<GiveawayParticipantStatus> &&status_, bool is_ended_);

  static const std::int32_t ID = 1649336400;

  static object_ptr<GiveawayInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayInfoCompleted final : public GiveawayInfo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 creation_date_;
  static jfieldID creation_date_fieldID;
  int32 actual_winners_selection_date_;
  static jfieldID actual_winners_selection_date_fieldID;
  bool was_refunded_;
  static jfieldID was_refunded_fieldID;
  bool is_winner_;
  static jfieldID is_winner_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int32 activation_count_;
  static jfieldID activation_count_fieldID;
  string gift_code_;
  static jfieldID gift_code_fieldID;
  int53 won_star_count_;
  static jfieldID won_star_count_fieldID;

  giveawayInfoCompleted();

  giveawayInfoCompleted(int32 creation_date_, int32 actual_winners_selection_date_, bool was_refunded_, bool is_winner_, int32 winner_count_, int32 activation_count_, string const &gift_code_, int53 won_star_count_);

  static const std::int32_t ID = 848085852;

  static object_ptr<GiveawayInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;
  array<int53> additional_chat_ids_;
  static jfieldID additional_chat_ids_fieldID;
  int32 winners_selection_date_;
  static jfieldID winners_selection_date_fieldID;
  bool only_new_members_;
  static jfieldID only_new_members_fieldID;
  bool has_public_winners_;
  static jfieldID has_public_winners_fieldID;
  array<string> country_codes_;
  static jfieldID country_codes_fieldID;
  string prize_description_;
  static jfieldID prize_description_fieldID;

  giveawayParameters();

  giveawayParameters(int53 boosted_chat_id_, array<int53> &&additional_chat_ids_, int32 winners_selection_date_, bool only_new_members_, bool has_public_winners_, array<string> &&country_codes_, string const &prize_description_);

  static const std::int32_t ID = 1171549354;

  static object_ptr<giveawayParameters> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayParticipantStatus: public Object {
 public:
  static jclass Class;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParticipantStatusEligible final : public GiveawayParticipantStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  giveawayParticipantStatusEligible();

  static const std::int32_t ID = 304799383;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParticipantStatusParticipating final : public GiveawayParticipantStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  giveawayParticipantStatusParticipating();

  static const std::int32_t ID = 492036975;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParticipantStatusAlreadyWasMember final : public GiveawayParticipantStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 joined_chat_date_;
  static jfieldID joined_chat_date_fieldID;

  giveawayParticipantStatusAlreadyWasMember();

  explicit giveawayParticipantStatusAlreadyWasMember(int32 joined_chat_date_);

  static const std::int32_t ID = 301577632;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParticipantStatusAdministrator final : public GiveawayParticipantStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  giveawayParticipantStatusAdministrator();

  explicit giveawayParticipantStatusAdministrator(int53 chat_id_);

  static const std::int32_t ID = -934593931;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParticipantStatusDisallowedCountry final : public GiveawayParticipantStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string user_country_code_;
  static jfieldID user_country_code_fieldID;

  giveawayParticipantStatusDisallowedCountry();

  explicit giveawayParticipantStatusDisallowedCountry(string const &user_country_code_);

  static const std::int32_t ID = 1879794779;

  static object_ptr<GiveawayParticipantStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayPrize: public Object {
 public:
  static jclass Class;

  static object_ptr<GiveawayPrize> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giveawayPrizePremium final : public GiveawayPrize {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 month_count_;
  static jfieldID month_count_fieldID;

  giveawayPrizePremium();

  explicit giveawayPrizePremium(int32 month_count_);

  static const std::int32_t ID = 454224248;

  static object_ptr<GiveawayPrize> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class giveawayPrizeStars final : public GiveawayPrize {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  giveawayPrizeStars();

  explicit giveawayPrizeStars(int53 star_count_);

  static const std::int32_t ID = -1790173276;

  static object_ptr<GiveawayPrize> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallRecentSpeaker;

class groupCall final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  int32 scheduled_start_date_;
  static jfieldID scheduled_start_date_fieldID;
  bool enabled_start_notification_;
  static jfieldID enabled_start_notification_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;
  bool is_video_chat_;
  static jfieldID is_video_chat_fieldID;
  bool is_rtmp_stream_;
  static jfieldID is_rtmp_stream_fieldID;
  bool is_joined_;
  static jfieldID is_joined_fieldID;
  bool need_rejoin_;
  static jfieldID need_rejoin_fieldID;
  bool is_owned_;
  static jfieldID is_owned_fieldID;
  bool can_be_managed_;
  static jfieldID can_be_managed_fieldID;
  int32 participant_count_;
  static jfieldID participant_count_fieldID;
  bool has_hidden_listeners_;
  static jfieldID has_hidden_listeners_fieldID;
  bool loaded_all_participants_;
  static jfieldID loaded_all_participants_fieldID;
  array<object_ptr<groupCallRecentSpeaker>> recent_speakers_;
  static jfieldID recent_speakers_fieldID;
  bool is_my_video_enabled_;
  static jfieldID is_my_video_enabled_fieldID;
  bool is_my_video_paused_;
  static jfieldID is_my_video_paused_fieldID;
  bool can_enable_video_;
  static jfieldID can_enable_video_fieldID;
  bool mute_new_participants_;
  static jfieldID mute_new_participants_fieldID;
  bool can_toggle_mute_new_participants_;
  static jfieldID can_toggle_mute_new_participants_fieldID;
  int32 record_duration_;
  static jfieldID record_duration_fieldID;
  bool is_video_recorded_;
  static jfieldID is_video_recorded_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;

  groupCall();

  groupCall(int32 id_, string const &title_, string const &invite_link_, int32 scheduled_start_date_, bool enabled_start_notification_, bool is_active_, bool is_video_chat_, bool is_rtmp_stream_, bool is_joined_, bool need_rejoin_, bool is_owned_, bool can_be_managed_, int32 participant_count_, bool has_hidden_listeners_, bool loaded_all_participants_, array<object_ptr<groupCallRecentSpeaker>> &&recent_speakers_, bool is_my_video_enabled_, bool is_my_video_paused_, bool can_enable_video_, bool mute_new_participants_, bool can_toggle_mute_new_participants_, int32 record_duration_, bool is_video_recorded_, int32 duration_);

  static const std::int32_t ID = -144438687;

  static object_ptr<groupCall> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GroupCallDataChannel: public Object {
 public:
  static jclass Class;

  static object_ptr<GroupCallDataChannel> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class groupCallDataChannelMain final : public GroupCallDataChannel {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  groupCallDataChannelMain();

  static const std::int32_t ID = -32177779;

  static object_ptr<GroupCallDataChannel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallDataChannelScreenSharing final : public GroupCallDataChannel {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  groupCallDataChannelScreenSharing();

  static const std::int32_t ID = -601649103;

  static object_ptr<GroupCallDataChannel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallId final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;

  groupCallId();

  explicit groupCallId(int32 id_);

  static const std::int32_t ID = 350534469;

  static object_ptr<groupCallId> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  string join_payload_;
  static jfieldID join_payload_fieldID;

  groupCallInfo();

  groupCallInfo(int32 group_call_id_, string const &join_payload_);

  static const std::int32_t ID = 892575956;

  static object_ptr<groupCallInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallJoinParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 audio_source_id_;
  static jfieldID audio_source_id_fieldID;
  string payload_;
  static jfieldID payload_fieldID;
  bool is_muted_;
  static jfieldID is_muted_fieldID;
  bool is_my_video_enabled_;
  static jfieldID is_my_video_enabled_fieldID;

  groupCallJoinParameters();

  groupCallJoinParameters(int32 audio_source_id_, string const &payload_, bool is_muted_, bool is_my_video_enabled_);

  static const std::int32_t ID = 1763438054;

  static object_ptr<groupCallJoinParameters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class groupCallParticipantVideoInfo;

class groupCallParticipant final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  int32 audio_source_id_;
  static jfieldID audio_source_id_fieldID;
  int32 screen_sharing_audio_source_id_;
  static jfieldID screen_sharing_audio_source_id_fieldID;
  object_ptr<groupCallParticipantVideoInfo> video_info_;
  static jfieldID video_info_fieldID;
  object_ptr<groupCallParticipantVideoInfo> screen_sharing_video_info_;
  static jfieldID screen_sharing_video_info_fieldID;
  string bio_;
  static jfieldID bio_fieldID;
  bool is_current_user_;
  static jfieldID is_current_user_fieldID;
  bool is_speaking_;
  static jfieldID is_speaking_fieldID;
  bool is_hand_raised_;
  static jfieldID is_hand_raised_fieldID;
  bool can_be_muted_for_all_users_;
  static jfieldID can_be_muted_for_all_users_fieldID;
  bool can_be_unmuted_for_all_users_;
  static jfieldID can_be_unmuted_for_all_users_fieldID;
  bool can_be_muted_for_current_user_;
  static jfieldID can_be_muted_for_current_user_fieldID;
  bool can_be_unmuted_for_current_user_;
  static jfieldID can_be_unmuted_for_current_user_fieldID;
  bool is_muted_for_all_users_;
  static jfieldID is_muted_for_all_users_fieldID;
  bool is_muted_for_current_user_;
  static jfieldID is_muted_for_current_user_fieldID;
  bool can_unmute_self_;
  static jfieldID can_unmute_self_fieldID;
  int32 volume_level_;
  static jfieldID volume_level_fieldID;
  string order_;
  static jfieldID order_fieldID;

  groupCallParticipant();

  groupCallParticipant(object_ptr<MessageSender> &&participant_id_, int32 audio_source_id_, int32 screen_sharing_audio_source_id_, object_ptr<groupCallParticipantVideoInfo> &&video_info_, object_ptr<groupCallParticipantVideoInfo> &&screen_sharing_video_info_, string const &bio_, bool is_current_user_, bool is_speaking_, bool is_hand_raised_, bool can_be_muted_for_all_users_, bool can_be_unmuted_for_all_users_, bool can_be_muted_for_current_user_, bool can_be_unmuted_for_current_user_, bool is_muted_for_all_users_, bool is_muted_for_current_user_, bool can_unmute_self_, int32 volume_level_, string const &order_);

  static const std::int32_t ID = 2059182571;

  static object_ptr<groupCallParticipant> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallVideoSourceGroup;

class groupCallParticipantVideoInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<groupCallVideoSourceGroup>> source_groups_;
  static jfieldID source_groups_fieldID;
  string endpoint_id_;
  static jfieldID endpoint_id_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;

  groupCallParticipantVideoInfo();

  groupCallParticipantVideoInfo(array<object_ptr<groupCallVideoSourceGroup>> &&source_groups_, string const &endpoint_id_, bool is_paused_);

  static const std::int32_t ID = -14294645;

  static object_ptr<groupCallParticipantVideoInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class groupCallParticipants final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<MessageSender>> participant_ids_;
  static jfieldID participant_ids_fieldID;

  groupCallParticipants();

  groupCallParticipants(int32 total_count_, array<object_ptr<MessageSender>> &&participant_ids_);

  static const std::int32_t ID = -1042491570;

  static object_ptr<groupCallParticipants> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class groupCallRecentSpeaker final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  bool is_speaking_;
  static jfieldID is_speaking_fieldID;

  groupCallRecentSpeaker();

  groupCallRecentSpeaker(object_ptr<MessageSender> &&participant_id_, bool is_speaking_);

  static const std::int32_t ID = 1819519436;

  static object_ptr<groupCallRecentSpeaker> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GroupCallVideoQuality: public Object {
 public:
  static jclass Class;

  static object_ptr<GroupCallVideoQuality> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class groupCallVideoQualityThumbnail final : public GroupCallVideoQuality {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  groupCallVideoQualityThumbnail();

  static const std::int32_t ID = -379186304;

  static object_ptr<GroupCallVideoQuality> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallVideoQualityMedium final : public GroupCallVideoQuality {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  groupCallVideoQualityMedium();

  static const std::int32_t ID = 394968234;

  static object_ptr<GroupCallVideoQuality> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallVideoQualityFull final : public GroupCallVideoQuality {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  groupCallVideoQualityFull();

  static const std::int32_t ID = -2125916617;

  static object_ptr<GroupCallVideoQuality> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class groupCallVideoSourceGroup final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string semantics_;
  static jfieldID semantics_fieldID;
  array<int32> source_ids_;
  static jfieldID source_ids_fieldID;

  groupCallVideoSourceGroup();

  groupCallVideoSourceGroup(string const &semantics_, array<int32> &&source_ids_);

  static const std::int32_t ID = -1190900785;

  static object_ptr<groupCallVideoSourceGroup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class hashtags final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> hashtags_;
  static jfieldID hashtags_fieldID;

  hashtags();

  explicit hashtags(array<string> &&hashtags_);

  static const std::int32_t ID = 676798885;

  static object_ptr<hashtags> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  httpUrl();

  explicit httpUrl(string const &url_);

  static const std::int32_t ID = -2018019930;

  static object_ptr<httpUrl> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class date;

class datedFile;

class identityDocument final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string number_;
  static jfieldID number_fieldID;
  object_ptr<date> expiration_date_;
  static jfieldID expiration_date_fieldID;
  object_ptr<datedFile> front_side_;
  static jfieldID front_side_fieldID;
  object_ptr<datedFile> reverse_side_;
  static jfieldID reverse_side_fieldID;
  object_ptr<datedFile> selfie_;
  static jfieldID selfie_fieldID;
  array<object_ptr<datedFile>> translation_;
  static jfieldID translation_fieldID;

  identityDocument();

  identityDocument(string const &number_, object_ptr<date> &&expiration_date_, object_ptr<datedFile> &&front_side_, object_ptr<datedFile> &&reverse_side_, object_ptr<datedFile> &&selfie_, array<object_ptr<datedFile>> &&translation_);

  static const std::int32_t ID = 1001703606;

  static object_ptr<identityDocument> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class importedContacts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;
  array<int32> importer_count_;
  static jfieldID importer_count_fieldID;

  importedContacts();

  importedContacts(array<int53> &&user_ids_, array<int32> &&importer_count_);

  static const std::int32_t ID = 2068432290;

  static object_ptr<importedContacts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InlineKeyboardButtonType;

class inlineKeyboardButton final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  object_ptr<InlineKeyboardButtonType> type_;
  static jfieldID type_fieldID;

  inlineKeyboardButton();

  inlineKeyboardButton(string const &text_, object_ptr<InlineKeyboardButtonType> &&type_);

  static const std::int32_t ID = -372105704;

  static object_ptr<inlineKeyboardButton> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TargetChat;

class InlineKeyboardButtonType: public Object {
 public:
  static jclass Class;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeUrl final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  inlineKeyboardButtonTypeUrl();

  explicit inlineKeyboardButtonTypeUrl(string const &url_);

  static const std::int32_t ID = 1130741420;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeLoginUrl final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  int53 id_;
  static jfieldID id_fieldID;
  string forward_text_;
  static jfieldID forward_text_fieldID;

  inlineKeyboardButtonTypeLoginUrl();

  inlineKeyboardButtonTypeLoginUrl(string const &url_, int53 id_, string const &forward_text_);

  static const std::int32_t ID = -1203413081;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeWebApp final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  inlineKeyboardButtonTypeWebApp();

  explicit inlineKeyboardButtonTypeWebApp(string const &url_);

  static const std::int32_t ID = -1767471672;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeCallback final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes data_;
  static jfieldID data_fieldID;

  inlineKeyboardButtonTypeCallback();

  explicit inlineKeyboardButtonTypeCallback(bytes const &data_);

  static const std::int32_t ID = -1127515139;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeCallbackWithPassword final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes data_;
  static jfieldID data_fieldID;

  inlineKeyboardButtonTypeCallbackWithPassword();

  explicit inlineKeyboardButtonTypeCallbackWithPassword(bytes const &data_);

  static const std::int32_t ID = 908018248;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeCallbackGame final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inlineKeyboardButtonTypeCallbackGame();

  static const std::int32_t ID = -383429528;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeSwitchInline final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  object_ptr<TargetChat> target_chat_;
  static jfieldID target_chat_fieldID;

  inlineKeyboardButtonTypeSwitchInline();

  inlineKeyboardButtonTypeSwitchInline(string const &query_, object_ptr<TargetChat> &&target_chat_);

  static const std::int32_t ID = 544906485;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeBuy final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inlineKeyboardButtonTypeBuy();

  static const std::int32_t ID = 1360739440;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeUser final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  inlineKeyboardButtonTypeUser();

  explicit inlineKeyboardButtonTypeUser(int53 user_id_);

  static const std::int32_t ID = 1836574114;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButtonTypeCopyText final : public InlineKeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  inlineKeyboardButtonTypeCopyText();

  explicit inlineKeyboardButtonTypeCopyText(string const &text_);

  static const std::int32_t ID = 68883206;

  static object_ptr<InlineKeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class animation;

class audio;

class contact;

class document;

class game;

class location;

class photo;

class sticker;

class thumbnail;

class venue;

class video;

class voiceNote;

class InlineQueryResult: public Object {
 public:
  static jclass Class;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultArticle final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string url_;
  static jfieldID url_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;

  inlineQueryResultArticle();

  inlineQueryResultArticle(string const &id_, string const &url_, string const &title_, string const &description_, object_ptr<thumbnail> &&thumbnail_);

  static const std::int32_t ID = 269930522;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultContact final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<contact> contact_;
  static jfieldID contact_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;

  inlineQueryResultContact();

  inlineQueryResultContact(string const &id_, object_ptr<contact> &&contact_, object_ptr<thumbnail> &&thumbnail_);

  static const std::int32_t ID = -181960174;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultLocation final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;

  inlineQueryResultLocation();

  inlineQueryResultLocation(string const &id_, object_ptr<location> &&location_, string const &title_, object_ptr<thumbnail> &&thumbnail_);

  static const std::int32_t ID = 466004752;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultVenue final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<venue> venue_;
  static jfieldID venue_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;

  inlineQueryResultVenue();

  inlineQueryResultVenue(string const &id_, object_ptr<venue> &&venue_, object_ptr<thumbnail> &&thumbnail_);

  static const std::int32_t ID = 1281036382;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultGame final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<game> game_;
  static jfieldID game_fieldID;

  inlineQueryResultGame();

  inlineQueryResultGame(string const &id_, object_ptr<game> &&game_);

  static const std::int32_t ID = 1706916987;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultAnimation final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;
  string title_;
  static jfieldID title_fieldID;

  inlineQueryResultAnimation();

  inlineQueryResultAnimation(string const &id_, object_ptr<animation> &&animation_, string const &title_);

  static const std::int32_t ID = 2009984267;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultAudio final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<audio> audio_;
  static jfieldID audio_fieldID;

  inlineQueryResultAudio();

  inlineQueryResultAudio(string const &id_, object_ptr<audio> &&audio_);

  static const std::int32_t ID = 842650360;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultDocument final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;

  inlineQueryResultDocument();

  inlineQueryResultDocument(string const &id_, object_ptr<document> &&document_, string const &title_, string const &description_);

  static const std::int32_t ID = -1491268539;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultPhoto final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;

  inlineQueryResultPhoto();

  inlineQueryResultPhoto(string const &id_, object_ptr<photo> &&photo_, string const &title_, string const &description_);

  static const std::int32_t ID = 1848319440;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultSticker final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  inlineQueryResultSticker();

  inlineQueryResultSticker(string const &id_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -1848224245;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultVideo final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;

  inlineQueryResultVideo();

  inlineQueryResultVideo(string const &id_, object_ptr<video> &&video_, string const &title_, string const &description_);

  static const std::int32_t ID = -1373158683;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultVoiceNote final : public InlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<voiceNote> voice_note_;
  static jfieldID voice_note_fieldID;
  string title_;
  static jfieldID title_fieldID;

  inlineQueryResultVoiceNote();

  inlineQueryResultVoiceNote(string const &id_, object_ptr<voiceNote> &&voice_note_, string const &title_);

  static const std::int32_t ID = -1897393105;

  static object_ptr<InlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InlineQueryResult;

class inlineQueryResultsButton;

class inlineQueryResults final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 inline_query_id_;
  static jfieldID inline_query_id_fieldID;
  object_ptr<inlineQueryResultsButton> button_;
  static jfieldID button_fieldID;
  array<object_ptr<InlineQueryResult>> results_;
  static jfieldID results_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  inlineQueryResults();

  inlineQueryResults(int64 inline_query_id_, object_ptr<inlineQueryResultsButton> &&button_, array<object_ptr<InlineQueryResult>> &&results_, string const &next_offset_);

  static const std::int32_t ID = 1830685615;

  static object_ptr<inlineQueryResults> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InlineQueryResultsButtonType;

class inlineQueryResultsButton final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  object_ptr<InlineQueryResultsButtonType> type_;
  static jfieldID type_fieldID;

  inlineQueryResultsButton();

  inlineQueryResultsButton(string const &text_, object_ptr<InlineQueryResultsButtonType> &&type_);

  static const std::int32_t ID = -790689618;

  static object_ptr<inlineQueryResultsButton> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InlineQueryResultsButtonType: public Object {
 public:
  static jclass Class;

  static object_ptr<InlineQueryResultsButtonType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultsButtonTypeStartBot final : public InlineQueryResultsButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string parameter_;
  static jfieldID parameter_fieldID;

  inlineQueryResultsButtonTypeStartBot();

  explicit inlineQueryResultsButtonTypeStartBot(string const &parameter_);

  static const std::int32_t ID = -23400235;

  static object_ptr<InlineQueryResultsButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResultsButtonTypeWebApp final : public InlineQueryResultsButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  inlineQueryResultsButtonTypeWebApp();

  explicit inlineQueryResultsButtonTypeWebApp(string const &url_);

  static const std::int32_t ID = -1197382814;

  static object_ptr<InlineQueryResultsButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class InputBackground: public Object {
 public:
  static jclass Class;

  static object_ptr<InputBackground> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputBackgroundLocal final : public InputBackground {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> background_;
  static jfieldID background_fieldID;

  inputBackgroundLocal();

  explicit inputBackgroundLocal(object_ptr<InputFile> &&background_);

  static const std::int32_t ID = -1747094364;

  static object_ptr<InputBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputBackgroundRemote final : public InputBackground {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 background_id_;
  static jfieldID background_id_fieldID;

  inputBackgroundRemote();

  explicit inputBackgroundRemote(int64 background_id_);

  static const std::int32_t ID = -274976231;

  static object_ptr<InputBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputBackgroundPrevious final : public InputBackground {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  inputBackgroundPrevious();

  explicit inputBackgroundPrevious(int53 message_id_);

  static const std::int32_t ID = -351905954;

  static object_ptr<InputBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class inputBusinessChatLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string title_;
  static jfieldID title_fieldID;

  inputBusinessChatLink();

  inputBusinessChatLink(object_ptr<formattedText> &&text_, string const &title_);

  static const std::int32_t ID = 237858296;

  static object_ptr<inputBusinessChatLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class inputBusinessStartPage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  string message_;
  static jfieldID message_fieldID;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  inputBusinessStartPage();

  inputBusinessStartPage(string const &title_, string const &message_, object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = -327383072;

  static object_ptr<inputBusinessStartPage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class chatPhotoSticker;

class InputChatPhoto: public Object {
 public:
  static jclass Class;

  static object_ptr<InputChatPhoto> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputChatPhotoPrevious final : public InputChatPhoto {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 chat_photo_id_;
  static jfieldID chat_photo_id_fieldID;

  inputChatPhotoPrevious();

  explicit inputChatPhotoPrevious(int64 chat_photo_id_);

  static const std::int32_t ID = 23128529;

  static object_ptr<InputChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputChatPhotoStatic final : public InputChatPhoto {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> photo_;
  static jfieldID photo_fieldID;

  inputChatPhotoStatic();

  explicit inputChatPhotoStatic(object_ptr<InputFile> &&photo_);

  static const std::int32_t ID = 1979179699;

  static object_ptr<InputChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputChatPhotoAnimation final : public InputChatPhoto {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> animation_;
  static jfieldID animation_fieldID;
  double main_frame_timestamp_;
  static jfieldID main_frame_timestamp_fieldID;

  inputChatPhotoAnimation();

  inputChatPhotoAnimation(object_ptr<InputFile> &&animation_, double main_frame_timestamp_);

  static const std::int32_t ID = 90846242;

  static object_ptr<InputChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputChatPhotoSticker final : public InputChatPhoto {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhotoSticker> sticker_;
  static jfieldID sticker_fieldID;

  inputChatPhotoSticker();

  explicit inputChatPhotoSticker(object_ptr<chatPhotoSticker> &&sticker_);

  static const std::int32_t ID = 1315861341;

  static object_ptr<InputChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputCredentials: public Object {
 public:
  static jclass Class;

  static object_ptr<InputCredentials> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputCredentialsSaved final : public InputCredentials {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string saved_credentials_id_;
  static jfieldID saved_credentials_id_fieldID;

  inputCredentialsSaved();

  explicit inputCredentialsSaved(string const &saved_credentials_id_);

  static const std::int32_t ID = -2034385364;

  static object_ptr<InputCredentials> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputCredentialsNew final : public InputCredentials {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string data_;
  static jfieldID data_fieldID;
  bool allow_save_;
  static jfieldID allow_save_fieldID;

  inputCredentialsNew();

  inputCredentialsNew(string const &data_, bool allow_save_);

  static const std::int32_t ID = -829689558;

  static object_ptr<InputCredentials> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputCredentialsApplePay final : public InputCredentials {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string data_;
  static jfieldID data_fieldID;

  inputCredentialsApplePay();

  explicit inputCredentialsApplePay(string const &data_);

  static const std::int32_t ID = -1246570799;

  static object_ptr<InputCredentials> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputCredentialsGooglePay final : public InputCredentials {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string data_;
  static jfieldID data_fieldID;

  inputCredentialsGooglePay();

  explicit inputCredentialsGooglePay(string const &data_);

  static const std::int32_t ID = 844384100;

  static object_ptr<InputCredentials> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile: public Object {
 public:
  static jclass Class;

  static object_ptr<InputFile> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputFileId final : public InputFile {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;

  inputFileId();

  explicit inputFileId(int32 id_);

  static const std::int32_t ID = 1788906253;

  static object_ptr<InputFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputFileRemote final : public InputFile {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;

  inputFileRemote();

  explicit inputFileRemote(string const &id_);

  static const std::int32_t ID = -107574466;

  static object_ptr<InputFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputFileLocal final : public InputFile {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string path_;
  static jfieldID path_fieldID;

  inputFileLocal();

  explicit inputFileLocal(string const &path_);

  static const std::int32_t ID = 2056030919;

  static object_ptr<InputFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputFileGenerated final : public InputFile {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string original_path_;
  static jfieldID original_path_fieldID;
  string conversion_;
  static jfieldID conversion_fieldID;
  int53 expected_size_;
  static jfieldID expected_size_fieldID;

  inputFileGenerated();

  inputFileGenerated(string const &original_path_, string const &conversion_, int53 expected_size_);

  static const std::int32_t ID = -1333385216;

  static object_ptr<InputFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputGroupCall: public Object {
 public:
  static jclass Class;

  static object_ptr<InputGroupCall> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputGroupCallLink final : public InputGroupCall {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  inputGroupCallLink();

  explicit inputGroupCallLink(string const &link_);

  static const std::int32_t ID = -812157480;

  static object_ptr<InputGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputGroupCallMessage final : public InputGroupCall {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  inputGroupCallMessage();

  inputGroupCallMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -341793768;

  static object_ptr<InputGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class date;

class inputIdentityDocument final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string number_;
  static jfieldID number_fieldID;
  object_ptr<date> expiration_date_;
  static jfieldID expiration_date_fieldID;
  object_ptr<InputFile> front_side_;
  static jfieldID front_side_fieldID;
  object_ptr<InputFile> reverse_side_;
  static jfieldID reverse_side_fieldID;
  object_ptr<InputFile> selfie_;
  static jfieldID selfie_fieldID;
  array<object_ptr<InputFile>> translation_;
  static jfieldID translation_fieldID;

  inputIdentityDocument();

  inputIdentityDocument(string const &number_, object_ptr<date> &&expiration_date_, object_ptr<InputFile> &&front_side_, object_ptr<InputFile> &&reverse_side_, object_ptr<InputFile> &&selfie_, array<object_ptr<InputFile>> &&translation_);

  static const std::int32_t ID = 767353688;

  static object_ptr<inputIdentityDocument> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class contact;

class location;

class venue;

class InputInlineQueryResult: public Object {
 public:
  static jclass Class;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultAnimation final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  string thumbnail_mime_type_;
  static jfieldID thumbnail_mime_type_fieldID;
  string video_url_;
  static jfieldID video_url_fieldID;
  string video_mime_type_;
  static jfieldID video_mime_type_fieldID;
  int32 video_duration_;
  static jfieldID video_duration_fieldID;
  int32 video_width_;
  static jfieldID video_width_fieldID;
  int32 video_height_;
  static jfieldID video_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultAnimation();

  inputInlineQueryResultAnimation(string const &id_, string const &title_, string const &thumbnail_url_, string const &thumbnail_mime_type_, string const &video_url_, string const &video_mime_type_, int32 video_duration_, int32 video_width_, int32 video_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1489808874;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultArticle final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string url_;
  static jfieldID url_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  int32 thumbnail_width_;
  static jfieldID thumbnail_width_fieldID;
  int32 thumbnail_height_;
  static jfieldID thumbnail_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultArticle();

  inputInlineQueryResultArticle(string const &id_, string const &url_, string const &title_, string const &description_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 1983218620;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultAudio final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string performer_;
  static jfieldID performer_fieldID;
  string audio_url_;
  static jfieldID audio_url_fieldID;
  int32 audio_duration_;
  static jfieldID audio_duration_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultAudio();

  inputInlineQueryResultAudio(string const &id_, string const &title_, string const &performer_, string const &audio_url_, int32 audio_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 1260139988;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultContact final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<contact> contact_;
  static jfieldID contact_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  int32 thumbnail_width_;
  static jfieldID thumbnail_width_fieldID;
  int32 thumbnail_height_;
  static jfieldID thumbnail_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultContact();

  inputInlineQueryResultContact(string const &id_, object_ptr<contact> &&contact_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 1846064594;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultDocument final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  string document_url_;
  static jfieldID document_url_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  int32 thumbnail_width_;
  static jfieldID thumbnail_width_fieldID;
  int32 thumbnail_height_;
  static jfieldID thumbnail_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultDocument();

  inputInlineQueryResultDocument(string const &id_, string const &title_, string const &description_, string const &document_url_, string const &mime_type_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 578801869;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultGame final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string game_short_name_;
  static jfieldID game_short_name_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  inputInlineQueryResultGame();

  inputInlineQueryResultGame(string const &id_, string const &game_short_name_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = 966074327;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultLocation final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  int32 thumbnail_width_;
  static jfieldID thumbnail_width_fieldID;
  int32 thumbnail_height_;
  static jfieldID thumbnail_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultLocation();

  inputInlineQueryResultLocation(string const &id_, object_ptr<location> &&location_, int32 live_period_, string const &title_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1887650218;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultPhoto final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  string photo_url_;
  static jfieldID photo_url_fieldID;
  int32 photo_width_;
  static jfieldID photo_width_fieldID;
  int32 photo_height_;
  static jfieldID photo_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultPhoto();

  inputInlineQueryResultPhoto(string const &id_, string const &title_, string const &description_, string const &thumbnail_url_, string const &photo_url_, int32 photo_width_, int32 photo_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1123338721;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultSticker final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  string sticker_url_;
  static jfieldID sticker_url_fieldID;
  int32 sticker_width_;
  static jfieldID sticker_width_fieldID;
  int32 sticker_height_;
  static jfieldID sticker_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultSticker();

  inputInlineQueryResultSticker(string const &id_, string const &thumbnail_url_, string const &sticker_url_, int32 sticker_width_, int32 sticker_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 274007129;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultVenue final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<venue> venue_;
  static jfieldID venue_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  int32 thumbnail_width_;
  static jfieldID thumbnail_width_fieldID;
  int32 thumbnail_height_;
  static jfieldID thumbnail_height_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultVenue();

  inputInlineQueryResultVenue(string const &id_, object_ptr<venue> &&venue_, string const &thumbnail_url_, int32 thumbnail_width_, int32 thumbnail_height_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 541704509;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultVideo final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  string thumbnail_url_;
  static jfieldID thumbnail_url_fieldID;
  string video_url_;
  static jfieldID video_url_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  int32 video_width_;
  static jfieldID video_width_fieldID;
  int32 video_height_;
  static jfieldID video_height_fieldID;
  int32 video_duration_;
  static jfieldID video_duration_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultVideo();

  inputInlineQueryResultVideo(string const &id_, string const &title_, string const &description_, string const &thumbnail_url_, string const &video_url_, string const &mime_type_, int32 video_width_, int32 video_height_, int32 video_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 1724073191;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInlineQueryResultVoiceNote final : public InputInlineQueryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string voice_note_url_;
  static jfieldID voice_note_url_fieldID;
  int32 voice_note_duration_;
  static jfieldID voice_note_duration_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  inputInlineQueryResultVoiceNote();

  inputInlineQueryResultVoiceNote(string const &id_, string const &title_, string const &voice_note_url_, int32 voice_note_duration_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1790072503;

  static object_ptr<InputInlineQueryResult> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TelegramPaymentPurpose;

class InputInvoice: public Object {
 public:
  static jclass Class;

  static object_ptr<InputInvoice> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputInvoiceMessage final : public InputInvoice {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  inputInvoiceMessage();

  inputInvoiceMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1490872848;

  static object_ptr<InputInvoice> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInvoiceName final : public InputInvoice {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  inputInvoiceName();

  explicit inputInvoiceName(string const &name_);

  static const std::int32_t ID = -1312155917;

  static object_ptr<InputInvoice> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputInvoiceTelegram final : public InputInvoice {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<TelegramPaymentPurpose> purpose_;
  static jfieldID purpose_fieldID;

  inputInvoiceTelegram();

  explicit inputInvoiceTelegram(object_ptr<TelegramPaymentPurpose> &&purpose_);

  static const std::int32_t ID = -1762853139;

  static object_ptr<InputInvoice> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class MessageSelfDestructType;

class PollType;

class contact;

class formattedText;

class inputPaidMedia;

class inputThumbnail;

class invoice;

class linkPreviewOptions;

class location;

class messageCopyOptions;

class venue;

class InputMessageContent: public Object {
 public:
  static jclass Class;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageText final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  object_ptr<linkPreviewOptions> link_preview_options_;
  static jfieldID link_preview_options_fieldID;
  bool clear_draft_;
  static jfieldID clear_draft_fieldID;

  inputMessageText();

  inputMessageText(object_ptr<formattedText> &&text_, object_ptr<linkPreviewOptions> &&link_preview_options_, bool clear_draft_);

  static const std::int32_t ID = -212805484;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageAnimation final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> animation_;
  static jfieldID animation_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;

  inputMessageAnimation();

  inputMessageAnimation(object_ptr<InputFile> &&animation_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 duration_, int32 width_, int32 height_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_);

  static const std::int32_t ID = -210404059;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageAudio final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> audio_;
  static jfieldID audio_fieldID;
  object_ptr<inputThumbnail> album_cover_thumbnail_;
  static jfieldID album_cover_thumbnail_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string performer_;
  static jfieldID performer_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  inputMessageAudio();

  inputMessageAudio(object_ptr<InputFile> &&audio_, object_ptr<inputThumbnail> &&album_cover_thumbnail_, int32 duration_, string const &title_, string const &performer_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = -626786126;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageDocument final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> document_;
  static jfieldID document_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  bool disable_content_type_detection_;
  static jfieldID disable_content_type_detection_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  inputMessageDocument();

  inputMessageDocument(object_ptr<InputFile> &&document_, object_ptr<inputThumbnail> &&thumbnail_, bool disable_content_type_detection_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = 1633383097;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessagePaidMedia final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  array<object_ptr<inputPaidMedia>> paid_media_;
  static jfieldID paid_media_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  string payload_;
  static jfieldID payload_fieldID;

  inputMessagePaidMedia();

  inputMessagePaidMedia(int53 star_count_, array<object_ptr<inputPaidMedia>> &&paid_media_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, string const &payload_);

  static const std::int32_t ID = -1274819374;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessagePhoto final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> photo_;
  static jfieldID photo_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  object_ptr<MessageSelfDestructType> self_destruct_type_;
  static jfieldID self_destruct_type_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;

  inputMessagePhoto();

  inputMessagePhoto(object_ptr<InputFile> &&photo_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 width_, int32 height_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, bool has_spoiler_);

  static const std::int32_t ID = -810129442;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageSticker final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;

  inputMessageSticker();

  inputMessageSticker(object_ptr<InputFile> &&sticker_, object_ptr<inputThumbnail> &&thumbnail_, int32 width_, int32 height_, string const &emoji_);

  static const std::int32_t ID = 1072805625;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageVideo final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> video_;
  static jfieldID video_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<InputFile> cover_;
  static jfieldID cover_fieldID;
  int32 start_timestamp_;
  static jfieldID start_timestamp_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  bool supports_streaming_;
  static jfieldID supports_streaming_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  object_ptr<MessageSelfDestructType> self_destruct_type_;
  static jfieldID self_destruct_type_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;

  inputMessageVideo();

  inputMessageVideo(object_ptr<InputFile> &&video_, object_ptr<inputThumbnail> &&thumbnail_, object_ptr<InputFile> &&cover_, int32 start_timestamp_, array<int32> &&added_sticker_file_ids_, int32 duration_, int32 width_, int32 height_, bool supports_streaming_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, bool has_spoiler_);

  static const std::int32_t ID = -605958271;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageVideoNote final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> video_note_;
  static jfieldID video_note_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 length_;
  static jfieldID length_fieldID;
  object_ptr<MessageSelfDestructType> self_destruct_type_;
  static jfieldID self_destruct_type_fieldID;

  inputMessageVideoNote();

  inputMessageVideoNote(object_ptr<InputFile> &&video_note_, object_ptr<inputThumbnail> &&thumbnail_, int32 duration_, int32 length_, object_ptr<MessageSelfDestructType> &&self_destruct_type_);

  static const std::int32_t ID = -714598691;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageVoiceNote final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> voice_note_;
  static jfieldID voice_note_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  bytes waveform_;
  static jfieldID waveform_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  object_ptr<MessageSelfDestructType> self_destruct_type_;
  static jfieldID self_destruct_type_fieldID;

  inputMessageVoiceNote();

  inputMessageVoiceNote(object_ptr<InputFile> &&voice_note_, int32 duration_, bytes const &waveform_, object_ptr<formattedText> &&caption_, object_ptr<MessageSelfDestructType> &&self_destruct_type_);

  static const std::int32_t ID = 1461977004;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageLocation final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  int32 heading_;
  static jfieldID heading_fieldID;
  int32 proximity_alert_radius_;
  static jfieldID proximity_alert_radius_fieldID;

  inputMessageLocation();

  inputMessageLocation(object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_);

  static const std::int32_t ID = 648735088;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageVenue final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<venue> venue_;
  static jfieldID venue_fieldID;

  inputMessageVenue();

  explicit inputMessageVenue(object_ptr<venue> &&venue_);

  static const std::int32_t ID = 1447926269;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageContact final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<contact> contact_;
  static jfieldID contact_fieldID;

  inputMessageContact();

  explicit inputMessageContact(object_ptr<contact> &&contact_);

  static const std::int32_t ID = -982446849;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageDice final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;
  bool clear_draft_;
  static jfieldID clear_draft_fieldID;

  inputMessageDice();

  inputMessageDice(string const &emoji_, bool clear_draft_);

  static const std::int32_t ID = 841574313;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageGame final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string game_short_name_;
  static jfieldID game_short_name_fieldID;

  inputMessageGame();

  inputMessageGame(int53 bot_user_id_, string const &game_short_name_);

  static const std::int32_t ID = 1252944610;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageInvoice final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<invoice> invoice_;
  static jfieldID invoice_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  string photo_url_;
  static jfieldID photo_url_fieldID;
  int32 photo_size_;
  static jfieldID photo_size_fieldID;
  int32 photo_width_;
  static jfieldID photo_width_fieldID;
  int32 photo_height_;
  static jfieldID photo_height_fieldID;
  bytes payload_;
  static jfieldID payload_fieldID;
  string provider_token_;
  static jfieldID provider_token_fieldID;
  string provider_data_;
  static jfieldID provider_data_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  object_ptr<inputPaidMedia> paid_media_;
  static jfieldID paid_media_fieldID;
  object_ptr<formattedText> paid_media_caption_;
  static jfieldID paid_media_caption_fieldID;

  inputMessageInvoice();

  inputMessageInvoice(object_ptr<invoice> &&invoice_, string const &title_, string const &description_, string const &photo_url_, int32 photo_size_, int32 photo_width_, int32 photo_height_, bytes const &payload_, string const &provider_token_, string const &provider_data_, string const &start_parameter_, object_ptr<inputPaidMedia> &&paid_media_, object_ptr<formattedText> &&paid_media_caption_);

  static const std::int32_t ID = -1162047631;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessagePoll final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> question_;
  static jfieldID question_fieldID;
  array<object_ptr<formattedText>> options_;
  static jfieldID options_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;
  object_ptr<PollType> type_;
  static jfieldID type_fieldID;
  int32 open_period_;
  static jfieldID open_period_fieldID;
  int32 close_date_;
  static jfieldID close_date_fieldID;
  bool is_closed_;
  static jfieldID is_closed_fieldID;

  inputMessagePoll();

  inputMessagePoll(object_ptr<formattedText> &&question_, array<object_ptr<formattedText>> &&options_, bool is_anonymous_, object_ptr<PollType> &&type_, int32 open_period_, int32 close_date_, bool is_closed_);

  static const std::int32_t ID = -263337164;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageStory final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  inputMessageStory();

  inputMessageStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -370732053;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageForwarded final : public InputMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 from_chat_id_;
  static jfieldID from_chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool in_game_share_;
  static jfieldID in_game_share_fieldID;
  bool replace_video_start_timestamp_;
  static jfieldID replace_video_start_timestamp_fieldID;
  int32 new_video_start_timestamp_;
  static jfieldID new_video_start_timestamp_fieldID;
  object_ptr<messageCopyOptions> copy_options_;
  static jfieldID copy_options_fieldID;

  inputMessageForwarded();

  inputMessageForwarded(int53 from_chat_id_, int53 message_id_, bool in_game_share_, bool replace_video_start_timestamp_, int32 new_video_start_timestamp_, object_ptr<messageCopyOptions> &&copy_options_);

  static const std::int32_t ID = -1076506316;

  static object_ptr<InputMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputTextQuote;

class InputMessageReplyTo: public Object {
 public:
  static jclass Class;

  static object_ptr<InputMessageReplyTo> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageReplyToMessage final : public InputMessageReplyTo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<inputTextQuote> quote_;
  static jfieldID quote_fieldID;

  inputMessageReplyToMessage();

  inputMessageReplyToMessage(int53 message_id_, object_ptr<inputTextQuote> &&quote_);

  static const std::int32_t ID = -1033987837;

  static object_ptr<InputMessageReplyTo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageReplyToExternalMessage final : public InputMessageReplyTo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<inputTextQuote> quote_;
  static jfieldID quote_fieldID;

  inputMessageReplyToExternalMessage();

  inputMessageReplyToExternalMessage(int53 chat_id_, int53 message_id_, object_ptr<inputTextQuote> &&quote_);

  static const std::int32_t ID = -1993530582;

  static object_ptr<InputMessageReplyTo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputMessageReplyToStory final : public InputMessageReplyTo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  inputMessageReplyToStory();

  inputMessageReplyToStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -1723842320;

  static object_ptr<InputMessageReplyTo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class InputPaidMediaType;

class inputThumbnail;

class inputPaidMedia final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputPaidMediaType> type_;
  static jfieldID type_fieldID;
  object_ptr<InputFile> media_;
  static jfieldID media_fieldID;
  object_ptr<inputThumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  inputPaidMedia();

  inputPaidMedia(object_ptr<InputPaidMediaType> &&type_, object_ptr<InputFile> &&media_, object_ptr<inputThumbnail> &&thumbnail_, array<int32> &&added_sticker_file_ids_, int32 width_, int32 height_);

  static const std::int32_t ID = 475844035;

  static object_ptr<inputPaidMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class InputPaidMediaType: public Object {
 public:
  static jclass Class;

  static object_ptr<InputPaidMediaType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputPaidMediaTypePhoto final : public InputPaidMediaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inputPaidMediaTypePhoto();

  static const std::int32_t ID = -761660134;

  static object_ptr<InputPaidMediaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPaidMediaTypeVideo final : public InputPaidMediaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> cover_;
  static jfieldID cover_fieldID;
  int32 start_timestamp_;
  static jfieldID start_timestamp_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  bool supports_streaming_;
  static jfieldID supports_streaming_fieldID;

  inputPaidMediaTypeVideo();

  inputPaidMediaTypeVideo(object_ptr<InputFile> &&cover_, int32 start_timestamp_, int32 duration_, bool supports_streaming_);

  static const std::int32_t ID = 1793741625;

  static object_ptr<InputPaidMediaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class address;

class inputIdentityDocument;

class inputPersonalDocument;

class personalDetails;

class InputPassportElement: public Object {
 public:
  static jclass Class;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementPersonalDetails final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDetails> personal_details_;
  static jfieldID personal_details_fieldID;

  inputPassportElementPersonalDetails();

  explicit inputPassportElementPersonalDetails(object_ptr<personalDetails> &&personal_details_);

  static const std::int32_t ID = 164791359;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementPassport final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputIdentityDocument> passport_;
  static jfieldID passport_fieldID;

  inputPassportElementPassport();

  explicit inputPassportElementPassport(object_ptr<inputIdentityDocument> &&passport_);

  static const std::int32_t ID = -497011356;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementDriverLicense final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputIdentityDocument> driver_license_;
  static jfieldID driver_license_fieldID;

  inputPassportElementDriverLicense();

  explicit inputPassportElementDriverLicense(object_ptr<inputIdentityDocument> &&driver_license_);

  static const std::int32_t ID = 304813264;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementIdentityCard final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputIdentityDocument> identity_card_;
  static jfieldID identity_card_fieldID;

  inputPassportElementIdentityCard();

  explicit inputPassportElementIdentityCard(object_ptr<inputIdentityDocument> &&identity_card_);

  static const std::int32_t ID = -9963390;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementInternalPassport final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputIdentityDocument> internal_passport_;
  static jfieldID internal_passport_fieldID;

  inputPassportElementInternalPassport();

  explicit inputPassportElementInternalPassport(object_ptr<inputIdentityDocument> &&internal_passport_);

  static const std::int32_t ID = 715360043;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementAddress final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<address> address_;
  static jfieldID address_fieldID;

  inputPassportElementAddress();

  explicit inputPassportElementAddress(object_ptr<address> &&address_);

  static const std::int32_t ID = 461630480;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementUtilityBill final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputPersonalDocument> utility_bill_;
  static jfieldID utility_bill_fieldID;

  inputPassportElementUtilityBill();

  explicit inputPassportElementUtilityBill(object_ptr<inputPersonalDocument> &&utility_bill_);

  static const std::int32_t ID = 1389203841;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementBankStatement final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputPersonalDocument> bank_statement_;
  static jfieldID bank_statement_fieldID;

  inputPassportElementBankStatement();

  explicit inputPassportElementBankStatement(object_ptr<inputPersonalDocument> &&bank_statement_);

  static const std::int32_t ID = -26585208;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementRentalAgreement final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputPersonalDocument> rental_agreement_;
  static jfieldID rental_agreement_fieldID;

  inputPassportElementRentalAgreement();

  explicit inputPassportElementRentalAgreement(object_ptr<inputPersonalDocument> &&rental_agreement_);

  static const std::int32_t ID = 1736154155;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementPassportRegistration final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputPersonalDocument> passport_registration_;
  static jfieldID passport_registration_fieldID;

  inputPassportElementPassportRegistration();

  explicit inputPassportElementPassportRegistration(object_ptr<inputPersonalDocument> &&passport_registration_);

  static const std::int32_t ID = 1314562128;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementTemporaryRegistration final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputPersonalDocument> temporary_registration_;
  static jfieldID temporary_registration_fieldID;

  inputPassportElementTemporaryRegistration();

  explicit inputPassportElementTemporaryRegistration(object_ptr<inputPersonalDocument> &&temporary_registration_);

  static const std::int32_t ID = -1913238047;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementPhoneNumber final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;

  inputPassportElementPhoneNumber();

  explicit inputPassportElementPhoneNumber(string const &phone_number_);

  static const std::int32_t ID = 1319357497;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementEmailAddress final : public InputPassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string email_address_;
  static jfieldID email_address_fieldID;

  inputPassportElementEmailAddress();

  explicit inputPassportElementEmailAddress(string const &email_address_);

  static const std::int32_t ID = -248605659;

  static object_ptr<InputPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputPassportElementErrorSource;

class PassportElementType;

class inputPassportElementError final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;
  string message_;
  static jfieldID message_fieldID;
  object_ptr<InputPassportElementErrorSource> source_;
  static jfieldID source_fieldID;

  inputPassportElementError();

  inputPassportElementError(object_ptr<PassportElementType> &&type_, string const &message_, object_ptr<InputPassportElementErrorSource> &&source_);

  static const std::int32_t ID = 285756898;

  static object_ptr<inputPassportElementError> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputPassportElementErrorSource: public Object {
 public:
  static jclass Class;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceUnspecified final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes element_hash_;
  static jfieldID element_hash_fieldID;

  inputPassportElementErrorSourceUnspecified();

  explicit inputPassportElementErrorSourceUnspecified(bytes const &element_hash_);

  static const std::int32_t ID = 267230319;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceDataField final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string field_name_;
  static jfieldID field_name_fieldID;
  bytes data_hash_;
  static jfieldID data_hash_fieldID;

  inputPassportElementErrorSourceDataField();

  inputPassportElementErrorSourceDataField(string const &field_name_, bytes const &data_hash_);

  static const std::int32_t ID = -426795002;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceFrontSide final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes file_hash_;
  static jfieldID file_hash_fieldID;

  inputPassportElementErrorSourceFrontSide();

  explicit inputPassportElementErrorSourceFrontSide(bytes const &file_hash_);

  static const std::int32_t ID = 588023741;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceReverseSide final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes file_hash_;
  static jfieldID file_hash_fieldID;

  inputPassportElementErrorSourceReverseSide();

  explicit inputPassportElementErrorSourceReverseSide(bytes const &file_hash_);

  static const std::int32_t ID = 413072891;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceSelfie final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes file_hash_;
  static jfieldID file_hash_fieldID;

  inputPassportElementErrorSourceSelfie();

  explicit inputPassportElementErrorSourceSelfie(bytes const &file_hash_);

  static const std::int32_t ID = -773575528;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceTranslationFile final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes file_hash_;
  static jfieldID file_hash_fieldID;

  inputPassportElementErrorSourceTranslationFile();

  explicit inputPassportElementErrorSourceTranslationFile(bytes const &file_hash_);

  static const std::int32_t ID = 505842299;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceTranslationFiles final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<bytes> file_hashes_;
  static jfieldID file_hashes_fieldID;

  inputPassportElementErrorSourceTranslationFiles();

  explicit inputPassportElementErrorSourceTranslationFiles(array<bytes> &&file_hashes_);

  static const std::int32_t ID = -527254048;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceFile final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes file_hash_;
  static jfieldID file_hash_fieldID;

  inputPassportElementErrorSourceFile();

  explicit inputPassportElementErrorSourceFile(bytes const &file_hash_);

  static const std::int32_t ID = -298492469;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementErrorSourceFiles final : public InputPassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<bytes> file_hashes_;
  static jfieldID file_hashes_fieldID;

  inputPassportElementErrorSourceFiles();

  explicit inputPassportElementErrorSourceFiles(array<bytes> &&file_hashes_);

  static const std::int32_t ID = -2008541640;

  static object_ptr<InputPassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class inputPersonalDocument final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<InputFile>> files_;
  static jfieldID files_fieldID;
  array<object_ptr<InputFile>> translation_;
  static jfieldID translation_fieldID;

  inputPersonalDocument();

  inputPersonalDocument(array<object_ptr<InputFile>> &&files_, array<object_ptr<InputFile>> &&translation_);

  static const std::int32_t ID = 1676966826;

  static object_ptr<inputPersonalDocument> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class StickerFormat;

class maskPosition;

class inputSticker final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  object_ptr<StickerFormat> format_;
  static jfieldID format_fieldID;
  string emojis_;
  static jfieldID emojis_fieldID;
  object_ptr<maskPosition> mask_position_;
  static jfieldID mask_position_fieldID;
  array<string> keywords_;
  static jfieldID keywords_fieldID;

  inputSticker();

  inputSticker(object_ptr<InputFile> &&sticker_, object_ptr<StickerFormat> &&format_, string const &emojis_, object_ptr<maskPosition> &&mask_position_, array<string> &&keywords_);

  static const std::int32_t ID = 1589392402;

  static object_ptr<inputSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryAreaType;

class storyAreaPosition;

class inputStoryArea final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<storyAreaPosition> position_;
  static jfieldID position_fieldID;
  object_ptr<InputStoryAreaType> type_;
  static jfieldID type_fieldID;

  inputStoryArea();

  inputStoryArea(object_ptr<storyAreaPosition> &&position_, object_ptr<InputStoryAreaType> &&type_);

  static const std::int32_t ID = 122859135;

  static object_ptr<inputStoryArea> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class location;

class locationAddress;

class InputStoryAreaType: public Object {
 public:
  static jclass Class;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeLocation final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  object_ptr<locationAddress> address_;
  static jfieldID address_fieldID;

  inputStoryAreaTypeLocation();

  inputStoryAreaTypeLocation(object_ptr<location> &&location_, object_ptr<locationAddress> &&address_);

  static const std::int32_t ID = -1433714887;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeFoundVenue final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 query_id_;
  static jfieldID query_id_fieldID;
  string result_id_;
  static jfieldID result_id_fieldID;

  inputStoryAreaTypeFoundVenue();

  inputStoryAreaTypeFoundVenue(int64 query_id_, string const &result_id_);

  static const std::int32_t ID = -1395809130;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypePreviousVenue final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string venue_provider_;
  static jfieldID venue_provider_fieldID;
  string venue_id_;
  static jfieldID venue_id_fieldID;

  inputStoryAreaTypePreviousVenue();

  inputStoryAreaTypePreviousVenue(string const &venue_provider_, string const &venue_id_);

  static const std::int32_t ID = 1846693388;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeSuggestedReaction final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;
  bool is_flipped_;
  static jfieldID is_flipped_fieldID;

  inputStoryAreaTypeSuggestedReaction();

  inputStoryAreaTypeSuggestedReaction(object_ptr<ReactionType> &&reaction_type_, bool is_dark_, bool is_flipped_);

  static const std::int32_t ID = 2101826003;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeMessage final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  inputStoryAreaTypeMessage();

  inputStoryAreaTypeMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -266607529;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeLink final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  inputStoryAreaTypeLink();

  explicit inputStoryAreaTypeLink(string const &url_);

  static const std::int32_t ID = 1408441160;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeWeather final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double temperature_;
  static jfieldID temperature_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  int32 background_color_;
  static jfieldID background_color_fieldID;

  inputStoryAreaTypeWeather();

  inputStoryAreaTypeWeather(double temperature_, string const &emoji_, int32 background_color_);

  static const std::int32_t ID = -1212686691;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryAreaTypeUpgradedGift final : public InputStoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string gift_name_;
  static jfieldID gift_name_fieldID;

  inputStoryAreaTypeUpgradedGift();

  explicit inputStoryAreaTypeUpgradedGift(string const &gift_name_);

  static const std::int32_t ID = 793059694;

  static object_ptr<InputStoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryArea;

class inputStoryAreas final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<inputStoryArea>> areas_;
  static jfieldID areas_fieldID;

  inputStoryAreas();

  explicit inputStoryAreas(array<object_ptr<inputStoryArea>> &&areas_);

  static const std::int32_t ID = -883247088;

  static object_ptr<inputStoryAreas> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class InputStoryContent: public Object {
 public:
  static jclass Class;

  static object_ptr<InputStoryContent> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryContentPhoto final : public InputStoryContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> photo_;
  static jfieldID photo_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;

  inputStoryContentPhoto();

  inputStoryContentPhoto(object_ptr<InputFile> &&photo_, array<int32> &&added_sticker_file_ids_);

  static const std::int32_t ID = -309196727;

  static object_ptr<InputStoryContent> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inputStoryContentVideo final : public InputStoryContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> video_;
  static jfieldID video_fieldID;
  array<int32> added_sticker_file_ids_;
  static jfieldID added_sticker_file_ids_fieldID;
  double duration_;
  static jfieldID duration_fieldID;
  double cover_frame_timestamp_;
  static jfieldID cover_frame_timestamp_fieldID;
  bool is_animation_;
  static jfieldID is_animation_fieldID;

  inputStoryContentVideo();

  inputStoryContentVideo(object_ptr<InputFile> &&video_, array<int32> &&added_sticker_file_ids_, double duration_, double cover_frame_timestamp_, bool is_animation_);

  static const std::int32_t ID = 3809243;

  static object_ptr<InputStoryContent> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class inputTextQuote final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int32 position_;
  static jfieldID position_fieldID;

  inputTextQuote();

  inputTextQuote(object_ptr<formattedText> &&text_, int32 position_);

  static const std::int32_t ID = -1219859172;

  static object_ptr<inputTextQuote> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class inputThumbnail final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  inputThumbnail();

  inputThumbnail(object_ptr<InputFile> &&thumbnail_, int32 width_, int32 height_);

  static const std::int32_t ID = 1582387236;

  static object_ptr<inputThumbnail> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType;

class TargetChat;

class WebAppOpenMode;

class chatAdministratorRights;

class formattedText;

class InternalLinkType: public Object {
 public:
  static jclass Class;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeActiveSessions final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeActiveSessions();

  static const std::int32_t ID = 1886108589;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeAttachmentMenuBot final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<TargetChat> target_chat_;
  static jfieldID target_chat_fieldID;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string url_;
  static jfieldID url_fieldID;

  internalLinkTypeAttachmentMenuBot();

  internalLinkTypeAttachmentMenuBot(object_ptr<TargetChat> &&target_chat_, string const &bot_username_, string const &url_);

  static const std::int32_t ID = 1682719269;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeAuthenticationCode final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  internalLinkTypeAuthenticationCode();

  explicit internalLinkTypeAuthenticationCode(string const &code_);

  static const std::int32_t ID = -209235982;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBackground final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string background_name_;
  static jfieldID background_name_fieldID;

  internalLinkTypeBackground();

  explicit internalLinkTypeBackground(string const &background_name_);

  static const std::int32_t ID = 185411848;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBotAddToChannel final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  object_ptr<chatAdministratorRights> administrator_rights_;
  static jfieldID administrator_rights_fieldID;

  internalLinkTypeBotAddToChannel();

  internalLinkTypeBotAddToChannel(string const &bot_username_, object_ptr<chatAdministratorRights> &&administrator_rights_);

  static const std::int32_t ID = 1401602752;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBotStart final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  bool autostart_;
  static jfieldID autostart_fieldID;

  internalLinkTypeBotStart();

  internalLinkTypeBotStart(string const &bot_username_, string const &start_parameter_, bool autostart_);

  static const std::int32_t ID = 1066950637;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBotStartInGroup final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  object_ptr<chatAdministratorRights> administrator_rights_;
  static jfieldID administrator_rights_fieldID;

  internalLinkTypeBotStartInGroup();

  internalLinkTypeBotStartInGroup(string const &bot_username_, string const &start_parameter_, object_ptr<chatAdministratorRights> &&administrator_rights_);

  static const std::int32_t ID = -905081650;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBusinessChat final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_name_;
  static jfieldID link_name_fieldID;

  internalLinkTypeBusinessChat();

  explicit internalLinkTypeBusinessChat(string const &link_name_);

  static const std::int32_t ID = -1606751785;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeBuyStars final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string purpose_;
  static jfieldID purpose_fieldID;

  internalLinkTypeBuyStars();

  internalLinkTypeBuyStars(int53 star_count_, string const &purpose_);

  static const std::int32_t ID = -1454587065;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChangePhoneNumber final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeChangePhoneNumber();

  static const std::int32_t ID = -265856255;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChatAffiliateProgram final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  string referrer_;
  static jfieldID referrer_fieldID;

  internalLinkTypeChatAffiliateProgram();

  internalLinkTypeChatAffiliateProgram(string const &username_, string const &referrer_);

  static const std::int32_t ID = 632049700;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChatBoost final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  internalLinkTypeChatBoost();

  explicit internalLinkTypeChatBoost(string const &url_);

  static const std::int32_t ID = -716571328;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChatFolderInvite final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  internalLinkTypeChatFolderInvite();

  explicit internalLinkTypeChatFolderInvite(string const &invite_link_);

  static const std::int32_t ID = -1984804546;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChatFolderSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeChatFolderSettings();

  static const std::int32_t ID = -1073805988;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeChatInvite final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  internalLinkTypeChatInvite();

  explicit internalLinkTypeChatInvite(string const &invite_link_);

  static const std::int32_t ID = 428621017;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeDefaultMessageAutoDeleteTimerSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeDefaultMessageAutoDeleteTimerSettings();

  static const std::int32_t ID = 732625201;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeEditProfileSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeEditProfileSettings();

  static const std::int32_t ID = -1022472090;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeGame final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string game_short_name_;
  static jfieldID game_short_name_fieldID;

  internalLinkTypeGame();

  internalLinkTypeGame(string const &bot_username_, string const &game_short_name_);

  static const std::int32_t ID = -260788787;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeGroupCall final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  internalLinkTypeGroupCall();

  explicit internalLinkTypeGroupCall(string const &invite_link_);

  static const std::int32_t ID = 1953084438;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeInstantView final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string fallback_url_;
  static jfieldID fallback_url_fieldID;

  internalLinkTypeInstantView();

  internalLinkTypeInstantView(string const &url_, string const &fallback_url_);

  static const std::int32_t ID = 1776607039;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeInvoice final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invoice_name_;
  static jfieldID invoice_name_fieldID;

  internalLinkTypeInvoice();

  explicit internalLinkTypeInvoice(string const &invoice_name_);

  static const std::int32_t ID = -213094996;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeLanguagePack final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;

  internalLinkTypeLanguagePack();

  explicit internalLinkTypeLanguagePack(string const &language_pack_id_);

  static const std::int32_t ID = -1450766996;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeLanguageSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeLanguageSettings();

  static const std::int32_t ID = -1340479770;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeMainWebApp final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  object_ptr<WebAppOpenMode> mode_;
  static jfieldID mode_fieldID;

  internalLinkTypeMainWebApp();

  internalLinkTypeMainWebApp(string const &bot_username_, string const &start_parameter_, object_ptr<WebAppOpenMode> &&mode_);

  static const std::int32_t ID = 1574925033;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeMessage final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  internalLinkTypeMessage();

  explicit internalLinkTypeMessage(string const &url_);

  static const std::int32_t ID = 978541650;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeMessageDraft final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool contains_link_;
  static jfieldID contains_link_fieldID;

  internalLinkTypeMessageDraft();

  internalLinkTypeMessageDraft(object_ptr<formattedText> &&text_, bool contains_link_);

  static const std::int32_t ID = 661633749;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeMyStars final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeMyStars();

  static const std::int32_t ID = 1613887070;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePassportDataRequest final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string scope_;
  static jfieldID scope_fieldID;
  string public_key_;
  static jfieldID public_key_fieldID;
  string nonce_;
  static jfieldID nonce_fieldID;
  string callback_url_;
  static jfieldID callback_url_fieldID;

  internalLinkTypePassportDataRequest();

  internalLinkTypePassportDataRequest(int53 bot_user_id_, string const &scope_, string const &public_key_, string const &nonce_, string const &callback_url_);

  static const std::int32_t ID = -988819839;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePhoneNumberConfirmation final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string hash_;
  static jfieldID hash_fieldID;
  string phone_number_;
  static jfieldID phone_number_fieldID;

  internalLinkTypePhoneNumberConfirmation();

  internalLinkTypePhoneNumberConfirmation(string const &hash_, string const &phone_number_);

  static const std::int32_t ID = 1757375254;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePremiumFeatures final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string referrer_;
  static jfieldID referrer_fieldID;

  internalLinkTypePremiumFeatures();

  explicit internalLinkTypePremiumFeatures(string const &referrer_);

  static const std::int32_t ID = 1216892745;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePremiumGift final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string referrer_;
  static jfieldID referrer_fieldID;

  internalLinkTypePremiumGift();

  explicit internalLinkTypePremiumGift(string const &referrer_);

  static const std::int32_t ID = 1523936577;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePremiumGiftCode final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  internalLinkTypePremiumGiftCode();

  explicit internalLinkTypePremiumGiftCode(string const &code_);

  static const std::int32_t ID = -564356974;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePrivacyAndSecuritySettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypePrivacyAndSecuritySettings();

  static const std::int32_t ID = -1386255665;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeProxy final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string server_;
  static jfieldID server_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  object_ptr<ProxyType> type_;
  static jfieldID type_fieldID;

  internalLinkTypeProxy();

  internalLinkTypeProxy(string const &server_, int32 port_, object_ptr<ProxyType> &&type_);

  static const std::int32_t ID = -1313788694;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypePublicChat final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string chat_username_;
  static jfieldID chat_username_fieldID;
  string draft_text_;
  static jfieldID draft_text_fieldID;
  bool open_profile_;
  static jfieldID open_profile_fieldID;

  internalLinkTypePublicChat();

  internalLinkTypePublicChat(string const &chat_username_, string const &draft_text_, bool open_profile_);

  static const std::int32_t ID = 1769614592;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeQrCodeAuthentication final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeQrCodeAuthentication();

  static const std::int32_t ID = -1089332956;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeRestorePurchases final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeRestorePurchases();

  static const std::int32_t ID = 606090371;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeSettings();

  static const std::int32_t ID = 393561524;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeStickerSet final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string sticker_set_name_;
  static jfieldID sticker_set_name_fieldID;
  bool expect_custom_emoji_;
  static jfieldID expect_custom_emoji_fieldID;

  internalLinkTypeStickerSet();

  internalLinkTypeStickerSet(string const &sticker_set_name_, bool expect_custom_emoji_);

  static const std::int32_t ID = -1589227614;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeStory final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string story_poster_username_;
  static jfieldID story_poster_username_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  internalLinkTypeStory();

  internalLinkTypeStory(string const &story_poster_username_, int32 story_id_);

  static const std::int32_t ID = 1852042869;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeTheme final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  internalLinkTypeTheme();

  explicit internalLinkTypeTheme(string const &theme_name_);

  static const std::int32_t ID = -200935417;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeThemeSettings final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeThemeSettings();

  static const std::int32_t ID = -1051903722;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeUnknownDeepLink final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  internalLinkTypeUnknownDeepLink();

  explicit internalLinkTypeUnknownDeepLink(string const &link_);

  static const std::int32_t ID = 625596379;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeUnsupportedProxy final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  internalLinkTypeUnsupportedProxy();

  static const std::int32_t ID = -566649079;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeUpgradedGift final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  internalLinkTypeUpgradedGift();

  explicit internalLinkTypeUpgradedGift(string const &name_);

  static const std::int32_t ID = -708405605;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeUserPhoneNumber final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  string draft_text_;
  static jfieldID draft_text_fieldID;
  bool open_profile_;
  static jfieldID open_profile_fieldID;

  internalLinkTypeUserPhoneNumber();

  internalLinkTypeUserPhoneNumber(string const &phone_number_, string const &draft_text_, bool open_profile_);

  static const std::int32_t ID = 273398536;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeUserToken final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  internalLinkTypeUserToken();

  explicit internalLinkTypeUserToken(string const &token_);

  static const std::int32_t ID = -1462248615;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeVideoChat final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string chat_username_;
  static jfieldID chat_username_fieldID;
  string invite_hash_;
  static jfieldID invite_hash_fieldID;
  bool is_live_stream_;
  static jfieldID is_live_stream_fieldID;

  internalLinkTypeVideoChat();

  internalLinkTypeVideoChat(string const &chat_username_, string const &invite_hash_, bool is_live_stream_);

  static const std::int32_t ID = -2020149068;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class internalLinkTypeWebApp final : public InternalLinkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bot_username_;
  static jfieldID bot_username_fieldID;
  string web_app_short_name_;
  static jfieldID web_app_short_name_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  object_ptr<WebAppOpenMode> mode_;
  static jfieldID mode_fieldID;

  internalLinkTypeWebApp();

  internalLinkTypeWebApp(string const &bot_username_, string const &web_app_short_name_, string const &start_parameter_, object_ptr<WebAppOpenMode> &&mode_);

  static const std::int32_t ID = 2062112045;

  static object_ptr<InternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InviteGroupCallParticipantResult: public Object {
 public:
  static jclass Class;

  static object_ptr<InviteGroupCallParticipantResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inviteGroupCallParticipantResultUserPrivacyRestricted final : public InviteGroupCallParticipantResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteGroupCallParticipantResultUserPrivacyRestricted();

  static const std::int32_t ID = 53003769;

  static object_ptr<InviteGroupCallParticipantResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inviteGroupCallParticipantResultUserAlreadyParticipant final : public InviteGroupCallParticipantResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteGroupCallParticipantResultUserAlreadyParticipant();

  static const std::int32_t ID = 661526151;

  static object_ptr<InviteGroupCallParticipantResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inviteGroupCallParticipantResultUserWasBanned final : public InviteGroupCallParticipantResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteGroupCallParticipantResultUserWasBanned();

  static const std::int32_t ID = -204345357;

  static object_ptr<InviteGroupCallParticipantResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inviteGroupCallParticipantResultSuccess final : public InviteGroupCallParticipantResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  inviteGroupCallParticipantResultSuccess();

  inviteGroupCallParticipantResultSuccess(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1914309427;

  static object_ptr<InviteGroupCallParticipantResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InviteLinkChatType: public Object {
 public:
  static jclass Class;

  static object_ptr<InviteLinkChatType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inviteLinkChatTypeBasicGroup final : public InviteLinkChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteLinkChatTypeBasicGroup();

  static const std::int32_t ID = 1296287214;

  static object_ptr<InviteLinkChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inviteLinkChatTypeSupergroup final : public InviteLinkChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteLinkChatTypeSupergroup();

  static const std::int32_t ID = 1038640984;

  static object_ptr<InviteLinkChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inviteLinkChatTypeChannel final : public InviteLinkChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  inviteLinkChatTypeChannel();

  static const std::int32_t ID = 806547211;

  static object_ptr<InviteLinkChatType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class labeledPricePart;

class invoice final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  array<object_ptr<labeledPricePart>> price_parts_;
  static jfieldID price_parts_fieldID;
  int32 subscription_period_;
  static jfieldID subscription_period_fieldID;
  int53 max_tip_amount_;
  static jfieldID max_tip_amount_fieldID;
  array<int53> suggested_tip_amounts_;
  static jfieldID suggested_tip_amounts_fieldID;
  string recurring_payment_terms_of_service_url_;
  static jfieldID recurring_payment_terms_of_service_url_fieldID;
  string terms_of_service_url_;
  static jfieldID terms_of_service_url_fieldID;
  bool is_test_;
  static jfieldID is_test_fieldID;
  bool need_name_;
  static jfieldID need_name_fieldID;
  bool need_phone_number_;
  static jfieldID need_phone_number_fieldID;
  bool need_email_address_;
  static jfieldID need_email_address_fieldID;
  bool need_shipping_address_;
  static jfieldID need_shipping_address_fieldID;
  bool send_phone_number_to_provider_;
  static jfieldID send_phone_number_to_provider_fieldID;
  bool send_email_address_to_provider_;
  static jfieldID send_email_address_to_provider_fieldID;
  bool is_flexible_;
  static jfieldID is_flexible_fieldID;

  invoice();

  invoice(string const &currency_, array<object_ptr<labeledPricePart>> &&price_parts_, int32 subscription_period_, int53 max_tip_amount_, array<int53> &&suggested_tip_amounts_, string const &recurring_payment_terms_of_service_url_, string const &terms_of_service_url_, bool is_test_, bool need_name_, bool need_phone_number_, bool need_email_address_, bool need_shipping_address_, bool send_phone_number_to_provider_, bool send_email_address_to_provider_, bool is_flexible_);

  static const std::int32_t ID = 113204876;

  static object_ptr<invoice> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class jsonObjectMember final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string key_;
  static jfieldID key_fieldID;
  object_ptr<JsonValue> value_;
  static jfieldID value_fieldID;

  jsonObjectMember();

  jsonObjectMember(string const &key_, object_ptr<JsonValue> &&value_);

  static const std::int32_t ID = -1803309418;

  static object_ptr<jsonObjectMember> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class jsonObjectMember;

class JsonValue: public Object {
 public:
  static jclass Class;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueNull final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  jsonValueNull();

  static const std::int32_t ID = -92872499;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueBoolean final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool value_;
  static jfieldID value_fieldID;

  jsonValueBoolean();

  explicit jsonValueBoolean(bool value_);

  static const std::int32_t ID = -2142186576;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueNumber final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double value_;
  static jfieldID value_fieldID;

  jsonValueNumber();

  explicit jsonValueNumber(double value_);

  static const std::int32_t ID = -1010822033;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueString final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string value_;
  static jfieldID value_fieldID;

  jsonValueString();

  explicit jsonValueString(string const &value_);

  static const std::int32_t ID = 1597947313;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueArray final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<JsonValue>> values_;
  static jfieldID values_fieldID;

  jsonValueArray();

  explicit jsonValueArray(array<object_ptr<JsonValue>> &&values_);

  static const std::int32_t ID = -183913546;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class jsonValueObject final : public JsonValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<jsonObjectMember>> members_;
  static jfieldID members_fieldID;

  jsonValueObject();

  explicit jsonValueObject(array<object_ptr<jsonObjectMember>> &&members_);

  static const std::int32_t ID = 520252026;

  static object_ptr<JsonValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class KeyboardButtonType;

class keyboardButton final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  object_ptr<KeyboardButtonType> type_;
  static jfieldID type_fieldID;

  keyboardButton();

  keyboardButton(string const &text_, object_ptr<KeyboardButtonType> &&type_);

  static const std::int32_t ID = -2069836172;

  static object_ptr<keyboardButton> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministratorRights;

class KeyboardButtonType: public Object {
 public:
  static jclass Class;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeText final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  keyboardButtonTypeText();

  static const std::int32_t ID = -1773037256;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeRequestPhoneNumber final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  keyboardButtonTypeRequestPhoneNumber();

  static const std::int32_t ID = -1529235527;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeRequestLocation final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  keyboardButtonTypeRequestLocation();

  static const std::int32_t ID = -125661955;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeRequestPoll final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool force_regular_;
  static jfieldID force_regular_fieldID;
  bool force_quiz_;
  static jfieldID force_quiz_fieldID;

  keyboardButtonTypeRequestPoll();

  keyboardButtonTypeRequestPoll(bool force_regular_, bool force_quiz_);

  static const std::int32_t ID = 1902435512;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeRequestUsers final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  bool restrict_user_is_bot_;
  static jfieldID restrict_user_is_bot_fieldID;
  bool user_is_bot_;
  static jfieldID user_is_bot_fieldID;
  bool restrict_user_is_premium_;
  static jfieldID restrict_user_is_premium_fieldID;
  bool user_is_premium_;
  static jfieldID user_is_premium_fieldID;
  int32 max_quantity_;
  static jfieldID max_quantity_fieldID;
  bool request_name_;
  static jfieldID request_name_fieldID;
  bool request_username_;
  static jfieldID request_username_fieldID;
  bool request_photo_;
  static jfieldID request_photo_fieldID;

  keyboardButtonTypeRequestUsers();

  keyboardButtonTypeRequestUsers(int32 id_, bool restrict_user_is_bot_, bool user_is_bot_, bool restrict_user_is_premium_, bool user_is_premium_, int32 max_quantity_, bool request_name_, bool request_username_, bool request_photo_);

  static const std::int32_t ID = -1738765315;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeRequestChat final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  bool chat_is_channel_;
  static jfieldID chat_is_channel_fieldID;
  bool restrict_chat_is_forum_;
  static jfieldID restrict_chat_is_forum_fieldID;
  bool chat_is_forum_;
  static jfieldID chat_is_forum_fieldID;
  bool restrict_chat_has_username_;
  static jfieldID restrict_chat_has_username_fieldID;
  bool chat_has_username_;
  static jfieldID chat_has_username_fieldID;
  bool chat_is_created_;
  static jfieldID chat_is_created_fieldID;
  object_ptr<chatAdministratorRights> user_administrator_rights_;
  static jfieldID user_administrator_rights_fieldID;
  object_ptr<chatAdministratorRights> bot_administrator_rights_;
  static jfieldID bot_administrator_rights_fieldID;
  bool bot_is_member_;
  static jfieldID bot_is_member_fieldID;
  bool request_title_;
  static jfieldID request_title_fieldID;
  bool request_username_;
  static jfieldID request_username_fieldID;
  bool request_photo_;
  static jfieldID request_photo_fieldID;

  keyboardButtonTypeRequestChat();

  keyboardButtonTypeRequestChat(int32 id_, bool chat_is_channel_, bool restrict_chat_is_forum_, bool chat_is_forum_, bool restrict_chat_has_username_, bool chat_has_username_, bool chat_is_created_, object_ptr<chatAdministratorRights> &&user_administrator_rights_, object_ptr<chatAdministratorRights> &&bot_administrator_rights_, bool bot_is_member_, bool request_title_, bool request_username_, bool request_photo_);

  static const std::int32_t ID = 1511138485;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class keyboardButtonTypeWebApp final : public KeyboardButtonType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  keyboardButtonTypeWebApp();

  explicit keyboardButtonTypeWebApp(string const &url_);

  static const std::int32_t ID = 1892220770;

  static object_ptr<KeyboardButtonType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class labeledPricePart final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string label_;
  static jfieldID label_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;

  labeledPricePart();

  labeledPricePart(string const &label_, int53 amount_);

  static const std::int32_t ID = 552789798;

  static object_ptr<labeledPricePart> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class languagePackInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string base_language_pack_id_;
  static jfieldID base_language_pack_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  string native_name_;
  static jfieldID native_name_fieldID;
  string plural_code_;
  static jfieldID plural_code_fieldID;
  bool is_official_;
  static jfieldID is_official_fieldID;
  bool is_rtl_;
  static jfieldID is_rtl_fieldID;
  bool is_beta_;
  static jfieldID is_beta_fieldID;
  bool is_installed_;
  static jfieldID is_installed_fieldID;
  int32 total_string_count_;
  static jfieldID total_string_count_fieldID;
  int32 translated_string_count_;
  static jfieldID translated_string_count_fieldID;
  int32 local_string_count_;
  static jfieldID local_string_count_fieldID;
  string translation_url_;
  static jfieldID translation_url_fieldID;

  languagePackInfo();

  languagePackInfo(string const &id_, string const &base_language_pack_id_, string const &name_, string const &native_name_, string const &plural_code_, bool is_official_, bool is_rtl_, bool is_beta_, bool is_installed_, int32 total_string_count_, int32 translated_string_count_, int32 local_string_count_, string const &translation_url_);

  static const std::int32_t ID = 542199642;

  static object_ptr<languagePackInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class LanguagePackStringValue;

class languagePackString final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string key_;
  static jfieldID key_fieldID;
  object_ptr<LanguagePackStringValue> value_;
  static jfieldID value_fieldID;

  languagePackString();

  languagePackString(string const &key_, object_ptr<LanguagePackStringValue> &&value_);

  static const std::int32_t ID = 1307632736;

  static object_ptr<languagePackString> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class LanguagePackStringValue: public Object {
 public:
  static jclass Class;

  static object_ptr<LanguagePackStringValue> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackStringValueOrdinary final : public LanguagePackStringValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string value_;
  static jfieldID value_fieldID;

  languagePackStringValueOrdinary();

  explicit languagePackStringValueOrdinary(string const &value_);

  static const std::int32_t ID = -249256352;

  static object_ptr<LanguagePackStringValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class languagePackStringValuePluralized final : public LanguagePackStringValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string zero_value_;
  static jfieldID zero_value_fieldID;
  string one_value_;
  static jfieldID one_value_fieldID;
  string two_value_;
  static jfieldID two_value_fieldID;
  string few_value_;
  static jfieldID few_value_fieldID;
  string many_value_;
  static jfieldID many_value_fieldID;
  string other_value_;
  static jfieldID other_value_fieldID;

  languagePackStringValuePluralized();

  languagePackStringValuePluralized(string const &zero_value_, string const &one_value_, string const &two_value_, string const &few_value_, string const &many_value_, string const &other_value_);

  static const std::int32_t ID = 1906840261;

  static object_ptr<LanguagePackStringValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class languagePackStringValueDeleted final : public LanguagePackStringValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  languagePackStringValueDeleted();

  static const std::int32_t ID = 1834792698;

  static object_ptr<LanguagePackStringValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class languagePackString;

class languagePackStrings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<languagePackString>> strings_;
  static jfieldID strings_fieldID;

  languagePackStrings();

  explicit languagePackStrings(array<object_ptr<languagePackString>> &&strings_);

  static const std::int32_t ID = 1172082922;

  static object_ptr<languagePackStrings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class LinkPreviewType;

class formattedText;

class linkPreview final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string display_url_;
  static jfieldID display_url_fieldID;
  string site_name_;
  static jfieldID site_name_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<formattedText> description_;
  static jfieldID description_fieldID;
  string author_;
  static jfieldID author_fieldID;
  object_ptr<LinkPreviewType> type_;
  static jfieldID type_fieldID;
  bool has_large_media_;
  static jfieldID has_large_media_fieldID;
  bool show_large_media_;
  static jfieldID show_large_media_fieldID;
  bool show_media_above_description_;
  static jfieldID show_media_above_description_fieldID;
  bool skip_confirmation_;
  static jfieldID skip_confirmation_fieldID;
  bool show_above_text_;
  static jfieldID show_above_text_fieldID;
  int32 instant_view_version_;
  static jfieldID instant_view_version_fieldID;

  linkPreview();

  linkPreview(string const &url_, string const &display_url_, string const &site_name_, string const &title_, object_ptr<formattedText> &&description_, string const &author_, object_ptr<LinkPreviewType> &&type_, bool has_large_media_, bool show_large_media_, bool show_media_above_description_, bool skip_confirmation_, bool show_above_text_, int32 instant_view_version_);

  static const std::int32_t ID = 1729417714;

  static object_ptr<linkPreview> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class video;

class LinkPreviewAlbumMedia: public Object {
 public:
  static jclass Class;

  static object_ptr<LinkPreviewAlbumMedia> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewAlbumMediaPhoto final : public LinkPreviewAlbumMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  linkPreviewAlbumMediaPhoto();

  explicit linkPreviewAlbumMediaPhoto(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -935480434;

  static object_ptr<LinkPreviewAlbumMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewAlbumMediaVideo final : public LinkPreviewAlbumMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;

  linkPreviewAlbumMediaVideo();

  explicit linkPreviewAlbumMediaVideo(object_ptr<video> &&video_);

  static const std::int32_t ID = 390616795;

  static object_ptr<LinkPreviewAlbumMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_disabled_;
  static jfieldID is_disabled_fieldID;
  string url_;
  static jfieldID url_fieldID;
  bool force_small_media_;
  static jfieldID force_small_media_fieldID;
  bool force_large_media_;
  static jfieldID force_large_media_fieldID;
  bool show_above_text_;
  static jfieldID show_above_text_fieldID;

  linkPreviewOptions();

  linkPreviewOptions(bool is_disabled_, string const &url_, bool force_small_media_, bool force_large_media_, bool show_above_text_);

  static const std::int32_t ID = 1046590451;

  static object_ptr<linkPreviewOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundType;

class InviteLinkChatType;

class LinkPreviewAlbumMedia;

class animation;

class audio;

class chatPhoto;

class document;

class photo;

class sticker;

class themeSettings;

class upgradedGift;

class video;

class videoNote;

class voiceNote;

class LinkPreviewType: public Object {
 public:
  static jclass Class;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeAlbum final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<LinkPreviewAlbumMedia>> media_;
  static jfieldID media_fieldID;
  string caption_;
  static jfieldID caption_fieldID;

  linkPreviewTypeAlbum();

  linkPreviewTypeAlbum(array<object_ptr<LinkPreviewAlbumMedia>> &&media_, string const &caption_);

  static const std::int32_t ID = -919156671;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeAnimation final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;

  linkPreviewTypeAnimation();

  explicit linkPreviewTypeAnimation(object_ptr<animation> &&animation_);

  static const std::int32_t ID = -1386429132;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeApp final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypeApp();

  explicit linkPreviewTypeApp(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -475623953;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeArticle final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypeArticle();

  explicit linkPreviewTypeArticle(object_ptr<photo> &&photo_);

  static const std::int32_t ID = 2093915097;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeAudio final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<audio> audio_;
  static jfieldID audio_fieldID;

  linkPreviewTypeAudio();

  explicit linkPreviewTypeAudio(object_ptr<audio> &&audio_);

  static const std::int32_t ID = 1977878482;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeBackground final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  object_ptr<BackgroundType> background_type_;
  static jfieldID background_type_fieldID;

  linkPreviewTypeBackground();

  linkPreviewTypeBackground(object_ptr<document> &&document_, object_ptr<BackgroundType> &&background_type_);

  static const std::int32_t ID = 977838560;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeChannelBoost final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypeChannelBoost();

  explicit linkPreviewTypeChannelBoost(object_ptr<chatPhoto> &&photo_);

  static const std::int32_t ID = -957086634;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeChat final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InviteLinkChatType> type_;
  static jfieldID type_fieldID;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  bool creates_join_request_;
  static jfieldID creates_join_request_fieldID;

  linkPreviewTypeChat();

  linkPreviewTypeChat(object_ptr<InviteLinkChatType> &&type_, object_ptr<chatPhoto> &&photo_, bool creates_join_request_);

  static const std::int32_t ID = -1372610270;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeDocument final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<document> document_;
  static jfieldID document_fieldID;

  linkPreviewTypeDocument();

  explicit linkPreviewTypeDocument(object_ptr<document> &&document_);

  static const std::int32_t ID = -1090426462;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeEmbeddedAnimationPlayer final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<photo> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  linkPreviewTypeEmbeddedAnimationPlayer();

  linkPreviewTypeEmbeddedAnimationPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_);

  static const std::int32_t ID = -1436887547;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeEmbeddedAudioPlayer final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<photo> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  linkPreviewTypeEmbeddedAudioPlayer();

  linkPreviewTypeEmbeddedAudioPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_);

  static const std::int32_t ID = 571163292;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeEmbeddedVideoPlayer final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<photo> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  linkPreviewTypeEmbeddedVideoPlayer();

  linkPreviewTypeEmbeddedVideoPlayer(string const &url_, object_ptr<photo> &&thumbnail_, int32 duration_, int32 width_, int32 height_);

  static const std::int32_t ID = -1480606973;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeExternalAudio final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;

  linkPreviewTypeExternalAudio();

  linkPreviewTypeExternalAudio(string const &url_, string const &mime_type_, int32 duration_);

  static const std::int32_t ID = -1971126291;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeExternalVideo final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;

  linkPreviewTypeExternalVideo();

  linkPreviewTypeExternalVideo(string const &url_, string const &mime_type_, int32 width_, int32 height_, int32 duration_);

  static const std::int32_t ID = 1367198616;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeGroupCall final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypeGroupCall();

  static const std::int32_t ID = -1242459936;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeInvoice final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypeInvoice();

  static const std::int32_t ID = -729855782;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeMessage final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypeMessage();

  static const std::int32_t ID = 435470750;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypePhoto final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypePhoto();

  explicit linkPreviewTypePhoto(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -1362122068;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypePremiumGiftCode final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypePremiumGiftCode();

  static const std::int32_t ID = 1309507761;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeShareableChatFolder final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypeShareableChatFolder();

  static const std::int32_t ID = -2141539524;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeSticker final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  linkPreviewTypeSticker();

  explicit linkPreviewTypeSticker(object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = 610225445;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeStickerSet final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<sticker>> stickers_;
  static jfieldID stickers_fieldID;

  linkPreviewTypeStickerSet();

  explicit linkPreviewTypeStickerSet(array<object_ptr<sticker>> &&stickers_);

  static const std::int32_t ID = -145958768;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeStory final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  linkPreviewTypeStory();

  linkPreviewTypeStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = 1045709531;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeSupergroupBoost final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypeSupergroupBoost();

  explicit linkPreviewTypeSupergroupBoost(object_ptr<chatPhoto> &&photo_);

  static const std::int32_t ID = -1873345418;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeTheme final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<document>> documents_;
  static jfieldID documents_fieldID;
  object_ptr<themeSettings> settings_;
  static jfieldID settings_fieldID;

  linkPreviewTypeTheme();

  linkPreviewTypeTheme(array<object_ptr<document>> &&documents_, object_ptr<themeSettings> &&settings_);

  static const std::int32_t ID = -226118489;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeUnsupported final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  linkPreviewTypeUnsupported();

  static const std::int32_t ID = 1924738233;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeUpgradedGift final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;

  linkPreviewTypeUpgradedGift();

  explicit linkPreviewTypeUpgradedGift(object_ptr<upgradedGift> &&gift_);

  static const std::int32_t ID = 293249807;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeUser final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  bool is_bot_;
  static jfieldID is_bot_fieldID;

  linkPreviewTypeUser();

  linkPreviewTypeUser(object_ptr<chatPhoto> &&photo_, bool is_bot_);

  static const std::int32_t ID = -1465024132;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeVideo final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  object_ptr<photo> cover_;
  static jfieldID cover_fieldID;
  int32 start_timestamp_;
  static jfieldID start_timestamp_fieldID;

  linkPreviewTypeVideo();

  linkPreviewTypeVideo(object_ptr<video> &&video_, object_ptr<photo> &&cover_, int32 start_timestamp_);

  static const std::int32_t ID = 1573057926;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeVideoChat final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  bool is_live_stream_;
  static jfieldID is_live_stream_fieldID;

  linkPreviewTypeVideoChat();

  linkPreviewTypeVideoChat(object_ptr<chatPhoto> &&photo_, bool is_live_stream_);

  static const std::int32_t ID = 420015635;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeVideoNote final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<videoNote> video_note_;
  static jfieldID video_note_fieldID;

  linkPreviewTypeVideoNote();

  explicit linkPreviewTypeVideoNote(object_ptr<videoNote> &&video_note_);

  static const std::int32_t ID = -814687391;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeVoiceNote final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<voiceNote> voice_note_;
  static jfieldID voice_note_fieldID;

  linkPreviewTypeVoiceNote();

  explicit linkPreviewTypeVoiceNote(object_ptr<voiceNote> &&voice_note_);

  static const std::int32_t ID = -757936341;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class linkPreviewTypeWebApp final : public LinkPreviewType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  linkPreviewTypeWebApp();

  explicit linkPreviewTypeWebApp(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -1506873462;

  static object_ptr<LinkPreviewType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class localFile final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string path_;
  static jfieldID path_fieldID;
  bool can_be_downloaded_;
  static jfieldID can_be_downloaded_fieldID;
  bool can_be_deleted_;
  static jfieldID can_be_deleted_fieldID;
  bool is_downloading_active_;
  static jfieldID is_downloading_active_fieldID;
  bool is_downloading_completed_;
  static jfieldID is_downloading_completed_fieldID;
  int53 download_offset_;
  static jfieldID download_offset_fieldID;
  int53 downloaded_prefix_size_;
  static jfieldID downloaded_prefix_size_fieldID;
  int53 downloaded_size_;
  static jfieldID downloaded_size_fieldID;

  localFile();

  localFile(string const &path_, bool can_be_downloaded_, bool can_be_deleted_, bool is_downloading_active_, bool is_downloading_completed_, int53 download_offset_, int53 downloaded_prefix_size_, int53 downloaded_size_);

  static const std::int32_t ID = -1562732153;

  static object_ptr<localFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class languagePackInfo;

class localizationTargetInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<languagePackInfo>> language_packs_;
  static jfieldID language_packs_fieldID;

  localizationTargetInfo();

  explicit localizationTargetInfo(array<object_ptr<languagePackInfo>> &&language_packs_);

  static const std::int32_t ID = -2048670809;

  static object_ptr<localizationTargetInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class location final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double latitude_;
  static jfieldID latitude_fieldID;
  double longitude_;
  static jfieldID longitude_fieldID;
  double horizontal_accuracy_;
  static jfieldID horizontal_accuracy_fieldID;

  location();

  location(double latitude_, double longitude_, double horizontal_accuracy_);

  static const std::int32_t ID = -443392141;

  static object_ptr<location> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class locationAddress final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string country_code_;
  static jfieldID country_code_fieldID;
  string state_;
  static jfieldID state_fieldID;
  string city_;
  static jfieldID city_fieldID;
  string street_;
  static jfieldID street_fieldID;

  locationAddress();

  locationAddress(string const &country_code_, string const &state_, string const &city_, string const &street_);

  static const std::int32_t ID = -1545940190;

  static object_ptr<locationAddress> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class LogStream: public Object {
 public:
  static jclass Class;

  static object_ptr<LogStream> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class logStreamDefault final : public LogStream {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  logStreamDefault();

  static const std::int32_t ID = 1390581436;

  static object_ptr<LogStream> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class logStreamFile final : public LogStream {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string path_;
  static jfieldID path_fieldID;
  int53 max_file_size_;
  static jfieldID max_file_size_fieldID;
  bool redirect_stderr_;
  static jfieldID redirect_stderr_fieldID;

  logStreamFile();

  logStreamFile(string const &path_, int53 max_file_size_, bool redirect_stderr_);

  static const std::int32_t ID = 1532136933;

  static object_ptr<LogStream> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class logStreamEmpty final : public LogStream {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  logStreamEmpty();

  static const std::int32_t ID = -499912244;

  static object_ptr<LogStream> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class logTags final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> tags_;
  static jfieldID tags_fieldID;

  logTags();

  explicit logTags(array<string> &&tags_);

  static const std::int32_t ID = -1604930601;

  static object_ptr<logTags> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class logVerbosityLevel final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 verbosity_level_;
  static jfieldID verbosity_level_fieldID;

  logVerbosityLevel();

  explicit logVerbosityLevel(int32 verbosity_level_);

  static const std::int32_t ID = 1734624234;

  static object_ptr<logVerbosityLevel> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class LoginUrlInfo: public Object {
 public:
  static jclass Class;

  static object_ptr<LoginUrlInfo> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class loginUrlInfoOpen final : public LoginUrlInfo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  bool skip_confirmation_;
  static jfieldID skip_confirmation_fieldID;

  loginUrlInfoOpen();

  loginUrlInfoOpen(string const &url_, bool skip_confirmation_);

  static const std::int32_t ID = 837282306;

  static object_ptr<LoginUrlInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class loginUrlInfoRequestConfirmation final : public LoginUrlInfo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string domain_;
  static jfieldID domain_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  bool request_write_access_;
  static jfieldID request_write_access_fieldID;

  loginUrlInfoRequestConfirmation();

  loginUrlInfoRequestConfirmation(string const &url_, string const &domain_, int53 bot_user_id_, bool request_write_access_);

  static const std::int32_t ID = 2128290863;

  static object_ptr<LoginUrlInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class WebAppOpenMode;

class mainWebApp final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<WebAppOpenMode> mode_;
  static jfieldID mode_fieldID;

  mainWebApp();

  mainWebApp(string const &url_, object_ptr<WebAppOpenMode> &&mode_);

  static const std::int32_t ID = 1940368506;

  static object_ptr<mainWebApp> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MaskPoint: public Object {
 public:
  static jclass Class;

  static object_ptr<MaskPoint> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class maskPointForehead final : public MaskPoint {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  maskPointForehead();

  static const std::int32_t ID = 1027512005;

  static object_ptr<MaskPoint> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class maskPointEyes final : public MaskPoint {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  maskPointEyes();

  static const std::int32_t ID = 1748310861;

  static object_ptr<MaskPoint> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class maskPointMouth final : public MaskPoint {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  maskPointMouth();

  static const std::int32_t ID = 411773406;

  static object_ptr<MaskPoint> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class maskPointChin final : public MaskPoint {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  maskPointChin();

  static const std::int32_t ID = 534995335;

  static object_ptr<MaskPoint> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MaskPoint;

class maskPosition final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MaskPoint> point_;
  static jfieldID point_fieldID;
  double x_shift_;
  static jfieldID x_shift_fieldID;
  double y_shift_;
  static jfieldID y_shift_fieldID;
  double scale_;
  static jfieldID scale_fieldID;

  maskPosition();

  maskPosition(object_ptr<MaskPoint> &&point_, double x_shift_, double y_shift_, double scale_);

  static const std::int32_t ID = -2097433026;

  static object_ptr<maskPosition> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageContent;

class MessageReplyTo;

class MessageSchedulingState;

class MessageSelfDestructType;

class MessageSender;

class MessageSendingState;

class ReplyMarkup;

class factCheck;

class messageForwardInfo;

class messageImportInfo;

class messageInteractionInfo;

class unreadReaction;

class message final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSendingState> sending_state_;
  static jfieldID sending_state_fieldID;
  object_ptr<MessageSchedulingState> scheduling_state_;
  static jfieldID scheduling_state_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  bool is_from_offline_;
  static jfieldID is_from_offline_fieldID;
  bool can_be_saved_;
  static jfieldID can_be_saved_fieldID;
  bool has_timestamped_media_;
  static jfieldID has_timestamped_media_fieldID;
  bool is_channel_post_;
  static jfieldID is_channel_post_fieldID;
  bool is_topic_message_;
  static jfieldID is_topic_message_fieldID;
  bool contains_unread_mention_;
  static jfieldID contains_unread_mention_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  int32 edit_date_;
  static jfieldID edit_date_fieldID;
  object_ptr<messageForwardInfo> forward_info_;
  static jfieldID forward_info_fieldID;
  object_ptr<messageImportInfo> import_info_;
  static jfieldID import_info_fieldID;
  object_ptr<messageInteractionInfo> interaction_info_;
  static jfieldID interaction_info_fieldID;
  array<object_ptr<unreadReaction>> unread_reactions_;
  static jfieldID unread_reactions_fieldID;
  object_ptr<factCheck> fact_check_;
  static jfieldID fact_check_fieldID;
  object_ptr<MessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  object_ptr<MessageSelfDestructType> self_destruct_type_;
  static jfieldID self_destruct_type_fieldID;
  double self_destruct_in_;
  static jfieldID self_destruct_in_fieldID;
  double auto_delete_in_;
  static jfieldID auto_delete_in_fieldID;
  int53 via_bot_user_id_;
  static jfieldID via_bot_user_id_fieldID;
  int53 sender_business_bot_user_id_;
  static jfieldID sender_business_bot_user_id_fieldID;
  int32 sender_boost_count_;
  static jfieldID sender_boost_count_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;
  string author_signature_;
  static jfieldID author_signature_fieldID;
  int64 media_album_id_;
  static jfieldID media_album_id_fieldID;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;
  bool has_sensitive_content_;
  static jfieldID has_sensitive_content_fieldID;
  string restriction_reason_;
  static jfieldID restriction_reason_fieldID;
  object_ptr<MessageContent> content_;
  static jfieldID content_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  message();

  message(int53 id_, object_ptr<MessageSender> &&sender_id_, int53 chat_id_, object_ptr<MessageSendingState> &&sending_state_, object_ptr<MessageSchedulingState> &&scheduling_state_, bool is_outgoing_, bool is_pinned_, bool is_from_offline_, bool can_be_saved_, bool has_timestamped_media_, bool is_channel_post_, bool is_topic_message_, bool contains_unread_mention_, int32 date_, int32 edit_date_, object_ptr<messageForwardInfo> &&forward_info_, object_ptr<messageImportInfo> &&import_info_, object_ptr<messageInteractionInfo> &&interaction_info_, array<object_ptr<unreadReaction>> &&unread_reactions_, object_ptr<factCheck> &&fact_check_, object_ptr<MessageReplyTo> &&reply_to_, int53 message_thread_id_, int53 saved_messages_topic_id_, object_ptr<MessageSelfDestructType> &&self_destruct_type_, double self_destruct_in_, double auto_delete_in_, int53 via_bot_user_id_, int53 sender_business_bot_user_id_, int32 sender_boost_count_, int53 paid_message_star_count_, string const &author_signature_, int64 media_album_id_, int64 effect_id_, bool has_sensitive_content_, string const &restriction_reason_, object_ptr<MessageContent> &&content_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = 726001662;

  static object_ptr<message> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageAutoDeleteTime final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 time_;
  static jfieldID time_fieldID;

  messageAutoDeleteTime();

  explicit messageAutoDeleteTime(int32 time_);

  static const std::int32_t ID = 1972045589;

  static object_ptr<messageAutoDeleteTime> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageCalendarDay;

class messageCalendar final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<messageCalendarDay>> days_;
  static jfieldID days_fieldID;

  messageCalendar();

  messageCalendar(int32 total_count_, array<object_ptr<messageCalendarDay>> &&days_);

  static const std::int32_t ID = -1682890519;

  static object_ptr<messageCalendar> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class messageCalendarDay final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  messageCalendarDay();

  messageCalendarDay(int32 total_count_, object_ptr<message> &&message_);

  static const std::int32_t ID = -376467614;

  static object_ptr<messageCalendarDay> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BotWriteAccessAllowReason;

class CallDiscardReason;

class DiceStickers;

class GiveawayPrize;

class MessageSender;

class PaidMedia;

class PassportElementType;

class alternativeVideo;

class animatedEmoji;

class animation;

class audio;

class chatBackground;

class chatPhoto;

class contact;

class document;

class encryptedCredentials;

class encryptedPassportElement;

class formattedText;

class forumTopicIcon;

class game;

class gift;

class giveawayParameters;

class linkPreview;

class linkPreviewOptions;

class location;

class orderInfo;

class photo;

class poll;

class productInfo;

class sharedChat;

class sharedUser;

class sticker;

class upgradedGift;

class venue;

class video;

class videoNote;

class voiceNote;

class MessageContent: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageText final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  object_ptr<linkPreview> link_preview_;
  static jfieldID link_preview_fieldID;
  object_ptr<linkPreviewOptions> link_preview_options_;
  static jfieldID link_preview_options_fieldID;

  messageText();

  messageText(object_ptr<formattedText> &&text_, object_ptr<linkPreview> &&link_preview_, object_ptr<linkPreviewOptions> &&link_preview_options_);

  static const std::int32_t ID = 1751469188;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageAnimation final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;

  messageAnimation();

  messageAnimation(object_ptr<animation> &&animation_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_);

  static const std::int32_t ID = -1899294424;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageAudio final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<audio> audio_;
  static jfieldID audio_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  messageAudio();

  messageAudio(object_ptr<audio> &&audio_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = 276722716;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageDocument final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  messageDocument();

  messageDocument(object_ptr<document> &&document_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = 596945783;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaidMedia final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  array<object_ptr<PaidMedia>> media_;
  static jfieldID media_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;

  messagePaidMedia();

  messagePaidMedia(int53 star_count_, array<object_ptr<PaidMedia>> &&media_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_);

  static const std::int32_t ID = -724750073;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePhoto final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;

  messagePhoto();

  messagePhoto(object_ptr<photo> &&photo_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_);

  static const std::int32_t ID = 1967947295;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSticker final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  bool is_premium_;
  static jfieldID is_premium_fieldID;

  messageSticker();

  messageSticker(object_ptr<sticker> &&sticker_, bool is_premium_);

  static const std::int32_t ID = -437199670;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVideo final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  array<object_ptr<alternativeVideo>> alternative_videos_;
  static jfieldID alternative_videos_fieldID;
  object_ptr<photo> cover_;
  static jfieldID cover_fieldID;
  int32 start_timestamp_;
  static jfieldID start_timestamp_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;
  bool has_spoiler_;
  static jfieldID has_spoiler_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;

  messageVideo();

  messageVideo(object_ptr<video> &&video_, array<object_ptr<alternativeVideo>> &&alternative_videos_, object_ptr<photo> &&cover_, int32 start_timestamp_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_, bool has_spoiler_, bool is_secret_);

  static const std::int32_t ID = 374791437;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVideoNote final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<videoNote> video_note_;
  static jfieldID video_note_fieldID;
  bool is_viewed_;
  static jfieldID is_viewed_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;

  messageVideoNote();

  messageVideoNote(object_ptr<videoNote> &&video_note_, bool is_viewed_, bool is_secret_);

  static const std::int32_t ID = 963323014;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVoiceNote final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<voiceNote> voice_note_;
  static jfieldID voice_note_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool is_listened_;
  static jfieldID is_listened_fieldID;

  messageVoiceNote();

  messageVoiceNote(object_ptr<voiceNote> &&voice_note_, object_ptr<formattedText> &&caption_, bool is_listened_);

  static const std::int32_t ID = 527777781;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageExpiredPhoto final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageExpiredPhoto();

  static const std::int32_t ID = -1404641801;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageExpiredVideo final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageExpiredVideo();

  static const std::int32_t ID = -1212209981;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageExpiredVideoNote final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageExpiredVideoNote();

  static const std::int32_t ID = 599540711;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageExpiredVoiceNote final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageExpiredVoiceNote();

  static const std::int32_t ID = 143684989;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageLocation final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  int32 expires_in_;
  static jfieldID expires_in_fieldID;
  int32 heading_;
  static jfieldID heading_fieldID;
  int32 proximity_alert_radius_;
  static jfieldID proximity_alert_radius_fieldID;

  messageLocation();

  messageLocation(object_ptr<location> &&location_, int32 live_period_, int32 expires_in_, int32 heading_, int32 proximity_alert_radius_);

  static const std::int32_t ID = 303973492;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVenue final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<venue> venue_;
  static jfieldID venue_fieldID;

  messageVenue();

  explicit messageVenue(object_ptr<venue> &&venue_);

  static const std::int32_t ID = -2146492043;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageContact final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<contact> contact_;
  static jfieldID contact_fieldID;

  messageContact();

  explicit messageContact(object_ptr<contact> &&contact_);

  static const std::int32_t ID = -512684966;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageAnimatedEmoji final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<animatedEmoji> animated_emoji_;
  static jfieldID animated_emoji_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;

  messageAnimatedEmoji();

  messageAnimatedEmoji(object_ptr<animatedEmoji> &&animated_emoji_, string const &emoji_);

  static const std::int32_t ID = 908195298;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageDice final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<DiceStickers> initial_state_;
  static jfieldID initial_state_fieldID;
  object_ptr<DiceStickers> final_state_;
  static jfieldID final_state_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  int32 value_;
  static jfieldID value_fieldID;
  int32 success_animation_frame_number_;
  static jfieldID success_animation_frame_number_fieldID;

  messageDice();

  messageDice(object_ptr<DiceStickers> &&initial_state_, object_ptr<DiceStickers> &&final_state_, string const &emoji_, int32 value_, int32 success_animation_frame_number_);

  static const std::int32_t ID = 1115779641;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGame final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<game> game_;
  static jfieldID game_fieldID;

  messageGame();

  explicit messageGame(object_ptr<game> &&game_);

  static const std::int32_t ID = -69441162;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePoll final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<poll> poll_;
  static jfieldID poll_fieldID;

  messagePoll();

  explicit messagePoll(object_ptr<poll> &&poll_);

  static const std::int32_t ID = -662130099;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageStory final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  bool via_mention_;
  static jfieldID via_mention_fieldID;

  messageStory();

  messageStory(int53 story_poster_chat_id_, int32 story_id_, bool via_mention_);

  static const std::int32_t ID = 1514236353;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageInvoice final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 total_amount_;
  static jfieldID total_amount_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  bool is_test_;
  static jfieldID is_test_fieldID;
  bool need_shipping_address_;
  static jfieldID need_shipping_address_fieldID;
  int53 receipt_message_id_;
  static jfieldID receipt_message_id_fieldID;
  object_ptr<PaidMedia> paid_media_;
  static jfieldID paid_media_fieldID;
  object_ptr<formattedText> paid_media_caption_;
  static jfieldID paid_media_caption_fieldID;

  messageInvoice();

  messageInvoice(object_ptr<productInfo> &&product_info_, string const &currency_, int53 total_amount_, string const &start_parameter_, bool is_test_, bool need_shipping_address_, int53 receipt_message_id_, object_ptr<PaidMedia> &&paid_media_, object_ptr<formattedText> &&paid_media_caption_);

  static const std::int32_t ID = 263060806;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageCall final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_video_;
  static jfieldID is_video_fieldID;
  object_ptr<CallDiscardReason> discard_reason_;
  static jfieldID discard_reason_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;

  messageCall();

  messageCall(bool is_video_, object_ptr<CallDiscardReason> &&discard_reason_, int32 duration_);

  static const std::int32_t ID = 538893824;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGroupCall final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_active_;
  static jfieldID is_active_fieldID;
  bool was_missed_;
  static jfieldID was_missed_fieldID;
  bool is_video_;
  static jfieldID is_video_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  array<object_ptr<MessageSender>> other_participant_ids_;
  static jfieldID other_participant_ids_fieldID;

  messageGroupCall();

  messageGroupCall(bool is_active_, bool was_missed_, bool is_video_, int32 duration_, array<object_ptr<MessageSender>> &&other_participant_ids_);

  static const std::int32_t ID = 1370396295;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVideoChatScheduled final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int32 start_date_;
  static jfieldID start_date_fieldID;

  messageVideoChatScheduled();

  messageVideoChatScheduled(int32 group_call_id_, int32 start_date_);

  static const std::int32_t ID = -1855185481;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVideoChatStarted final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  messageVideoChatStarted();

  explicit messageVideoChatStarted(int32 group_call_id_);

  static const std::int32_t ID = 521225561;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageVideoChatEnded final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;

  messageVideoChatEnded();

  explicit messageVideoChatEnded(int32 duration_);

  static const std::int32_t ID = 2032544855;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageInviteVideoChatParticipants final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  messageInviteVideoChatParticipants();

  messageInviteVideoChatParticipants(int32 group_call_id_, array<int53> &&user_ids_);

  static const std::int32_t ID = -1459065585;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageBasicGroupChatCreate final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  array<int53> member_user_ids_;
  static jfieldID member_user_ids_fieldID;

  messageBasicGroupChatCreate();

  messageBasicGroupChatCreate(string const &title_, array<int53> &&member_user_ids_);

  static const std::int32_t ID = 795404060;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSupergroupChatCreate final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;

  messageSupergroupChatCreate();

  explicit messageSupergroupChatCreate(string const &title_);

  static const std::int32_t ID = -434325733;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatChangeTitle final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;

  messageChatChangeTitle();

  explicit messageChatChangeTitle(string const &title_);

  static const std::int32_t ID = 748272449;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatChangePhoto final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;

  messageChatChangePhoto();

  explicit messageChatChangePhoto(object_ptr<chatPhoto> &&photo_);

  static const std::int32_t ID = -813415093;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatDeletePhoto final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageChatDeletePhoto();

  static const std::int32_t ID = -184374809;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatAddMembers final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> member_user_ids_;
  static jfieldID member_user_ids_fieldID;

  messageChatAddMembers();

  explicit messageChatAddMembers(array<int53> &&member_user_ids_);

  static const std::int32_t ID = 1701117908;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatJoinByLink final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageChatJoinByLink();

  static const std::int32_t ID = 1846493311;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatJoinByRequest final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageChatJoinByRequest();

  static const std::int32_t ID = 1195428732;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatDeleteMember final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  messageChatDeleteMember();

  explicit messageChatDeleteMember(int53 user_id_);

  static const std::int32_t ID = 938029481;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatUpgradeTo final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  messageChatUpgradeTo();

  explicit messageChatUpgradeTo(int53 supergroup_id_);

  static const std::int32_t ID = 104813723;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatUpgradeFrom final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;

  messageChatUpgradeFrom();

  messageChatUpgradeFrom(string const &title_, int53 basic_group_id_);

  static const std::int32_t ID = 325954268;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePinMessage final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  messagePinMessage();

  explicit messagePinMessage(int53 message_id_);

  static const std::int32_t ID = 953503801;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageScreenshotTaken final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageScreenshotTaken();

  static const std::int32_t ID = -1564971605;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatSetBackground final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 old_background_message_id_;
  static jfieldID old_background_message_id_fieldID;
  object_ptr<chatBackground> background_;
  static jfieldID background_fieldID;
  bool only_for_self_;
  static jfieldID only_for_self_fieldID;

  messageChatSetBackground();

  messageChatSetBackground(int53 old_background_message_id_, object_ptr<chatBackground> &&background_, bool only_for_self_);

  static const std::int32_t ID = 1029536832;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatSetTheme final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  messageChatSetTheme();

  explicit messageChatSetTheme(string const &theme_name_);

  static const std::int32_t ID = -1716612088;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatSetMessageAutoDeleteTime final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;
  int53 from_user_id_;
  static jfieldID from_user_id_fieldID;

  messageChatSetMessageAutoDeleteTime();

  messageChatSetMessageAutoDeleteTime(int32 message_auto_delete_time_, int53 from_user_id_);

  static const std::int32_t ID = 1637745966;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatBoost final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 boost_count_;
  static jfieldID boost_count_fieldID;

  messageChatBoost();

  explicit messageChatBoost(int32 boost_count_);

  static const std::int32_t ID = 1583310219;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageForumTopicCreated final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<forumTopicIcon> icon_;
  static jfieldID icon_fieldID;

  messageForumTopicCreated();

  messageForumTopicCreated(string const &name_, object_ptr<forumTopicIcon> &&icon_);

  static const std::int32_t ID = -1194440751;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageForumTopicEdited final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  bool edit_icon_custom_emoji_id_;
  static jfieldID edit_icon_custom_emoji_id_fieldID;
  int64 icon_custom_emoji_id_;
  static jfieldID icon_custom_emoji_id_fieldID;

  messageForumTopicEdited();

  messageForumTopicEdited(string const &name_, bool edit_icon_custom_emoji_id_, int64 icon_custom_emoji_id_);

  static const std::int32_t ID = 12629888;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageForumTopicIsClosedToggled final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_closed_;
  static jfieldID is_closed_fieldID;

  messageForumTopicIsClosedToggled();

  explicit messageForumTopicIsClosedToggled(bool is_closed_);

  static const std::int32_t ID = 1264029664;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageForumTopicIsHiddenToggled final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_hidden_;
  static jfieldID is_hidden_fieldID;

  messageForumTopicIsHiddenToggled();

  explicit messageForumTopicIsHiddenToggled(bool is_hidden_);

  static const std::int32_t ID = -1751936002;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSuggestProfilePhoto final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;

  messageSuggestProfilePhoto();

  explicit messageSuggestProfilePhoto(object_ptr<chatPhoto> &&photo_);

  static const std::int32_t ID = -1251926297;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageCustomServiceAction final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  messageCustomServiceAction();

  explicit messageCustomServiceAction(string const &text_);

  static const std::int32_t ID = 1435879282;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGameScore final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 game_message_id_;
  static jfieldID game_message_id_fieldID;
  int64 game_id_;
  static jfieldID game_id_fieldID;
  int32 score_;
  static jfieldID score_fieldID;

  messageGameScore();

  messageGameScore(int53 game_message_id_, int64 game_id_, int32 score_);

  static const std::int32_t ID = 1344904575;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaymentSuccessful final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 invoice_chat_id_;
  static jfieldID invoice_chat_id_fieldID;
  int53 invoice_message_id_;
  static jfieldID invoice_message_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 total_amount_;
  static jfieldID total_amount_fieldID;
  int32 subscription_until_date_;
  static jfieldID subscription_until_date_fieldID;
  bool is_recurring_;
  static jfieldID is_recurring_fieldID;
  bool is_first_recurring_;
  static jfieldID is_first_recurring_fieldID;
  string invoice_name_;
  static jfieldID invoice_name_fieldID;

  messagePaymentSuccessful();

  messagePaymentSuccessful(int53 invoice_chat_id_, int53 invoice_message_id_, string const &currency_, int53 total_amount_, int32 subscription_until_date_, bool is_recurring_, bool is_first_recurring_, string const &invoice_name_);

  static const std::int32_t ID = 1046878481;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaymentSuccessfulBot final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 total_amount_;
  static jfieldID total_amount_fieldID;
  int32 subscription_until_date_;
  static jfieldID subscription_until_date_fieldID;
  bool is_recurring_;
  static jfieldID is_recurring_fieldID;
  bool is_first_recurring_;
  static jfieldID is_first_recurring_fieldID;
  bytes invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  string shipping_option_id_;
  static jfieldID shipping_option_id_fieldID;
  object_ptr<orderInfo> order_info_;
  static jfieldID order_info_fieldID;
  string telegram_payment_charge_id_;
  static jfieldID telegram_payment_charge_id_fieldID;
  string provider_payment_charge_id_;
  static jfieldID provider_payment_charge_id_fieldID;

  messagePaymentSuccessfulBot();

  messagePaymentSuccessfulBot(string const &currency_, int53 total_amount_, int32 subscription_until_date_, bool is_recurring_, bool is_first_recurring_, bytes const &invoice_payload_, string const &shipping_option_id_, object_ptr<orderInfo> &&order_info_, string const &telegram_payment_charge_id_, string const &provider_payment_charge_id_);

  static const std::int32_t ID = -949596737;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaymentRefunded final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 total_amount_;
  static jfieldID total_amount_fieldID;
  bytes invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  string telegram_payment_charge_id_;
  static jfieldID telegram_payment_charge_id_fieldID;
  string provider_payment_charge_id_;
  static jfieldID provider_payment_charge_id_fieldID;

  messagePaymentRefunded();

  messagePaymentRefunded(object_ptr<MessageSender> &&owner_id_, string const &currency_, int53 total_amount_, bytes const &invoice_payload_, string const &telegram_payment_charge_id_, string const &provider_payment_charge_id_);

  static const std::int32_t ID = 297580787;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiftedPremium final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 gifter_user_id_;
  static jfieldID gifter_user_id_fieldID;
  int53 receiver_user_id_;
  static jfieldID receiver_user_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 cryptocurrency_amount_;
  static jfieldID cryptocurrency_amount_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  messageGiftedPremium();

  messageGiftedPremium(int53 gifter_user_id_, int53 receiver_user_id_, object_ptr<formattedText> &&text_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int32 month_count_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -456073094;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePremiumGiftCode final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> creator_id_;
  static jfieldID creator_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool is_from_giveaway_;
  static jfieldID is_from_giveaway_fieldID;
  bool is_unclaimed_;
  static jfieldID is_unclaimed_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 cryptocurrency_amount_;
  static jfieldID cryptocurrency_amount_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  string code_;
  static jfieldID code_fieldID;

  messagePremiumGiftCode();

  messagePremiumGiftCode(object_ptr<MessageSender> &&creator_id_, object_ptr<formattedText> &&text_, bool is_from_giveaway_, bool is_unclaimed_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int32 month_count_, object_ptr<sticker> &&sticker_, string const &code_);

  static const std::int32_t ID = 701640270;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiveawayCreated final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  messageGiveawayCreated();

  explicit messageGiveawayCreated(int53 star_count_);

  static const std::int32_t ID = 972252063;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiveaway final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  object_ptr<GiveawayPrize> prize_;
  static jfieldID prize_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  messageGiveaway();

  messageGiveaway(object_ptr<giveawayParameters> &&parameters_, int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -345908568;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiveawayCompleted final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  bool is_star_giveaway_;
  static jfieldID is_star_giveaway_fieldID;
  int32 unclaimed_prize_count_;
  static jfieldID unclaimed_prize_count_fieldID;

  messageGiveawayCompleted();

  messageGiveawayCompleted(int53 giveaway_message_id_, int32 winner_count_, bool is_star_giveaway_, int32 unclaimed_prize_count_);

  static const std::int32_t ID = -467351305;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiveawayWinners final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;
  int32 additional_chat_count_;
  static jfieldID additional_chat_count_fieldID;
  int32 actual_winners_selection_date_;
  static jfieldID actual_winners_selection_date_fieldID;
  bool only_new_members_;
  static jfieldID only_new_members_fieldID;
  bool was_refunded_;
  static jfieldID was_refunded_fieldID;
  object_ptr<GiveawayPrize> prize_;
  static jfieldID prize_fieldID;
  string prize_description_;
  static jfieldID prize_description_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  array<int53> winner_user_ids_;
  static jfieldID winner_user_ids_fieldID;
  int32 unclaimed_prize_count_;
  static jfieldID unclaimed_prize_count_fieldID;

  messageGiveawayWinners();

  messageGiveawayWinners(int53 boosted_chat_id_, int53 giveaway_message_id_, int32 additional_chat_count_, int32 actual_winners_selection_date_, bool only_new_members_, bool was_refunded_, object_ptr<GiveawayPrize> &&prize_, string const &prize_description_, int32 winner_count_, array<int53> &&winner_user_ids_, int32 unclaimed_prize_count_);

  static const std::int32_t ID = 2098585405;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiftedStars final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 gifter_user_id_;
  static jfieldID gifter_user_id_fieldID;
  int53 receiver_user_id_;
  static jfieldID receiver_user_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  string cryptocurrency_;
  static jfieldID cryptocurrency_fieldID;
  int64 cryptocurrency_amount_;
  static jfieldID cryptocurrency_amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string transaction_id_;
  static jfieldID transaction_id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  messageGiftedStars();

  messageGiftedStars(int53 gifter_user_id_, int53 receiver_user_id_, string const &currency_, int53 amount_, string const &cryptocurrency_, int64 cryptocurrency_amount_, int53 star_count_, string const &transaction_id_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = 1102954151;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGiveawayPrizeStars final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string transaction_id_;
  static jfieldID transaction_id_fieldID;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;
  bool is_unclaimed_;
  static jfieldID is_unclaimed_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  messageGiveawayPrizeStars();

  messageGiveawayPrizeStars(int53 star_count_, string const &transaction_id_, int53 boosted_chat_id_, int53 giveaway_message_id_, bool is_unclaimed_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -1441833501;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageGift final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int53 sell_star_count_;
  static jfieldID sell_star_count_fieldID;
  int53 prepaid_upgrade_star_count_;
  static jfieldID prepaid_upgrade_star_count_fieldID;
  bool is_private_;
  static jfieldID is_private_fieldID;
  bool is_saved_;
  static jfieldID is_saved_fieldID;
  bool can_be_upgraded_;
  static jfieldID can_be_upgraded_fieldID;
  bool was_converted_;
  static jfieldID was_converted_fieldID;
  bool was_upgraded_;
  static jfieldID was_upgraded_fieldID;
  bool was_refunded_;
  static jfieldID was_refunded_fieldID;
  string upgraded_received_gift_id_;
  static jfieldID upgraded_received_gift_id_fieldID;

  messageGift();

  messageGift(object_ptr<gift> &&gift_, object_ptr<MessageSender> &&sender_id_, string const &received_gift_id_, object_ptr<formattedText> &&text_, int53 sell_star_count_, int53 prepaid_upgrade_star_count_, bool is_private_, bool is_saved_, bool can_be_upgraded_, bool was_converted_, bool was_upgraded_, bool was_refunded_, string const &upgraded_received_gift_id_);

  static const std::int32_t ID = 2090444659;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageUpgradedGift final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  bool is_upgrade_;
  static jfieldID is_upgrade_fieldID;
  bool is_saved_;
  static jfieldID is_saved_fieldID;
  bool can_be_transferred_;
  static jfieldID can_be_transferred_fieldID;
  bool was_transferred_;
  static jfieldID was_transferred_fieldID;
  int53 last_resale_star_count_;
  static jfieldID last_resale_star_count_fieldID;
  int53 transfer_star_count_;
  static jfieldID transfer_star_count_fieldID;
  int32 next_transfer_date_;
  static jfieldID next_transfer_date_fieldID;
  int32 next_resale_date_;
  static jfieldID next_resale_date_fieldID;
  int32 export_date_;
  static jfieldID export_date_fieldID;

  messageUpgradedGift();

  messageUpgradedGift(object_ptr<upgradedGift> &&gift_, object_ptr<MessageSender> &&sender_id_, string const &received_gift_id_, bool is_upgrade_, bool is_saved_, bool can_be_transferred_, bool was_transferred_, int53 last_resale_star_count_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_);

  static const std::int32_t ID = -1586797067;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageRefundedUpgradedGift final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  bool is_upgrade_;
  static jfieldID is_upgrade_fieldID;

  messageRefundedUpgradedGift();

  messageRefundedUpgradedGift(object_ptr<gift> &&gift_, object_ptr<MessageSender> &&sender_id_, bool is_upgrade_);

  static const std::int32_t ID = -1308949479;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaidMessagesRefunded final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 message_count_;
  static jfieldID message_count_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  messagePaidMessagesRefunded();

  messagePaidMessagesRefunded(int32 message_count_, int53 star_count_);

  static const std::int32_t ID = 580403343;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePaidMessagePriceChanged final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;

  messagePaidMessagePriceChanged();

  explicit messagePaidMessagePriceChanged(int53 paid_message_star_count_);

  static const std::int32_t ID = -344945397;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageContactRegistered final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageContactRegistered();

  static const std::int32_t ID = -1502020353;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageUsersShared final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<sharedUser>> users_;
  static jfieldID users_fieldID;
  int32 button_id_;
  static jfieldID button_id_fieldID;

  messageUsersShared();

  messageUsersShared(array<object_ptr<sharedUser>> &&users_, int32 button_id_);

  static const std::int32_t ID = -842442318;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageChatShared final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sharedChat> chat_;
  static jfieldID chat_fieldID;
  int32 button_id_;
  static jfieldID button_id_fieldID;

  messageChatShared();

  messageChatShared(object_ptr<sharedChat> &&chat_, int32 button_id_);

  static const std::int32_t ID = -1362699935;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageBotWriteAccessAllowed final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BotWriteAccessAllowReason> reason_;
  static jfieldID reason_fieldID;

  messageBotWriteAccessAllowed();

  explicit messageBotWriteAccessAllowed(object_ptr<BotWriteAccessAllowReason> &&reason_);

  static const std::int32_t ID = -1702185036;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageWebAppDataSent final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string button_text_;
  static jfieldID button_text_fieldID;

  messageWebAppDataSent();

  explicit messageWebAppDataSent(string const &button_text_);

  static const std::int32_t ID = -83674862;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageWebAppDataReceived final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string button_text_;
  static jfieldID button_text_fieldID;
  string data_;
  static jfieldID data_fieldID;

  messageWebAppDataReceived();

  messageWebAppDataReceived(string const &button_text_, string const &data_);

  static const std::int32_t ID = -8578539;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePassportDataSent final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PassportElementType>> types_;
  static jfieldID types_fieldID;

  messagePassportDataSent();

  explicit messagePassportDataSent(array<object_ptr<PassportElementType>> &&types_);

  static const std::int32_t ID = 1017405171;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePassportDataReceived final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<encryptedPassportElement>> elements_;
  static jfieldID elements_fieldID;
  object_ptr<encryptedCredentials> credentials_;
  static jfieldID credentials_fieldID;

  messagePassportDataReceived();

  messagePassportDataReceived(array<object_ptr<encryptedPassportElement>> &&elements_, object_ptr<encryptedCredentials> &&credentials_);

  static const std::int32_t ID = -1367863624;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageProximityAlertTriggered final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> traveler_id_;
  static jfieldID traveler_id_fieldID;
  object_ptr<MessageSender> watcher_id_;
  static jfieldID watcher_id_fieldID;
  int32 distance_;
  static jfieldID distance_fieldID;

  messageProximityAlertTriggered();

  messageProximityAlertTriggered(object_ptr<MessageSender> &&traveler_id_, object_ptr<MessageSender> &&watcher_id_, int32 distance_);

  static const std::int32_t ID = 67761875;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageUnsupported final : public MessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageUnsupported();

  static const std::int32_t ID = -1816726139;

  static object_ptr<MessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class messageCopyOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool send_copy_;
  static jfieldID send_copy_fieldID;
  bool replace_caption_;
  static jfieldID replace_caption_fieldID;
  object_ptr<formattedText> new_caption_;
  static jfieldID new_caption_fieldID;
  bool new_show_caption_above_media_;
  static jfieldID new_show_caption_above_media_fieldID;

  messageCopyOptions();

  messageCopyOptions(bool send_copy_, bool replace_caption_, object_ptr<formattedText> &&new_caption_, bool new_show_caption_above_media_);

  static const std::int32_t ID = 1079772090;

  static object_ptr<messageCopyOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageEffectType;

class sticker;

class messageEffect final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  object_ptr<sticker> static_icon_;
  static jfieldID static_icon_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  bool is_premium_;
  static jfieldID is_premium_fieldID;
  object_ptr<MessageEffectType> type_;
  static jfieldID type_fieldID;

  messageEffect();

  messageEffect(int64 id_, object_ptr<sticker> &&static_icon_, string const &emoji_, bool is_premium_, object_ptr<MessageEffectType> &&type_);

  static const std::int32_t ID = -1758836433;

  static object_ptr<messageEffect> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class MessageEffectType: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageEffectType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageEffectTypeEmojiReaction final : public MessageEffectType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> select_animation_;
  static jfieldID select_animation_fieldID;
  object_ptr<sticker> effect_animation_;
  static jfieldID effect_animation_fieldID;

  messageEffectTypeEmojiReaction();

  messageEffectTypeEmojiReaction(object_ptr<sticker> &&select_animation_, object_ptr<sticker> &&effect_animation_);

  static const std::int32_t ID = 1756079678;

  static object_ptr<MessageEffectType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageEffectTypePremiumSticker final : public MessageEffectType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  messageEffectTypePremiumSticker();

  explicit messageEffectTypePremiumSticker(object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = 1637231609;

  static object_ptr<MessageEffectType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageFileType: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageFileType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageFileTypePrivate final : public MessageFileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  messageFileTypePrivate();

  explicit messageFileTypePrivate(string const &name_);

  static const std::int32_t ID = -521908524;

  static object_ptr<MessageFileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageFileTypeGroup final : public MessageFileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;

  messageFileTypeGroup();

  explicit messageFileTypeGroup(string const &title_);

  static const std::int32_t ID = -219836568;

  static object_ptr<MessageFileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageFileTypeUnknown final : public MessageFileType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageFileTypeUnknown();

  static const std::int32_t ID = 1176353458;

  static object_ptr<MessageFileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageOrigin;

class forwardSource;

class messageForwardInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageOrigin> origin_;
  static jfieldID origin_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<forwardSource> source_;
  static jfieldID source_fieldID;
  string public_service_announcement_type_;
  static jfieldID public_service_announcement_type_fieldID;

  messageForwardInfo();

  messageForwardInfo(object_ptr<MessageOrigin> &&origin_, int32 date_, object_ptr<forwardSource> &&source_, string const &public_service_announcement_type_);

  static const std::int32_t ID = -880313475;

  static object_ptr<messageForwardInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageImportInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string sender_name_;
  static jfieldID sender_name_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  messageImportInfo();

  messageImportInfo(string const &sender_name_, int32 date_);

  static const std::int32_t ID = -421549105;

  static object_ptr<messageImportInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReactions;

class messageReplyInfo;

class messageInteractionInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 view_count_;
  static jfieldID view_count_fieldID;
  int32 forward_count_;
  static jfieldID forward_count_fieldID;
  object_ptr<messageReplyInfo> reply_info_;
  static jfieldID reply_info_fieldID;
  object_ptr<messageReactions> reactions_;
  static jfieldID reactions_fieldID;

  messageInteractionInfo();

  messageInteractionInfo(int32 view_count_, int32 forward_count_, object_ptr<messageReplyInfo> &&reply_info_, object_ptr<messageReactions> &&reactions_);

  static const std::int32_t ID = 733797893;

  static object_ptr<messageInteractionInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;
  bool is_public_;
  static jfieldID is_public_fieldID;

  messageLink();

  messageLink(string const &link_, bool is_public_);

  static const std::int32_t ID = -1354089818;

  static object_ptr<messageLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class messageLinkInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_public_;
  static jfieldID is_public_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  int32 media_timestamp_;
  static jfieldID media_timestamp_fieldID;
  bool for_album_;
  static jfieldID for_album_fieldID;

  messageLinkInfo();

  messageLinkInfo(bool is_public_, int53 chat_id_, int53 message_thread_id_, object_ptr<message> &&message_, int32 media_timestamp_, bool for_album_);

  static const std::int32_t ID = 731315024;

  static object_ptr<messageLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageOrigin: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageOrigin> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageOriginUser final : public MessageOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;

  messageOriginUser();

  explicit messageOriginUser(int53 sender_user_id_);

  static const std::int32_t ID = -1677684669;

  static object_ptr<MessageOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageOriginHiddenUser final : public MessageOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string sender_name_;
  static jfieldID sender_name_fieldID;

  messageOriginHiddenUser();

  explicit messageOriginHiddenUser(string const &sender_name_);

  static const std::int32_t ID = -317971494;

  static object_ptr<MessageOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageOriginChat final : public MessageOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sender_chat_id_;
  static jfieldID sender_chat_id_fieldID;
  string author_signature_;
  static jfieldID author_signature_fieldID;

  messageOriginChat();

  messageOriginChat(int53 sender_chat_id_, string const &author_signature_);

  static const std::int32_t ID = -205824332;

  static object_ptr<MessageOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageOriginChannel final : public MessageOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  string author_signature_;
  static jfieldID author_signature_fieldID;

  messageOriginChannel();

  messageOriginChannel(int53 chat_id_, int53 message_id_, string const &author_signature_);

  static const std::int32_t ID = -1451535938;

  static object_ptr<MessageOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePosition final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 position_;
  static jfieldID position_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  messagePosition();

  messagePosition(int32 position_, int53 message_id_, int32 date_);

  static const std::int32_t ID = 1292189935;

  static object_ptr<messagePosition> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messagePosition;

class messagePositions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<messagePosition>> positions_;
  static jfieldID positions_fieldID;

  messagePositions();

  messagePositions(int32 total_count_, array<object_ptr<messagePosition>> &&positions_);

  static const std::int32_t ID = -1930466649;

  static object_ptr<messagePositions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageProperties final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_be_copied_to_secret_chat_;
  static jfieldID can_be_copied_to_secret_chat_fieldID;
  bool can_be_deleted_only_for_self_;
  static jfieldID can_be_deleted_only_for_self_fieldID;
  bool can_be_deleted_for_all_users_;
  static jfieldID can_be_deleted_for_all_users_fieldID;
  bool can_be_edited_;
  static jfieldID can_be_edited_fieldID;
  bool can_be_forwarded_;
  static jfieldID can_be_forwarded_fieldID;
  bool can_be_paid_;
  static jfieldID can_be_paid_fieldID;
  bool can_be_pinned_;
  static jfieldID can_be_pinned_fieldID;
  bool can_be_replied_;
  static jfieldID can_be_replied_fieldID;
  bool can_be_replied_in_another_chat_;
  static jfieldID can_be_replied_in_another_chat_fieldID;
  bool can_be_saved_;
  static jfieldID can_be_saved_fieldID;
  bool can_be_shared_in_story_;
  static jfieldID can_be_shared_in_story_fieldID;
  bool can_edit_media_;
  static jfieldID can_edit_media_fieldID;
  bool can_edit_scheduling_state_;
  static jfieldID can_edit_scheduling_state_fieldID;
  bool can_get_embedding_code_;
  static jfieldID can_get_embedding_code_fieldID;
  bool can_get_link_;
  static jfieldID can_get_link_fieldID;
  bool can_get_media_timestamp_links_;
  static jfieldID can_get_media_timestamp_links_fieldID;
  bool can_get_message_thread_;
  static jfieldID can_get_message_thread_fieldID;
  bool can_get_read_date_;
  static jfieldID can_get_read_date_fieldID;
  bool can_get_statistics_;
  static jfieldID can_get_statistics_fieldID;
  bool can_get_viewers_;
  static jfieldID can_get_viewers_fieldID;
  bool can_recognize_speech_;
  static jfieldID can_recognize_speech_fieldID;
  bool can_report_chat_;
  static jfieldID can_report_chat_fieldID;
  bool can_report_reactions_;
  static jfieldID can_report_reactions_fieldID;
  bool can_report_supergroup_spam_;
  static jfieldID can_report_supergroup_spam_fieldID;
  bool can_set_fact_check_;
  static jfieldID can_set_fact_check_fieldID;
  bool need_show_statistics_;
  static jfieldID need_show_statistics_fieldID;

  messageProperties();

  messageProperties(bool can_be_copied_to_secret_chat_, bool can_be_deleted_only_for_self_, bool can_be_deleted_for_all_users_, bool can_be_edited_, bool can_be_forwarded_, bool can_be_paid_, bool can_be_pinned_, bool can_be_replied_, bool can_be_replied_in_another_chat_, bool can_be_saved_, bool can_be_shared_in_story_, bool can_edit_media_, bool can_edit_scheduling_state_, bool can_get_embedding_code_, bool can_get_link_, bool can_get_media_timestamp_links_, bool can_get_message_thread_, bool can_get_read_date_, bool can_get_statistics_, bool can_get_viewers_, bool can_recognize_speech_, bool can_report_chat_, bool can_report_reactions_, bool can_report_supergroup_spam_, bool can_set_fact_check_, bool need_show_statistics_);

  static const std::int32_t ID = -27014655;

  static object_ptr<messageProperties> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ReactionType;

class messageReaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> type_;
  static jfieldID type_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  bool is_chosen_;
  static jfieldID is_chosen_fieldID;
  object_ptr<MessageSender> used_sender_id_;
  static jfieldID used_sender_id_fieldID;
  array<object_ptr<MessageSender>> recent_sender_ids_;
  static jfieldID recent_sender_ids_fieldID;

  messageReaction();

  messageReaction(object_ptr<ReactionType> &&type_, int32 total_count_, bool is_chosen_, object_ptr<MessageSender> &&used_sender_id_, array<object_ptr<MessageSender>> &&recent_sender_ids_);

  static const std::int32_t ID = -1093994369;

  static object_ptr<messageReaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReaction;

class paidReactor;

class messageReactions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<messageReaction>> reactions_;
  static jfieldID reactions_fieldID;
  bool are_tags_;
  static jfieldID are_tags_fieldID;
  array<object_ptr<paidReactor>> paid_reactors_;
  static jfieldID paid_reactors_fieldID;
  bool can_get_added_reactions_;
  static jfieldID can_get_added_reactions_fieldID;

  messageReactions();

  messageReactions(array<object_ptr<messageReaction>> &&reactions_, bool are_tags_, array<object_ptr<paidReactor>> &&paid_reactors_, bool can_get_added_reactions_);

  static const std::int32_t ID = 1475966817;

  static object_ptr<messageReactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageReadDate: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageReadDateRead final : public MessageReadDate {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 read_date_;
  static jfieldID read_date_fieldID;

  messageReadDateRead();

  explicit messageReadDateRead(int32 read_date_);

  static const std::int32_t ID = 1972186672;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReadDateUnread final : public MessageReadDate {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageReadDateUnread();

  static const std::int32_t ID = 397549868;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReadDateTooOld final : public MessageReadDate {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageReadDateTooOld();

  static const std::int32_t ID = -1233773024;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReadDateUserPrivacyRestricted final : public MessageReadDate {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageReadDateUserPrivacyRestricted();

  static const std::int32_t ID = -1282567130;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReadDateMyPrivacyRestricted final : public MessageReadDate {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageReadDateMyPrivacyRestricted();

  static const std::int32_t ID = -693971852;

  static object_ptr<MessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class messageReplyInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 reply_count_;
  static jfieldID reply_count_fieldID;
  array<object_ptr<MessageSender>> recent_replier_ids_;
  static jfieldID recent_replier_ids_fieldID;
  int53 last_read_inbox_message_id_;
  static jfieldID last_read_inbox_message_id_fieldID;
  int53 last_read_outbox_message_id_;
  static jfieldID last_read_outbox_message_id_fieldID;
  int53 last_message_id_;
  static jfieldID last_message_id_fieldID;

  messageReplyInfo();

  messageReplyInfo(int32 reply_count_, array<object_ptr<MessageSender>> &&recent_replier_ids_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, int53 last_message_id_);

  static const std::int32_t ID = -2093702263;

  static object_ptr<messageReplyInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageContent;

class MessageOrigin;

class textQuote;

class MessageReplyTo: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageReplyTo> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageReplyToMessage final : public MessageReplyTo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<textQuote> quote_;
  static jfieldID quote_fieldID;
  object_ptr<MessageOrigin> origin_;
  static jfieldID origin_fieldID;
  int32 origin_send_date_;
  static jfieldID origin_send_date_fieldID;
  object_ptr<MessageContent> content_;
  static jfieldID content_fieldID;

  messageReplyToMessage();

  messageReplyToMessage(int53 chat_id_, int53 message_id_, object_ptr<textQuote> &&quote_, object_ptr<MessageOrigin> &&origin_, int32 origin_send_date_, object_ptr<MessageContent> &&content_);

  static const std::int32_t ID = -300918393;

  static object_ptr<MessageReplyTo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageReplyToStory final : public MessageReplyTo {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  messageReplyToStory();

  messageReplyToStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -674492596;

  static object_ptr<MessageReplyTo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSchedulingState: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageSchedulingState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSchedulingStateSendAtDate final : public MessageSchedulingState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 send_date_;
  static jfieldID send_date_fieldID;

  messageSchedulingStateSendAtDate();

  explicit messageSchedulingStateSendAtDate(int32 send_date_);

  static const std::int32_t ID = -1485570073;

  static object_ptr<MessageSchedulingState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSchedulingStateSendWhenOnline final : public MessageSchedulingState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSchedulingStateSendWhenOnline();

  static const std::int32_t ID = 2092947464;

  static object_ptr<MessageSchedulingState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSchedulingStateSendWhenVideoProcessed final : public MessageSchedulingState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 send_date_;
  static jfieldID send_date_fieldID;

  messageSchedulingStateSendWhenVideoProcessed();

  explicit messageSchedulingStateSendWhenVideoProcessed(int32 send_date_);

  static const std::int32_t ID = 2101578734;

  static object_ptr<MessageSchedulingState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSelfDestructType: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageSelfDestructType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSelfDestructTypeTimer final : public MessageSelfDestructType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 self_destruct_time_;
  static jfieldID self_destruct_time_fieldID;

  messageSelfDestructTypeTimer();

  explicit messageSelfDestructTypeTimer(int32 self_destruct_time_);

  static const std::int32_t ID = 1351440333;

  static object_ptr<MessageSelfDestructType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSelfDestructTypeImmediately final : public MessageSelfDestructType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSelfDestructTypeImmediately();

  static const std::int32_t ID = -1036218363;

  static object_ptr<MessageSelfDestructType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSchedulingState;

class messageSendOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;
  bool from_background_;
  static jfieldID from_background_fieldID;
  bool protect_content_;
  static jfieldID protect_content_fieldID;
  bool allow_paid_broadcast_;
  static jfieldID allow_paid_broadcast_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;
  bool update_order_of_installed_sticker_sets_;
  static jfieldID update_order_of_installed_sticker_sets_fieldID;
  object_ptr<MessageSchedulingState> scheduling_state_;
  static jfieldID scheduling_state_fieldID;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;
  int32 sending_id_;
  static jfieldID sending_id_fieldID;
  bool only_preview_;
  static jfieldID only_preview_fieldID;

  messageSendOptions();

  messageSendOptions(bool disable_notification_, bool from_background_, bool protect_content_, bool allow_paid_broadcast_, int53 paid_message_star_count_, bool update_order_of_installed_sticker_sets_, object_ptr<MessageSchedulingState> &&scheduling_state_, int64 effect_id_, int32 sending_id_, bool only_preview_);

  static const std::int32_t ID = -128496262;

  static object_ptr<messageSendOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageSender> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSenderUser final : public MessageSender {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  messageSenderUser();

  explicit messageSenderUser(int53 user_id_);

  static const std::int32_t ID = -336109341;

  static object_ptr<MessageSender> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSenderChat final : public MessageSender {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  messageSenderChat();

  explicit messageSenderChat(int53 chat_id_);

  static const std::int32_t ID = -239660751;

  static object_ptr<MessageSender> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class messageSenders final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<MessageSender>> senders_;
  static jfieldID senders_fieldID;

  messageSenders();

  messageSenders(int32 total_count_, array<object_ptr<MessageSender>> &&senders_);

  static const std::int32_t ID = -690158467;

  static object_ptr<messageSenders> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class error;

class MessageSendingState: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageSendingState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSendingStatePending final : public MessageSendingState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 sending_id_;
  static jfieldID sending_id_fieldID;

  messageSendingStatePending();

  explicit messageSendingStatePending(int32 sending_id_);

  static const std::int32_t ID = -215260236;

  static object_ptr<MessageSendingState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSendingStateFailed final : public MessageSendingState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<error> error_;
  static jfieldID error_fieldID;
  bool can_retry_;
  static jfieldID can_retry_fieldID;
  bool need_another_sender_;
  static jfieldID need_another_sender_fieldID;
  bool need_another_reply_quote_;
  static jfieldID need_another_reply_quote_fieldID;
  bool need_drop_reply_;
  static jfieldID need_drop_reply_fieldID;
  int53 required_paid_message_star_count_;
  static jfieldID required_paid_message_star_count_fieldID;
  double retry_after_;
  static jfieldID retry_after_fieldID;

  messageSendingStateFailed();

  messageSendingStateFailed(object_ptr<error> &&error_, bool can_retry_, bool need_another_sender_, bool need_another_reply_quote_, bool need_drop_reply_, int53 required_paid_message_star_count_, double retry_after_);

  static const std::int32_t ID = -777630522;

  static object_ptr<MessageSendingState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSource: public Object {
 public:
  static jclass Class;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceChatHistory final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceChatHistory();

  static const std::int32_t ID = -1090386116;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceMessageThreadHistory final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceMessageThreadHistory();

  static const std::int32_t ID = 290427142;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceForumTopicHistory final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceForumTopicHistory();

  static const std::int32_t ID = -1518064457;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceHistoryPreview final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceHistoryPreview();

  static const std::int32_t ID = 1024254993;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceChatList final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceChatList();

  static const std::int32_t ID = -2047406102;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceSearch final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceSearch();

  static const std::int32_t ID = 1921333105;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceChatEventLog final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceChatEventLog();

  static const std::int32_t ID = -1028777540;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceNotification final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceNotification();

  static const std::int32_t ID = -1046406163;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceScreenshot final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceScreenshot();

  static const std::int32_t ID = 469982474;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageSourceOther final : public MessageSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  messageSourceOther();

  static const std::int32_t ID = 901818114;

  static object_ptr<MessageSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class messageSponsor final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  string info_;
  static jfieldID info_fieldID;

  messageSponsor();

  messageSponsor(string const &url_, object_ptr<photo> &&photo_, string const &info_);

  static const std::int32_t ID = 2009223646;

  static object_ptr<messageSponsor> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class messageStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StatisticalGraph> message_interaction_graph_;
  static jfieldID message_interaction_graph_fieldID;
  object_ptr<StatisticalGraph> message_reaction_graph_;
  static jfieldID message_reaction_graph_fieldID;

  messageStatistics();

  messageStatistics(object_ptr<StatisticalGraph> &&message_interaction_graph_, object_ptr<StatisticalGraph> &&message_reaction_graph_);

  static const std::int32_t ID = -1563537657;

  static object_ptr<messageStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class draftMessage;

class message;

class messageReplyInfo;

class messageThreadInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<messageReplyInfo> reply_info_;
  static jfieldID reply_info_fieldID;
  int32 unread_message_count_;
  static jfieldID unread_message_count_fieldID;
  array<object_ptr<message>> messages_;
  static jfieldID messages_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;

  messageThreadInfo();

  messageThreadInfo(int53 chat_id_, int53 message_thread_id_, object_ptr<messageReplyInfo> &&reply_info_, int32 unread_message_count_, array<object_ptr<message>> &&messages_, object_ptr<draftMessage> &&draft_message_);

  static const std::int32_t ID = -248536056;

  static object_ptr<messageThreadInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageViewer final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 view_date_;
  static jfieldID view_date_fieldID;

  messageViewer();

  messageViewer(int53 user_id_, int32 view_date_);

  static const std::int32_t ID = 1458639309;

  static object_ptr<messageViewer> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class messageViewer;

class messageViewers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<messageViewer>> viewers_;
  static jfieldID viewers_fieldID;

  messageViewers();

  explicit messageViewers(array<object_ptr<messageViewer>> &&viewers_);

  static const std::int32_t ID = 2116480287;

  static object_ptr<messageViewers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class messages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<message>> messages_;
  static jfieldID messages_fieldID;

  messages();

  messages(int32 total_count_, array<object_ptr<message>> &&messages_);

  static const std::int32_t ID = -16498159;

  static object_ptr<messages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class minithumbnail final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  minithumbnail();

  minithumbnail(int32 width_, int32 height_, bytes const &data_);

  static const std::int32_t ID = -328540758;

  static object_ptr<minithumbnail> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NetworkStatisticsEntry;

class networkStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 since_date_;
  static jfieldID since_date_fieldID;
  array<object_ptr<NetworkStatisticsEntry>> entries_;
  static jfieldID entries_fieldID;

  networkStatistics();

  networkStatistics(int32 since_date_, array<object_ptr<NetworkStatisticsEntry>> &&entries_);

  static const std::int32_t ID = 1615554212;

  static object_ptr<networkStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FileType;

class NetworkType;

class NetworkStatisticsEntry: public Object {
 public:
  static jclass Class;

  static object_ptr<NetworkStatisticsEntry> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class networkStatisticsEntryFile final : public NetworkStatisticsEntry {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<FileType> file_type_;
  static jfieldID file_type_fieldID;
  object_ptr<NetworkType> network_type_;
  static jfieldID network_type_fieldID;
  int53 sent_bytes_;
  static jfieldID sent_bytes_fieldID;
  int53 received_bytes_;
  static jfieldID received_bytes_fieldID;

  networkStatisticsEntryFile();

  networkStatisticsEntryFile(object_ptr<FileType> &&file_type_, object_ptr<NetworkType> &&network_type_, int53 sent_bytes_, int53 received_bytes_);

  static const std::int32_t ID = 188452706;

  static object_ptr<NetworkStatisticsEntry> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class networkStatisticsEntryCall final : public NetworkStatisticsEntry {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NetworkType> network_type_;
  static jfieldID network_type_fieldID;
  int53 sent_bytes_;
  static jfieldID sent_bytes_fieldID;
  int53 received_bytes_;
  static jfieldID received_bytes_fieldID;
  double duration_;
  static jfieldID duration_fieldID;

  networkStatisticsEntryCall();

  networkStatisticsEntryCall(object_ptr<NetworkType> &&network_type_, int53 sent_bytes_, int53 received_bytes_, double duration_);

  static const std::int32_t ID = 737000365;

  static object_ptr<NetworkStatisticsEntry> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NetworkType: public Object {
 public:
  static jclass Class;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class networkTypeNone final : public NetworkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  networkTypeNone();

  static const std::int32_t ID = -1971691759;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class networkTypeMobile final : public NetworkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  networkTypeMobile();

  static const std::int32_t ID = 819228239;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class networkTypeMobileRoaming final : public NetworkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  networkTypeMobileRoaming();

  static const std::int32_t ID = -1435199760;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class networkTypeWiFi final : public NetworkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  networkTypeWiFi();

  static const std::int32_t ID = -633872070;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class networkTypeOther final : public NetworkType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  networkTypeOther();

  static const std::int32_t ID = 1942128539;

  static object_ptr<NetworkType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class newChatPrivacySettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_new_chats_from_unknown_users_;
  static jfieldID allow_new_chats_from_unknown_users_fieldID;
  int53 incoming_paid_message_star_count_;
  static jfieldID incoming_paid_message_star_count_fieldID;

  newChatPrivacySettings();

  newChatPrivacySettings(bool allow_new_chats_from_unknown_users_, int53 incoming_paid_message_star_count_);

  static const std::int32_t ID = 123716192;

  static object_ptr<newChatPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NotificationType;

class notification final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  bool is_silent_;
  static jfieldID is_silent_fieldID;
  object_ptr<NotificationType> type_;
  static jfieldID type_fieldID;

  notification();

  notification(int32 id_, int32 date_, bool is_silent_, object_ptr<NotificationType> &&type_);

  static const std::int32_t ID = 788743120;

  static object_ptr<notification> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NotificationGroupType;

class notification;

class notificationGroup final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  object_ptr<NotificationGroupType> type_;
  static jfieldID type_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<notification>> notifications_;
  static jfieldID notifications_fieldID;

  notificationGroup();

  notificationGroup(int32 id_, object_ptr<NotificationGroupType> &&type_, int53 chat_id_, int32 total_count_, array<object_ptr<notification>> &&notifications_);

  static const std::int32_t ID = 780691541;

  static object_ptr<notificationGroup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NotificationGroupType: public Object {
 public:
  static jclass Class;

  static object_ptr<NotificationGroupType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class notificationGroupTypeMessages final : public NotificationGroupType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationGroupTypeMessages();

  static const std::int32_t ID = -1702481123;

  static object_ptr<NotificationGroupType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationGroupTypeMentions final : public NotificationGroupType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationGroupTypeMentions();

  static const std::int32_t ID = -2050324051;

  static object_ptr<NotificationGroupType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationGroupTypeSecretChat final : public NotificationGroupType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationGroupTypeSecretChat();

  static const std::int32_t ID = 1390759476;

  static object_ptr<NotificationGroupType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationGroupTypeCalls final : public NotificationGroupType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationGroupTypeCalls();

  static const std::int32_t ID = 1379123538;

  static object_ptr<NotificationGroupType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class NotificationSettingsScope: public Object {
 public:
  static jclass Class;

  static object_ptr<NotificationSettingsScope> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class notificationSettingsScopePrivateChats final : public NotificationSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationSettingsScopePrivateChats();

  static const std::int32_t ID = 937446759;

  static object_ptr<NotificationSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationSettingsScopeGroupChats final : public NotificationSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationSettingsScopeGroupChats();

  static const std::int32_t ID = 1212142067;

  static object_ptr<NotificationSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationSettingsScopeChannelChats final : public NotificationSettingsScope {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationSettingsScopeChannelChats();

  static const std::int32_t ID = 548013448;

  static object_ptr<NotificationSettingsScope> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class notificationSound final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string data_;
  static jfieldID data_fieldID;
  object_ptr<file> sound_;
  static jfieldID sound_fieldID;

  notificationSound();

  notificationSound(int64 id_, int32 duration_, int32 date_, string const &title_, string const &data_, object_ptr<file> &&sound_);

  static const std::int32_t ID = -185638601;

  static object_ptr<notificationSound> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationSound;

class notificationSounds final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<notificationSound>> notification_sounds_;
  static jfieldID notification_sounds_fieldID;

  notificationSounds();

  explicit notificationSounds(array<object_ptr<notificationSound>> &&notification_sounds_);

  static const std::int32_t ID = -630813169;

  static object_ptr<notificationSounds> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class PushMessageContent;

class message;

class NotificationType: public Object {
 public:
  static jclass Class;

  static object_ptr<NotificationType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class notificationTypeNewMessage final : public NotificationType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  bool show_preview_;
  static jfieldID show_preview_fieldID;

  notificationTypeNewMessage();

  notificationTypeNewMessage(object_ptr<message> &&message_, bool show_preview_);

  static const std::int32_t ID = -254745614;

  static object_ptr<NotificationType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationTypeNewSecretChat final : public NotificationType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  notificationTypeNewSecretChat();

  static const std::int32_t ID = 1198638768;

  static object_ptr<NotificationType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationTypeNewCall final : public NotificationType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;

  notificationTypeNewCall();

  explicit notificationTypeNewCall(int32 call_id_);

  static const std::int32_t ID = 1712734585;

  static object_ptr<NotificationType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class notificationTypeNewPushMessage final : public NotificationType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  string sender_name_;
  static jfieldID sender_name_fieldID;
  bool is_outgoing_;
  static jfieldID is_outgoing_fieldID;
  object_ptr<PushMessageContent> content_;
  static jfieldID content_fieldID;

  notificationTypeNewPushMessage();

  notificationTypeNewPushMessage(int53 message_id_, object_ptr<MessageSender> &&sender_id_, string const &sender_name_, bool is_outgoing_, object_ptr<PushMessageContent> &&content_);

  static const std::int32_t ID = -711680462;

  static object_ptr<NotificationType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ok final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  ok();

  static const std::int32_t ID = -722616727;

  static object_ptr<ok> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class OptionValue: public Object {
 public:
  static jclass Class;

  static object_ptr<OptionValue> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class optionValueBoolean final : public OptionValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool value_;
  static jfieldID value_fieldID;

  optionValueBoolean();

  explicit optionValueBoolean(bool value_);

  static const std::int32_t ID = 63135518;

  static object_ptr<OptionValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class optionValueEmpty final : public OptionValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  optionValueEmpty();

  static const std::int32_t ID = 918955155;

  static object_ptr<OptionValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class optionValueInteger final : public OptionValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 value_;
  static jfieldID value_fieldID;

  optionValueInteger();

  explicit optionValueInteger(int64 value_);

  static const std::int32_t ID = -186858780;

  static object_ptr<OptionValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class optionValueString final : public OptionValue {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string value_;
  static jfieldID value_fieldID;

  optionValueString();

  explicit optionValueString(string const &value_);

  static const std::int32_t ID = 756248212;

  static object_ptr<OptionValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class address;

class orderInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  string email_address_;
  static jfieldID email_address_fieldID;
  object_ptr<address> shipping_address_;
  static jfieldID shipping_address_fieldID;

  orderInfo();

  orderInfo(string const &name_, string const &phone_number_, string const &email_address_, object_ptr<address> &&shipping_address_);

  static const std::int32_t ID = 783997294;

  static object_ptr<orderInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class closedVectorPath;

class outline final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<closedVectorPath>> paths_;
  static jfieldID paths_fieldID;

  outline();

  explicit outline(array<object_ptr<closedVectorPath>> &&paths_);

  static const std::int32_t ID = -161506702;

  static object_ptr<outline> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PageBlock;

class RichText;

class animation;

class audio;

class chatPhotoInfo;

class location;

class pageBlockCaption;

class pageBlockListItem;

class pageBlockRelatedArticle;

class pageBlockTableCell;

class photo;

class video;

class voiceNote;

class PageBlock: public Object {
 public:
  static jclass Class;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockTitle final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> title_;
  static jfieldID title_fieldID;

  pageBlockTitle();

  explicit pageBlockTitle(object_ptr<RichText> &&title_);

  static const std::int32_t ID = 1629664784;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockSubtitle final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> subtitle_;
  static jfieldID subtitle_fieldID;

  pageBlockSubtitle();

  explicit pageBlockSubtitle(object_ptr<RichText> &&subtitle_);

  static const std::int32_t ID = 264524263;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockAuthorDate final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> author_;
  static jfieldID author_fieldID;
  int32 publish_date_;
  static jfieldID publish_date_fieldID;

  pageBlockAuthorDate();

  pageBlockAuthorDate(object_ptr<RichText> &&author_, int32 publish_date_);

  static const std::int32_t ID = 1300231184;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockHeader final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> header_;
  static jfieldID header_fieldID;

  pageBlockHeader();

  explicit pageBlockHeader(object_ptr<RichText> &&header_);

  static const std::int32_t ID = 1402854811;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockSubheader final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> subheader_;
  static jfieldID subheader_fieldID;

  pageBlockSubheader();

  explicit pageBlockSubheader(object_ptr<RichText> &&subheader_);

  static const std::int32_t ID = 1263956774;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockKicker final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> kicker_;
  static jfieldID kicker_fieldID;

  pageBlockKicker();

  explicit pageBlockKicker(object_ptr<RichText> &&kicker_);

  static const std::int32_t ID = 1361282635;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockParagraph final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  pageBlockParagraph();

  explicit pageBlockParagraph(object_ptr<RichText> &&text_);

  static const std::int32_t ID = 1182402406;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockPreformatted final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string language_;
  static jfieldID language_fieldID;

  pageBlockPreformatted();

  pageBlockPreformatted(object_ptr<RichText> &&text_, string const &language_);

  static const std::int32_t ID = -1066346178;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockFooter final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> footer_;
  static jfieldID footer_fieldID;

  pageBlockFooter();

  explicit pageBlockFooter(object_ptr<RichText> &&footer_);

  static const std::int32_t ID = 886429480;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockDivider final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockDivider();

  static const std::int32_t ID = -618614392;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockAnchor final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  pageBlockAnchor();

  explicit pageBlockAnchor(string const &name_);

  static const std::int32_t ID = -837994576;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockList final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<pageBlockListItem>> items_;
  static jfieldID items_fieldID;

  pageBlockList();

  explicit pageBlockList(array<object_ptr<pageBlockListItem>> &&items_);

  static const std::int32_t ID = -1037074852;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockBlockQuote final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  object_ptr<RichText> credit_;
  static jfieldID credit_fieldID;

  pageBlockBlockQuote();

  pageBlockBlockQuote(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_);

  static const std::int32_t ID = 1657834142;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockPullQuote final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  object_ptr<RichText> credit_;
  static jfieldID credit_fieldID;

  pageBlockPullQuote();

  pageBlockPullQuote(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_);

  static const std::int32_t ID = 490242317;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockAnimation final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;
  bool need_autoplay_;
  static jfieldID need_autoplay_fieldID;

  pageBlockAnimation();

  pageBlockAnimation(object_ptr<animation> &&animation_, object_ptr<pageBlockCaption> &&caption_, bool need_autoplay_);

  static const std::int32_t ID = 1355669513;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockAudio final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<audio> audio_;
  static jfieldID audio_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockAudio();

  pageBlockAudio(object_ptr<audio> &&audio_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = -63371245;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockPhoto final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;
  string url_;
  static jfieldID url_fieldID;

  pageBlockPhoto();

  pageBlockPhoto(object_ptr<photo> &&photo_, object_ptr<pageBlockCaption> &&caption_, string const &url_);

  static const std::int32_t ID = 417601156;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockVideo final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;
  bool need_autoplay_;
  static jfieldID need_autoplay_fieldID;
  bool is_looped_;
  static jfieldID is_looped_fieldID;

  pageBlockVideo();

  pageBlockVideo(object_ptr<video> &&video_, object_ptr<pageBlockCaption> &&caption_, bool need_autoplay_, bool is_looped_);

  static const std::int32_t ID = 510041394;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockVoiceNote final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<voiceNote> voice_note_;
  static jfieldID voice_note_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockVoiceNote();

  pageBlockVoiceNote(object_ptr<voiceNote> &&voice_note_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = 1823310463;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockCover final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PageBlock> cover_;
  static jfieldID cover_fieldID;

  pageBlockCover();

  explicit pageBlockCover(object_ptr<PageBlock> &&cover_);

  static const std::int32_t ID = 972174080;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockEmbedded final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string html_;
  static jfieldID html_fieldID;
  object_ptr<photo> poster_photo_;
  static jfieldID poster_photo_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;
  bool is_full_width_;
  static jfieldID is_full_width_fieldID;
  bool allow_scrolling_;
  static jfieldID allow_scrolling_fieldID;

  pageBlockEmbedded();

  pageBlockEmbedded(string const &url_, string const &html_, object_ptr<photo> &&poster_photo_, int32 width_, int32 height_, object_ptr<pageBlockCaption> &&caption_, bool is_full_width_, bool allow_scrolling_);

  static const std::int32_t ID = -1942577763;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockEmbeddedPost final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string author_;
  static jfieldID author_fieldID;
  object_ptr<photo> author_photo_;
  static jfieldID author_photo_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockEmbeddedPost();

  pageBlockEmbeddedPost(string const &url_, string const &author_, object_ptr<photo> &&author_photo_, int32 date_, array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = 397600949;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockCollage final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockCollage();

  pageBlockCollage(array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = 1163760110;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockSlideshow final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockSlideshow();

  pageBlockSlideshow(array<object_ptr<PageBlock>> &&page_blocks_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = 539217375;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockChatLink final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<chatPhotoInfo> photo_;
  static jfieldID photo_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  string username_;
  static jfieldID username_fieldID;

  pageBlockChatLink();

  pageBlockChatLink(string const &title_, object_ptr<chatPhotoInfo> &&photo_, int32 accent_color_id_, string const &username_);

  static const std::int32_t ID = 1646188731;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockTable final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> caption_;
  static jfieldID caption_fieldID;
  array<array<object_ptr<pageBlockTableCell>>> cells_;
  static jfieldID cells_fieldID;
  bool is_bordered_;
  static jfieldID is_bordered_fieldID;
  bool is_striped_;
  static jfieldID is_striped_fieldID;

  pageBlockTable();

  pageBlockTable(object_ptr<RichText> &&caption_, array<array<object_ptr<pageBlockTableCell>>> &&cells_, bool is_bordered_, bool is_striped_);

  static const std::int32_t ID = -942649288;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockDetails final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> header_;
  static jfieldID header_fieldID;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;
  bool is_open_;
  static jfieldID is_open_fieldID;

  pageBlockDetails();

  pageBlockDetails(object_ptr<RichText> &&header_, array<object_ptr<PageBlock>> &&page_blocks_, bool is_open_);

  static const std::int32_t ID = -1599869809;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockRelatedArticles final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> header_;
  static jfieldID header_fieldID;
  array<object_ptr<pageBlockRelatedArticle>> articles_;
  static jfieldID articles_fieldID;

  pageBlockRelatedArticles();

  pageBlockRelatedArticles(object_ptr<RichText> &&header_, array<object_ptr<pageBlockRelatedArticle>> &&articles_);

  static const std::int32_t ID = -1807324374;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockMap final : public PageBlock {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 zoom_;
  static jfieldID zoom_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  object_ptr<pageBlockCaption> caption_;
  static jfieldID caption_fieldID;

  pageBlockMap();

  pageBlockMap(object_ptr<location> &&location_, int32 zoom_, int32 width_, int32 height_, object_ptr<pageBlockCaption> &&caption_);

  static const std::int32_t ID = 1510961171;

  static object_ptr<PageBlock> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class RichText;

class pageBlockCaption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  object_ptr<RichText> credit_;
  static jfieldID credit_fieldID;

  pageBlockCaption();

  pageBlockCaption(object_ptr<RichText> &&text_, object_ptr<RichText> &&credit_);

  static const std::int32_t ID = -1180064650;

  static object_ptr<pageBlockCaption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PageBlockHorizontalAlignment: public Object {
 public:
  static jclass Class;

  static object_ptr<PageBlockHorizontalAlignment> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockHorizontalAlignmentLeft final : public PageBlockHorizontalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockHorizontalAlignmentLeft();

  static const std::int32_t ID = 848701417;

  static object_ptr<PageBlockHorizontalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockHorizontalAlignmentCenter final : public PageBlockHorizontalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockHorizontalAlignmentCenter();

  static const std::int32_t ID = -1009203990;

  static object_ptr<PageBlockHorizontalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockHorizontalAlignmentRight final : public PageBlockHorizontalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockHorizontalAlignmentRight();

  static const std::int32_t ID = 1371369214;

  static object_ptr<PageBlockHorizontalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PageBlock;

class pageBlockListItem final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string label_;
  static jfieldID label_fieldID;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;

  pageBlockListItem();

  pageBlockListItem(string const &label_, array<object_ptr<PageBlock>> &&page_blocks_);

  static const std::int32_t ID = 323186259;

  static object_ptr<pageBlockListItem> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class pageBlockRelatedArticle final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  string author_;
  static jfieldID author_fieldID;
  int32 publish_date_;
  static jfieldID publish_date_fieldID;

  pageBlockRelatedArticle();

  pageBlockRelatedArticle(string const &url_, string const &title_, string const &description_, object_ptr<photo> &&photo_, string const &author_, int32 publish_date_);

  static const std::int32_t ID = 481199251;

  static object_ptr<pageBlockRelatedArticle> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PageBlockHorizontalAlignment;

class PageBlockVerticalAlignment;

class RichText;

class pageBlockTableCell final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  bool is_header_;
  static jfieldID is_header_fieldID;
  int32 colspan_;
  static jfieldID colspan_fieldID;
  int32 rowspan_;
  static jfieldID rowspan_fieldID;
  object_ptr<PageBlockHorizontalAlignment> align_;
  static jfieldID align_fieldID;
  object_ptr<PageBlockVerticalAlignment> valign_;
  static jfieldID valign_fieldID;

  pageBlockTableCell();

  pageBlockTableCell(object_ptr<RichText> &&text_, bool is_header_, int32 colspan_, int32 rowspan_, object_ptr<PageBlockHorizontalAlignment> &&align_, object_ptr<PageBlockVerticalAlignment> &&valign_);

  static const std::int32_t ID = 1417658214;

  static object_ptr<pageBlockTableCell> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PageBlockVerticalAlignment: public Object {
 public:
  static jclass Class;

  static object_ptr<PageBlockVerticalAlignment> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockVerticalAlignmentTop final : public PageBlockVerticalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockVerticalAlignmentTop();

  static const std::int32_t ID = 195500454;

  static object_ptr<PageBlockVerticalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockVerticalAlignmentMiddle final : public PageBlockVerticalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockVerticalAlignmentMiddle();

  static const std::int32_t ID = -2123096587;

  static object_ptr<PageBlockVerticalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pageBlockVerticalAlignmentBottom final : public PageBlockVerticalAlignment {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pageBlockVerticalAlignmentBottom();

  static const std::int32_t ID = 2092531158;

  static object_ptr<PageBlockVerticalAlignment> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class minithumbnail;

class photo;

class video;

class PaidMedia: public Object {
 public:
  static jclass Class;

  static object_ptr<PaidMedia> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paidMediaPreview final : public PaidMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;

  paidMediaPreview();

  paidMediaPreview(int32 width_, int32 height_, int32 duration_, object_ptr<minithumbnail> &&minithumbnail_);

  static const std::int32_t ID = -1128151948;

  static object_ptr<PaidMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paidMediaPhoto final : public PaidMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  paidMediaPhoto();

  explicit paidMediaPhoto(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -1165863654;

  static object_ptr<PaidMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paidMediaVideo final : public PaidMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  object_ptr<photo> cover_;
  static jfieldID cover_fieldID;
  int32 start_timestamp_;
  static jfieldID start_timestamp_fieldID;

  paidMediaVideo();

  paidMediaVideo(object_ptr<video> &&video_, object_ptr<photo> &&cover_, int32 start_timestamp_);

  static const std::int32_t ID = 870838318;

  static object_ptr<PaidMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paidMediaUnsupported final : public PaidMedia {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  paidMediaUnsupported();

  static const std::int32_t ID = 112999974;

  static object_ptr<PaidMedia> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PaidReactionType: public Object {
 public:
  static jclass Class;

  static object_ptr<PaidReactionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paidReactionTypeRegular final : public PaidReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  paidReactionTypeRegular();

  static const std::int32_t ID = -1199187333;

  static object_ptr<PaidReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paidReactionTypeAnonymous final : public PaidReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  paidReactionTypeAnonymous();

  static const std::int32_t ID = 47892621;

  static object_ptr<PaidReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paidReactionTypeChat final : public PaidReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  paidReactionTypeChat();

  explicit paidReactionTypeChat(int53 chat_id_);

  static const std::int32_t ID = -675782044;

  static object_ptr<PaidReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class paidReactor final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  int32 star_count_;
  static jfieldID star_count_fieldID;
  bool is_top_;
  static jfieldID is_top_fieldID;
  bool is_me_;
  static jfieldID is_me_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;

  paidReactor();

  paidReactor(object_ptr<MessageSender> &&sender_id_, int32 star_count_, bool is_top_, bool is_me_, bool is_anonymous_);

  static const std::int32_t ID = -1657303032;

  static object_ptr<paidReactor> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportRequiredElement;

class passportAuthorizationForm final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  array<object_ptr<passportRequiredElement>> required_elements_;
  static jfieldID required_elements_fieldID;
  string privacy_policy_url_;
  static jfieldID privacy_policy_url_fieldID;

  passportAuthorizationForm();

  passportAuthorizationForm(int32 id_, array<object_ptr<passportRequiredElement>> &&required_elements_, string const &privacy_policy_url_);

  static const std::int32_t ID = -1070673218;

  static object_ptr<passportAuthorizationForm> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class address;

class identityDocument;

class personalDetails;

class personalDocument;

class PassportElement: public Object {
 public:
  static jclass Class;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportElementPersonalDetails final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDetails> personal_details_;
  static jfieldID personal_details_fieldID;

  passportElementPersonalDetails();

  explicit passportElementPersonalDetails(object_ptr<personalDetails> &&personal_details_);

  static const std::int32_t ID = 1217724035;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementPassport final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<identityDocument> passport_;
  static jfieldID passport_fieldID;

  passportElementPassport();

  explicit passportElementPassport(object_ptr<identityDocument> &&passport_);

  static const std::int32_t ID = -263985373;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementDriverLicense final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<identityDocument> driver_license_;
  static jfieldID driver_license_fieldID;

  passportElementDriverLicense();

  explicit passportElementDriverLicense(object_ptr<identityDocument> &&driver_license_);

  static const std::int32_t ID = 1643580589;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementIdentityCard final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<identityDocument> identity_card_;
  static jfieldID identity_card_fieldID;

  passportElementIdentityCard();

  explicit passportElementIdentityCard(object_ptr<identityDocument> &&identity_card_);

  static const std::int32_t ID = 2083775797;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementInternalPassport final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<identityDocument> internal_passport_;
  static jfieldID internal_passport_fieldID;

  passportElementInternalPassport();

  explicit passportElementInternalPassport(object_ptr<identityDocument> &&internal_passport_);

  static const std::int32_t ID = 36220295;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementAddress final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<address> address_;
  static jfieldID address_fieldID;

  passportElementAddress();

  explicit passportElementAddress(object_ptr<address> &&address_);

  static const std::int32_t ID = -782625232;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementUtilityBill final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDocument> utility_bill_;
  static jfieldID utility_bill_fieldID;

  passportElementUtilityBill();

  explicit passportElementUtilityBill(object_ptr<personalDocument> &&utility_bill_);

  static const std::int32_t ID = -234611246;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementBankStatement final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDocument> bank_statement_;
  static jfieldID bank_statement_fieldID;

  passportElementBankStatement();

  explicit passportElementBankStatement(object_ptr<personalDocument> &&bank_statement_);

  static const std::int32_t ID = -366464408;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementRentalAgreement final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDocument> rental_agreement_;
  static jfieldID rental_agreement_fieldID;

  passportElementRentalAgreement();

  explicit passportElementRentalAgreement(object_ptr<personalDocument> &&rental_agreement_);

  static const std::int32_t ID = -290141400;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementPassportRegistration final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDocument> passport_registration_;
  static jfieldID passport_registration_fieldID;

  passportElementPassportRegistration();

  explicit passportElementPassportRegistration(object_ptr<personalDocument> &&passport_registration_);

  static const std::int32_t ID = 618323071;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTemporaryRegistration final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<personalDocument> temporary_registration_;
  static jfieldID temporary_registration_fieldID;

  passportElementTemporaryRegistration();

  explicit passportElementTemporaryRegistration(object_ptr<personalDocument> &&temporary_registration_);

  static const std::int32_t ID = 1237626864;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementPhoneNumber final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;

  passportElementPhoneNumber();

  explicit passportElementPhoneNumber(string const &phone_number_);

  static const std::int32_t ID = -1320118375;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementEmailAddress final : public PassportElement {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string email_address_;
  static jfieldID email_address_fieldID;

  passportElementEmailAddress();

  explicit passportElementEmailAddress(string const &email_address_);

  static const std::int32_t ID = -1528129531;

  static object_ptr<PassportElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementErrorSource;

class PassportElementType;

class passportElementError final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;
  string message_;
  static jfieldID message_fieldID;
  object_ptr<PassportElementErrorSource> source_;
  static jfieldID source_fieldID;

  passportElementError();

  passportElementError(object_ptr<PassportElementType> &&type_, string const &message_, object_ptr<PassportElementErrorSource> &&source_);

  static const std::int32_t ID = -1861902395;

  static object_ptr<passportElementError> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementErrorSource: public Object {
 public:
  static jclass Class;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceUnspecified final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceUnspecified();

  static const std::int32_t ID = -378320830;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceDataField final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string field_name_;
  static jfieldID field_name_fieldID;

  passportElementErrorSourceDataField();

  explicit passportElementErrorSourceDataField(string const &field_name_);

  static const std::int32_t ID = -308650776;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceFrontSide final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceFrontSide();

  static const std::int32_t ID = 1895658292;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceReverseSide final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceReverseSide();

  static const std::int32_t ID = 1918630391;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceSelfie final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceSelfie();

  static const std::int32_t ID = -797043672;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceTranslationFile final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_index_;
  static jfieldID file_index_fieldID;

  passportElementErrorSourceTranslationFile();

  explicit passportElementErrorSourceTranslationFile(int32 file_index_);

  static const std::int32_t ID = -689621228;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceTranslationFiles final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceTranslationFiles();

  static const std::int32_t ID = 581280796;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceFile final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_index_;
  static jfieldID file_index_fieldID;

  passportElementErrorSourceFile();

  explicit passportElementErrorSourceFile(int32 file_index_);

  static const std::int32_t ID = 2020358960;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementErrorSourceFiles final : public PassportElementErrorSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementErrorSourceFiles();

  static const std::int32_t ID = 1894164178;

  static object_ptr<PassportElementErrorSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementType: public Object {
 public:
  static jclass Class;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypePersonalDetails final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypePersonalDetails();

  static const std::int32_t ID = -1032136365;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypePassport final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypePassport();

  static const std::int32_t ID = -436360376;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeDriverLicense final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeDriverLicense();

  static const std::int32_t ID = 1827298379;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeIdentityCard final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeIdentityCard();

  static const std::int32_t ID = -502356132;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeInternalPassport final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeInternalPassport();

  static const std::int32_t ID = -793781959;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeAddress final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeAddress();

  static const std::int32_t ID = 496327874;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeUtilityBill final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeUtilityBill();

  static const std::int32_t ID = 627084906;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeBankStatement final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeBankStatement();

  static const std::int32_t ID = 574095667;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeRentalAgreement final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeRentalAgreement();

  static const std::int32_t ID = -2060583280;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypePassportRegistration final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypePassportRegistration();

  static const std::int32_t ID = -159478209;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeTemporaryRegistration final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeTemporaryRegistration();

  static const std::int32_t ID = 1092498527;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypePhoneNumber final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypePhoneNumber();

  static const std::int32_t ID = -995361172;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportElementTypeEmailAddress final : public PassportElementType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  passportElementTypeEmailAddress();

  static const std::int32_t ID = -79321405;

  static object_ptr<PassportElementType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElement;

class passportElements final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PassportElement>> elements_;
  static jfieldID elements_fieldID;

  passportElements();

  explicit passportElements(array<object_ptr<PassportElement>> &&elements_);

  static const std::int32_t ID = 1264617556;

  static object_ptr<passportElements> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElement;

class passportElementError;

class passportElementsWithErrors final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PassportElement>> elements_;
  static jfieldID elements_fieldID;
  array<object_ptr<passportElementError>> errors_;
  static jfieldID errors_fieldID;

  passportElementsWithErrors();

  passportElementsWithErrors(array<object_ptr<PassportElement>> &&elements_, array<object_ptr<passportElementError>> &&errors_);

  static const std::int32_t ID = 1308923044;

  static object_ptr<passportElementsWithErrors> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class passportSuitableElement;

class passportRequiredElement final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<passportSuitableElement>> suitable_elements_;
  static jfieldID suitable_elements_fieldID;

  passportRequiredElement();

  explicit passportRequiredElement(array<object_ptr<passportSuitableElement>> &&suitable_elements_);

  static const std::int32_t ID = -1983641651;

  static object_ptr<passportRequiredElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementType;

class passportSuitableElement final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;
  bool is_selfie_required_;
  static jfieldID is_selfie_required_fieldID;
  bool is_translation_required_;
  static jfieldID is_translation_required_fieldID;
  bool is_native_name_required_;
  static jfieldID is_native_name_required_fieldID;

  passportSuitableElement();

  passportSuitableElement(object_ptr<PassportElementType> &&type_, bool is_selfie_required_, bool is_translation_required_, bool is_native_name_required_);

  static const std::int32_t ID = -789019876;

  static object_ptr<passportSuitableElement> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class passwordState final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_password_;
  static jfieldID has_password_fieldID;
  string password_hint_;
  static jfieldID password_hint_fieldID;
  bool has_recovery_email_address_;
  static jfieldID has_recovery_email_address_fieldID;
  bool has_passport_data_;
  static jfieldID has_passport_data_fieldID;
  object_ptr<emailAddressAuthenticationCodeInfo> recovery_email_address_code_info_;
  static jfieldID recovery_email_address_code_info_fieldID;
  string login_email_address_pattern_;
  static jfieldID login_email_address_pattern_fieldID;
  int32 pending_reset_date_;
  static jfieldID pending_reset_date_fieldID;

  passwordState();

  passwordState(bool has_password_, string const &password_hint_, bool has_recovery_email_address_, bool has_passport_data_, object_ptr<emailAddressAuthenticationCodeInfo> &&recovery_email_address_code_info_, string const &login_email_address_pattern_, int32 pending_reset_date_);

  static const std::int32_t ID = 483801128;

  static object_ptr<passwordState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PaymentFormType;

class productInfo;

class paymentForm final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  object_ptr<PaymentFormType> type_;
  static jfieldID type_fieldID;
  int53 seller_bot_user_id_;
  static jfieldID seller_bot_user_id_fieldID;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;

  paymentForm();

  paymentForm(int64 id_, object_ptr<PaymentFormType> &&type_, int53 seller_bot_user_id_, object_ptr<productInfo> &&product_info_);

  static const std::int32_t ID = 1998651315;

  static object_ptr<paymentForm> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PaymentProvider;

class invoice;

class orderInfo;

class paymentOption;

class savedCredentials;

class starSubscriptionPricing;

class PaymentFormType: public Object {
 public:
  static jclass Class;

  static object_ptr<PaymentFormType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paymentFormTypeRegular final : public PaymentFormType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<invoice> invoice_;
  static jfieldID invoice_fieldID;
  int53 payment_provider_user_id_;
  static jfieldID payment_provider_user_id_fieldID;
  object_ptr<PaymentProvider> payment_provider_;
  static jfieldID payment_provider_fieldID;
  array<object_ptr<paymentOption>> additional_payment_options_;
  static jfieldID additional_payment_options_fieldID;
  object_ptr<orderInfo> saved_order_info_;
  static jfieldID saved_order_info_fieldID;
  array<object_ptr<savedCredentials>> saved_credentials_;
  static jfieldID saved_credentials_fieldID;
  bool can_save_credentials_;
  static jfieldID can_save_credentials_fieldID;
  bool need_password_;
  static jfieldID need_password_fieldID;

  paymentFormTypeRegular();

  paymentFormTypeRegular(object_ptr<invoice> &&invoice_, int53 payment_provider_user_id_, object_ptr<PaymentProvider> &&payment_provider_, array<object_ptr<paymentOption>> &&additional_payment_options_, object_ptr<orderInfo> &&saved_order_info_, array<object_ptr<savedCredentials>> &&saved_credentials_, bool can_save_credentials_, bool need_password_);

  static const std::int32_t ID = -615089778;

  static object_ptr<PaymentFormType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentFormTypeStars final : public PaymentFormType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  paymentFormTypeStars();

  explicit paymentFormTypeStars(int53 star_count_);

  static const std::int32_t ID = 90938685;

  static object_ptr<PaymentFormType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentFormTypeStarSubscription final : public PaymentFormType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starSubscriptionPricing> pricing_;
  static jfieldID pricing_fieldID;

  paymentFormTypeStarSubscription();

  explicit paymentFormTypeStarSubscription(object_ptr<starSubscriptionPricing> &&pricing_);

  static const std::int32_t ID = 271444827;

  static object_ptr<PaymentFormType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  string url_;
  static jfieldID url_fieldID;

  paymentOption();

  paymentOption(string const &title_, string const &url_);

  static const std::int32_t ID = -294020965;

  static object_ptr<paymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PaymentProvider: public Object {
 public:
  static jclass Class;

  static object_ptr<PaymentProvider> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paymentProviderSmartGlocal final : public PaymentProvider {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string public_token_;
  static jfieldID public_token_fieldID;
  string tokenize_url_;
  static jfieldID tokenize_url_fieldID;

  paymentProviderSmartGlocal();

  paymentProviderSmartGlocal(string const &public_token_, string const &tokenize_url_);

  static const std::int32_t ID = -1174112396;

  static object_ptr<PaymentProvider> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentProviderStripe final : public PaymentProvider {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string publishable_key_;
  static jfieldID publishable_key_fieldID;
  bool need_country_;
  static jfieldID need_country_fieldID;
  bool need_postal_code_;
  static jfieldID need_postal_code_fieldID;
  bool need_cardholder_name_;
  static jfieldID need_cardholder_name_fieldID;

  paymentProviderStripe();

  paymentProviderStripe(string const &publishable_key_, bool need_country_, bool need_postal_code_, bool need_cardholder_name_);

  static const std::int32_t ID = 370467227;

  static object_ptr<PaymentProvider> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentProviderOther final : public PaymentProvider {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  paymentProviderOther();

  explicit paymentProviderOther(string const &url_);

  static const std::int32_t ID = -1336876828;

  static object_ptr<PaymentProvider> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PaymentReceiptType;

class productInfo;

class paymentReceipt final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  int53 seller_bot_user_id_;
  static jfieldID seller_bot_user_id_fieldID;
  object_ptr<PaymentReceiptType> type_;
  static jfieldID type_fieldID;

  paymentReceipt();

  paymentReceipt(object_ptr<productInfo> &&product_info_, int32 date_, int53 seller_bot_user_id_, object_ptr<PaymentReceiptType> &&type_);

  static const std::int32_t ID = 758199186;

  static object_ptr<paymentReceipt> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class invoice;

class orderInfo;

class shippingOption;

class PaymentReceiptType: public Object {
 public:
  static jclass Class;

  static object_ptr<PaymentReceiptType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paymentReceiptTypeRegular final : public PaymentReceiptType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 payment_provider_user_id_;
  static jfieldID payment_provider_user_id_fieldID;
  object_ptr<invoice> invoice_;
  static jfieldID invoice_fieldID;
  object_ptr<orderInfo> order_info_;
  static jfieldID order_info_fieldID;
  object_ptr<shippingOption> shipping_option_;
  static jfieldID shipping_option_fieldID;
  string credentials_title_;
  static jfieldID credentials_title_fieldID;
  int53 tip_amount_;
  static jfieldID tip_amount_fieldID;

  paymentReceiptTypeRegular();

  paymentReceiptTypeRegular(int53 payment_provider_user_id_, object_ptr<invoice> &&invoice_, object_ptr<orderInfo> &&order_info_, object_ptr<shippingOption> &&shipping_option_, string const &credentials_title_, int53 tip_amount_);

  static const std::int32_t ID = -1636362826;

  static object_ptr<PaymentReceiptType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentReceiptTypeStars final : public PaymentReceiptType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string transaction_id_;
  static jfieldID transaction_id_fieldID;

  paymentReceiptTypeStars();

  paymentReceiptTypeStars(int53 star_count_, string const &transaction_id_);

  static const std::int32_t ID = 294913868;

  static object_ptr<PaymentReceiptType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class paymentResult final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool success_;
  static jfieldID success_fieldID;
  string verification_url_;
  static jfieldID verification_url_fieldID;

  paymentResult();

  paymentResult(bool success_, string const &verification_url_);

  static const std::int32_t ID = -804263843;

  static object_ptr<paymentResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class date;

class personalDetails final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string first_name_;
  static jfieldID first_name_fieldID;
  string middle_name_;
  static jfieldID middle_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;
  string native_first_name_;
  static jfieldID native_first_name_fieldID;
  string native_middle_name_;
  static jfieldID native_middle_name_fieldID;
  string native_last_name_;
  static jfieldID native_last_name_fieldID;
  object_ptr<date> birthdate_;
  static jfieldID birthdate_fieldID;
  string gender_;
  static jfieldID gender_fieldID;
  string country_code_;
  static jfieldID country_code_fieldID;
  string residence_country_code_;
  static jfieldID residence_country_code_fieldID;

  personalDetails();

  personalDetails(string const &first_name_, string const &middle_name_, string const &last_name_, string const &native_first_name_, string const &native_middle_name_, string const &native_last_name_, object_ptr<date> &&birthdate_, string const &gender_, string const &country_code_, string const &residence_country_code_);

  static const std::int32_t ID = -1061656137;

  static object_ptr<personalDetails> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class datedFile;

class personalDocument final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<datedFile>> files_;
  static jfieldID files_fieldID;
  array<object_ptr<datedFile>> translation_;
  static jfieldID translation_fieldID;

  personalDocument();

  personalDocument(array<object_ptr<datedFile>> &&files_, array<object_ptr<datedFile>> &&translation_);

  static const std::int32_t ID = -1011634661;

  static object_ptr<personalDocument> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FirebaseAuthenticationSettings;

class phoneNumberAuthenticationSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_flash_call_;
  static jfieldID allow_flash_call_fieldID;
  bool allow_missed_call_;
  static jfieldID allow_missed_call_fieldID;
  bool is_current_phone_number_;
  static jfieldID is_current_phone_number_fieldID;
  bool has_unknown_phone_number_;
  static jfieldID has_unknown_phone_number_fieldID;
  bool allow_sms_retriever_api_;
  static jfieldID allow_sms_retriever_api_fieldID;
  object_ptr<FirebaseAuthenticationSettings> firebase_authentication_settings_;
  static jfieldID firebase_authentication_settings_fieldID;
  array<string> authentication_tokens_;
  static jfieldID authentication_tokens_fieldID;

  phoneNumberAuthenticationSettings();

  phoneNumberAuthenticationSettings(bool allow_flash_call_, bool allow_missed_call_, bool is_current_phone_number_, bool has_unknown_phone_number_, bool allow_sms_retriever_api_, object_ptr<FirebaseAuthenticationSettings> &&firebase_authentication_settings_, array<string> &&authentication_tokens_);

  static const std::int32_t ID = 1881885547;

  static object_ptr<phoneNumberAuthenticationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PhoneNumberCodeType: public Object {
 public:
  static jclass Class;

  static object_ptr<PhoneNumberCodeType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class phoneNumberCodeTypeChange final : public PhoneNumberCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  phoneNumberCodeTypeChange();

  static const std::int32_t ID = 87144986;

  static object_ptr<PhoneNumberCodeType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class phoneNumberCodeTypeVerify final : public PhoneNumberCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  phoneNumberCodeTypeVerify();

  static const std::int32_t ID = -1029402661;

  static object_ptr<PhoneNumberCodeType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class phoneNumberCodeTypeConfirmOwnership final : public PhoneNumberCodeType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string hash_;
  static jfieldID hash_fieldID;

  phoneNumberCodeTypeConfirmOwnership();

  explicit phoneNumberCodeTypeConfirmOwnership(string const &hash_);

  static const std::int32_t ID = -485404696;

  static object_ptr<PhoneNumberCodeType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class countryInfo;

class phoneNumberInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<countryInfo> country_;
  static jfieldID country_fieldID;
  string country_calling_code_;
  static jfieldID country_calling_code_fieldID;
  string formatted_phone_number_;
  static jfieldID formatted_phone_number_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;

  phoneNumberInfo();

  phoneNumberInfo(object_ptr<countryInfo> &&country_, string const &country_calling_code_, string const &formatted_phone_number_, bool is_anonymous_);

  static const std::int32_t ID = -758933343;

  static object_ptr<phoneNumberInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class minithumbnail;

class photoSize;

class photo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_stickers_;
  static jfieldID has_stickers_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  array<object_ptr<photoSize>> sizes_;
  static jfieldID sizes_fieldID;

  photo();

  photo(bool has_stickers_, object_ptr<minithumbnail> &&minithumbnail_, array<object_ptr<photoSize>> &&sizes_);

  static const std::int32_t ID = -2022871583;

  static object_ptr<photo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class photoSize final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string type_;
  static jfieldID type_fieldID;
  object_ptr<file> photo_;
  static jfieldID photo_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  array<int32> progressive_sizes_;
  static jfieldID progressive_sizes_fieldID;

  photoSize();

  photoSize(string const &type_, object_ptr<file> &&photo_, int32 width_, int32 height_, array<int32> &&progressive_sizes_);

  static const std::int32_t ID = 1609182352;

  static object_ptr<photoSize> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class point final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double x_;
  static jfieldID x_fieldID;
  double y_;
  static jfieldID y_fieldID;

  point();

  point(double x_, double y_);

  static const std::int32_t ID = 437515705;

  static object_ptr<point> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class PollType;

class formattedText;

class pollOption;

class poll final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  object_ptr<formattedText> question_;
  static jfieldID question_fieldID;
  array<object_ptr<pollOption>> options_;
  static jfieldID options_fieldID;
  int32 total_voter_count_;
  static jfieldID total_voter_count_fieldID;
  array<object_ptr<MessageSender>> recent_voter_ids_;
  static jfieldID recent_voter_ids_fieldID;
  bool is_anonymous_;
  static jfieldID is_anonymous_fieldID;
  object_ptr<PollType> type_;
  static jfieldID type_fieldID;
  int32 open_period_;
  static jfieldID open_period_fieldID;
  int32 close_date_;
  static jfieldID close_date_fieldID;
  bool is_closed_;
  static jfieldID is_closed_fieldID;

  poll();

  poll(int64 id_, object_ptr<formattedText> &&question_, array<object_ptr<pollOption>> &&options_, int32 total_voter_count_, array<object_ptr<MessageSender>> &&recent_voter_ids_, bool is_anonymous_, object_ptr<PollType> &&type_, int32 open_period_, int32 close_date_, bool is_closed_);

  static const std::int32_t ID = 1913016502;

  static object_ptr<poll> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class pollOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int32 voter_count_;
  static jfieldID voter_count_fieldID;
  int32 vote_percentage_;
  static jfieldID vote_percentage_fieldID;
  bool is_chosen_;
  static jfieldID is_chosen_fieldID;
  bool is_being_chosen_;
  static jfieldID is_being_chosen_fieldID;

  pollOption();

  pollOption(object_ptr<formattedText> &&text_, int32 voter_count_, int32 vote_percentage_, bool is_chosen_, bool is_being_chosen_);

  static const std::int32_t ID = 1676243088;

  static object_ptr<pollOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class PollType: public Object {
 public:
  static jclass Class;

  static object_ptr<PollType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pollTypeRegular final : public PollType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_multiple_answers_;
  static jfieldID allow_multiple_answers_fieldID;

  pollTypeRegular();

  explicit pollTypeRegular(bool allow_multiple_answers_);

  static const std::int32_t ID = 641265698;

  static object_ptr<PollType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pollTypeQuiz final : public PollType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 correct_option_id_;
  static jfieldID correct_option_id_fieldID;
  object_ptr<formattedText> explanation_;
  static jfieldID explanation_fieldID;

  pollTypeQuiz();

  pollTypeQuiz(int32 correct_option_id_, object_ptr<formattedText> &&explanation_);

  static const std::int32_t ID = 657013913;

  static object_ptr<PollType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PremiumFeature: public Object {
 public:
  static jclass Class;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureIncreasedLimits final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureIncreasedLimits();

  static const std::int32_t ID = 1785455031;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureIncreasedUploadFileSize final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureIncreasedUploadFileSize();

  static const std::int32_t ID = 1825367155;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureImprovedDownloadSpeed final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureImprovedDownloadSpeed();

  static const std::int32_t ID = -267695554;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureVoiceRecognition final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureVoiceRecognition();

  static const std::int32_t ID = 1288216542;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureDisabledAds final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureDisabledAds();

  static const std::int32_t ID = -2008587702;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureUniqueReactions final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureUniqueReactions();

  static const std::int32_t ID = 766750743;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureUniqueStickers final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureUniqueStickers();

  static const std::int32_t ID = -2101773312;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureCustomEmoji final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureCustomEmoji();

  static const std::int32_t ID = 1332599628;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureAdvancedChatManagement final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureAdvancedChatManagement();

  static const std::int32_t ID = 796347674;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureProfileBadge final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureProfileBadge();

  static const std::int32_t ID = 233648322;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureEmojiStatus final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureEmojiStatus();

  static const std::int32_t ID = -36516639;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureAnimatedProfilePhoto final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureAnimatedProfilePhoto();

  static const std::int32_t ID = -100741914;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureForumTopicIcon final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureForumTopicIcon();

  static const std::int32_t ID = -823172286;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureAppIcons final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureAppIcons();

  static const std::int32_t ID = 1585050761;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureRealTimeChatTranslation final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureRealTimeChatTranslation();

  static const std::int32_t ID = -1143471488;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureUpgradedStories final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureUpgradedStories();

  static const std::int32_t ID = -1878522597;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureChatBoost final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureChatBoost();

  static const std::int32_t ID = 1576574747;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureAccentColor final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureAccentColor();

  static const std::int32_t ID = 907724190;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureBackgroundForBoth final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureBackgroundForBoth();

  static const std::int32_t ID = 575074042;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureSavedMessagesTags final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureSavedMessagesTags();

  static const std::int32_t ID = 1003219334;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureMessagePrivacy final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureMessagePrivacy();

  static const std::int32_t ID = 802322678;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureLastSeenTimes final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureLastSeenTimes();

  static const std::int32_t ID = -762230129;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureBusiness final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureBusiness();

  static const std::int32_t ID = -1503619324;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumFeatureMessageEffects final : public PremiumFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumFeatureMessageEffects();

  static const std::int32_t ID = -723300255;

  static object_ptr<PremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PremiumFeature;

class animation;

class premiumFeaturePromotionAnimation final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumFeature> feature_;
  static jfieldID feature_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;

  premiumFeaturePromotionAnimation();

  premiumFeaturePromotionAnimation(object_ptr<PremiumFeature> &&feature_, object_ptr<animation> &&animation_);

  static const std::int32_t ID = -1986155748;

  static object_ptr<premiumFeaturePromotionAnimation> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class PremiumFeature;

class premiumLimit;

class premiumFeatures final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PremiumFeature>> features_;
  static jfieldID features_fieldID;
  array<object_ptr<premiumLimit>> limits_;
  static jfieldID limits_fieldID;
  object_ptr<InternalLinkType> payment_link_;
  static jfieldID payment_link_fieldID;

  premiumFeatures();

  premiumFeatures(array<object_ptr<PremiumFeature>> &&features_, array<object_ptr<premiumLimit>> &&limits_, object_ptr<InternalLinkType> &&payment_link_);

  static const std::int32_t ID = 1875162172;

  static object_ptr<premiumFeatures> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class premiumGiftCodeInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> creator_id_;
  static jfieldID creator_id_fieldID;
  int32 creation_date_;
  static jfieldID creation_date_fieldID;
  bool is_from_giveaway_;
  static jfieldID is_from_giveaway_fieldID;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 use_date_;
  static jfieldID use_date_fieldID;

  premiumGiftCodeInfo();

  premiumGiftCodeInfo(object_ptr<MessageSender> &&creator_id_, int32 creation_date_, bool is_from_giveaway_, int53 giveaway_message_id_, int32 month_count_, int53 user_id_, int32 use_date_);

  static const std::int32_t ID = -1198544674;

  static object_ptr<premiumGiftCodeInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class premiumGiftPaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  int32 discount_percentage_;
  static jfieldID discount_percentage_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  premiumGiftPaymentOption();

  premiumGiftPaymentOption(string const &currency_, int53 amount_, int53 star_count_, int32 discount_percentage_, int32 month_count_, string const &store_product_id_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -338085027;

  static object_ptr<premiumGiftPaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiftPaymentOption;

class premiumGiftPaymentOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<premiumGiftPaymentOption>> options_;
  static jfieldID options_fieldID;

  premiumGiftPaymentOptions();

  explicit premiumGiftPaymentOptions(array<object_ptr<premiumGiftPaymentOption>> &&options_);

  static const std::int32_t ID = -1347543032;

  static object_ptr<premiumGiftPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiveawayPaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  int32 store_product_quantity_;
  static jfieldID store_product_quantity_fieldID;

  premiumGiveawayPaymentOption();

  premiumGiveawayPaymentOption(string const &currency_, int53 amount_, int32 winner_count_, int32 month_count_, string const &store_product_id_, int32 store_product_quantity_);

  static const std::int32_t ID = 1099221896;

  static object_ptr<premiumGiveawayPaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiveawayPaymentOption;

class premiumGiveawayPaymentOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<premiumGiveawayPaymentOption>> options_;
  static jfieldID options_fieldID;

  premiumGiveawayPaymentOptions();

  explicit premiumGiveawayPaymentOptions(array<object_ptr<premiumGiveawayPaymentOption>> &&options_);

  static const std::int32_t ID = -1587397823;

  static object_ptr<premiumGiveawayPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PremiumLimitType;

class premiumLimit final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumLimitType> type_;
  static jfieldID type_fieldID;
  int32 default_value_;
  static jfieldID default_value_fieldID;
  int32 premium_value_;
  static jfieldID premium_value_fieldID;

  premiumLimit();

  premiumLimit(object_ptr<PremiumLimitType> &&type_, int32 default_value_, int32 premium_value_);

  static const std::int32_t ID = 2127786726;

  static object_ptr<premiumLimit> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PremiumLimitType: public Object {
 public:
  static jclass Class;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeSupergroupCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeSupergroupCount();

  static const std::int32_t ID = -247467131;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypePinnedChatCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypePinnedChatCount();

  static const std::int32_t ID = -998947871;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeCreatedPublicChatCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeCreatedPublicChatCount();

  static const std::int32_t ID = 446086841;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeSavedAnimationCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeSavedAnimationCount();

  static const std::int32_t ID = -19759735;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeFavoriteStickerCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeFavoriteStickerCount();

  static const std::int32_t ID = 639754787;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeChatFolderCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeChatFolderCount();

  static const std::int32_t ID = 377489774;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeChatFolderChosenChatCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeChatFolderChosenChatCount();

  static const std::int32_t ID = 1691435861;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypePinnedArchivedChatCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypePinnedArchivedChatCount();

  static const std::int32_t ID = 1485515276;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypePinnedSavedMessagesTopicCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypePinnedSavedMessagesTopicCount();

  static const std::int32_t ID = -1544854305;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeCaptionLength final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeCaptionLength();

  static const std::int32_t ID = 293984314;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeBioLength final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeBioLength();

  static const std::int32_t ID = -1146976765;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeChatFolderInviteLinkCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeChatFolderInviteLinkCount();

  static const std::int32_t ID = -128702950;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeShareableChatFolderCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeShareableChatFolderCount();

  static const std::int32_t ID = 1612625095;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeActiveStoryCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeActiveStoryCount();

  static const std::int32_t ID = -1926486372;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeWeeklyPostedStoryCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeWeeklyPostedStoryCount();

  static const std::int32_t ID = -506354313;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeMonthlyPostedStoryCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeMonthlyPostedStoryCount();

  static const std::int32_t ID = 26329490;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeStoryCaptionLength final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeStoryCaptionLength();

  static const std::int32_t ID = -1093324030;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeStorySuggestedReactionAreaCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeStorySuggestedReactionAreaCount();

  static const std::int32_t ID = -1170032633;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumLimitTypeSimilarChatCount final : public PremiumLimitType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumLimitTypeSimilarChatCount();

  static const std::int32_t ID = -1563549935;

  static object_ptr<PremiumLimitType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class premiumPaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int32 discount_percentage_;
  static jfieldID discount_percentage_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  object_ptr<InternalLinkType> payment_link_;
  static jfieldID payment_link_fieldID;

  premiumPaymentOption();

  premiumPaymentOption(string const &currency_, int53 amount_, int32 discount_percentage_, int32 month_count_, string const &store_product_id_, object_ptr<InternalLinkType> &&payment_link_);

  static const std::int32_t ID = -1945346126;

  static object_ptr<premiumPaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BusinessFeature;

class PremiumFeature;

class PremiumLimitType;

class PremiumStoryFeature;

class PremiumSource: public Object {
 public:
  static jclass Class;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceLimitExceeded final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumLimitType> limit_type_;
  static jfieldID limit_type_fieldID;

  premiumSourceLimitExceeded();

  explicit premiumSourceLimitExceeded(object_ptr<PremiumLimitType> &&limit_type_);

  static const std::int32_t ID = -2052159742;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceFeature final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumFeature> feature_;
  static jfieldID feature_fieldID;

  premiumSourceFeature();

  explicit premiumSourceFeature(object_ptr<PremiumFeature> &&feature_);

  static const std::int32_t ID = 445813541;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceBusinessFeature final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BusinessFeature> feature_;
  static jfieldID feature_fieldID;

  premiumSourceBusinessFeature();

  explicit premiumSourceBusinessFeature(object_ptr<BusinessFeature> &&feature_);

  static const std::int32_t ID = -1492946340;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceStoryFeature final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumStoryFeature> feature_;
  static jfieldID feature_fieldID;

  premiumSourceStoryFeature();

  explicit premiumSourceStoryFeature(object_ptr<PremiumStoryFeature> &&feature_);

  static const std::int32_t ID = -1030737556;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceLink final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string referrer_;
  static jfieldID referrer_fieldID;

  premiumSourceLink();

  explicit premiumSourceLink(string const &referrer_);

  static const std::int32_t ID = 2135071132;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumSourceSettings final : public PremiumSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumSourceSettings();

  static const std::int32_t ID = -285702859;

  static object_ptr<PremiumSource> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class businessFeaturePromotionAnimation;

class formattedText;

class premiumFeaturePromotionAnimation;

class premiumStatePaymentOption;

class premiumState final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> state_;
  static jfieldID state_fieldID;
  array<object_ptr<premiumStatePaymentOption>> payment_options_;
  static jfieldID payment_options_fieldID;
  array<object_ptr<premiumFeaturePromotionAnimation>> animations_;
  static jfieldID animations_fieldID;
  array<object_ptr<businessFeaturePromotionAnimation>> business_animations_;
  static jfieldID business_animations_fieldID;

  premiumState();

  premiumState(object_ptr<formattedText> &&state_, array<object_ptr<premiumStatePaymentOption>> &&payment_options_, array<object_ptr<premiumFeaturePromotionAnimation>> &&animations_, array<object_ptr<businessFeaturePromotionAnimation>> &&business_animations_);

  static const std::int32_t ID = 1772082178;

  static object_ptr<premiumState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumPaymentOption;

class premiumStatePaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<premiumPaymentOption> payment_option_;
  static jfieldID payment_option_fieldID;
  bool is_current_;
  static jfieldID is_current_fieldID;
  bool is_upgrade_;
  static jfieldID is_upgrade_fieldID;
  string last_transaction_id_;
  static jfieldID last_transaction_id_fieldID;

  premiumStatePaymentOption();

  premiumStatePaymentOption(object_ptr<premiumPaymentOption> &&payment_option_, bool is_current_, bool is_upgrade_, string const &last_transaction_id_);

  static const std::int32_t ID = 2097591673;

  static object_ptr<premiumStatePaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PremiumStoryFeature: public Object {
 public:
  static jclass Class;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeaturePriorityOrder final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeaturePriorityOrder();

  static const std::int32_t ID = -1880001849;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeatureStealthMode final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeatureStealthMode();

  static const std::int32_t ID = 1194605988;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeaturePermanentViewsHistory final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeaturePermanentViewsHistory();

  static const std::int32_t ID = -1029683296;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeatureCustomExpirationDuration final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeatureCustomExpirationDuration();

  static const std::int32_t ID = -593229162;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeatureSaveStories final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeatureSaveStories();

  static const std::int32_t ID = -1501286467;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeatureLinksAndFormatting final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeatureLinksAndFormatting();

  static const std::int32_t ID = -622623753;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class premiumStoryFeatureVideoQuality final : public PremiumStoryFeature {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  premiumStoryFeatureVideoQuality();

  static const std::int32_t ID = -1162887511;

  static object_ptr<PremiumStoryFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayPrize;

class prepaidGiveaway final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  object_ptr<GiveawayPrize> prize_;
  static jfieldID prize_fieldID;
  int32 boost_count_;
  static jfieldID boost_count_fieldID;
  int32 payment_date_;
  static jfieldID payment_date_fieldID;

  prepaidGiveaway();

  prepaidGiveaway(int64 id_, int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, int32 boost_count_, int32 payment_date_);

  static const std::int32_t ID = -277859441;

  static object_ptr<prepaidGiveaway> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InlineQueryResult;

class targetChatTypes;

class preparedInlineMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 inline_query_id_;
  static jfieldID inline_query_id_fieldID;
  object_ptr<InlineQueryResult> result_;
  static jfieldID result_fieldID;
  object_ptr<targetChatTypes> chat_types_;
  static jfieldID chat_types_fieldID;

  preparedInlineMessage();

  preparedInlineMessage(int64 inline_query_id_, object_ptr<InlineQueryResult> &&result_, object_ptr<targetChatTypes> &&chat_types_);

  static const std::int32_t ID = -1808892734;

  static object_ptr<preparedInlineMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class preparedInlineMessageId final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;

  preparedInlineMessageId();

  preparedInlineMessageId(string const &id_, int32 expiration_date_);

  static const std::int32_t ID = 940415972;

  static object_ptr<preparedInlineMessageId> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class photo;

class productInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<formattedText> description_;
  static jfieldID description_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  productInfo();

  productInfo(string const &title_, object_ptr<formattedText> &&description_, object_ptr<photo> &&photo_);

  static const std::int32_t ID = -2015069020;

  static object_ptr<productInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class profileAccentColors;

class profileAccentColor final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  object_ptr<profileAccentColors> light_theme_colors_;
  static jfieldID light_theme_colors_fieldID;
  object_ptr<profileAccentColors> dark_theme_colors_;
  static jfieldID dark_theme_colors_fieldID;
  int32 min_supergroup_chat_boost_level_;
  static jfieldID min_supergroup_chat_boost_level_fieldID;
  int32 min_channel_chat_boost_level_;
  static jfieldID min_channel_chat_boost_level_fieldID;

  profileAccentColor();

  profileAccentColor(int32 id_, object_ptr<profileAccentColors> &&light_theme_colors_, object_ptr<profileAccentColors> &&dark_theme_colors_, int32 min_supergroup_chat_boost_level_, int32 min_channel_chat_boost_level_);

  static const std::int32_t ID = 557679253;

  static object_ptr<profileAccentColor> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class profileAccentColors final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> palette_colors_;
  static jfieldID palette_colors_fieldID;
  array<int32> background_colors_;
  static jfieldID background_colors_fieldID;
  array<int32> story_colors_;
  static jfieldID story_colors_fieldID;

  profileAccentColors();

  profileAccentColors(array<int32> &&palette_colors_, array<int32> &&background_colors_, array<int32> &&story_colors_);

  static const std::int32_t ID = -596042431;

  static object_ptr<profileAccentColors> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class profilePhoto final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  object_ptr<file> small_;
  static jfieldID small_fieldID;
  object_ptr<file> big_;
  static jfieldID big_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  bool has_animation_;
  static jfieldID has_animation_fieldID;
  bool is_personal_;
  static jfieldID is_personal_fieldID;

  profilePhoto();

  profilePhoto(int64 id_, object_ptr<file> &&small_, object_ptr<file> &&big_, object_ptr<minithumbnail> &&minithumbnail_, bool has_animation_, bool is_personal_);

  static const std::int32_t ID = -1025754018;

  static object_ptr<profilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class proxy;

class proxies final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<proxy>> proxies_;
  static jfieldID proxies_fieldID;

  proxies();

  explicit proxies(array<object_ptr<proxy>> &&proxies_);

  static const std::int32_t ID = 1200447205;

  static object_ptr<proxies> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType;

class proxy final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  string server_;
  static jfieldID server_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  int32 last_used_date_;
  static jfieldID last_used_date_fieldID;
  bool is_enabled_;
  static jfieldID is_enabled_fieldID;
  object_ptr<ProxyType> type_;
  static jfieldID type_fieldID;

  proxy();

  proxy(int32 id_, string const &server_, int32 port_, int32 last_used_date_, bool is_enabled_, object_ptr<ProxyType> &&type_);

  static const std::int32_t ID = 196049779;

  static object_ptr<proxy> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType: public Object {
 public:
  static jclass Class;

  static object_ptr<ProxyType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class proxyTypeSocks5 final : public ProxyType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  string password_;
  static jfieldID password_fieldID;

  proxyTypeSocks5();

  proxyTypeSocks5(string const &username_, string const &password_);

  static const std::int32_t ID = -890027341;

  static object_ptr<ProxyType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class proxyTypeHttp final : public ProxyType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  string password_;
  static jfieldID password_fieldID;
  bool http_only_;
  static jfieldID http_only_fieldID;

  proxyTypeHttp();

  proxyTypeHttp(string const &username_, string const &password_, bool http_only_);

  static const std::int32_t ID = -1547188361;

  static object_ptr<ProxyType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class proxyTypeMtproto final : public ProxyType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string secret_;
  static jfieldID secret_fieldID;

  proxyTypeMtproto();

  explicit proxyTypeMtproto(string const &secret_);

  static const std::int32_t ID = -1964826627;

  static object_ptr<ProxyType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PublicChatType: public Object {
 public:
  static jclass Class;

  static object_ptr<PublicChatType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class publicChatTypeHasUsername final : public PublicChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  publicChatTypeHasUsername();

  static const std::int32_t ID = 350789758;

  static object_ptr<PublicChatType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class publicChatTypeIsLocationBased final : public PublicChatType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  publicChatTypeIsLocationBased();

  static const std::int32_t ID = 1183735952;

  static object_ptr<PublicChatType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class message;

class story;

class PublicForward: public Object {
 public:
  static jclass Class;

  static object_ptr<PublicForward> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class publicForwardMessage final : public PublicForward {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  publicForwardMessage();

  explicit publicForwardMessage(object_ptr<message> &&message_);

  static const std::int32_t ID = 51885010;

  static object_ptr<PublicForward> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class publicForwardStory final : public PublicForward {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<story> story_;
  static jfieldID story_fieldID;

  publicForwardStory();

  explicit publicForwardStory(object_ptr<story> &&story_);

  static const std::int32_t ID = 2145330863;

  static object_ptr<PublicForward> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class PublicForward;

class publicForwards final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<PublicForward>> forwards_;
  static jfieldID forwards_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  publicForwards();

  publicForwards(int32 total_count_, array<object_ptr<PublicForward>> &&forwards_, string const &next_offset_);

  static const std::int32_t ID = -2011272719;

  static object_ptr<publicForwards> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayPrize;

class animation;

class audio;

class document;

class photo;

class sticker;

class video;

class videoNote;

class voiceNote;

class PushMessageContent: public Object {
 public:
  static jclass Class;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentHidden final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentHidden();

  explicit pushMessageContentHidden(bool is_pinned_);

  static const std::int32_t ID = -316950436;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentAnimation final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;
  string caption_;
  static jfieldID caption_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentAnimation();

  pushMessageContentAnimation(object_ptr<animation> &&animation_, string const &caption_, bool is_pinned_);

  static const std::int32_t ID = 1034215396;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentAudio final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<audio> audio_;
  static jfieldID audio_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentAudio();

  pushMessageContentAudio(object_ptr<audio> &&audio_, bool is_pinned_);

  static const std::int32_t ID = 381581426;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentContact final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentContact();

  pushMessageContentContact(string const &name_, bool is_pinned_);

  static const std::int32_t ID = -12219820;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentContactRegistered final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentContactRegistered();

  static const std::int32_t ID = -303962720;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentDocument final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentDocument();

  pushMessageContentDocument(object_ptr<document> &&document_, bool is_pinned_);

  static const std::int32_t ID = -458379775;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentGame final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentGame();

  pushMessageContentGame(string const &title_, bool is_pinned_);

  static const std::int32_t ID = -515131109;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentGameScore final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  int32 score_;
  static jfieldID score_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentGameScore();

  pushMessageContentGameScore(string const &title_, int32 score_, bool is_pinned_);

  static const std::int32_t ID = 901303688;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentInvoice final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string price_;
  static jfieldID price_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentInvoice();

  pushMessageContentInvoice(string const &price_, bool is_pinned_);

  static const std::int32_t ID = -1731687492;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentLocation final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_live_;
  static jfieldID is_live_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentLocation();

  pushMessageContentLocation(bool is_live_, bool is_pinned_);

  static const std::int32_t ID = -1288005709;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentPaidMedia final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentPaidMedia();

  pushMessageContentPaidMedia(int53 star_count_, bool is_pinned_);

  static const std::int32_t ID = -1252595894;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentPhoto final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  string caption_;
  static jfieldID caption_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentPhoto();

  pushMessageContentPhoto(object_ptr<photo> &&photo_, string const &caption_, bool is_secret_, bool is_pinned_);

  static const std::int32_t ID = 140631122;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentPoll final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string question_;
  static jfieldID question_fieldID;
  bool is_regular_;
  static jfieldID is_regular_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentPoll();

  pushMessageContentPoll(string const &question_, bool is_regular_, bool is_pinned_);

  static const std::int32_t ID = -44403654;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentPremiumGiftCode final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 month_count_;
  static jfieldID month_count_fieldID;

  pushMessageContentPremiumGiftCode();

  explicit pushMessageContentPremiumGiftCode(int32 month_count_);

  static const std::int32_t ID = 413224997;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentGiveaway final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  object_ptr<GiveawayPrize> prize_;
  static jfieldID prize_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentGiveaway();

  pushMessageContentGiveaway(int32 winner_count_, object_ptr<GiveawayPrize> &&prize_, bool is_pinned_);

  static const std::int32_t ID = -700547186;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentGift final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  pushMessageContentGift();

  explicit pushMessageContentGift(int53 star_count_);

  static const std::int32_t ID = -2069312245;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentUpgradedGift final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_upgrade_;
  static jfieldID is_upgrade_fieldID;

  pushMessageContentUpgradedGift();

  explicit pushMessageContentUpgradedGift(bool is_upgrade_);

  static const std::int32_t ID = -1711666466;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentScreenshotTaken final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentScreenshotTaken();

  static const std::int32_t ID = 214245369;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentSticker final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentSticker();

  pushMessageContentSticker(object_ptr<sticker> &&sticker_, string const &emoji_, bool is_pinned_);

  static const std::int32_t ID = 1553513939;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentStory final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_mention_;
  static jfieldID is_mention_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentStory();

  pushMessageContentStory(bool is_mention_, bool is_pinned_);

  static const std::int32_t ID = 599622223;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentText final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentText();

  pushMessageContentText(string const &text_, bool is_pinned_);

  static const std::int32_t ID = 274587305;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentVideo final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<video> video_;
  static jfieldID video_fieldID;
  string caption_;
  static jfieldID caption_fieldID;
  bool is_secret_;
  static jfieldID is_secret_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentVideo();

  pushMessageContentVideo(object_ptr<video> &&video_, string const &caption_, bool is_secret_, bool is_pinned_);

  static const std::int32_t ID = 310038831;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentVideoNote final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<videoNote> video_note_;
  static jfieldID video_note_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentVideoNote();

  pushMessageContentVideoNote(object_ptr<videoNote> &&video_note_, bool is_pinned_);

  static const std::int32_t ID = -1122764417;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentVoiceNote final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<voiceNote> voice_note_;
  static jfieldID voice_note_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  pushMessageContentVoiceNote();

  pushMessageContentVoiceNote(object_ptr<voiceNote> &&voice_note_, bool is_pinned_);

  static const std::int32_t ID = 88910987;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentBasicGroupChatCreate final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentBasicGroupChatCreate();

  static const std::int32_t ID = -2114855172;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentVideoChatStarted final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentVideoChatStarted();

  static const std::int32_t ID = -566547393;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentVideoChatEnded final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentVideoChatEnded();

  static const std::int32_t ID = -1250265885;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentInviteVideoChatParticipants final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_current_user_;
  static jfieldID is_current_user_fieldID;

  pushMessageContentInviteVideoChatParticipants();

  explicit pushMessageContentInviteVideoChatParticipants(bool is_current_user_);

  static const std::int32_t ID = 517620365;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatAddMembers final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string member_name_;
  static jfieldID member_name_fieldID;
  bool is_current_user_;
  static jfieldID is_current_user_fieldID;
  bool is_returned_;
  static jfieldID is_returned_fieldID;

  pushMessageContentChatAddMembers();

  pushMessageContentChatAddMembers(string const &member_name_, bool is_current_user_, bool is_returned_);

  static const std::int32_t ID = -1087145158;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatChangePhoto final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentChatChangePhoto();

  static const std::int32_t ID = -1114222051;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatChangeTitle final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;

  pushMessageContentChatChangeTitle();

  explicit pushMessageContentChatChangeTitle(string const &title_);

  static const std::int32_t ID = -1964902749;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatSetBackground final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_same_;
  static jfieldID is_same_fieldID;

  pushMessageContentChatSetBackground();

  explicit pushMessageContentChatSetBackground(bool is_same_);

  static const std::int32_t ID = -1490331933;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatSetTheme final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  pushMessageContentChatSetTheme();

  explicit pushMessageContentChatSetTheme(string const &theme_name_);

  static const std::int32_t ID = 173882216;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatDeleteMember final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string member_name_;
  static jfieldID member_name_fieldID;
  bool is_current_user_;
  static jfieldID is_current_user_fieldID;
  bool is_left_;
  static jfieldID is_left_fieldID;

  pushMessageContentChatDeleteMember();

  pushMessageContentChatDeleteMember(string const &member_name_, bool is_current_user_, bool is_left_);

  static const std::int32_t ID = 598714783;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatJoinByLink final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentChatJoinByLink();

  static const std::int32_t ID = 1553719113;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentChatJoinByRequest final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentChatJoinByRequest();

  static const std::int32_t ID = -205823627;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentRecurringPayment final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string amount_;
  static jfieldID amount_fieldID;

  pushMessageContentRecurringPayment();

  explicit pushMessageContentRecurringPayment(string const &amount_);

  static const std::int32_t ID = 1619211802;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentSuggestProfilePhoto final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  pushMessageContentSuggestProfilePhoto();

  static const std::int32_t ID = 2104225963;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentProximityAlertTriggered final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 distance_;
  static jfieldID distance_fieldID;

  pushMessageContentProximityAlertTriggered();

  explicit pushMessageContentProximityAlertTriggered(int32 distance_);

  static const std::int32_t ID = -264601594;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentMessageForwards final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;

  pushMessageContentMessageForwards();

  explicit pushMessageContentMessageForwards(int32 total_count_);

  static const std::int32_t ID = -1913083876;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushMessageContentMediaAlbum final : public PushMessageContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  bool has_photos_;
  static jfieldID has_photos_fieldID;
  bool has_videos_;
  static jfieldID has_videos_fieldID;
  bool has_audios_;
  static jfieldID has_audios_fieldID;
  bool has_documents_;
  static jfieldID has_documents_fieldID;

  pushMessageContentMediaAlbum();

  pushMessageContentMediaAlbum(int32 total_count_, bool has_photos_, bool has_videos_, bool has_audios_, bool has_documents_);

  static const std::int32_t ID = -748426897;

  static object_ptr<PushMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class pushReceiverId final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;

  pushReceiverId();

  explicit pushReceiverId(int64 id_);

  static const std::int32_t ID = 371056428;

  static object_ptr<pushReceiverId> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageContent;

class MessageSendingState;

class ReplyMarkup;

class quickReplyMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  object_ptr<MessageSendingState> sending_state_;
  static jfieldID sending_state_fieldID;
  bool can_be_edited_;
  static jfieldID can_be_edited_fieldID;
  int53 reply_to_message_id_;
  static jfieldID reply_to_message_id_fieldID;
  int53 via_bot_user_id_;
  static jfieldID via_bot_user_id_fieldID;
  int64 media_album_id_;
  static jfieldID media_album_id_fieldID;
  object_ptr<MessageContent> content_;
  static jfieldID content_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  quickReplyMessage();

  quickReplyMessage(int53 id_, object_ptr<MessageSendingState> &&sending_state_, bool can_be_edited_, int53 reply_to_message_id_, int53 via_bot_user_id_, int64 media_album_id_, object_ptr<MessageContent> &&content_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = -1090965757;

  static object_ptr<quickReplyMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class quickReplyMessage;

class quickReplyMessages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<quickReplyMessage>> messages_;
  static jfieldID messages_fieldID;

  quickReplyMessages();

  explicit quickReplyMessages(array<object_ptr<quickReplyMessage>> &&messages_);

  static const std::int32_t ID = 743214375;

  static object_ptr<quickReplyMessages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class quickReplyMessage;

class quickReplyShortcut final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<quickReplyMessage> first_message_;
  static jfieldID first_message_fieldID;
  int32 message_count_;
  static jfieldID message_count_fieldID;

  quickReplyShortcut();

  quickReplyShortcut(int32 id_, string const &name_, object_ptr<quickReplyMessage> &&first_message_, int32 message_count_);

  static const std::int32_t ID = -1107453291;

  static object_ptr<quickReplyShortcut> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionNotificationSource;

class reactionNotificationSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionNotificationSource> message_reaction_source_;
  static jfieldID message_reaction_source_fieldID;
  object_ptr<ReactionNotificationSource> story_reaction_source_;
  static jfieldID story_reaction_source_fieldID;
  int64 sound_id_;
  static jfieldID sound_id_fieldID;
  bool show_preview_;
  static jfieldID show_preview_fieldID;

  reactionNotificationSettings();

  reactionNotificationSettings(object_ptr<ReactionNotificationSource> &&message_reaction_source_, object_ptr<ReactionNotificationSource> &&story_reaction_source_, int64 sound_id_, bool show_preview_);

  static const std::int32_t ID = 733017684;

  static object_ptr<reactionNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionNotificationSource: public Object {
 public:
  static jclass Class;

  static object_ptr<ReactionNotificationSource> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reactionNotificationSourceNone final : public ReactionNotificationSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionNotificationSourceNone();

  static const std::int32_t ID = 366374940;

  static object_ptr<ReactionNotificationSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reactionNotificationSourceContacts final : public ReactionNotificationSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionNotificationSourceContacts();

  static const std::int32_t ID = 555501621;

  static object_ptr<ReactionNotificationSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reactionNotificationSourceAll final : public ReactionNotificationSource {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionNotificationSourceAll();

  static const std::int32_t ID = 1241689234;

  static object_ptr<ReactionNotificationSource> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType: public Object {
 public:
  static jclass Class;

  static object_ptr<ReactionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reactionTypeEmoji final : public ReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;

  reactionTypeEmoji();

  explicit reactionTypeEmoji(string const &emoji_);

  static const std::int32_t ID = -1942084920;

  static object_ptr<ReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reactionTypeCustomEmoji final : public ReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  reactionTypeCustomEmoji();

  explicit reactionTypeCustomEmoji(int64 custom_emoji_id_);

  static const std::int32_t ID = -989117709;

  static object_ptr<ReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reactionTypePaid final : public ReactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionTypePaid();

  static const std::int32_t ID = 436294381;

  static object_ptr<ReactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionUnavailabilityReason: public Object {
 public:
  static jclass Class;

  static object_ptr<ReactionUnavailabilityReason> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reactionUnavailabilityReasonAnonymousAdministrator final : public ReactionUnavailabilityReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionUnavailabilityReasonAnonymousAdministrator();

  static const std::int32_t ID = -499612677;

  static object_ptr<ReactionUnavailabilityReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reactionUnavailabilityReasonGuest final : public ReactionUnavailabilityReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reactionUnavailabilityReasonGuest();

  static const std::int32_t ID = 1357861444;

  static object_ptr<ReactionUnavailabilityReason> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class readDatePrivacySettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool show_read_date_;
  static jfieldID show_read_date_fieldID;

  readDatePrivacySettings();

  explicit readDatePrivacySettings(bool show_read_date_);

  static const std::int32_t ID = 1654842920;

  static object_ptr<readDatePrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class SentGift;

class formattedText;

class receivedGift final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool is_private_;
  static jfieldID is_private_fieldID;
  bool is_saved_;
  static jfieldID is_saved_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  bool can_be_upgraded_;
  static jfieldID can_be_upgraded_fieldID;
  bool can_be_transferred_;
  static jfieldID can_be_transferred_fieldID;
  bool was_refunded_;
  static jfieldID was_refunded_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<SentGift> gift_;
  static jfieldID gift_fieldID;
  int53 sell_star_count_;
  static jfieldID sell_star_count_fieldID;
  int53 prepaid_upgrade_star_count_;
  static jfieldID prepaid_upgrade_star_count_fieldID;
  int53 transfer_star_count_;
  static jfieldID transfer_star_count_fieldID;
  int32 next_transfer_date_;
  static jfieldID next_transfer_date_fieldID;
  int32 next_resale_date_;
  static jfieldID next_resale_date_fieldID;
  int32 export_date_;
  static jfieldID export_date_fieldID;

  receivedGift();

  receivedGift(string const &received_gift_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<formattedText> &&text_, bool is_private_, bool is_saved_, bool is_pinned_, bool can_be_upgraded_, bool can_be_transferred_, bool was_refunded_, int32 date_, object_ptr<SentGift> &&gift_, int53 sell_star_count_, int53 prepaid_upgrade_star_count_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_);

  static const std::int32_t ID = 61550296;

  static object_ptr<receivedGift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class receivedGift;

class receivedGifts final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<receivedGift>> gifts_;
  static jfieldID gifts_fieldID;
  bool are_notifications_enabled_;
  static jfieldID are_notifications_enabled_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  receivedGifts();

  receivedGifts(int32 total_count_, array<object_ptr<receivedGift>> &&gifts_, bool are_notifications_enabled_, string const &next_offset_);

  static const std::int32_t ID = 1237114400;

  static object_ptr<receivedGifts> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class recommendedChatFolder final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolder> folder_;
  static jfieldID folder_fieldID;
  string description_;
  static jfieldID description_fieldID;

  recommendedChatFolder();

  recommendedChatFolder(object_ptr<chatFolder> &&folder_, string const &description_);

  static const std::int32_t ID = -2116569930;

  static object_ptr<recommendedChatFolder> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class recommendedChatFolder;

class recommendedChatFolders final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<recommendedChatFolder>> chat_folders_;
  static jfieldID chat_folders_fieldID;

  recommendedChatFolders();

  explicit recommendedChatFolders(array<object_ptr<recommendedChatFolder>> &&chat_folders_);

  static const std::int32_t ID = -739217656;

  static object_ptr<recommendedChatFolders> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class recoveryEmailAddress final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string recovery_email_address_;
  static jfieldID recovery_email_address_fieldID;

  recoveryEmailAddress();

  explicit recoveryEmailAddress(string const &recovery_email_address_);

  static const std::int32_t ID = 1290526187;

  static object_ptr<recoveryEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class remoteFile final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string unique_id_;
  static jfieldID unique_id_fieldID;
  bool is_uploading_active_;
  static jfieldID is_uploading_active_fieldID;
  bool is_uploading_completed_;
  static jfieldID is_uploading_completed_fieldID;
  int53 uploaded_size_;
  static jfieldID uploaded_size_fieldID;

  remoteFile();

  remoteFile(string const &id_, string const &unique_id_, bool is_uploading_active_, bool is_uploading_completed_, int53 uploaded_size_);

  static const std::int32_t ID = 747731030;

  static object_ptr<remoteFile> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class inlineKeyboardButton;

class keyboardButton;

class ReplyMarkup: public Object {
 public:
  static jclass Class;

  static object_ptr<ReplyMarkup> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class replyMarkupRemoveKeyboard final : public ReplyMarkup {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_personal_;
  static jfieldID is_personal_fieldID;

  replyMarkupRemoveKeyboard();

  explicit replyMarkupRemoveKeyboard(bool is_personal_);

  static const std::int32_t ID = -691252879;

  static object_ptr<ReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class replyMarkupForceReply final : public ReplyMarkup {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_personal_;
  static jfieldID is_personal_fieldID;
  string input_field_placeholder_;
  static jfieldID input_field_placeholder_fieldID;

  replyMarkupForceReply();

  replyMarkupForceReply(bool is_personal_, string const &input_field_placeholder_);

  static const std::int32_t ID = 1101461919;

  static object_ptr<ReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class replyMarkupShowKeyboard final : public ReplyMarkup {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<array<object_ptr<keyboardButton>>> rows_;
  static jfieldID rows_fieldID;
  bool is_persistent_;
  static jfieldID is_persistent_fieldID;
  bool resize_keyboard_;
  static jfieldID resize_keyboard_fieldID;
  bool one_time_;
  static jfieldID one_time_fieldID;
  bool is_personal_;
  static jfieldID is_personal_fieldID;
  string input_field_placeholder_;
  static jfieldID input_field_placeholder_fieldID;

  replyMarkupShowKeyboard();

  replyMarkupShowKeyboard(array<array<object_ptr<keyboardButton>>> &&rows_, bool is_persistent_, bool resize_keyboard_, bool one_time_, bool is_personal_, string const &input_field_placeholder_);

  static const std::int32_t ID = -791495984;

  static object_ptr<ReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class replyMarkupInlineKeyboard final : public ReplyMarkup {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<array<object_ptr<inlineKeyboardButton>>> rows_;
  static jfieldID rows_fieldID;

  replyMarkupInlineKeyboard();

  explicit replyMarkupInlineKeyboard(array<array<object_ptr<inlineKeyboardButton>>> &&rows_);

  static const std::int32_t ID = -619317658;

  static object_ptr<ReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportOption;

class ReportChatResult: public Object {
 public:
  static jclass Class;

  static object_ptr<ReportChatResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reportChatResultOk final : public ReportChatResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportChatResultOk();

  static const std::int32_t ID = 1209685373;

  static object_ptr<ReportChatResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportChatResultOptionRequired final : public ReportChatResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  array<object_ptr<reportOption>> options_;
  static jfieldID options_fieldID;

  reportChatResultOptionRequired();

  reportChatResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_);

  static const std::int32_t ID = -881375669;

  static object_ptr<ReportChatResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportChatResultTextRequired final : public ReportChatResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes option_id_;
  static jfieldID option_id_fieldID;
  bool is_optional_;
  static jfieldID is_optional_fieldID;

  reportChatResultTextRequired();

  reportChatResultTextRequired(bytes const &option_id_, bool is_optional_);

  static const std::int32_t ID = -1949552447;

  static object_ptr<ReportChatResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportChatResultMessagesRequired final : public ReportChatResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportChatResultMessagesRequired();

  static const std::int32_t ID = 106043280;

  static object_ptr<ReportChatResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes id_;
  static jfieldID id_fieldID;
  string text_;
  static jfieldID text_fieldID;

  reportOption();

  reportOption(bytes const &id_, string const &text_);

  static const std::int32_t ID = 1106390048;

  static object_ptr<reportOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReportReason: public Object {
 public:
  static jclass Class;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonSpam final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonSpam();

  static const std::int32_t ID = -1207032897;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonViolence final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonViolence();

  static const std::int32_t ID = 2038679353;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonPornography final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonPornography();

  static const std::int32_t ID = 1306467575;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonChildAbuse final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonChildAbuse();

  static const std::int32_t ID = 761086718;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonCopyright final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonCopyright();

  static const std::int32_t ID = 1474441135;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonUnrelatedLocation final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonUnrelatedLocation();

  static const std::int32_t ID = 87562288;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonFake final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonFake();

  static const std::int32_t ID = 352862176;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonIllegalDrugs final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonIllegalDrugs();

  static const std::int32_t ID = -61599200;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonPersonalDetails final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonPersonalDetails();

  static const std::int32_t ID = -1588882414;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportReasonCustom final : public ReportReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportReasonCustom();

  static const std::int32_t ID = -1380459917;

  static object_ptr<ReportReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportOption;

class ReportSponsoredResult: public Object {
 public:
  static jclass Class;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reportSponsoredResultOk final : public ReportSponsoredResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportSponsoredResultOk();

  static const std::int32_t ID = -128473456;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportSponsoredResultFailed final : public ReportSponsoredResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportSponsoredResultFailed();

  static const std::int32_t ID = 1231714278;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportSponsoredResultOptionRequired final : public ReportSponsoredResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  array<object_ptr<reportOption>> options_;
  static jfieldID options_fieldID;

  reportSponsoredResultOptionRequired();

  reportSponsoredResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_);

  static const std::int32_t ID = -1646687318;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportSponsoredResultAdsHidden final : public ReportSponsoredResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportSponsoredResultAdsHidden();

  static const std::int32_t ID = -372279531;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportSponsoredResultPremiumRequired final : public ReportSponsoredResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportSponsoredResultPremiumRequired();

  static const std::int32_t ID = -55411887;

  static object_ptr<ReportSponsoredResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportOption;

class ReportStoryResult: public Object {
 public:
  static jclass Class;

  static object_ptr<ReportStoryResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class reportStoryResultOk final : public ReportStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  reportStoryResultOk();

  static const std::int32_t ID = -1405328461;

  static object_ptr<ReportStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportStoryResultOptionRequired final : public ReportStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  array<object_ptr<reportOption>> options_;
  static jfieldID options_fieldID;

  reportStoryResultOptionRequired();

  reportStoryResultOptionRequired(string const &title_, array<object_ptr<reportOption>> &&options_);

  static const std::int32_t ID = 1632974839;

  static object_ptr<ReportStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class reportStoryResultTextRequired final : public ReportStoryResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes option_id_;
  static jfieldID option_id_fieldID;
  bool is_optional_;
  static jfieldID is_optional_fieldID;

  reportStoryResultTextRequired();

  reportStoryResultTextRequired(bytes const &option_id_, bool is_optional_);

  static const std::int32_t ID = 334339473;

  static object_ptr<ReportStoryResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ResendCodeReason: public Object {
 public:
  static jclass Class;

  static object_ptr<ResendCodeReason> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class resendCodeReasonUserRequest final : public ResendCodeReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resendCodeReasonUserRequest();

  static const std::int32_t ID = -441923456;

  static object_ptr<ResendCodeReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class resendCodeReasonVerificationFailed final : public ResendCodeReason {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string error_message_;
  static jfieldID error_message_fieldID;

  resendCodeReasonVerificationFailed();

  explicit resendCodeReasonVerificationFailed(string const &error_message_);

  static const std::int32_t ID = 529870273;

  static object_ptr<ResendCodeReason> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ResetPasswordResult: public Object {
 public:
  static jclass Class;

  static object_ptr<ResetPasswordResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class resetPasswordResultOk final : public ResetPasswordResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetPasswordResultOk();

  static const std::int32_t ID = -1397267463;

  static object_ptr<ResetPasswordResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class resetPasswordResultPending final : public ResetPasswordResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 pending_reset_date_;
  static jfieldID pending_reset_date_fieldID;

  resetPasswordResultPending();

  explicit resetPasswordResultPending(int32 pending_reset_date_);

  static const std::int32_t ID = 1193925721;

  static object_ptr<ResetPasswordResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class resetPasswordResultDeclined final : public ResetPasswordResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 retry_date_;
  static jfieldID retry_date_fieldID;

  resetPasswordResultDeclined();

  explicit resetPasswordResultDeclined(int32 retry_date_);

  static const std::int32_t ID = -1202200373;

  static object_ptr<ResetPasswordResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class RevenueWithdrawalState: public Object {
 public:
  static jclass Class;

  static object_ptr<RevenueWithdrawalState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class revenueWithdrawalStatePending final : public RevenueWithdrawalState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  revenueWithdrawalStatePending();

  static const std::int32_t ID = 1563512741;

  static object_ptr<RevenueWithdrawalState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class revenueWithdrawalStateSucceeded final : public RevenueWithdrawalState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 date_;
  static jfieldID date_fieldID;
  string url_;
  static jfieldID url_fieldID;

  revenueWithdrawalStateSucceeded();

  revenueWithdrawalStateSucceeded(int32 date_, string const &url_);

  static const std::int32_t ID = 265375242;

  static object_ptr<RevenueWithdrawalState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class revenueWithdrawalStateFailed final : public RevenueWithdrawalState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  revenueWithdrawalStateFailed();

  static const std::int32_t ID = -12504951;

  static object_ptr<RevenueWithdrawalState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class RichText;

class document;

class RichText: public Object {
 public:
  static jclass Class;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class richTextPlain final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  richTextPlain();

  explicit richTextPlain(string const &text_);

  static const std::int32_t ID = 482617702;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextBold final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextBold();

  explicit richTextBold(object_ptr<RichText> &&text_);

  static const std::int32_t ID = 1670844268;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextItalic final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextItalic();

  explicit richTextItalic(object_ptr<RichText> &&text_);

  static const std::int32_t ID = 1853354047;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextUnderline final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextUnderline();

  explicit richTextUnderline(object_ptr<RichText> &&text_);

  static const std::int32_t ID = -536019572;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextStrikethrough final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextStrikethrough();

  explicit richTextStrikethrough(object_ptr<RichText> &&text_);

  static const std::int32_t ID = 723413585;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextFixed final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextFixed();

  explicit richTextFixed(object_ptr<RichText> &&text_);

  static const std::int32_t ID = -1271496249;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextUrl final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string url_;
  static jfieldID url_fieldID;
  bool is_cached_;
  static jfieldID is_cached_fieldID;

  richTextUrl();

  richTextUrl(object_ptr<RichText> &&text_, string const &url_, bool is_cached_);

  static const std::int32_t ID = 83939092;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextEmailAddress final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string email_address_;
  static jfieldID email_address_fieldID;

  richTextEmailAddress();

  richTextEmailAddress(object_ptr<RichText> &&text_, string const &email_address_);

  static const std::int32_t ID = 40018679;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextSubscript final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextSubscript();

  explicit richTextSubscript(object_ptr<RichText> &&text_);

  static const std::int32_t ID = -868197812;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextSuperscript final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextSuperscript();

  explicit richTextSuperscript(object_ptr<RichText> &&text_);

  static const std::int32_t ID = -382241437;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextMarked final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;

  richTextMarked();

  explicit richTextMarked(object_ptr<RichText> &&text_);

  static const std::int32_t ID = -1271999614;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextPhoneNumber final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string phone_number_;
  static jfieldID phone_number_fieldID;

  richTextPhoneNumber();

  richTextPhoneNumber(object_ptr<RichText> &&text_, string const &phone_number_);

  static const std::int32_t ID = 128521539;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextIcon final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<document> document_;
  static jfieldID document_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;

  richTextIcon();

  richTextIcon(object_ptr<document> &&document_, int32 width_, int32 height_);

  static const std::int32_t ID = -1480316158;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextReference final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string anchor_name_;
  static jfieldID anchor_name_fieldID;
  string url_;
  static jfieldID url_fieldID;

  richTextReference();

  richTextReference(object_ptr<RichText> &&text_, string const &anchor_name_, string const &url_);

  static const std::int32_t ID = -1147530634;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextAnchor final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  richTextAnchor();

  explicit richTextAnchor(string const &name_);

  static const std::int32_t ID = 1316950068;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTextAnchorLink final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RichText> text_;
  static jfieldID text_fieldID;
  string anchor_name_;
  static jfieldID anchor_name_fieldID;
  string url_;
  static jfieldID url_fieldID;

  richTextAnchorLink();

  richTextAnchorLink(object_ptr<RichText> &&text_, string const &anchor_name_, string const &url_);

  static const std::int32_t ID = -1541418282;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class richTexts final : public RichText {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<RichText>> texts_;
  static jfieldID texts_fieldID;

  richTexts();

  explicit richTexts(array<object_ptr<RichText>> &&texts_);

  static const std::int32_t ID = 1647457821;

  static object_ptr<RichText> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class rtmpUrl final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  string stream_key_;
  static jfieldID stream_key_fieldID;

  rtmpUrl();

  rtmpUrl(string const &url_, string const &stream_key_);

  static const std::int32_t ID = 1009302613;

  static object_ptr<rtmpUrl> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class savedCredentials final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;

  savedCredentials();

  savedCredentials(string const &id_, string const &title_);

  static const std::int32_t ID = -370273060;

  static object_ptr<savedCredentials> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class savedMessagesTag final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> tag_;
  static jfieldID tag_fieldID;
  string label_;
  static jfieldID label_fieldID;
  int32 count_;
  static jfieldID count_fieldID;

  savedMessagesTag();

  savedMessagesTag(object_ptr<ReactionType> &&tag_, string const &label_, int32 count_);

  static const std::int32_t ID = 1785183329;

  static object_ptr<savedMessagesTag> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class savedMessagesTag;

class savedMessagesTags final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<savedMessagesTag>> tags_;
  static jfieldID tags_fieldID;

  savedMessagesTags();

  explicit savedMessagesTags(array<object_ptr<savedMessagesTag>> &&tags_);

  static const std::int32_t ID = -1749291430;

  static object_ptr<savedMessagesTags> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SavedMessagesTopicType;

class draftMessage;

class message;

class savedMessagesTopic final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  object_ptr<SavedMessagesTopicType> type_;
  static jfieldID type_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  int64 order_;
  static jfieldID order_fieldID;
  object_ptr<message> last_message_;
  static jfieldID last_message_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;

  savedMessagesTopic();

  savedMessagesTopic(int53 id_, object_ptr<SavedMessagesTopicType> &&type_, bool is_pinned_, int64 order_, object_ptr<message> &&last_message_, object_ptr<draftMessage> &&draft_message_);

  static const std::int32_t ID = -760684124;

  static object_ptr<savedMessagesTopic> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SavedMessagesTopicType: public Object {
 public:
  static jclass Class;

  static object_ptr<SavedMessagesTopicType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class savedMessagesTopicTypeMyNotes final : public SavedMessagesTopicType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  savedMessagesTopicTypeMyNotes();

  static const std::int32_t ID = -1282784779;

  static object_ptr<SavedMessagesTopicType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class savedMessagesTopicTypeAuthorHidden final : public SavedMessagesTopicType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  savedMessagesTopicTypeAuthorHidden();

  static const std::int32_t ID = 1882997141;

  static object_ptr<SavedMessagesTopicType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class savedMessagesTopicTypeSavedFromChat final : public SavedMessagesTopicType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  savedMessagesTopicTypeSavedFromChat();

  explicit savedMessagesTopicTypeSavedFromChat(int53 chat_id_);

  static const std::int32_t ID = -1723880104;

  static object_ptr<SavedMessagesTopicType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class scopeAutosaveSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool autosave_photos_;
  static jfieldID autosave_photos_fieldID;
  bool autosave_videos_;
  static jfieldID autosave_videos_fieldID;
  int53 max_video_file_size_;
  static jfieldID max_video_file_size_fieldID;

  scopeAutosaveSettings();

  scopeAutosaveSettings(bool autosave_photos_, bool autosave_videos_, int53 max_video_file_size_);

  static const std::int32_t ID = 1546821427;

  static object_ptr<scopeAutosaveSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class scopeNotificationSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 mute_for_;
  static jfieldID mute_for_fieldID;
  int64 sound_id_;
  static jfieldID sound_id_fieldID;
  bool show_preview_;
  static jfieldID show_preview_fieldID;
  bool use_default_mute_stories_;
  static jfieldID use_default_mute_stories_fieldID;
  bool mute_stories_;
  static jfieldID mute_stories_fieldID;
  int64 story_sound_id_;
  static jfieldID story_sound_id_fieldID;
  bool show_story_poster_;
  static jfieldID show_story_poster_fieldID;
  bool disable_pinned_message_notifications_;
  static jfieldID disable_pinned_message_notifications_fieldID;
  bool disable_mention_notifications_;
  static jfieldID disable_mention_notifications_fieldID;

  scopeNotificationSettings();

  scopeNotificationSettings(int32 mute_for_, int64 sound_id_, bool show_preview_, bool use_default_mute_stories_, bool mute_stories_, int64 story_sound_id_, bool show_story_poster_, bool disable_pinned_message_notifications_, bool disable_mention_notifications_);

  static const std::int32_t ID = 88369150;

  static object_ptr<scopeNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesChatTypeFilter: public Object {
 public:
  static jclass Class;

  static object_ptr<SearchMessagesChatTypeFilter> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesChatTypeFilterPrivate final : public SearchMessagesChatTypeFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesChatTypeFilterPrivate();

  static const std::int32_t ID = 1169248975;

  static object_ptr<SearchMessagesChatTypeFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesChatTypeFilterGroup final : public SearchMessagesChatTypeFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesChatTypeFilterGroup();

  static const std::int32_t ID = -2059426022;

  static object_ptr<SearchMessagesChatTypeFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesChatTypeFilterChannel final : public SearchMessagesChatTypeFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesChatTypeFilterChannel();

  static const std::int32_t ID = -773540139;

  static object_ptr<SearchMessagesChatTypeFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter: public Object {
 public:
  static jclass Class;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterEmpty final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterEmpty();

  static const std::int32_t ID = -869395657;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterAnimation final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterAnimation();

  static const std::int32_t ID = -155713339;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterAudio final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterAudio();

  static const std::int32_t ID = 867505275;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterDocument final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterDocument();

  static const std::int32_t ID = 1526331215;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterPhoto final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterPhoto();

  static const std::int32_t ID = 925932293;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterVideo final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterVideo();

  static const std::int32_t ID = 115538222;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterVoiceNote final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterVoiceNote();

  static const std::int32_t ID = 1841439357;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterPhotoAndVideo final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterPhotoAndVideo();

  static const std::int32_t ID = 1352130963;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterUrl final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterUrl();

  static const std::int32_t ID = -1828724341;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterChatPhoto final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterChatPhoto();

  static const std::int32_t ID = -1247751329;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterVideoNote final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterVideoNote();

  static const std::int32_t ID = 564323321;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterVoiceAndVideoNote final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterVoiceAndVideoNote();

  static const std::int32_t ID = 664174819;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterMention final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterMention();

  static const std::int32_t ID = 2001258652;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterUnreadMention final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterUnreadMention();

  static const std::int32_t ID = -95769149;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterUnreadReaction final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterUnreadReaction();

  static const std::int32_t ID = -1379651328;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterFailedToSend final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterFailedToSend();

  static const std::int32_t ID = -596322564;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class searchMessagesFilterPinned final : public SearchMessagesFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  searchMessagesFilterPinned();

  static const std::int32_t ID = 371805512;

  static object_ptr<SearchMessagesFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class seconds final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double seconds_;
  static jfieldID seconds_fieldID;

  seconds();

  explicit seconds(double seconds_);

  static const std::int32_t ID = 959899022;

  static object_ptr<seconds> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SecretChatState;

class secretChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<SecretChatState> state_;
  static jfieldID state_fieldID;
  bool is_outbound_;
  static jfieldID is_outbound_fieldID;
  bytes key_hash_;
  static jfieldID key_hash_fieldID;
  int32 layer_;
  static jfieldID layer_fieldID;

  secretChat();

  secretChat(int32 id_, int53 user_id_, object_ptr<SecretChatState> &&state_, bool is_outbound_, bytes const &key_hash_, int32 layer_);

  static const std::int32_t ID = -676918325;

  static object_ptr<secretChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SecretChatState: public Object {
 public:
  static jclass Class;

  static object_ptr<SecretChatState> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class secretChatStatePending final : public SecretChatState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  secretChatStatePending();

  static const std::int32_t ID = -1637050756;

  static object_ptr<SecretChatState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class secretChatStateReady final : public SecretChatState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  secretChatStateReady();

  static const std::int32_t ID = -1611352087;

  static object_ptr<SecretChatState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class secretChatStateClosed final : public SecretChatState {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  secretChatStateClosed();

  static const std::int32_t ID = -1945106707;

  static object_ptr<SecretChatState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class gift;

class upgradedGift;

class SentGift: public Object {
 public:
  static jclass Class;

  static object_ptr<SentGift> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sentGiftRegular final : public SentGift {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;

  sentGiftRegular();

  explicit sentGiftRegular(object_ptr<gift> &&gift_);

  static const std::int32_t ID = 594062617;

  static object_ptr<SentGift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sentGiftUpgraded final : public SentGift {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;

  sentGiftUpgraded();

  explicit sentGiftUpgraded(object_ptr<upgradedGift> &&gift_);

  static const std::int32_t ID = 627524736;

  static object_ptr<SentGift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sentWebAppMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;

  sentWebAppMessage();

  explicit sentWebAppMessage(string const &inline_message_id_);

  static const std::int32_t ID = 1243934400;

  static object_ptr<sentWebAppMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SessionType;

class session final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  bool is_current_;
  static jfieldID is_current_fieldID;
  bool is_password_pending_;
  static jfieldID is_password_pending_fieldID;
  bool is_unconfirmed_;
  static jfieldID is_unconfirmed_fieldID;
  bool can_accept_secret_chats_;
  static jfieldID can_accept_secret_chats_fieldID;
  bool can_accept_calls_;
  static jfieldID can_accept_calls_fieldID;
  object_ptr<SessionType> type_;
  static jfieldID type_fieldID;
  int32 api_id_;
  static jfieldID api_id_fieldID;
  string application_name_;
  static jfieldID application_name_fieldID;
  string application_version_;
  static jfieldID application_version_fieldID;
  bool is_official_application_;
  static jfieldID is_official_application_fieldID;
  string device_model_;
  static jfieldID device_model_fieldID;
  string platform_;
  static jfieldID platform_fieldID;
  string system_version_;
  static jfieldID system_version_fieldID;
  int32 log_in_date_;
  static jfieldID log_in_date_fieldID;
  int32 last_active_date_;
  static jfieldID last_active_date_fieldID;
  string ip_address_;
  static jfieldID ip_address_fieldID;
  string location_;
  static jfieldID location_fieldID;

  session();

  session(int64 id_, bool is_current_, bool is_password_pending_, bool is_unconfirmed_, bool can_accept_secret_chats_, bool can_accept_calls_, object_ptr<SessionType> &&type_, int32 api_id_, string const &application_name_, string const &application_version_, bool is_official_application_, string const &device_model_, string const &platform_, string const &system_version_, int32 log_in_date_, int32 last_active_date_, string const &ip_address_, string const &location_);

  static const std::int32_t ID = 158702140;

  static object_ptr<session> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SessionType: public Object {
 public:
  static jclass Class;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeAndroid final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeAndroid();

  static const std::int32_t ID = -2071764840;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeApple final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeApple();

  static const std::int32_t ID = -1818635701;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeBrave final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeBrave();

  static const std::int32_t ID = -1216812563;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeChrome final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeChrome();

  static const std::int32_t ID = 1573464425;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeEdge final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeEdge();

  static const std::int32_t ID = -538916005;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeFirefox final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeFirefox();

  static const std::int32_t ID = 2122579364;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeIpad final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeIpad();

  static const std::int32_t ID = 1294647023;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeIphone final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeIphone();

  static const std::int32_t ID = 97616573;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeLinux final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeLinux();

  static const std::int32_t ID = -1487422871;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeMac final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeMac();

  static const std::int32_t ID = -612250975;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeOpera final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeOpera();

  static const std::int32_t ID = -1463673734;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeSafari final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeSafari();

  static const std::int32_t ID = 710646873;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeUbuntu final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeUbuntu();

  static const std::int32_t ID = 1569680069;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeUnknown final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeUnknown();

  static const std::int32_t ID = 233926704;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeVivaldi final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeVivaldi();

  static const std::int32_t ID = 1120503279;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeWindows final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeWindows();

  static const std::int32_t ID = -1676512600;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sessionTypeXbox final : public SessionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  sessionTypeXbox();

  static const std::int32_t ID = 1856216492;

  static object_ptr<SessionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class session;

class sessions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<session>> sessions_;
  static jfieldID sessions_fieldID;
  int32 inactive_session_ttl_days_;
  static jfieldID inactive_session_ttl_days_fieldID;

  sessions();

  sessions(array<object_ptr<session>> &&sessions_, int32 inactive_session_ttl_days_);

  static const std::int32_t ID = 842912274;

  static object_ptr<sessions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class sharedChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string username_;
  static jfieldID username_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  sharedChat();

  sharedChat(int53 chat_id_, string const &title_, string const &username_, object_ptr<photo> &&photo_);

  static const std::int32_t ID = 1250406426;

  static object_ptr<sharedChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class sharedUser final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;
  string username_;
  static jfieldID username_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  sharedUser();

  sharedUser(int53 user_id_, string const &first_name_, string const &last_name_, string const &username_, object_ptr<photo> &&photo_);

  static const std::int32_t ID = 293020919;

  static object_ptr<sharedUser> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class labeledPricePart;

class shippingOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  array<object_ptr<labeledPricePart>> price_parts_;
  static jfieldID price_parts_fieldID;

  shippingOption();

  shippingOption(string const &id_, string const &title_, array<object_ptr<labeledPricePart>> &&price_parts_);

  static const std::int32_t ID = 1425690001;

  static object_ptr<shippingOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class error;

class SpeechRecognitionResult: public Object {
 public:
  static jclass Class;

  static object_ptr<SpeechRecognitionResult> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class speechRecognitionResultPending final : public SpeechRecognitionResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string partial_text_;
  static jfieldID partial_text_fieldID;

  speechRecognitionResultPending();

  explicit speechRecognitionResultPending(string const &partial_text_);

  static const std::int32_t ID = -1631810048;

  static object_ptr<SpeechRecognitionResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class speechRecognitionResultText final : public SpeechRecognitionResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  speechRecognitionResultText();

  explicit speechRecognitionResultText(string const &text_);

  static const std::int32_t ID = -2132377123;

  static object_ptr<SpeechRecognitionResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class speechRecognitionResultError final : public SpeechRecognitionResult {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<error> error_;
  static jfieldID error_fieldID;

  speechRecognitionResultError();

  explicit speechRecognitionResultError(object_ptr<error> &&error_);

  static const std::int32_t ID = 164774908;

  static object_ptr<SpeechRecognitionResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sponsoredChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 unique_id_;
  static jfieldID unique_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string sponsor_info_;
  static jfieldID sponsor_info_fieldID;
  string additional_info_;
  static jfieldID additional_info_fieldID;

  sponsoredChat();

  sponsoredChat(int53 unique_id_, int53 chat_id_, string const &sponsor_info_, string const &additional_info_);

  static const std::int32_t ID = -325763489;

  static object_ptr<sponsoredChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sponsoredChat;

class sponsoredChats final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<sponsoredChat>> chats_;
  static jfieldID chats_fieldID;

  sponsoredChats();

  explicit sponsoredChats(array<object_ptr<sponsoredChat>> &&chats_);

  static const std::int32_t ID = 536300641;

  static object_ptr<sponsoredChats> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageContent;

class messageSponsor;

class sponsoredMessage final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_recommended_;
  static jfieldID is_recommended_fieldID;
  bool can_be_reported_;
  static jfieldID can_be_reported_fieldID;
  object_ptr<MessageContent> content_;
  static jfieldID content_fieldID;
  object_ptr<messageSponsor> sponsor_;
  static jfieldID sponsor_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string button_text_;
  static jfieldID button_text_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;
  string additional_info_;
  static jfieldID additional_info_fieldID;

  sponsoredMessage();

  sponsoredMessage(int53 message_id_, bool is_recommended_, bool can_be_reported_, object_ptr<MessageContent> &&content_, object_ptr<messageSponsor> &&sponsor_, string const &title_, string const &button_text_, int32 accent_color_id_, int64 background_custom_emoji_id_, string const &additional_info_);

  static const std::int32_t ID = -1215476699;

  static object_ptr<sponsoredMessage> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sponsoredMessage;

class sponsoredMessages final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<sponsoredMessage>> messages_;
  static jfieldID messages_fieldID;
  int32 messages_between_;
  static jfieldID messages_between_fieldID;

  sponsoredMessages();

  sponsoredMessages(array<object_ptr<sponsoredMessage>> &&messages_, int32 messages_between_);

  static const std::int32_t ID = -537674389;

  static object_ptr<sponsoredMessages> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starAmount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  int32 nanostar_count_;
  static jfieldID nanostar_count_fieldID;

  starAmount();

  starAmount(int53 star_count_, int32 nanostar_count_);

  static const std::int32_t ID = 1863216512;

  static object_ptr<starAmount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starCount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  starCount();

  explicit starCount(int53 star_count_);

  static const std::int32_t ID = -1566395144;

  static object_ptr<starCount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starGiveawayWinnerOption;

class starGiveawayPaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  int32 yearly_boost_count_;
  static jfieldID yearly_boost_count_fieldID;
  array<object_ptr<starGiveawayWinnerOption>> winner_options_;
  static jfieldID winner_options_fieldID;
  bool is_default_;
  static jfieldID is_default_fieldID;
  bool is_additional_;
  static jfieldID is_additional_fieldID;

  starGiveawayPaymentOption();

  starGiveawayPaymentOption(string const &currency_, int53 amount_, int53 star_count_, string const &store_product_id_, int32 yearly_boost_count_, array<object_ptr<starGiveawayWinnerOption>> &&winner_options_, bool is_default_, bool is_additional_);

  static const std::int32_t ID = 565089625;

  static object_ptr<starGiveawayPaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starGiveawayPaymentOption;

class starGiveawayPaymentOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<starGiveawayPaymentOption>> options_;
  static jfieldID options_fieldID;

  starGiveawayPaymentOptions();

  explicit starGiveawayPaymentOptions(array<object_ptr<starGiveawayPaymentOption>> &&options_);

  static const std::int32_t ID = -1216716679;

  static object_ptr<starGiveawayPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starGiveawayWinnerOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int53 won_star_count_;
  static jfieldID won_star_count_fieldID;
  bool is_default_;
  static jfieldID is_default_fieldID;

  starGiveawayWinnerOption();

  starGiveawayWinnerOption(int32 winner_count_, int53 won_star_count_, bool is_default_);

  static const std::int32_t ID = -865888761;

  static object_ptr<starGiveawayWinnerOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starPaymentOption final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  bool is_additional_;
  static jfieldID is_additional_fieldID;

  starPaymentOption();

  starPaymentOption(string const &currency_, int53 amount_, int53 star_count_, string const &store_product_id_, bool is_additional_);

  static const std::int32_t ID = -1364056047;

  static object_ptr<starPaymentOption> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starPaymentOption;

class starPaymentOptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<starPaymentOption>> options_;
  static jfieldID options_fieldID;

  starPaymentOptions();

  explicit starPaymentOptions(array<object_ptr<starPaymentOption>> &&options_);

  static const std::int32_t ID = -423720498;

  static object_ptr<starPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class starRevenueStatus;

class starRevenueStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StatisticalGraph> revenue_by_day_graph_;
  static jfieldID revenue_by_day_graph_fieldID;
  object_ptr<starRevenueStatus> status_;
  static jfieldID status_fieldID;
  double usd_rate_;
  static jfieldID usd_rate_fieldID;

  starRevenueStatistics();

  starRevenueStatistics(object_ptr<StatisticalGraph> &&revenue_by_day_graph_, object_ptr<starRevenueStatus> &&status_, double usd_rate_);

  static const std::int32_t ID = -1121086889;

  static object_ptr<starRevenueStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starAmount;

class starRevenueStatus final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starAmount> total_amount_;
  static jfieldID total_amount_fieldID;
  object_ptr<starAmount> current_amount_;
  static jfieldID current_amount_fieldID;
  object_ptr<starAmount> available_amount_;
  static jfieldID available_amount_fieldID;
  bool withdrawal_enabled_;
  static jfieldID withdrawal_enabled_fieldID;
  int32 next_withdrawal_in_;
  static jfieldID next_withdrawal_in_fieldID;

  starRevenueStatus();

  starRevenueStatus(object_ptr<starAmount> &&total_amount_, object_ptr<starAmount> &&current_amount_, object_ptr<starAmount> &&available_amount_, bool withdrawal_enabled_, int32 next_withdrawal_in_);

  static const std::int32_t ID = 2006266600;

  static object_ptr<starRevenueStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StarSubscriptionType;

class starSubscriptionPricing;

class starSubscription final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;
  bool is_canceled_;
  static jfieldID is_canceled_fieldID;
  bool is_expiring_;
  static jfieldID is_expiring_fieldID;
  object_ptr<starSubscriptionPricing> pricing_;
  static jfieldID pricing_fieldID;
  object_ptr<StarSubscriptionType> type_;
  static jfieldID type_fieldID;

  starSubscription();

  starSubscription(string const &id_, int53 chat_id_, int32 expiration_date_, bool is_canceled_, bool is_expiring_, object_ptr<starSubscriptionPricing> &&pricing_, object_ptr<StarSubscriptionType> &&type_);

  static const std::int32_t ID = 976753141;

  static object_ptr<starSubscription> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptionPricing final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 period_;
  static jfieldID period_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  starSubscriptionPricing();

  starSubscriptionPricing(int32 period_, int53 star_count_);

  static const std::int32_t ID = -1767733162;

  static object_ptr<starSubscriptionPricing> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class StarSubscriptionType: public Object {
 public:
  static jclass Class;

  static object_ptr<StarSubscriptionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptionTypeChannel final : public StarSubscriptionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_reuse_;
  static jfieldID can_reuse_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  starSubscriptionTypeChannel();

  starSubscriptionTypeChannel(bool can_reuse_, string const &invite_link_);

  static const std::int32_t ID = -1030048011;

  static object_ptr<StarSubscriptionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptionTypeBot final : public StarSubscriptionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_canceled_by_bot_;
  static jfieldID is_canceled_by_bot_fieldID;
  string title_;
  static jfieldID title_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  string invoice_link_;
  static jfieldID invoice_link_fieldID;

  starSubscriptionTypeBot();

  starSubscriptionTypeBot(bool is_canceled_by_bot_, string const &title_, object_ptr<photo> &&photo_, string const &invoice_link_);

  static const std::int32_t ID = 226024914;

  static object_ptr<StarSubscriptionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starAmount;

class starSubscription;

class starSubscriptions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starAmount> star_amount_;
  static jfieldID star_amount_fieldID;
  array<object_ptr<starSubscription>> subscriptions_;
  static jfieldID subscriptions_fieldID;
  int53 required_star_count_;
  static jfieldID required_star_count_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  starSubscriptions();

  starSubscriptions(object_ptr<starAmount> &&star_amount_, array<object_ptr<starSubscription>> &&subscriptions_, int53 required_star_count_, string const &next_offset_);

  static const std::int32_t ID = 151169395;

  static object_ptr<starSubscriptions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StarTransactionType;

class starAmount;

class starTransaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  object_ptr<starAmount> star_amount_;
  static jfieldID star_amount_fieldID;
  bool is_refund_;
  static jfieldID is_refund_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<StarTransactionType> type_;
  static jfieldID type_fieldID;

  starTransaction();

  starTransaction(string const &id_, object_ptr<starAmount> &&star_amount_, bool is_refund_, int32 date_, object_ptr<StarTransactionType> &&type_);

  static const std::int32_t ID = 2139228816;

  static object_ptr<starTransaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StarTransactionDirection: public Object {
 public:
  static jclass Class;

  static object_ptr<StarTransactionDirection> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionDirectionIncoming final : public StarTransactionDirection {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionDirectionIncoming();

  static const std::int32_t ID = -1295335866;

  static object_ptr<StarTransactionDirection> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionDirectionOutgoing final : public StarTransactionDirection {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionDirectionOutgoing();

  static const std::int32_t ID = 1854125472;

  static object_ptr<StarTransactionDirection> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class PaidMedia;

class RevenueWithdrawalState;

class affiliateInfo;

class gift;

class productInfo;

class starAmount;

class sticker;

class upgradedGift;

class StarTransactionType: public Object {
 public:
  static jclass Class;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypePremiumBotDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypePremiumBotDeposit();

  static const std::int32_t ID = -663156466;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeAppStoreDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypeAppStoreDeposit();

  static const std::int32_t ID = 136853825;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGooglePlayDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypeGooglePlayDeposit();

  static const std::int32_t ID = -323111338;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeFragmentDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypeFragmentDeposit();

  static const std::int32_t ID = 123887172;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeUserDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  starTransactionTypeUserDeposit();

  starTransactionTypeUserDeposit(int53 user_id_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = 204085481;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGiveawayDeposit final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 giveaway_message_id_;
  static jfieldID giveaway_message_id_fieldID;

  starTransactionTypeGiveawayDeposit();

  starTransactionTypeGiveawayDeposit(int53 chat_id_, int53 giveaway_message_id_);

  static const std::int32_t ID = -1318977338;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeFragmentWithdrawal final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<RevenueWithdrawalState> withdrawal_state_;
  static jfieldID withdrawal_state_fieldID;

  starTransactionTypeFragmentWithdrawal();

  explicit starTransactionTypeFragmentWithdrawal(object_ptr<RevenueWithdrawalState> &&withdrawal_state_);

  static const std::int32_t ID = -1355142766;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeTelegramAdsWithdrawal final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypeTelegramAdsWithdrawal();

  static const std::int32_t ID = -1517386647;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeTelegramApiUsage final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 request_count_;
  static jfieldID request_count_fieldID;

  starTransactionTypeTelegramApiUsage();

  explicit starTransactionTypeTelegramApiUsage(int32 request_count_);

  static const std::int32_t ID = 665332478;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotPaidMediaPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  array<object_ptr<PaidMedia>> media_;
  static jfieldID media_fieldID;

  starTransactionTypeBotPaidMediaPurchase();

  starTransactionTypeBotPaidMediaPurchase(int53 user_id_, array<object_ptr<PaidMedia>> &&media_);

  static const std::int32_t ID = 976645509;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotPaidMediaSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  array<object_ptr<PaidMedia>> media_;
  static jfieldID media_fieldID;
  string payload_;
  static jfieldID payload_fieldID;
  object_ptr<affiliateInfo> affiliate_;
  static jfieldID affiliate_fieldID;

  starTransactionTypeBotPaidMediaSale();

  starTransactionTypeBotPaidMediaSale(int53 user_id_, array<object_ptr<PaidMedia>> &&media_, string const &payload_, object_ptr<affiliateInfo> &&affiliate_);

  static const std::int32_t ID = -1034408372;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelPaidMediaPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  array<object_ptr<PaidMedia>> media_;
  static jfieldID media_fieldID;

  starTransactionTypeChannelPaidMediaPurchase();

  starTransactionTypeChannelPaidMediaPurchase(int53 chat_id_, int53 message_id_, array<object_ptr<PaidMedia>> &&media_);

  static const std::int32_t ID = -1321281338;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelPaidMediaSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  array<object_ptr<PaidMedia>> media_;
  static jfieldID media_fieldID;

  starTransactionTypeChannelPaidMediaSale();

  starTransactionTypeChannelPaidMediaSale(int53 user_id_, int53 message_id_, array<object_ptr<PaidMedia>> &&media_);

  static const std::int32_t ID = 52587085;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotInvoicePurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;

  starTransactionTypeBotInvoicePurchase();

  starTransactionTypeBotInvoicePurchase(int53 user_id_, object_ptr<productInfo> &&product_info_);

  static const std::int32_t ID = 501066764;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotInvoiceSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;
  bytes invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  object_ptr<affiliateInfo> affiliate_;
  static jfieldID affiliate_fieldID;

  starTransactionTypeBotInvoiceSale();

  starTransactionTypeBotInvoiceSale(int53 user_id_, object_ptr<productInfo> &&product_info_, bytes const &invoice_payload_, object_ptr<affiliateInfo> &&affiliate_);

  static const std::int32_t ID = 1534954799;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotSubscriptionPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 subscription_period_;
  static jfieldID subscription_period_fieldID;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;

  starTransactionTypeBotSubscriptionPurchase();

  starTransactionTypeBotSubscriptionPurchase(int53 user_id_, int32 subscription_period_, object_ptr<productInfo> &&product_info_);

  static const std::int32_t ID = 1086264149;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBotSubscriptionSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 subscription_period_;
  static jfieldID subscription_period_fieldID;
  object_ptr<productInfo> product_info_;
  static jfieldID product_info_fieldID;
  bytes invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  object_ptr<affiliateInfo> affiliate_;
  static jfieldID affiliate_fieldID;

  starTransactionTypeBotSubscriptionSale();

  starTransactionTypeBotSubscriptionSale(int53 user_id_, int32 subscription_period_, object_ptr<productInfo> &&product_info_, bytes const &invoice_payload_, object_ptr<affiliateInfo> &&affiliate_);

  static const std::int32_t ID = 526936201;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelSubscriptionPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 subscription_period_;
  static jfieldID subscription_period_fieldID;

  starTransactionTypeChannelSubscriptionPurchase();

  starTransactionTypeChannelSubscriptionPurchase(int53 chat_id_, int32 subscription_period_);

  static const std::int32_t ID = 940487633;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelSubscriptionSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 subscription_period_;
  static jfieldID subscription_period_fieldID;

  starTransactionTypeChannelSubscriptionSale();

  starTransactionTypeChannelSubscriptionSale(int53 user_id_, int32 subscription_period_);

  static const std::int32_t ID = -32342910;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGiftPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;

  starTransactionTypeGiftPurchase();

  starTransactionTypeGiftPurchase(object_ptr<MessageSender> &&owner_id_, object_ptr<gift> &&gift_);

  static const std::int32_t ID = -1819045664;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGiftTransfer final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;

  starTransactionTypeGiftTransfer();

  starTransactionTypeGiftTransfer(object_ptr<MessageSender> &&owner_id_, object_ptr<upgradedGift> &&gift_);

  static const std::int32_t ID = 9835767;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGiftSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<gift> gift_;
  static jfieldID gift_fieldID;

  starTransactionTypeGiftSale();

  starTransactionTypeGiftSale(int53 user_id_, object_ptr<gift> &&gift_);

  static const std::int32_t ID = 1691750743;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeGiftUpgrade final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;

  starTransactionTypeGiftUpgrade();

  starTransactionTypeGiftUpgrade(int53 user_id_, object_ptr<upgradedGift> &&gift_);

  static const std::int32_t ID = -632388839;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeUpgradedGiftPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;

  starTransactionTypeUpgradedGiftPurchase();

  starTransactionTypeUpgradedGiftPurchase(int53 user_id_, object_ptr<upgradedGift> &&gift_);

  static const std::int32_t ID = -59050247;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeUpgradedGiftSale final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;
  object_ptr<affiliateInfo> affiliate_;
  static jfieldID affiliate_fieldID;

  starTransactionTypeUpgradedGiftSale();

  starTransactionTypeUpgradedGiftSale(int53 user_id_, object_ptr<upgradedGift> &&gift_, object_ptr<affiliateInfo> &&affiliate_);

  static const std::int32_t ID = 2041948603;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelPaidReactionSend final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  starTransactionTypeChannelPaidReactionSend();

  starTransactionTypeChannelPaidReactionSend(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1071224896;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeChannelPaidReactionReceive final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  starTransactionTypeChannelPaidReactionReceive();

  starTransactionTypeChannelPaidReactionReceive(int53 user_id_, int53 message_id_);

  static const std::int32_t ID = 601291243;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeAffiliateProgramCommission final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 commission_per_mille_;
  static jfieldID commission_per_mille_fieldID;

  starTransactionTypeAffiliateProgramCommission();

  starTransactionTypeAffiliateProgramCommission(int53 chat_id_, int32 commission_per_mille_);

  static const std::int32_t ID = -1704757901;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypePaidMessageSend final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 message_count_;
  static jfieldID message_count_fieldID;

  starTransactionTypePaidMessageSend();

  starTransactionTypePaidMessageSend(int53 chat_id_, int32 message_count_);

  static const std::int32_t ID = -1709611931;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypePaidMessageReceive final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  int32 message_count_;
  static jfieldID message_count_fieldID;
  int32 commission_per_mille_;
  static jfieldID commission_per_mille_fieldID;
  object_ptr<starAmount> commission_star_amount_;
  static jfieldID commission_star_amount_fieldID;

  starTransactionTypePaidMessageReceive();

  starTransactionTypePaidMessageReceive(object_ptr<MessageSender> &&sender_id_, int32 message_count_, int32 commission_per_mille_, object_ptr<starAmount> &&commission_star_amount_);

  static const std::int32_t ID = -676839994;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypePremiumPurchase final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  starTransactionTypePremiumPurchase();

  starTransactionTypePremiumPurchase(int53 user_id_, int32 month_count_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = 998094851;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBusinessBotTransferSend final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  starTransactionTypeBusinessBotTransferSend();

  explicit starTransactionTypeBusinessBotTransferSend(int53 user_id_);

  static const std::int32_t ID = 1221227814;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeBusinessBotTransferReceive final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  starTransactionTypeBusinessBotTransferReceive();

  explicit starTransactionTypeBusinessBotTransferReceive(int53 user_id_);

  static const std::int32_t ID = 532496778;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starTransactionTypeUnsupported final : public StarTransactionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  starTransactionTypeUnsupported();

  static const std::int32_t ID = 1993329330;

  static object_ptr<StarTransactionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class starAmount;

class starTransaction;

class starTransactions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starAmount> star_amount_;
  static jfieldID star_amount_fieldID;
  array<object_ptr<starTransaction>> transactions_;
  static jfieldID transactions_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  starTransactions();

  starTransactions(object_ptr<starAmount> &&star_amount_, array<object_ptr<starTransaction>> &&transactions_, string const &next_offset_);

  static const std::int32_t ID = 1218437859;

  static object_ptr<starTransactions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph: public Object {
 public:
  static jclass Class;

  static object_ptr<StatisticalGraph> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class statisticalGraphData final : public StatisticalGraph {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string json_data_;
  static jfieldID json_data_fieldID;
  string zoom_token_;
  static jfieldID zoom_token_fieldID;

  statisticalGraphData();

  statisticalGraphData(string const &json_data_, string const &zoom_token_);

  static const std::int32_t ID = -1988940244;

  static object_ptr<StatisticalGraph> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class statisticalGraphAsync final : public StatisticalGraph {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  statisticalGraphAsync();

  explicit statisticalGraphAsync(string const &token_);

  static const std::int32_t ID = 435891103;

  static object_ptr<StatisticalGraph> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class statisticalGraphError final : public StatisticalGraph {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string error_message_;
  static jfieldID error_message_fieldID;

  statisticalGraphError();

  explicit statisticalGraphError(string const &error_message_);

  static const std::int32_t ID = -1006788526;

  static object_ptr<StatisticalGraph> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class statisticalValue final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double value_;
  static jfieldID value_fieldID;
  double previous_value_;
  static jfieldID previous_value_fieldID;
  double growth_rate_percentage_;
  static jfieldID growth_rate_percentage_fieldID;

  statisticalValue();

  statisticalValue(double value_, double previous_value_, double growth_rate_percentage_);

  static const std::int32_t ID = 1651337846;

  static object_ptr<statisticalValue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StickerFormat;

class StickerFullType;

class file;

class thumbnail;

class sticker final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int64 set_id_;
  static jfieldID set_id_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  object_ptr<StickerFormat> format_;
  static jfieldID format_fieldID;
  object_ptr<StickerFullType> full_type_;
  static jfieldID full_type_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<file> sticker_;
  static jfieldID sticker_fieldID;

  sticker();

  sticker(int64 id_, int64 set_id_, int32 width_, int32 height_, string const &emoji_, object_ptr<StickerFormat> &&format_, object_ptr<StickerFullType> &&full_type_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&sticker_);

  static const std::int32_t ID = -647013057;

  static object_ptr<sticker> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StickerFormat: public Object {
 public:
  static jclass Class;

  static object_ptr<StickerFormat> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerFormatWebp final : public StickerFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerFormatWebp();

  static const std::int32_t ID = -2123043040;

  static object_ptr<StickerFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerFormatTgs final : public StickerFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerFormatTgs();

  static const std::int32_t ID = 1614588662;

  static object_ptr<StickerFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerFormatWebm final : public StickerFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerFormatWebm();

  static const std::int32_t ID = -2070162097;

  static object_ptr<StickerFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class maskPosition;

class StickerFullType: public Object {
 public:
  static jclass Class;

  static object_ptr<StickerFullType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerFullTypeRegular final : public StickerFullType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<file> premium_animation_;
  static jfieldID premium_animation_fieldID;

  stickerFullTypeRegular();

  explicit stickerFullTypeRegular(object_ptr<file> &&premium_animation_);

  static const std::int32_t ID = -2006425865;

  static object_ptr<StickerFullType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerFullTypeMask final : public StickerFullType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<maskPosition> mask_position_;
  static jfieldID mask_position_fieldID;

  stickerFullTypeMask();

  explicit stickerFullTypeMask(object_ptr<maskPosition> &&mask_position_);

  static const std::int32_t ID = 652197687;

  static object_ptr<StickerFullType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerFullTypeCustomEmoji final : public StickerFullType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;
  bool needs_repainting_;
  static jfieldID needs_repainting_fieldID;

  stickerFullTypeCustomEmoji();

  stickerFullTypeCustomEmoji(int64 custom_emoji_id_, bool needs_repainting_);

  static const std::int32_t ID = -1015085653;

  static object_ptr<StickerFullType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class emojis;

class outline;

class sticker;

class thumbnail;

class stickerSet final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<outline> thumbnail_outline_;
  static jfieldID thumbnail_outline_fieldID;
  bool is_owned_;
  static jfieldID is_owned_fieldID;
  bool is_installed_;
  static jfieldID is_installed_fieldID;
  bool is_archived_;
  static jfieldID is_archived_fieldID;
  bool is_official_;
  static jfieldID is_official_fieldID;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  bool needs_repainting_;
  static jfieldID needs_repainting_fieldID;
  bool is_allowed_as_chat_emoji_status_;
  static jfieldID is_allowed_as_chat_emoji_status_fieldID;
  bool is_viewed_;
  static jfieldID is_viewed_fieldID;
  array<object_ptr<sticker>> stickers_;
  static jfieldID stickers_fieldID;
  array<object_ptr<emojis>> emojis_;
  static jfieldID emojis_fieldID;

  stickerSet();

  stickerSet(int64 id_, string const &title_, string const &name_, object_ptr<thumbnail> &&thumbnail_, object_ptr<outline> &&thumbnail_outline_, bool is_owned_, bool is_installed_, bool is_archived_, bool is_official_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, bool is_allowed_as_chat_emoji_status_, bool is_viewed_, array<object_ptr<sticker>> &&stickers_, array<object_ptr<emojis>> &&emojis_);

  static const std::int32_t ID = -1783150210;

  static object_ptr<stickerSet> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class outline;

class sticker;

class thumbnail;

class stickerSetInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<outline> thumbnail_outline_;
  static jfieldID thumbnail_outline_fieldID;
  bool is_owned_;
  static jfieldID is_owned_fieldID;
  bool is_installed_;
  static jfieldID is_installed_fieldID;
  bool is_archived_;
  static jfieldID is_archived_fieldID;
  bool is_official_;
  static jfieldID is_official_fieldID;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  bool needs_repainting_;
  static jfieldID needs_repainting_fieldID;
  bool is_allowed_as_chat_emoji_status_;
  static jfieldID is_allowed_as_chat_emoji_status_fieldID;
  bool is_viewed_;
  static jfieldID is_viewed_fieldID;
  int32 size_;
  static jfieldID size_fieldID;
  array<object_ptr<sticker>> covers_;
  static jfieldID covers_fieldID;

  stickerSetInfo();

  stickerSetInfo(int64 id_, string const &title_, string const &name_, object_ptr<thumbnail> &&thumbnail_, object_ptr<outline> &&thumbnail_outline_, bool is_owned_, bool is_installed_, bool is_archived_, bool is_official_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, bool is_allowed_as_chat_emoji_status_, bool is_viewed_, int32 size_, array<object_ptr<sticker>> &&covers_);

  static const std::int32_t ID = -1649074729;

  static object_ptr<stickerSetInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerSetInfo;

class stickerSets final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<stickerSetInfo>> sets_;
  static jfieldID sets_fieldID;

  stickerSets();

  stickerSets(int32 total_count_, array<object_ptr<stickerSetInfo>> &&sets_);

  static const std::int32_t ID = -1883828812;

  static object_ptr<stickerSets> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StickerType: public Object {
 public:
  static jclass Class;

  static object_ptr<StickerType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerTypeRegular final : public StickerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerTypeRegular();

  static const std::int32_t ID = 56345973;

  static object_ptr<StickerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerTypeMask final : public StickerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerTypeMask();

  static const std::int32_t ID = -1765394796;

  static object_ptr<StickerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerTypeCustomEmoji final : public StickerType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  stickerTypeCustomEmoji();

  static const std::int32_t ID = -120752249;

  static object_ptr<StickerType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class stickers final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<sticker>> stickers_;
  static jfieldID stickers_fieldID;

  stickers();

  explicit stickers(array<object_ptr<sticker>> &&stickers_);

  static const std::int32_t ID = 1974859260;

  static object_ptr<stickers> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storageStatisticsByChat;

class storageStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 size_;
  static jfieldID size_fieldID;
  int32 count_;
  static jfieldID count_fieldID;
  array<object_ptr<storageStatisticsByChat>> by_chat_;
  static jfieldID by_chat_fieldID;

  storageStatistics();

  storageStatistics(int53 size_, int32 count_, array<object_ptr<storageStatisticsByChat>> &&by_chat_);

  static const std::int32_t ID = 217237013;

  static object_ptr<storageStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storageStatisticsByFileType;

class storageStatisticsByChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 size_;
  static jfieldID size_fieldID;
  int32 count_;
  static jfieldID count_fieldID;
  array<object_ptr<storageStatisticsByFileType>> by_file_type_;
  static jfieldID by_file_type_fieldID;

  storageStatisticsByChat();

  storageStatisticsByChat(int53 chat_id_, int53 size_, int32 count_, array<object_ptr<storageStatisticsByFileType>> &&by_file_type_);

  static const std::int32_t ID = 635434531;

  static object_ptr<storageStatisticsByChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class FileType;

class storageStatisticsByFileType final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<FileType> file_type_;
  static jfieldID file_type_fieldID;
  int53 size_;
  static jfieldID size_fieldID;
  int32 count_;
  static jfieldID count_fieldID;

  storageStatisticsByFileType();

  storageStatisticsByFileType(object_ptr<FileType> &&file_type_, int53 size_, int32 count_);

  static const std::int32_t ID = 714012840;

  static object_ptr<storageStatisticsByFileType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storageStatisticsFast final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 files_size_;
  static jfieldID files_size_fieldID;
  int32 file_count_;
  static jfieldID file_count_fieldID;
  int53 database_size_;
  static jfieldID database_size_fieldID;
  int53 language_pack_database_size_;
  static jfieldID language_pack_database_size_fieldID;
  int53 log_size_;
  static jfieldID log_size_fieldID;

  storageStatisticsFast();

  storageStatisticsFast(int53 files_size_, int32 file_count_, int53 database_size_, int53 language_pack_database_size_, int53 log_size_);

  static const std::int32_t ID = -884922271;

  static object_ptr<storageStatisticsFast> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class giveawayParameters;

class StorePaymentPurpose: public Object {
 public:
  static jclass Class;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposePremiumSubscription final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_restore_;
  static jfieldID is_restore_fieldID;
  bool is_upgrade_;
  static jfieldID is_upgrade_fieldID;

  storePaymentPurposePremiumSubscription();

  storePaymentPurposePremiumSubscription(bool is_restore_, bool is_upgrade_);

  static const std::int32_t ID = 1263894804;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposePremiumGift final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  storePaymentPurposePremiumGift();

  storePaymentPurposePremiumGift(string const &currency_, int53 amount_, int53 user_id_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -39502443;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposePremiumGiftCodes final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  storePaymentPurposePremiumGiftCodes();

  storePaymentPurposePremiumGiftCodes(int53 boosted_chat_id_, string const &currency_, int53 amount_, array<int53> &&user_ids_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -1072286736;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposePremiumGiveaway final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;

  storePaymentPurposePremiumGiveaway();

  storePaymentPurposePremiumGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_);

  static const std::int32_t ID = 1302624938;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposeStarGiveaway final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  storePaymentPurposeStarGiveaway();

  storePaymentPurposeStarGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int53 star_count_);

  static const std::int32_t ID = 211212441;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposeStars final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  storePaymentPurposeStars();

  storePaymentPurposeStars(string const &currency_, int53 amount_, int53 star_count_);

  static const std::int32_t ID = -1803497708;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storePaymentPurposeGiftedStars final : public StorePaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  storePaymentPurposeGiftedStars();

  storePaymentPurposeGiftedStars(int53 user_id_, string const &currency_, int53 amount_, int53 star_count_);

  static const std::int32_t ID = 893691428;

  static object_ptr<StorePaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoreTransaction: public Object {
 public:
  static jclass Class;

  static object_ptr<StoreTransaction> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storeTransactionAppStore final : public StoreTransaction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes receipt_;
  static jfieldID receipt_fieldID;

  storeTransactionAppStore();

  explicit storeTransactionAppStore(bytes const &receipt_);

  static const std::int32_t ID = 1625562441;

  static object_ptr<StoreTransaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storeTransactionGooglePlay final : public StoreTransaction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string package_name_;
  static jfieldID package_name_fieldID;
  string store_product_id_;
  static jfieldID store_product_id_fieldID;
  string purchase_token_;
  static jfieldID purchase_token_fieldID;

  storeTransactionGooglePlay();

  storeTransactionGooglePlay(string const &package_name_, string const &store_product_id_, string const &purchase_token_);

  static const std::int32_t ID = 1094018617;

  static object_ptr<StoreTransaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class story;

class stories final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<story>> stories_;
  static jfieldID stories_fieldID;
  array<int32> pinned_story_ids_;
  static jfieldID pinned_story_ids_fieldID;

  stories();

  stories(int32 total_count_, array<object_ptr<story>> &&stories_, array<int32> &&pinned_story_ids_);

  static const std::int32_t ID = 670157595;

  static object_ptr<stories> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ReactionType;

class StoryContent;

class StoryPrivacySettings;

class formattedText;

class storyArea;

class storyInteractionInfo;

class storyRepostInfo;

class story final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  int53 poster_chat_id_;
  static jfieldID poster_chat_id_fieldID;
  object_ptr<MessageSender> poster_id_;
  static jfieldID poster_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  bool is_being_posted_;
  static jfieldID is_being_posted_fieldID;
  bool is_being_edited_;
  static jfieldID is_being_edited_fieldID;
  bool is_edited_;
  static jfieldID is_edited_fieldID;
  bool is_posted_to_chat_page_;
  static jfieldID is_posted_to_chat_page_fieldID;
  bool is_visible_only_for_self_;
  static jfieldID is_visible_only_for_self_fieldID;
  bool can_be_deleted_;
  static jfieldID can_be_deleted_fieldID;
  bool can_be_edited_;
  static jfieldID can_be_edited_fieldID;
  bool can_be_forwarded_;
  static jfieldID can_be_forwarded_fieldID;
  bool can_be_replied_;
  static jfieldID can_be_replied_fieldID;
  bool can_toggle_is_posted_to_chat_page_;
  static jfieldID can_toggle_is_posted_to_chat_page_fieldID;
  bool can_get_statistics_;
  static jfieldID can_get_statistics_fieldID;
  bool can_get_interactions_;
  static jfieldID can_get_interactions_fieldID;
  bool has_expired_viewers_;
  static jfieldID has_expired_viewers_fieldID;
  object_ptr<storyRepostInfo> repost_info_;
  static jfieldID repost_info_fieldID;
  object_ptr<storyInteractionInfo> interaction_info_;
  static jfieldID interaction_info_fieldID;
  object_ptr<ReactionType> chosen_reaction_type_;
  static jfieldID chosen_reaction_type_fieldID;
  object_ptr<StoryPrivacySettings> privacy_settings_;
  static jfieldID privacy_settings_fieldID;
  object_ptr<StoryContent> content_;
  static jfieldID content_fieldID;
  array<object_ptr<storyArea>> areas_;
  static jfieldID areas_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  story();

  story(int32 id_, int53 poster_chat_id_, object_ptr<MessageSender> &&poster_id_, int32 date_, bool is_being_posted_, bool is_being_edited_, bool is_edited_, bool is_posted_to_chat_page_, bool is_visible_only_for_self_, bool can_be_deleted_, bool can_be_edited_, bool can_be_forwarded_, bool can_be_replied_, bool can_toggle_is_posted_to_chat_page_, bool can_get_statistics_, bool can_get_interactions_, bool has_expired_viewers_, object_ptr<storyRepostInfo> &&repost_info_, object_ptr<storyInteractionInfo> &&interaction_info_, object_ptr<ReactionType> &&chosen_reaction_type_, object_ptr<StoryPrivacySettings> &&privacy_settings_, object_ptr<StoryContent> &&content_, array<object_ptr<storyArea>> &&areas_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = 1206531707;

  static object_ptr<story> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryAreaType;

class storyAreaPosition;

class storyArea final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<storyAreaPosition> position_;
  static jfieldID position_fieldID;
  object_ptr<StoryAreaType> type_;
  static jfieldID type_fieldID;

  storyArea();

  storyArea(object_ptr<storyAreaPosition> &&position_, object_ptr<StoryAreaType> &&type_);

  static const std::int32_t ID = -906033314;

  static object_ptr<storyArea> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaPosition final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double x_percentage_;
  static jfieldID x_percentage_fieldID;
  double y_percentage_;
  static jfieldID y_percentage_fieldID;
  double width_percentage_;
  static jfieldID width_percentage_fieldID;
  double height_percentage_;
  static jfieldID height_percentage_fieldID;
  double rotation_angle_;
  static jfieldID rotation_angle_fieldID;
  double corner_radius_percentage_;
  static jfieldID corner_radius_percentage_fieldID;

  storyAreaPosition();

  storyAreaPosition(double x_percentage_, double y_percentage_, double width_percentage_, double height_percentage_, double rotation_angle_, double corner_radius_percentage_);

  static const std::int32_t ID = -1533023124;

  static object_ptr<storyAreaPosition> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class location;

class locationAddress;

class venue;

class StoryAreaType: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeLocation final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  object_ptr<locationAddress> address_;
  static jfieldID address_fieldID;

  storyAreaTypeLocation();

  storyAreaTypeLocation(object_ptr<location> &&location_, object_ptr<locationAddress> &&address_);

  static const std::int32_t ID = -1464612189;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeVenue final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<venue> venue_;
  static jfieldID venue_fieldID;

  storyAreaTypeVenue();

  explicit storyAreaTypeVenue(object_ptr<venue> &&venue_);

  static const std::int32_t ID = 414076166;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeSuggestedReaction final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;
  bool is_flipped_;
  static jfieldID is_flipped_fieldID;

  storyAreaTypeSuggestedReaction();

  storyAreaTypeSuggestedReaction(object_ptr<ReactionType> &&reaction_type_, int32 total_count_, bool is_dark_, bool is_flipped_);

  static const std::int32_t ID = -111177092;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeMessage final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  storyAreaTypeMessage();

  storyAreaTypeMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1074825548;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeLink final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  storyAreaTypeLink();

  explicit storyAreaTypeLink(string const &url_);

  static const std::int32_t ID = -127770235;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeWeather final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double temperature_;
  static jfieldID temperature_fieldID;
  string emoji_;
  static jfieldID emoji_fieldID;
  int32 background_color_;
  static jfieldID background_color_fieldID;

  storyAreaTypeWeather();

  storyAreaTypeWeather(double temperature_, string const &emoji_, int32 background_color_);

  static const std::int32_t ID = -1504150082;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyAreaTypeUpgradedGift final : public StoryAreaType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string gift_name_;
  static jfieldID gift_name_fieldID;

  storyAreaTypeUpgradedGift();

  explicit storyAreaTypeUpgradedGift(string const &gift_name_);

  static const std::int32_t ID = 760281479;

  static object_ptr<StoryAreaType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class photo;

class storyVideo;

class StoryContent: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryContent> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyContentPhoto final : public StoryContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;

  storyContentPhoto();

  explicit storyContentPhoto(object_ptr<photo> &&photo_);

  static const std::int32_t ID = -731971504;

  static object_ptr<StoryContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyContentVideo final : public StoryContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<storyVideo> video_;
  static jfieldID video_fieldID;
  object_ptr<storyVideo> alternative_video_;
  static jfieldID alternative_video_fieldID;

  storyContentVideo();

  storyContentVideo(object_ptr<storyVideo> &&video_, object_ptr<storyVideo> &&alternative_video_);

  static const std::int32_t ID = -1291754842;

  static object_ptr<StoryContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyContentUnsupported final : public StoryContent {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  storyContentUnsupported();

  static const std::int32_t ID = -2033715858;

  static object_ptr<StoryContent> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyFullId final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 poster_chat_id_;
  static jfieldID poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  storyFullId();

  storyFullId(int53 poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = 765952419;

  static object_ptr<storyFullId> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  bool is_for_close_friends_;
  static jfieldID is_for_close_friends_fieldID;

  storyInfo();

  storyInfo(int32 story_id_, int32 date_, bool is_for_close_friends_);

  static const std::int32_t ID = -1986542766;

  static object_ptr<storyInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BlockList;

class MessageSender;

class StoryInteractionType;

class storyInteraction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> actor_id_;
  static jfieldID actor_id_fieldID;
  int32 interaction_date_;
  static jfieldID interaction_date_fieldID;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;
  object_ptr<StoryInteractionType> type_;
  static jfieldID type_fieldID;

  storyInteraction();

  storyInteraction(object_ptr<MessageSender> &&actor_id_, int32 interaction_date_, object_ptr<BlockList> &&block_list_, object_ptr<StoryInteractionType> &&type_);

  static const std::int32_t ID = -702229982;

  static object_ptr<storyInteraction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyInteractionInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 view_count_;
  static jfieldID view_count_fieldID;
  int32 forward_count_;
  static jfieldID forward_count_fieldID;
  int32 reaction_count_;
  static jfieldID reaction_count_fieldID;
  array<int53> recent_viewer_user_ids_;
  static jfieldID recent_viewer_user_ids_fieldID;

  storyInteractionInfo();

  storyInteractionInfo(int32 view_count_, int32 forward_count_, int32 reaction_count_, array<int53> &&recent_viewer_user_ids_);

  static const std::int32_t ID = -846542065;

  static object_ptr<storyInteractionInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class message;

class story;

class StoryInteractionType: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryInteractionType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyInteractionTypeView final : public StoryInteractionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> chosen_reaction_type_;
  static jfieldID chosen_reaction_type_fieldID;

  storyInteractionTypeView();

  explicit storyInteractionTypeView(object_ptr<ReactionType> &&chosen_reaction_type_);

  static const std::int32_t ID = 1407399888;

  static object_ptr<StoryInteractionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyInteractionTypeForward final : public StoryInteractionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  storyInteractionTypeForward();

  explicit storyInteractionTypeForward(object_ptr<message> &&message_);

  static const std::int32_t ID = 668089599;

  static object_ptr<StoryInteractionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyInteractionTypeRepost final : public StoryInteractionType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<story> story_;
  static jfieldID story_fieldID;

  storyInteractionTypeRepost();

  explicit storyInteractionTypeRepost(object_ptr<story> &&story_);

  static const std::int32_t ID = -1021150780;

  static object_ptr<StoryInteractionType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyInteraction;

class storyInteractions final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  int32 total_forward_count_;
  static jfieldID total_forward_count_fieldID;
  int32 total_reaction_count_;
  static jfieldID total_reaction_count_fieldID;
  array<object_ptr<storyInteraction>> interactions_;
  static jfieldID interactions_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  storyInteractions();

  storyInteractions(int32 total_count_, int32 total_forward_count_, int32 total_reaction_count_, array<object_ptr<storyInteraction>> &&interactions_, string const &next_offset_);

  static const std::int32_t ID = 1537062962;

  static object_ptr<storyInteractions> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryList: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryList> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyListMain final : public StoryList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  storyListMain();

  static const std::int32_t ID = -672222209;

  static object_ptr<StoryList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyListArchive final : public StoryList {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  storyListArchive();

  static const std::int32_t ID = -41900223;

  static object_ptr<StoryList> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryOrigin: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryOrigin> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyOriginPublicStory final : public StoryOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  storyOriginPublicStory();

  storyOriginPublicStory(int53 chat_id_, int32 story_id_);

  static const std::int32_t ID = 741842878;

  static object_ptr<StoryOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyOriginHiddenUser final : public StoryOrigin {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string poster_name_;
  static jfieldID poster_name_fieldID;

  storyOriginHiddenUser();

  explicit storyOriginHiddenUser(string const &poster_name_);

  static const std::int32_t ID = -96348585;

  static object_ptr<StoryOrigin> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryPrivacySettings: public Object {
 public:
  static jclass Class;

  static object_ptr<StoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyPrivacySettingsEveryone final : public StoryPrivacySettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> except_user_ids_;
  static jfieldID except_user_ids_fieldID;

  storyPrivacySettingsEveryone();

  explicit storyPrivacySettingsEveryone(array<int53> &&except_user_ids_);

  static const std::int32_t ID = 890847843;

  static object_ptr<StoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyPrivacySettingsContacts final : public StoryPrivacySettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> except_user_ids_;
  static jfieldID except_user_ids_fieldID;

  storyPrivacySettingsContacts();

  explicit storyPrivacySettingsContacts(array<int53> &&except_user_ids_);

  static const std::int32_t ID = 50285309;

  static object_ptr<StoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyPrivacySettingsCloseFriends final : public StoryPrivacySettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  storyPrivacySettingsCloseFriends();

  static const std::int32_t ID = 2097122144;

  static object_ptr<StoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class storyPrivacySettingsSelectedUsers final : public StoryPrivacySettings {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  storyPrivacySettingsSelectedUsers();

  explicit storyPrivacySettingsSelectedUsers(array<int53> &&user_ids_);

  static const std::int32_t ID = -1885772602;

  static object_ptr<StoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StoryOrigin;

class storyRepostInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StoryOrigin> origin_;
  static jfieldID origin_fieldID;
  bool is_content_modified_;
  static jfieldID is_content_modified_fieldID;

  storyRepostInfo();

  storyRepostInfo(object_ptr<StoryOrigin> &&origin_, bool is_content_modified_);

  static const std::int32_t ID = -8412096;

  static object_ptr<storyRepostInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class storyStatistics final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StatisticalGraph> story_interaction_graph_;
  static jfieldID story_interaction_graph_fieldID;
  object_ptr<StatisticalGraph> story_reaction_graph_;
  static jfieldID story_reaction_graph_fieldID;

  storyStatistics();

  storyStatistics(object_ptr<StatisticalGraph> &&story_interaction_graph_, object_ptr<StatisticalGraph> &&story_reaction_graph_);

  static const std::int32_t ID = 1178897259;

  static object_ptr<storyStatistics> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class thumbnail;

class storyVideo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  bool has_stickers_;
  static jfieldID has_stickers_fieldID;
  bool is_animation_;
  static jfieldID is_animation_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  int32 preload_prefix_size_;
  static jfieldID preload_prefix_size_fieldID;
  double cover_frame_timestamp_;
  static jfieldID cover_frame_timestamp_fieldID;
  object_ptr<file> video_;
  static jfieldID video_fieldID;

  storyVideo();

  storyVideo(double duration_, int32 width_, int32 height_, bool has_stickers_, bool is_animation_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, int32 preload_prefix_size_, double cover_frame_timestamp_, object_ptr<file> &&video_);

  static const std::int32_t ID = 1445661253;

  static object_ptr<storyVideo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class SuggestedAction: public Object {
 public:
  static jclass Class;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionEnableArchiveAndMuteNewChats final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionEnableArchiveAndMuteNewChats();

  static const std::int32_t ID = 2017586255;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionCheckPassword final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionCheckPassword();

  static const std::int32_t ID = 1910534839;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionCheckPhoneNumber final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionCheckPhoneNumber();

  static const std::int32_t ID = 648771563;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionViewChecksHint final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionViewChecksHint();

  static const std::int32_t ID = 891303239;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionConvertToBroadcastGroup final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  suggestedActionConvertToBroadcastGroup();

  explicit suggestedActionConvertToBroadcastGroup(int53 supergroup_id_);

  static const std::int32_t ID = -965071304;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionSetPassword final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 authorization_delay_;
  static jfieldID authorization_delay_fieldID;

  suggestedActionSetPassword();

  explicit suggestedActionSetPassword(int32 authorization_delay_);

  static const std::int32_t ID = 1863613848;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionUpgradePremium final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionUpgradePremium();

  static const std::int32_t ID = 1890220539;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionRestorePremium final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionRestorePremium();

  static const std::int32_t ID = -385229468;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionSubscribeToAnnualPremium final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionSubscribeToAnnualPremium();

  static const std::int32_t ID = 373913787;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionGiftPremiumForChristmas final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionGiftPremiumForChristmas();

  static const std::int32_t ID = -1816924561;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionSetBirthdate final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionSetBirthdate();

  static const std::int32_t ID = -356672766;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionSetProfilePhoto final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionSetProfilePhoto();

  static const std::int32_t ID = -1612563093;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionExtendPremium final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string manage_premium_subscription_url_;
  static jfieldID manage_premium_subscription_url_fieldID;

  suggestedActionExtendPremium();

  explicit suggestedActionExtendPremium(string const &manage_premium_subscription_url_);

  static const std::int32_t ID = -566207286;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionExtendStarSubscriptions final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  suggestedActionExtendStarSubscriptions();

  static const std::int32_t ID = -47000234;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class suggestedActionCustom final : public SuggestedAction {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<formattedText> title_;
  static jfieldID title_fieldID;
  object_ptr<formattedText> description_;
  static jfieldID description_fieldID;
  string url_;
  static jfieldID url_fieldID;

  suggestedActionCustom();

  suggestedActionCustom(string const &name_, object_ptr<formattedText> &&title_, object_ptr<formattedText> &&description_, string const &url_);

  static const std::int32_t ID = 2092876611;

  static object_ptr<SuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ChatMemberStatus;

class usernames;

class verificationStatus;

class supergroup final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  object_ptr<usernames> usernames_;
  static jfieldID usernames_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<ChatMemberStatus> status_;
  static jfieldID status_fieldID;
  int32 member_count_;
  static jfieldID member_count_fieldID;
  int32 boost_level_;
  static jfieldID boost_level_fieldID;
  bool has_automatic_translation_;
  static jfieldID has_automatic_translation_fieldID;
  bool has_linked_chat_;
  static jfieldID has_linked_chat_fieldID;
  bool has_location_;
  static jfieldID has_location_fieldID;
  bool sign_messages_;
  static jfieldID sign_messages_fieldID;
  bool show_message_sender_;
  static jfieldID show_message_sender_fieldID;
  bool join_to_send_messages_;
  static jfieldID join_to_send_messages_fieldID;
  bool join_by_request_;
  static jfieldID join_by_request_fieldID;
  bool is_slow_mode_enabled_;
  static jfieldID is_slow_mode_enabled_fieldID;
  bool is_channel_;
  static jfieldID is_channel_fieldID;
  bool is_broadcast_group_;
  static jfieldID is_broadcast_group_fieldID;
  bool is_forum_;
  static jfieldID is_forum_fieldID;
  object_ptr<verificationStatus> verification_status_;
  static jfieldID verification_status_fieldID;
  bool has_sensitive_content_;
  static jfieldID has_sensitive_content_fieldID;
  string restriction_reason_;
  static jfieldID restriction_reason_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;
  bool has_active_stories_;
  static jfieldID has_active_stories_fieldID;
  bool has_unread_active_stories_;
  static jfieldID has_unread_active_stories_fieldID;

  supergroup();

  supergroup(int53 id_, object_ptr<usernames> &&usernames_, int32 date_, object_ptr<ChatMemberStatus> &&status_, int32 member_count_, int32 boost_level_, bool has_automatic_translation_, bool has_linked_chat_, bool has_location_, bool sign_messages_, bool show_message_sender_, bool join_to_send_messages_, bool join_by_request_, bool is_slow_mode_enabled_, bool is_channel_, bool is_broadcast_group_, bool is_forum_, object_ptr<verificationStatus> &&verification_status_, bool has_sensitive_content_, string const &restriction_reason_, int53 paid_message_star_count_, bool has_active_stories_, bool has_unread_active_stories_);

  static const std::int32_t ID = -1355919719;

  static object_ptr<supergroup> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class botCommands;

class botVerification;

class chatInviteLink;

class chatLocation;

class chatPhoto;

class supergroupFullInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  string description_;
  static jfieldID description_fieldID;
  int32 member_count_;
  static jfieldID member_count_fieldID;
  int32 administrator_count_;
  static jfieldID administrator_count_fieldID;
  int32 restricted_count_;
  static jfieldID restricted_count_fieldID;
  int32 banned_count_;
  static jfieldID banned_count_fieldID;
  int53 linked_chat_id_;
  static jfieldID linked_chat_id_fieldID;
  int32 slow_mode_delay_;
  static jfieldID slow_mode_delay_fieldID;
  double slow_mode_delay_expires_in_;
  static jfieldID slow_mode_delay_expires_in_fieldID;
  bool can_enable_paid_messages_;
  static jfieldID can_enable_paid_messages_fieldID;
  bool can_enable_paid_reaction_;
  static jfieldID can_enable_paid_reaction_fieldID;
  bool can_get_members_;
  static jfieldID can_get_members_fieldID;
  bool has_hidden_members_;
  static jfieldID has_hidden_members_fieldID;
  bool can_hide_members_;
  static jfieldID can_hide_members_fieldID;
  bool can_set_sticker_set_;
  static jfieldID can_set_sticker_set_fieldID;
  bool can_set_location_;
  static jfieldID can_set_location_fieldID;
  bool can_get_statistics_;
  static jfieldID can_get_statistics_fieldID;
  bool can_get_revenue_statistics_;
  static jfieldID can_get_revenue_statistics_fieldID;
  bool can_get_star_revenue_statistics_;
  static jfieldID can_get_star_revenue_statistics_fieldID;
  bool can_send_gift_;
  static jfieldID can_send_gift_fieldID;
  bool can_toggle_aggressive_anti_spam_;
  static jfieldID can_toggle_aggressive_anti_spam_fieldID;
  bool is_all_history_available_;
  static jfieldID is_all_history_available_fieldID;
  bool can_have_sponsored_messages_;
  static jfieldID can_have_sponsored_messages_fieldID;
  bool has_aggressive_anti_spam_enabled_;
  static jfieldID has_aggressive_anti_spam_enabled_fieldID;
  bool has_paid_media_allowed_;
  static jfieldID has_paid_media_allowed_fieldID;
  bool has_pinned_stories_;
  static jfieldID has_pinned_stories_fieldID;
  int32 gift_count_;
  static jfieldID gift_count_fieldID;
  int32 my_boost_count_;
  static jfieldID my_boost_count_fieldID;
  int32 unrestrict_boost_count_;
  static jfieldID unrestrict_boost_count_fieldID;
  int64 sticker_set_id_;
  static jfieldID sticker_set_id_fieldID;
  int64 custom_emoji_sticker_set_id_;
  static jfieldID custom_emoji_sticker_set_id_fieldID;
  object_ptr<chatLocation> location_;
  static jfieldID location_fieldID;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;
  array<object_ptr<botCommands>> bot_commands_;
  static jfieldID bot_commands_fieldID;
  object_ptr<botVerification> bot_verification_;
  static jfieldID bot_verification_fieldID;
  int53 upgraded_from_basic_group_id_;
  static jfieldID upgraded_from_basic_group_id_fieldID;
  int53 upgraded_from_max_message_id_;
  static jfieldID upgraded_from_max_message_id_fieldID;

  supergroupFullInfo();

  supergroupFullInfo(object_ptr<chatPhoto> &&photo_, string const &description_, int32 member_count_, int32 administrator_count_, int32 restricted_count_, int32 banned_count_, int53 linked_chat_id_, int32 slow_mode_delay_, double slow_mode_delay_expires_in_, bool can_enable_paid_messages_, bool can_enable_paid_reaction_, bool can_get_members_, bool has_hidden_members_, bool can_hide_members_, bool can_set_sticker_set_, bool can_set_location_, bool can_get_statistics_, bool can_get_revenue_statistics_, bool can_get_star_revenue_statistics_, bool can_send_gift_, bool can_toggle_aggressive_anti_spam_, bool is_all_history_available_, bool can_have_sponsored_messages_, bool has_aggressive_anti_spam_enabled_, bool has_paid_media_allowed_, bool has_pinned_stories_, int32 gift_count_, int32 my_boost_count_, int32 unrestrict_boost_count_, int64 sticker_set_id_, int64 custom_emoji_sticker_set_id_, object_ptr<chatLocation> &&location_, object_ptr<chatInviteLink> &&invite_link_, array<object_ptr<botCommands>> &&bot_commands_, object_ptr<botVerification> &&bot_verification_, int53 upgraded_from_basic_group_id_, int53 upgraded_from_max_message_id_);

  static const std::int32_t ID = 2114051007;

  static object_ptr<supergroupFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SupergroupMembersFilter: public Object {
 public:
  static jclass Class;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterRecent final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  supergroupMembersFilterRecent();

  static const std::int32_t ID = 1178199509;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterContacts final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  supergroupMembersFilterContacts();

  explicit supergroupMembersFilterContacts(string const &query_);

  static const std::int32_t ID = -1282910856;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterAdministrators final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  supergroupMembersFilterAdministrators();

  static const std::int32_t ID = -2097380265;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterSearch final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  supergroupMembersFilterSearch();

  explicit supergroupMembersFilterSearch(string const &query_);

  static const std::int32_t ID = -1696358469;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterRestricted final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  supergroupMembersFilterRestricted();

  explicit supergroupMembersFilterRestricted(string const &query_);

  static const std::int32_t ID = -1107800034;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterBanned final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  supergroupMembersFilterBanned();

  explicit supergroupMembersFilterBanned(string const &query_);

  static const std::int32_t ID = -1210621683;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterMention final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  supergroupMembersFilterMention();

  supergroupMembersFilterMention(string const &query_, int53 message_thread_id_);

  static const std::int32_t ID = 947915036;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class supergroupMembersFilterBots final : public SupergroupMembersFilter {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  supergroupMembersFilterBots();

  static const std::int32_t ID = 492138918;

  static object_ptr<SupergroupMembersFilter> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TMeUrlType;

class tMeUrl final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<TMeUrlType> type_;
  static jfieldID type_fieldID;

  tMeUrl();

  tMeUrl(string const &url_, object_ptr<TMeUrlType> &&type_);

  static const std::int32_t ID = -1140786622;

  static object_ptr<tMeUrl> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkInfo;

class TMeUrlType: public Object {
 public:
  static jclass Class;

  static object_ptr<TMeUrlType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrlTypeUser final : public TMeUrlType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  tMeUrlTypeUser();

  explicit tMeUrlTypeUser(int53 user_id_);

  static const std::int32_t ID = 125336602;

  static object_ptr<TMeUrlType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrlTypeSupergroup final : public TMeUrlType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  tMeUrlTypeSupergroup();

  explicit tMeUrlTypeSupergroup(int53 supergroup_id_);

  static const std::int32_t ID = -1353369944;

  static object_ptr<TMeUrlType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrlTypeChatInvite final : public TMeUrlType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatInviteLinkInfo> info_;
  static jfieldID info_fieldID;

  tMeUrlTypeChatInvite();

  explicit tMeUrlTypeChatInvite(object_ptr<chatInviteLinkInfo> &&info_);

  static const std::int32_t ID = 313907785;

  static object_ptr<TMeUrlType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrlTypeStickerSet final : public TMeUrlType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 sticker_set_id_;
  static jfieldID sticker_set_id_fieldID;

  tMeUrlTypeStickerSet();

  explicit tMeUrlTypeStickerSet(int64 sticker_set_id_);

  static const std::int32_t ID = 1602473196;

  static object_ptr<TMeUrlType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrl;

class tMeUrls final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<tMeUrl>> urls_;
  static jfieldID urls_fieldID;

  tMeUrls();

  explicit tMeUrls(array<object_ptr<tMeUrl>> &&urls_);

  static const std::int32_t ID = -1130595098;

  static object_ptr<tMeUrls> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class targetChatTypes;

class TargetChat: public Object {
 public:
  static jclass Class;

  static object_ptr<TargetChat> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class targetChatCurrent final : public TargetChat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  targetChatCurrent();

  static const std::int32_t ID = -416689904;

  static object_ptr<TargetChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class targetChatChosen final : public TargetChat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<targetChatTypes> types_;
  static jfieldID types_fieldID;

  targetChatChosen();

  explicit targetChatChosen(object_ptr<targetChatTypes> &&types_);

  static const std::int32_t ID = -1392978522;

  static object_ptr<TargetChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class targetChatInternalLink final : public TargetChat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InternalLinkType> link_;
  static jfieldID link_fieldID;

  targetChatInternalLink();

  explicit targetChatInternalLink(object_ptr<InternalLinkType> &&link_);

  static const std::int32_t ID = -579301408;

  static object_ptr<TargetChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class targetChatTypes final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool allow_user_chats_;
  static jfieldID allow_user_chats_fieldID;
  bool allow_bot_chats_;
  static jfieldID allow_bot_chats_fieldID;
  bool allow_group_chats_;
  static jfieldID allow_group_chats_fieldID;
  bool allow_channel_chats_;
  static jfieldID allow_channel_chats_fieldID;

  targetChatTypes();

  targetChatTypes(bool allow_user_chats_, bool allow_bot_chats_, bool allow_group_chats_, bool allow_channel_chats_);

  static const std::int32_t ID = 1513098833;

  static object_ptr<targetChatTypes> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class giveawayParameters;

class TelegramPaymentPurpose: public Object {
 public:
  static jclass Class;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposePremiumGift final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  telegramPaymentPurposePremiumGift();

  telegramPaymentPurposePremiumGift(string const &currency_, int53 amount_, int53 user_id_, int32 month_count_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -1600286150;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposePremiumGiftCodes final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  telegramPaymentPurposePremiumGiftCodes();

  telegramPaymentPurposePremiumGiftCodes(int53 boosted_chat_id_, string const &currency_, int53 amount_, array<int53> &&user_ids_, int32 month_count_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -1863495348;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposePremiumGiveaway final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;

  telegramPaymentPurposePremiumGiveaway();

  telegramPaymentPurposePremiumGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int32 month_count_);

  static const std::int32_t ID = -760757441;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposeStars final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  telegramPaymentPurposeStars();

  telegramPaymentPurposeStars(string const &currency_, int53 amount_, int53 star_count_);

  static const std::int32_t ID = -495718830;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposeGiftedStars final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  telegramPaymentPurposeGiftedStars();

  telegramPaymentPurposeGiftedStars(int53 user_id_, string const &currency_, int53 amount_, int53 star_count_);

  static const std::int32_t ID = -1850308042;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposeStarGiveaway final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  telegramPaymentPurposeStarGiveaway();

  telegramPaymentPurposeStarGiveaway(object_ptr<giveawayParameters> &&parameters_, string const &currency_, int53 amount_, int32 winner_count_, int53 star_count_);

  static const std::int32_t ID = 1014604689;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class telegramPaymentPurposeJoinChat final : public TelegramPaymentPurpose {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  telegramPaymentPurposeJoinChat();

  explicit telegramPaymentPurposeJoinChat(string const &invite_link_);

  static const std::int32_t ID = -1914869880;

  static object_ptr<TelegramPaymentPurpose> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class temporaryPasswordState final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_password_;
  static jfieldID has_password_fieldID;
  int32 valid_for_;
  static jfieldID valid_for_fieldID;

  temporaryPasswordState();

  temporaryPasswordState(bool has_password_, int32 valid_for_);

  static const std::int32_t ID = 939837410;

  static object_ptr<temporaryPasswordState> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class termsOfService final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int32 min_user_age_;
  static jfieldID min_user_age_fieldID;
  bool show_popup_;
  static jfieldID show_popup_fieldID;

  termsOfService();

  termsOfService(object_ptr<formattedText> &&text_, int32 min_user_age_, bool show_popup_);

  static const std::int32_t ID = 739422597;

  static object_ptr<termsOfService> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testBytes final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes value_;
  static jfieldID value_fieldID;

  testBytes();

  explicit testBytes(bytes const &value_);

  static const std::int32_t ID = -1541225250;

  static object_ptr<testBytes> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testInt final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 value_;
  static jfieldID value_fieldID;

  testInt();

  explicit testInt(int32 value_);

  static const std::int32_t ID = -574804983;

  static object_ptr<testInt> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testString final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string value_;
  static jfieldID value_fieldID;

  testString();

  explicit testString(string const &value_);

  static const std::int32_t ID = -27891572;

  static object_ptr<testString> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testVectorInt final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> value_;
  static jfieldID value_fieldID;

  testVectorInt();

  explicit testVectorInt(array<int32> &&value_);

  static const std::int32_t ID = 593682027;

  static object_ptr<testVectorInt> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testInt;

class testVectorIntObject final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<testInt>> value_;
  static jfieldID value_fieldID;

  testVectorIntObject();

  explicit testVectorIntObject(array<object_ptr<testInt>> &&value_);

  static const std::int32_t ID = 125891546;

  static object_ptr<testVectorIntObject> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testVectorString final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> value_;
  static jfieldID value_fieldID;

  testVectorString();

  explicit testVectorString(array<string> &&value_);

  static const std::int32_t ID = 79339995;

  static object_ptr<testVectorString> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class testString;

class testVectorStringObject final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<testString>> value_;
  static jfieldID value_fieldID;

  testVectorStringObject();

  explicit testVectorStringObject(array<object_ptr<testString>> &&value_);

  static const std::int32_t ID = 80780537;

  static object_ptr<testVectorStringObject> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class text final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  text();

  explicit text(string const &text_);

  static const std::int32_t ID = 578181272;

  static object_ptr<text> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntity;

class textEntities final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<textEntity>> entities_;
  static jfieldID entities_fieldID;

  textEntities();

  explicit textEntities(array<object_ptr<textEntity>> &&entities_);

  static const std::int32_t ID = -933199172;

  static object_ptr<textEntities> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TextEntityType;

class textEntity final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 length_;
  static jfieldID length_fieldID;
  object_ptr<TextEntityType> type_;
  static jfieldID type_fieldID;

  textEntity();

  textEntity(int32 offset_, int32 length_, object_ptr<TextEntityType> &&type_);

  static const std::int32_t ID = -1951688280;

  static object_ptr<textEntity> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TextEntityType: public Object {
 public:
  static jclass Class;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeMention final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeMention();

  static const std::int32_t ID = 934535013;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeHashtag final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeHashtag();

  static const std::int32_t ID = -1023958307;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeCashtag final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeCashtag();

  static const std::int32_t ID = 1222915915;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeBotCommand final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeBotCommand();

  static const std::int32_t ID = -1150997581;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeUrl final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeUrl();

  static const std::int32_t ID = -1312762756;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeEmailAddress final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeEmailAddress();

  static const std::int32_t ID = 1425545249;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypePhoneNumber final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypePhoneNumber();

  static const std::int32_t ID = -1160140246;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeBankCardNumber final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeBankCardNumber();

  static const std::int32_t ID = 105986320;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeBold final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeBold();

  static const std::int32_t ID = -1128210000;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeItalic final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeItalic();

  static const std::int32_t ID = -118253987;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeUnderline final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeUnderline();

  static const std::int32_t ID = 792317842;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeStrikethrough final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeStrikethrough();

  static const std::int32_t ID = 961529082;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeSpoiler final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeSpoiler();

  static const std::int32_t ID = 544019899;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeCode final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeCode();

  static const std::int32_t ID = -974534326;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypePre final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypePre();

  static const std::int32_t ID = 1648958606;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypePreCode final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_;
  static jfieldID language_fieldID;

  textEntityTypePreCode();

  explicit textEntityTypePreCode(string const &language_);

  static const std::int32_t ID = -945325397;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeBlockQuote final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeBlockQuote();

  static const std::int32_t ID = -1003999032;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeExpandableBlockQuote final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textEntityTypeExpandableBlockQuote();

  static const std::int32_t ID = 36572261;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeTextUrl final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  textEntityTypeTextUrl();

  explicit textEntityTypeTextUrl(string const &url_);

  static const std::int32_t ID = 445719651;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeMentionName final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  textEntityTypeMentionName();

  explicit textEntityTypeMentionName(int53 user_id_);

  static const std::int32_t ID = -1570974289;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeCustomEmoji final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  textEntityTypeCustomEmoji();

  explicit textEntityTypeCustomEmoji(int64 custom_emoji_id_);

  static const std::int32_t ID = 1724820677;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textEntityTypeMediaTimestamp final : public TextEntityType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 media_timestamp_;
  static jfieldID media_timestamp_fieldID;

  textEntityTypeMediaTimestamp();

  explicit textEntityTypeMediaTimestamp(int32 media_timestamp_);

  static const std::int32_t ID = -1841898992;

  static object_ptr<TextEntityType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TextParseMode: public Object {
 public:
  static jclass Class;

  static object_ptr<TextParseMode> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class textParseModeMarkdown final : public TextParseMode {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 version_;
  static jfieldID version_fieldID;

  textParseModeMarkdown();

  explicit textParseModeMarkdown(int32 version_);

  static const std::int32_t ID = 360073407;

  static object_ptr<TextParseMode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class textParseModeHTML final : public TextParseMode {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  textParseModeHTML();

  static const std::int32_t ID = 1660208627;

  static object_ptr<TextParseMode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class textQuote final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int32 position_;
  static jfieldID position_fieldID;
  bool is_manual_;
  static jfieldID is_manual_fieldID;

  textQuote();

  textQuote(object_ptr<formattedText> &&text_, int32 position_, bool is_manual_);

  static const std::int32_t ID = -2039105358;

  static object_ptr<textQuote> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class themeParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 background_color_;
  static jfieldID background_color_fieldID;
  int32 secondary_background_color_;
  static jfieldID secondary_background_color_fieldID;
  int32 header_background_color_;
  static jfieldID header_background_color_fieldID;
  int32 bottom_bar_background_color_;
  static jfieldID bottom_bar_background_color_fieldID;
  int32 section_background_color_;
  static jfieldID section_background_color_fieldID;
  int32 section_separator_color_;
  static jfieldID section_separator_color_fieldID;
  int32 text_color_;
  static jfieldID text_color_fieldID;
  int32 accent_text_color_;
  static jfieldID accent_text_color_fieldID;
  int32 section_header_text_color_;
  static jfieldID section_header_text_color_fieldID;
  int32 subtitle_text_color_;
  static jfieldID subtitle_text_color_fieldID;
  int32 destructive_text_color_;
  static jfieldID destructive_text_color_fieldID;
  int32 hint_color_;
  static jfieldID hint_color_fieldID;
  int32 link_color_;
  static jfieldID link_color_fieldID;
  int32 button_color_;
  static jfieldID button_color_fieldID;
  int32 button_text_color_;
  static jfieldID button_text_color_fieldID;

  themeParameters();

  themeParameters(int32 background_color_, int32 secondary_background_color_, int32 header_background_color_, int32 bottom_bar_background_color_, int32 section_background_color_, int32 section_separator_color_, int32 text_color_, int32 accent_text_color_, int32 section_header_text_color_, int32 subtitle_text_color_, int32 destructive_text_color_, int32 hint_color_, int32 link_color_, int32 button_color_, int32 button_text_color_);

  static const std::int32_t ID = -276589137;

  static object_ptr<themeParameters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundFill;

class background;

class themeSettings final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 accent_color_;
  static jfieldID accent_color_fieldID;
  object_ptr<background> background_;
  static jfieldID background_fieldID;
  object_ptr<BackgroundFill> outgoing_message_fill_;
  static jfieldID outgoing_message_fill_fieldID;
  bool animate_outgoing_message_fill_;
  static jfieldID animate_outgoing_message_fill_fieldID;
  int32 outgoing_message_accent_color_;
  static jfieldID outgoing_message_accent_color_fieldID;

  themeSettings();

  themeSettings(int32 accent_color_, object_ptr<background> &&background_, object_ptr<BackgroundFill> &&outgoing_message_fill_, bool animate_outgoing_message_fill_, int32 outgoing_message_accent_color_);

  static const std::int32_t ID = -62120942;

  static object_ptr<themeSettings> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ThumbnailFormat;

class file;

class thumbnail final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ThumbnailFormat> format_;
  static jfieldID format_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  object_ptr<file> file_;
  static jfieldID file_fieldID;

  thumbnail();

  thumbnail(object_ptr<ThumbnailFormat> &&format_, int32 width_, int32 height_, object_ptr<file> &&file_);

  static const std::int32_t ID = 1243275371;

  static object_ptr<thumbnail> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class ThumbnailFormat: public Object {
 public:
  static jclass Class;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatJpeg final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatJpeg();

  static const std::int32_t ID = -653503352;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatGif final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatGif();

  static const std::int32_t ID = 1252205962;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatMpeg4 final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatMpeg4();

  static const std::int32_t ID = 278616062;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatPng final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatPng();

  static const std::int32_t ID = 1577490421;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatTgs final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatTgs();

  static const std::int32_t ID = 1315522642;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatWebm final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatWebm();

  static const std::int32_t ID = -660084953;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class thumbnailFormatWebp final : public ThumbnailFormat {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  thumbnailFormatWebp();

  static const std::int32_t ID = -53588974;

  static object_ptr<ThumbnailFormat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class timeZone final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string id_;
  static jfieldID id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  int32 utc_time_offset_;
  static jfieldID utc_time_offset_fieldID;

  timeZone();

  timeZone(string const &id_, string const &name_, int32 utc_time_offset_);

  static const std::int32_t ID = -1189481763;

  static object_ptr<timeZone> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class timeZone;

class timeZones final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<timeZone>> time_zones_;
  static jfieldID time_zones_fieldID;

  timeZones();

  explicit timeZones(array<object_ptr<timeZone>> &&time_zones_);

  static const std::int32_t ID = -334655570;

  static object_ptr<timeZones> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class TopChatCategory: public Object {
 public:
  static jclass Class;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryUsers final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryUsers();

  static const std::int32_t ID = 1026706816;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryBots final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryBots();

  static const std::int32_t ID = -1577129195;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryGroups final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryGroups();

  static const std::int32_t ID = 1530056846;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryChannels final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryChannels();

  static const std::int32_t ID = -500825885;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryInlineBots final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryInlineBots();

  static const std::int32_t ID = 377023356;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryWebAppBots final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryWebAppBots();

  static const std::int32_t ID = 100062973;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryCalls final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryCalls();

  static const std::int32_t ID = 356208861;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class topChatCategoryForwardChats final : public TopChatCategory {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  topChatCategoryForwardChats();

  static const std::int32_t ID = 1695922133;

  static object_ptr<TopChatCategory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class stickerSetInfo;

class trendingStickerSets final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<stickerSetInfo>> sets_;
  static jfieldID sets_fieldID;
  bool is_premium_;
  static jfieldID is_premium_fieldID;

  trendingStickerSets();

  trendingStickerSets(int32 total_count_, array<object_ptr<stickerSetInfo>> &&sets_, bool is_premium_);

  static const std::int32_t ID = 41028940;

  static object_ptr<trendingStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class unconfirmedSession final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int32 log_in_date_;
  static jfieldID log_in_date_fieldID;
  string device_model_;
  static jfieldID device_model_fieldID;
  string location_;
  static jfieldID location_fieldID;

  unconfirmedSession();

  unconfirmedSession(int64 id_, int32 log_in_date_, string const &device_model_, string const &location_);

  static const std::int32_t ID = -2062726663;

  static object_ptr<unconfirmedSession> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ReactionType;

class unreadReaction final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> type_;
  static jfieldID type_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  bool is_big_;
  static jfieldID is_big_fieldID;

  unreadReaction();

  unreadReaction(object_ptr<ReactionType> &&type_, object_ptr<MessageSender> &&sender_id_, bool is_big_);

  static const std::int32_t ID = -1940178046;

  static object_ptr<unreadReaction> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class AuthorizationState;

class AutosaveSettingsScope;

class BlockList;

class CallbackQueryPayload;

class CanPostStoryResult;

class ChatAction;

class ChatActionBar;

class ChatAvailableReactions;

class ChatList;

class ChatType;

class ConnectionState;

class MessageContent;

class MessageSender;

class NotificationGroupType;

class NotificationSettingsScope;

class OptionValue;

class PaidReactionType;

class ReactionType;

class ReplyMarkup;

class StickerType;

class StoryList;

class SuggestedAction;

class UserPrivacySetting;

class UserStatus;

class accentColor;

class address;

class attachmentMenuBot;

class background;

class basicGroup;

class basicGroupFullInfo;

class businessBotManageBar;

class businessConnection;

class businessMessage;

class call;

class chat;

class chatActiveStories;

class chatBackground;

class chatBoost;

class chatFolderInfo;

class chatInviteLink;

class chatJoinRequest;

class chatJoinRequestsInfo;

class chatMember;

class chatNotificationSettings;

class chatPermissions;

class chatPhotoInfo;

class chatPosition;

class chatRevenueAmount;

class chatTheme;

class closeBirthdayUser;

class downloadedFileCounts;

class draftMessage;

class emojiStatus;

class error;

class factCheck;

class file;

class fileDownload;

class forumTopicInfo;

class groupCall;

class groupCallParticipant;

class languagePackString;

class location;

class message;

class messageInteractionInfo;

class messageReaction;

class notification;

class notificationGroup;

class orderInfo;

class poll;

class profileAccentColor;

class quickReplyMessage;

class quickReplyShortcut;

class reactionNotificationSettings;

class savedMessagesTags;

class savedMessagesTopic;

class scopeAutosaveSettings;

class scopeNotificationSettings;

class secretChat;

class starAmount;

class starRevenueStatus;

class sticker;

class stickerSet;

class story;

class supergroup;

class supergroupFullInfo;

class termsOfService;

class trendingStickerSets;

class unconfirmedSession;

class unreadReaction;

class user;

class userFullInfo;

class userPrivacySettingRules;

class videoChat;

class Update: public Object {
 public:
  static jclass Class;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class updateAuthorizationState final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AuthorizationState> authorization_state_;
  static jfieldID authorization_state_fieldID;

  updateAuthorizationState();

  explicit updateAuthorizationState(object_ptr<AuthorizationState> &&authorization_state_);

  static const std::int32_t ID = 1622347490;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewMessage final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;

  updateNewMessage();

  explicit updateNewMessage(object_ptr<message> &&message_);

  static const std::int32_t ID = -563105266;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageSendAcknowledged final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  updateMessageSendAcknowledged();

  updateMessageSendAcknowledged(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1302843961;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageSendSucceeded final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  int53 old_message_id_;
  static jfieldID old_message_id_fieldID;

  updateMessageSendSucceeded();

  updateMessageSendSucceeded(object_ptr<message> &&message_, int53 old_message_id_);

  static const std::int32_t ID = 1815715197;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageSendFailed final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<message> message_;
  static jfieldID message_fieldID;
  int53 old_message_id_;
  static jfieldID old_message_id_fieldID;
  object_ptr<error> error_;
  static jfieldID error_fieldID;

  updateMessageSendFailed();

  updateMessageSendFailed(object_ptr<message> &&message_, int53 old_message_id_, object_ptr<error> &&error_);

  static const std::int32_t ID = -635701017;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageContent final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageContent> new_content_;
  static jfieldID new_content_fieldID;

  updateMessageContent();

  updateMessageContent(int53 chat_id_, int53 message_id_, object_ptr<MessageContent> &&new_content_);

  static const std::int32_t ID = 506903332;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageEdited final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 edit_date_;
  static jfieldID edit_date_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  updateMessageEdited();

  updateMessageEdited(int53 chat_id_, int53 message_id_, int32 edit_date_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = -559545626;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageIsPinned final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  updateMessageIsPinned();

  updateMessageIsPinned(int53 chat_id_, int53 message_id_, bool is_pinned_);

  static const std::int32_t ID = 1102848829;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageInteractionInfo final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<messageInteractionInfo> interaction_info_;
  static jfieldID interaction_info_fieldID;

  updateMessageInteractionInfo();

  updateMessageInteractionInfo(int53 chat_id_, int53 message_id_, object_ptr<messageInteractionInfo> &&interaction_info_);

  static const std::int32_t ID = -1417659394;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageContentOpened final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  updateMessageContentOpened();

  updateMessageContentOpened(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1520523131;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageMentionRead final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 unread_mention_count_;
  static jfieldID unread_mention_count_fieldID;

  updateMessageMentionRead();

  updateMessageMentionRead(int53 chat_id_, int53 message_id_, int32 unread_mention_count_);

  static const std::int32_t ID = -252228282;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageUnreadReactions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  array<object_ptr<unreadReaction>> unread_reactions_;
  static jfieldID unread_reactions_fieldID;
  int32 unread_reaction_count_;
  static jfieldID unread_reaction_count_fieldID;

  updateMessageUnreadReactions();

  updateMessageUnreadReactions(int53 chat_id_, int53 message_id_, array<object_ptr<unreadReaction>> &&unread_reactions_, int32 unread_reaction_count_);

  static const std::int32_t ID = 942840008;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageFactCheck final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<factCheck> fact_check_;
  static jfieldID fact_check_fieldID;

  updateMessageFactCheck();

  updateMessageFactCheck(int53 chat_id_, int53 message_id_, object_ptr<factCheck> &&fact_check_);

  static const std::int32_t ID = 1014561538;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageLiveLocationViewed final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  updateMessageLiveLocationViewed();

  updateMessageLiveLocationViewed(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1308260971;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateVideoPublished final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  updateVideoPublished();

  updateVideoPublished(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -352575406;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewChat final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chat> chat_;
  static jfieldID chat_fieldID;

  updateNewChat();

  explicit updateNewChat(object_ptr<chat> &&chat_);

  static const std::int32_t ID = 2075757773;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatTitle final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string title_;
  static jfieldID title_fieldID;

  updateChatTitle();

  updateChatTitle(int53 chat_id_, string const &title_);

  static const std::int32_t ID = -175405660;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatPhoto final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatPhotoInfo> photo_;
  static jfieldID photo_fieldID;

  updateChatPhoto();

  updateChatPhoto(int53 chat_id_, object_ptr<chatPhotoInfo> &&photo_);

  static const std::int32_t ID = -324713921;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatAccentColors final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;
  int32 profile_accent_color_id_;
  static jfieldID profile_accent_color_id_fieldID;
  int64 profile_background_custom_emoji_id_;
  static jfieldID profile_background_custom_emoji_id_fieldID;

  updateChatAccentColors();

  updateChatAccentColors(int53 chat_id_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_);

  static const std::int32_t ID = -1212614407;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatPermissions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatPermissions> permissions_;
  static jfieldID permissions_fieldID;

  updateChatPermissions();

  updateChatPermissions(int53 chat_id_, object_ptr<chatPermissions> &&permissions_);

  static const std::int32_t ID = -1622010003;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatLastMessage final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<message> last_message_;
  static jfieldID last_message_fieldID;
  array<object_ptr<chatPosition>> positions_;
  static jfieldID positions_fieldID;

  updateChatLastMessage();

  updateChatLastMessage(int53 chat_id_, object_ptr<message> &&last_message_, array<object_ptr<chatPosition>> &&positions_);

  static const std::int32_t ID = -923244537;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatPosition final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatPosition> position_;
  static jfieldID position_fieldID;

  updateChatPosition();

  updateChatPosition(int53 chat_id_, object_ptr<chatPosition> &&position_);

  static const std::int32_t ID = -8979849;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatAddedToList final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;

  updateChatAddedToList();

  updateChatAddedToList(int53 chat_id_, object_ptr<ChatList> &&chat_list_);

  static const std::int32_t ID = -1418722068;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatRemovedFromList final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;

  updateChatRemovedFromList();

  updateChatRemovedFromList(int53 chat_id_, object_ptr<ChatList> &&chat_list_);

  static const std::int32_t ID = 1294647836;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatReadInbox final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 last_read_inbox_message_id_;
  static jfieldID last_read_inbox_message_id_fieldID;
  int32 unread_count_;
  static jfieldID unread_count_fieldID;

  updateChatReadInbox();

  updateChatReadInbox(int53 chat_id_, int53 last_read_inbox_message_id_, int32 unread_count_);

  static const std::int32_t ID = -797952281;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatReadOutbox final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 last_read_outbox_message_id_;
  static jfieldID last_read_outbox_message_id_fieldID;

  updateChatReadOutbox();

  updateChatReadOutbox(int53 chat_id_, int53 last_read_outbox_message_id_);

  static const std::int32_t ID = 708334213;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatActionBar final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatActionBar> action_bar_;
  static jfieldID action_bar_fieldID;

  updateChatActionBar();

  updateChatActionBar(int53 chat_id_, object_ptr<ChatActionBar> &&action_bar_);

  static const std::int32_t ID = -643671870;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatBusinessBotManageBar final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<businessBotManageBar> business_bot_manage_bar_;
  static jfieldID business_bot_manage_bar_fieldID;

  updateChatBusinessBotManageBar();

  updateChatBusinessBotManageBar(int53 chat_id_, object_ptr<businessBotManageBar> &&business_bot_manage_bar_);

  static const std::int32_t ID = -1104091145;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatAvailableReactions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatAvailableReactions> available_reactions_;
  static jfieldID available_reactions_fieldID;

  updateChatAvailableReactions();

  updateChatAvailableReactions(int53 chat_id_, object_ptr<ChatAvailableReactions> &&available_reactions_);

  static const std::int32_t ID = -1967909895;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatDraftMessage final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;
  array<object_ptr<chatPosition>> positions_;
  static jfieldID positions_fieldID;

  updateChatDraftMessage();

  updateChatDraftMessage(int53 chat_id_, object_ptr<draftMessage> &&draft_message_, array<object_ptr<chatPosition>> &&positions_);

  static const std::int32_t ID = 1455190380;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatEmojiStatus final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;

  updateChatEmojiStatus();

  updateChatEmojiStatus(int53 chat_id_, object_ptr<emojiStatus> &&emoji_status_);

  static const std::int32_t ID = 2004444432;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatMessageSender final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> message_sender_id_;
  static jfieldID message_sender_id_fieldID;

  updateChatMessageSender();

  updateChatMessageSender(int53 chat_id_, object_ptr<MessageSender> &&message_sender_id_);

  static const std::int32_t ID = 2003849793;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatMessageAutoDeleteTime final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;

  updateChatMessageAutoDeleteTime();

  updateChatMessageAutoDeleteTime(int53 chat_id_, int32 message_auto_delete_time_);

  static const std::int32_t ID = 1900174821;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatNotificationSettings final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  updateChatNotificationSettings();

  updateChatNotificationSettings(int53 chat_id_, object_ptr<chatNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = -803163050;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatPendingJoinRequests final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatJoinRequestsInfo> pending_join_requests_;
  static jfieldID pending_join_requests_fieldID;

  updateChatPendingJoinRequests();

  updateChatPendingJoinRequests(int53 chat_id_, object_ptr<chatJoinRequestsInfo> &&pending_join_requests_);

  static const std::int32_t ID = 348578785;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatReplyMarkup final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 reply_markup_message_id_;
  static jfieldID reply_markup_message_id_fieldID;

  updateChatReplyMarkup();

  updateChatReplyMarkup(int53 chat_id_, int53 reply_markup_message_id_);

  static const std::int32_t ID = 1309386144;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatBackground final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatBackground> background_;
  static jfieldID background_fieldID;

  updateChatBackground();

  updateChatBackground(int53 chat_id_, object_ptr<chatBackground> &&background_);

  static const std::int32_t ID = -6473549;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatTheme final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  updateChatTheme();

  updateChatTheme(int53 chat_id_, string const &theme_name_);

  static const std::int32_t ID = 838063205;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatUnreadMentionCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 unread_mention_count_;
  static jfieldID unread_mention_count_fieldID;

  updateChatUnreadMentionCount();

  updateChatUnreadMentionCount(int53 chat_id_, int32 unread_mention_count_);

  static const std::int32_t ID = -2131461348;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatUnreadReactionCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 unread_reaction_count_;
  static jfieldID unread_reaction_count_fieldID;

  updateChatUnreadReactionCount();

  updateChatUnreadReactionCount(int53 chat_id_, int32 unread_reaction_count_);

  static const std::int32_t ID = -2124399395;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatVideoChat final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<videoChat> video_chat_;
  static jfieldID video_chat_fieldID;

  updateChatVideoChat();

  updateChatVideoChat(int53 chat_id_, object_ptr<videoChat> &&video_chat_);

  static const std::int32_t ID = 637226150;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatDefaultDisableNotification final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool default_disable_notification_;
  static jfieldID default_disable_notification_fieldID;

  updateChatDefaultDisableNotification();

  updateChatDefaultDisableNotification(int53 chat_id_, bool default_disable_notification_);

  static const std::int32_t ID = 464087707;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatHasProtectedContent final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool has_protected_content_;
  static jfieldID has_protected_content_fieldID;

  updateChatHasProtectedContent();

  updateChatHasProtectedContent(int53 chat_id_, bool has_protected_content_);

  static const std::int32_t ID = 1800406811;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatIsTranslatable final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_translatable_;
  static jfieldID is_translatable_fieldID;

  updateChatIsTranslatable();

  updateChatIsTranslatable(int53 chat_id_, bool is_translatable_);

  static const std::int32_t ID = 2063799831;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatIsMarkedAsUnread final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_marked_as_unread_;
  static jfieldID is_marked_as_unread_fieldID;

  updateChatIsMarkedAsUnread();

  updateChatIsMarkedAsUnread(int53 chat_id_, bool is_marked_as_unread_);

  static const std::int32_t ID = 1468347188;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatViewAsTopics final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool view_as_topics_;
  static jfieldID view_as_topics_fieldID;

  updateChatViewAsTopics();

  updateChatViewAsTopics(int53 chat_id_, bool view_as_topics_);

  static const std::int32_t ID = 1543444029;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatBlockList final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;

  updateChatBlockList();

  updateChatBlockList(int53 chat_id_, object_ptr<BlockList> &&block_list_);

  static const std::int32_t ID = -2027228018;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatHasScheduledMessages final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool has_scheduled_messages_;
  static jfieldID has_scheduled_messages_fieldID;

  updateChatHasScheduledMessages();

  updateChatHasScheduledMessages(int53 chat_id_, bool has_scheduled_messages_);

  static const std::int32_t ID = 2064958167;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatFolders final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatFolderInfo>> chat_folders_;
  static jfieldID chat_folders_fieldID;
  int32 main_chat_list_position_;
  static jfieldID main_chat_list_position_fieldID;
  bool are_tags_enabled_;
  static jfieldID are_tags_enabled_fieldID;

  updateChatFolders();

  updateChatFolders(array<object_ptr<chatFolderInfo>> &&chat_folders_, int32 main_chat_list_position_, bool are_tags_enabled_);

  static const std::int32_t ID = 1998101395;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatOnlineMemberCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 online_member_count_;
  static jfieldID online_member_count_fieldID;

  updateChatOnlineMemberCount();

  updateChatOnlineMemberCount(int53 chat_id_, int32 online_member_count_);

  static const std::int32_t ID = 487369373;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSavedMessagesTopic final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<savedMessagesTopic> topic_;
  static jfieldID topic_fieldID;

  updateSavedMessagesTopic();

  explicit updateSavedMessagesTopic(object_ptr<savedMessagesTopic> &&topic_);

  static const std::int32_t ID = -1618855120;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSavedMessagesTopicCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 topic_count_;
  static jfieldID topic_count_fieldID;

  updateSavedMessagesTopicCount();

  explicit updateSavedMessagesTopicCount(int32 topic_count_);

  static const std::int32_t ID = -70092335;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateQuickReplyShortcut final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<quickReplyShortcut> shortcut_;
  static jfieldID shortcut_fieldID;

  updateQuickReplyShortcut();

  explicit updateQuickReplyShortcut(object_ptr<quickReplyShortcut> &&shortcut_);

  static const std::int32_t ID = -963430193;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateQuickReplyShortcutDeleted final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;

  updateQuickReplyShortcutDeleted();

  explicit updateQuickReplyShortcutDeleted(int32 shortcut_id_);

  static const std::int32_t ID = -390480838;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateQuickReplyShortcuts final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> shortcut_ids_;
  static jfieldID shortcut_ids_fieldID;

  updateQuickReplyShortcuts();

  explicit updateQuickReplyShortcuts(array<int32> &&shortcut_ids_);

  static const std::int32_t ID = -1994849731;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateQuickReplyShortcutMessages final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  array<object_ptr<quickReplyMessage>> messages_;
  static jfieldID messages_fieldID;

  updateQuickReplyShortcutMessages();

  updateQuickReplyShortcutMessages(int32 shortcut_id_, array<object_ptr<quickReplyMessage>> &&messages_);

  static const std::int32_t ID = -1396685225;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateForumTopicInfo final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<forumTopicInfo> info_;
  static jfieldID info_fieldID;

  updateForumTopicInfo();

  explicit updateForumTopicInfo(object_ptr<forumTopicInfo> &&info_);

  static const std::int32_t ID = 1420762696;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateForumTopic final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;
  int53 last_read_inbox_message_id_;
  static jfieldID last_read_inbox_message_id_fieldID;
  int53 last_read_outbox_message_id_;
  static jfieldID last_read_outbox_message_id_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  updateForumTopic();

  updateForumTopic(int53 chat_id_, int53 message_thread_id_, bool is_pinned_, int53 last_read_inbox_message_id_, int53 last_read_outbox_message_id_, object_ptr<chatNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = 266794534;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateScopeNotificationSettings final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NotificationSettingsScope> scope_;
  static jfieldID scope_fieldID;
  object_ptr<scopeNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  updateScopeNotificationSettings();

  updateScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_, object_ptr<scopeNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = -1203975309;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateReactionNotificationSettings final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<reactionNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  updateReactionNotificationSettings();

  explicit updateReactionNotificationSettings(object_ptr<reactionNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = -447932436;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNotification final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 notification_group_id_;
  static jfieldID notification_group_id_fieldID;
  object_ptr<notification> notification_;
  static jfieldID notification_fieldID;

  updateNotification();

  updateNotification(int32 notification_group_id_, object_ptr<notification> &&notification_);

  static const std::int32_t ID = -1897496876;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNotificationGroup final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 notification_group_id_;
  static jfieldID notification_group_id_fieldID;
  object_ptr<NotificationGroupType> type_;
  static jfieldID type_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 notification_settings_chat_id_;
  static jfieldID notification_settings_chat_id_fieldID;
  int64 notification_sound_id_;
  static jfieldID notification_sound_id_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<object_ptr<notification>> added_notifications_;
  static jfieldID added_notifications_fieldID;
  array<int32> removed_notification_ids_;
  static jfieldID removed_notification_ids_fieldID;

  updateNotificationGroup();

  updateNotificationGroup(int32 notification_group_id_, object_ptr<NotificationGroupType> &&type_, int53 chat_id_, int53 notification_settings_chat_id_, int64 notification_sound_id_, int32 total_count_, array<object_ptr<notification>> &&added_notifications_, array<int32> &&removed_notification_ids_);

  static const std::int32_t ID = 1381081378;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateActiveNotifications final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<notificationGroup>> groups_;
  static jfieldID groups_fieldID;

  updateActiveNotifications();

  explicit updateActiveNotifications(array<object_ptr<notificationGroup>> &&groups_);

  static const std::int32_t ID = -1306672221;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateHavePendingNotifications final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool have_delayed_notifications_;
  static jfieldID have_delayed_notifications_fieldID;
  bool have_unreceived_notifications_;
  static jfieldID have_unreceived_notifications_fieldID;

  updateHavePendingNotifications();

  updateHavePendingNotifications(bool have_delayed_notifications_, bool have_unreceived_notifications_);

  static const std::int32_t ID = 179233243;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateDeleteMessages final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  bool is_permanent_;
  static jfieldID is_permanent_fieldID;
  bool from_cache_;
  static jfieldID from_cache_fieldID;

  updateDeleteMessages();

  updateDeleteMessages(int53 chat_id_, array<int53> &&message_ids_, bool is_permanent_, bool from_cache_);

  static const std::int32_t ID = 1669252686;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatAction final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  object_ptr<ChatAction> action_;
  static jfieldID action_fieldID;

  updateChatAction();

  updateChatAction(int53 chat_id_, int53 message_thread_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<ChatAction> &&action_);

  static const std::int32_t ID = -1698703832;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUserStatus final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<UserStatus> status_;
  static jfieldID status_fieldID;

  updateUserStatus();

  updateUserStatus(int53 user_id_, object_ptr<UserStatus> &&status_);

  static const std::int32_t ID = 958468625;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUser final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<user> user_;
  static jfieldID user_fieldID;

  updateUser();

  explicit updateUser(object_ptr<user> &&user_);

  static const std::int32_t ID = 1183394041;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateBasicGroup final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<basicGroup> basic_group_;
  static jfieldID basic_group_fieldID;

  updateBasicGroup();

  explicit updateBasicGroup(object_ptr<basicGroup> &&basic_group_);

  static const std::int32_t ID = -1003239581;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSupergroup final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<supergroup> supergroup_;
  static jfieldID supergroup_fieldID;

  updateSupergroup();

  explicit updateSupergroup(object_ptr<supergroup> &&supergroup_);

  static const std::int32_t ID = -76782300;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSecretChat final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<secretChat> secret_chat_;
  static jfieldID secret_chat_fieldID;

  updateSecretChat();

  explicit updateSecretChat(object_ptr<secretChat> &&secret_chat_);

  static const std::int32_t ID = -1666903253;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUserFullInfo final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<userFullInfo> user_full_info_;
  static jfieldID user_full_info_fieldID;

  updateUserFullInfo();

  updateUserFullInfo(int53 user_id_, object_ptr<userFullInfo> &&user_full_info_);

  static const std::int32_t ID = -51197161;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateBasicGroupFullInfo final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;
  object_ptr<basicGroupFullInfo> basic_group_full_info_;
  static jfieldID basic_group_full_info_fieldID;

  updateBasicGroupFullInfo();

  updateBasicGroupFullInfo(int53 basic_group_id_, object_ptr<basicGroupFullInfo> &&basic_group_full_info_);

  static const std::int32_t ID = 1391881151;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSupergroupFullInfo final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  object_ptr<supergroupFullInfo> supergroup_full_info_;
  static jfieldID supergroup_full_info_fieldID;

  updateSupergroupFullInfo();

  updateSupergroupFullInfo(int53 supergroup_id_, object_ptr<supergroupFullInfo> &&supergroup_full_info_);

  static const std::int32_t ID = 435539214;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateServiceNotification final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string type_;
  static jfieldID type_fieldID;
  object_ptr<MessageContent> content_;
  static jfieldID content_fieldID;

  updateServiceNotification();

  updateServiceNotification(string const &type_, object_ptr<MessageContent> &&content_);

  static const std::int32_t ID = 1318622637;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFile final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<file> file_;
  static jfieldID file_fieldID;

  updateFile();

  explicit updateFile(object_ptr<file> &&file_);

  static const std::int32_t ID = 114132831;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileGenerationStart final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 generation_id_;
  static jfieldID generation_id_fieldID;
  string original_path_;
  static jfieldID original_path_fieldID;
  string destination_path_;
  static jfieldID destination_path_fieldID;
  string conversion_;
  static jfieldID conversion_fieldID;

  updateFileGenerationStart();

  updateFileGenerationStart(int64 generation_id_, string const &original_path_, string const &destination_path_, string const &conversion_);

  static const std::int32_t ID = 216817388;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileGenerationStop final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 generation_id_;
  static jfieldID generation_id_fieldID;

  updateFileGenerationStop();

  explicit updateFileGenerationStop(int64 generation_id_);

  static const std::int32_t ID = -1894449685;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileDownloads final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 total_size_;
  static jfieldID total_size_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  int53 downloaded_size_;
  static jfieldID downloaded_size_fieldID;

  updateFileDownloads();

  updateFileDownloads(int53 total_size_, int32 total_count_, int53 downloaded_size_);

  static const std::int32_t ID = -389213497;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileAddedToDownloads final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<fileDownload> file_download_;
  static jfieldID file_download_fieldID;
  object_ptr<downloadedFileCounts> counts_;
  static jfieldID counts_fieldID;

  updateFileAddedToDownloads();

  updateFileAddedToDownloads(object_ptr<fileDownload> &&file_download_, object_ptr<downloadedFileCounts> &&counts_);

  static const std::int32_t ID = 1609929242;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileDownload final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  int32 complete_date_;
  static jfieldID complete_date_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;
  object_ptr<downloadedFileCounts> counts_;
  static jfieldID counts_fieldID;

  updateFileDownload();

  updateFileDownload(int32 file_id_, int32 complete_date_, bool is_paused_, object_ptr<downloadedFileCounts> &&counts_);

  static const std::int32_t ID = 875529162;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFileRemovedFromDownloads final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  object_ptr<downloadedFileCounts> counts_;
  static jfieldID counts_fieldID;

  updateFileRemovedFromDownloads();

  updateFileRemovedFromDownloads(int32 file_id_, object_ptr<downloadedFileCounts> &&counts_);

  static const std::int32_t ID = 1853625576;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateApplicationVerificationRequired final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 verification_id_;
  static jfieldID verification_id_fieldID;
  string nonce_;
  static jfieldID nonce_fieldID;
  int64 cloud_project_number_;
  static jfieldID cloud_project_number_fieldID;

  updateApplicationVerificationRequired();

  updateApplicationVerificationRequired(int53 verification_id_, string const &nonce_, int64 cloud_project_number_);

  static const std::int32_t ID = -979607081;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateApplicationRecaptchaVerificationRequired final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 verification_id_;
  static jfieldID verification_id_fieldID;
  string action_;
  static jfieldID action_fieldID;
  string recaptcha_key_id_;
  static jfieldID recaptcha_key_id_fieldID;

  updateApplicationRecaptchaVerificationRequired();

  updateApplicationRecaptchaVerificationRequired(int53 verification_id_, string const &action_, string const &recaptcha_key_id_);

  static const std::int32_t ID = -1796351554;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateCall final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<call> call_;
  static jfieldID call_fieldID;

  updateCall();

  explicit updateCall(object_ptr<call> &&call_);

  static const std::int32_t ID = 1337184477;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateGroupCall final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<groupCall> group_call_;
  static jfieldID group_call_fieldID;

  updateGroupCall();

  explicit updateGroupCall(object_ptr<groupCall> &&group_call_);

  static const std::int32_t ID = 808603136;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateGroupCallParticipant final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<groupCallParticipant> participant_;
  static jfieldID participant_fieldID;

  updateGroupCallParticipant();

  updateGroupCallParticipant(int32 group_call_id_, object_ptr<groupCallParticipant> &&participant_);

  static const std::int32_t ID = -803128071;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateGroupCallParticipants final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  array<int64> participant_user_ids_;
  static jfieldID participant_user_ids_fieldID;

  updateGroupCallParticipants();

  updateGroupCallParticipants(int32 group_call_id_, array<int64> &&participant_user_ids_);

  static const std::int32_t ID = -1874006374;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateGroupCallVerificationState final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int32 generation_;
  static jfieldID generation_fieldID;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  updateGroupCallVerificationState();

  updateGroupCallVerificationState(int32 group_call_id_, int32 generation_, array<string> &&emojis_);

  static const std::int32_t ID = 682965060;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewCallSignalingData final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  updateNewCallSignalingData();

  updateNewCallSignalingData(int32 call_id_, bytes const &data_);

  static const std::int32_t ID = 583634317;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUserPrivacySettingRules final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<UserPrivacySetting> setting_;
  static jfieldID setting_fieldID;
  object_ptr<userPrivacySettingRules> rules_;
  static jfieldID rules_fieldID;

  updateUserPrivacySettingRules();

  updateUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_, object_ptr<userPrivacySettingRules> &&rules_);

  static const std::int32_t ID = -912960778;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUnreadMessageCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  int32 unread_count_;
  static jfieldID unread_count_fieldID;
  int32 unread_unmuted_count_;
  static jfieldID unread_unmuted_count_fieldID;

  updateUnreadMessageCount();

  updateUnreadMessageCount(object_ptr<ChatList> &&chat_list_, int32 unread_count_, int32 unread_unmuted_count_);

  static const std::int32_t ID = 78987721;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUnreadChatCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  int32 unread_count_;
  static jfieldID unread_count_fieldID;
  int32 unread_unmuted_count_;
  static jfieldID unread_unmuted_count_fieldID;
  int32 marked_as_unread_count_;
  static jfieldID marked_as_unread_count_fieldID;
  int32 marked_as_unread_unmuted_count_;
  static jfieldID marked_as_unread_unmuted_count_fieldID;

  updateUnreadChatCount();

  updateUnreadChatCount(object_ptr<ChatList> &&chat_list_, int32 total_count_, int32 unread_count_, int32 unread_unmuted_count_, int32 marked_as_unread_count_, int32 marked_as_unread_unmuted_count_);

  static const std::int32_t ID = 1994494530;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStory final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<story> story_;
  static jfieldID story_fieldID;

  updateStory();

  explicit updateStory(object_ptr<story> &&story_);

  static const std::int32_t ID = 419845935;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStoryDeleted final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  updateStoryDeleted();

  updateStoryDeleted(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -582743166;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStoryPostSucceeded final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<story> story_;
  static jfieldID story_fieldID;
  int32 old_story_id_;
  static jfieldID old_story_id_fieldID;

  updateStoryPostSucceeded();

  updateStoryPostSucceeded(object_ptr<story> &&story_, int32 old_story_id_);

  static const std::int32_t ID = -1712432318;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStoryPostFailed final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<story> story_;
  static jfieldID story_fieldID;
  object_ptr<error> error_;
  static jfieldID error_fieldID;
  object_ptr<CanPostStoryResult> error_type_;
  static jfieldID error_type_fieldID;

  updateStoryPostFailed();

  updateStoryPostFailed(object_ptr<story> &&story_, object_ptr<error> &&error_, object_ptr<CanPostStoryResult> &&error_type_);

  static const std::int32_t ID = 1248168444;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatActiveStories final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatActiveStories> active_stories_;
  static jfieldID active_stories_fieldID;

  updateChatActiveStories();

  explicit updateChatActiveStories(object_ptr<chatActiveStories> &&active_stories_);

  static const std::int32_t ID = 2037935148;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStoryListChatCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StoryList> story_list_;
  static jfieldID story_list_fieldID;
  int32 chat_count_;
  static jfieldID chat_count_fieldID;

  updateStoryListChatCount();

  updateStoryListChatCount(object_ptr<StoryList> &&story_list_, int32 chat_count_);

  static const std::int32_t ID = -2009871041;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStoryStealthMode final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 active_until_date_;
  static jfieldID active_until_date_fieldID;
  int32 cooldown_until_date_;
  static jfieldID cooldown_until_date_fieldID;

  updateStoryStealthMode();

  updateStoryStealthMode(int32 active_until_date_, int32 cooldown_until_date_);

  static const std::int32_t ID = 1878506778;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateOption final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<OptionValue> value_;
  static jfieldID value_fieldID;

  updateOption();

  updateOption(string const &name_, object_ptr<OptionValue> &&value_);

  static const std::int32_t ID = 900822020;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStickerSet final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<stickerSet> sticker_set_;
  static jfieldID sticker_set_fieldID;

  updateStickerSet();

  explicit updateStickerSet(object_ptr<stickerSet> &&sticker_set_);

  static const std::int32_t ID = 1879268812;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateInstalledStickerSets final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  array<int64> sticker_set_ids_;
  static jfieldID sticker_set_ids_fieldID;

  updateInstalledStickerSets();

  updateInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, array<int64> &&sticker_set_ids_);

  static const std::int32_t ID = -1735084182;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateTrendingStickerSets final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  object_ptr<trendingStickerSets> sticker_sets_;
  static jfieldID sticker_sets_fieldID;

  updateTrendingStickerSets();

  updateTrendingStickerSets(object_ptr<StickerType> &&sticker_type_, object_ptr<trendingStickerSets> &&sticker_sets_);

  static const std::int32_t ID = 1266307239;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateRecentStickers final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_attached_;
  static jfieldID is_attached_fieldID;
  array<int32> sticker_ids_;
  static jfieldID sticker_ids_fieldID;

  updateRecentStickers();

  updateRecentStickers(bool is_attached_, array<int32> &&sticker_ids_);

  static const std::int32_t ID = 1906403540;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFavoriteStickers final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> sticker_ids_;
  static jfieldID sticker_ids_fieldID;

  updateFavoriteStickers();

  explicit updateFavoriteStickers(array<int32> &&sticker_ids_);

  static const std::int32_t ID = 1662240999;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSavedAnimations final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> animation_ids_;
  static jfieldID animation_ids_fieldID;

  updateSavedAnimations();

  explicit updateSavedAnimations(array<int32> &&animation_ids_);

  static const std::int32_t ID = 65563814;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSavedNotificationSounds final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int64> notification_sound_ids_;
  static jfieldID notification_sound_ids_fieldID;

  updateSavedNotificationSounds();

  explicit updateSavedNotificationSounds(array<int64> &&notification_sound_ids_);

  static const std::int32_t ID = 1052725698;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateDefaultBackground final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool for_dark_theme_;
  static jfieldID for_dark_theme_fieldID;
  object_ptr<background> background_;
  static jfieldID background_fieldID;

  updateDefaultBackground();

  updateDefaultBackground(bool for_dark_theme_, object_ptr<background> &&background_);

  static const std::int32_t ID = -716139217;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatThemes final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<chatTheme>> chat_themes_;
  static jfieldID chat_themes_fieldID;

  updateChatThemes();

  explicit updateChatThemes(array<object_ptr<chatTheme>> &&chat_themes_);

  static const std::int32_t ID = -1588098376;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAccentColors final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<accentColor>> colors_;
  static jfieldID colors_fieldID;
  array<int32> available_accent_color_ids_;
  static jfieldID available_accent_color_ids_fieldID;

  updateAccentColors();

  updateAccentColors(array<object_ptr<accentColor>> &&colors_, array<int32> &&available_accent_color_ids_);

  static const std::int32_t ID = -1197047738;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateProfileAccentColors final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<profileAccentColor>> colors_;
  static jfieldID colors_fieldID;
  array<int32> available_accent_color_ids_;
  static jfieldID available_accent_color_ids_fieldID;

  updateProfileAccentColors();

  updateProfileAccentColors(array<object_ptr<profileAccentColor>> &&colors_, array<int32> &&available_accent_color_ids_);

  static const std::int32_t ID = 605202104;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateLanguagePackStrings final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string localization_target_;
  static jfieldID localization_target_fieldID;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;
  array<object_ptr<languagePackString>> strings_;
  static jfieldID strings_fieldID;

  updateLanguagePackStrings();

  updateLanguagePackStrings(string const &localization_target_, string const &language_pack_id_, array<object_ptr<languagePackString>> &&strings_);

  static const std::int32_t ID = -1056319886;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateConnectionState final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ConnectionState> state_;
  static jfieldID state_fieldID;

  updateConnectionState();

  explicit updateConnectionState(object_ptr<ConnectionState> &&state_);

  static const std::int32_t ID = 1469292078;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateFreezeState final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_frozen_;
  static jfieldID is_frozen_fieldID;
  int32 freezing_date_;
  static jfieldID freezing_date_fieldID;
  int32 deletion_date_;
  static jfieldID deletion_date_fieldID;
  string appeal_link_;
  static jfieldID appeal_link_fieldID;

  updateFreezeState();

  updateFreezeState(bool is_frozen_, int32 freezing_date_, int32 deletion_date_, string const &appeal_link_);

  static const std::int32_t ID = 1692410540;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateTermsOfService final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string terms_of_service_id_;
  static jfieldID terms_of_service_id_fieldID;
  object_ptr<termsOfService> terms_of_service_;
  static jfieldID terms_of_service_fieldID;

  updateTermsOfService();

  updateTermsOfService(string const &terms_of_service_id_, object_ptr<termsOfService> &&terms_of_service_);

  static const std::int32_t ID = -1304640162;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateUnconfirmedSession final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<unconfirmedSession> session_;
  static jfieldID session_fieldID;

  updateUnconfirmedSession();

  explicit updateUnconfirmedSession(object_ptr<unconfirmedSession> &&session_);

  static const std::int32_t ID = -22673268;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAttachmentMenuBots final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<attachmentMenuBot>> bots_;
  static jfieldID bots_fieldID;

  updateAttachmentMenuBots();

  explicit updateAttachmentMenuBots(array<object_ptr<attachmentMenuBot>> &&bots_);

  static const std::int32_t ID = 291369922;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateWebAppMessageSent final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 web_app_launch_id_;
  static jfieldID web_app_launch_id_fieldID;

  updateWebAppMessageSent();

  explicit updateWebAppMessageSent(int64 web_app_launch_id_);

  static const std::int32_t ID = 1480790569;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateActiveEmojiReactions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  updateActiveEmojiReactions();

  explicit updateActiveEmojiReactions(array<string> &&emojis_);

  static const std::int32_t ID = 77556818;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAvailableMessageEffects final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int64> reaction_effect_ids_;
  static jfieldID reaction_effect_ids_fieldID;
  array<int64> sticker_effect_ids_;
  static jfieldID sticker_effect_ids_fieldID;

  updateAvailableMessageEffects();

  updateAvailableMessageEffects(array<int64> &&reaction_effect_ids_, array<int64> &&sticker_effect_ids_);

  static const std::int32_t ID = 1964701061;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateDefaultReactionType final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;

  updateDefaultReactionType();

  explicit updateDefaultReactionType(object_ptr<ReactionType> &&reaction_type_);

  static const std::int32_t ID = 1264668933;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateDefaultPaidReactionType final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PaidReactionType> type_;
  static jfieldID type_fieldID;

  updateDefaultPaidReactionType();

  explicit updateDefaultPaidReactionType(object_ptr<PaidReactionType> &&type_);

  static const std::int32_t ID = 38198599;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSavedMessagesTags final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  object_ptr<savedMessagesTags> tags_;
  static jfieldID tags_fieldID;

  updateSavedMessagesTags();

  updateSavedMessagesTags(int53 saved_messages_topic_id_, object_ptr<savedMessagesTags> &&tags_);

  static const std::int32_t ID = 1938178634;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateActiveLiveLocationMessages final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<message>> messages_;
  static jfieldID messages_fieldID;

  updateActiveLiveLocationMessages();

  explicit updateActiveLiveLocationMessages(array<object_ptr<message>> &&messages_);

  static const std::int32_t ID = -1308142440;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateOwnedStarCount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<starAmount> star_amount_;
  static jfieldID star_amount_fieldID;

  updateOwnedStarCount();

  explicit updateOwnedStarCount(object_ptr<starAmount> &&star_amount_);

  static const std::int32_t ID = -1350647928;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatRevenueAmount final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatRevenueAmount> revenue_amount_;
  static jfieldID revenue_amount_fieldID;

  updateChatRevenueAmount();

  updateChatRevenueAmount(int53 chat_id_, object_ptr<chatRevenueAmount> &&revenue_amount_);

  static const std::int32_t ID = -959857468;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateStarRevenueStatus final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  object_ptr<starRevenueStatus> status_;
  static jfieldID status_fieldID;

  updateStarRevenueStatus();

  updateStarRevenueStatus(object_ptr<MessageSender> &&owner_id_, object_ptr<starRevenueStatus> &&status_);

  static const std::int32_t ID = -280232757;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSpeechRecognitionTrial final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 max_media_duration_;
  static jfieldID max_media_duration_fieldID;
  int32 weekly_count_;
  static jfieldID weekly_count_fieldID;
  int32 left_count_;
  static jfieldID left_count_fieldID;
  int32 next_reset_date_;
  static jfieldID next_reset_date_fieldID;

  updateSpeechRecognitionTrial();

  updateSpeechRecognitionTrial(int32 max_media_duration_, int32 weekly_count_, int32 left_count_, int32 next_reset_date_);

  static const std::int32_t ID = -11600703;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateDiceEmojis final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  updateDiceEmojis();

  explicit updateDiceEmojis(array<string> &&emojis_);

  static const std::int32_t ID = -1069066940;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAnimatedEmojiMessageClicked final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;

  updateAnimatedEmojiMessageClicked();

  updateAnimatedEmojiMessageClicked(int53 chat_id_, int53 message_id_, object_ptr<sticker> &&sticker_);

  static const std::int32_t ID = -1558809595;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAnimationSearchParameters final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string provider_;
  static jfieldID provider_fieldID;
  array<string> emojis_;
  static jfieldID emojis_fieldID;

  updateAnimationSearchParameters();

  updateAnimationSearchParameters(string const &provider_, array<string> &&emojis_);

  static const std::int32_t ID = -1144983202;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSuggestedActions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<SuggestedAction>> added_actions_;
  static jfieldID added_actions_fieldID;
  array<object_ptr<SuggestedAction>> removed_actions_;
  static jfieldID removed_actions_fieldID;

  updateSuggestedActions();

  updateSuggestedActions(array<object_ptr<SuggestedAction>> &&added_actions_, array<object_ptr<SuggestedAction>> &&removed_actions_);

  static const std::int32_t ID = 1459452346;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateSpeedLimitNotification final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_upload_;
  static jfieldID is_upload_fieldID;

  updateSpeedLimitNotification();

  explicit updateSpeedLimitNotification(bool is_upload_);

  static const std::int32_t ID = -964437912;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateContactCloseBirthdays final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<closeBirthdayUser>> close_birthday_users_;
  static jfieldID close_birthday_users_fieldID;

  updateContactCloseBirthdays();

  explicit updateContactCloseBirthdays(array<object_ptr<closeBirthdayUser>> &&close_birthday_users_);

  static const std::int32_t ID = -36007873;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateAutosaveSettings final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AutosaveSettingsScope> scope_;
  static jfieldID scope_fieldID;
  object_ptr<scopeAutosaveSettings> settings_;
  static jfieldID settings_fieldID;

  updateAutosaveSettings();

  updateAutosaveSettings(object_ptr<AutosaveSettingsScope> &&scope_, object_ptr<scopeAutosaveSettings> &&settings_);

  static const std::int32_t ID = -634958069;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateBusinessConnection final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessConnection> connection_;
  static jfieldID connection_fieldID;

  updateBusinessConnection();

  explicit updateBusinessConnection(object_ptr<businessConnection> &&connection_);

  static const std::int32_t ID = -2043480970;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewBusinessMessage final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string connection_id_;
  static jfieldID connection_id_fieldID;
  object_ptr<businessMessage> message_;
  static jfieldID message_fieldID;

  updateNewBusinessMessage();

  updateNewBusinessMessage(string const &connection_id_, object_ptr<businessMessage> &&message_);

  static const std::int32_t ID = -2034350524;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateBusinessMessageEdited final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string connection_id_;
  static jfieldID connection_id_fieldID;
  object_ptr<businessMessage> message_;
  static jfieldID message_fieldID;

  updateBusinessMessageEdited();

  updateBusinessMessageEdited(string const &connection_id_, object_ptr<businessMessage> &&message_);

  static const std::int32_t ID = -2119799415;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateBusinessMessagesDeleted final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string connection_id_;
  static jfieldID connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  updateBusinessMessagesDeleted();

  updateBusinessMessagesDeleted(string const &connection_id_, int53 chat_id_, array<int53> &&message_ids_);

  static const std::int32_t ID = -1106703050;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewInlineQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  object_ptr<location> user_location_;
  static jfieldID user_location_fieldID;
  object_ptr<ChatType> chat_type_;
  static jfieldID chat_type_fieldID;
  string query_;
  static jfieldID query_fieldID;
  string offset_;
  static jfieldID offset_fieldID;

  updateNewInlineQuery();

  updateNewInlineQuery(int64 id_, int53 sender_user_id_, object_ptr<location> &&user_location_, object_ptr<ChatType> &&chat_type_, string const &query_, string const &offset_);

  static const std::int32_t ID = 1903279924;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewChosenInlineResult final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  object_ptr<location> user_location_;
  static jfieldID user_location_fieldID;
  string query_;
  static jfieldID query_fieldID;
  string result_id_;
  static jfieldID result_id_fieldID;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;

  updateNewChosenInlineResult();

  updateNewChosenInlineResult(int53 sender_user_id_, object_ptr<location> &&user_location_, string const &query_, string const &result_id_, string const &inline_message_id_);

  static const std::int32_t ID = -884191395;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewCallbackQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int64 chat_instance_;
  static jfieldID chat_instance_fieldID;
  object_ptr<CallbackQueryPayload> payload_;
  static jfieldID payload_fieldID;

  updateNewCallbackQuery();

  updateNewCallbackQuery(int64 id_, int53 sender_user_id_, int53 chat_id_, int53 message_id_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_);

  static const std::int32_t ID = -1989881762;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewInlineCallbackQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  int64 chat_instance_;
  static jfieldID chat_instance_fieldID;
  object_ptr<CallbackQueryPayload> payload_;
  static jfieldID payload_fieldID;

  updateNewInlineCallbackQuery();

  updateNewInlineCallbackQuery(int64 id_, int53 sender_user_id_, string const &inline_message_id_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_);

  static const std::int32_t ID = -319212358;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewBusinessCallbackQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  string connection_id_;
  static jfieldID connection_id_fieldID;
  object_ptr<businessMessage> message_;
  static jfieldID message_fieldID;
  int64 chat_instance_;
  static jfieldID chat_instance_fieldID;
  object_ptr<CallbackQueryPayload> payload_;
  static jfieldID payload_fieldID;

  updateNewBusinessCallbackQuery();

  updateNewBusinessCallbackQuery(int64 id_, int53 sender_user_id_, string const &connection_id_, object_ptr<businessMessage> &&message_, int64 chat_instance_, object_ptr<CallbackQueryPayload> &&payload_);

  static const std::int32_t ID = 336745316;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewShippingQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  string invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  object_ptr<address> shipping_address_;
  static jfieldID shipping_address_fieldID;

  updateNewShippingQuery();

  updateNewShippingQuery(int64 id_, int53 sender_user_id_, string const &invoice_payload_, object_ptr<address> &&shipping_address_);

  static const std::int32_t ID = 693651058;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewPreCheckoutQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  int53 sender_user_id_;
  static jfieldID sender_user_id_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 total_amount_;
  static jfieldID total_amount_fieldID;
  bytes invoice_payload_;
  static jfieldID invoice_payload_fieldID;
  string shipping_option_id_;
  static jfieldID shipping_option_id_fieldID;
  object_ptr<orderInfo> order_info_;
  static jfieldID order_info_fieldID;

  updateNewPreCheckoutQuery();

  updateNewPreCheckoutQuery(int64 id_, int53 sender_user_id_, string const &currency_, int53 total_amount_, bytes const &invoice_payload_, string const &shipping_option_id_, object_ptr<orderInfo> &&order_info_);

  static const std::int32_t ID = 708342217;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewCustomEvent final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string event_;
  static jfieldID event_fieldID;

  updateNewCustomEvent();

  explicit updateNewCustomEvent(string const &event_);

  static const std::int32_t ID = 1994222092;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewCustomQuery final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string data_;
  static jfieldID data_fieldID;
  int32 timeout_;
  static jfieldID timeout_fieldID;

  updateNewCustomQuery();

  updateNewCustomQuery(int64 id_, string const &data_, int32 timeout_);

  static const std::int32_t ID = -687670874;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updatePoll final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<poll> poll_;
  static jfieldID poll_fieldID;

  updatePoll();

  explicit updatePoll(object_ptr<poll> &&poll_);

  static const std::int32_t ID = -1771342902;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updatePollAnswer final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 poll_id_;
  static jfieldID poll_id_fieldID;
  object_ptr<MessageSender> voter_id_;
  static jfieldID voter_id_fieldID;
  array<int32> option_ids_;
  static jfieldID option_ids_fieldID;

  updatePollAnswer();

  updatePollAnswer(int64 poll_id_, object_ptr<MessageSender> &&voter_id_, array<int32> &&option_ids_);

  static const std::int32_t ID = 1104905219;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatMember final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 actor_user_id_;
  static jfieldID actor_user_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;
  bool via_join_request_;
  static jfieldID via_join_request_fieldID;
  bool via_chat_folder_invite_link_;
  static jfieldID via_chat_folder_invite_link_fieldID;
  object_ptr<chatMember> old_chat_member_;
  static jfieldID old_chat_member_fieldID;
  object_ptr<chatMember> new_chat_member_;
  static jfieldID new_chat_member_fieldID;

  updateChatMember();

  updateChatMember(int53 chat_id_, int53 actor_user_id_, int32 date_, object_ptr<chatInviteLink> &&invite_link_, bool via_join_request_, bool via_chat_folder_invite_link_, object_ptr<chatMember> &&old_chat_member_, object_ptr<chatMember> &&new_chat_member_);

  static const std::int32_t ID = -1736025145;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateNewChatJoinRequest final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatJoinRequest> request_;
  static jfieldID request_fieldID;
  int53 user_chat_id_;
  static jfieldID user_chat_id_fieldID;
  object_ptr<chatInviteLink> invite_link_;
  static jfieldID invite_link_fieldID;

  updateNewChatJoinRequest();

  updateNewChatJoinRequest(int53 chat_id_, object_ptr<chatJoinRequest> &&request_, int53 user_chat_id_, object_ptr<chatInviteLink> &&invite_link_);

  static const std::int32_t ID = 2118694979;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateChatBoost final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatBoost> boost_;
  static jfieldID boost_fieldID;

  updateChatBoost();

  updateChatBoost(int53 chat_id_, object_ptr<chatBoost> &&boost_);

  static const std::int32_t ID = 1349680676;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageReaction final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageSender> actor_id_;
  static jfieldID actor_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  array<object_ptr<ReactionType>> old_reaction_types_;
  static jfieldID old_reaction_types_fieldID;
  array<object_ptr<ReactionType>> new_reaction_types_;
  static jfieldID new_reaction_types_fieldID;

  updateMessageReaction();

  updateMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&actor_id_, int32 date_, array<object_ptr<ReactionType>> &&old_reaction_types_, array<object_ptr<ReactionType>> &&new_reaction_types_);

  static const std::int32_t ID = 1084895706;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updateMessageReactions final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;
  array<object_ptr<messageReaction>> reactions_;
  static jfieldID reactions_fieldID;

  updateMessageReactions();

  updateMessageReactions(int53 chat_id_, int53 message_id_, int32 date_, array<object_ptr<messageReaction>> &&reactions_);

  static const std::int32_t ID = 955237189;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class updatePaidMediaPurchased final : public Update {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string payload_;
  static jfieldID payload_fieldID;

  updatePaidMediaPurchased();

  updatePaidMediaPurchased(int53 user_id_, string const &payload_);

  static const std::int32_t ID = -1542396325;

  static object_ptr<Update> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class Update;

class updates final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<Update>> updates_;
  static jfieldID updates_fieldID;

  updates();

  explicit updates(array<object_ptr<Update>> &&updates_);

  static const std::int32_t ID = 475842347;

  static object_ptr<updates> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGift;

class upgradeGiftResult final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGift> gift_;
  static jfieldID gift_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  bool is_saved_;
  static jfieldID is_saved_fieldID;
  bool can_be_transferred_;
  static jfieldID can_be_transferred_fieldID;
  int53 transfer_star_count_;
  static jfieldID transfer_star_count_fieldID;
  int32 next_transfer_date_;
  static jfieldID next_transfer_date_fieldID;
  int32 next_resale_date_;
  static jfieldID next_resale_date_fieldID;
  int32 export_date_;
  static jfieldID export_date_fieldID;

  upgradeGiftResult();

  upgradeGiftResult(object_ptr<upgradedGift> &&gift_, string const &received_gift_id_, bool is_saved_, bool can_be_transferred_, int53 transfer_star_count_, int32 next_transfer_date_, int32 next_resale_date_, int32 export_date_);

  static const std::int32_t ID = 233376346;

  static object_ptr<upgradeGiftResult> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class upgradedGiftBackdrop;

class upgradedGiftModel;

class upgradedGiftOriginalDetails;

class upgradedGiftSymbol;

class upgradedGift final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 id_;
  static jfieldID id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string name_;
  static jfieldID name_fieldID;
  int32 number_;
  static jfieldID number_fieldID;
  int32 total_upgraded_count_;
  static jfieldID total_upgraded_count_fieldID;
  int32 max_upgraded_count_;
  static jfieldID max_upgraded_count_fieldID;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  string owner_address_;
  static jfieldID owner_address_fieldID;
  string owner_name_;
  static jfieldID owner_name_fieldID;
  string gift_address_;
  static jfieldID gift_address_fieldID;
  object_ptr<upgradedGiftModel> model_;
  static jfieldID model_fieldID;
  object_ptr<upgradedGiftSymbol> symbol_;
  static jfieldID symbol_fieldID;
  object_ptr<upgradedGiftBackdrop> backdrop_;
  static jfieldID backdrop_fieldID;
  object_ptr<upgradedGiftOriginalDetails> original_details_;
  static jfieldID original_details_fieldID;
  int53 resale_star_count_;
  static jfieldID resale_star_count_fieldID;

  upgradedGift();

  upgradedGift(int64 id_, string const &title_, string const &name_, int32 number_, int32 total_upgraded_count_, int32 max_upgraded_count_, object_ptr<MessageSender> &&owner_id_, string const &owner_address_, string const &owner_name_, string const &gift_address_, object_ptr<upgradedGiftModel> &&model_, object_ptr<upgradedGiftSymbol> &&symbol_, object_ptr<upgradedGiftBackdrop> &&backdrop_, object_ptr<upgradedGiftOriginalDetails> &&original_details_, int53 resale_star_count_);

  static const std::int32_t ID = 992832410;

  static object_ptr<upgradedGift> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UpgradedGiftAttributeId: public Object {
 public:
  static jclass Class;

  static object_ptr<UpgradedGiftAttributeId> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftAttributeIdModel final : public UpgradedGiftAttributeId {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 sticker_id_;
  static jfieldID sticker_id_fieldID;

  upgradedGiftAttributeIdModel();

  explicit upgradedGiftAttributeIdModel(int64 sticker_id_);

  static const std::int32_t ID = 1053287307;

  static object_ptr<UpgradedGiftAttributeId> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftAttributeIdSymbol final : public UpgradedGiftAttributeId {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 sticker_id_;
  static jfieldID sticker_id_fieldID;

  upgradedGiftAttributeIdSymbol();

  explicit upgradedGiftAttributeIdSymbol(int64 sticker_id_);

  static const std::int32_t ID = 1188205608;

  static object_ptr<UpgradedGiftAttributeId> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftAttributeIdBackdrop final : public UpgradedGiftAttributeId {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 backdrop_id_;
  static jfieldID backdrop_id_fieldID;

  upgradedGiftAttributeIdBackdrop();

  explicit upgradedGiftAttributeIdBackdrop(int32 backdrop_id_);

  static const std::int32_t ID = 1461997935;

  static object_ptr<UpgradedGiftAttributeId> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftBackdropColors;

class upgradedGiftBackdrop final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 id_;
  static jfieldID id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<upgradedGiftBackdropColors> colors_;
  static jfieldID colors_fieldID;
  int32 rarity_per_mille_;
  static jfieldID rarity_per_mille_fieldID;

  upgradedGiftBackdrop();

  upgradedGiftBackdrop(int32 id_, string const &name_, object_ptr<upgradedGiftBackdropColors> &&colors_, int32 rarity_per_mille_);

  static const std::int32_t ID = -378289242;

  static object_ptr<upgradedGiftBackdrop> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftBackdropColors final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 center_color_;
  static jfieldID center_color_fieldID;
  int32 edge_color_;
  static jfieldID edge_color_fieldID;
  int32 symbol_color_;
  static jfieldID symbol_color_fieldID;
  int32 text_color_;
  static jfieldID text_color_fieldID;

  upgradedGiftBackdropColors();

  upgradedGiftBackdropColors(int32 center_color_, int32 edge_color_, int32 symbol_color_, int32 text_color_);

  static const std::int32_t ID = 4227529;

  static object_ptr<upgradedGiftBackdropColors> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftBackdrop;

class upgradedGiftBackdropCount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGiftBackdrop> backdrop_;
  static jfieldID backdrop_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;

  upgradedGiftBackdropCount();

  upgradedGiftBackdropCount(object_ptr<upgradedGiftBackdrop> &&backdrop_, int32 total_count_);

  static const std::int32_t ID = -562274120;

  static object_ptr<upgradedGiftBackdropCount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class upgradedGiftModel final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  int32 rarity_per_mille_;
  static jfieldID rarity_per_mille_fieldID;

  upgradedGiftModel();

  upgradedGiftModel(string const &name_, object_ptr<sticker> &&sticker_, int32 rarity_per_mille_);

  static const std::int32_t ID = 1360156751;

  static object_ptr<upgradedGiftModel> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftModel;

class upgradedGiftModelCount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGiftModel> model_;
  static jfieldID model_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;

  upgradedGiftModelCount();

  upgradedGiftModelCount(object_ptr<upgradedGiftModel> &&model_, int32 total_count_);

  static const std::int32_t ID = -1127238023;

  static object_ptr<upgradedGiftModelCount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class formattedText;

class upgradedGiftOriginalDetails final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  object_ptr<MessageSender> receiver_id_;
  static jfieldID receiver_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  upgradedGiftOriginalDetails();

  upgradedGiftOriginalDetails(object_ptr<MessageSender> &&sender_id_, object_ptr<MessageSender> &&receiver_id_, object_ptr<formattedText> &&text_, int32 date_);

  static const std::int32_t ID = 55247728;

  static object_ptr<upgradedGiftOriginalDetails> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class upgradedGiftSymbol final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<sticker> sticker_;
  static jfieldID sticker_fieldID;
  int32 rarity_per_mille_;
  static jfieldID rarity_per_mille_fieldID;

  upgradedGiftSymbol();

  upgradedGiftSymbol(string const &name_, object_ptr<sticker> &&sticker_, int32 rarity_per_mille_);

  static const std::int32_t ID = 1128318383;

  static object_ptr<upgradedGiftSymbol> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGiftSymbol;

class upgradedGiftSymbolCount final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<upgradedGiftSymbol> symbol_;
  static jfieldID symbol_fieldID;
  int32 total_count_;
  static jfieldID total_count_fieldID;

  upgradedGiftSymbolCount();

  upgradedGiftSymbolCount(object_ptr<upgradedGiftSymbol> &&symbol_, int32 total_count_);

  static const std::int32_t ID = -773579874;

  static object_ptr<upgradedGiftSymbolCount> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserStatus;

class UserType;

class emojiStatus;

class profilePhoto;

class usernames;

class verificationStatus;

class user final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 id_;
  static jfieldID id_fieldID;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;
  object_ptr<usernames> usernames_;
  static jfieldID usernames_fieldID;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  object_ptr<UserStatus> status_;
  static jfieldID status_fieldID;
  object_ptr<profilePhoto> profile_photo_;
  static jfieldID profile_photo_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;
  int32 profile_accent_color_id_;
  static jfieldID profile_accent_color_id_fieldID;
  int64 profile_background_custom_emoji_id_;
  static jfieldID profile_background_custom_emoji_id_fieldID;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;
  bool is_contact_;
  static jfieldID is_contact_fieldID;
  bool is_mutual_contact_;
  static jfieldID is_mutual_contact_fieldID;
  bool is_close_friend_;
  static jfieldID is_close_friend_fieldID;
  object_ptr<verificationStatus> verification_status_;
  static jfieldID verification_status_fieldID;
  bool is_premium_;
  static jfieldID is_premium_fieldID;
  bool is_support_;
  static jfieldID is_support_fieldID;
  string restriction_reason_;
  static jfieldID restriction_reason_fieldID;
  bool has_active_stories_;
  static jfieldID has_active_stories_fieldID;
  bool has_unread_active_stories_;
  static jfieldID has_unread_active_stories_fieldID;
  bool restricts_new_chats_;
  static jfieldID restricts_new_chats_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;
  bool have_access_;
  static jfieldID have_access_fieldID;
  object_ptr<UserType> type_;
  static jfieldID type_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  bool added_to_attachment_menu_;
  static jfieldID added_to_attachment_menu_fieldID;

  user();

  user(int53 id_, string const &first_name_, string const &last_name_, object_ptr<usernames> &&usernames_, string const &phone_number_, object_ptr<UserStatus> &&status_, object_ptr<profilePhoto> &&profile_photo_, int32 accent_color_id_, int64 background_custom_emoji_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_, object_ptr<emojiStatus> &&emoji_status_, bool is_contact_, bool is_mutual_contact_, bool is_close_friend_, object_ptr<verificationStatus> &&verification_status_, bool is_premium_, bool is_support_, string const &restriction_reason_, bool has_active_stories_, bool has_unread_active_stories_, bool restricts_new_chats_, int53 paid_message_star_count_, bool have_access_, object_ptr<UserType> &&type_, string const &language_code_, bool added_to_attachment_menu_);

  static const std::int32_t ID = -1848524366;

  static object_ptr<user> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class BlockList;

class birthdate;

class botInfo;

class botVerification;

class businessInfo;

class chatPhoto;

class formattedText;

class giftSettings;

class userFullInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatPhoto> personal_photo_;
  static jfieldID personal_photo_fieldID;
  object_ptr<chatPhoto> photo_;
  static jfieldID photo_fieldID;
  object_ptr<chatPhoto> public_photo_;
  static jfieldID public_photo_fieldID;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;
  bool can_be_called_;
  static jfieldID can_be_called_fieldID;
  bool supports_video_calls_;
  static jfieldID supports_video_calls_fieldID;
  bool has_private_calls_;
  static jfieldID has_private_calls_fieldID;
  bool has_private_forwards_;
  static jfieldID has_private_forwards_fieldID;
  bool has_restricted_voice_and_video_note_messages_;
  static jfieldID has_restricted_voice_and_video_note_messages_fieldID;
  bool has_posted_to_profile_stories_;
  static jfieldID has_posted_to_profile_stories_fieldID;
  bool has_sponsored_messages_enabled_;
  static jfieldID has_sponsored_messages_enabled_fieldID;
  bool need_phone_number_privacy_exception_;
  static jfieldID need_phone_number_privacy_exception_fieldID;
  bool set_chat_background_;
  static jfieldID set_chat_background_fieldID;
  object_ptr<formattedText> bio_;
  static jfieldID bio_fieldID;
  object_ptr<birthdate> birthdate_;
  static jfieldID birthdate_fieldID;
  int53 personal_chat_id_;
  static jfieldID personal_chat_id_fieldID;
  int32 gift_count_;
  static jfieldID gift_count_fieldID;
  int32 group_in_common_count_;
  static jfieldID group_in_common_count_fieldID;
  int53 incoming_paid_message_star_count_;
  static jfieldID incoming_paid_message_star_count_fieldID;
  int53 outgoing_paid_message_star_count_;
  static jfieldID outgoing_paid_message_star_count_fieldID;
  object_ptr<giftSettings> gift_settings_;
  static jfieldID gift_settings_fieldID;
  object_ptr<botVerification> bot_verification_;
  static jfieldID bot_verification_fieldID;
  object_ptr<businessInfo> business_info_;
  static jfieldID business_info_fieldID;
  object_ptr<botInfo> bot_info_;
  static jfieldID bot_info_fieldID;

  userFullInfo();

  userFullInfo(object_ptr<chatPhoto> &&personal_photo_, object_ptr<chatPhoto> &&photo_, object_ptr<chatPhoto> &&public_photo_, object_ptr<BlockList> &&block_list_, bool can_be_called_, bool supports_video_calls_, bool has_private_calls_, bool has_private_forwards_, bool has_restricted_voice_and_video_note_messages_, bool has_posted_to_profile_stories_, bool has_sponsored_messages_enabled_, bool need_phone_number_privacy_exception_, bool set_chat_background_, object_ptr<formattedText> &&bio_, object_ptr<birthdate> &&birthdate_, int53 personal_chat_id_, int32 gift_count_, int32 group_in_common_count_, int53 incoming_paid_message_star_count_, int53 outgoing_paid_message_star_count_, object_ptr<giftSettings> &&gift_settings_, object_ptr<botVerification> &&bot_verification_, object_ptr<businessInfo> &&business_info_, object_ptr<botInfo> &&bot_info_);

  static const std::int32_t ID = -121173496;

  static object_ptr<userFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userLink final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  int32 expires_in_;
  static jfieldID expires_in_fieldID;

  userLink();

  userLink(string const &url_, int32 expires_in_);

  static const std::int32_t ID = 498138872;

  static object_ptr<userLink> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserPrivacySetting: public Object {
 public:
  static jclass Class;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowStatus final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowStatus();

  static const std::int32_t ID = 1862829310;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowProfilePhoto final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowProfilePhoto();

  static const std::int32_t ID = 1408485877;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowLinkInForwardedMessages final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowLinkInForwardedMessages();

  static const std::int32_t ID = 592688870;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowPhoneNumber final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowPhoneNumber();

  static const std::int32_t ID = -791567831;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowBio final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowBio();

  static const std::int32_t ID = 959981409;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingShowBirthdate final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingShowBirthdate();

  static const std::int32_t ID = 1167504607;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowChatInvites final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowChatInvites();

  static const std::int32_t ID = 1271668007;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowCalls final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowCalls();

  static const std::int32_t ID = -906967291;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowPeerToPeerCalls final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowPeerToPeerCalls();

  static const std::int32_t ID = 352500032;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowFindingByPhoneNumber final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowFindingByPhoneNumber();

  static const std::int32_t ID = -1846645423;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowPrivateVoiceAndVideoNoteMessages();

  static const std::int32_t ID = 338112060;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAutosaveGifts final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAutosaveGifts();

  static const std::int32_t ID = 1889167821;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingAllowUnpaidMessages final : public UserPrivacySetting {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingAllowUnpaidMessages();

  static const std::int32_t ID = 1430051047;

  static object_ptr<UserPrivacySetting> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserPrivacySettingRule: public Object {
 public:
  static jclass Class;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowAll final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleAllowAll();

  static const std::int32_t ID = -1967186881;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowContacts final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleAllowContacts();

  static const std::int32_t ID = -1892733680;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowBots final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleAllowBots();

  static const std::int32_t ID = 1404208925;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowPremiumUsers final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleAllowPremiumUsers();

  static const std::int32_t ID = 1624147265;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowUsers final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  userPrivacySettingRuleAllowUsers();

  explicit userPrivacySettingRuleAllowUsers(array<int53> &&user_ids_);

  static const std::int32_t ID = 1110988334;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleAllowChatMembers final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  userPrivacySettingRuleAllowChatMembers();

  explicit userPrivacySettingRuleAllowChatMembers(array<int53> &&chat_ids_);

  static const std::int32_t ID = -2048749863;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleRestrictAll final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleRestrictAll();

  static const std::int32_t ID = -1406495408;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleRestrictContacts final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleRestrictContacts();

  static const std::int32_t ID = 1008389378;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleRestrictBots final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userPrivacySettingRuleRestrictBots();

  static const std::int32_t ID = -1902547363;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleRestrictUsers final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  userPrivacySettingRuleRestrictUsers();

  explicit userPrivacySettingRuleRestrictUsers(array<int53> &&user_ids_);

  static const std::int32_t ID = 622796522;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userPrivacySettingRuleRestrictChatMembers final : public UserPrivacySettingRule {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  userPrivacySettingRuleRestrictChatMembers();

  explicit userPrivacySettingRuleRestrictChatMembers(array<int53> &&chat_ids_);

  static const std::int32_t ID = 392530897;

  static object_ptr<UserPrivacySettingRule> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserPrivacySettingRule;

class userPrivacySettingRules final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<UserPrivacySettingRule>> rules_;
  static jfieldID rules_fieldID;

  userPrivacySettingRules();

  explicit userPrivacySettingRules(array<object_ptr<UserPrivacySettingRule>> &&rules_);

  static const std::int32_t ID = 322477541;

  static object_ptr<userPrivacySettingRules> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserStatus: public Object {
 public:
  static jclass Class;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userStatusEmpty final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userStatusEmpty();

  static const std::int32_t ID = 164646985;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userStatusOnline final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 expires_;
  static jfieldID expires_fieldID;

  userStatusOnline();

  explicit userStatusOnline(int32 expires_);

  static const std::int32_t ID = -1529460876;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userStatusOffline final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 was_online_;
  static jfieldID was_online_fieldID;

  userStatusOffline();

  explicit userStatusOffline(int32 was_online_);

  static const std::int32_t ID = -759984891;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userStatusRecently final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool by_my_privacy_settings_;
  static jfieldID by_my_privacy_settings_fieldID;

  userStatusRecently();

  explicit userStatusRecently(bool by_my_privacy_settings_);

  static const std::int32_t ID = 262824117;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userStatusLastWeek final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool by_my_privacy_settings_;
  static jfieldID by_my_privacy_settings_fieldID;

  userStatusLastWeek();

  explicit userStatusLastWeek(bool by_my_privacy_settings_);

  static const std::int32_t ID = 310385495;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userStatusLastMonth final : public UserStatus {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool by_my_privacy_settings_;
  static jfieldID by_my_privacy_settings_fieldID;

  userStatusLastMonth();

  explicit userStatusLastMonth(bool by_my_privacy_settings_);

  static const std::int32_t ID = -1194644996;

  static object_ptr<UserStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class userSupportInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> message_;
  static jfieldID message_fieldID;
  string author_;
  static jfieldID author_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  userSupportInfo();

  userSupportInfo(object_ptr<formattedText> &&message_, string const &author_, int32 date_);

  static const std::int32_t ID = -1257366487;

  static object_ptr<userSupportInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class UserType: public Object {
 public:
  static jclass Class;

  static object_ptr<UserType> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userTypeRegular final : public UserType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userTypeRegular();

  static const std::int32_t ID = -598644325;

  static object_ptr<UserType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userTypeDeleted final : public UserType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userTypeDeleted();

  static const std::int32_t ID = -1807729372;

  static object_ptr<UserType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userTypeBot final : public UserType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool can_be_edited_;
  static jfieldID can_be_edited_fieldID;
  bool can_join_groups_;
  static jfieldID can_join_groups_fieldID;
  bool can_read_all_group_messages_;
  static jfieldID can_read_all_group_messages_fieldID;
  bool has_main_web_app_;
  static jfieldID has_main_web_app_fieldID;
  bool is_inline_;
  static jfieldID is_inline_fieldID;
  string inline_query_placeholder_;
  static jfieldID inline_query_placeholder_fieldID;
  bool need_location_;
  static jfieldID need_location_fieldID;
  bool can_connect_to_business_;
  static jfieldID can_connect_to_business_fieldID;
  bool can_be_added_to_attachment_menu_;
  static jfieldID can_be_added_to_attachment_menu_fieldID;
  int32 active_user_count_;
  static jfieldID active_user_count_fieldID;

  userTypeBot();

  userTypeBot(bool can_be_edited_, bool can_join_groups_, bool can_read_all_group_messages_, bool has_main_web_app_, bool is_inline_, string const &inline_query_placeholder_, bool need_location_, bool can_connect_to_business_, bool can_be_added_to_attachment_menu_, int32 active_user_count_);

  static const std::int32_t ID = -1952199642;

  static object_ptr<UserType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class userTypeUnknown final : public UserType {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  userTypeUnknown();

  static const std::int32_t ID = -724541123;

  static object_ptr<UserType> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class usernames final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> active_usernames_;
  static jfieldID active_usernames_fieldID;
  array<string> disabled_usernames_;
  static jfieldID disabled_usernames_fieldID;
  string editable_username_;
  static jfieldID editable_username_fieldID;

  usernames();

  usernames(array<string> &&active_usernames_, array<string> &&disabled_usernames_, string const &editable_username_);

  static const std::int32_t ID = 799608565;

  static object_ptr<usernames> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class users final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 total_count_;
  static jfieldID total_count_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  users();

  users(int32 total_count_, array<int53> &&user_ids_);

  static const std::int32_t ID = 171203420;

  static object_ptr<users> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class shippingOption;

class validatedOrderInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string order_info_id_;
  static jfieldID order_info_id_fieldID;
  array<object_ptr<shippingOption>> shipping_options_;
  static jfieldID shipping_options_fieldID;

  validatedOrderInfo();

  validatedOrderInfo(string const &order_info_id_, array<object_ptr<shippingOption>> &&shipping_options_);

  static const std::int32_t ID = 1511451484;

  static object_ptr<validatedOrderInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class point;

class VectorPathCommand: public Object {
 public:
  static jclass Class;

  static object_ptr<VectorPathCommand> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class vectorPathCommandLine final : public VectorPathCommand {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<point> end_point_;
  static jfieldID end_point_fieldID;

  vectorPathCommandLine();

  explicit vectorPathCommandLine(object_ptr<point> &&end_point_);

  static const std::int32_t ID = -614056822;

  static object_ptr<VectorPathCommand> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class vectorPathCommandCubicBezierCurve final : public VectorPathCommand {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<point> start_control_point_;
  static jfieldID start_control_point_fieldID;
  object_ptr<point> end_control_point_;
  static jfieldID end_control_point_fieldID;
  object_ptr<point> end_point_;
  static jfieldID end_point_fieldID;

  vectorPathCommandCubicBezierCurve();

  vectorPathCommandCubicBezierCurve(object_ptr<point> &&start_control_point_, object_ptr<point> &&end_control_point_, object_ptr<point> &&end_point_);

  static const std::int32_t ID = 1229733434;

  static object_ptr<VectorPathCommand> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class location;

class venue final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string address_;
  static jfieldID address_fieldID;
  string provider_;
  static jfieldID provider_fieldID;
  string id_;
  static jfieldID id_fieldID;
  string type_;
  static jfieldID type_fieldID;

  venue();

  venue(object_ptr<location> &&location_, string const &title_, string const &address_, string const &provider_, string const &id_, string const &type_);

  static const std::int32_t ID = 1070406393;

  static object_ptr<venue> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class verificationStatus final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_verified_;
  static jfieldID is_verified_fieldID;
  bool is_scam_;
  static jfieldID is_scam_fieldID;
  bool is_fake_;
  static jfieldID is_fake_fieldID;
  int64 bot_verification_icon_custom_emoji_id_;
  static jfieldID bot_verification_icon_custom_emoji_id_fieldID;

  verificationStatus();

  verificationStatus(bool is_verified_, bool is_scam_, bool is_fake_, int64 bot_verification_icon_custom_emoji_id_);

  static const std::int32_t ID = 988193164;

  static object_ptr<verificationStatus> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class file;

class minithumbnail;

class thumbnail;

class video final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  string file_name_;
  static jfieldID file_name_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  bool has_stickers_;
  static jfieldID has_stickers_fieldID;
  bool supports_streaming_;
  static jfieldID supports_streaming_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<file> video_;
  static jfieldID video_fieldID;

  video();

  video(int32 duration_, int32 width_, int32 height_, string const &file_name_, string const &mime_type_, bool has_stickers_, bool supports_streaming_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<file> &&video_);

  static const std::int32_t ID = 832856268;

  static object_ptr<video> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class videoChat final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool has_participants_;
  static jfieldID has_participants_fieldID;
  object_ptr<MessageSender> default_participant_id_;
  static jfieldID default_participant_id_fieldID;

  videoChat();

  videoChat(int32 group_call_id_, bool has_participants_, object_ptr<MessageSender> &&default_participant_id_);

  static const std::int32_t ID = -1374319320;

  static object_ptr<videoChat> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class videoChatStream final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 channel_id_;
  static jfieldID channel_id_fieldID;
  int32 scale_;
  static jfieldID scale_fieldID;
  int53 time_offset_;
  static jfieldID time_offset_fieldID;

  videoChatStream();

  videoChatStream(int32 channel_id_, int32 scale_, int53 time_offset_);

  static const std::int32_t ID = -1151707172;

  static object_ptr<videoChatStream> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class videoChatStream;

class videoChatStreams final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<videoChatStream>> streams_;
  static jfieldID streams_fieldID;

  videoChatStreams();

  explicit videoChatStreams(array<object_ptr<videoChatStream>> &&streams_);

  static const std::int32_t ID = -434622720;

  static object_ptr<videoChatStreams> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SpeechRecognitionResult;

class file;

class minithumbnail;

class thumbnail;

class videoNote final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;
  bytes waveform_;
  static jfieldID waveform_fieldID;
  int32 length_;
  static jfieldID length_fieldID;
  object_ptr<minithumbnail> minithumbnail_;
  static jfieldID minithumbnail_fieldID;
  object_ptr<thumbnail> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<SpeechRecognitionResult> speech_recognition_result_;
  static jfieldID speech_recognition_result_fieldID;
  object_ptr<file> video_;
  static jfieldID video_fieldID;

  videoNote();

  videoNote(int32 duration_, bytes const &waveform_, int32 length_, object_ptr<minithumbnail> &&minithumbnail_, object_ptr<thumbnail> &&thumbnail_, object_ptr<SpeechRecognitionResult> &&speech_recognition_result_, object_ptr<file> &&video_);

  static const std::int32_t ID = 2062096581;

  static object_ptr<videoNote> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class SpeechRecognitionResult;

class file;

class voiceNote final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 duration_;
  static jfieldID duration_fieldID;
  bytes waveform_;
  static jfieldID waveform_fieldID;
  string mime_type_;
  static jfieldID mime_type_fieldID;
  object_ptr<SpeechRecognitionResult> speech_recognition_result_;
  static jfieldID speech_recognition_result_fieldID;
  object_ptr<file> voice_;
  static jfieldID voice_fieldID;

  voiceNote();

  voiceNote(int32 duration_, bytes const &waveform_, string const &mime_type_, object_ptr<SpeechRecognitionResult> &&speech_recognition_result_, object_ptr<file> &&voice_);

  static const std::int32_t ID = -1175302923;

  static object_ptr<voiceNote> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class animation;

class photo;

class webApp final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string short_name_;
  static jfieldID short_name_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<photo> photo_;
  static jfieldID photo_fieldID;
  object_ptr<animation> animation_;
  static jfieldID animation_fieldID;

  webApp();

  webApp(string const &short_name_, string const &title_, string const &description_, object_ptr<photo> &&photo_, object_ptr<animation> &&animation_);

  static const std::int32_t ID = 1616619763;

  static object_ptr<webApp> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class webAppInfo final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 launch_id_;
  static jfieldID launch_id_fieldID;
  string url_;
  static jfieldID url_fieldID;

  webAppInfo();

  webAppInfo(int64 launch_id_, string const &url_);

  static const std::int32_t ID = 788378344;

  static object_ptr<webAppInfo> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class WebAppOpenMode: public Object {
 public:
  static jclass Class;

  static object_ptr<WebAppOpenMode> fetch(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class webAppOpenModeCompact final : public WebAppOpenMode {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  webAppOpenModeCompact();

  static const std::int32_t ID = 1711603675;

  static object_ptr<WebAppOpenMode> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class webAppOpenModeFullSize final : public WebAppOpenMode {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  webAppOpenModeFullSize();

  static const std::int32_t ID = 189320513;

  static object_ptr<WebAppOpenMode> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class webAppOpenModeFullScreen final : public WebAppOpenMode {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  webAppOpenModeFullScreen();

  static const std::int32_t ID = 1871315357;

  static object_ptr<WebAppOpenMode> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class WebAppOpenMode;

class themeParameters;

class webAppOpenParameters final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<themeParameters> theme_;
  static jfieldID theme_fieldID;
  string application_name_;
  static jfieldID application_name_fieldID;
  object_ptr<WebAppOpenMode> mode_;
  static jfieldID mode_fieldID;

  webAppOpenParameters();

  webAppOpenParameters(object_ptr<themeParameters> &&theme_, string const &application_name_, object_ptr<WebAppOpenMode> &&mode_);

  static const std::int32_t ID = 1375356527;

  static object_ptr<webAppOpenParameters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class PageBlock;

class webPageInstantView final : public Object {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<PageBlock>> page_blocks_;
  static jfieldID page_blocks_fieldID;
  int32 view_count_;
  static jfieldID view_count_fieldID;
  int32 version_;
  static jfieldID version_fieldID;
  bool is_rtl_;
  static jfieldID is_rtl_fieldID;
  bool is_full_;
  static jfieldID is_full_fieldID;
  object_ptr<InternalLinkType> feedback_link_;
  static jfieldID feedback_link_fieldID;

  webPageInstantView();

  webPageInstantView(array<object_ptr<PageBlock>> &&page_blocks_, int32 view_count_, int32 version_, bool is_rtl_, bool is_full_, object_ptr<InternalLinkType> &&feedback_link_);

  static const std::int32_t ID = 778202453;

  static object_ptr<webPageInstantView> fetch(JNIEnv *env, jobject &p);

  void store(JNIEnv *env, jobject &s) const final;

  void store(TlStorerToString &s, const char *field_name) const final;

  static void init_jni_vars(JNIEnv *env);
};

class callProtocol;

class ok;

class acceptCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  object_ptr<callProtocol> protocol_;
  static jfieldID protocol_fieldID;

  acceptCall();

  acceptCall(int32 call_id_, object_ptr<callProtocol> &&protocol_);

  static const std::int32_t ID = -646618416;

  using ReturnType = object_ptr<ok>;

  static object_ptr<acceptCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class acceptTermsOfService final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string terms_of_service_id_;
  static jfieldID terms_of_service_id_fieldID;

  acceptTermsOfService();

  explicit acceptTermsOfService(string const &terms_of_service_id_);

  static const std::int32_t ID = 2130576356;

  using ReturnType = object_ptr<ok>;

  static object_ptr<acceptTermsOfService> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class activateStoryStealthMode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  activateStoryStealthMode();

  static const std::int32_t ID = -1009023855;

  using ReturnType = object_ptr<ok>;

  static object_ptr<activateStoryStealthMode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryContent;

class botMediaPreview;

class addBotMediaPreview final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  object_ptr<InputStoryContent> content_;
  static jfieldID content_fieldID;

  addBotMediaPreview();

  addBotMediaPreview(int53 bot_user_id_, string const &language_code_, object_ptr<InputStoryContent> &&content_);

  static const std::int32_t ID = 1347126571;

  using ReturnType = object_ptr<botMediaPreview>;

  static object_ptr<addBotMediaPreview> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class addChatFolderByInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  addChatFolderByInviteLink();

  addChatFolderByInviteLink(string const &invite_link_, array<int53> &&chat_ids_);

  static const std::int32_t ID = -858593816;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addChatFolderByInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class failedToAddMembers;

class addChatMember final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 forward_limit_;
  static jfieldID forward_limit_fieldID;

  addChatMember();

  addChatMember(int53 chat_id_, int53 user_id_, int32 forward_limit_);

  static const std::int32_t ID = 1720144407;

  using ReturnType = object_ptr<failedToAddMembers>;

  static object_ptr<addChatMember> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class failedToAddMembers;

class addChatMembers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  addChatMembers();

  addChatMembers(int53 chat_id_, array<int53> &&user_ids_);

  static const std::int32_t ID = -1675991329;

  using ReturnType = object_ptr<failedToAddMembers>;

  static object_ptr<addChatMembers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ok;

class addChatToList final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;

  addChatToList();

  addChatToList(int53 chat_id_, object_ptr<ChatList> &&chat_list_);

  static const std::int32_t ID = -80523595;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addChatToList> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class contact;

class ok;

class addContact final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<contact> contact_;
  static jfieldID contact_fieldID;
  bool share_phone_number_;
  static jfieldID share_phone_number_fieldID;

  addContact();

  addContact(object_ptr<contact> &&contact_, bool share_phone_number_);

  static const std::int32_t ID = 1869640000;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addContact> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class addCustomServerLanguagePack final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;

  addCustomServerLanguagePack();

  explicit addCustomServerLanguagePack(string const &language_pack_id_);

  static const std::int32_t ID = 4492771;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addCustomServerLanguagePack> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class addFavoriteSticker final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  addFavoriteSticker();

  explicit addFavoriteSticker(object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = 324504799;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addFavoriteSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class file;

class addFileToDownloads final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 priority_;
  static jfieldID priority_fieldID;

  addFileToDownloads();

  addFileToDownloads(int32 file_id_, int53 chat_id_, int53 message_id_, int32 priority_);

  static const std::int32_t ID = 867533751;

  using ReturnType = object_ptr<file>;

  static object_ptr<addFileToDownloads> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class MessageSender;

class message;

class addLocalMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  addLocalMessage();

  addLocalMessage(int53 chat_id_, object_ptr<MessageSender> &&sender_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -166217823;

  using ReturnType = object_ptr<message>;

  static object_ptr<addLocalMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class addLogMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 verbosity_level_;
  static jfieldID verbosity_level_fieldID;
  string text_;
  static jfieldID text_fieldID;

  addLogMessage();

  addLogMessage(int32 verbosity_level_, string const &text_);

  static const std::int32_t ID = 1597427692;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addLogMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class addMessageReaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  bool is_big_;
  static jfieldID is_big_fieldID;
  bool update_recent_reactions_;
  static jfieldID update_recent_reactions_fieldID;

  addMessageReaction();

  addMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_, bool is_big_, bool update_recent_reactions_);

  static const std::int32_t ID = 1419269613;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addMessageReaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NetworkStatisticsEntry;

class ok;

class addNetworkStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NetworkStatisticsEntry> entry_;
  static jfieldID entry_fieldID;

  addNetworkStatistics();

  explicit addNetworkStatistics(object_ptr<NetworkStatisticsEntry> &&entry_);

  static const std::int32_t ID = 1264825305;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addNetworkStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PaidReactionType;

class ok;

class addPendingPaidMessageReaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  object_ptr<PaidReactionType> type_;
  static jfieldID type_fieldID;

  addPendingPaidMessageReaction();

  addPendingPaidMessageReaction(int53 chat_id_, int53 message_id_, int53 star_count_, object_ptr<PaidReactionType> &&type_);

  static const std::int32_t ID = -342110765;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addPendingPaidMessageReaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType;

class proxy;

class addProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string server_;
  static jfieldID server_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  bool enable_;
  static jfieldID enable_fieldID;
  object_ptr<ProxyType> type_;
  static jfieldID type_fieldID;

  addProxy();

  addProxy(string const &server_, int32 port_, bool enable_, object_ptr<ProxyType> &&type_);

  static const std::int32_t ID = 331529432;

  using ReturnType = object_ptr<proxy>;

  static object_ptr<addProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class quickReplyMessage;

class addQuickReplyShortcutInlineQueryResultMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string shortcut_name_;
  static jfieldID shortcut_name_fieldID;
  int53 reply_to_message_id_;
  static jfieldID reply_to_message_id_fieldID;
  int64 query_id_;
  static jfieldID query_id_fieldID;
  string result_id_;
  static jfieldID result_id_fieldID;
  bool hide_via_bot_;
  static jfieldID hide_via_bot_fieldID;

  addQuickReplyShortcutInlineQueryResultMessage();

  addQuickReplyShortcutInlineQueryResultMessage(string const &shortcut_name_, int53 reply_to_message_id_, int64 query_id_, string const &result_id_, bool hide_via_bot_);

  static const std::int32_t ID = -2017449468;

  using ReturnType = object_ptr<quickReplyMessage>;

  static object_ptr<addQuickReplyShortcutInlineQueryResultMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class quickReplyMessage;

class addQuickReplyShortcutMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string shortcut_name_;
  static jfieldID shortcut_name_fieldID;
  int53 reply_to_message_id_;
  static jfieldID reply_to_message_id_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  addQuickReplyShortcutMessage();

  addQuickReplyShortcutMessage(string const &shortcut_name_, int53 reply_to_message_id_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 1058573098;

  using ReturnType = object_ptr<quickReplyMessage>;

  static object_ptr<addQuickReplyShortcutMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class quickReplyMessages;

class addQuickReplyShortcutMessageAlbum final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string shortcut_name_;
  static jfieldID shortcut_name_fieldID;
  int53 reply_to_message_id_;
  static jfieldID reply_to_message_id_fieldID;
  array<object_ptr<InputMessageContent>> input_message_contents_;
  static jfieldID input_message_contents_fieldID;

  addQuickReplyShortcutMessageAlbum();

  addQuickReplyShortcutMessageAlbum(string const &shortcut_name_, int53 reply_to_message_id_, array<object_ptr<InputMessageContent>> &&input_message_contents_);

  static const std::int32_t ID = 1348436244;

  using ReturnType = object_ptr<quickReplyMessages>;

  static object_ptr<addQuickReplyShortcutMessageAlbum> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class stickers;

class addRecentSticker final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_attached_;
  static jfieldID is_attached_fieldID;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  addRecentSticker();

  addRecentSticker(bool is_attached_, object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = -1478109026;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<addRecentSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class addRecentlyFoundChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  addRecentlyFoundChat();

  explicit addRecentlyFoundChat(int53 chat_id_);

  static const std::int32_t ID = -1746396787;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addRecentlyFoundChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class addSavedAnimation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> animation_;
  static jfieldID animation_fieldID;

  addSavedAnimation();

  explicit addSavedAnimation(object_ptr<InputFile> &&animation_);

  static const std::int32_t ID = -1538525088;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addSavedAnimation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class notificationSound;

class addSavedNotificationSound final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sound_;
  static jfieldID sound_fieldID;

  addSavedNotificationSound();

  explicit addSavedNotificationSound(object_ptr<InputFile> &&sound_);

  static const std::int32_t ID = 1043956975;

  using ReturnType = object_ptr<notificationSound>;

  static object_ptr<addSavedNotificationSound> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputSticker;

class ok;

class addStickerToSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<inputSticker> sticker_;
  static jfieldID sticker_fieldID;

  addStickerToSet();

  addStickerToSet(int53 user_id_, string const &name_, object_ptr<inputSticker> &&sticker_);

  static const std::int32_t ID = 1457266235;

  using ReturnType = object_ptr<ok>;

  static object_ptr<addStickerToSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class allowBotToSendMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  allowBotToSendMessages();

  explicit allowBotToSendMessages(int53 bot_user_id_);

  static const std::int32_t ID = 1776928142;

  using ReturnType = object_ptr<ok>;

  static object_ptr<allowBotToSendMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class allowUnpaidMessagesFromUser final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool refund_payments_;
  static jfieldID refund_payments_fieldID;

  allowUnpaidMessagesFromUser();

  allowUnpaidMessagesFromUser(int53 user_id_, bool refund_payments_);

  static const std::int32_t ID = 803569495;

  using ReturnType = object_ptr<ok>;

  static object_ptr<allowUnpaidMessagesFromUser> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class answerCallbackQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 callback_query_id_;
  static jfieldID callback_query_id_fieldID;
  string text_;
  static jfieldID text_fieldID;
  bool show_alert_;
  static jfieldID show_alert_fieldID;
  string url_;
  static jfieldID url_fieldID;
  int32 cache_time_;
  static jfieldID cache_time_fieldID;

  answerCallbackQuery();

  answerCallbackQuery(int64 callback_query_id_, string const &text_, bool show_alert_, string const &url_, int32 cache_time_);

  static const std::int32_t ID = -1153028490;

  using ReturnType = object_ptr<ok>;

  static object_ptr<answerCallbackQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class answerCustomQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 custom_query_id_;
  static jfieldID custom_query_id_fieldID;
  string data_;
  static jfieldID data_fieldID;

  answerCustomQuery();

  answerCustomQuery(int64 custom_query_id_, string const &data_);

  static const std::int32_t ID = -1293603521;

  using ReturnType = object_ptr<ok>;

  static object_ptr<answerCustomQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputInlineQueryResult;

class inlineQueryResultsButton;

class ok;

class answerInlineQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 inline_query_id_;
  static jfieldID inline_query_id_fieldID;
  bool is_personal_;
  static jfieldID is_personal_fieldID;
  object_ptr<inlineQueryResultsButton> button_;
  static jfieldID button_fieldID;
  array<object_ptr<InputInlineQueryResult>> results_;
  static jfieldID results_fieldID;
  int32 cache_time_;
  static jfieldID cache_time_fieldID;
  string next_offset_;
  static jfieldID next_offset_fieldID;

  answerInlineQuery();

  answerInlineQuery(int64 inline_query_id_, bool is_personal_, object_ptr<inlineQueryResultsButton> &&button_, array<object_ptr<InputInlineQueryResult>> &&results_, int32 cache_time_, string const &next_offset_);

  static const std::int32_t ID = 1343853844;

  using ReturnType = object_ptr<ok>;

  static object_ptr<answerInlineQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class answerPreCheckoutQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 pre_checkout_query_id_;
  static jfieldID pre_checkout_query_id_fieldID;
  string error_message_;
  static jfieldID error_message_fieldID;

  answerPreCheckoutQuery();

  answerPreCheckoutQuery(int64 pre_checkout_query_id_, string const &error_message_);

  static const std::int32_t ID = -1486789653;

  using ReturnType = object_ptr<ok>;

  static object_ptr<answerPreCheckoutQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class shippingOption;

class answerShippingQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 shipping_query_id_;
  static jfieldID shipping_query_id_fieldID;
  array<object_ptr<shippingOption>> shipping_options_;
  static jfieldID shipping_options_fieldID;
  string error_message_;
  static jfieldID error_message_fieldID;

  answerShippingQuery();

  answerShippingQuery(int64 shipping_query_id_, array<object_ptr<shippingOption>> &&shipping_options_, string const &error_message_);

  static const std::int32_t ID = -434601324;

  using ReturnType = object_ptr<ok>;

  static object_ptr<answerShippingQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputInlineQueryResult;

class sentWebAppMessage;

class answerWebAppQuery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string web_app_query_id_;
  static jfieldID web_app_query_id_fieldID;
  object_ptr<InputInlineQueryResult> result_;
  static jfieldID result_fieldID;

  answerWebAppQuery();

  answerWebAppQuery(string const &web_app_query_id_, object_ptr<InputInlineQueryResult> &&result_);

  static const std::int32_t ID = -1598776079;

  using ReturnType = object_ptr<sentWebAppMessage>;

  static object_ptr<answerWebAppQuery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class applyPremiumGiftCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  applyPremiumGiftCode();

  explicit applyPremiumGiftCode(string const &code_);

  static const std::int32_t ID = -1347138530;

  using ReturnType = object_ptr<ok>;

  static object_ptr<applyPremiumGiftCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StorePaymentPurpose;

class StoreTransaction;

class ok;

class assignStoreTransaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StoreTransaction> transaction_;
  static jfieldID transaction_fieldID;
  object_ptr<StorePaymentPurpose> purpose_;
  static jfieldID purpose_fieldID;

  assignStoreTransaction();

  assignStoreTransaction(object_ptr<StoreTransaction> &&transaction_, object_ptr<StorePaymentPurpose> &&purpose_);

  static const std::int32_t ID = -2046202900;

  using ReturnType = object_ptr<ok>;

  static object_ptr<assignStoreTransaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class banChatMember final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;
  int32 banned_until_date_;
  static jfieldID banned_until_date_fieldID;
  bool revoke_messages_;
  static jfieldID revoke_messages_fieldID;

  banChatMember();

  banChatMember(int53 chat_id_, object_ptr<MessageSender> &&member_id_, int32 banned_until_date_, bool revoke_messages_);

  static const std::int32_t ID = -888111748;

  using ReturnType = object_ptr<ok>;

  static object_ptr<banChatMember> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class banGroupCallParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  array<int64> user_ids_;
  static jfieldID user_ids_fieldID;

  banGroupCallParticipants();

  banGroupCallParticipants(int32 group_call_id_, array<int64> &&user_ids_);

  static const std::int32_t ID = 624883173;

  using ReturnType = object_ptr<ok>;

  static object_ptr<banGroupCallParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class blockMessageSenderFromReplies final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool delete_message_;
  static jfieldID delete_message_fieldID;
  bool delete_all_messages_;
  static jfieldID delete_all_messages_fieldID;
  bool report_spam_;
  static jfieldID report_spam_fieldID;

  blockMessageSenderFromReplies();

  blockMessageSenderFromReplies(int53 message_id_, bool delete_message_, bool delete_all_messages_, bool report_spam_);

  static const std::int32_t ID = -1214384757;

  using ReturnType = object_ptr<ok>;

  static object_ptr<blockMessageSenderFromReplies> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSlots;

class boostChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int32> slot_ids_;
  static jfieldID slot_ids_fieldID;

  boostChat();

  boostChat(int53 chat_id_, array<int32> &&slot_ids_);

  static const std::int32_t ID = 1945750252;

  using ReturnType = object_ptr<chatBoostSlots>;

  static object_ptr<boostChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class canBotSendMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  canBotSendMessages();

  explicit canBotSendMessages(int53 bot_user_id_);

  static const std::int32_t ID = 544052364;

  using ReturnType = object_ptr<ok>;

  static object_ptr<canBotSendMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CanPostStoryResult;

class canPostStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  canPostStory();

  explicit canPostStory(int53 chat_id_);

  static const std::int32_t ID = 668621518;

  using ReturnType = object_ptr<CanPostStoryResult>;

  static object_ptr<canPostStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StorePaymentPurpose;

class ok;

class canPurchaseFromStore final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StorePaymentPurpose> purpose_;
  static jfieldID purpose_fieldID;

  canPurchaseFromStore();

  explicit canPurchaseFromStore(object_ptr<StorePaymentPurpose> &&purpose_);

  static const std::int32_t ID = 1017811816;

  using ReturnType = object_ptr<ok>;

  static object_ptr<canPurchaseFromStore> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CanSendMessageToUserResult;

class canSendMessageToUser final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool only_local_;
  static jfieldID only_local_fieldID;

  canSendMessageToUser();

  canSendMessageToUser(int53 user_id_, bool only_local_);

  static const std::int32_t ID = 1529489462;

  using ReturnType = object_ptr<CanSendMessageToUserResult>;

  static object_ptr<canSendMessageToUser> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CanTransferOwnershipResult;

class canTransferOwnership final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  canTransferOwnership();

  static const std::int32_t ID = 634602508;

  using ReturnType = object_ptr<CanTransferOwnershipResult>;

  static object_ptr<canTransferOwnership> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class cancelDownloadFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  bool only_if_pending_;
  static jfieldID only_if_pending_fieldID;

  cancelDownloadFile();

  cancelDownloadFile(int32 file_id_, bool only_if_pending_);

  static const std::int32_t ID = -1954524450;

  using ReturnType = object_ptr<ok>;

  static object_ptr<cancelDownloadFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class cancelPasswordReset final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  cancelPasswordReset();

  static const std::int32_t ID = 940733538;

  using ReturnType = object_ptr<ok>;

  static object_ptr<cancelPasswordReset> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class cancelPreliminaryUploadFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;

  cancelPreliminaryUploadFile();

  explicit cancelPreliminaryUploadFile(int32 file_id_);

  static const std::int32_t ID = 823412414;

  using ReturnType = object_ptr<ok>;

  static object_ptr<cancelPreliminaryUploadFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class cancelRecoveryEmailAddressVerification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  cancelRecoveryEmailAddressVerification();

  static const std::int32_t ID = -1516728691;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<cancelRecoveryEmailAddressVerification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class contact;

class importedContacts;

class changeImportedContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<contact>> contacts_;
  static jfieldID contacts_fieldID;

  changeImportedContacts();

  explicit changeImportedContacts(array<object_ptr<contact>> &&contacts_);

  static const std::int32_t ID = 1968207955;

  using ReturnType = object_ptr<importedContacts>;

  static object_ptr<changeImportedContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class changeStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 set_id_;
  static jfieldID set_id_fieldID;
  bool is_installed_;
  static jfieldID is_installed_fieldID;
  bool is_archived_;
  static jfieldID is_archived_fieldID;

  changeStickerSet();

  changeStickerSet(int64 set_id_, bool is_installed_, bool is_archived_);

  static const std::int32_t ID = 449357293;

  using ReturnType = object_ptr<ok>;

  static object_ptr<changeStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkAuthenticationBotToken final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  checkAuthenticationBotToken();

  explicit checkAuthenticationBotToken(string const &token_);

  static const std::int32_t ID = 639321206;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationBotToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkAuthenticationCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  checkAuthenticationCode();

  explicit checkAuthenticationCode(string const &code_);

  static const std::int32_t ID = -302103382;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class EmailAddressAuthentication;

class ok;

class checkAuthenticationEmailCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<EmailAddressAuthentication> code_;
  static jfieldID code_fieldID;

  checkAuthenticationEmailCode();

  explicit checkAuthenticationEmailCode(object_ptr<EmailAddressAuthentication> &&code_);

  static const std::int32_t ID = -582827361;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationEmailCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkAuthenticationPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;

  checkAuthenticationPassword();

  explicit checkAuthenticationPassword(string const &password_);

  static const std::int32_t ID = -2025698400;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkAuthenticationPasswordRecoveryCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string recovery_code_;
  static jfieldID recovery_code_fieldID;

  checkAuthenticationPasswordRecoveryCode();

  explicit checkAuthenticationPasswordRecoveryCode(string const &recovery_code_);

  static const std::int32_t ID = -603309083;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationPasswordRecoveryCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkAuthenticationPremiumPurchase final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;

  checkAuthenticationPremiumPurchase();

  checkAuthenticationPremiumPurchase(string const &currency_, int53 amount_);

  static const std::int32_t ID = 1588959934;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkAuthenticationPremiumPurchase> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLinkInfo;

class checkChatFolderInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  checkChatFolderInviteLink();

  explicit checkChatFolderInviteLink(string const &invite_link_);

  static const std::int32_t ID = 522557851;

  using ReturnType = object_ptr<chatFolderInviteLinkInfo>;

  static object_ptr<checkChatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkInfo;

class checkChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  checkChatInviteLink();

  explicit checkChatInviteLink(string const &invite_link_);

  static const std::int32_t ID = -496940997;

  using ReturnType = object_ptr<chatInviteLinkInfo>;

  static object_ptr<checkChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CheckChatUsernameResult;

class checkChatUsername final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string username_;
  static jfieldID username_fieldID;

  checkChatUsername();

  checkChatUsername(int53 chat_id_, string const &username_);

  static const std::int32_t ID = -119119344;

  using ReturnType = object_ptr<CheckChatUsernameResult>;

  static object_ptr<checkChatUsername> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PublicChatType;

class ok;

class checkCreatedPublicChatsLimit final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PublicChatType> type_;
  static jfieldID type_fieldID;

  checkCreatedPublicChatsLimit();

  explicit checkCreatedPublicChatsLimit(object_ptr<PublicChatType> &&type_);

  static const std::int32_t ID = -445546591;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkCreatedPublicChatsLimit> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkEmailAddressVerificationCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  checkEmailAddressVerificationCode();

  explicit checkEmailAddressVerificationCode(string const &code_);

  static const std::int32_t ID = -426386685;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkEmailAddressVerificationCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class EmailAddressAuthentication;

class ok;

class checkLoginEmailAddressCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<EmailAddressAuthentication> code_;
  static jfieldID code_fieldID;

  checkLoginEmailAddressCode();

  explicit checkLoginEmailAddressCode(object_ptr<EmailAddressAuthentication> &&code_);

  static const std::int32_t ID = -1454244766;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkLoginEmailAddressCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkPasswordRecoveryCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string recovery_code_;
  static jfieldID recovery_code_fieldID;

  checkPasswordRecoveryCode();

  explicit checkPasswordRecoveryCode(string const &recovery_code_);

  static const std::int32_t ID = -200794600;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkPasswordRecoveryCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkPhoneNumberCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  checkPhoneNumberCode();

  explicit checkPhoneNumberCode(string const &code_);

  static const std::int32_t ID = -603626079;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkPhoneNumberCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiftCodeInfo;

class checkPremiumGiftCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  checkPremiumGiftCode();

  explicit checkPremiumGiftCode(string const &code_);

  static const std::int32_t ID = -1786063260;

  using ReturnType = object_ptr<premiumGiftCodeInfo>;

  static object_ptr<checkPremiumGiftCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkQuickReplyShortcutName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  checkQuickReplyShortcutName();

  explicit checkQuickReplyShortcutName(string const &name_);

  static const std::int32_t ID = 2101203241;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkQuickReplyShortcutName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class checkRecoveryEmailAddressCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string code_;
  static jfieldID code_fieldID;

  checkRecoveryEmailAddressCode();

  explicit checkRecoveryEmailAddressCode(string const &code_);

  static const std::int32_t ID = -1997039589;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<checkRecoveryEmailAddressCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CheckStickerSetNameResult;

class checkStickerSetName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  checkStickerSetName();

  explicit checkStickerSetName(string const &name_);

  static const std::int32_t ID = -1789392642;

  using ReturnType = object_ptr<CheckStickerSetNameResult>;

  static object_ptr<checkStickerSetName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class checkWebAppFileDownload final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string file_name_;
  static jfieldID file_name_fieldID;
  string url_;
  static jfieldID url_fieldID;

  checkWebAppFileDownload();

  checkWebAppFileDownload(int53 bot_user_id_, string const &file_name_, string const &url_);

  static const std::int32_t ID = -389397278;

  using ReturnType = object_ptr<ok>;

  static object_ptr<checkWebAppFileDownload> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class cleanFileName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string file_name_;
  static jfieldID file_name_fieldID;

  cleanFileName();

  explicit cleanFileName(string const &file_name_);

  static const std::int32_t ID = 967964667;

  using ReturnType = object_ptr<text>;

  static object_ptr<cleanFileName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearAllDraftMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool exclude_secret_chats_;
  static jfieldID exclude_secret_chats_fieldID;

  clearAllDraftMessages();

  explicit clearAllDraftMessages(bool exclude_secret_chats_);

  static const std::int32_t ID = -46369573;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearAllDraftMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearAutosaveSettingsExceptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clearAutosaveSettingsExceptions();

  static const std::int32_t ID = 1475109874;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearAutosaveSettingsExceptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearImportedContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clearImportedContacts();

  static const std::int32_t ID = 869503298;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearImportedContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearRecentEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clearRecentEmojiStatuses();

  static const std::int32_t ID = -428749986;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearRecentEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearRecentReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clearRecentReactions();

  static const std::int32_t ID = 1298253650;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearRecentReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearRecentStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_attached_;
  static jfieldID is_attached_fieldID;

  clearRecentStickers();

  explicit clearRecentStickers(bool is_attached_);

  static const std::int32_t ID = -321242684;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearRecentStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearRecentlyFoundChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clearRecentlyFoundChats();

  static const std::int32_t ID = -285582542;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearRecentlyFoundChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clearSearchedForTags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool clear_cashtags_;
  static jfieldID clear_cashtags_fieldID;

  clearSearchedForTags();

  explicit clearSearchedForTags(bool clear_cashtags_);

  static const std::int32_t ID = 512017238;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clearSearchedForTags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class clickAnimatedEmojiMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  clickAnimatedEmojiMessage();

  clickAnimatedEmojiMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 196179554;

  using ReturnType = object_ptr<sticker>;

  static object_ptr<clickAnimatedEmojiMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clickChatSponsoredMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_media_click_;
  static jfieldID is_media_click_fieldID;
  bool from_fullscreen_;
  static jfieldID from_fullscreen_fieldID;

  clickChatSponsoredMessage();

  clickChatSponsoredMessage(int53 chat_id_, int53 message_id_, bool is_media_click_, bool from_fullscreen_);

  static const std::int32_t ID = 971995671;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clickChatSponsoredMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class clickPremiumSubscriptionButton final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  clickPremiumSubscriptionButton();

  static const std::int32_t ID = -369319162;

  using ReturnType = object_ptr<ok>;

  static object_ptr<clickPremiumSubscriptionButton> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class close final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  close();

  static const std::int32_t ID = -1187782273;

  using ReturnType = object_ptr<ok>;

  static object_ptr<close> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class closeChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  closeChat();

  explicit closeChat(int53 chat_id_);

  static const std::int32_t ID = 39749353;

  using ReturnType = object_ptr<ok>;

  static object_ptr<closeChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class closeSecretChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 secret_chat_id_;
  static jfieldID secret_chat_id_fieldID;

  closeSecretChat();

  explicit closeSecretChat(int32 secret_chat_id_);

  static const std::int32_t ID = -471006133;

  using ReturnType = object_ptr<ok>;

  static object_ptr<closeSecretChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class closeStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  closeStory();

  closeStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = 1553967851;

  using ReturnType = object_ptr<ok>;

  static object_ptr<closeStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class closeWebApp final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 web_app_launch_id_;
  static jfieldID web_app_launch_id_fieldID;

  closeWebApp();

  explicit closeWebApp(int64 web_app_launch_id_);

  static const std::int32_t ID = 1755391174;

  using ReturnType = object_ptr<ok>;

  static object_ptr<closeWebApp> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class commitPendingPaidMessageReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  commitPendingPaidMessageReactions();

  commitPendingPaidMessageReactions(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -171354618;

  using ReturnType = object_ptr<ok>;

  static object_ptr<commitPendingPaidMessageReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class session;

class confirmQrCodeAuthentication final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  confirmQrCodeAuthentication();

  explicit confirmQrCodeAuthentication(string const &link_);

  static const std::int32_t ID = -376199379;

  using ReturnType = object_ptr<session>;

  static object_ptr<confirmQrCodeAuthentication> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class confirmSession final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 session_id_;
  static jfieldID session_id_fieldID;

  confirmSession();

  explicit confirmSession(int64 session_id_);

  static const std::int32_t ID = -674647009;

  using ReturnType = object_ptr<ok>;

  static object_ptr<confirmSession> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateType;

class connectedAffiliateProgram;

class connectAffiliateProgram final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AffiliateType> affiliate_;
  static jfieldID affiliate_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  connectAffiliateProgram();

  connectAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, int53 bot_user_id_);

  static const std::int32_t ID = 1661392684;

  using ReturnType = object_ptr<connectedAffiliateProgram>;

  static object_ptr<connectAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class createBasicGroupChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;
  bool force_;
  static jfieldID force_fieldID;

  createBasicGroupChat();

  createBasicGroupChat(int53 basic_group_id_, bool force_);

  static const std::int32_t ID = 1972024548;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createBasicGroupChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessChatLink;

class inputBusinessChatLink;

class createBusinessChatLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputBusinessChatLink> link_info_;
  static jfieldID link_info_fieldID;

  createBusinessChatLink();

  explicit createBusinessChatLink(object_ptr<inputBusinessChatLink> &&link_info_);

  static const std::int32_t ID = -1861018304;

  using ReturnType = object_ptr<businessChatLink>;

  static object_ptr<createBusinessChatLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class callId;

class callProtocol;

class createCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<callProtocol> protocol_;
  static jfieldID protocol_fieldID;
  bool is_video_;
  static jfieldID is_video_fieldID;

  createCall();

  createCall(int53 user_id_, object_ptr<callProtocol> &&protocol_, bool is_video_);

  static const std::int32_t ID = -1104663024;

  using ReturnType = object_ptr<callId>;

  static object_ptr<createCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class chatFolderInfo;

class createChatFolder final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolder> folder_;
  static jfieldID folder_fieldID;

  createChatFolder();

  explicit createChatFolder(object_ptr<chatFolder> &&folder_);

  static const std::int32_t ID = 1015399680;

  using ReturnType = object_ptr<chatFolderInfo>;

  static object_ptr<createChatFolder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLink;

class createChatFolderInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  createChatFolderInviteLink();

  createChatFolderInviteLink(int32 chat_folder_id_, string const &name_, array<int53> &&chat_ids_);

  static const std::int32_t ID = -2037911099;

  using ReturnType = object_ptr<chatFolderInviteLink>;

  static object_ptr<createChatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class createChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;
  int32 member_limit_;
  static jfieldID member_limit_fieldID;
  bool creates_join_request_;
  static jfieldID creates_join_request_fieldID;

  createChatInviteLink();

  createChatInviteLink(int53 chat_id_, string const &name_, int32 expiration_date_, int32 member_limit_, bool creates_join_request_);

  static const std::int32_t ID = 287744833;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<createChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class starSubscriptionPricing;

class createChatSubscriptionInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<starSubscriptionPricing> subscription_pricing_;
  static jfieldID subscription_pricing_fieldID;

  createChatSubscriptionInviteLink();

  createChatSubscriptionInviteLink(int53 chat_id_, string const &name_, object_ptr<starSubscriptionPricing> &&subscription_pricing_);

  static const std::int32_t ID = 2255717;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<createChatSubscriptionInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class forumTopicIcon;

class forumTopicInfo;

class createForumTopic final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<forumTopicIcon> icon_;
  static jfieldID icon_fieldID;

  createForumTopic();

  createForumTopic(int53 chat_id_, string const &name_, object_ptr<forumTopicIcon> &&icon_);

  static const std::int32_t ID = -1040570140;

  using ReturnType = object_ptr<forumTopicInfo>;

  static object_ptr<createForumTopic> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class groupCallInfo;

class groupCallJoinParameters;

class createGroupCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<groupCallJoinParameters> join_parameters_;
  static jfieldID join_parameters_fieldID;

  createGroupCall();

  explicit createGroupCall(object_ptr<groupCallJoinParameters> &&join_parameters_);

  static const std::int32_t ID = 1930068672;

  using ReturnType = object_ptr<groupCallInfo>;

  static object_ptr<createGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class httpUrl;

class createInvoiceLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  object_ptr<InputMessageContent> invoice_;
  static jfieldID invoice_fieldID;

  createInvoiceLink();

  createInvoiceLink(string const &business_connection_id_, object_ptr<InputMessageContent> &&invoice_);

  static const std::int32_t ID = -814692249;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<createInvoiceLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class createdBasicGroupChat;

class createNewBasicGroupChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;
  string title_;
  static jfieldID title_fieldID;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;

  createNewBasicGroupChat();

  createNewBasicGroupChat(array<int53> &&user_ids_, string const &title_, int32 message_auto_delete_time_);

  static const std::int32_t ID = 1806454709;

  using ReturnType = object_ptr<createdBasicGroupChat>;

  static object_ptr<createNewBasicGroupChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class createNewSecretChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  createNewSecretChat();

  explicit createNewSecretChat(int53 user_id_);

  static const std::int32_t ID = -620682651;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createNewSecretChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class inputSticker;

class stickerSet;

class createNewStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  bool needs_repainting_;
  static jfieldID needs_repainting_fieldID;
  array<object_ptr<inputSticker>> stickers_;
  static jfieldID stickers_fieldID;
  string source_;
  static jfieldID source_fieldID;

  createNewStickerSet();

  createNewStickerSet(int53 user_id_, string const &title_, string const &name_, object_ptr<StickerType> &&sticker_type_, bool needs_repainting_, array<object_ptr<inputSticker>> &&stickers_, string const &source_);

  static const std::int32_t ID = -481065727;

  using ReturnType = object_ptr<stickerSet>;

  static object_ptr<createNewStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class chatLocation;

class createNewSupergroupChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;
  bool is_forum_;
  static jfieldID is_forum_fieldID;
  bool is_channel_;
  static jfieldID is_channel_fieldID;
  string description_;
  static jfieldID description_fieldID;
  object_ptr<chatLocation> location_;
  static jfieldID location_fieldID;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;
  bool for_import_;
  static jfieldID for_import_fieldID;

  createNewSupergroupChat();

  createNewSupergroupChat(string const &title_, bool is_forum_, bool is_channel_, string const &description_, object_ptr<chatLocation> &&location_, int32 message_auto_delete_time_, bool for_import_);

  static const std::int32_t ID = 804058822;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createNewSupergroupChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class createPrivateChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool force_;
  static jfieldID force_fieldID;

  createPrivateChat();

  createPrivateChat(int53 user_id_, bool force_);

  static const std::int32_t ID = -947758327;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createPrivateChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class createSecretChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 secret_chat_id_;
  static jfieldID secret_chat_id_fieldID;

  createSecretChat();

  explicit createSecretChat(int32 secret_chat_id_);

  static const std::int32_t ID = 1930285615;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createSecretChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class createSupergroupChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool force_;
  static jfieldID force_fieldID;

  createSupergroupChat();

  createSupergroupChat(int53 supergroup_id_, bool force_);

  static const std::int32_t ID = 1187475691;

  using ReturnType = object_ptr<chat>;

  static object_ptr<createSupergroupChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class temporaryPasswordState;

class createTemporaryPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;
  int32 valid_for_;
  static jfieldID valid_for_fieldID;

  createTemporaryPassword();

  createTemporaryPassword(string const &password_, int32 valid_for_);

  static const std::int32_t ID = -1626509434;

  using ReturnType = object_ptr<temporaryPasswordState>;

  static object_ptr<createTemporaryPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class groupCallId;

class createVideoChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  int32 start_date_;
  static jfieldID start_date_fieldID;
  bool is_rtmp_stream_;
  static jfieldID is_rtmp_stream_fieldID;

  createVideoChat();

  createVideoChat(int53 chat_id_, string const &title_, int32 start_date_, bool is_rtmp_stream_);

  static const std::int32_t ID = 2124715405;

  using ReturnType = object_ptr<groupCallId>;

  static object_ptr<createVideoChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class declineGroupCallInvitation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  declineGroupCallInvitation();

  declineGroupCallInvitation(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1843919377;

  using ReturnType = object_ptr<ok>;

  static object_ptr<declineGroupCallInvitation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class GroupCallDataChannel;

class MessageSender;

class data;

class decryptGroupCallData final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  object_ptr<GroupCallDataChannel> data_channel_;
  static jfieldID data_channel_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  decryptGroupCallData();

  decryptGroupCallData(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, object_ptr<GroupCallDataChannel> &&data_channel_, bytes const &data_);

  static const std::int32_t ID = 1781743076;

  using ReturnType = object_ptr<data>;

  static object_ptr<decryptGroupCallData> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteAccount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string reason_;
  static jfieldID reason_fieldID;
  string password_;
  static jfieldID password_fieldID;

  deleteAccount();

  deleteAccount(string const &reason_, string const &password_);

  static const std::int32_t ID = 1395816134;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteAccount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteAllCallMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool revoke_;
  static jfieldID revoke_fieldID;

  deleteAllCallMessages();

  explicit deleteAllCallMessages(bool revoke_);

  static const std::int32_t ID = -1466445325;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteAllCallMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteAllRevokedChatInviteLinks final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 creator_user_id_;
  static jfieldID creator_user_id_fieldID;

  deleteAllRevokedChatInviteLinks();

  deleteAllRevokedChatInviteLinks(int53 chat_id_, int53 creator_user_id_);

  static const std::int32_t ID = 1112020698;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteAllRevokedChatInviteLinks> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteBotMediaPreviews final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  array<int32> file_ids_;
  static jfieldID file_ids_fieldID;

  deleteBotMediaPreviews();

  deleteBotMediaPreviews(int53 bot_user_id_, string const &language_code_, array<int32> &&file_ids_);

  static const std::int32_t ID = -1397512722;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteBotMediaPreviews> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteBusinessChatLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  deleteBusinessChatLink();

  explicit deleteBusinessChatLink(string const &link_);

  static const std::int32_t ID = -1101895865;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteBusinessChatLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteBusinessConnectedBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  deleteBusinessConnectedBot();

  explicit deleteBusinessConnectedBot(int53 bot_user_id_);

  static const std::int32_t ID = -1633976747;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteBusinessConnectedBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteBusinessMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  deleteBusinessMessages();

  deleteBusinessMessages(string const &business_connection_id_, array<int53> &&message_ids_);

  static const std::int32_t ID = 1425721828;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteBusinessMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteBusinessStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  deleteBusinessStory();

  deleteBusinessStory(string const &business_connection_id_, int32 story_id_);

  static const std::int32_t ID = 1024585042;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteBusinessStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  deleteChat();

  explicit deleteChat(int53 chat_id_);

  static const std::int32_t ID = -171253666;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool restore_previous_;
  static jfieldID restore_previous_fieldID;

  deleteChatBackground();

  deleteChatBackground(int53 chat_id_, bool restore_previous_);

  static const std::int32_t ID = 320267896;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatFolder final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  array<int53> leave_chat_ids_;
  static jfieldID leave_chat_ids_fieldID;

  deleteChatFolder();

  deleteChatFolder(int32 chat_folder_id_, array<int53> &&leave_chat_ids_);

  static const std::int32_t ID = -1956364551;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatFolder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatFolderInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  deleteChatFolderInviteLink();

  deleteChatFolderInviteLink(int32 chat_folder_id_, string const &invite_link_);

  static const std::int32_t ID = -930057858;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatHistory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool remove_from_chat_list_;
  static jfieldID remove_from_chat_list_fieldID;
  bool revoke_;
  static jfieldID revoke_fieldID;

  deleteChatHistory();

  deleteChatHistory(int53 chat_id_, bool remove_from_chat_list_, bool revoke_);

  static const std::int32_t ID = -1472081761;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatHistory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatMessagesByDate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 min_date_;
  static jfieldID min_date_fieldID;
  int32 max_date_;
  static jfieldID max_date_fieldID;
  bool revoke_;
  static jfieldID revoke_fieldID;

  deleteChatMessagesByDate();

  deleteChatMessagesByDate(int53 chat_id_, int32 min_date_, int32 max_date_, bool revoke_);

  static const std::int32_t ID = -1639653185;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatMessagesByDate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class deleteChatMessagesBySender final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;

  deleteChatMessagesBySender();

  deleteChatMessagesBySender(int53 chat_id_, object_ptr<MessageSender> &&sender_id_);

  static const std::int32_t ID = -1164235161;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatMessagesBySender> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteChatReplyMarkup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  deleteChatReplyMarkup();

  deleteChatReplyMarkup(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 100637531;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteChatReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BotCommandScope;

class ok;

class deleteCommands final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BotCommandScope> scope_;
  static jfieldID scope_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  deleteCommands();

  deleteCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_);

  static const std::int32_t ID = 1002732586;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteCommands> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteDefaultBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool for_dark_theme_;
  static jfieldID for_dark_theme_fieldID;

  deleteDefaultBackground();

  explicit deleteDefaultBackground(bool for_dark_theme_);

  static const std::int32_t ID = -1297814210;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteDefaultBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;

  deleteFile();

  explicit deleteFile(int32 file_id_);

  static const std::int32_t ID = 1807653676;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteForumTopic final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  deleteForumTopic();

  deleteForumTopic(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = 1864916152;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteForumTopic> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteLanguagePack final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;

  deleteLanguagePack();

  explicit deleteLanguagePack(string const &language_pack_id_);

  static const std::int32_t ID = -2108761026;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteLanguagePack> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  bool revoke_;
  static jfieldID revoke_fieldID;

  deleteMessages();

  deleteMessages(int53 chat_id_, array<int53> &&message_ids_, bool revoke_);

  static const std::int32_t ID = 1130090173;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementType;

class ok;

class deletePassportElement final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;

  deletePassportElement();

  explicit deletePassportElement(object_ptr<PassportElementType> &&type_);

  static const std::int32_t ID = -1719555468;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deletePassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 profile_photo_id_;
  static jfieldID profile_photo_id_fieldID;

  deleteProfilePhoto();

  explicit deleteProfilePhoto(int64 profile_photo_id_);

  static const std::int32_t ID = 1319794625;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteQuickReplyShortcut final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;

  deleteQuickReplyShortcut();

  explicit deleteQuickReplyShortcut(int32 shortcut_id_);

  static const std::int32_t ID = -246911978;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteQuickReplyShortcut> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteQuickReplyShortcutMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  deleteQuickReplyShortcutMessages();

  deleteQuickReplyShortcutMessages(int32 shortcut_id_, array<int53> &&message_ids_);

  static const std::int32_t ID = -40522947;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteQuickReplyShortcutMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteRevokedChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  deleteRevokedChatInviteLink();

  deleteRevokedChatInviteLink(int53 chat_id_, string const &invite_link_);

  static const std::int32_t ID = -1859711873;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteRevokedChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteSavedCredentials final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  deleteSavedCredentials();

  static const std::int32_t ID = 826300114;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteSavedCredentials> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteSavedMessagesTopicHistory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  deleteSavedMessagesTopicHistory();

  explicit deleteSavedMessagesTopicHistory(int53 saved_messages_topic_id_);

  static const std::int32_t ID = 1776237930;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteSavedMessagesTopicHistory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteSavedMessagesTopicMessagesByDate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  int32 min_date_;
  static jfieldID min_date_fieldID;
  int32 max_date_;
  static jfieldID max_date_fieldID;

  deleteSavedMessagesTopicMessagesByDate();

  deleteSavedMessagesTopicMessagesByDate(int53 saved_messages_topic_id_, int32 min_date_, int32 max_date_);

  static const std::int32_t ID = 1444389;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteSavedMessagesTopicMessagesByDate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteSavedOrderInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  deleteSavedOrderInfo();

  static const std::int32_t ID = 1629058164;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteSavedOrderInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  deleteStickerSet();

  explicit deleteStickerSet(string const &name_);

  static const std::int32_t ID = 1577745325;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class deleteStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  deleteStory();

  deleteStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -2020144472;

  using ReturnType = object_ptr<ok>;

  static object_ptr<deleteStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class destroy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  destroy();

  static const std::int32_t ID = 685331274;

  using ReturnType = object_ptr<ok>;

  static object_ptr<destroy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class disableAllSupergroupUsernames final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  disableAllSupergroupUsernames();

  explicit disableAllSupergroupUsernames(int53 supergroup_id_);

  static const std::int32_t ID = 843511216;

  using ReturnType = object_ptr<ok>;

  static object_ptr<disableAllSupergroupUsernames> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class disableProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  disableProxy();

  static const std::int32_t ID = -2100095102;

  using ReturnType = object_ptr<ok>;

  static object_ptr<disableProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class discardCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  bool is_disconnected_;
  static jfieldID is_disconnected_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  int32 duration_;
  static jfieldID duration_fieldID;
  bool is_video_;
  static jfieldID is_video_fieldID;
  int64 connection_id_;
  static jfieldID connection_id_fieldID;

  discardCall();

  discardCall(int32 call_id_, bool is_disconnected_, string const &invite_link_, int32 duration_, bool is_video_, int64 connection_id_);

  static const std::int32_t ID = -1545983346;

  using ReturnType = object_ptr<ok>;

  static object_ptr<discardCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateType;

class connectedAffiliateProgram;

class disconnectAffiliateProgram final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AffiliateType> affiliate_;
  static jfieldID affiliate_fieldID;
  string url_;
  static jfieldID url_fieldID;

  disconnectAffiliateProgram();

  disconnectAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, string const &url_);

  static const std::int32_t ID = -105831172;

  using ReturnType = object_ptr<connectedAffiliateProgram>;

  static object_ptr<disconnectAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class disconnectAllWebsites final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  disconnectAllWebsites();

  static const std::int32_t ID = -1082985981;

  using ReturnType = object_ptr<ok>;

  static object_ptr<disconnectAllWebsites> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class disconnectWebsite final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 website_id_;
  static jfieldID website_id_fieldID;

  disconnectWebsite();

  explicit disconnectWebsite(int64 website_id_);

  static const std::int32_t ID = -778767395;

  using ReturnType = object_ptr<ok>;

  static object_ptr<disconnectWebsite> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class file;

class downloadFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  int32 priority_;
  static jfieldID priority_fieldID;
  int53 offset_;
  static jfieldID offset_fieldID;
  int53 limit_;
  static jfieldID limit_fieldID;
  bool synchronous_;
  static jfieldID synchronous_fieldID;

  downloadFile();

  downloadFile(int32 file_id_, int32 priority_, int53 offset_, int53 limit_, bool synchronous_);

  static const std::int32_t ID = 1059402292;

  using ReturnType = object_ptr<file>;

  static object_ptr<downloadFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryContent;

class botMediaPreview;

class editBotMediaPreview final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  object_ptr<InputStoryContent> content_;
  static jfieldID content_fieldID;

  editBotMediaPreview();

  editBotMediaPreview(int53 bot_user_id_, string const &language_code_, int32 file_id_, object_ptr<InputStoryContent> &&content_);

  static const std::int32_t ID = -2037031582;

  using ReturnType = object_ptr<botMediaPreview>;

  static object_ptr<editBotMediaPreview> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessChatLink;

class inputBusinessChatLink;

class editBusinessChatLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;
  object_ptr<inputBusinessChatLink> link_info_;
  static jfieldID link_info_fieldID;

  editBusinessChatLink();

  editBusinessChatLink(string const &link_, object_ptr<inputBusinessChatLink> &&link_info_);

  static const std::int32_t ID = 1594947110;

  using ReturnType = object_ptr<businessChatLink>;

  static object_ptr<editBusinessChatLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class businessMessage;

class formattedText;

class editBusinessMessageCaption final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;

  editBusinessMessageCaption();

  editBusinessMessageCaption(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_);

  static const std::int32_t ID = -1071562045;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<editBusinessMessageCaption> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class businessMessage;

class location;

class editBusinessMessageLiveLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  int32 heading_;
  static jfieldID heading_fieldID;
  int32 proximity_alert_radius_;
  static jfieldID proximity_alert_radius_fieldID;

  editBusinessMessageLiveLocation();

  editBusinessMessageLiveLocation(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_);

  static const std::int32_t ID = 494972447;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<editBusinessMessageLiveLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class businessMessage;

class editBusinessMessageMedia final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editBusinessMessageMedia();

  editBusinessMessageMedia(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -60733576;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<editBusinessMessageMedia> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class businessMessage;

class editBusinessMessageReplyMarkup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  editBusinessMessageReplyMarkup();

  editBusinessMessageReplyMarkup(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = 701787159;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<editBusinessMessageReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class businessMessage;

class editBusinessMessageText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editBusinessMessageText();

  editBusinessMessageText(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1149169252;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<editBusinessMessageText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryContent;

class StoryPrivacySettings;

class formattedText;

class inputStoryAreas;

class story;

class editBusinessStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  object_ptr<InputStoryContent> content_;
  static jfieldID content_fieldID;
  object_ptr<inputStoryAreas> areas_;
  static jfieldID areas_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  object_ptr<StoryPrivacySettings> privacy_settings_;
  static jfieldID privacy_settings_fieldID;

  editBusinessStory();

  editBusinessStory(int53 story_poster_chat_id_, int32 story_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_, object_ptr<StoryPrivacySettings> &&privacy_settings_);

  static const std::int32_t ID = 472538940;

  using ReturnType = object_ptr<story>;

  static object_ptr<editBusinessStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class chatFolderInfo;

class editChatFolder final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  object_ptr<chatFolder> folder_;
  static jfieldID folder_fieldID;

  editChatFolder();

  editChatFolder(int32 chat_folder_id_, object_ptr<chatFolder> &&folder_);

  static const std::int32_t ID = 53672754;

  using ReturnType = object_ptr<chatFolderInfo>;

  static object_ptr<editChatFolder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLink;

class editChatFolderInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string name_;
  static jfieldID name_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  editChatFolderInviteLink();

  editChatFolderInviteLink(int32 chat_folder_id_, string const &invite_link_, string const &name_, array<int53> &&chat_ids_);

  static const std::int32_t ID = -2141872095;

  using ReturnType = object_ptr<chatFolderInviteLink>;

  static object_ptr<editChatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class editChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string name_;
  static jfieldID name_fieldID;
  int32 expiration_date_;
  static jfieldID expiration_date_fieldID;
  int32 member_limit_;
  static jfieldID member_limit_fieldID;
  bool creates_join_request_;
  static jfieldID creates_join_request_fieldID;

  editChatInviteLink();

  editChatInviteLink(int53 chat_id_, string const &invite_link_, string const &name_, int32 expiration_date_, int32 member_limit_, bool creates_join_request_);

  static const std::int32_t ID = 1320303996;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<editChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class editChatSubscriptionInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string name_;
  static jfieldID name_fieldID;

  editChatSubscriptionInviteLink();

  editChatSubscriptionInviteLink(int53 chat_id_, string const &invite_link_, string const &name_);

  static const std::int32_t ID = -951826989;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<editChatSubscriptionInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackInfo;

class ok;

class editCustomLanguagePackInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<languagePackInfo> info_;
  static jfieldID info_fieldID;

  editCustomLanguagePackInfo();

  explicit editCustomLanguagePackInfo(object_ptr<languagePackInfo> &&info_);

  static const std::int32_t ID = 1320751257;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editCustomLanguagePackInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class editForumTopic final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  bool edit_icon_custom_emoji_;
  static jfieldID edit_icon_custom_emoji_fieldID;
  int64 icon_custom_emoji_id_;
  static jfieldID icon_custom_emoji_id_fieldID;

  editForumTopic();

  editForumTopic(int53 chat_id_, int53 message_thread_id_, string const &name_, bool edit_icon_custom_emoji_, int64 icon_custom_emoji_id_);

  static const std::int32_t ID = -1485402016;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editForumTopic> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class formattedText;

class ok;

class editInlineMessageCaption final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;

  editInlineMessageCaption();

  editInlineMessageCaption(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_);

  static const std::int32_t ID = 1409762552;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editInlineMessageCaption> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class location;

class ok;

class editInlineMessageLiveLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  int32 heading_;
  static jfieldID heading_fieldID;
  int32 proximity_alert_radius_;
  static jfieldID proximity_alert_radius_fieldID;

  editInlineMessageLiveLocation();

  editInlineMessageLiveLocation(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_);

  static const std::int32_t ID = 2134352044;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editInlineMessageLiveLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class ok;

class editInlineMessageMedia final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editInlineMessageMedia();

  editInlineMessageMedia(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 23553921;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editInlineMessageMedia> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class ok;

class editInlineMessageReplyMarkup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  editInlineMessageReplyMarkup();

  editInlineMessageReplyMarkup(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = -67565858;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editInlineMessageReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class ok;

class editInlineMessageText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editInlineMessageText();

  editInlineMessageText(string const &inline_message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -855457307;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editInlineMessageText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class formattedText;

class message;

class editMessageCaption final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  bool show_caption_above_media_;
  static jfieldID show_caption_above_media_fieldID;

  editMessageCaption();

  editMessageCaption(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<formattedText> &&caption_, bool show_caption_above_media_);

  static const std::int32_t ID = -2020117951;

  using ReturnType = object_ptr<message>;

  static object_ptr<editMessageCaption> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class location;

class message;

class editMessageLiveLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 live_period_;
  static jfieldID live_period_fieldID;
  int32 heading_;
  static jfieldID heading_fieldID;
  int32 proximity_alert_radius_;
  static jfieldID proximity_alert_radius_fieldID;

  editMessageLiveLocation();

  editMessageLiveLocation(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<location> &&location_, int32 live_period_, int32 heading_, int32 proximity_alert_radius_);

  static const std::int32_t ID = -1890511980;

  using ReturnType = object_ptr<message>;

  static object_ptr<editMessageLiveLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class message;

class editMessageMedia final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editMessageMedia();

  editMessageMedia(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -1152678125;

  using ReturnType = object_ptr<message>;

  static object_ptr<editMessageMedia> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class message;

class editMessageReplyMarkup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  editMessageReplyMarkup();

  editMessageReplyMarkup(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = 332127881;

  using ReturnType = object_ptr<message>;

  static object_ptr<editMessageReplyMarkup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSchedulingState;

class ok;

class editMessageSchedulingState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageSchedulingState> scheduling_state_;
  static jfieldID scheduling_state_fieldID;

  editMessageSchedulingState();

  editMessageSchedulingState(int53 chat_id_, int53 message_id_, object_ptr<MessageSchedulingState> &&scheduling_state_);

  static const std::int32_t ID = -1372976192;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editMessageSchedulingState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ReplyMarkup;

class message;

class editMessageText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editMessageText();

  editMessageText(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 196272567;

  using ReturnType = object_ptr<message>;

  static object_ptr<editMessageText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType;

class proxy;

class editProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 proxy_id_;
  static jfieldID proxy_id_fieldID;
  string server_;
  static jfieldID server_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  bool enable_;
  static jfieldID enable_fieldID;
  object_ptr<ProxyType> type_;
  static jfieldID type_fieldID;

  editProxy();

  editProxy(int32 proxy_id_, string const &server_, int32 port_, bool enable_, object_ptr<ProxyType> &&type_);

  static const std::int32_t ID = -1605883821;

  using ReturnType = object_ptr<proxy>;

  static object_ptr<editProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class ok;

class editQuickReplyMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  editQuickReplyMessage();

  editQuickReplyMessage(int32 shortcut_id_, int53 message_id_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 80517006;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editQuickReplyMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class editStarSubscription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string subscription_id_;
  static jfieldID subscription_id_fieldID;
  bool is_canceled_;
  static jfieldID is_canceled_fieldID;

  editStarSubscription();

  editStarSubscription(string const &subscription_id_, bool is_canceled_);

  static const std::int32_t ID = 2048538904;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editStarSubscription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryContent;

class formattedText;

class inputStoryAreas;

class ok;

class editStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  object_ptr<InputStoryContent> content_;
  static jfieldID content_fieldID;
  object_ptr<inputStoryAreas> areas_;
  static jfieldID areas_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;

  editStory();

  editStory(int53 story_poster_chat_id_, int32 story_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_);

  static const std::int32_t ID = 355296788;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class editStoryCover final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  double cover_frame_timestamp_;
  static jfieldID cover_frame_timestamp_fieldID;

  editStoryCover();

  editStoryCover(int53 story_poster_chat_id_, int32 story_id_, double cover_frame_timestamp_);

  static const std::int32_t ID = -1035823266;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editStoryCover> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class editUserStarSubscription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string telegram_payment_charge_id_;
  static jfieldID telegram_payment_charge_id_fieldID;
  bool is_canceled_;
  static jfieldID is_canceled_fieldID;

  editUserStarSubscription();

  editUserStarSubscription(int53 user_id_, string const &telegram_payment_charge_id_, bool is_canceled_);

  static const std::int32_t ID = 1370582665;

  using ReturnType = object_ptr<ok>;

  static object_ptr<editUserStarSubscription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class enableProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 proxy_id_;
  static jfieldID proxy_id_fieldID;

  enableProxy();

  explicit enableProxy(int32 proxy_id_);

  static const std::int32_t ID = 1494450838;

  using ReturnType = object_ptr<ok>;

  static object_ptr<enableProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class GroupCallDataChannel;

class data;

class encryptGroupCallData final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<GroupCallDataChannel> data_channel_;
  static jfieldID data_channel_fieldID;
  bytes data_;
  static jfieldID data_fieldID;
  int32 unencrypted_prefix_size_;
  static jfieldID unencrypted_prefix_size_fieldID;

  encryptGroupCallData();

  encryptGroupCallData(int32 group_call_id_, object_ptr<GroupCallDataChannel> &&data_channel_, bytes const &data_, int32 unencrypted_prefix_size_);

  static const std::int32_t ID = -377997690;

  using ReturnType = object_ptr<data>;

  static object_ptr<encryptGroupCallData> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class endGroupCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  endGroupCall();

  explicit endGroupCall(int32 group_call_id_);

  static const std::int32_t ID = 573131959;

  using ReturnType = object_ptr<ok>;

  static object_ptr<endGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class endGroupCallRecording final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  endGroupCallRecording();

  explicit endGroupCallRecording(int32 group_call_id_);

  static const std::int32_t ID = -75799927;

  using ReturnType = object_ptr<ok>;

  static object_ptr<endGroupCallRecording> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class endGroupCallScreenSharing final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  endGroupCallScreenSharing();

  explicit endGroupCallScreenSharing(int32 group_call_id_);

  static const std::int32_t ID = -2047599540;

  using ReturnType = object_ptr<ok>;

  static object_ptr<endGroupCallScreenSharing> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class error;

class ok;

class finishFileGeneration final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 generation_id_;
  static jfieldID generation_id_fieldID;
  object_ptr<error> error_;
  static jfieldID error_fieldID;

  finishFileGeneration();

  finishFileGeneration(int64 generation_id_, object_ptr<error> &&error_);

  static const std::int32_t ID = -1055060835;

  using ReturnType = object_ptr<ok>;

  static object_ptr<finishFileGeneration> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSendOptions;

class messages;

class forwardMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  int53 from_chat_id_;
  static jfieldID from_chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  object_ptr<messageSendOptions> options_;
  static jfieldID options_fieldID;
  bool send_copy_;
  static jfieldID send_copy_fieldID;
  bool remove_caption_;
  static jfieldID remove_caption_fieldID;

  forwardMessages();

  forwardMessages(int53 chat_id_, int53 message_thread_id_, int53 from_chat_id_, array<int53> &&message_ids_, object_ptr<messageSendOptions> &&options_, bool send_copy_, bool remove_caption_);

  static const std::int32_t ID = 966156347;

  using ReturnType = object_ptr<messages>;

  static object_ptr<forwardMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class accountTtl;

class getAccountTtl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAccountTtl();

  static const std::int32_t ID = -443905161;

  using ReturnType = object_ptr<accountTtl>;

  static object_ptr<getAccountTtl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sessions;

class getActiveSessions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getActiveSessions();

  static const std::int32_t ID = 1119710526;

  using ReturnType = object_ptr<sessions>;

  static object_ptr<getActiveSessions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportElements;

class getAllPassportElements final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;

  getAllPassportElements();

  explicit getAllPassportElements(string const &password_);

  static const std::int32_t ID = -2038945045;

  using ReturnType = object_ptr<passportElements>;

  static object_ptr<getAllPassportElements> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class emojis;

class getAllStickerEmojis final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool return_only_main_emoji_;
  static jfieldID return_only_main_emoji_fieldID;

  getAllStickerEmojis();

  getAllStickerEmojis(object_ptr<StickerType> &&sticker_type_, string const &query_, int53 chat_id_, bool return_only_main_emoji_);

  static const std::int32_t ID = 296562224;

  using ReturnType = object_ptr<emojis>;

  static object_ptr<getAllStickerEmojis> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class animatedEmoji;

class getAnimatedEmoji final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;

  getAnimatedEmoji();

  explicit getAnimatedEmoji(string const &emoji_);

  static const std::int32_t ID = 1065635702;

  using ReturnType = object_ptr<animatedEmoji>;

  static object_ptr<getAnimatedEmoji> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class getApplicationConfig final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getApplicationConfig();

  static const std::int32_t ID = -1823144318;

  using ReturnType = object_ptr<JsonValue>;

  static object_ptr<getApplicationConfig> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getApplicationDownloadLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getApplicationDownloadLink();

  static const std::int32_t ID = 112013252;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getApplicationDownloadLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class archiveChatListSettings;

class getArchiveChatListSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getArchiveChatListSettings();

  static const std::int32_t ID = -2087874976;

  using ReturnType = object_ptr<archiveChatListSettings>;

  static object_ptr<getArchiveChatListSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickerSets;

class getArchivedStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  int64 offset_sticker_set_id_;
  static jfieldID offset_sticker_set_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getArchivedStickerSets();

  getArchivedStickerSets(object_ptr<StickerType> &&sticker_type_, int64 offset_sticker_set_id_, int32 limit_);

  static const std::int32_t ID = 1001931341;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<getArchivedStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerSets;

class getAttachedStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;

  getAttachedStickerSets();

  explicit getAttachedStickerSets(int32 file_id_);

  static const std::int32_t ID = 1302172429;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<getAttachedStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class attachmentMenuBot;

class getAttachmentMenuBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  getAttachmentMenuBot();

  explicit getAttachmentMenuBot(int53 bot_user_id_);

  static const std::int32_t ID = 1034248699;

  using ReturnType = object_ptr<attachmentMenuBot>;

  static object_ptr<getAttachmentMenuBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AuthorizationState;

class getAuthorizationState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAuthorizationState();

  static const std::int32_t ID = 1949154877;

  using ReturnType = object_ptr<AuthorizationState>;

  static object_ptr<getAuthorizationState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class autoDownloadSettingsPresets;

class getAutoDownloadSettingsPresets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAutoDownloadSettingsPresets();

  static const std::int32_t ID = -1721088201;

  using ReturnType = object_ptr<autoDownloadSettingsPresets>;

  static object_ptr<getAutoDownloadSettingsPresets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class autosaveSettings;

class getAutosaveSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAutosaveSettings();

  static const std::int32_t ID = 2136207914;

  using ReturnType = object_ptr<autosaveSettings>;

  static object_ptr<getAutosaveSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostSlots;

class getAvailableChatBoostSlots final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAvailableChatBoostSlots();

  static const std::int32_t ID = 1929898965;

  using ReturnType = object_ptr<chatBoostSlots>;

  static object_ptr<getAvailableChatBoostSlots> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class availableGifts;

class getAvailableGifts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getAvailableGifts();

  static const std::int32_t ID = -4559695;

  using ReturnType = object_ptr<availableGifts>;

  static object_ptr<getAvailableGifts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundType;

class httpUrl;

class getBackgroundUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<BackgroundType> type_;
  static jfieldID type_fieldID;

  getBackgroundUrl();

  getBackgroundUrl(string const &name_, object_ptr<BackgroundType> &&type_);

  static const std::int32_t ID = 733769682;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getBackgroundUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class bankCardInfo;

class getBankCardInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bank_card_number_;
  static jfieldID bank_card_number_fieldID;

  getBankCardInfo();

  explicit getBankCardInfo(string const &bank_card_number_);

  static const std::int32_t ID = -1310515792;

  using ReturnType = object_ptr<bankCardInfo>;

  static object_ptr<getBankCardInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class basicGroup;

class getBasicGroup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;

  getBasicGroup();

  explicit getBasicGroup(int53 basic_group_id_);

  static const std::int32_t ID = -1635174828;

  using ReturnType = object_ptr<basicGroup>;

  static object_ptr<getBasicGroup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class basicGroupFullInfo;

class getBasicGroupFullInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 basic_group_id_;
  static jfieldID basic_group_id_fieldID;

  getBasicGroupFullInfo();

  explicit getBasicGroupFullInfo(int53 basic_group_id_);

  static const std::int32_t ID = -1822039253;

  using ReturnType = object_ptr<basicGroupFullInfo>;

  static object_ptr<getBasicGroupFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BlockList;

class messageSenders;

class getBlockedMessageSenders final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getBlockedMessageSenders();

  getBlockedMessageSenders(object_ptr<BlockList> &&block_list_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -1931137258;

  using ReturnType = object_ptr<messageSenders>;

  static object_ptr<getBlockedMessageSenders> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getBotInfoDescription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  getBotInfoDescription();

  getBotInfoDescription(int53 bot_user_id_, string const &language_code_);

  static const std::int32_t ID = -762841035;

  using ReturnType = object_ptr<text>;

  static object_ptr<getBotInfoDescription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getBotInfoShortDescription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  getBotInfoShortDescription();

  getBotInfoShortDescription(int53 bot_user_id_, string const &language_code_);

  static const std::int32_t ID = 1243358740;

  using ReturnType = object_ptr<text>;

  static object_ptr<getBotInfoShortDescription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botMediaPreviewInfo;

class getBotMediaPreviewInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  getBotMediaPreviewInfo();

  getBotMediaPreviewInfo(int53 bot_user_id_, string const &language_code_);

  static const std::int32_t ID = 1358299446;

  using ReturnType = object_ptr<botMediaPreviewInfo>;

  static object_ptr<getBotMediaPreviewInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botMediaPreviews;

class getBotMediaPreviews final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  getBotMediaPreviews();

  explicit getBotMediaPreviews(int53 bot_user_id_);

  static const std::int32_t ID = 577131608;

  using ReturnType = object_ptr<botMediaPreviews>;

  static object_ptr<getBotMediaPreviews> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getBotName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  getBotName();

  getBotName(int53 bot_user_id_, string const &language_code_);

  static const std::int32_t ID = -1707118036;

  using ReturnType = object_ptr<text>;

  static object_ptr<getBotName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class count;

class getBotSimilarBotCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  bool return_local_;
  static jfieldID return_local_fieldID;

  getBotSimilarBotCount();

  getBotSimilarBotCount(int53 bot_user_id_, bool return_local_);

  static const std::int32_t ID = -1271545369;

  using ReturnType = object_ptr<count>;

  static object_ptr<getBotSimilarBotCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class getBotSimilarBots final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  getBotSimilarBots();

  explicit getBotSimilarBots(int53 bot_user_id_);

  static const std::int32_t ID = -825139275;

  using ReturnType = object_ptr<users>;

  static object_ptr<getBotSimilarBots> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starAmount;

class getBusinessAccountStarAmount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;

  getBusinessAccountStarAmount();

  explicit getBusinessAccountStarAmount(string const &business_connection_id_);

  static const std::int32_t ID = -1817136693;

  using ReturnType = object_ptr<starAmount>;

  static object_ptr<getBusinessAccountStarAmount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessChatLinkInfo;

class getBusinessChatLinkInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_name_;
  static jfieldID link_name_fieldID;

  getBusinessChatLinkInfo();

  explicit getBusinessChatLinkInfo(string const &link_name_);

  static const std::int32_t ID = 797670986;

  using ReturnType = object_ptr<businessChatLinkInfo>;

  static object_ptr<getBusinessChatLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessChatLinks;

class getBusinessChatLinks final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getBusinessChatLinks();

  static const std::int32_t ID = 710287703;

  using ReturnType = object_ptr<businessChatLinks>;

  static object_ptr<getBusinessChatLinks> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessConnectedBot;

class getBusinessConnectedBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getBusinessConnectedBot();

  static const std::int32_t ID = 911058883;

  using ReturnType = object_ptr<businessConnectedBot>;

  static object_ptr<getBusinessConnectedBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessConnection;

class getBusinessConnection final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string connection_id_;
  static jfieldID connection_id_fieldID;

  getBusinessConnection();

  explicit getBusinessConnection(string const &connection_id_);

  static const std::int32_t ID = -2114706400;

  using ReturnType = object_ptr<businessConnection>;

  static object_ptr<getBusinessConnection> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BusinessFeature;

class businessFeatures;

class getBusinessFeatures final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BusinessFeature> source_;
  static jfieldID source_fieldID;

  getBusinessFeatures();

  explicit getBusinessFeatures(object_ptr<BusinessFeature> &&source_);

  static const std::int32_t ID = -997171199;

  using ReturnType = object_ptr<businessFeatures>;

  static object_ptr<getBusinessFeatures> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CallbackQueryPayload;

class callbackQueryAnswer;

class getCallbackQueryAnswer final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<CallbackQueryPayload> payload_;
  static jfieldID payload_fieldID;

  getCallbackQueryAnswer();

  getCallbackQueryAnswer(int53 chat_id_, int53 message_id_, object_ptr<CallbackQueryPayload> &&payload_);

  static const std::int32_t ID = 116357727;

  using ReturnType = object_ptr<callbackQueryAnswer>;

  static object_ptr<getCallbackQueryAnswer> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getCallbackQueryMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int64 callback_query_id_;
  static jfieldID callback_query_id_fieldID;

  getCallbackQueryMessage();

  getCallbackQueryMessage(int53 chat_id_, int53 message_id_, int64 callback_query_id_);

  static const std::int32_t ID = -1121939086;

  using ReturnType = object_ptr<message>;

  static object_ptr<getCallbackQueryMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class getChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChat();

  explicit getChat(int53 chat_id_);

  static const std::int32_t ID = 1866601536;

  using ReturnType = object_ptr<chat>;

  static object_ptr<getChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatActiveStories;

class getChatActiveStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatActiveStories();

  explicit getChatActiveStories(int53 chat_id_);

  static const std::int32_t ID = 776993781;

  using ReturnType = object_ptr<chatActiveStories>;

  static object_ptr<getChatActiveStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministrators;

class getChatAdministrators final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatAdministrators();

  explicit getChatAdministrators(int53 chat_id_);

  static const std::int32_t ID = 1544468155;

  using ReturnType = object_ptr<chatAdministrators>;

  static object_ptr<getChatAdministrators> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stories;

class getChatArchivedStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 from_story_id_;
  static jfieldID from_story_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatArchivedStories();

  getChatArchivedStories(int53 chat_id_, int32 from_story_id_, int32 limit_);

  static const std::int32_t ID = -1356950392;

  using ReturnType = object_ptr<stories>;

  static object_ptr<getChatArchivedStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatMessageSenders;

class getChatAvailableMessageSenders final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatAvailableMessageSenders();

  explicit getChatAvailableMessageSenders(int53 chat_id_);

  static const std::int32_t ID = 1158670635;

  using ReturnType = object_ptr<chatMessageSenders>;

  static object_ptr<getChatAvailableMessageSenders> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSenders;

class getChatAvailablePaidMessageReactionSenders final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatAvailablePaidMessageReactionSenders();

  explicit getChatAvailablePaidMessageReactionSenders(int53 chat_id_);

  static const std::int32_t ID = -1244619639;

  using ReturnType = object_ptr<messageSenders>;

  static object_ptr<getChatAvailablePaidMessageReactionSenders> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostFeatures;

class getChatBoostFeatures final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_channel_;
  static jfieldID is_channel_fieldID;

  getChatBoostFeatures();

  explicit getChatBoostFeatures(bool is_channel_);

  static const std::int32_t ID = -389994336;

  using ReturnType = object_ptr<chatBoostFeatures>;

  static object_ptr<getChatBoostFeatures> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLevelFeatures;

class getChatBoostLevelFeatures final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_channel_;
  static jfieldID is_channel_fieldID;
  int32 level_;
  static jfieldID level_fieldID;

  getChatBoostLevelFeatures();

  getChatBoostLevelFeatures(bool is_channel_, int32 level_);

  static const std::int32_t ID = 1172717195;

  using ReturnType = object_ptr<chatBoostLevelFeatures>;

  static object_ptr<getChatBoostLevelFeatures> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLink;

class getChatBoostLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatBoostLink();

  explicit getChatBoostLink(int53 chat_id_);

  static const std::int32_t ID = 1458662533;

  using ReturnType = object_ptr<chatBoostLink>;

  static object_ptr<getChatBoostLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostLinkInfo;

class getChatBoostLinkInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  getChatBoostLinkInfo();

  explicit getChatBoostLinkInfo(string const &url_);

  static const std::int32_t ID = 654068572;

  using ReturnType = object_ptr<chatBoostLinkInfo>;

  static object_ptr<getChatBoostLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatBoostStatus;

class getChatBoostStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatBoostStatus();

  explicit getChatBoostStatus(int53 chat_id_);

  static const std::int32_t ID = -810775857;

  using ReturnType = object_ptr<chatBoostStatus>;

  static object_ptr<getChatBoostStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundChatBoosts;

class getChatBoosts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool only_gift_codes_;
  static jfieldID only_gift_codes_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatBoosts();

  getChatBoosts(int53 chat_id_, bool only_gift_codes_, string const &offset_, int32 limit_);

  static const std::int32_t ID = -1419859400;

  using ReturnType = object_ptr<foundChatBoosts>;

  static object_ptr<getChatBoosts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatEventLogFilters;

class chatEvents;

class getChatEventLog final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int64 from_event_id_;
  static jfieldID from_event_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  object_ptr<chatEventLogFilters> filters_;
  static jfieldID filters_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  getChatEventLog();

  getChatEventLog(int53 chat_id_, string const &query_, int64 from_event_id_, int32 limit_, object_ptr<chatEventLogFilters> &&filters_, array<int53> &&user_ids_);

  static const std::int32_t ID = -1281344669;

  using ReturnType = object_ptr<chatEvents>;

  static object_ptr<getChatEventLog> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class getChatFolder final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  getChatFolder();

  explicit getChatFolder(int32 chat_folder_id_);

  static const std::int32_t ID = 92809880;

  using ReturnType = object_ptr<chatFolder>;

  static object_ptr<getChatFolder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class count;

class getChatFolderChatCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolder> folder_;
  static jfieldID folder_fieldID;

  getChatFolderChatCount();

  explicit getChatFolderChatCount(object_ptr<chatFolder> &&folder_);

  static const std::int32_t ID = 2111097790;

  using ReturnType = object_ptr<count>;

  static object_ptr<getChatFolderChatCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getChatFolderChatsToLeave final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  getChatFolderChatsToLeave();

  explicit getChatFolderChatsToLeave(int32 chat_folder_id_);

  static const std::int32_t ID = -1916672337;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatFolderChatsToLeave> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolder;

class chatFolderIcon;

class getChatFolderDefaultIconName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatFolder> folder_;
  static jfieldID folder_fieldID;

  getChatFolderDefaultIconName();

  explicit getChatFolderDefaultIconName(object_ptr<chatFolder> &&folder_);

  static const std::int32_t ID = 754425959;

  using ReturnType = object_ptr<chatFolderIcon>;

  static object_ptr<getChatFolderDefaultIconName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatFolderInviteLinks;

class getChatFolderInviteLinks final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  getChatFolderInviteLinks();

  explicit getChatFolderInviteLinks(int32 chat_folder_id_);

  static const std::int32_t ID = 329079776;

  using ReturnType = object_ptr<chatFolderInviteLinks>;

  static object_ptr<getChatFolderInviteLinks> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getChatFolderNewChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  getChatFolderNewChats();

  explicit getChatFolderNewChats(int32 chat_folder_id_);

  static const std::int32_t ID = 2123181260;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatFolderNewChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class getChatHistory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  bool only_local_;
  static jfieldID only_local_fieldID;

  getChatHistory();

  getChatHistory(int53 chat_id_, int53 from_message_id_, int32 offset_, int32 limit_, bool only_local_);

  static const std::int32_t ID = -799960451;

  using ReturnType = object_ptr<messages>;

  static object_ptr<getChatHistory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class getChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  getChatInviteLink();

  getChatInviteLink(int53 chat_id_, string const &invite_link_);

  static const std::int32_t ID = -479575555;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<getChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkCounts;

class getChatInviteLinkCounts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatInviteLinkCounts();

  explicit getChatInviteLinkCounts(int53 chat_id_);

  static const std::int32_t ID = 890299025;

  using ReturnType = object_ptr<chatInviteLinkCounts>;

  static object_ptr<getChatInviteLinkCounts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinkMember;

class chatInviteLinkMembers;

class getChatInviteLinkMembers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  bool only_with_expired_subscription_;
  static jfieldID only_with_expired_subscription_fieldID;
  object_ptr<chatInviteLinkMember> offset_member_;
  static jfieldID offset_member_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatInviteLinkMembers();

  getChatInviteLinkMembers(int53 chat_id_, string const &invite_link_, bool only_with_expired_subscription_, object_ptr<chatInviteLinkMember> &&offset_member_, int32 limit_);

  static const std::int32_t ID = 1728376124;

  using ReturnType = object_ptr<chatInviteLinkMembers>;

  static object_ptr<getChatInviteLinkMembers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinks;

class getChatInviteLinks final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 creator_user_id_;
  static jfieldID creator_user_id_fieldID;
  bool is_revoked_;
  static jfieldID is_revoked_fieldID;
  int32 offset_date_;
  static jfieldID offset_date_fieldID;
  string offset_invite_link_;
  static jfieldID offset_invite_link_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatInviteLinks();

  getChatInviteLinks(int53 chat_id_, int53 creator_user_id_, bool is_revoked_, int32 offset_date_, string const &offset_invite_link_, int32 limit_);

  static const std::int32_t ID = 883252396;

  using ReturnType = object_ptr<chatInviteLinks>;

  static object_ptr<getChatInviteLinks> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatJoinRequest;

class chatJoinRequests;

class getChatJoinRequests final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  string query_;
  static jfieldID query_fieldID;
  object_ptr<chatJoinRequest> offset_request_;
  static jfieldID offset_request_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatJoinRequests();

  getChatJoinRequests(int53 chat_id_, string const &invite_link_, string const &query_, object_ptr<chatJoinRequest> &&offset_request_, int32 limit_);

  static const std::int32_t ID = -388428126;

  using ReturnType = object_ptr<chatJoinRequests>;

  static object_ptr<getChatJoinRequests> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatLists;

class getChatListsToAddChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatListsToAddChat();

  explicit getChatListsToAddChat(int53 chat_id_);

  static const std::int32_t ID = 654956193;

  using ReturnType = object_ptr<chatLists>;

  static object_ptr<getChatListsToAddChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class chatMember;

class getChatMember final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;

  getChatMember();

  getChatMember(int53 chat_id_, object_ptr<MessageSender> &&member_id_);

  static const std::int32_t ID = -792636814;

  using ReturnType = object_ptr<chatMember>;

  static object_ptr<getChatMember> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getChatMessageByDate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  getChatMessageByDate();

  getChatMessageByDate(int53 chat_id_, int32 date_);

  static const std::int32_t ID = 1062564150;

  using ReturnType = object_ptr<message>;

  static object_ptr<getChatMessageByDate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter;

class messageCalendar;

class getChatMessageCalendar final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  getChatMessageCalendar();

  getChatMessageCalendar(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 from_message_id_, int53 saved_messages_topic_id_);

  static const std::int32_t ID = -2119225929;

  using ReturnType = object_ptr<messageCalendar>;

  static object_ptr<getChatMessageCalendar> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter;

class count;

class getChatMessageCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  bool return_local_;
  static jfieldID return_local_fieldID;

  getChatMessageCount();

  getChatMessageCount(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 saved_messages_topic_id_, bool return_local_);

  static const std::int32_t ID = 955746569;

  using ReturnType = object_ptr<count>;

  static object_ptr<getChatMessageCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter;

class count;

class getChatMessagePosition final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  getChatMessagePosition();

  getChatMessagePosition(int53 chat_id_, int53 message_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 message_thread_id_, int53 saved_messages_topic_id_);

  static const std::int32_t ID = 136051911;

  using ReturnType = object_ptr<count>;

  static object_ptr<getChatMessagePosition> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NotificationSettingsScope;

class chats;

class getChatNotificationSettingsExceptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NotificationSettingsScope> scope_;
  static jfieldID scope_fieldID;
  bool compare_sound_;
  static jfieldID compare_sound_fieldID;

  getChatNotificationSettingsExceptions();

  getChatNotificationSettingsExceptions(object_ptr<NotificationSettingsScope> &&scope_, bool compare_sound_);

  static const std::int32_t ID = 201199121;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatNotificationSettingsExceptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getChatPinnedMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatPinnedMessage();

  explicit getChatPinnedMessage(int53 chat_id_);

  static const std::int32_t ID = 359865008;

  using ReturnType = object_ptr<message>;

  static object_ptr<getChatPinnedMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stories;

class getChatPostedToChatPageStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 from_story_id_;
  static jfieldID from_story_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatPostedToChatPageStories();

  getChatPostedToChatPageStories(int53 chat_id_, int32 from_story_id_, int32 limit_);

  static const std::int32_t ID = -46414037;

  using ReturnType = object_ptr<stories>;

  static object_ptr<getChatPostedToChatPageStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueStatistics;

class getChatRevenueStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;

  getChatRevenueStatistics();

  getChatRevenueStatistics(int53 chat_id_, bool is_dark_);

  static const std::int32_t ID = 701995836;

  using ReturnType = object_ptr<chatRevenueStatistics>;

  static object_ptr<getChatRevenueStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatRevenueTransactions;

class getChatRevenueTransactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatRevenueTransactions();

  getChatRevenueTransactions(int53 chat_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = 1194264341;

  using ReturnType = object_ptr<chatRevenueTransactions>;

  static object_ptr<getChatRevenueTransactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getChatRevenueWithdrawalUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string password_;
  static jfieldID password_fieldID;

  getChatRevenueWithdrawalUrl();

  getChatRevenueWithdrawalUrl(int53 chat_id_, string const &password_);

  static const std::int32_t ID = 506595104;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getChatRevenueWithdrawalUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class getChatScheduledMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatScheduledMessages();

  explicit getChatScheduledMessages(int53 chat_id_);

  static const std::int32_t ID = -549638149;

  using ReturnType = object_ptr<messages>;

  static object_ptr<getChatScheduledMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class count;

class getChatSimilarChatCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool return_local_;
  static jfieldID return_local_fieldID;

  getChatSimilarChatCount();

  getChatSimilarChatCount(int53 chat_id_, bool return_local_);

  static const std::int32_t ID = 1178506894;

  using ReturnType = object_ptr<count>;

  static object_ptr<getChatSimilarChatCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getChatSimilarChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatSimilarChats();

  explicit getChatSimilarChats(int53 chat_id_);

  static const std::int32_t ID = -1152348285;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatSimilarChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter;

class messagePositions;

class getChatSparseMessagePositions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  getChatSparseMessagePositions();

  getChatSparseMessagePositions(int53 chat_id_, object_ptr<SearchMessagesFilter> &&filter_, int53 from_message_id_, int32 limit_, int53 saved_messages_topic_id_);

  static const std::int32_t ID = 994389757;

  using ReturnType = object_ptr<messagePositions>;

  static object_ptr<getChatSparseMessagePositions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sponsoredMessages;

class getChatSponsoredMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getChatSponsoredMessages();

  explicit getChatSponsoredMessages(int53 chat_id_);

  static const std::int32_t ID = 1353203864;

  using ReturnType = object_ptr<sponsoredMessages>;

  static object_ptr<getChatSponsoredMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatStatistics;

class getChatStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;

  getChatStatistics();

  getChatStatistics(int53 chat_id_, bool is_dark_);

  static const std::int32_t ID = 327057816;

  using ReturnType = object_ptr<ChatStatistics>;

  static object_ptr<getChatStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class storyInteractions;

class getChatStoryInteractions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  bool prefer_forwards_;
  static jfieldID prefer_forwards_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChatStoryInteractions();

  getChatStoryInteractions(int53 story_poster_chat_id_, int32 story_id_, object_ptr<ReactionType> &&reaction_type_, bool prefer_forwards_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 354545268;

  using ReturnType = object_ptr<storyInteractions>;

  static object_ptr<getChatStoryInteractions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class chats;

class getChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getChats();

  getChats(object_ptr<ChatList> &&chat_list_, int32 limit_);

  static const std::int32_t ID = -972768574;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getChatsForChatFolderInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;

  getChatsForChatFolderInviteLink();

  explicit getChatsForChatFolderInviteLink(int32 chat_folder_id_);

  static const std::int32_t ID = 1873561929;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatsForChatFolderInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getChatsToPostStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getChatsToPostStories();

  static const std::int32_t ID = 1893901427;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getChatsToPostStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class getCloseFriends final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getCloseFriends();

  static const std::int32_t ID = -1445628722;

  using ReturnType = object_ptr<users>;

  static object_ptr<getCloseFriends> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CollectibleItemType;

class collectibleItemInfo;

class getCollectibleItemInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<CollectibleItemType> type_;
  static jfieldID type_fieldID;

  getCollectibleItemInfo();

  explicit getCollectibleItemInfo(object_ptr<CollectibleItemType> &&type_);

  static const std::int32_t ID = -217797238;

  using ReturnType = object_ptr<collectibleItemInfo>;

  static object_ptr<getCollectibleItemInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BotCommandScope;

class botCommands;

class getCommands final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BotCommandScope> scope_;
  static jfieldID scope_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;

  getCommands();

  getCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_);

  static const std::int32_t ID = 1488621559;

  using ReturnType = object_ptr<botCommands>;

  static object_ptr<getCommands> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateType;

class connectedAffiliateProgram;

class getConnectedAffiliateProgram final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AffiliateType> affiliate_;
  static jfieldID affiliate_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  getConnectedAffiliateProgram();

  getConnectedAffiliateProgram(object_ptr<AffiliateType> &&affiliate_, int53 bot_user_id_);

  static const std::int32_t ID = -1755191440;

  using ReturnType = object_ptr<connectedAffiliateProgram>;

  static object_ptr<getConnectedAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateType;

class connectedAffiliatePrograms;

class getConnectedAffiliatePrograms final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AffiliateType> affiliate_;
  static jfieldID affiliate_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getConnectedAffiliatePrograms();

  getConnectedAffiliatePrograms(object_ptr<AffiliateType> &&affiliate_, string const &offset_, int32 limit_);

  static const std::int32_t ID = -1960029582;

  using ReturnType = object_ptr<connectedAffiliatePrograms>;

  static object_ptr<getConnectedAffiliatePrograms> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class connectedWebsites;

class getConnectedWebsites final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getConnectedWebsites();

  static const std::int32_t ID = -170536110;

  using ReturnType = object_ptr<connectedWebsites>;

  static object_ptr<getConnectedWebsites> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class getContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getContacts();

  static const std::int32_t ID = -1417722768;

  using ReturnType = object_ptr<users>;

  static object_ptr<getContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class countries;

class getCountries final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getCountries();

  static const std::int32_t ID = -51902050;

  using ReturnType = object_ptr<countries>;

  static object_ptr<getCountries> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getCountryCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getCountryCode();

  static const std::int32_t ID = 1540593906;

  using ReturnType = object_ptr<text>;

  static object_ptr<getCountryCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getCountryFlagEmoji final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string country_code_;
  static jfieldID country_code_fieldID;

  getCountryFlagEmoji();

  explicit getCountryFlagEmoji(string const &country_code_);

  static const std::int32_t ID = 981871098;

  using ReturnType = object_ptr<text>;

  static object_ptr<getCountryFlagEmoji> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PublicChatType;

class chats;

class getCreatedPublicChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PublicChatType> type_;
  static jfieldID type_fieldID;

  getCreatedPublicChats();

  explicit getCreatedPublicChats(object_ptr<PublicChatType> &&type_);

  static const std::int32_t ID = 710354415;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getCreatedPublicChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class updates;

class getCurrentState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getCurrentState();

  static const std::int32_t ID = -1191417719;

  using ReturnType = object_ptr<updates>;

  static object_ptr<getCurrentState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class currentWeather;

class location;

class getCurrentWeather final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;

  getCurrentWeather();

  explicit getCurrentWeather(object_ptr<location> &&location_);

  static const std::int32_t ID = -1965384759;

  using ReturnType = object_ptr<currentWeather>;

  static object_ptr<getCurrentWeather> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getCustomEmojiReactionAnimations final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getCustomEmojiReactionAnimations();

  static const std::int32_t ID = 1232375250;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getCustomEmojiReactionAnimations> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getCustomEmojiStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int64> custom_emoji_ids_;
  static jfieldID custom_emoji_ids_fieldID;

  getCustomEmojiStickers();

  explicit getCustomEmojiStickers(array<int64> &&custom_emoji_ids_);

  static const std::int32_t ID = -2127427955;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getCustomEmojiStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class databaseStatistics;

class getDatabaseStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDatabaseStatistics();

  static const std::int32_t ID = -1942760263;

  using ReturnType = object_ptr<databaseStatistics>;

  static object_ptr<getDatabaseStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class deepLinkInfo;

class getDeepLinkInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  getDeepLinkInfo();

  explicit getDeepLinkInfo(string const &link_);

  static const std::int32_t ID = 680673150;

  using ReturnType = object_ptr<deepLinkInfo>;

  static object_ptr<getDeepLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getDefaultBackgroundCustomEmojiStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultBackgroundCustomEmojiStickers();

  static const std::int32_t ID = 485910542;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getDefaultBackgroundCustomEmojiStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis;

class getDefaultChatEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultChatEmojiStatuses();

  static const std::int32_t ID = 1553698018;

  using ReturnType = object_ptr<emojiStatusCustomEmojis>;

  static object_ptr<getDefaultChatEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getDefaultChatPhotoCustomEmojiStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultChatPhotoCustomEmojiStickers();

  static const std::int32_t ID = -376342683;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getDefaultChatPhotoCustomEmojiStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis;

class getDefaultEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultEmojiStatuses();

  static const std::int32_t ID = -539392025;

  using ReturnType = object_ptr<emojiStatusCustomEmojis>;

  static object_ptr<getDefaultEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageAutoDeleteTime;

class getDefaultMessageAutoDeleteTime final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultMessageAutoDeleteTime();

  static const std::int32_t ID = -450857574;

  using ReturnType = object_ptr<messageAutoDeleteTime>;

  static object_ptr<getDefaultMessageAutoDeleteTime> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getDefaultProfilePhotoCustomEmojiStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDefaultProfilePhotoCustomEmojiStickers();

  static const std::int32_t ID = 1280041655;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getDefaultProfilePhotoCustomEmojiStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis;

class getDisallowedChatEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getDisallowedChatEmojiStatuses();

  static const std::int32_t ID = -2004787831;

  using ReturnType = object_ptr<emojiStatusCustomEmojis>;

  static object_ptr<getDisallowedChatEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class EmojiCategoryType;

class emojiCategories;

class getEmojiCategories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<EmojiCategoryType> type_;
  static jfieldID type_fieldID;

  getEmojiCategories();

  explicit getEmojiCategories(object_ptr<EmojiCategoryType> &&type_);

  static const std::int32_t ID = 2139537774;

  using ReturnType = object_ptr<emojiCategories>;

  static object_ptr<getEmojiCategories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiReaction;

class getEmojiReaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string emoji_;
  static jfieldID emoji_fieldID;

  getEmojiReaction();

  explicit getEmojiReaction(string const &emoji_);

  static const std::int32_t ID = -449572388;

  using ReturnType = object_ptr<emojiReaction>;

  static object_ptr<getEmojiReaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getEmojiSuggestionsUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_code_;
  static jfieldID language_code_fieldID;

  getEmojiSuggestionsUrl();

  explicit getEmojiSuggestionsUrl(string const &language_code_);

  static const std::int32_t ID = -1404101841;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getEmojiSuggestionsUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getExternalLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;
  bool allow_write_access_;
  static jfieldID allow_write_access_fieldID;

  getExternalLink();

  getExternalLink(string const &link_, bool allow_write_access_);

  static const std::int32_t ID = 1586688235;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getExternalLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class LoginUrlInfo;

class getExternalLinkInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  getExternalLinkInfo();

  explicit getExternalLinkInfo(string const &link_);

  static const std::int32_t ID = 1175288383;

  using ReturnType = object_ptr<LoginUrlInfo>;

  static object_ptr<getExternalLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getFavoriteStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getFavoriteStickers();

  static const std::int32_t ID = -338964672;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getFavoriteStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class file;

class getFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;

  getFile();

  explicit getFile(int32 file_id_);

  static const std::int32_t ID = 1553923406;

  using ReturnType = object_ptr<file>;

  static object_ptr<getFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class fileDownloadedPrefixSize;

class getFileDownloadedPrefixSize final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  int53 offset_;
  static jfieldID offset_fieldID;

  getFileDownloadedPrefixSize();

  getFileDownloadedPrefixSize(int32 file_id_, int53 offset_);

  static const std::int32_t ID = 855948589;

  using ReturnType = object_ptr<fileDownloadedPrefixSize>;

  static object_ptr<getFileDownloadedPrefixSize> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getFileExtension final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string mime_type_;
  static jfieldID mime_type_fieldID;

  getFileExtension();

  explicit getFileExtension(string const &mime_type_);

  static const std::int32_t ID = -106055372;

  using ReturnType = object_ptr<text>;

  static object_ptr<getFileExtension> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getFileMimeType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string file_name_;
  static jfieldID file_name_fieldID;

  getFileMimeType();

  explicit getFileMimeType(string const &file_name_);

  static const std::int32_t ID = -2073879671;

  using ReturnType = object_ptr<text>;

  static object_ptr<getFileMimeType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class forumTopic;

class getForumTopic final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  getForumTopic();

  getForumTopic(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = -442761663;

  using ReturnType = object_ptr<forumTopic>;

  static object_ptr<getForumTopic> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getForumTopicDefaultIcons final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getForumTopicDefaultIcons();

  static const std::int32_t ID = 1479898332;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getForumTopicDefaultIcons> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageLink;

class getForumTopicLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  getForumTopicLink();

  getForumTopicLink(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = -914650933;

  using ReturnType = object_ptr<messageLink>;

  static object_ptr<getForumTopicLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class forumTopics;

class getForumTopics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int32 offset_date_;
  static jfieldID offset_date_fieldID;
  int53 offset_message_id_;
  static jfieldID offset_message_id_fieldID;
  int53 offset_message_thread_id_;
  static jfieldID offset_message_thread_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getForumTopics();

  getForumTopics(int53 chat_id_, string const &query_, int32 offset_date_, int53 offset_message_id_, int53 offset_message_thread_id_, int32 limit_);

  static const std::int32_t ID = -72647334;

  using ReturnType = object_ptr<forumTopics>;

  static object_ptr<getForumTopics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class gameHighScores;

class getGameHighScores final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getGameHighScores();

  getGameHighScores(int53 chat_id_, int53 message_id_, int53 user_id_);

  static const std::int32_t ID = 15746459;

  using ReturnType = object_ptr<gameHighScores>;

  static object_ptr<getGameHighScores> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giftUpgradePreview;

class getGiftUpgradePreview final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 gift_id_;
  static jfieldID gift_id_fieldID;

  getGiftUpgradePreview();

  explicit getGiftUpgradePreview(int64 gift_id_);

  static const std::int32_t ID = -1110719907;

  using ReturnType = object_ptr<giftUpgradePreview>;

  static object_ptr<getGiftUpgradePreview> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class GiveawayInfo;

class getGiveawayInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getGiveawayInfo();

  getGiveawayInfo(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1215852357;

  using ReturnType = object_ptr<GiveawayInfo>;

  static object_ptr<getGiveawayInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getGreetingStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getGreetingStickers();

  static const std::int32_t ID = 374873372;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getGreetingStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundUsers;

class getGrossingWebAppBots final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getGrossingWebAppBots();

  getGrossingWebAppBots(string const &offset_, int32 limit_);

  static const std::int32_t ID = 1696779802;

  using ReturnType = object_ptr<foundUsers>;

  static object_ptr<getGrossingWebAppBots> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class groupCall;

class getGroupCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  getGroupCall();

  explicit getGroupCall(int32 group_call_id_);

  static const std::int32_t ID = 1468491406;

  using ReturnType = object_ptr<groupCall>;

  static object_ptr<getGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputGroupCall;

class groupCallParticipants;

class getGroupCallParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputGroupCall> input_group_call_;
  static jfieldID input_group_call_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getGroupCallParticipants();

  getGroupCallParticipants(object_ptr<InputGroupCall> &&input_group_call_, int32 limit_);

  static const std::int32_t ID = 1986739394;

  using ReturnType = object_ptr<groupCallParticipants>;

  static object_ptr<getGroupCallParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getGroupsInCommon final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int53 offset_chat_id_;
  static jfieldID offset_chat_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getGroupsInCommon();

  getGroupsInCommon(int53 user_id_, int53 offset_chat_id_, int32 limit_);

  static const std::int32_t ID = 381539178;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getGroupsInCommon> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class count;

class getImportedContactCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getImportedContactCount();

  static const std::int32_t ID = -656336346;

  using ReturnType = object_ptr<count>;

  static object_ptr<getImportedContactCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getInactiveSupergroupChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getInactiveSupergroupChats();

  static const std::int32_t ID = -657720907;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getInactiveSupergroupChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class gameHighScores;

class getInlineGameHighScores final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getInlineGameHighScores();

  getInlineGameHighScores(string const &inline_message_id_, int53 user_id_);

  static const std::int32_t ID = -533107798;

  using ReturnType = object_ptr<gameHighScores>;

  static object_ptr<getInlineGameHighScores> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inlineQueryResults;

class location;

class getInlineQueryResults final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<location> user_location_;
  static jfieldID user_location_fieldID;
  string query_;
  static jfieldID query_fieldID;
  string offset_;
  static jfieldID offset_fieldID;

  getInlineQueryResults();

  getInlineQueryResults(int53 bot_user_id_, int53 chat_id_, object_ptr<location> &&user_location_, string const &query_, string const &offset_);

  static const std::int32_t ID = 2044524652;

  using ReturnType = object_ptr<inlineQueryResults>;

  static object_ptr<getInlineQueryResults> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class backgrounds;

class getInstalledBackgrounds final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool for_dark_theme_;
  static jfieldID for_dark_theme_fieldID;

  getInstalledBackgrounds();

  explicit getInstalledBackgrounds(bool for_dark_theme_);

  static const std::int32_t ID = -1051406241;

  using ReturnType = object_ptr<backgrounds>;

  static object_ptr<getInstalledBackgrounds> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickerSets;

class getInstalledStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;

  getInstalledStickerSets();

  explicit getInstalledStickerSets(object_ptr<StickerType> &&sticker_type_);

  static const std::int32_t ID = 1630467830;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<getInstalledStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class httpUrl;

class getInternalLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InternalLinkType> type_;
  static jfieldID type_fieldID;
  bool is_http_;
  static jfieldID is_http_fieldID;

  getInternalLink();

  getInternalLink(object_ptr<InternalLinkType> &&type_, bool is_http_);

  static const std::int32_t ID = 962654640;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getInternalLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InternalLinkType;

class getInternalLinkType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string link_;
  static jfieldID link_fieldID;

  getInternalLinkType();

  explicit getInternalLinkType(string const &link_);

  static const std::int32_t ID = -1948428535;

  using ReturnType = object_ptr<InternalLinkType>;

  static object_ptr<getInternalLinkType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class text;

class getJsonString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<JsonValue> json_value_;
  static jfieldID json_value_fieldID;

  getJsonString();

  explicit getJsonString(object_ptr<JsonValue> &&json_value_);

  static const std::int32_t ID = 663458849;

  using ReturnType = object_ptr<text>;

  static object_ptr<getJsonString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class getJsonValue final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string json_;
  static jfieldID json_fieldID;

  getJsonValue();

  explicit getJsonValue(string const &json_);

  static const std::int32_t ID = -1829086715;

  using ReturnType = object_ptr<JsonValue>;

  static object_ptr<getJsonValue> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojis;

class getKeywordEmojis final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  array<string> input_language_codes_;
  static jfieldID input_language_codes_fieldID;

  getKeywordEmojis();

  getKeywordEmojis(string const &text_, array<string> &&input_language_codes_);

  static const std::int32_t ID = -1969795990;

  using ReturnType = object_ptr<emojis>;

  static object_ptr<getKeywordEmojis> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackInfo;

class getLanguagePackInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;

  getLanguagePackInfo();

  explicit getLanguagePackInfo(string const &language_pack_id_);

  static const std::int32_t ID = 2077809320;

  using ReturnType = object_ptr<languagePackInfo>;

  static object_ptr<getLanguagePackInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class LanguagePackStringValue;

class getLanguagePackString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_database_path_;
  static jfieldID language_pack_database_path_fieldID;
  string localization_target_;
  static jfieldID localization_target_fieldID;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;
  string key_;
  static jfieldID key_fieldID;

  getLanguagePackString();

  getLanguagePackString(string const &language_pack_database_path_, string const &localization_target_, string const &language_pack_id_, string const &key_);

  static const std::int32_t ID = 150789747;

  using ReturnType = object_ptr<LanguagePackStringValue>;

  static object_ptr<getLanguagePackString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackStrings;

class getLanguagePackStrings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;
  array<string> keys_;
  static jfieldID keys_fieldID;

  getLanguagePackStrings();

  getLanguagePackStrings(string const &language_pack_id_, array<string> &&keys_);

  static const std::int32_t ID = 1246259088;

  using ReturnType = object_ptr<languagePackStrings>;

  static object_ptr<getLanguagePackStrings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class linkPreview;

class linkPreviewOptions;

class getLinkPreview final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  object_ptr<linkPreviewOptions> link_preview_options_;
  static jfieldID link_preview_options_fieldID;

  getLinkPreview();

  getLinkPreview(object_ptr<formattedText> &&text_, object_ptr<linkPreviewOptions> &&link_preview_options_);

  static const std::int32_t ID = -1039572191;

  using ReturnType = object_ptr<linkPreview>;

  static object_ptr<getLinkPreview> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class localizationTargetInfo;

class getLocalizationTargetInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool only_local_;
  static jfieldID only_local_fieldID;

  getLocalizationTargetInfo();

  explicit getLocalizationTargetInfo(bool only_local_);

  static const std::int32_t ID = 1849499526;

  using ReturnType = object_ptr<localizationTargetInfo>;

  static object_ptr<getLocalizationTargetInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class LogStream;

class getLogStream final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getLogStream();

  static const std::int32_t ID = 1167608667;

  using ReturnType = object_ptr<LogStream>;

  static object_ptr<getLogStream> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class logVerbosityLevel;

class getLogTagVerbosityLevel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string tag_;
  static jfieldID tag_fieldID;

  getLogTagVerbosityLevel();

  explicit getLogTagVerbosityLevel(string const &tag_);

  static const std::int32_t ID = 951004547;

  using ReturnType = object_ptr<logVerbosityLevel>;

  static object_ptr<getLogTagVerbosityLevel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class logTags;

class getLogTags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getLogTags();

  static const std::int32_t ID = -254449190;

  using ReturnType = object_ptr<logTags>;

  static object_ptr<getLogTags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class logVerbosityLevel;

class getLogVerbosityLevel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getLogVerbosityLevel();

  static const std::int32_t ID = 594057956;

  using ReturnType = object_ptr<logVerbosityLevel>;

  static object_ptr<getLogVerbosityLevel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getLoginUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int53 button_id_;
  static jfieldID button_id_fieldID;
  bool allow_write_access_;
  static jfieldID allow_write_access_fieldID;

  getLoginUrl();

  getLoginUrl(int53 chat_id_, int53 message_id_, int53 button_id_, bool allow_write_access_);

  static const std::int32_t ID = 791844305;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getLoginUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class LoginUrlInfo;

class getLoginUrlInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int53 button_id_;
  static jfieldID button_id_fieldID;

  getLoginUrlInfo();

  getLoginUrlInfo(int53 chat_id_, int53 message_id_, int53 button_id_);

  static const std::int32_t ID = -859202125;

  using ReturnType = object_ptr<LoginUrlInfo>;

  static object_ptr<getLoginUrlInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class mainWebApp;

class webAppOpenParameters;

class getMainWebApp final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  object_ptr<webAppOpenParameters> parameters_;
  static jfieldID parameters_fieldID;

  getMainWebApp();

  getMainWebApp(int53 chat_id_, int53 bot_user_id_, string const &start_parameter_, object_ptr<webAppOpenParameters> &&parameters_);

  static const std::int32_t ID = 594050214;

  using ReturnType = object_ptr<mainWebApp>;

  static object_ptr<getMainWebApp> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class file;

class location;

class getMapThumbnailFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<location> location_;
  static jfieldID location_fieldID;
  int32 zoom_;
  static jfieldID zoom_fieldID;
  int32 width_;
  static jfieldID width_fieldID;
  int32 height_;
  static jfieldID height_fieldID;
  int32 scale_;
  static jfieldID scale_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getMapThumbnailFile();

  getMapThumbnailFile(object_ptr<location> &&location_, int32 zoom_, int32 width_, int32 height_, int32 scale_, int53 chat_id_);

  static const std::int32_t ID = -152660070;

  using ReturnType = object_ptr<file>;

  static object_ptr<getMapThumbnailFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class getMarkdownText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  getMarkdownText();

  explicit getMarkdownText(object_ptr<formattedText> &&text_);

  static const std::int32_t ID = 164524584;

  using ReturnType = object_ptr<formattedText>;

  static object_ptr<getMarkdownText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class user;

class getMe final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getMe();

  static const std::int32_t ID = -191516033;

  using ReturnType = object_ptr<user>;

  static object_ptr<getMe> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botMenuButton;

class getMenuButton final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getMenuButton();

  explicit getMenuButton(int53 user_id_);

  static const std::int32_t ID = -437324736;

  using ReturnType = object_ptr<botMenuButton>;

  static object_ptr<getMenuButton> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessage();

  getMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1821196160;

  using ReturnType = object_ptr<message>;

  static object_ptr<getMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class addedReactions;

class getMessageAddedReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getMessageAddedReactions();

  getMessageAddedReactions(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 2110172754;

  using ReturnType = object_ptr<addedReactions>;

  static object_ptr<getMessageAddedReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class availableReactions;

class getMessageAvailableReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 row_size_;
  static jfieldID row_size_fieldID;

  getMessageAvailableReactions();

  getMessageAvailableReactions(int53 chat_id_, int53 message_id_, int32 row_size_);

  static const std::int32_t ID = 1994098354;

  using ReturnType = object_ptr<availableReactions>;

  static object_ptr<getMessageAvailableReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageEffect;

class getMessageEffect final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;

  getMessageEffect();

  explicit getMessageEffect(int64 effect_id_);

  static const std::int32_t ID = -1638843116;

  using ReturnType = object_ptr<messageEffect>;

  static object_ptr<getMessageEffect> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getMessageEmbeddingCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool for_album_;
  static jfieldID for_album_fieldID;

  getMessageEmbeddingCode();

  getMessageEmbeddingCode(int53 chat_id_, int53 message_id_, bool for_album_);

  static const std::int32_t ID = 1654967561;

  using ReturnType = object_ptr<text>;

  static object_ptr<getMessageEmbeddingCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageFileType;

class getMessageFileType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string message_file_head_;
  static jfieldID message_file_head_fieldID;

  getMessageFileType();

  explicit getMessageFileType(string const &message_file_head_);

  static const std::int32_t ID = -490270764;

  using ReturnType = object_ptr<MessageFileType>;

  static object_ptr<getMessageFileType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getMessageImportConfirmationText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getMessageImportConfirmationText();

  explicit getMessageImportConfirmationText(int53 chat_id_);

  static const std::int32_t ID = 390627752;

  using ReturnType = object_ptr<text>;

  static object_ptr<getMessageImportConfirmationText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageLink;

class getMessageLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 media_timestamp_;
  static jfieldID media_timestamp_fieldID;
  bool for_album_;
  static jfieldID for_album_fieldID;
  bool in_message_thread_;
  static jfieldID in_message_thread_fieldID;

  getMessageLink();

  getMessageLink(int53 chat_id_, int53 message_id_, int32 media_timestamp_, bool for_album_, bool in_message_thread_);

  static const std::int32_t ID = -984158342;

  using ReturnType = object_ptr<messageLink>;

  static object_ptr<getMessageLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageLinkInfo;

class getMessageLinkInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;

  getMessageLinkInfo();

  explicit getMessageLinkInfo(string const &url_);

  static const std::int32_t ID = -700533672;

  using ReturnType = object_ptr<messageLinkInfo>;

  static object_ptr<getMessageLinkInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getMessageLocally final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessageLocally();

  getMessageLocally(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -603575444;

  using ReturnType = object_ptr<message>;

  static object_ptr<getMessageLocally> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageProperties;

class getMessageProperties final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessageProperties();

  getMessageProperties(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 773382571;

  using ReturnType = object_ptr<messageProperties>;

  static object_ptr<getMessageProperties> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class publicForwards;

class getMessagePublicForwards final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getMessagePublicForwards();

  getMessagePublicForwards(int53 chat_id_, int53 message_id_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 1369285812;

  using ReturnType = object_ptr<publicForwards>;

  static object_ptr<getMessagePublicForwards> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageReadDate;

class getMessageReadDate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessageReadDate();

  getMessageReadDate(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1484455101;

  using ReturnType = object_ptr<MessageReadDate>;

  static object_ptr<getMessageReadDate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageStatistics;

class getMessageStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;

  getMessageStatistics();

  getMessageStatistics(int53 chat_id_, int53 message_id_, bool is_dark_);

  static const std::int32_t ID = 1270194648;

  using ReturnType = object_ptr<messageStatistics>;

  static object_ptr<getMessageStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageThreadInfo;

class getMessageThread final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessageThread();

  getMessageThread(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 2062695998;

  using ReturnType = object_ptr<messageThreadInfo>;

  static object_ptr<getMessageThread> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class getMessageThreadHistory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getMessageThreadHistory();

  getMessageThreadHistory(int53 chat_id_, int53 message_id_, int53 from_message_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -1808411608;

  using ReturnType = object_ptr<messages>;

  static object_ptr<getMessageThreadHistory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageViewers;

class getMessageViewers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getMessageViewers();

  getMessageViewers(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1584457010;

  using ReturnType = object_ptr<messageViewers>;

  static object_ptr<getMessageViewers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class getMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  getMessages();

  getMessages(int53 chat_id_, array<int53> &&message_ids_);

  static const std::int32_t ID = 425299338;

  using ReturnType = object_ptr<messages>;

  static object_ptr<getMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class networkStatistics;

class getNetworkStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool only_current_;
  static jfieldID only_current_fieldID;

  getNetworkStatistics();

  explicit getNetworkStatistics(bool only_current_);

  static const std::int32_t ID = -986228706;

  using ReturnType = object_ptr<networkStatistics>;

  static object_ptr<getNetworkStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class newChatPrivacySettings;

class getNewChatPrivacySettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getNewChatPrivacySettings();

  static const std::int32_t ID = -1295299657;

  using ReturnType = object_ptr<newChatPrivacySettings>;

  static object_ptr<getNewChatPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class OptionValue;

class getOption final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  getOption();

  explicit getOption(string const &name_);

  static const std::int32_t ID = -1572495746;

  using ReturnType = object_ptr<OptionValue>;

  static object_ptr<getOption> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class getOwnedBots final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getOwnedBots();

  static const std::int32_t ID = -1954035715;

  using ReturnType = object_ptr<users>;

  static object_ptr<getOwnedBots> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerSets;

class getOwnedStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 offset_sticker_set_id_;
  static jfieldID offset_sticker_set_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getOwnedStickerSets();

  getOwnedStickerSets(int64 offset_sticker_set_id_, int32 limit_);

  static const std::int32_t ID = 1493074208;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<getOwnedStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starCount;

class getPaidMessageRevenue final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getPaidMessageRevenue();

  explicit getPaidMessageRevenue(int53 user_id_);

  static const std::int32_t ID = 1976589102;

  using ReturnType = object_ptr<starCount>;

  static object_ptr<getPaidMessageRevenue> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportAuthorizationForm;

class getPassportAuthorizationForm final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string scope_;
  static jfieldID scope_fieldID;
  string public_key_;
  static jfieldID public_key_fieldID;
  string nonce_;
  static jfieldID nonce_fieldID;

  getPassportAuthorizationForm();

  getPassportAuthorizationForm(int53 bot_user_id_, string const &scope_, string const &public_key_, string const &nonce_);

  static const std::int32_t ID = 1636107398;

  using ReturnType = object_ptr<passportAuthorizationForm>;

  static object_ptr<getPassportAuthorizationForm> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passportElementsWithErrors;

class getPassportAuthorizationFormAvailableElements final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 authorization_form_id_;
  static jfieldID authorization_form_id_fieldID;
  string password_;
  static jfieldID password_fieldID;

  getPassportAuthorizationFormAvailableElements();

  getPassportAuthorizationFormAvailableElements(int32 authorization_form_id_, string const &password_);

  static const std::int32_t ID = 1068700924;

  using ReturnType = object_ptr<passportElementsWithErrors>;

  static object_ptr<getPassportAuthorizationFormAvailableElements> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PassportElement;

class PassportElementType;

class getPassportElement final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PassportElementType> type_;
  static jfieldID type_fieldID;
  string password_;
  static jfieldID password_fieldID;

  getPassportElement();

  getPassportElement(object_ptr<PassportElementType> &&type_, string const &password_);

  static const std::int32_t ID = -1882398342;

  using ReturnType = object_ptr<PassportElement>;

  static object_ptr<getPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class getPasswordState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getPasswordState();

  static const std::int32_t ID = -174752904;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<getPasswordState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputInvoice;

class paymentForm;

class themeParameters;

class getPaymentForm final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputInvoice> input_invoice_;
  static jfieldID input_invoice_fieldID;
  object_ptr<themeParameters> theme_;
  static jfieldID theme_fieldID;

  getPaymentForm();

  getPaymentForm(object_ptr<InputInvoice> &&input_invoice_, object_ptr<themeParameters> &&theme_);

  static const std::int32_t ID = -1924172076;

  using ReturnType = object_ptr<paymentForm>;

  static object_ptr<getPaymentForm> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class paymentReceipt;

class getPaymentReceipt final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getPaymentReceipt();

  getPaymentReceipt(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1013758294;

  using ReturnType = object_ptr<paymentReceipt>;

  static object_ptr<getPaymentReceipt> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class phoneNumberInfo;

class getPhoneNumberInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_prefix_;
  static jfieldID phone_number_prefix_fieldID;

  getPhoneNumberInfo();

  explicit getPhoneNumberInfo(string const &phone_number_prefix_);

  static const std::int32_t ID = -1608344583;

  using ReturnType = object_ptr<phoneNumberInfo>;

  static object_ptr<getPhoneNumberInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class phoneNumberInfo;

class getPhoneNumberInfoSync final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_code_;
  static jfieldID language_code_fieldID;
  string phone_number_prefix_;
  static jfieldID phone_number_prefix_fieldID;

  getPhoneNumberInfoSync();

  getPhoneNumberInfoSync(string const &language_code_, string const &phone_number_prefix_);

  static const std::int32_t ID = 547061048;

  using ReturnType = object_ptr<phoneNumberInfo>;

  static object_ptr<getPhoneNumberInfoSync> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSenders;

class getPollVoters final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 option_id_;
  static jfieldID option_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getPollVoters();

  getPollVoters(int53 chat_id_, int53 message_id_, int32 option_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -1000625748;

  using ReturnType = object_ptr<messageSenders>;

  static object_ptr<getPollVoters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getPreferredCountryLanguage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string country_code_;
  static jfieldID country_code_fieldID;

  getPreferredCountryLanguage();

  explicit getPreferredCountryLanguage(string const &country_code_);

  static const std::int32_t ID = -933049386;

  using ReturnType = object_ptr<text>;

  static object_ptr<getPreferredCountryLanguage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PremiumSource;

class premiumFeatures;

class getPremiumFeatures final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumSource> source_;
  static jfieldID source_fieldID;

  getPremiumFeatures();

  explicit getPremiumFeatures(object_ptr<PremiumSource> &&source_);

  static const std::int32_t ID = -1260640695;

  using ReturnType = object_ptr<premiumFeatures>;

  static object_ptr<getPremiumFeatures> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiftPaymentOptions;

class getPremiumGiftPaymentOptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getPremiumGiftPaymentOptions();

  static const std::int32_t ID = -480334244;

  using ReturnType = object_ptr<premiumGiftPaymentOptions>;

  static object_ptr<getPremiumGiftPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumGiveawayPaymentOptions;

class getPremiumGiveawayPaymentOptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 boosted_chat_id_;
  static jfieldID boosted_chat_id_fieldID;

  getPremiumGiveawayPaymentOptions();

  explicit getPremiumGiveawayPaymentOptions(int53 boosted_chat_id_);

  static const std::int32_t ID = 1222168073;

  using ReturnType = object_ptr<premiumGiveawayPaymentOptions>;

  static object_ptr<getPremiumGiveawayPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sticker;

class getPremiumInfoSticker final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 month_count_;
  static jfieldID month_count_fieldID;

  getPremiumInfoSticker();

  explicit getPremiumInfoSticker(int32 month_count_);

  static const std::int32_t ID = 2043562651;

  using ReturnType = object_ptr<sticker>;

  static object_ptr<getPremiumInfoSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PremiumLimitType;

class premiumLimit;

class getPremiumLimit final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumLimitType> limit_type_;
  static jfieldID limit_type_fieldID;

  getPremiumLimit();

  explicit getPremiumLimit(object_ptr<PremiumLimitType> &&limit_type_);

  static const std::int32_t ID = 1075313898;

  using ReturnType = object_ptr<premiumLimit>;

  static object_ptr<getPremiumLimit> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class premiumState;

class getPremiumState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getPremiumState();

  static const std::int32_t ID = 663632610;

  using ReturnType = object_ptr<premiumState>;

  static object_ptr<getPremiumState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getPremiumStickerExamples final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getPremiumStickerExamples();

  static const std::int32_t ID = 1399442328;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getPremiumStickerExamples> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getPremiumStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 limit_;
  static jfieldID limit_fieldID;

  getPremiumStickers();

  explicit getPremiumStickers(int32 limit_);

  static const std::int32_t ID = -280950192;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getPremiumStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class preparedInlineMessage;

class getPreparedInlineMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string prepared_message_id_;
  static jfieldID prepared_message_id_fieldID;

  getPreparedInlineMessage();

  getPreparedInlineMessage(int53 bot_user_id_, string const &prepared_message_id_);

  static const std::int32_t ID = -83179701;

  using ReturnType = object_ptr<preparedInlineMessage>;

  static object_ptr<getPreparedInlineMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class proxies;

class getProxies final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getProxies();

  static const std::int32_t ID = -95026381;

  using ReturnType = object_ptr<proxies>;

  static object_ptr<getProxies> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getProxyLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 proxy_id_;
  static jfieldID proxy_id_fieldID;

  getProxyLink();

  explicit getProxyLink(int32 proxy_id_);

  static const std::int32_t ID = -1054495112;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getProxyLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class pushReceiverId;

class getPushReceiverId final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string payload_;
  static jfieldID payload_fieldID;

  getPushReceiverId();

  explicit getPushReceiverId(string const &payload_);

  static const std::int32_t ID = -286505294;

  using ReturnType = object_ptr<pushReceiverId>;

  static object_ptr<getPushReceiverId> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class readDatePrivacySettings;

class getReadDatePrivacySettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getReadDatePrivacySettings();

  static const std::int32_t ID = 451435451;

  using ReturnType = object_ptr<readDatePrivacySettings>;

  static object_ptr<getReadDatePrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class receivedGift;

class getReceivedGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;

  getReceivedGift();

  explicit getReceivedGift(string const &received_gift_id_);

  static const std::int32_t ID = -446535239;

  using ReturnType = object_ptr<receivedGift>;

  static object_ptr<getReceivedGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class receivedGifts;

class getReceivedGifts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  bool exclude_unsaved_;
  static jfieldID exclude_unsaved_fieldID;
  bool exclude_saved_;
  static jfieldID exclude_saved_fieldID;
  bool exclude_unlimited_;
  static jfieldID exclude_unlimited_fieldID;
  bool exclude_limited_;
  static jfieldID exclude_limited_fieldID;
  bool exclude_upgraded_;
  static jfieldID exclude_upgraded_fieldID;
  bool sort_by_price_;
  static jfieldID sort_by_price_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getReceivedGifts();

  getReceivedGifts(string const &business_connection_id_, object_ptr<MessageSender> &&owner_id_, bool exclude_unsaved_, bool exclude_saved_, bool exclude_unlimited_, bool exclude_limited_, bool exclude_upgraded_, bool sort_by_price_, string const &offset_, int32 limit_);

  static const std::int32_t ID = -586538672;

  using ReturnType = object_ptr<receivedGifts>;

  static object_ptr<getReceivedGifts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatuses;

class getRecentEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getRecentEmojiStatuses();

  static const std::int32_t ID = -1371914967;

  using ReturnType = object_ptr<emojiStatuses>;

  static object_ptr<getRecentEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class getRecentInlineBots final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getRecentInlineBots();

  static const std::int32_t ID = 1437823548;

  using ReturnType = object_ptr<users>;

  static object_ptr<getRecentInlineBots> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickers;

class getRecentStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_attached_;
  static jfieldID is_attached_fieldID;

  getRecentStickers();

  explicit getRecentStickers(bool is_attached_);

  static const std::int32_t ID = -579622241;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getRecentStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getRecentlyOpenedChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 limit_;
  static jfieldID limit_fieldID;

  getRecentlyOpenedChats();

  explicit getRecentlyOpenedChats(int32 limit_);

  static const std::int32_t ID = -1924156893;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getRecentlyOpenedChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class tMeUrls;

class getRecentlyVisitedTMeUrls final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string referrer_;
  static jfieldID referrer_fieldID;

  getRecentlyVisitedTMeUrls();

  explicit getRecentlyVisitedTMeUrls(string const &referrer_);

  static const std::int32_t ID = 806754961;

  using ReturnType = object_ptr<tMeUrls>;

  static object_ptr<getRecentlyVisitedTMeUrls> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class recommendedChatFolders;

class getRecommendedChatFolders final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getRecommendedChatFolders();

  static const std::int32_t ID = -145540217;

  using ReturnType = object_ptr<recommendedChatFolders>;

  static object_ptr<getRecommendedChatFolders> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getRecommendedChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getRecommendedChats();

  static const std::int32_t ID = -649884303;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getRecommendedChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class recoveryEmailAddress;

class getRecoveryEmailAddress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;

  getRecoveryEmailAddress();

  explicit getRecoveryEmailAddress(string const &password_);

  static const std::int32_t ID = -1594770947;

  using ReturnType = object_ptr<recoveryEmailAddress>;

  static object_ptr<getRecoveryEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class FileType;

class file;

class getRemoteFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string remote_file_id_;
  static jfieldID remote_file_id_fieldID;
  object_ptr<FileType> file_type_;
  static jfieldID file_type_fieldID;

  getRemoteFile();

  getRemoteFile(string const &remote_file_id_, object_ptr<FileType> &&file_type_);

  static const std::int32_t ID = 2137204530;

  using ReturnType = object_ptr<file>;

  static object_ptr<getRemoteFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getRepliedMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  getRepliedMessage();

  getRepliedMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -641918531;

  using ReturnType = object_ptr<message>;

  static object_ptr<getRepliedMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class animations;

class getSavedAnimations final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSavedAnimations();

  static const std::int32_t ID = 7051032;

  using ReturnType = object_ptr<animations>;

  static object_ptr<getSavedAnimations> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class savedMessagesTags;

class getSavedMessagesTags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  getSavedMessagesTags();

  explicit getSavedMessagesTags(int53 saved_messages_topic_id_);

  static const std::int32_t ID = -1932105815;

  using ReturnType = object_ptr<savedMessagesTags>;

  static object_ptr<getSavedMessagesTags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class getSavedMessagesTopicHistory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getSavedMessagesTopicHistory();

  getSavedMessagesTopicHistory(int53 saved_messages_topic_id_, int53 from_message_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = 2011552360;

  using ReturnType = object_ptr<messages>;

  static object_ptr<getSavedMessagesTopicHistory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class getSavedMessagesTopicMessageByDate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  int32 date_;
  static jfieldID date_fieldID;

  getSavedMessagesTopicMessageByDate();

  getSavedMessagesTopicMessageByDate(int53 saved_messages_topic_id_, int32 date_);

  static const std::int32_t ID = -1050786176;

  using ReturnType = object_ptr<message>;

  static object_ptr<getSavedMessagesTopicMessageByDate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class notificationSounds;

class getSavedNotificationSound final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 notification_sound_id_;
  static jfieldID notification_sound_id_fieldID;

  getSavedNotificationSound();

  explicit getSavedNotificationSound(int64 notification_sound_id_);

  static const std::int32_t ID = 459569431;

  using ReturnType = object_ptr<notificationSounds>;

  static object_ptr<getSavedNotificationSound> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class notificationSounds;

class getSavedNotificationSounds final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSavedNotificationSounds();

  static const std::int32_t ID = -1070305368;

  using ReturnType = object_ptr<notificationSounds>;

  static object_ptr<getSavedNotificationSounds> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class orderInfo;

class getSavedOrderInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSavedOrderInfo();

  static const std::int32_t ID = -1152016675;

  using ReturnType = object_ptr<orderInfo>;

  static object_ptr<getSavedOrderInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NotificationSettingsScope;

class scopeNotificationSettings;

class getScopeNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NotificationSettingsScope> scope_;
  static jfieldID scope_fieldID;

  getScopeNotificationSettings();

  explicit getScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_);

  static const std::int32_t ID = -995613361;

  using ReturnType = object_ptr<scopeNotificationSettings>;

  static object_ptr<getScopeNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class sponsoredChats;

class getSearchSponsoredChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  getSearchSponsoredChats();

  explicit getSearchSponsoredChats(string const &query_);

  static const std::int32_t ID = 1568505164;

  using ReturnType = object_ptr<sponsoredChats>;

  static object_ptr<getSearchSponsoredChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class hashtags;

class getSearchedForTags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string tag_prefix_;
  static jfieldID tag_prefix_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getSearchedForTags();

  getSearchedForTags(string const &tag_prefix_, int32 limit_);

  static const std::int32_t ID = -1692716851;

  using ReturnType = object_ptr<hashtags>;

  static object_ptr<getSearchedForTags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class secretChat;

class getSecretChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 secret_chat_id_;
  static jfieldID secret_chat_id_fieldID;

  getSecretChat();

  explicit getSecretChat(int32 secret_chat_id_);

  static const std::int32_t ID = 40599169;

  using ReturnType = object_ptr<secretChat>;

  static object_ptr<getSecretChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class httpUrl;

class getStarAdAccountUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;

  getStarAdAccountUrl();

  explicit getStarAdAccountUrl(object_ptr<MessageSender> &&owner_id_);

  static const std::int32_t ID = 1940473181;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getStarAdAccountUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starPaymentOptions;

class getStarGiftPaymentOptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getStarGiftPaymentOptions();

  explicit getStarGiftPaymentOptions(int53 user_id_);

  static const std::int32_t ID = -500735773;

  using ReturnType = object_ptr<starPaymentOptions>;

  static object_ptr<getStarGiftPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starGiveawayPaymentOptions;

class getStarGiveawayPaymentOptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getStarGiveawayPaymentOptions();

  static const std::int32_t ID = -883172578;

  using ReturnType = object_ptr<starGiveawayPaymentOptions>;

  static object_ptr<getStarGiveawayPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starPaymentOptions;

class getStarPaymentOptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getStarPaymentOptions();

  static const std::int32_t ID = 1838351940;

  using ReturnType = object_ptr<starPaymentOptions>;

  static object_ptr<getStarPaymentOptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class starRevenueStatistics;

class getStarRevenueStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;

  getStarRevenueStatistics();

  getStarRevenueStatistics(object_ptr<MessageSender> &&owner_id_, bool is_dark_);

  static const std::int32_t ID = -260356841;

  using ReturnType = object_ptr<starRevenueStatistics>;

  static object_ptr<getStarRevenueStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class starSubscriptions;

class getStarSubscriptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool only_expiring_;
  static jfieldID only_expiring_fieldID;
  string offset_;
  static jfieldID offset_fieldID;

  getStarSubscriptions();

  getStarSubscriptions(bool only_expiring_, string const &offset_);

  static const std::int32_t ID = -641223956;

  using ReturnType = object_ptr<starSubscriptions>;

  static object_ptr<getStarSubscriptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class StarTransactionDirection;

class starTransactions;

class getStarTransactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  string subscription_id_;
  static jfieldID subscription_id_fieldID;
  object_ptr<StarTransactionDirection> direction_;
  static jfieldID direction_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getStarTransactions();

  getStarTransactions(object_ptr<MessageSender> &&owner_id_, string const &subscription_id_, object_ptr<StarTransactionDirection> &&direction_, string const &offset_, int32 limit_);

  static const std::int32_t ID = -258541327;

  using ReturnType = object_ptr<starTransactions>;

  static object_ptr<getStarTransactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class httpUrl;

class getStarWithdrawalUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  string password_;
  static jfieldID password_fieldID;

  getStarWithdrawalUrl();

  getStarWithdrawalUrl(object_ptr<MessageSender> &&owner_id_, int53 star_count_, string const &password_);

  static const std::int32_t ID = -1445841134;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getStarWithdrawalUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StatisticalGraph;

class getStatisticalGraph final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string token_;
  static jfieldID token_fieldID;
  int53 x_;
  static jfieldID x_fieldID;

  getStatisticalGraph();

  getStatisticalGraph(int53 chat_id_, string const &token_, int53 x_);

  static const std::int32_t ID = 1100975515;

  using ReturnType = object_ptr<StatisticalGraph>;

  static object_ptr<getStatisticalGraph> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class emojis;

class getStickerEmojis final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  getStickerEmojis();

  explicit getStickerEmojis(object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = -1895508665;

  using ReturnType = object_ptr<emojis>;

  static object_ptr<getStickerEmojis> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class outline;

class getStickerOutline final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 sticker_file_id_;
  static jfieldID sticker_file_id_fieldID;
  bool for_animated_emoji_;
  static jfieldID for_animated_emoji_fieldID;
  bool for_clicked_animated_emoji_message_;
  static jfieldID for_clicked_animated_emoji_message_fieldID;

  getStickerOutline();

  getStickerOutline(int32 sticker_file_id_, bool for_animated_emoji_, bool for_clicked_animated_emoji_message_);

  static const std::int32_t ID = -1550504539;

  using ReturnType = object_ptr<outline>;

  static object_ptr<getStickerOutline> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerSet;

class getStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 set_id_;
  static jfieldID set_id_fieldID;

  getStickerSet();

  explicit getStickerSet(int64 set_id_);

  static const std::int32_t ID = 1052318659;

  using ReturnType = object_ptr<stickerSet>;

  static object_ptr<getStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getStickerSetName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 set_id_;
  static jfieldID set_id_fieldID;

  getStickerSetName();

  explicit getStickerSetName(int64 set_id_);

  static const std::int32_t ID = 1039849089;

  using ReturnType = object_ptr<text>;

  static object_ptr<getStickerSetName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickers;

class getStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getStickers();

  getStickers(object_ptr<StickerType> &&sticker_type_, string const &query_, int32 limit_, int53 chat_id_);

  static const std::int32_t ID = 1158058819;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<getStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storageStatistics;

class getStorageStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_limit_;
  static jfieldID chat_limit_fieldID;

  getStorageStatistics();

  explicit getStorageStatistics(int32 chat_limit_);

  static const std::int32_t ID = -853193929;

  using ReturnType = object_ptr<storageStatistics>;

  static object_ptr<getStorageStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storageStatisticsFast;

class getStorageStatisticsFast final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getStorageStatisticsFast();

  static const std::int32_t ID = 61368066;

  using ReturnType = object_ptr<storageStatisticsFast>;

  static object_ptr<getStorageStatisticsFast> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class story;

class getStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  bool only_local_;
  static jfieldID only_local_fieldID;

  getStory();

  getStory(int53 story_poster_chat_id_, int32 story_id_, bool only_local_);

  static const std::int32_t ID = -2011076366;

  using ReturnType = object_ptr<story>;

  static object_ptr<getStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class availableReactions;

class getStoryAvailableReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 row_size_;
  static jfieldID row_size_fieldID;

  getStoryAvailableReactions();

  explicit getStoryAvailableReactions(int32 row_size_);

  static const std::int32_t ID = 595938619;

  using ReturnType = object_ptr<availableReactions>;

  static object_ptr<getStoryAvailableReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyInteractions;

class getStoryInteractions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  bool only_contacts_;
  static jfieldID only_contacts_fieldID;
  bool prefer_forwards_;
  static jfieldID prefer_forwards_fieldID;
  bool prefer_with_reaction_;
  static jfieldID prefer_with_reaction_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getStoryInteractions();

  getStoryInteractions(int32 story_id_, string const &query_, bool only_contacts_, bool prefer_forwards_, bool prefer_with_reaction_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 483475469;

  using ReturnType = object_ptr<storyInteractions>;

  static object_ptr<getStoryInteractions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getStoryNotificationSettingsExceptions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getStoryNotificationSettingsExceptions();

  static const std::int32_t ID = 627715760;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getStoryNotificationSettingsExceptions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class publicForwards;

class getStoryPublicForwards final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getStoryPublicForwards();

  getStoryPublicForwards(int53 story_poster_chat_id_, int32 story_id_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 1810378546;

  using ReturnType = object_ptr<publicForwards>;

  static object_ptr<getStoryPublicForwards> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class storyStatistics;

class getStoryStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  bool is_dark_;
  static jfieldID is_dark_fieldID;

  getStoryStatistics();

  getStoryStatistics(int53 chat_id_, int32 story_id_, bool is_dark_);

  static const std::int32_t ID = 982926146;

  using ReturnType = object_ptr<storyStatistics>;

  static object_ptr<getStoryStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getSuggestedFileName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  string directory_;
  static jfieldID directory_fieldID;

  getSuggestedFileName();

  getSuggestedFileName(int32 file_id_, string const &directory_);

  static const std::int32_t ID = -2049399674;

  using ReturnType = object_ptr<text>;

  static object_ptr<getSuggestedFileName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getSuggestedStickerSetName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string title_;
  static jfieldID title_fieldID;

  getSuggestedStickerSetName();

  explicit getSuggestedStickerSetName(string const &title_);

  static const std::int32_t ID = -1340995520;

  using ReturnType = object_ptr<text>;

  static object_ptr<getSuggestedStickerSetName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getSuitableDiscussionChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSuitableDiscussionChats();

  static const std::int32_t ID = 49044982;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getSuitableDiscussionChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class getSuitablePersonalChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSuitablePersonalChats();

  static const std::int32_t ID = -1870357515;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getSuitablePersonalChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class supergroup;

class getSupergroup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  getSupergroup();

  explicit getSupergroup(int53 supergroup_id_);

  static const std::int32_t ID = 989663458;

  using ReturnType = object_ptr<supergroup>;

  static object_ptr<getSupergroup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class supergroupFullInfo;

class getSupergroupFullInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  getSupergroupFullInfo();

  explicit getSupergroupFullInfo(int53 supergroup_id_);

  static const std::int32_t ID = 1099776056;

  using ReturnType = object_ptr<supergroupFullInfo>;

  static object_ptr<getSupergroupFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SupergroupMembersFilter;

class chatMembers;

class getSupergroupMembers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  object_ptr<SupergroupMembersFilter> filter_;
  static jfieldID filter_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getSupergroupMembers();

  getSupergroupMembers(int53 supergroup_id_, object_ptr<SupergroupMembersFilter> &&filter_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -570940984;

  using ReturnType = object_ptr<chatMembers>;

  static object_ptr<getSupergroupMembers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class getSupportName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSupportName();

  static const std::int32_t ID = 1302205794;

  using ReturnType = object_ptr<text>;

  static object_ptr<getSupportName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class user;

class getSupportUser final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getSupportUser();

  static const std::int32_t ID = -1733497700;

  using ReturnType = object_ptr<user>;

  static object_ptr<getSupportUser> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class temporaryPasswordState;

class getTemporaryPasswordState final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getTemporaryPasswordState();

  static const std::int32_t ID = -12670830;

  using ReturnType = object_ptr<temporaryPasswordState>;

  static object_ptr<getTemporaryPasswordState> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class textEntities;

class getTextEntities final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;

  getTextEntities();

  explicit getTextEntities(string const &text_);

  static const std::int32_t ID = -341490693;

  using ReturnType = object_ptr<textEntities>;

  static object_ptr<getTextEntities> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class themeParameters;

class getThemeParametersJsonString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<themeParameters> theme_;
  static jfieldID theme_fieldID;

  getThemeParametersJsonString();

  explicit getThemeParametersJsonString(object_ptr<themeParameters> &&theme_);

  static const std::int32_t ID = -1850145288;

  using ReturnType = object_ptr<text>;

  static object_ptr<getThemeParametersJsonString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis;

class getThemedChatEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getThemedChatEmojiStatuses();

  static const std::int32_t ID = 1924568314;

  using ReturnType = object_ptr<emojiStatusCustomEmojis>;

  static object_ptr<getThemedChatEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatusCustomEmojis;

class getThemedEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getThemedEmojiStatuses();

  static const std::int32_t ID = -1468220543;

  using ReturnType = object_ptr<emojiStatusCustomEmojis>;

  static object_ptr<getThemedEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class timeZones;

class getTimeZones final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getTimeZones();

  static const std::int32_t ID = 1340268632;

  using ReturnType = object_ptr<timeZones>;

  static object_ptr<getTimeZones> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class TopChatCategory;

class chats;

class getTopChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<TopChatCategory> category_;
  static jfieldID category_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getTopChats();

  getTopChats(object_ptr<TopChatCategory> &&category_, int32 limit_);

  static const std::int32_t ID = -388410847;

  using ReturnType = object_ptr<chats>;

  static object_ptr<getTopChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class trendingStickerSets;

class getTrendingStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getTrendingStickerSets();

  getTrendingStickerSets(object_ptr<StickerType> &&sticker_type_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -531085986;

  using ReturnType = object_ptr<trendingStickerSets>;

  static object_ptr<getTrendingStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class upgradedGift;

class getUpgradedGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  getUpgradedGift();

  explicit getUpgradedGift(string const &name_);

  static const std::int32_t ID = -1331821135;

  using ReturnType = object_ptr<upgradedGift>;

  static object_ptr<getUpgradedGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatuses;

class getUpgradedGiftEmojiStatuses final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getUpgradedGiftEmojiStatuses();

  static const std::int32_t ID = -1748975723;

  using ReturnType = object_ptr<emojiStatuses>;

  static object_ptr<getUpgradedGiftEmojiStatuses> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getUpgradedGiftWithdrawalUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  string password_;
  static jfieldID password_fieldID;

  getUpgradedGiftWithdrawalUrl();

  getUpgradedGiftWithdrawalUrl(string const &received_gift_id_, string const &password_);

  static const std::int32_t ID = -784331188;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getUpgradedGiftWithdrawalUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class user;

class getUser final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getUser();

  explicit getUser(int53 user_id_);

  static const std::int32_t ID = 1117363211;

  using ReturnType = object_ptr<user>;

  static object_ptr<getUser> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundChatBoosts;

class getUserChatBoosts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getUserChatBoosts();

  getUserChatBoosts(int53 chat_id_, int53 user_id_);

  static const std::int32_t ID = -1190205543;

  using ReturnType = object_ptr<foundChatBoosts>;

  static object_ptr<getUserChatBoosts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userFullInfo;

class getUserFullInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getUserFullInfo();

  explicit getUserFullInfo(int53 user_id_);

  static const std::int32_t ID = -776823720;

  using ReturnType = object_ptr<userFullInfo>;

  static object_ptr<getUserFullInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userLink;

class getUserLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  getUserLink();

  static const std::int32_t ID = 1226839270;

  using ReturnType = object_ptr<userLink>;

  static object_ptr<getUserLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class UserPrivacySetting;

class userPrivacySettingRules;

class getUserPrivacySettingRules final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<UserPrivacySetting> setting_;
  static jfieldID setting_fieldID;

  getUserPrivacySettingRules();

  explicit getUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_);

  static const std::int32_t ID = -2077223311;

  using ReturnType = object_ptr<userPrivacySettingRules>;

  static object_ptr<getUserPrivacySettingRules> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatPhotos;

class getUserProfilePhotos final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  getUserProfilePhotos();

  getUserProfilePhotos(int53 user_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -908132798;

  using ReturnType = object_ptr<chatPhotos>;

  static object_ptr<getUserProfilePhotos> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class userSupportInfo;

class getUserSupportInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  getUserSupportInfo();

  explicit getUserSupportInfo(int53 user_id_);

  static const std::int32_t ID = 1957008133;

  using ReturnType = object_ptr<userSupportInfo>;

  static object_ptr<getUserSupportInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageSenders;

class getVideoChatAvailableParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getVideoChatAvailableParticipants();

  explicit getVideoChatAvailableParticipants(int53 chat_id_);

  static const std::int32_t ID = -1000496379;

  using ReturnType = object_ptr<messageSenders>;

  static object_ptr<getVideoChatAvailableParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class getVideoChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool can_self_unmute_;
  static jfieldID can_self_unmute_fieldID;

  getVideoChatInviteLink();

  getVideoChatInviteLink(int32 group_call_id_, bool can_self_unmute_);

  static const std::int32_t ID = -1394707321;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getVideoChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class rtmpUrl;

class getVideoChatRtmpUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  getVideoChatRtmpUrl();

  explicit getVideoChatRtmpUrl(int53 chat_id_);

  static const std::int32_t ID = 1210784543;

  using ReturnType = object_ptr<rtmpUrl>;

  static object_ptr<getVideoChatRtmpUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class GroupCallVideoQuality;

class data;

class getVideoChatStreamSegment final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int53 time_offset_;
  static jfieldID time_offset_fieldID;
  int32 scale_;
  static jfieldID scale_fieldID;
  int32 channel_id_;
  static jfieldID channel_id_fieldID;
  object_ptr<GroupCallVideoQuality> video_quality_;
  static jfieldID video_quality_fieldID;

  getVideoChatStreamSegment();

  getVideoChatStreamSegment(int32 group_call_id_, int53 time_offset_, int32 scale_, int32 channel_id_, object_ptr<GroupCallVideoQuality> &&video_quality_);

  static const std::int32_t ID = 773343220;

  using ReturnType = object_ptr<data>;

  static object_ptr<getVideoChatStreamSegment> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class videoChatStreams;

class getVideoChatStreams final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  getVideoChatStreams();

  explicit getVideoChatStreams(int32 group_call_id_);

  static const std::int32_t ID = 531757765;

  using ReturnType = object_ptr<videoChatStreams>;

  static object_ptr<getVideoChatStreams> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class webAppOpenParameters;

class getWebAppLinkUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string web_app_short_name_;
  static jfieldID web_app_short_name_fieldID;
  string start_parameter_;
  static jfieldID start_parameter_fieldID;
  bool allow_write_access_;
  static jfieldID allow_write_access_fieldID;
  object_ptr<webAppOpenParameters> parameters_;
  static jfieldID parameters_fieldID;

  getWebAppLinkUrl();

  getWebAppLinkUrl(int53 chat_id_, int53 bot_user_id_, string const &web_app_short_name_, string const &start_parameter_, bool allow_write_access_, object_ptr<webAppOpenParameters> &&parameters_);

  static const std::int32_t ID = 1627284161;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getWebAppLinkUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class outline;

class getWebAppPlaceholder final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;

  getWebAppPlaceholder();

  explicit getWebAppPlaceholder(int53 bot_user_id_);

  static const std::int32_t ID = 583470479;

  using ReturnType = object_ptr<outline>;

  static object_ptr<getWebAppPlaceholder> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class httpUrl;

class webAppOpenParameters;

class getWebAppUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string url_;
  static jfieldID url_fieldID;
  object_ptr<webAppOpenParameters> parameters_;
  static jfieldID parameters_fieldID;

  getWebAppUrl();

  getWebAppUrl(int53 bot_user_id_, string const &url_, object_ptr<webAppOpenParameters> &&parameters_);

  static const std::int32_t ID = -1526784188;

  using ReturnType = object_ptr<httpUrl>;

  static object_ptr<getWebAppUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class webPageInstantView;

class getWebPageInstantView final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string url_;
  static jfieldID url_fieldID;
  bool only_local_;
  static jfieldID only_local_fieldID;

  getWebPageInstantView();

  getWebPageInstantView(string const &url_, bool only_local_);

  static const std::int32_t ID = 1741395197;

  using ReturnType = object_ptr<webPageInstantView>;

  static object_ptr<getWebPageInstantView> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class ok;

class giftPremiumWithStars final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;
  int32 month_count_;
  static jfieldID month_count_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  giftPremiumWithStars();

  giftPremiumWithStars(int53 user_id_, int53 star_count_, int32 month_count_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -2058395432;

  using ReturnType = object_ptr<ok>;

  static object_ptr<giftPremiumWithStars> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class hideContactCloseBirthdays final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  hideContactCloseBirthdays();

  static const std::int32_t ID = -1163065221;

  using ReturnType = object_ptr<ok>;

  static object_ptr<hideContactCloseBirthdays> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SuggestedAction;

class ok;

class hideSuggestedAction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<SuggestedAction> action_;
  static jfieldID action_fieldID;

  hideSuggestedAction();

  explicit hideSuggestedAction(object_ptr<SuggestedAction> &&action_);

  static const std::int32_t ID = -1561384065;

  using ReturnType = object_ptr<ok>;

  static object_ptr<hideSuggestedAction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class contact;

class importedContacts;

class importContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<contact>> contacts_;
  static jfieldID contacts_fieldID;

  importContacts();

  explicit importContacts(array<object_ptr<contact>> &&contacts_);

  static const std::int32_t ID = -215132767;

  using ReturnType = object_ptr<importedContacts>;

  static object_ptr<importContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class importMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputFile> message_file_;
  static jfieldID message_file_fieldID;
  array<object_ptr<InputFile>> attached_files_;
  static jfieldID attached_files_fieldID;

  importMessages();

  importMessages(int53 chat_id_, object_ptr<InputFile> &&message_file_, array<object_ptr<InputFile>> &&attached_files_);

  static const std::int32_t ID = -1864116784;

  using ReturnType = object_ptr<ok>;

  static object_ptr<importMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InviteGroupCallParticipantResult;

class inviteGroupCallParticipant final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool is_video_;
  static jfieldID is_video_fieldID;

  inviteGroupCallParticipant();

  inviteGroupCallParticipant(int32 group_call_id_, int53 user_id_, bool is_video_);

  static const std::int32_t ID = -631535414;

  using ReturnType = object_ptr<InviteGroupCallParticipantResult>;

  static object_ptr<inviteGroupCallParticipant> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class inviteVideoChatParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  inviteVideoChatParticipants();

  inviteVideoChatParticipants(int32 group_call_id_, array<int53> &&user_ids_);

  static const std::int32_t ID = -473149298;

  using ReturnType = object_ptr<ok>;

  static object_ptr<inviteVideoChatParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class joinChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  joinChat();

  explicit joinChat(int53 chat_id_);

  static const std::int32_t ID = 326769313;

  using ReturnType = object_ptr<ok>;

  static object_ptr<joinChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class joinChatByInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  joinChatByInviteLink();

  explicit joinChatByInviteLink(string const &invite_link_);

  static const std::int32_t ID = -1049973882;

  using ReturnType = object_ptr<chat>;

  static object_ptr<joinChatByInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputGroupCall;

class groupCallInfo;

class groupCallJoinParameters;

class joinGroupCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputGroupCall> input_group_call_;
  static jfieldID input_group_call_fieldID;
  object_ptr<groupCallJoinParameters> join_parameters_;
  static jfieldID join_parameters_fieldID;

  joinGroupCall();

  joinGroupCall(object_ptr<InputGroupCall> &&input_group_call_, object_ptr<groupCallJoinParameters> &&join_parameters_);

  static const std::int32_t ID = -2026079917;

  using ReturnType = object_ptr<groupCallInfo>;

  static object_ptr<joinGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class groupCallJoinParameters;

class text;

class joinVideoChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  object_ptr<groupCallJoinParameters> join_parameters_;
  static jfieldID join_parameters_fieldID;
  string invite_hash_;
  static jfieldID invite_hash_fieldID;

  joinVideoChat();

  joinVideoChat(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, object_ptr<groupCallJoinParameters> &&join_parameters_, string const &invite_hash_);

  static const std::int32_t ID = 1322989999;

  using ReturnType = object_ptr<text>;

  static object_ptr<joinVideoChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giveawayParameters;

class ok;

class launchPrepaidGiveaway final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 giveaway_id_;
  static jfieldID giveaway_id_fieldID;
  object_ptr<giveawayParameters> parameters_;
  static jfieldID parameters_fieldID;
  int32 winner_count_;
  static jfieldID winner_count_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  launchPrepaidGiveaway();

  launchPrepaidGiveaway(int64 giveaway_id_, object_ptr<giveawayParameters> &&parameters_, int32 winner_count_, int53 star_count_);

  static const std::int32_t ID = 639465530;

  using ReturnType = object_ptr<ok>;

  static object_ptr<launchPrepaidGiveaway> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class leaveChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  leaveChat();

  explicit leaveChat(int53 chat_id_);

  static const std::int32_t ID = -1825080735;

  using ReturnType = object_ptr<ok>;

  static object_ptr<leaveChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class leaveGroupCall final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  leaveGroupCall();

  explicit leaveGroupCall(int32 group_call_id_);

  static const std::int32_t ID = 980152233;

  using ReturnType = object_ptr<ok>;

  static object_ptr<leaveGroupCall> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StoryList;

class ok;

class loadActiveStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StoryList> story_list_;
  static jfieldID story_list_fieldID;

  loadActiveStories();

  explicit loadActiveStories(object_ptr<StoryList> &&story_list_);

  static const std::int32_t ID = 2106390328;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadActiveStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ok;

class loadChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  loadChats();

  loadChats(object_ptr<ChatList> &&chat_list_, int32 limit_);

  static const std::int32_t ID = -1885635205;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class loadGroupCallParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  loadGroupCallParticipants();

  loadGroupCallParticipants(int32 group_call_id_, int32 limit_);

  static const std::int32_t ID = 938720974;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadGroupCallParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class loadQuickReplyShortcutMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;

  loadQuickReplyShortcutMessages();

  explicit loadQuickReplyShortcutMessages(int32 shortcut_id_);

  static const std::int32_t ID = -46092588;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadQuickReplyShortcutMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class loadQuickReplyShortcuts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  loadQuickReplyShortcuts();

  static const std::int32_t ID = -1016614243;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadQuickReplyShortcuts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class loadSavedMessagesTopics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 limit_;
  static jfieldID limit_fieldID;

  loadSavedMessagesTopics();

  explicit loadSavedMessagesTopics(int32 limit_);

  static const std::int32_t ID = 289855160;

  using ReturnType = object_ptr<ok>;

  static object_ptr<loadSavedMessagesTopics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class logOut final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  logOut();

  static const std::int32_t ID = -1581923301;

  using ReturnType = object_ptr<ok>;

  static object_ptr<logOut> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openBotSimilarBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  int53 opened_bot_user_id_;
  static jfieldID opened_bot_user_id_fieldID;

  openBotSimilarBot();

  openBotSimilarBot(int53 bot_user_id_, int53 opened_bot_user_id_);

  static const std::int32_t ID = -369688872;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openBotSimilarBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  openChat();

  explicit openChat(int53 chat_id_);

  static const std::int32_t ID = -323371509;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openChatSimilarChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 opened_chat_id_;
  static jfieldID opened_chat_id_fieldID;

  openChatSimilarChat();

  openChatSimilarChat(int53 chat_id_, int53 opened_chat_id_);

  static const std::int32_t ID = -1884883949;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openChatSimilarChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openMessageContent final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  openMessageContent();

  openMessageContent(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -739088005;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openMessageContent> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openSponsoredChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sponsored_chat_unique_id_;
  static jfieldID sponsored_chat_unique_id_fieldID;

  openSponsoredChat();

  explicit openSponsoredChat(int53 sponsored_chat_unique_id_);

  static const std::int32_t ID = 865985573;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openSponsoredChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class openStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;

  openStory();

  openStory(int53 story_poster_chat_id_, int32 story_id_);

  static const std::int32_t ID = -696723005;

  using ReturnType = object_ptr<ok>;

  static object_ptr<openStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageReplyTo;

class webAppInfo;

class webAppOpenParameters;

class openWebApp final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string url_;
  static jfieldID url_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  object_ptr<webAppOpenParameters> parameters_;
  static jfieldID parameters_fieldID;

  openWebApp();

  openWebApp(int53 chat_id_, int53 bot_user_id_, string const &url_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<webAppOpenParameters> &&parameters_);

  static const std::int32_t ID = 662795170;

  using ReturnType = object_ptr<webAppInfo>;

  static object_ptr<openWebApp> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class FileType;

class storageStatistics;

class optimizeStorage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 size_;
  static jfieldID size_fieldID;
  int32 ttl_;
  static jfieldID ttl_fieldID;
  int32 count_;
  static jfieldID count_fieldID;
  int32 immunity_delay_;
  static jfieldID immunity_delay_fieldID;
  array<object_ptr<FileType>> file_types_;
  static jfieldID file_types_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;
  array<int53> exclude_chat_ids_;
  static jfieldID exclude_chat_ids_fieldID;
  bool return_deleted_file_statistics_;
  static jfieldID return_deleted_file_statistics_fieldID;
  int32 chat_limit_;
  static jfieldID chat_limit_fieldID;

  optimizeStorage();

  optimizeStorage(int53 size_, int32 ttl_, int32 count_, int32 immunity_delay_, array<object_ptr<FileType>> &&file_types_, array<int53> &&chat_ids_, array<int53> &&exclude_chat_ids_, bool return_deleted_file_statistics_, int32 chat_limit_);

  static const std::int32_t ID = 853186759;

  using ReturnType = object_ptr<storageStatistics>;

  static object_ptr<optimizeStorage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class parseMarkdown final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  parseMarkdown();

  explicit parseMarkdown(object_ptr<formattedText> &&text_);

  static const std::int32_t ID = 756366063;

  using ReturnType = object_ptr<formattedText>;

  static object_ptr<parseMarkdown> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class TextParseMode;

class formattedText;

class parseTextEntities final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  object_ptr<TextParseMode> parse_mode_;
  static jfieldID parse_mode_fieldID;

  parseTextEntities();

  parseTextEntities(string const &text_, object_ptr<TextParseMode> &&parse_mode_);

  static const std::int32_t ID = -1709194593;

  using ReturnType = object_ptr<formattedText>;

  static object_ptr<parseTextEntities> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class pinChatMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;
  bool only_for_self_;
  static jfieldID only_for_self_fieldID;

  pinChatMessage();

  pinChatMessage(int53 chat_id_, int53 message_id_, bool disable_notification_, bool only_for_self_);

  static const std::int32_t ID = 2034719663;

  using ReturnType = object_ptr<ok>;

  static object_ptr<pinChatMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class seconds;

class pingProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 proxy_id_;
  static jfieldID proxy_id_fieldID;

  pingProxy();

  explicit pingProxy(int32 proxy_id_);

  static const std::int32_t ID = -979681103;

  using ReturnType = object_ptr<seconds>;

  static object_ptr<pingProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputStoryContent;

class StoryPrivacySettings;

class formattedText;

class inputStoryAreas;

class story;

class storyFullId;

class postStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputStoryContent> content_;
  static jfieldID content_fieldID;
  object_ptr<inputStoryAreas> areas_;
  static jfieldID areas_fieldID;
  object_ptr<formattedText> caption_;
  static jfieldID caption_fieldID;
  object_ptr<StoryPrivacySettings> privacy_settings_;
  static jfieldID privacy_settings_fieldID;
  int32 active_period_;
  static jfieldID active_period_fieldID;
  object_ptr<storyFullId> from_story_full_id_;
  static jfieldID from_story_full_id_fieldID;
  bool is_posted_to_chat_page_;
  static jfieldID is_posted_to_chat_page_fieldID;
  bool protect_content_;
  static jfieldID protect_content_fieldID;

  postStory();

  postStory(int53 chat_id_, object_ptr<InputStoryContent> &&content_, object_ptr<inputStoryAreas> &&areas_, object_ptr<formattedText> &&caption_, object_ptr<StoryPrivacySettings> &&privacy_settings_, int32 active_period_, object_ptr<storyFullId> &&from_story_full_id_, bool is_posted_to_chat_page_, bool protect_content_);

  static const std::int32_t ID = -257605513;

  using ReturnType = object_ptr<story>;

  static object_ptr<postStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class FileType;

class InputFile;

class file;

class preliminaryUploadFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> file_;
  static jfieldID file_fieldID;
  object_ptr<FileType> file_type_;
  static jfieldID file_type_fieldID;
  int32 priority_;
  static jfieldID priority_fieldID;

  preliminaryUploadFile();

  preliminaryUploadFile(object_ptr<InputFile> &&file_, object_ptr<FileType> &&file_type_, int32 priority_);

  static const std::int32_t ID = 1894239129;

  using ReturnType = object_ptr<file>;

  static object_ptr<preliminaryUploadFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class processChatFolderNewChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 chat_folder_id_;
  static jfieldID chat_folder_id_fieldID;
  array<int53> added_chat_ids_;
  static jfieldID added_chat_ids_fieldID;

  processChatFolderNewChats();

  processChatFolderNewChats(int32 chat_folder_id_, array<int53> &&added_chat_ids_);

  static const std::int32_t ID = 1498280672;

  using ReturnType = object_ptr<ok>;

  static object_ptr<processChatFolderNewChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class processChatJoinRequest final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  bool approve_;
  static jfieldID approve_fieldID;

  processChatJoinRequest();

  processChatJoinRequest(int53 chat_id_, int53 user_id_, bool approve_);

  static const std::int32_t ID = 1004876963;

  using ReturnType = object_ptr<ok>;

  static object_ptr<processChatJoinRequest> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class processChatJoinRequests final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;
  bool approve_;
  static jfieldID approve_fieldID;

  processChatJoinRequests();

  processChatJoinRequests(int53 chat_id_, string const &invite_link_, bool approve_);

  static const std::int32_t ID = 1048722894;

  using ReturnType = object_ptr<ok>;

  static object_ptr<processChatJoinRequests> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class processPushNotification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string payload_;
  static jfieldID payload_fieldID;

  processPushNotification();

  explicit processPushNotification(string const &payload_);

  static const std::int32_t ID = 786679952;

  using ReturnType = object_ptr<ok>;

  static object_ptr<processPushNotification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class rateSpeechRecognition final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_good_;
  static jfieldID is_good_fieldID;

  rateSpeechRecognition();

  rateSpeechRecognition(int53 chat_id_, int53 message_id_, bool is_good_);

  static const std::int32_t ID = -287521867;

  using ReturnType = object_ptr<ok>;

  static object_ptr<rateSpeechRecognition> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readAllChatMentions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  readAllChatMentions();

  explicit readAllChatMentions(int53 chat_id_);

  static const std::int32_t ID = 1357558453;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readAllChatMentions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readAllChatReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  readAllChatReactions();

  explicit readAllChatReactions(int53 chat_id_);

  static const std::int32_t ID = 1421973357;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readAllChatReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readAllMessageThreadMentions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  readAllMessageThreadMentions();

  readAllMessageThreadMentions(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = 1323136341;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readAllMessageThreadMentions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readAllMessageThreadReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  readAllMessageThreadReactions();

  readAllMessageThreadReactions(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = -792975554;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readAllMessageThreadReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readBusinessMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  readBusinessMessage();

  readBusinessMessage(string const &business_connection_id_, int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = -1723531538;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readBusinessMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ok;

class readChatList final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;

  readChatList();

  explicit readChatList(object_ptr<ChatList> &&chat_list_);

  static const std::int32_t ID = -1117480790;

  using ReturnType = object_ptr<ok>;

  static object_ptr<readChatList> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class data;

class readFilePart final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  int53 offset_;
  static jfieldID offset_fieldID;
  int53 count_;
  static jfieldID count_fieldID;

  readFilePart();

  readFilePart(int32 file_id_, int53 offset_, int53 count_);

  static const std::int32_t ID = -174576822;

  using ReturnType = object_ptr<data>;

  static object_ptr<readFilePart> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class quickReplyMessages;

class readdQuickReplyShortcutMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string shortcut_name_;
  static jfieldID shortcut_name_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  readdQuickReplyShortcutMessages();

  readdQuickReplyShortcutMessages(string const &shortcut_name_, array<int53> &&message_ids_);

  static const std::int32_t ID = 387399566;

  using ReturnType = object_ptr<quickReplyMessages>;

  static object_ptr<readdQuickReplyShortcutMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class recognizeSpeech final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  recognizeSpeech();

  recognizeSpeech(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1741947577;

  using ReturnType = object_ptr<ok>;

  static object_ptr<recognizeSpeech> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class recoverAuthenticationPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string recovery_code_;
  static jfieldID recovery_code_fieldID;
  string new_password_;
  static jfieldID new_password_fieldID;
  string new_hint_;
  static jfieldID new_hint_fieldID;

  recoverAuthenticationPassword();

  recoverAuthenticationPassword(string const &recovery_code_, string const &new_password_, string const &new_hint_);

  static const std::int32_t ID = -131001053;

  using ReturnType = object_ptr<ok>;

  static object_ptr<recoverAuthenticationPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class recoverPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string recovery_code_;
  static jfieldID recovery_code_fieldID;
  string new_password_;
  static jfieldID new_password_fieldID;
  string new_hint_;
  static jfieldID new_hint_fieldID;

  recoverPassword();

  recoverPassword(string const &recovery_code_, string const &new_password_, string const &new_hint_);

  static const std::int32_t ID = -1524262541;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<recoverPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class refundStarPayment final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string telegram_payment_charge_id_;
  static jfieldID telegram_payment_charge_id_fieldID;

  refundStarPayment();

  refundStarPayment(int53 user_id_, string const &telegram_payment_charge_id_);

  static const std::int32_t ID = -1804165035;

  using ReturnType = object_ptr<ok>;

  static object_ptr<refundStarPayment> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class DeviceToken;

class pushReceiverId;

class registerDevice final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<DeviceToken> device_token_;
  static jfieldID device_token_fieldID;
  array<int53> other_user_ids_;
  static jfieldID other_user_ids_fieldID;

  registerDevice();

  registerDevice(object_ptr<DeviceToken> &&device_token_, array<int53> &&other_user_ids_);

  static const std::int32_t ID = 366088823;

  using ReturnType = object_ptr<pushReceiverId>;

  static object_ptr<registerDevice> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class registerUser final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;

  registerUser();

  registerUser(string const &first_name_, string const &last_name_, bool disable_notification_);

  static const std::int32_t ID = -1012247828;

  using ReturnType = object_ptr<ok>;

  static object_ptr<registerUser> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeAllFilesFromDownloads final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool only_active_;
  static jfieldID only_active_fieldID;
  bool only_completed_;
  static jfieldID only_completed_fieldID;
  bool delete_from_cache_;
  static jfieldID delete_from_cache_fieldID;

  removeAllFilesFromDownloads();

  removeAllFilesFromDownloads(bool only_active_, bool only_completed_, bool delete_from_cache_);

  static const std::int32_t ID = -1186433402;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeAllFilesFromDownloads> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeBusinessConnectedBotFromChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  removeBusinessConnectedBotFromChat();

  explicit removeBusinessConnectedBotFromChat(int53 chat_id_);

  static const std::int32_t ID = 2020766707;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeBusinessConnectedBotFromChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeChatActionBar final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  removeChatActionBar();

  explicit removeChatActionBar(int53 chat_id_);

  static const std::int32_t ID = -1650968070;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeChatActionBar> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  removeContacts();

  explicit removeContacts(array<int53> &&user_ids_);

  static const std::int32_t ID = 1943858054;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class removeFavoriteSticker final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  removeFavoriteSticker();

  explicit removeFavoriteSticker(object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = 1152945264;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeFavoriteSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeFileFromDownloads final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  bool delete_from_cache_;
  static jfieldID delete_from_cache_fieldID;

  removeFileFromDownloads();

  removeFileFromDownloads(int32 file_id_, bool delete_from_cache_);

  static const std::int32_t ID = 1460060142;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeFileFromDownloads> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeInstalledBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 background_id_;
  static jfieldID background_id_fieldID;

  removeInstalledBackground();

  explicit removeInstalledBackground(int64 background_id_);

  static const std::int32_t ID = 1346446652;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeInstalledBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class removeMessageReaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;

  removeMessageReaction();

  removeMessageReaction(int53 chat_id_, int53 message_id_, object_ptr<ReactionType> &&reaction_type_);

  static const std::int32_t ID = -1756934789;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeMessageReaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class removeMessageSenderBotVerification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<MessageSender> verified_id_;
  static jfieldID verified_id_fieldID;

  removeMessageSenderBotVerification();

  removeMessageSenderBotVerification(int53 bot_user_id_, object_ptr<MessageSender> &&verified_id_);

  static const std::int32_t ID = -1710174374;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeMessageSenderBotVerification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeNotification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 notification_group_id_;
  static jfieldID notification_group_id_fieldID;
  int32 notification_id_;
  static jfieldID notification_id_fieldID;

  removeNotification();

  removeNotification(int32 notification_group_id_, int32 notification_id_);

  static const std::int32_t ID = 862630734;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeNotification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeNotificationGroup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 notification_group_id_;
  static jfieldID notification_group_id_fieldID;
  int32 max_notification_id_;
  static jfieldID max_notification_id_fieldID;

  removeNotificationGroup();

  removeNotificationGroup(int32 notification_group_id_, int32 max_notification_id_);

  static const std::int32_t ID = 1713005454;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeNotificationGroup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removePendingPaidMessageReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  removePendingPaidMessageReactions();

  removePendingPaidMessageReactions(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 1100258555;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removePendingPaidMessageReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 proxy_id_;
  static jfieldID proxy_id_fieldID;

  removeProxy();

  explicit removeProxy(int32 proxy_id_);

  static const std::int32_t ID = 1369219847;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeRecentHashtag final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string hashtag_;
  static jfieldID hashtag_fieldID;

  removeRecentHashtag();

  explicit removeRecentHashtag(string const &hashtag_);

  static const std::int32_t ID = -1013735260;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeRecentHashtag> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class removeRecentSticker final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool is_attached_;
  static jfieldID is_attached_fieldID;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  removeRecentSticker();

  removeRecentSticker(bool is_attached_, object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = 1246577677;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeRecentSticker> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeRecentlyFoundChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  removeRecentlyFoundChat();

  explicit removeRecentlyFoundChat(int53 chat_id_);

  static const std::int32_t ID = 717340444;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeRecentlyFoundChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class removeSavedAnimation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> animation_;
  static jfieldID animation_fieldID;

  removeSavedAnimation();

  explicit removeSavedAnimation(object_ptr<InputFile> &&animation_);

  static const std::int32_t ID = -495605479;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeSavedAnimation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeSavedNotificationSound final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 notification_sound_id_;
  static jfieldID notification_sound_id_fieldID;

  removeSavedNotificationSound();

  explicit removeSavedNotificationSound(int64 notification_sound_id_);

  static const std::int32_t ID = -480032946;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeSavedNotificationSound> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class removeSearchedForTag final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string tag_;
  static jfieldID tag_fieldID;

  removeSearchedForTag();

  explicit removeSearchedForTag(string const &tag_);

  static const std::int32_t ID = 891382730;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeSearchedForTag> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class removeStickerFromSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  removeStickerFromSet();

  explicit removeStickerFromSet(object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = 1642196644;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeStickerFromSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class TopChatCategory;

class ok;

class removeTopChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<TopChatCategory> category_;
  static jfieldID category_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  removeTopChat();

  removeTopChat(object_ptr<TopChatCategory> &&category_, int53 chat_id_);

  static const std::int32_t ID = -1907876267;

  using ReturnType = object_ptr<ok>;

  static object_ptr<removeTopChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderActiveUsernames final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> usernames_;
  static jfieldID usernames_fieldID;

  reorderActiveUsernames();

  explicit reorderActiveUsernames(array<string> &&usernames_);

  static const std::int32_t ID = -455399375;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderActiveUsernames> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderBotActiveUsernames final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  array<string> usernames_;
  static jfieldID usernames_fieldID;

  reorderBotActiveUsernames();

  reorderBotActiveUsernames(int53 bot_user_id_, array<string> &&usernames_);

  static const std::int32_t ID = -1602301664;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderBotActiveUsernames> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderBotMediaPreviews final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  array<int32> file_ids_;
  static jfieldID file_ids_fieldID;

  reorderBotMediaPreviews();

  reorderBotMediaPreviews(int53 bot_user_id_, string const &language_code_, array<int32> &&file_ids_);

  static const std::int32_t ID = 630851043;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderBotMediaPreviews> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderChatFolders final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> chat_folder_ids_;
  static jfieldID chat_folder_ids_fieldID;
  int32 main_chat_list_position_;
  static jfieldID main_chat_list_position_fieldID;

  reorderChatFolders();

  reorderChatFolders(array<int32> &&chat_folder_ids_, int32 main_chat_list_position_);

  static const std::int32_t ID = 1665299546;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderChatFolders> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class ok;

class reorderInstalledStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  array<int64> sticker_set_ids_;
  static jfieldID sticker_set_ids_fieldID;

  reorderInstalledStickerSets();

  reorderInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, array<int64> &&sticker_set_ids_);

  static const std::int32_t ID = 1074928158;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderInstalledStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderQuickReplyShortcuts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> shortcut_ids_;
  static jfieldID shortcut_ids_fieldID;

  reorderQuickReplyShortcuts();

  explicit reorderQuickReplyShortcuts(array<int32> &&shortcut_ids_);

  static const std::int32_t ID = -2052799232;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderQuickReplyShortcuts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reorderSupergroupActiveUsernames final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  array<string> usernames_;
  static jfieldID usernames_fieldID;

  reorderSupergroupActiveUsernames();

  reorderSupergroupActiveUsernames(int53 supergroup_id_, array<string> &&usernames_);

  static const std::int32_t ID = -1962466095;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reorderSupergroupActiveUsernames> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLink;

class replacePrimaryChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  replacePrimaryChatInviteLink();

  explicit replacePrimaryChatInviteLink(int53 chat_id_);

  static const std::int32_t ID = 1067350941;

  using ReturnType = object_ptr<chatInviteLink>;

  static object_ptr<replacePrimaryChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class inputSticker;

class ok;

class replaceStickerInSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<InputFile> old_sticker_;
  static jfieldID old_sticker_fieldID;
  object_ptr<inputSticker> new_sticker_;
  static jfieldID new_sticker_fieldID;

  replaceStickerInSet();

  replaceStickerInSet(int53 user_id_, string const &name_, object_ptr<InputFile> &&old_sticker_, object_ptr<inputSticker> &&new_sticker_);

  static const std::int32_t ID = -406311399;

  using ReturnType = object_ptr<ok>;

  static object_ptr<replaceStickerInSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class rtmpUrl;

class replaceVideoChatRtmpUrl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  replaceVideoChatRtmpUrl();

  explicit replaceVideoChatRtmpUrl(int53 chat_id_);

  static const std::int32_t ID = 558862304;

  using ReturnType = object_ptr<rtmpUrl>;

  static object_ptr<replaceVideoChatRtmpUrl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reportAuthenticationCodeMissing final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string mobile_network_code_;
  static jfieldID mobile_network_code_fieldID;

  reportAuthenticationCodeMissing();

  explicit reportAuthenticationCodeMissing(string const &mobile_network_code_);

  static const std::int32_t ID = -1846555064;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportAuthenticationCodeMissing> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReportChatResult;

class reportChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bytes option_id_;
  static jfieldID option_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  string text_;
  static jfieldID text_fieldID;

  reportChat();

  reportChat(int53 chat_id_, bytes const &option_id_, array<int53> &&message_ids_, string const &text_);

  static const std::int32_t ID = 1058475058;

  using ReturnType = object_ptr<ReportChatResult>;

  static object_ptr<reportChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReportReason;

class ok;

class reportChatPhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  object_ptr<ReportReason> reason_;
  static jfieldID reason_fieldID;
  string text_;
  static jfieldID text_fieldID;

  reportChatPhoto();

  reportChatPhoto(int53 chat_id_, int32 file_id_, object_ptr<ReportReason> &&reason_, string const &text_);

  static const std::int32_t ID = -646966648;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReportSponsoredResult;

class reportChatSponsoredMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bytes option_id_;
  static jfieldID option_id_fieldID;

  reportChatSponsoredMessage();

  reportChatSponsoredMessage(int53 chat_id_, int53 message_id_, bytes const &option_id_);

  static const std::int32_t ID = -979984820;

  using ReturnType = object_ptr<ReportSponsoredResult>;

  static object_ptr<reportChatSponsoredMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class reportMessageReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;

  reportMessageReactions();

  reportMessageReactions(int53 chat_id_, int53 message_id_, object_ptr<MessageSender> &&sender_id_);

  static const std::int32_t ID = 919111719;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportMessageReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reportPhoneNumberCodeMissing final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string mobile_network_code_;
  static jfieldID mobile_network_code_fieldID;

  reportPhoneNumberCodeMissing();

  explicit reportPhoneNumberCodeMissing(string const &mobile_network_code_);

  static const std::int32_t ID = -895175341;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportPhoneNumberCodeMissing> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReportSponsoredResult;

class reportSponsoredChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sponsored_chat_unique_id_;
  static jfieldID sponsored_chat_unique_id_fieldID;
  bytes option_id_;
  static jfieldID option_id_fieldID;

  reportSponsoredChat();

  reportSponsoredChat(int53 sponsored_chat_unique_id_, bytes const &option_id_);

  static const std::int32_t ID = -902673019;

  using ReturnType = object_ptr<ReportSponsoredResult>;

  static object_ptr<reportSponsoredChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReportStoryResult;

class reportStory final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  bytes option_id_;
  static jfieldID option_id_fieldID;
  string text_;
  static jfieldID text_fieldID;

  reportStory();

  reportStory(int53 story_poster_chat_id_, int32 story_id_, bytes const &option_id_, string const &text_);

  static const std::int32_t ID = 1823256372;

  using ReturnType = object_ptr<ReportStoryResult>;

  static object_ptr<reportStory> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reportSupergroupAntiSpamFalsePositive final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  reportSupergroupAntiSpamFalsePositive();

  reportSupergroupAntiSpamFalsePositive(int53 supergroup_id_, int53 message_id_);

  static const std::int32_t ID = -516050872;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportSupergroupAntiSpamFalsePositive> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reportSupergroupSpam final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;

  reportSupergroupSpam();

  reportSupergroupSpam(int53 supergroup_id_, array<int53> &&message_ids_);

  static const std::int32_t ID = -94825000;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reportSupergroupSpam> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class requestAuthenticationPasswordRecovery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  requestAuthenticationPasswordRecovery();

  static const std::int32_t ID = 1393896118;

  using ReturnType = object_ptr<ok>;

  static object_ptr<requestAuthenticationPasswordRecovery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class requestPasswordRecovery final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  requestPasswordRecovery();

  static const std::int32_t ID = -13777582;

  using ReturnType = object_ptr<emailAddressAuthenticationCodeInfo>;

  static object_ptr<requestPasswordRecovery> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class requestQrCodeAuthentication final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> other_user_ids_;
  static jfieldID other_user_ids_fieldID;

  requestQrCodeAuthentication();

  explicit requestQrCodeAuthentication(array<int53> &&other_user_ids_);

  static const std::int32_t ID = 1363496527;

  using ReturnType = object_ptr<ok>;

  static object_ptr<requestQrCodeAuthentication> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ResendCodeReason;

class ok;

class resendAuthenticationCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ResendCodeReason> reason_;
  static jfieldID reason_fieldID;

  resendAuthenticationCode();

  explicit resendAuthenticationCode(object_ptr<ResendCodeReason> &&reason_);

  static const std::int32_t ID = -1506755656;

  using ReturnType = object_ptr<ok>;

  static object_ptr<resendAuthenticationCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class resendEmailAddressVerificationCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resendEmailAddressVerificationCode();

  static const std::int32_t ID = -1872416732;

  using ReturnType = object_ptr<emailAddressAuthenticationCodeInfo>;

  static object_ptr<resendEmailAddressVerificationCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class resendLoginEmailAddressCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resendLoginEmailAddressCode();

  static const std::int32_t ID = 292966933;

  using ReturnType = object_ptr<emailAddressAuthenticationCodeInfo>;

  static object_ptr<resendLoginEmailAddressCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputTextQuote;

class messages;

class resendMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  object_ptr<inputTextQuote> quote_;
  static jfieldID quote_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;

  resendMessages();

  resendMessages(int53 chat_id_, array<int53> &&message_ids_, object_ptr<inputTextQuote> &&quote_, int53 paid_message_star_count_);

  static const std::int32_t ID = 526374678;

  using ReturnType = object_ptr<messages>;

  static object_ptr<resendMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ResendCodeReason;

class authenticationCodeInfo;

class resendPhoneNumberCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ResendCodeReason> reason_;
  static jfieldID reason_fieldID;

  resendPhoneNumberCode();

  explicit resendPhoneNumberCode(object_ptr<ResendCodeReason> &&reason_);

  static const std::int32_t ID = 1808704551;

  using ReturnType = object_ptr<authenticationCodeInfo>;

  static object_ptr<resendPhoneNumberCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class resendRecoveryEmailAddressCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resendRecoveryEmailAddressCode();

  static const std::int32_t ID = 433483548;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<resendRecoveryEmailAddressCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class resetAllNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetAllNotificationSettings();

  static const std::int32_t ID = -174020359;

  using ReturnType = object_ptr<ok>;

  static object_ptr<resetAllNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class resetAuthenticationEmailAddress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetAuthenticationEmailAddress();

  static const std::int32_t ID = -415075796;

  using ReturnType = object_ptr<ok>;

  static object_ptr<resetAuthenticationEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class resetInstalledBackgrounds final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetInstalledBackgrounds();

  static const std::int32_t ID = 1884553559;

  using ReturnType = object_ptr<ok>;

  static object_ptr<resetInstalledBackgrounds> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class resetNetworkStatistics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetNetworkStatistics();

  static const std::int32_t ID = 1646452102;

  using ReturnType = object_ptr<ok>;

  static object_ptr<resetNetworkStatistics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ResetPasswordResult;

class resetPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  resetPassword();

  static const std::int32_t ID = -593589091;

  using ReturnType = object_ptr<ResetPasswordResult>;

  static object_ptr<resetPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reuseStarSubscription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string subscription_id_;
  static jfieldID subscription_id_fieldID;

  reuseStarSubscription();

  explicit reuseStarSubscription(string const &subscription_id_);

  static const std::int32_t ID = 778531905;

  using ReturnType = object_ptr<ok>;

  static object_ptr<reuseStarSubscription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatInviteLinks;

class revokeChatInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string invite_link_;
  static jfieldID invite_link_fieldID;

  revokeChatInviteLink();

  revokeChatInviteLink(int53 chat_id_, string const &invite_link_);

  static const std::int32_t ID = -776514135;

  using ReturnType = object_ptr<chatInviteLinks>;

  static object_ptr<revokeChatInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class revokeGroupCallInviteLink final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  revokeGroupCallInviteLink();

  explicit revokeGroupCallInviteLink(int32 group_call_id_);

  static const std::int32_t ID = 501589140;

  using ReturnType = object_ptr<ok>;

  static object_ptr<revokeGroupCallInviteLink> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class JsonValue;

class ok;

class saveApplicationLogEvent final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string type_;
  static jfieldID type_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<JsonValue> data_;
  static jfieldID data_fieldID;

  saveApplicationLogEvent();

  saveApplicationLogEvent(string const &type_, int53 chat_id_, object_ptr<JsonValue> &&data_);

  static const std::int32_t ID = -811154930;

  using ReturnType = object_ptr<ok>;

  static object_ptr<saveApplicationLogEvent> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputInlineQueryResult;

class preparedInlineMessageId;

class targetChatTypes;

class savePreparedInlineMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<InputInlineQueryResult> result_;
  static jfieldID result_fieldID;
  object_ptr<targetChatTypes> chat_types_;
  static jfieldID chat_types_fieldID;

  savePreparedInlineMessage();

  savePreparedInlineMessage(int53 user_id_, object_ptr<InputInlineQueryResult> &&result_, object_ptr<targetChatTypes> &&chat_types_);

  static const std::int32_t ID = -954963751;

  using ReturnType = object_ptr<preparedInlineMessageId>;

  static object_ptr<savePreparedInlineMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AffiliateProgramSortOrder;

class AffiliateType;

class foundAffiliatePrograms;

class searchAffiliatePrograms final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AffiliateType> affiliate_;
  static jfieldID affiliate_fieldID;
  object_ptr<AffiliateProgramSortOrder> sort_order_;
  static jfieldID sort_order_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchAffiliatePrograms();

  searchAffiliatePrograms(object_ptr<AffiliateType> &&affiliate_, object_ptr<AffiliateProgramSortOrder> &&sort_order_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 681156625;

  using ReturnType = object_ptr<foundAffiliatePrograms>;

  static object_ptr<searchAffiliatePrograms> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class background;

class searchBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;

  searchBackground();

  explicit searchBackground(string const &name_);

  static const std::int32_t ID = -2130996959;

  using ReturnType = object_ptr<background>;

  static object_ptr<searchBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundMessages;

class searchCallMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  bool only_missed_;
  static jfieldID only_missed_fieldID;

  searchCallMessages();

  searchCallMessages(string const &offset_, int32 limit_, bool only_missed_);

  static const std::int32_t ID = -1942229221;

  using ReturnType = object_ptr<foundMessages>;

  static object_ptr<searchCallMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class searchChatAffiliateProgram final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  string referrer_;
  static jfieldID referrer_fieldID;

  searchChatAffiliateProgram();

  searchChatAffiliateProgram(string const &username_, string const &referrer_);

  static const std::int32_t ID = -1339291206;

  using ReturnType = object_ptr<chat>;

  static object_ptr<searchChatAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatMembersFilter;

class chatMembers;

class searchChatMembers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  object_ptr<ChatMembersFilter> filter_;
  static jfieldID filter_fieldID;

  searchChatMembers();

  searchChatMembers(int53 chat_id_, string const &query_, int32 limit_, object_ptr<ChatMembersFilter> &&filter_);

  static const std::int32_t ID = -445823291;

  using ReturnType = object_ptr<chatMembers>;

  static object_ptr<searchChatMembers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class SearchMessagesFilter;

class foundChatMessages;

class searchChatMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;

  searchChatMessages();

  searchChatMessages(int53 chat_id_, string const &query_, object_ptr<MessageSender> &&sender_id_, int53 from_message_id_, int32 offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_, int53 message_thread_id_, int53 saved_messages_topic_id_);

  static const std::int32_t ID = -539052602;

  using ReturnType = object_ptr<foundChatMessages>;

  static object_ptr<searchChatMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class searchChatRecentLocationMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchChatRecentLocationMessages();

  searchChatRecentLocationMessages(int53 chat_id_, int32 limit_);

  static const std::int32_t ID = 950238950;

  using ReturnType = object_ptr<messages>;

  static object_ptr<searchChatRecentLocationMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class searchChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchChats();

  searchChats(string const &query_, int32 limit_);

  static const std::int32_t ID = -1879787060;

  using ReturnType = object_ptr<chats>;

  static object_ptr<searchChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class searchChatsOnServer final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchChatsOnServer();

  searchChatsOnServer(string const &query_, int32 limit_);

  static const std::int32_t ID = -1158402188;

  using ReturnType = object_ptr<chats>;

  static object_ptr<searchChatsOnServer> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class users;

class searchContacts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchContacts();

  searchContacts(string const &query_, int32 limit_);

  static const std::int32_t ID = -1794690715;

  using ReturnType = object_ptr<users>;

  static object_ptr<searchContacts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiKeywords;

class searchEmojis final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string text_;
  static jfieldID text_fieldID;
  array<string> input_language_codes_;
  static jfieldID input_language_codes_fieldID;

  searchEmojis();

  searchEmojis(string const &text_, array<string> &&input_language_codes_);

  static const std::int32_t ID = -1456187668;

  using ReturnType = object_ptr<emojiKeywords>;

  static object_ptr<searchEmojis> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundFileDownloads;

class searchFileDownloads final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  bool only_active_;
  static jfieldID only_active_fieldID;
  bool only_completed_;
  static jfieldID only_completed_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchFileDownloads();

  searchFileDownloads(string const &query_, bool only_active_, bool only_completed_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 706611286;

  using ReturnType = object_ptr<foundFileDownloads>;

  static object_ptr<searchFileDownloads> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class GiftForResaleOrder;

class UpgradedGiftAttributeId;

class giftsForResale;

class searchGiftsForResale final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 gift_id_;
  static jfieldID gift_id_fieldID;
  object_ptr<GiftForResaleOrder> order_;
  static jfieldID order_fieldID;
  array<object_ptr<UpgradedGiftAttributeId>> attributes_;
  static jfieldID attributes_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchGiftsForResale();

  searchGiftsForResale(int64 gift_id_, object_ptr<GiftForResaleOrder> &&order_, array<object_ptr<UpgradedGiftAttributeId>> &&attributes_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 1659300894;

  using ReturnType = object_ptr<giftsForResale>;

  static object_ptr<searchGiftsForResale> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class hashtags;

class searchHashtags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string prefix_;
  static jfieldID prefix_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchHashtags();

  searchHashtags(string const &prefix_, int32 limit_);

  static const std::int32_t ID = 1043637617;

  using ReturnType = object_ptr<hashtags>;

  static object_ptr<searchHashtags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickerSets;

class searchInstalledStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchInstalledStickerSets();

  searchInstalledStickerSets(object_ptr<StickerType> &&sticker_type_, string const &query_, int32 limit_);

  static const std::int32_t ID = 2120122276;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<searchInstalledStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class SearchMessagesChatTypeFilter;

class SearchMessagesFilter;

class foundMessages;

class searchMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  string query_;
  static jfieldID query_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;
  object_ptr<SearchMessagesChatTypeFilter> chat_type_filter_;
  static jfieldID chat_type_filter_fieldID;
  int32 min_date_;
  static jfieldID min_date_fieldID;
  int32 max_date_;
  static jfieldID max_date_fieldID;

  searchMessages();

  searchMessages(object_ptr<ChatList> &&chat_list_, string const &query_, string const &offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_, object_ptr<SearchMessagesChatTypeFilter> &&chat_type_filter_, int32 min_date_, int32 max_date_);

  static const std::int32_t ID = 1225448885;

  using ReturnType = object_ptr<foundMessages>;

  static object_ptr<searchMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundMessages;

class searchOutgoingDocumentMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchOutgoingDocumentMessages();

  searchOutgoingDocumentMessages(string const &query_, int32 limit_);

  static const std::int32_t ID = -1071397762;

  using ReturnType = object_ptr<foundMessages>;

  static object_ptr<searchOutgoingDocumentMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class searchPublicChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;

  searchPublicChat();

  explicit searchPublicChat(string const &username_);

  static const std::int32_t ID = 857135533;

  using ReturnType = object_ptr<chat>;

  static object_ptr<searchPublicChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class searchPublicChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;

  searchPublicChats();

  explicit searchPublicChats(string const &query_);

  static const std::int32_t ID = 970385337;

  using ReturnType = object_ptr<chats>;

  static object_ptr<searchPublicChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundMessages;

class searchPublicMessagesByTag final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string tag_;
  static jfieldID tag_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchPublicMessagesByTag();

  searchPublicMessagesByTag(string const &tag_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 630680746;

  using ReturnType = object_ptr<foundMessages>;

  static object_ptr<searchPublicMessagesByTag> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundStories;

class locationAddress;

class searchPublicStoriesByLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<locationAddress> address_;
  static jfieldID address_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchPublicStoriesByLocation();

  searchPublicStoriesByLocation(object_ptr<locationAddress> &&address_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 1596709256;

  using ReturnType = object_ptr<foundStories>;

  static object_ptr<searchPublicStoriesByLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundStories;

class searchPublicStoriesByTag final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  string tag_;
  static jfieldID tag_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchPublicStoriesByTag();

  searchPublicStoriesByTag(int53 story_poster_chat_id_, string const &tag_, string const &offset_, int32 limit_);

  static const std::int32_t ID = 1778102602;

  using ReturnType = object_ptr<foundStories>;

  static object_ptr<searchPublicStoriesByTag> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundStories;

class searchPublicStoriesByVenue final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string venue_provider_;
  static jfieldID venue_provider_fieldID;
  string venue_id_;
  static jfieldID venue_id_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchPublicStoriesByVenue();

  searchPublicStoriesByVenue(string const &venue_provider_, string const &venue_id_, string const &offset_, int32 limit_);

  static const std::int32_t ID = -686136790;

  using ReturnType = object_ptr<foundStories>;

  static object_ptr<searchPublicStoriesByVenue> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class foundPosition;

class searchQuote final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  object_ptr<formattedText> quote_;
  static jfieldID quote_fieldID;
  int32 quote_position_;
  static jfieldID quote_position_fieldID;

  searchQuote();

  searchQuote(object_ptr<formattedText> &&text_, object_ptr<formattedText> &&quote_, int32 quote_position_);

  static const std::int32_t ID = 1751384351;

  using ReturnType = object_ptr<foundPosition>;

  static object_ptr<searchQuote> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chats;

class searchRecentlyFoundChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchRecentlyFoundChats();

  searchRecentlyFoundChats(string const &query_, int32 limit_);

  static const std::int32_t ID = 1647445393;

  using ReturnType = object_ptr<chats>;

  static object_ptr<searchRecentlyFoundChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class foundChatMessages;

class searchSavedMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  object_ptr<ReactionType> tag_;
  static jfieldID tag_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int53 from_message_id_;
  static jfieldID from_message_id_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchSavedMessages();

  searchSavedMessages(int53 saved_messages_topic_id_, object_ptr<ReactionType> &&tag_, string const &query_, int53 from_message_id_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -1969512554;

  using ReturnType = object_ptr<foundChatMessages>;

  static object_ptr<searchSavedMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class SearchMessagesFilter;

class foundMessages;

class searchSecretMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string query_;
  static jfieldID query_fieldID;
  string offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  object_ptr<SearchMessagesFilter> filter_;
  static jfieldID filter_fieldID;

  searchSecretMessages();

  searchSecretMessages(int53 chat_id_, string const &query_, string const &offset_, int32 limit_, object_ptr<SearchMessagesFilter> &&filter_);

  static const std::int32_t ID = -852865892;

  using ReturnType = object_ptr<foundMessages>;

  static object_ptr<searchSecretMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class stickerSet;

class searchStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  bool ignore_cache_;
  static jfieldID ignore_cache_fieldID;

  searchStickerSet();

  searchStickerSet(string const &name_, bool ignore_cache_);

  static const std::int32_t ID = 1676592898;

  using ReturnType = object_ptr<stickerSet>;

  static object_ptr<searchStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickerSets;

class searchStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  string query_;
  static jfieldID query_fieldID;

  searchStickerSets();

  searchStickerSets(object_ptr<StickerType> &&sticker_type_, string const &query_);

  static const std::int32_t ID = 262801004;

  using ReturnType = object_ptr<stickerSets>;

  static object_ptr<searchStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StickerType;

class stickers;

class searchStickers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StickerType> sticker_type_;
  static jfieldID sticker_type_fieldID;
  string emojis_;
  static jfieldID emojis_fieldID;
  string query_;
  static jfieldID query_fieldID;
  array<string> input_language_codes_;
  static jfieldID input_language_codes_fieldID;
  int32 offset_;
  static jfieldID offset_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;

  searchStickers();

  searchStickers(object_ptr<StickerType> &&sticker_type_, string const &emojis_, string const &query_, array<string> &&input_language_codes_, int32 offset_, int32 limit_);

  static const std::int32_t ID = -1856294754;

  using ReturnType = object_ptr<stickers>;

  static object_ptr<searchStickers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundPositions;

class searchStringsByPrefix final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> strings_;
  static jfieldID strings_fieldID;
  string query_;
  static jfieldID query_fieldID;
  int32 limit_;
  static jfieldID limit_fieldID;
  bool return_none_for_empty_query_;
  static jfieldID return_none_for_empty_query_fieldID;

  searchStringsByPrefix();

  searchStringsByPrefix(array<string> &&strings_, string const &query_, int32 limit_, bool return_none_for_empty_query_);

  static const std::int32_t ID = -2023251463;

  using ReturnType = object_ptr<foundPositions>;

  static object_ptr<searchStringsByPrefix> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class user;

class searchUserByPhoneNumber final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  bool only_local_;
  static jfieldID only_local_fieldID;

  searchUserByPhoneNumber();

  searchUserByPhoneNumber(string const &phone_number_, bool only_local_);

  static const std::int32_t ID = -343757368;

  using ReturnType = object_ptr<user>;

  static object_ptr<searchUserByPhoneNumber> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class user;

class searchUserByToken final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  searchUserByToken();

  explicit searchUserByToken(string const &token_);

  static const std::int32_t ID = -666766282;

  using ReturnType = object_ptr<user>;

  static object_ptr<searchUserByToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class foundWebApp;

class searchWebApp final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string web_app_short_name_;
  static jfieldID web_app_short_name_fieldID;

  searchWebApp();

  searchWebApp(int53 bot_user_id_, string const &web_app_short_name_);

  static const std::int32_t ID = -1241740747;

  using ReturnType = object_ptr<foundWebApp>;

  static object_ptr<searchWebApp> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sellGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;

  sellGift();

  sellGift(string const &business_connection_id_, string const &received_gift_id_);

  static const std::int32_t ID = -298298375;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sellGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sendAuthenticationFirebaseSms final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  sendAuthenticationFirebaseSms();

  explicit sendAuthenticationFirebaseSms(string const &token_);

  static const std::int32_t ID = 364994111;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendAuthenticationFirebaseSms> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class sendBotStartMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string parameter_;
  static jfieldID parameter_fieldID;

  sendBotStartMessage();

  sendBotStartMessage(int53 bot_user_id_, int53 chat_id_, string const &parameter_);

  static const std::int32_t ID = -1435877650;

  using ReturnType = object_ptr<message>;

  static object_ptr<sendBotStartMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class ReplyMarkup;

class businessMessage;

class sendBusinessMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;
  bool protect_content_;
  static jfieldID protect_content_fieldID;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  sendBusinessMessage();

  sendBusinessMessage(string const &business_connection_id_, int53 chat_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, bool protect_content_, int64 effect_id_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = 159888387;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<sendBusinessMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class businessMessages;

class sendBusinessMessageAlbum final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  bool disable_notification_;
  static jfieldID disable_notification_fieldID;
  bool protect_content_;
  static jfieldID protect_content_fieldID;
  int64 effect_id_;
  static jfieldID effect_id_fieldID;
  array<object_ptr<InputMessageContent>> input_message_contents_;
  static jfieldID input_message_contents_fieldID;

  sendBusinessMessageAlbum();

  sendBusinessMessageAlbum(string const &business_connection_id_, int53 chat_id_, object_ptr<InputMessageReplyTo> &&reply_to_, bool disable_notification_, bool protect_content_, int64 effect_id_, array<object_ptr<InputMessageContent>> &&input_message_contents_);

  static const std::int32_t ID = 788608366;

  using ReturnType = object_ptr<businessMessages>;

  static object_ptr<sendBusinessMessageAlbum> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sendCallDebugInformation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  string debug_information_;
  static jfieldID debug_information_fieldID;

  sendCallDebugInformation();

  sendCallDebugInformation(int32 call_id_, string const &debug_information_);

  static const std::int32_t ID = 2019243839;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendCallDebugInformation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class sendCallLog final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  object_ptr<InputFile> log_file_;
  static jfieldID log_file_fieldID;

  sendCallLog();

  sendCallLog(int32 call_id_, object_ptr<InputFile> &&log_file_);

  static const std::int32_t ID = 1057638353;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendCallLog> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class CallProblem;

class ok;

class sendCallRating final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  int32 rating_;
  static jfieldID rating_fieldID;
  string comment_;
  static jfieldID comment_fieldID;
  array<object_ptr<CallProblem>> problems_;
  static jfieldID problems_fieldID;

  sendCallRating();

  sendCallRating(int32 call_id_, int32 rating_, string const &comment_, array<object_ptr<CallProblem>> &&problems_);

  static const std::int32_t ID = -1402719502;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendCallRating> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sendCallSignalingData final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 call_id_;
  static jfieldID call_id_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  sendCallSignalingData();

  sendCallSignalingData(int32 call_id_, bytes const &data_);

  static const std::int32_t ID = 1412280732;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendCallSignalingData> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatAction;

class ok;

class sendChatAction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  object_ptr<ChatAction> action_;
  static jfieldID action_fieldID;

  sendChatAction();

  sendChatAction(int53 chat_id_, int53 message_thread_id_, string const &business_connection_id_, object_ptr<ChatAction> &&action_);

  static const std::int32_t ID = -2010910050;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendChatAction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class customRequestResult;

class sendCustomRequest final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string method_;
  static jfieldID method_fieldID;
  string parameters_;
  static jfieldID parameters_fieldID;

  sendCustomRequest();

  sendCustomRequest(string const &method_, string const &parameters_);

  static const std::int32_t ID = 285045153;

  using ReturnType = object_ptr<customRequestResult>;

  static object_ptr<sendCustomRequest> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class sendEmailAddressVerificationCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string email_address_;
  static jfieldID email_address_fieldID;

  sendEmailAddressVerificationCode();

  explicit sendEmailAddressVerificationCode(string const &email_address_);

  static const std::int32_t ID = -221621379;

  using ReturnType = object_ptr<emailAddressAuthenticationCodeInfo>;

  static object_ptr<sendEmailAddressVerificationCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class formattedText;

class ok;

class sendGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 gift_id_;
  static jfieldID gift_id_fieldID;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  bool is_private_;
  static jfieldID is_private_fieldID;
  bool pay_for_upgrade_;
  static jfieldID pay_for_upgrade_fieldID;

  sendGift();

  sendGift(int64 gift_id_, object_ptr<MessageSender> &&owner_id_, object_ptr<formattedText> &&text_, bool is_private_, bool pay_for_upgrade_);

  static const std::int32_t ID = -1199356118;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageReplyTo;

class message;

class messageSendOptions;

class sendInlineQueryResultMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  object_ptr<messageSendOptions> options_;
  static jfieldID options_fieldID;
  int64 query_id_;
  static jfieldID query_id_fieldID;
  string result_id_;
  static jfieldID result_id_fieldID;
  bool hide_via_bot_;
  static jfieldID hide_via_bot_fieldID;

  sendInlineQueryResultMessage();

  sendInlineQueryResultMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, int64 query_id_, string const &result_id_, bool hide_via_bot_);

  static const std::int32_t ID = 1182553208;

  using ReturnType = object_ptr<message>;

  static object_ptr<sendInlineQueryResultMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class ReplyMarkup;

class message;

class messageSendOptions;

class sendMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  object_ptr<messageSendOptions> options_;
  static jfieldID options_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;
  object_ptr<InputMessageContent> input_message_content_;
  static jfieldID input_message_content_fieldID;

  sendMessage();

  sendMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, object_ptr<ReplyMarkup> &&reply_markup_, object_ptr<InputMessageContent> &&input_message_content_);

  static const std::int32_t ID = -533921303;

  using ReturnType = object_ptr<message>;

  static object_ptr<sendMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputMessageContent;

class InputMessageReplyTo;

class messageSendOptions;

class messages;

class sendMessageAlbum final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<InputMessageReplyTo> reply_to_;
  static jfieldID reply_to_fieldID;
  object_ptr<messageSendOptions> options_;
  static jfieldID options_fieldID;
  array<object_ptr<InputMessageContent>> input_message_contents_;
  static jfieldID input_message_contents_fieldID;

  sendMessageAlbum();

  sendMessageAlbum(int53 chat_id_, int53 message_thread_id_, object_ptr<InputMessageReplyTo> &&reply_to_, object_ptr<messageSendOptions> &&options_, array<object_ptr<InputMessageContent>> &&input_message_contents_);

  static const std::int32_t ID = -1985013029;

  using ReturnType = object_ptr<messages>;

  static object_ptr<sendMessageAlbum> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PassportElementType;

class ok;

class sendPassportAuthorizationForm final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 authorization_form_id_;
  static jfieldID authorization_form_id_fieldID;
  array<object_ptr<PassportElementType>> types_;
  static jfieldID types_fieldID;

  sendPassportAuthorizationForm();

  sendPassportAuthorizationForm(int32 authorization_form_id_, array<object_ptr<PassportElementType>> &&types_);

  static const std::int32_t ID = 652160701;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendPassportAuthorizationForm> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputCredentials;

class InputInvoice;

class paymentResult;

class sendPaymentForm final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputInvoice> input_invoice_;
  static jfieldID input_invoice_fieldID;
  int64 payment_form_id_;
  static jfieldID payment_form_id_fieldID;
  string order_info_id_;
  static jfieldID order_info_id_fieldID;
  string shipping_option_id_;
  static jfieldID shipping_option_id_fieldID;
  object_ptr<InputCredentials> credentials_;
  static jfieldID credentials_fieldID;
  int53 tip_amount_;
  static jfieldID tip_amount_fieldID;

  sendPaymentForm();

  sendPaymentForm(object_ptr<InputInvoice> &&input_invoice_, int64 payment_form_id_, string const &order_info_id_, string const &shipping_option_id_, object_ptr<InputCredentials> &&credentials_, int53 tip_amount_);

  static const std::int32_t ID = -965855094;

  using ReturnType = object_ptr<paymentResult>;

  static object_ptr<sendPaymentForm> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PhoneNumberCodeType;

class authenticationCodeInfo;

class phoneNumberAuthenticationSettings;

class sendPhoneNumberCode final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  object_ptr<phoneNumberAuthenticationSettings> settings_;
  static jfieldID settings_fieldID;
  object_ptr<PhoneNumberCodeType> type_;
  static jfieldID type_fieldID;

  sendPhoneNumberCode();

  sendPhoneNumberCode(string const &phone_number_, object_ptr<phoneNumberAuthenticationSettings> &&settings_, object_ptr<PhoneNumberCodeType> &&type_);

  static const std::int32_t ID = 1084112144;

  using ReturnType = object_ptr<authenticationCodeInfo>;

  static object_ptr<sendPhoneNumberCode> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sendPhoneNumberFirebaseSms final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string token_;
  static jfieldID token_fieldID;

  sendPhoneNumberFirebaseSms();

  explicit sendPhoneNumberFirebaseSms(string const &token_);

  static const std::int32_t ID = 261910660;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendPhoneNumberFirebaseSms> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messages;

class sendQuickReplyShortcutMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  int32 sending_id_;
  static jfieldID sending_id_fieldID;

  sendQuickReplyShortcutMessages();

  sendQuickReplyShortcutMessages(int53 chat_id_, int32 shortcut_id_, int32 sending_id_);

  static const std::int32_t ID = 232068765;

  using ReturnType = object_ptr<messages>;

  static object_ptr<sendQuickReplyShortcutMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class sendResoldGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string gift_name_;
  static jfieldID gift_name_fieldID;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  sendResoldGift();

  sendResoldGift(string const &gift_name_, object_ptr<MessageSender> &&owner_id_, int53 star_count_);

  static const std::int32_t ID = -1284812948;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendResoldGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class customRequestResult;

class sendWebAppCustomRequest final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string method_;
  static jfieldID method_fieldID;
  string parameters_;
  static jfieldID parameters_fieldID;

  sendWebAppCustomRequest();

  sendWebAppCustomRequest(int53 bot_user_id_, string const &method_, string const &parameters_);

  static const std::int32_t ID = 922705352;

  using ReturnType = object_ptr<customRequestResult>;

  static object_ptr<sendWebAppCustomRequest> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sendWebAppData final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string button_text_;
  static jfieldID button_text_fieldID;
  string data_;
  static jfieldID data_fieldID;

  sendWebAppData();

  sendWebAppData(int53 bot_user_id_, string const &button_text_, string const &data_);

  static const std::int32_t ID = -1423978996;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sendWebAppData> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setAccentColor final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;

  setAccentColor();

  setAccentColor(int32 accent_color_id_, int64 background_custom_emoji_id_);

  static const std::int32_t ID = 1669974841;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAccentColor> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class accountTtl;

class ok;

class setAccountTtl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<accountTtl> ttl_;
  static jfieldID ttl_fieldID;

  setAccountTtl();

  explicit setAccountTtl(object_ptr<accountTtl> &&ttl_);

  static const std::int32_t ID = 701389032;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAccountTtl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setAlarm final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  double seconds_;
  static jfieldID seconds_fieldID;

  setAlarm();

  explicit setAlarm(double seconds_);

  static const std::int32_t ID = -873497067;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAlarm> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setApplicationVerificationToken final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 verification_id_;
  static jfieldID verification_id_fieldID;
  string token_;
  static jfieldID token_fieldID;

  setApplicationVerificationToken();

  setApplicationVerificationToken(int53 verification_id_, string const &token_);

  static const std::int32_t ID = 927248261;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setApplicationVerificationToken> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class archiveChatListSettings;

class ok;

class setArchiveChatListSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<archiveChatListSettings> settings_;
  static jfieldID settings_fieldID;

  setArchiveChatListSettings();

  explicit setArchiveChatListSettings(object_ptr<archiveChatListSettings> &&settings_);

  static const std::int32_t ID = -884650998;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setArchiveChatListSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setAuthenticationEmailAddress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string email_address_;
  static jfieldID email_address_fieldID;

  setAuthenticationEmailAddress();

  explicit setAuthenticationEmailAddress(string const &email_address_);

  static const std::int32_t ID = 1773323522;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAuthenticationEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class phoneNumberAuthenticationSettings;

class setAuthenticationPhoneNumber final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string phone_number_;
  static jfieldID phone_number_fieldID;
  object_ptr<phoneNumberAuthenticationSettings> settings_;
  static jfieldID settings_fieldID;

  setAuthenticationPhoneNumber();

  setAuthenticationPhoneNumber(string const &phone_number_, object_ptr<phoneNumberAuthenticationSettings> &&settings_);

  static const std::int32_t ID = 868276259;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAuthenticationPhoneNumber> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StoreTransaction;

class ok;

class setAuthenticationPremiumPurchaseTransaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<StoreTransaction> transaction_;
  static jfieldID transaction_fieldID;
  bool is_restore_;
  static jfieldID is_restore_fieldID;
  string currency_;
  static jfieldID currency_fieldID;
  int53 amount_;
  static jfieldID amount_fieldID;

  setAuthenticationPremiumPurchaseTransaction();

  setAuthenticationPremiumPurchaseTransaction(object_ptr<StoreTransaction> &&transaction_, bool is_restore_, string const &currency_, int53 amount_);

  static const std::int32_t ID = -450986887;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAuthenticationPremiumPurchaseTransaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NetworkType;

class autoDownloadSettings;

class ok;

class setAutoDownloadSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<autoDownloadSettings> settings_;
  static jfieldID settings_fieldID;
  object_ptr<NetworkType> type_;
  static jfieldID type_fieldID;

  setAutoDownloadSettings();

  setAutoDownloadSettings(object_ptr<autoDownloadSettings> &&settings_, object_ptr<NetworkType> &&type_);

  static const std::int32_t ID = -353671948;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAutoDownloadSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class AutosaveSettingsScope;

class ok;

class scopeAutosaveSettings;

class setAutosaveSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<AutosaveSettingsScope> scope_;
  static jfieldID scope_fieldID;
  object_ptr<scopeAutosaveSettings> settings_;
  static jfieldID settings_fieldID;

  setAutosaveSettings();

  setAutosaveSettings(object_ptr<AutosaveSettingsScope> &&scope_, object_ptr<scopeAutosaveSettings> &&settings_);

  static const std::int32_t ID = 6846656;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setAutosaveSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBio final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string bio_;
  static jfieldID bio_fieldID;

  setBio();

  explicit setBio(string const &bio_);

  static const std::int32_t ID = -1619582124;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBio> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class birthdate;

class ok;

class setBirthdate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<birthdate> birthdate_;
  static jfieldID birthdate_fieldID;

  setBirthdate();

  explicit setBirthdate(object_ptr<birthdate> &&birthdate_);

  static const std::int32_t ID = 1319755160;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBirthdate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBotInfoDescription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  string description_;
  static jfieldID description_fieldID;

  setBotInfoDescription();

  setBotInfoDescription(int53 bot_user_id_, string const &language_code_, string const &description_);

  static const std::int32_t ID = 693574984;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBotInfoDescription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBotInfoShortDescription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  string short_description_;
  static jfieldID short_description_fieldID;

  setBotInfoShortDescription();

  setBotInfoShortDescription(int53 bot_user_id_, string const &language_code_, string const &short_description_);

  static const std::int32_t ID = 982956771;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBotInfoShortDescription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBotName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  string name_;
  static jfieldID name_fieldID;

  setBotName();

  setBotName(int53 bot_user_id_, string const &language_code_, string const &name_);

  static const std::int32_t ID = -761922959;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBotName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class setBotProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;

  setBotProfilePhoto();

  setBotProfilePhoto(int53 bot_user_id_, object_ptr<InputChatPhoto> &&photo_);

  static const std::int32_t ID = -1115272346;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBotProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBotUpdatesStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 pending_update_count_;
  static jfieldID pending_update_count_fieldID;
  string error_message_;
  static jfieldID error_message_fieldID;

  setBotUpdatesStatus();

  setBotUpdatesStatus(int32 pending_update_count_, string const &error_message_);

  static const std::int32_t ID = -1154926191;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBotUpdatesStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBusinessAccountBio final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string bio_;
  static jfieldID bio_fieldID;

  setBusinessAccountBio();

  setBusinessAccountBio(string const &business_connection_id_, string const &bio_);

  static const std::int32_t ID = -1698538041;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAccountBio> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giftSettings;

class ok;

class setBusinessAccountGiftSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  object_ptr<giftSettings> settings_;
  static jfieldID settings_fieldID;

  setBusinessAccountGiftSettings();

  setBusinessAccountGiftSettings(string const &business_connection_id_, object_ptr<giftSettings> &&settings_);

  static const std::int32_t ID = 1757763090;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAccountGiftSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBusinessAccountName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;

  setBusinessAccountName();

  setBusinessAccountName(string const &business_connection_id_, string const &first_name_, string const &last_name_);

  static const std::int32_t ID = 999582546;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAccountName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class setBusinessAccountProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;
  bool is_public_;
  static jfieldID is_public_fieldID;

  setBusinessAccountProfilePhoto();

  setBusinessAccountProfilePhoto(string const &business_connection_id_, object_ptr<InputChatPhoto> &&photo_, bool is_public_);

  static const std::int32_t ID = 1174440149;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAccountProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBusinessAccountUsername final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string username_;
  static jfieldID username_fieldID;

  setBusinessAccountUsername();

  setBusinessAccountUsername(string const &business_connection_id_, string const &username_);

  static const std::int32_t ID = -1520126367;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAccountUsername> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessAwayMessageSettings;

class ok;

class setBusinessAwayMessageSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessAwayMessageSettings> away_message_settings_;
  static jfieldID away_message_settings_fieldID;

  setBusinessAwayMessageSettings();

  explicit setBusinessAwayMessageSettings(object_ptr<businessAwayMessageSettings> &&away_message_settings_);

  static const std::int32_t ID = 1232357484;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessAwayMessageSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessConnectedBot;

class ok;

class setBusinessConnectedBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessConnectedBot> bot_;
  static jfieldID bot_fieldID;

  setBusinessConnectedBot();

  explicit setBusinessConnectedBot(object_ptr<businessConnectedBot> &&bot_);

  static const std::int32_t ID = -1393459472;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessConnectedBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessGreetingMessageSettings;

class ok;

class setBusinessGreetingMessageSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessGreetingMessageSettings> greeting_message_settings_;
  static jfieldID greeting_message_settings_fieldID;

  setBusinessGreetingMessageSettings();

  explicit setBusinessGreetingMessageSettings(object_ptr<businessGreetingMessageSettings> &&greeting_message_settings_);

  static const std::int32_t ID = -873120707;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessGreetingMessageSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessLocation;

class ok;

class setBusinessLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessLocation> location_;
  static jfieldID location_fieldID;

  setBusinessLocation();

  explicit setBusinessLocation(object_ptr<businessLocation> &&location_);

  static const std::int32_t ID = -344717547;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setBusinessMessageIsPinned final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  setBusinessMessageIsPinned();

  setBusinessMessageIsPinned(string const &business_connection_id_, int53 chat_id_, int53 message_id_, bool is_pinned_);

  static const std::int32_t ID = -15403536;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessMessageIsPinned> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class businessOpeningHours;

class ok;

class setBusinessOpeningHours final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<businessOpeningHours> opening_hours_;
  static jfieldID opening_hours_fieldID;

  setBusinessOpeningHours();

  explicit setBusinessOpeningHours(object_ptr<businessOpeningHours> &&opening_hours_);

  static const std::int32_t ID = -462379918;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessOpeningHours> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputBusinessStartPage;

class ok;

class setBusinessStartPage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<inputBusinessStartPage> start_page_;
  static jfieldID start_page_fieldID;

  setBusinessStartPage();

  explicit setBusinessStartPage(object_ptr<inputBusinessStartPage> &&start_page_);

  static const std::int32_t ID = -1628616290;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setBusinessStartPage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatAccentColor final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 accent_color_id_;
  static jfieldID accent_color_id_fieldID;
  int64 background_custom_emoji_id_;
  static jfieldID background_custom_emoji_id_fieldID;

  setChatAccentColor();

  setChatAccentColor(int53 chat_id_, int32 accent_color_id_, int64 background_custom_emoji_id_);

  static const std::int32_t ID = 882857930;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatAccentColor> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StoryList;

class ok;

class setChatActiveStoriesList final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<StoryList> story_list_;
  static jfieldID story_list_fieldID;

  setChatActiveStoriesList();

  setChatActiveStoriesList(int53 chat_id_, object_ptr<StoryList> &&story_list_);

  static const std::int32_t ID = -521970415;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatActiveStoriesList> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class affiliateProgramParameters;

class ok;

class setChatAffiliateProgram final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<affiliateProgramParameters> parameters_;
  static jfieldID parameters_fieldID;

  setChatAffiliateProgram();

  setChatAffiliateProgram(int53 chat_id_, object_ptr<affiliateProgramParameters> &&parameters_);

  static const std::int32_t ID = 14680631;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatAffiliateProgram> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatAvailableReactions;

class ok;

class setChatAvailableReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<ChatAvailableReactions> available_reactions_;
  static jfieldID available_reactions_fieldID;

  setChatAvailableReactions();

  setChatAvailableReactions(int53 chat_id_, object_ptr<ChatAvailableReactions> &&available_reactions_);

  static const std::int32_t ID = 267075078;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatAvailableReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundType;

class InputBackground;

class ok;

class setChatBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputBackground> background_;
  static jfieldID background_fieldID;
  object_ptr<BackgroundType> type_;
  static jfieldID type_fieldID;
  int32 dark_theme_dimming_;
  static jfieldID dark_theme_dimming_fieldID;
  bool only_for_self_;
  static jfieldID only_for_self_fieldID;

  setChatBackground();

  setChatBackground(int53 chat_id_, object_ptr<InputBackground> &&background_, object_ptr<BackgroundType> &&type_, int32 dark_theme_dimming_, bool only_for_self_);

  static const std::int32_t ID = 246727678;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatClientData final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string client_data_;
  static jfieldID client_data_fieldID;

  setChatClientData();

  setChatClientData(int53 chat_id_, string const &client_data_);

  static const std::int32_t ID = -827119811;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatClientData> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatDescription final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string description_;
  static jfieldID description_fieldID;

  setChatDescription();

  setChatDescription(int53 chat_id_, string const &description_);

  static const std::int32_t ID = 1957213277;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatDescription> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatDiscussionGroup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 discussion_chat_id_;
  static jfieldID discussion_chat_id_fieldID;

  setChatDiscussionGroup();

  setChatDiscussionGroup(int53 chat_id_, int53 discussion_chat_id_);

  static const std::int32_t ID = -918801736;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatDiscussionGroup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class draftMessage;

class ok;

class setChatDraftMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<draftMessage> draft_message_;
  static jfieldID draft_message_fieldID;

  setChatDraftMessage();

  setChatDraftMessage(int53 chat_id_, int53 message_thread_id_, object_ptr<draftMessage> &&draft_message_);

  static const std::int32_t ID = 1683889946;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatDraftMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatus;

class ok;

class setChatEmojiStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;

  setChatEmojiStatus();

  setChatEmojiStatus(int53 chat_id_, object_ptr<emojiStatus> &&emoji_status_);

  static const std::int32_t ID = 1434982674;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatEmojiStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatLocation;

class ok;

class setChatLocation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatLocation> location_;
  static jfieldID location_fieldID;

  setChatLocation();

  setChatLocation(int53 chat_id_, object_ptr<chatLocation> &&location_);

  static const std::int32_t ID = -767091286;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatLocation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatMemberStatus;

class MessageSender;

class ok;

class setChatMemberStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> member_id_;
  static jfieldID member_id_fieldID;
  object_ptr<ChatMemberStatus> status_;
  static jfieldID status_fieldID;

  setChatMemberStatus();

  setChatMemberStatus(int53 chat_id_, object_ptr<MessageSender> &&member_id_, object_ptr<ChatMemberStatus> &&status_);

  static const std::int32_t ID = 81794847;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatMemberStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatMessageAutoDeleteTime final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;

  setChatMessageAutoDeleteTime();

  setChatMessageAutoDeleteTime(int53 chat_id_, int32 message_auto_delete_time_);

  static const std::int32_t ID = -1505643265;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatMessageAutoDeleteTime> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class setChatMessageSender final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> message_sender_id_;
  static jfieldID message_sender_id_fieldID;

  setChatMessageSender();

  setChatMessageSender(int53 chat_id_, object_ptr<MessageSender> &&message_sender_id_);

  static const std::int32_t ID = -1421513858;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatMessageSender> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatNotificationSettings;

class ok;

class setChatNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  setChatNotificationSettings();

  setChatNotificationSettings(int53 chat_id_, object_ptr<chatNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = 777199614;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatPaidMessageStarCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 paid_message_star_count_;
  static jfieldID paid_message_star_count_fieldID;

  setChatPaidMessageStarCount();

  setChatPaidMessageStarCount(int53 chat_id_, int53 paid_message_star_count_);

  static const std::int32_t ID = -1187053289;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatPaidMessageStarCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatPermissions;

class ok;

class setChatPermissions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<chatPermissions> permissions_;
  static jfieldID permissions_fieldID;

  setChatPermissions();

  setChatPermissions(int53 chat_id_, object_ptr<chatPermissions> &&permissions_);

  static const std::int32_t ID = 2138507006;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatPermissions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class setChatPhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;

  setChatPhoto();

  setChatPhoto(int53 chat_id_, object_ptr<InputChatPhoto> &&photo_);

  static const std::int32_t ID = -377778941;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatPhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatPinnedStories final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int32> story_ids_;
  static jfieldID story_ids_fieldID;

  setChatPinnedStories();

  setChatPinnedStories(int53 chat_id_, array<int32> &&story_ids_);

  static const std::int32_t ID = -669062355;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatPinnedStories> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatProfileAccentColor final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 profile_accent_color_id_;
  static jfieldID profile_accent_color_id_fieldID;
  int64 profile_background_custom_emoji_id_;
  static jfieldID profile_background_custom_emoji_id_fieldID;

  setChatProfileAccentColor();

  setChatProfileAccentColor(int53 chat_id_, int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_);

  static const std::int32_t ID = 1109896826;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatProfileAccentColor> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatSlowModeDelay final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int32 slow_mode_delay_;
  static jfieldID slow_mode_delay_fieldID;

  setChatSlowModeDelay();

  setChatSlowModeDelay(int53 chat_id_, int32 slow_mode_delay_);

  static const std::int32_t ID = -540350914;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatSlowModeDelay> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatTheme final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string theme_name_;
  static jfieldID theme_name_fieldID;

  setChatTheme();

  setChatTheme(int53 chat_id_, string const &theme_name_);

  static const std::int32_t ID = -1895234925;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatTheme> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setChatTitle final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  string title_;
  static jfieldID title_fieldID;

  setChatTitle();

  setChatTitle(int53 chat_id_, string const &title_);

  static const std::int32_t ID = 164282047;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setChatTitle> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setCloseFriends final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> user_ids_;
  static jfieldID user_ids_fieldID;

  setCloseFriends();

  explicit setCloseFriends(array<int53> &&user_ids_);

  static const std::int32_t ID = -1908013258;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setCloseFriends> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BotCommandScope;

class botCommand;

class ok;

class setCommands final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<BotCommandScope> scope_;
  static jfieldID scope_fieldID;
  string language_code_;
  static jfieldID language_code_fieldID;
  array<object_ptr<botCommand>> commands_;
  static jfieldID commands_fieldID;

  setCommands();

  setCommands(object_ptr<BotCommandScope> &&scope_, string const &language_code_, array<object_ptr<botCommand>> &&commands_);

  static const std::int32_t ID = -907165606;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setCommands> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setCustomEmojiStickerSetThumbnail final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  int64 custom_emoji_id_;
  static jfieldID custom_emoji_id_fieldID;

  setCustomEmojiStickerSetThumbnail();

  setCustomEmojiStickerSetThumbnail(string const &name_, int64 custom_emoji_id_);

  static const std::int32_t ID = -1122836246;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setCustomEmojiStickerSetThumbnail> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackInfo;

class languagePackString;

class ok;

class setCustomLanguagePack final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<languagePackInfo> info_;
  static jfieldID info_fieldID;
  array<object_ptr<languagePackString>> strings_;
  static jfieldID strings_fieldID;

  setCustomLanguagePack();

  setCustomLanguagePack(object_ptr<languagePackInfo> &&info_, array<object_ptr<languagePackString>> &&strings_);

  static const std::int32_t ID = -296742819;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setCustomLanguagePack> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class languagePackString;

class ok;

class setCustomLanguagePackString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;
  object_ptr<languagePackString> new_string_;
  static jfieldID new_string_fieldID;

  setCustomLanguagePackString();

  setCustomLanguagePackString(string const &language_pack_id_, object_ptr<languagePackString> &&new_string_);

  static const std::int32_t ID = 1316365592;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setCustomLanguagePackString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setDatabaseEncryptionKey final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes new_encryption_key_;
  static jfieldID new_encryption_key_fieldID;

  setDatabaseEncryptionKey();

  explicit setDatabaseEncryptionKey(bytes const &new_encryption_key_);

  static const std::int32_t ID = -1204599371;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setDatabaseEncryptionKey> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BackgroundType;

class InputBackground;

class background;

class setDefaultBackground final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputBackground> background_;
  static jfieldID background_fieldID;
  object_ptr<BackgroundType> type_;
  static jfieldID type_fieldID;
  bool for_dark_theme_;
  static jfieldID for_dark_theme_fieldID;

  setDefaultBackground();

  setDefaultBackground(object_ptr<InputBackground> &&background_, object_ptr<BackgroundType> &&type_, bool for_dark_theme_);

  static const std::int32_t ID = -1982748511;

  using ReturnType = object_ptr<background>;

  static object_ptr<setDefaultBackground> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministratorRights;

class ok;

class setDefaultChannelAdministratorRights final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatAdministratorRights> default_channel_administrator_rights_;
  static jfieldID default_channel_administrator_rights_fieldID;

  setDefaultChannelAdministratorRights();

  explicit setDefaultChannelAdministratorRights(object_ptr<chatAdministratorRights> &&default_channel_administrator_rights_);

  static const std::int32_t ID = -234004967;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setDefaultChannelAdministratorRights> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatAdministratorRights;

class ok;

class setDefaultGroupAdministratorRights final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<chatAdministratorRights> default_group_administrator_rights_;
  static jfieldID default_group_administrator_rights_fieldID;

  setDefaultGroupAdministratorRights();

  explicit setDefaultGroupAdministratorRights(object_ptr<chatAdministratorRights> &&default_group_administrator_rights_);

  static const std::int32_t ID = 1700231016;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setDefaultGroupAdministratorRights> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class messageAutoDeleteTime;

class ok;

class setDefaultMessageAutoDeleteTime final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<messageAutoDeleteTime> message_auto_delete_time_;
  static jfieldID message_auto_delete_time_fieldID;

  setDefaultMessageAutoDeleteTime();

  explicit setDefaultMessageAutoDeleteTime(object_ptr<messageAutoDeleteTime> &&message_auto_delete_time_);

  static const std::int32_t ID = -1772301460;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setDefaultMessageAutoDeleteTime> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class setDefaultReactionType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;

  setDefaultReactionType();

  explicit setDefaultReactionType(object_ptr<ReactionType> &&reaction_type_);

  static const std::int32_t ID = 1694730813;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setDefaultReactionType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatus;

class ok;

class setEmojiStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;

  setEmojiStatus();

  explicit setEmojiStatus(object_ptr<emojiStatus> &&emoji_status_);

  static const std::int32_t ID = -1829224867;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setEmojiStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setFileGenerationProgress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 generation_id_;
  static jfieldID generation_id_fieldID;
  int53 expected_size_;
  static jfieldID expected_size_fieldID;
  int53 local_prefix_size_;
  static jfieldID local_prefix_size_fieldID;

  setFileGenerationProgress();

  setFileGenerationProgress(int64 generation_id_, int53 expected_size_, int53 local_prefix_size_);

  static const std::int32_t ID = 1836403518;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setFileGenerationProgress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chatNotificationSettings;

class ok;

class setForumTopicNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  object_ptr<chatNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  setForumTopicNotificationSettings();

  setForumTopicNotificationSettings(int53 chat_id_, int53 message_thread_id_, object_ptr<chatNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = 524498023;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setForumTopicNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class message;

class setGameScore final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  bool edit_message_;
  static jfieldID edit_message_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 score_;
  static jfieldID score_fieldID;
  bool force_;
  static jfieldID force_fieldID;

  setGameScore();

  setGameScore(int53 chat_id_, int53 message_id_, bool edit_message_, int53 user_id_, int32 score_, bool force_);

  static const std::int32_t ID = 2127359430;

  using ReturnType = object_ptr<message>;

  static object_ptr<setGameScore> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setGiftResalePrice final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  int53 resale_star_count_;
  static jfieldID resale_star_count_fieldID;

  setGiftResalePrice();

  setGiftResalePrice(string const &received_gift_id_, int53 resale_star_count_);

  static const std::int32_t ID = -1430492787;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setGiftResalePrice> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class giftSettings;

class ok;

class setGiftSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<giftSettings> settings_;
  static jfieldID settings_fieldID;

  setGiftSettings();

  explicit setGiftSettings(object_ptr<giftSettings> &&settings_);

  static const std::int32_t ID = -519330046;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setGiftSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setGroupCallParticipantIsSpeaking final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int32 audio_source_;
  static jfieldID audio_source_fieldID;
  bool is_speaking_;
  static jfieldID is_speaking_fieldID;

  setGroupCallParticipantIsSpeaking();

  setGroupCallParticipantIsSpeaking(int32 group_call_id_, int32 audio_source_, bool is_speaking_);

  static const std::int32_t ID = 927506917;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setGroupCallParticipantIsSpeaking> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class setGroupCallParticipantVolumeLevel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  int32 volume_level_;
  static jfieldID volume_level_fieldID;

  setGroupCallParticipantVolumeLevel();

  setGroupCallParticipantVolumeLevel(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, int32 volume_level_);

  static const std::int32_t ID = -1753769944;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setGroupCallParticipantVolumeLevel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setInactiveSessionTtl final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 inactive_session_ttl_days_;
  static jfieldID inactive_session_ttl_days_fieldID;

  setInactiveSessionTtl();

  explicit setInactiveSessionTtl(int32 inactive_session_ttl_days_);

  static const std::int32_t ID = 1570548048;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setInactiveSessionTtl> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setInlineGameScore final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string inline_message_id_;
  static jfieldID inline_message_id_fieldID;
  bool edit_message_;
  static jfieldID edit_message_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  int32 score_;
  static jfieldID score_fieldID;
  bool force_;
  static jfieldID force_fieldID;

  setInlineGameScore();

  setInlineGameScore(string const &inline_message_id_, bool edit_message_, int53 user_id_, int32 score_, bool force_);

  static const std::int32_t ID = -948871797;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setInlineGameScore> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class LogStream;

class ok;

class setLogStream final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<LogStream> log_stream_;
  static jfieldID log_stream_fieldID;

  setLogStream();

  explicit setLogStream(object_ptr<LogStream> &&log_stream_);

  static const std::int32_t ID = -1364199535;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setLogStream> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setLogTagVerbosityLevel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string tag_;
  static jfieldID tag_fieldID;
  int32 new_verbosity_level_;
  static jfieldID new_verbosity_level_fieldID;

  setLogTagVerbosityLevel();

  setLogTagVerbosityLevel(string const &tag_, int32 new_verbosity_level_);

  static const std::int32_t ID = -2095589738;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setLogTagVerbosityLevel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setLogVerbosityLevel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 new_verbosity_level_;
  static jfieldID new_verbosity_level_fieldID;

  setLogVerbosityLevel();

  explicit setLogVerbosityLevel(int32 new_verbosity_level_);

  static const std::int32_t ID = -303429678;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setLogVerbosityLevel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emailAddressAuthenticationCodeInfo;

class setLoginEmailAddress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string new_login_email_address_;
  static jfieldID new_login_email_address_fieldID;

  setLoginEmailAddress();

  explicit setLoginEmailAddress(string const &new_login_email_address_);

  static const std::int32_t ID = 935019476;

  using ReturnType = object_ptr<emailAddressAuthenticationCodeInfo>;

  static object_ptr<setLoginEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class botMenuButton;

class ok;

class setMenuButton final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<botMenuButton> menu_button_;
  static jfieldID menu_button_fieldID;

  setMenuButton();

  setMenuButton(int53 user_id_, object_ptr<botMenuButton> &&menu_button_);

  static const std::int32_t ID = -1269841599;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setMenuButton> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class ok;

class setMessageFactCheck final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;

  setMessageFactCheck();

  setMessageFactCheck(int53 chat_id_, int53 message_id_, object_ptr<formattedText> &&text_);

  static const std::int32_t ID = -4309752;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setMessageFactCheck> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class setMessageReactions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  array<object_ptr<ReactionType>> reaction_types_;
  static jfieldID reaction_types_fieldID;
  bool is_big_;
  static jfieldID is_big_fieldID;

  setMessageReactions();

  setMessageReactions(int53 chat_id_, int53 message_id_, array<object_ptr<ReactionType>> &&reaction_types_, bool is_big_);

  static const std::int32_t ID = -372524900;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setMessageReactions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class BlockList;

class MessageSender;

class ok;

class setMessageSenderBlockList final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> sender_id_;
  static jfieldID sender_id_fieldID;
  object_ptr<BlockList> block_list_;
  static jfieldID block_list_fieldID;

  setMessageSenderBlockList();

  setMessageSenderBlockList(object_ptr<MessageSender> &&sender_id_, object_ptr<BlockList> &&block_list_);

  static const std::int32_t ID = -1987355503;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setMessageSenderBlockList> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class setMessageSenderBotVerification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  object_ptr<MessageSender> verified_id_;
  static jfieldID verified_id_fieldID;
  string custom_description_;
  static jfieldID custom_description_fieldID;

  setMessageSenderBotVerification();

  setMessageSenderBotVerification(int53 bot_user_id_, object_ptr<MessageSender> &&verified_id_, string const &custom_description_);

  static const std::int32_t ID = -1262364086;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setMessageSenderBotVerification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string first_name_;
  static jfieldID first_name_fieldID;
  string last_name_;
  static jfieldID last_name_fieldID;

  setName();

  setName(string const &first_name_, string const &last_name_);

  static const std::int32_t ID = 1711693584;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NetworkType;

class ok;

class setNetworkType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NetworkType> type_;
  static jfieldID type_fieldID;

  setNetworkType();

  explicit setNetworkType(object_ptr<NetworkType> &&type_);

  static const std::int32_t ID = -701635234;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setNetworkType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class newChatPrivacySettings;

class ok;

class setNewChatPrivacySettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<newChatPrivacySettings> settings_;
  static jfieldID settings_fieldID;

  setNewChatPrivacySettings();

  explicit setNewChatPrivacySettings(object_ptr<newChatPrivacySettings> &&settings_);

  static const std::int32_t ID = 1774139215;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setNewChatPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class OptionValue;

class ok;

class setOption final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<OptionValue> value_;
  static jfieldID value_fieldID;

  setOption();

  setOption(string const &name_, object_ptr<OptionValue> &&value_);

  static const std::int32_t ID = 2114670322;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setOption> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PaidReactionType;

class ok;

class setPaidMessageReactionType final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<PaidReactionType> type_;
  static jfieldID type_fieldID;

  setPaidMessageReactionType();

  setPaidMessageReactionType(int53 chat_id_, int53 message_id_, object_ptr<PaidReactionType> &&type_);

  static const std::int32_t ID = -829934930;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPaidMessageReactionType> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputPassportElement;

class PassportElement;

class setPassportElement final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputPassportElement> element_;
  static jfieldID element_fieldID;
  string password_;
  static jfieldID password_fieldID;

  setPassportElement();

  setPassportElement(object_ptr<InputPassportElement> &&element_, string const &password_);

  static const std::int32_t ID = 2068173212;

  using ReturnType = object_ptr<PassportElement>;

  static object_ptr<setPassportElement> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class inputPassportElementError;

class ok;

class setPassportElementErrors final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  array<object_ptr<inputPassportElementError>> errors_;
  static jfieldID errors_fieldID;

  setPassportElementErrors();

  setPassportElementErrors(int53 user_id_, array<object_ptr<inputPassportElementError>> &&errors_);

  static const std::int32_t ID = -2056754881;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPassportElementErrors> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class setPassword final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string old_password_;
  static jfieldID old_password_fieldID;
  string new_password_;
  static jfieldID new_password_fieldID;
  string new_hint_;
  static jfieldID new_hint_fieldID;
  bool set_recovery_email_address_;
  static jfieldID set_recovery_email_address_fieldID;
  string new_recovery_email_address_;
  static jfieldID new_recovery_email_address_fieldID;

  setPassword();

  setPassword(string const &old_password_, string const &new_password_, string const &new_hint_, bool set_recovery_email_address_, string const &new_recovery_email_address_);

  static const std::int32_t ID = -1193589027;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<setPassword> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setPersonalChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  setPersonalChat();

  explicit setPersonalChat(int53 chat_id_);

  static const std::int32_t ID = -1068782668;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPersonalChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ok;

class setPinnedChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  array<int53> chat_ids_;
  static jfieldID chat_ids_fieldID;

  setPinnedChats();

  setPinnedChats(object_ptr<ChatList> &&chat_list_, array<int53> &&chat_ids_);

  static const std::int32_t ID = -695640000;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPinnedChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setPinnedForumTopics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_thread_ids_;
  static jfieldID message_thread_ids_fieldID;

  setPinnedForumTopics();

  setPinnedForumTopics(int53 chat_id_, array<int53> &&message_thread_ids_);

  static const std::int32_t ID = -475084011;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPinnedForumTopics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class setPinnedGifts final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<MessageSender> owner_id_;
  static jfieldID owner_id_fieldID;
  array<string> received_gift_ids_;
  static jfieldID received_gift_ids_fieldID;

  setPinnedGifts();

  setPinnedGifts(object_ptr<MessageSender> &&owner_id_, array<string> &&received_gift_ids_);

  static const std::int32_t ID = 1613526306;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPinnedGifts> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setPinnedSavedMessagesTopics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int53> saved_messages_topic_ids_;
  static jfieldID saved_messages_topic_ids_fieldID;

  setPinnedSavedMessagesTopics();

  explicit setPinnedSavedMessagesTopics(array<int53> &&saved_messages_topic_ids_);

  static const std::int32_t ID = -194818924;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPinnedSavedMessagesTopics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setPollAnswer final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  array<int32> option_ids_;
  static jfieldID option_ids_fieldID;

  setPollAnswer();

  setPollAnswer(int53 chat_id_, int53 message_id_, array<int32> &&option_ids_);

  static const std::int32_t ID = -1399388792;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setPollAnswer> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setProfileAccentColor final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 profile_accent_color_id_;
  static jfieldID profile_accent_color_id_fieldID;
  int64 profile_background_custom_emoji_id_;
  static jfieldID profile_background_custom_emoji_id_fieldID;

  setProfileAccentColor();

  setProfileAccentColor(int32 profile_accent_color_id_, int64 profile_background_custom_emoji_id_);

  static const std::int32_t ID = -1986281112;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setProfileAccentColor> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class setProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;
  bool is_public_;
  static jfieldID is_public_fieldID;

  setProfilePhoto();

  setProfilePhoto(object_ptr<InputChatPhoto> &&photo_, bool is_public_);

  static const std::int32_t ID = -2048260627;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setQuickReplyShortcutName final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 shortcut_id_;
  static jfieldID shortcut_id_fieldID;
  string name_;
  static jfieldID name_fieldID;

  setQuickReplyShortcutName();

  setQuickReplyShortcutName(int32 shortcut_id_, string const &name_);

  static const std::int32_t ID = 186709105;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setQuickReplyShortcutName> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class reactionNotificationSettings;

class setReactionNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<reactionNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  setReactionNotificationSettings();

  explicit setReactionNotificationSettings(object_ptr<reactionNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = 1186124949;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setReactionNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class readDatePrivacySettings;

class setReadDatePrivacySettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<readDatePrivacySettings> settings_;
  static jfieldID settings_fieldID;

  setReadDatePrivacySettings();

  explicit setReadDatePrivacySettings(object_ptr<readDatePrivacySettings> &&settings_);

  static const std::int32_t ID = 493913782;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setReadDatePrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class passwordState;

class setRecoveryEmailAddress final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string password_;
  static jfieldID password_fieldID;
  string new_recovery_email_address_;
  static jfieldID new_recovery_email_address_fieldID;

  setRecoveryEmailAddress();

  setRecoveryEmailAddress(string const &password_, string const &new_recovery_email_address_);

  static const std::int32_t ID = -1981836385;

  using ReturnType = object_ptr<passwordState>;

  static object_ptr<setRecoveryEmailAddress> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class setSavedMessagesTagLabel final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ReactionType> tag_;
  static jfieldID tag_fieldID;
  string label_;
  static jfieldID label_fieldID;

  setSavedMessagesTagLabel();

  setSavedMessagesTagLabel(object_ptr<ReactionType> &&tag_, string const &label_);

  static const std::int32_t ID = -1338323696;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setSavedMessagesTagLabel> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class NotificationSettingsScope;

class ok;

class scopeNotificationSettings;

class setScopeNotificationSettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<NotificationSettingsScope> scope_;
  static jfieldID scope_fieldID;
  object_ptr<scopeNotificationSettings> notification_settings_;
  static jfieldID notification_settings_fieldID;

  setScopeNotificationSettings();

  setScopeNotificationSettings(object_ptr<NotificationSettingsScope> &&scope_, object_ptr<scopeNotificationSettings> &&notification_settings_);

  static const std::int32_t ID = -2049984966;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setScopeNotificationSettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class setStickerEmojis final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  string emojis_;
  static jfieldID emojis_fieldID;

  setStickerEmojis();

  setStickerEmojis(object_ptr<InputFile> &&sticker_, string const &emojis_);

  static const std::int32_t ID = -638843855;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerEmojis> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class setStickerKeywords final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  array<string> keywords_;
  static jfieldID keywords_fieldID;

  setStickerKeywords();

  setStickerKeywords(object_ptr<InputFile> &&sticker_, array<string> &&keywords_);

  static const std::int32_t ID = 137223565;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerKeywords> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class maskPosition;

class ok;

class setStickerMaskPosition final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  object_ptr<maskPosition> mask_position_;
  static jfieldID mask_position_fieldID;

  setStickerMaskPosition();

  setStickerMaskPosition(object_ptr<InputFile> &&sticker_, object_ptr<maskPosition> &&mask_position_);

  static const std::int32_t ID = 1202280912;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerMaskPosition> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class ok;

class setStickerPositionInSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;
  int32 position_;
  static jfieldID position_fieldID;

  setStickerPositionInSet();

  setStickerPositionInSet(object_ptr<InputFile> &&sticker_, int32 position_);

  static const std::int32_t ID = 2075281185;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerPositionInSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class StickerFormat;

class ok;

class setStickerSetThumbnail final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string name_;
  static jfieldID name_fieldID;
  object_ptr<InputFile> thumbnail_;
  static jfieldID thumbnail_fieldID;
  object_ptr<StickerFormat> format_;
  static jfieldID format_fieldID;

  setStickerSetThumbnail();

  setStickerSetThumbnail(int53 user_id_, string const &name_, object_ptr<InputFile> &&thumbnail_, object_ptr<StickerFormat> &&format_);

  static const std::int32_t ID = 1677617458;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerSetThumbnail> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setStickerSetTitle final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string name_;
  static jfieldID name_fieldID;
  string title_;
  static jfieldID title_fieldID;

  setStickerSetTitle();

  setStickerSetTitle(string const &name_, string const &title_);

  static const std::int32_t ID = 1693004706;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStickerSetTitle> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class StoryPrivacySettings;

class ok;

class setStoryPrivacySettings final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  object_ptr<StoryPrivacySettings> privacy_settings_;
  static jfieldID privacy_settings_fieldID;

  setStoryPrivacySettings();

  setStoryPrivacySettings(int32 story_id_, object_ptr<StoryPrivacySettings> &&privacy_settings_);

  static const std::int32_t ID = -655801550;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStoryPrivacySettings> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReactionType;

class ok;

class setStoryReaction final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  object_ptr<ReactionType> reaction_type_;
  static jfieldID reaction_type_fieldID;
  bool update_recent_reactions_;
  static jfieldID update_recent_reactions_fieldID;

  setStoryReaction();

  setStoryReaction(int53 story_poster_chat_id_, int32 story_id_, object_ptr<ReactionType> &&reaction_type_, bool update_recent_reactions_);

  static const std::int32_t ID = 250731529;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setStoryReaction> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setSupergroupCustomEmojiStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  int64 custom_emoji_sticker_set_id_;
  static jfieldID custom_emoji_sticker_set_id_fieldID;

  setSupergroupCustomEmojiStickerSet();

  setSupergroupCustomEmojiStickerSet(int53 supergroup_id_, int64 custom_emoji_sticker_set_id_);

  static const std::int32_t ID = 1328894639;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setSupergroupCustomEmojiStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setSupergroupStickerSet final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  int64 sticker_set_id_;
  static jfieldID sticker_set_id_fieldID;

  setSupergroupStickerSet();

  setSupergroupStickerSet(int53 supergroup_id_, int64 sticker_set_id_);

  static const std::int32_t ID = -2056344215;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setSupergroupStickerSet> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setSupergroupUnrestrictBoostCount final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  int32 unrestrict_boost_count_;
  static jfieldID unrestrict_boost_count_fieldID;

  setSupergroupUnrestrictBoostCount();

  setSupergroupUnrestrictBoostCount(int53 supergroup_id_, int32 unrestrict_boost_count_);

  static const std::int32_t ID = 969814179;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setSupergroupUnrestrictBoostCount> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setSupergroupUsername final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  string username_;
  static jfieldID username_fieldID;

  setSupergroupUsername();

  setSupergroupUsername(int53 supergroup_id_, string const &username_);

  static const std::int32_t ID = 1346325252;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setSupergroupUsername> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setTdlibParameters final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool use_test_dc_;
  static jfieldID use_test_dc_fieldID;
  string database_directory_;
  static jfieldID database_directory_fieldID;
  string files_directory_;
  static jfieldID files_directory_fieldID;
  bytes database_encryption_key_;
  static jfieldID database_encryption_key_fieldID;
  bool use_file_database_;
  static jfieldID use_file_database_fieldID;
  bool use_chat_info_database_;
  static jfieldID use_chat_info_database_fieldID;
  bool use_message_database_;
  static jfieldID use_message_database_fieldID;
  bool use_secret_chats_;
  static jfieldID use_secret_chats_fieldID;
  int32 api_id_;
  static jfieldID api_id_fieldID;
  string api_hash_;
  static jfieldID api_hash_fieldID;
  string system_language_code_;
  static jfieldID system_language_code_fieldID;
  string device_model_;
  static jfieldID device_model_fieldID;
  string system_version_;
  static jfieldID system_version_fieldID;
  string application_version_;
  static jfieldID application_version_fieldID;

  setTdlibParameters();

  setTdlibParameters(bool use_test_dc_, string const &database_directory_, string const &files_directory_, bytes const &database_encryption_key_, bool use_file_database_, bool use_chat_info_database_, bool use_message_database_, bool use_secret_chats_, int32 api_id_, string const &api_hash_, string const &system_language_code_, string const &device_model_, string const &system_version_, string const &application_version_);

  static const std::int32_t ID = -775883218;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setTdlibParameters> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class emojiStatus;

class ok;

class setUserEmojiStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<emojiStatus> emoji_status_;
  static jfieldID emoji_status_fieldID;

  setUserEmojiStatus();

  setUserEmojiStatus(int53 user_id_, object_ptr<emojiStatus> &&emoji_status_);

  static const std::int32_t ID = -451519541;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setUserEmojiStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class setUserPersonalProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;

  setUserPersonalProfilePhoto();

  setUserPersonalProfilePhoto(int53 user_id_, object_ptr<InputChatPhoto> &&photo_);

  static const std::int32_t ID = 464136438;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setUserPersonalProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class UserPrivacySetting;

class ok;

class userPrivacySettingRules;

class setUserPrivacySettingRules final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<UserPrivacySetting> setting_;
  static jfieldID setting_fieldID;
  object_ptr<userPrivacySettingRules> rules_;
  static jfieldID rules_fieldID;

  setUserPrivacySettingRules();

  setUserPrivacySettingRules(object_ptr<UserPrivacySetting> &&setting_, object_ptr<userPrivacySettingRules> &&rules_);

  static const std::int32_t ID = -473812741;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setUserPrivacySettingRules> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class userSupportInfo;

class setUserSupportInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<formattedText> message_;
  static jfieldID message_fieldID;

  setUserSupportInfo();

  setUserSupportInfo(int53 user_id_, object_ptr<formattedText> &&message_);

  static const std::int32_t ID = -2088986621;

  using ReturnType = object_ptr<userSupportInfo>;

  static object_ptr<setUserSupportInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setUsername final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;

  setUsername();

  explicit setUsername(string const &username_);

  static const std::int32_t ID = 439901214;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setUsername> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class setVideoChatDefaultParticipant final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  object_ptr<MessageSender> default_participant_id_;
  static jfieldID default_participant_id_fieldID;

  setVideoChatDefaultParticipant();

  setVideoChatDefaultParticipant(int53 chat_id_, object_ptr<MessageSender> &&default_participant_id_);

  static const std::int32_t ID = -240749901;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setVideoChatDefaultParticipant> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class setVideoChatTitle final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  string title_;
  static jfieldID title_fieldID;

  setVideoChatTitle();

  setVideoChatTitle(int32 group_call_id_, string const &title_);

  static const std::int32_t ID = 1915482994;

  using ReturnType = object_ptr<ok>;

  static object_ptr<setVideoChatTitle> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class shareChatWithBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 button_id_;
  static jfieldID button_id_fieldID;
  int53 shared_chat_id_;
  static jfieldID shared_chat_id_fieldID;
  bool only_check_;
  static jfieldID only_check_fieldID;

  shareChatWithBot();

  shareChatWithBot(int53 chat_id_, int53 message_id_, int32 button_id_, int53 shared_chat_id_, bool only_check_);

  static const std::int32_t ID = -1504507166;

  using ReturnType = object_ptr<ok>;

  static object_ptr<shareChatWithBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class sharePhoneNumber final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;

  sharePhoneNumber();

  explicit sharePhoneNumber(int53 user_id_);

  static const std::int32_t ID = 1097130069;

  using ReturnType = object_ptr<ok>;

  static object_ptr<sharePhoneNumber> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class shareUsersWithBot final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  int32 button_id_;
  static jfieldID button_id_fieldID;
  array<int53> shared_user_ids_;
  static jfieldID shared_user_ids_fieldID;
  bool only_check_;
  static jfieldID only_check_fieldID;

  shareUsersWithBot();

  shareUsersWithBot(int53 chat_id_, int53 message_id_, int32 button_id_, array<int53> &&shared_user_ids_, bool only_check_);

  static const std::int32_t ID = -1574608333;

  using ReturnType = object_ptr<ok>;

  static object_ptr<shareUsersWithBot> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class startGroupCallRecording final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  string title_;
  static jfieldID title_fieldID;
  bool record_video_;
  static jfieldID record_video_fieldID;
  bool use_portrait_orientation_;
  static jfieldID use_portrait_orientation_fieldID;

  startGroupCallRecording();

  startGroupCallRecording(int32 group_call_id_, string const &title_, bool record_video_, bool use_portrait_orientation_);

  static const std::int32_t ID = 1757774971;

  using ReturnType = object_ptr<ok>;

  static object_ptr<startGroupCallRecording> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class text;

class startGroupCallScreenSharing final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  int32 audio_source_id_;
  static jfieldID audio_source_id_fieldID;
  string payload_;
  static jfieldID payload_fieldID;

  startGroupCallScreenSharing();

  startGroupCallScreenSharing(int32 group_call_id_, int32 audio_source_id_, string const &payload_);

  static const std::int32_t ID = -884068051;

  using ReturnType = object_ptr<text>;

  static object_ptr<startGroupCallScreenSharing> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class startScheduledVideoChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;

  startScheduledVideoChat();

  explicit startScheduledVideoChat(int32 group_call_id_);

  static const std::int32_t ID = -1300829822;

  using ReturnType = object_ptr<ok>;

  static object_ptr<startScheduledVideoChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class businessMessage;

class stopBusinessPoll final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  stopBusinessPoll();

  stopBusinessPoll(string const &business_connection_id_, int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = -1142218400;

  using ReturnType = object_ptr<businessMessage>;

  static object_ptr<stopBusinessPoll> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ReplyMarkup;

class ok;

class stopPoll final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  object_ptr<ReplyMarkup> reply_markup_;
  static jfieldID reply_markup_fieldID;

  stopPoll();

  stopPoll(int53 chat_id_, int53 message_id_, object_ptr<ReplyMarkup> &&reply_markup_);

  static const std::int32_t ID = 1659374253;

  using ReturnType = object_ptr<ok>;

  static object_ptr<stopPoll> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputChatPhoto;

class ok;

class suggestUserProfilePhoto final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<InputChatPhoto> photo_;
  static jfieldID photo_fieldID;

  suggestUserProfilePhoto();

  suggestUserProfilePhoto(int53 user_id_, object_ptr<InputChatPhoto> &&photo_);

  static const std::int32_t ID = -1788742557;

  using ReturnType = object_ptr<ok>;

  static object_ptr<suggestUserProfilePhoto> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class synchronizeLanguagePack final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string language_pack_id_;
  static jfieldID language_pack_id_fieldID;

  synchronizeLanguagePack();

  explicit synchronizeLanguagePack(string const &language_pack_id_);

  static const std::int32_t ID = -2065307858;

  using ReturnType = object_ptr<ok>;

  static object_ptr<synchronizeLanguagePack> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class terminateAllOtherSessions final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  terminateAllOtherSessions();

  static const std::int32_t ID = 1874485523;

  using ReturnType = object_ptr<ok>;

  static object_ptr<terminateAllOtherSessions> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class terminateSession final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 session_id_;
  static jfieldID session_id_fieldID;

  terminateSession();

  explicit terminateSession(int64 session_id_);

  static const std::int32_t ID = -407385812;

  using ReturnType = object_ptr<ok>;

  static object_ptr<terminateSession> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testBytes;

class testCallBytes final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bytes x_;
  static jfieldID x_fieldID;

  testCallBytes();

  explicit testCallBytes(bytes const &x_);

  static const std::int32_t ID = -736011607;

  using ReturnType = object_ptr<testBytes>;

  static object_ptr<testCallBytes> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class testCallEmpty final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  testCallEmpty();

  static const std::int32_t ID = -627291626;

  using ReturnType = object_ptr<ok>;

  static object_ptr<testCallEmpty> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testString;

class testCallString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string x_;
  static jfieldID x_fieldID;

  testCallString();

  explicit testCallString(string const &x_);

  static const std::int32_t ID = -1732818385;

  using ReturnType = object_ptr<testString>;

  static object_ptr<testCallString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testVectorInt;

class testCallVectorInt final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int32> x_;
  static jfieldID x_fieldID;

  testCallVectorInt();

  explicit testCallVectorInt(array<int32> &&x_);

  static const std::int32_t ID = -2137277793;

  using ReturnType = object_ptr<testVectorInt>;

  static object_ptr<testCallVectorInt> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testInt;

class testVectorIntObject;

class testCallVectorIntObject final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<testInt>> x_;
  static jfieldID x_fieldID;

  testCallVectorIntObject();

  explicit testCallVectorIntObject(array<object_ptr<testInt>> &&x_);

  static const std::int32_t ID = 1825428218;

  using ReturnType = object_ptr<testVectorIntObject>;

  static object_ptr<testCallVectorIntObject> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testVectorString;

class testCallVectorString final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<string> x_;
  static jfieldID x_fieldID;

  testCallVectorString();

  explicit testCallVectorString(array<string> &&x_);

  static const std::int32_t ID = -408600900;

  using ReturnType = object_ptr<testVectorString>;

  static object_ptr<testCallVectorString> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testString;

class testVectorStringObject;

class testCallVectorStringObject final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<object_ptr<testString>> x_;
  static jfieldID x_fieldID;

  testCallVectorStringObject();

  explicit testCallVectorStringObject(array<object_ptr<testString>> &&x_);

  static const std::int32_t ID = 1527666429;

  using ReturnType = object_ptr<testVectorStringObject>;

  static object_ptr<testCallVectorStringObject> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class testGetDifference final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  testGetDifference();

  static const std::int32_t ID = 1747084069;

  using ReturnType = object_ptr<ok>;

  static object_ptr<testGetDifference> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class testNetwork final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  testNetwork();

  static const std::int32_t ID = -1343998901;

  using ReturnType = object_ptr<ok>;

  static object_ptr<testNetwork> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ProxyType;

class ok;

class testProxy final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string server_;
  static jfieldID server_fieldID;
  int32 port_;
  static jfieldID port_fieldID;
  object_ptr<ProxyType> type_;
  static jfieldID type_fieldID;
  int32 dc_id_;
  static jfieldID dc_id_fieldID;
  double timeout_;
  static jfieldID timeout_fieldID;

  testProxy();

  testProxy(string const &server_, int32 port_, object_ptr<ProxyType> &&type_, int32 dc_id_, double timeout_);

  static const std::int32_t ID = -1197366626;

  using ReturnType = object_ptr<ok>;

  static object_ptr<testProxy> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class error;

class testReturnError final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<error> error_;
  static jfieldID error_fieldID;

  testReturnError();

  explicit testReturnError(object_ptr<error> &&error_);

  static const std::int32_t ID = 455179506;

  using ReturnType = object_ptr<error>;

  static object_ptr<testReturnError> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class testInt;

class testSquareInt final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 x_;
  static jfieldID x_fieldID;

  testSquareInt();

  explicit testSquareInt(int32 x_);

  static const std::int32_t ID = -60135024;

  using ReturnType = object_ptr<testInt>;

  static object_ptr<testSquareInt> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class Update;

class testUseUpdate final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;

  testUseUpdate();

  static const std::int32_t ID = 717094686;

  using ReturnType = object_ptr<Update>;

  static object_ptr<testUseUpdate> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleAllDownloadsArePaused final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool are_paused_;
  static jfieldID are_paused_fieldID;

  toggleAllDownloadsArePaused();

  explicit toggleAllDownloadsArePaused(bool are_paused_);

  static const std::int32_t ID = 1251512322;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleAllDownloadsArePaused> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleBotCanManageEmojiStatus final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  bool can_manage_emoji_status_;
  static jfieldID can_manage_emoji_status_fieldID;

  toggleBotCanManageEmojiStatus();

  toggleBotCanManageEmojiStatus(int53 bot_user_id_, bool can_manage_emoji_status_);

  static const std::int32_t ID = 622495770;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleBotCanManageEmojiStatus> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleBotIsAddedToAttachmentMenu final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  bool is_added_;
  static jfieldID is_added_fieldID;
  bool allow_write_access_;
  static jfieldID allow_write_access_fieldID;

  toggleBotIsAddedToAttachmentMenu();

  toggleBotIsAddedToAttachmentMenu(int53 bot_user_id_, bool is_added_, bool allow_write_access_);

  static const std::int32_t ID = -1906712934;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleBotIsAddedToAttachmentMenu> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleBotUsernameIsActive final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 bot_user_id_;
  static jfieldID bot_user_id_fieldID;
  string username_;
  static jfieldID username_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;

  toggleBotUsernameIsActive();

  toggleBotUsernameIsActive(int53 bot_user_id_, string const &username_, bool is_active_);

  static const std::int32_t ID = 2036569097;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleBotUsernameIsActive> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleBusinessConnectedBotChatIsPaused final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;

  toggleBusinessConnectedBotChatIsPaused();

  toggleBusinessConnectedBotChatIsPaused(int53 chat_id_, bool is_paused_);

  static const std::int32_t ID = 1328957509;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleBusinessConnectedBotChatIsPaused> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatDefaultDisableNotification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool default_disable_notification_;
  static jfieldID default_disable_notification_fieldID;

  toggleChatDefaultDisableNotification();

  toggleChatDefaultDisableNotification(int53 chat_id_, bool default_disable_notification_);

  static const std::int32_t ID = 314794002;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatDefaultDisableNotification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatFolderTags final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool are_tags_enabled_;
  static jfieldID are_tags_enabled_fieldID;

  toggleChatFolderTags();

  explicit toggleChatFolderTags(bool are_tags_enabled_);

  static const std::int32_t ID = -2092209084;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatFolderTags> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatGiftNotifications final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool are_enabled_;
  static jfieldID are_enabled_fieldID;

  toggleChatGiftNotifications();

  toggleChatGiftNotifications(int53 chat_id_, bool are_enabled_);

  static const std::int32_t ID = -2069429154;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatGiftNotifications> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatHasProtectedContent final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool has_protected_content_;
  static jfieldID has_protected_content_fieldID;

  toggleChatHasProtectedContent();

  toggleChatHasProtectedContent(int53 chat_id_, bool has_protected_content_);

  static const std::int32_t ID = 975231309;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatHasProtectedContent> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatIsMarkedAsUnread final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_marked_as_unread_;
  static jfieldID is_marked_as_unread_fieldID;

  toggleChatIsMarkedAsUnread();

  toggleChatIsMarkedAsUnread(int53 chat_id_, bool is_marked_as_unread_);

  static const std::int32_t ID = -986129697;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatIsMarkedAsUnread> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ChatList;

class ok;

class toggleChatIsPinned final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<ChatList> chat_list_;
  static jfieldID chat_list_fieldID;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  toggleChatIsPinned();

  toggleChatIsPinned(object_ptr<ChatList> &&chat_list_, int53 chat_id_, bool is_pinned_);

  static const std::int32_t ID = -1485429186;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatIsPinned> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatIsTranslatable final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_translatable_;
  static jfieldID is_translatable_fieldID;

  toggleChatIsTranslatable();

  toggleChatIsTranslatable(int53 chat_id_, bool is_translatable_);

  static const std::int32_t ID = -1812345889;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatIsTranslatable> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleChatViewAsTopics final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool view_as_topics_;
  static jfieldID view_as_topics_fieldID;

  toggleChatViewAsTopics();

  toggleChatViewAsTopics(int53 chat_id_, bool view_as_topics_);

  static const std::int32_t ID = 724009948;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleChatViewAsTopics> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleDownloadIsPaused final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 file_id_;
  static jfieldID file_id_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;

  toggleDownloadIsPaused();

  toggleDownloadIsPaused(int32 file_id_, bool is_paused_);

  static const std::int32_t ID = -947493099;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleDownloadIsPaused> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleForumTopicIsClosed final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  bool is_closed_;
  static jfieldID is_closed_fieldID;

  toggleForumTopicIsClosed();

  toggleForumTopicIsClosed(int53 chat_id_, int53 message_thread_id_, bool is_closed_);

  static const std::int32_t ID = -949712141;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleForumTopicIsClosed> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleForumTopicIsPinned final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  toggleForumTopicIsPinned();

  toggleForumTopicIsPinned(int53 chat_id_, int53 message_thread_id_, bool is_pinned_);

  static const std::int32_t ID = 1181543092;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleForumTopicIsPinned> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleGeneralForumTopicIsHidden final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  bool is_hidden_;
  static jfieldID is_hidden_fieldID;

  toggleGeneralForumTopicIsHidden();

  toggleGeneralForumTopicIsHidden(int53 chat_id_, bool is_hidden_);

  static const std::int32_t ID = 1595741256;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGeneralForumTopicIsHidden> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleGiftIsSaved final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  bool is_saved_;
  static jfieldID is_saved_fieldID;

  toggleGiftIsSaved();

  toggleGiftIsSaved(string const &received_gift_id_, bool is_saved_);

  static const std::int32_t ID = 693198065;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGiftIsSaved> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleGroupCallIsMyVideoEnabled final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool is_my_video_enabled_;
  static jfieldID is_my_video_enabled_fieldID;

  toggleGroupCallIsMyVideoEnabled();

  toggleGroupCallIsMyVideoEnabled(int32 group_call_id_, bool is_my_video_enabled_);

  static const std::int32_t ID = -1624289030;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGroupCallIsMyVideoEnabled> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleGroupCallIsMyVideoPaused final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool is_my_video_paused_;
  static jfieldID is_my_video_paused_fieldID;

  toggleGroupCallIsMyVideoPaused();

  toggleGroupCallIsMyVideoPaused(int32 group_call_id_, bool is_my_video_paused_);

  static const std::int32_t ID = -478875239;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGroupCallIsMyVideoPaused> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class toggleGroupCallParticipantIsHandRaised final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  bool is_hand_raised_;
  static jfieldID is_hand_raised_fieldID;

  toggleGroupCallParticipantIsHandRaised();

  toggleGroupCallParticipantIsHandRaised(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, bool is_hand_raised_);

  static const std::int32_t ID = -1896127519;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGroupCallParticipantIsHandRaised> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class toggleGroupCallParticipantIsMuted final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  object_ptr<MessageSender> participant_id_;
  static jfieldID participant_id_fieldID;
  bool is_muted_;
  static jfieldID is_muted_fieldID;

  toggleGroupCallParticipantIsMuted();

  toggleGroupCallParticipantIsMuted(int32 group_call_id_, object_ptr<MessageSender> &&participant_id_, bool is_muted_);

  static const std::int32_t ID = -1308093433;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGroupCallParticipantIsMuted> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleGroupCallScreenSharingIsPaused final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool is_paused_;
  static jfieldID is_paused_fieldID;

  toggleGroupCallScreenSharingIsPaused();

  toggleGroupCallScreenSharingIsPaused(int32 group_call_id_, bool is_paused_);

  static const std::int32_t ID = -1602530464;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleGroupCallScreenSharingIsPaused> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleHasSponsoredMessagesEnabled final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  bool has_sponsored_messages_enabled_;
  static jfieldID has_sponsored_messages_enabled_fieldID;

  toggleHasSponsoredMessagesEnabled();

  explicit toggleHasSponsoredMessagesEnabled(bool has_sponsored_messages_enabled_);

  static const std::int32_t ID = 1963285740;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleHasSponsoredMessagesEnabled> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSavedMessagesTopicIsPinned final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 saved_messages_topic_id_;
  static jfieldID saved_messages_topic_id_fieldID;
  bool is_pinned_;
  static jfieldID is_pinned_fieldID;

  toggleSavedMessagesTopicIsPinned();

  toggleSavedMessagesTopicIsPinned(int53 saved_messages_topic_id_, bool is_pinned_);

  static const std::int32_t ID = -1588378164;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSavedMessagesTopicIsPinned> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSessionCanAcceptCalls final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 session_id_;
  static jfieldID session_id_fieldID;
  bool can_accept_calls_;
  static jfieldID can_accept_calls_fieldID;

  toggleSessionCanAcceptCalls();

  toggleSessionCanAcceptCalls(int64 session_id_, bool can_accept_calls_);

  static const std::int32_t ID = 1819027208;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSessionCanAcceptCalls> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSessionCanAcceptSecretChats final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 session_id_;
  static jfieldID session_id_fieldID;
  bool can_accept_secret_chats_;
  static jfieldID can_accept_secret_chats_fieldID;

  toggleSessionCanAcceptSecretChats();

  toggleSessionCanAcceptSecretChats(int64 session_id_, bool can_accept_secret_chats_);

  static const std::int32_t ID = 1000843390;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSessionCanAcceptSecretChats> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleStoryIsPostedToChatPage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 story_poster_chat_id_;
  static jfieldID story_poster_chat_id_fieldID;
  int32 story_id_;
  static jfieldID story_id_fieldID;
  bool is_posted_to_chat_page_;
  static jfieldID is_posted_to_chat_page_fieldID;

  toggleStoryIsPostedToChatPage();

  toggleStoryIsPostedToChatPage(int53 story_poster_chat_id_, int32 story_id_, bool is_posted_to_chat_page_);

  static const std::int32_t ID = -2141806228;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleStoryIsPostedToChatPage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupCanHaveSponsoredMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool can_have_sponsored_messages_;
  static jfieldID can_have_sponsored_messages_fieldID;

  toggleSupergroupCanHaveSponsoredMessages();

  toggleSupergroupCanHaveSponsoredMessages(int53 supergroup_id_, bool can_have_sponsored_messages_);

  static const std::int32_t ID = -1098204302;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupCanHaveSponsoredMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupHasAggressiveAntiSpamEnabled final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool has_aggressive_anti_spam_enabled_;
  static jfieldID has_aggressive_anti_spam_enabled_fieldID;

  toggleSupergroupHasAggressiveAntiSpamEnabled();

  toggleSupergroupHasAggressiveAntiSpamEnabled(int53 supergroup_id_, bool has_aggressive_anti_spam_enabled_);

  static const std::int32_t ID = 1748956943;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupHasAggressiveAntiSpamEnabled> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupHasAutomaticTranslation final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool has_automatic_translation_;
  static jfieldID has_automatic_translation_fieldID;

  toggleSupergroupHasAutomaticTranslation();

  toggleSupergroupHasAutomaticTranslation(int53 supergroup_id_, bool has_automatic_translation_);

  static const std::int32_t ID = -184993048;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupHasAutomaticTranslation> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupHasHiddenMembers final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool has_hidden_members_;
  static jfieldID has_hidden_members_fieldID;

  toggleSupergroupHasHiddenMembers();

  toggleSupergroupHasHiddenMembers(int53 supergroup_id_, bool has_hidden_members_);

  static const std::int32_t ID = -1537892918;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupHasHiddenMembers> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupIsAllHistoryAvailable final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool is_all_history_available_;
  static jfieldID is_all_history_available_fieldID;

  toggleSupergroupIsAllHistoryAvailable();

  toggleSupergroupIsAllHistoryAvailable(int53 supergroup_id_, bool is_all_history_available_);

  static const std::int32_t ID = 1155110478;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupIsAllHistoryAvailable> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupIsBroadcastGroup final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;

  toggleSupergroupIsBroadcastGroup();

  explicit toggleSupergroupIsBroadcastGroup(int53 supergroup_id_);

  static const std::int32_t ID = 884089365;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupIsBroadcastGroup> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupIsForum final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool is_forum_;
  static jfieldID is_forum_fieldID;

  toggleSupergroupIsForum();

  toggleSupergroupIsForum(int53 supergroup_id_, bool is_forum_);

  static const std::int32_t ID = -1771071990;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupIsForum> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupJoinByRequest final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool join_by_request_;
  static jfieldID join_by_request_fieldID;

  toggleSupergroupJoinByRequest();

  toggleSupergroupJoinByRequest(int53 supergroup_id_, bool join_by_request_);

  static const std::int32_t ID = 2111807454;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupJoinByRequest> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupJoinToSendMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool join_to_send_messages_;
  static jfieldID join_to_send_messages_fieldID;

  toggleSupergroupJoinToSendMessages();

  toggleSupergroupJoinToSendMessages(int53 supergroup_id_, bool join_to_send_messages_);

  static const std::int32_t ID = -182022642;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupJoinToSendMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupSignMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  bool sign_messages_;
  static jfieldID sign_messages_fieldID;
  bool show_message_sender_;
  static jfieldID show_message_sender_fieldID;

  toggleSupergroupSignMessages();

  toggleSupergroupSignMessages(int53 supergroup_id_, bool sign_messages_, bool show_message_sender_);

  static const std::int32_t ID = 572268491;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupSignMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleSupergroupUsernameIsActive final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 supergroup_id_;
  static jfieldID supergroup_id_fieldID;
  string username_;
  static jfieldID username_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;

  toggleSupergroupUsernameIsActive();

  toggleSupergroupUsernameIsActive(int53 supergroup_id_, string const &username_, bool is_active_);

  static const std::int32_t ID = -1500811777;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleSupergroupUsernameIsActive> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleUsernameIsActive final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string username_;
  static jfieldID username_fieldID;
  bool is_active_;
  static jfieldID is_active_fieldID;

  toggleUsernameIsActive();

  toggleUsernameIsActive(string const &username_, bool is_active_);

  static const std::int32_t ID = 1244098019;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleUsernameIsActive> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleVideoChatEnabledStartNotification final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool enabled_start_notification_;
  static jfieldID enabled_start_notification_fieldID;

  toggleVideoChatEnabledStartNotification();

  toggleVideoChatEnabledStartNotification(int32 group_call_id_, bool enabled_start_notification_);

  static const std::int32_t ID = 1851489086;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleVideoChatEnabledStartNotification> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class toggleVideoChatMuteNewParticipants final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int32 group_call_id_;
  static jfieldID group_call_id_fieldID;
  bool mute_new_participants_;
  static jfieldID mute_new_participants_fieldID;

  toggleVideoChatMuteNewParticipants();

  toggleVideoChatMuteNewParticipants(int32 group_call_id_, bool mute_new_participants_);

  static const std::int32_t ID = 987023756;

  using ReturnType = object_ptr<ok>;

  static object_ptr<toggleVideoChatMuteNewParticipants> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class transferBusinessAccountStars final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  transferBusinessAccountStars();

  transferBusinessAccountStars(string const &business_connection_id_, int53 star_count_);

  static const std::int32_t ID = 732562464;

  using ReturnType = object_ptr<ok>;

  static object_ptr<transferBusinessAccountStars> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class transferChatOwnership final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  string password_;
  static jfieldID password_fieldID;

  transferChatOwnership();

  transferChatOwnership(int53 chat_id_, int53 user_id_, string const &password_);

  static const std::int32_t ID = 2006977043;

  using ReturnType = object_ptr<ok>;

  static object_ptr<transferChatOwnership> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSender;

class ok;

class transferGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  object_ptr<MessageSender> new_owner_id_;
  static jfieldID new_owner_id_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  transferGift();

  transferGift(string const &business_connection_id_, string const &received_gift_id_, object_ptr<MessageSender> &&new_owner_id_, int53 star_count_);

  static const std::int32_t ID = -1167293126;

  using ReturnType = object_ptr<ok>;

  static object_ptr<transferGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class translateMessageText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;
  string to_language_code_;
  static jfieldID to_language_code_fieldID;

  translateMessageText();

  translateMessageText(int53 chat_id_, int53 message_id_, string const &to_language_code_);

  static const std::int32_t ID = 1405427410;

  using ReturnType = object_ptr<formattedText>;

  static object_ptr<translateMessageText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class formattedText;

class translateText final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<formattedText> text_;
  static jfieldID text_fieldID;
  string to_language_code_;
  static jfieldID to_language_code_fieldID;

  translateText();

  translateText(object_ptr<formattedText> &&text_, string const &to_language_code_);

  static const std::int32_t ID = 623011058;

  using ReturnType = object_ptr<formattedText>;

  static object_ptr<translateText> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class unpinAllChatMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  unpinAllChatMessages();

  explicit unpinAllChatMessages(int53 chat_id_);

  static const std::int32_t ID = -1437805385;

  using ReturnType = object_ptr<ok>;

  static object_ptr<unpinAllChatMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class unpinAllMessageThreadMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_thread_id_;
  static jfieldID message_thread_id_fieldID;

  unpinAllMessageThreadMessages();

  unpinAllMessageThreadMessages(int53 chat_id_, int53 message_thread_id_);

  static const std::int32_t ID = -1211719936;

  using ReturnType = object_ptr<ok>;

  static object_ptr<unpinAllMessageThreadMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class unpinChatMessage final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  int53 message_id_;
  static jfieldID message_id_fieldID;

  unpinChatMessage();

  unpinChatMessage(int53 chat_id_, int53 message_id_);

  static const std::int32_t ID = 2065448670;

  using ReturnType = object_ptr<ok>;

  static object_ptr<unpinChatMessage> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class chat;

class upgradeBasicGroupChatToSupergroupChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;

  upgradeBasicGroupChatToSupergroupChat();

  explicit upgradeBasicGroupChatToSupergroupChat(int53 chat_id_);

  static const std::int32_t ID = 300488122;

  using ReturnType = object_ptr<chat>;

  static object_ptr<upgradeBasicGroupChatToSupergroupChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class upgradeGiftResult;

class upgradeGift final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  string business_connection_id_;
  static jfieldID business_connection_id_fieldID;
  string received_gift_id_;
  static jfieldID received_gift_id_fieldID;
  bool keep_original_details_;
  static jfieldID keep_original_details_fieldID;
  int53 star_count_;
  static jfieldID star_count_fieldID;

  upgradeGift();

  upgradeGift(string const &business_connection_id_, string const &received_gift_id_, bool keep_original_details_, int53 star_count_);

  static const std::int32_t ID = -1782136103;

  using ReturnType = object_ptr<upgradeGiftResult>;

  static object_ptr<upgradeGift> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputFile;

class StickerFormat;

class file;

class uploadStickerFile final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 user_id_;
  static jfieldID user_id_fieldID;
  object_ptr<StickerFormat> sticker_format_;
  static jfieldID sticker_format_fieldID;
  object_ptr<InputFile> sticker_;
  static jfieldID sticker_fieldID;

  uploadStickerFile();

  uploadStickerFile(int53 user_id_, object_ptr<StickerFormat> &&sticker_format_, object_ptr<InputFile> &&sticker_);

  static const std::int32_t ID = 647385283;

  using ReturnType = object_ptr<file>;

  static object_ptr<uploadStickerFile> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class InputInvoice;

class orderInfo;

class validatedOrderInfo;

class validateOrderInfo final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<InputInvoice> input_invoice_;
  static jfieldID input_invoice_fieldID;
  object_ptr<orderInfo> order_info_;
  static jfieldID order_info_fieldID;
  bool allow_save_;
  static jfieldID allow_save_fieldID;

  validateOrderInfo();

  validateOrderInfo(object_ptr<InputInvoice> &&input_invoice_, object_ptr<orderInfo> &&order_info_, bool allow_save_);

  static const std::int32_t ID = -1248305201;

  using ReturnType = object_ptr<validatedOrderInfo>;

  static object_ptr<validateOrderInfo> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class MessageSource;

class ok;

class viewMessages final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 chat_id_;
  static jfieldID chat_id_fieldID;
  array<int53> message_ids_;
  static jfieldID message_ids_fieldID;
  object_ptr<MessageSource> source_;
  static jfieldID source_fieldID;
  bool force_read_;
  static jfieldID force_read_fieldID;

  viewMessages();

  viewMessages(int53 chat_id_, array<int53> &&message_ids_, object_ptr<MessageSource> &&source_, bool force_read_);

  static const std::int32_t ID = 960236656;

  using ReturnType = object_ptr<ok>;

  static object_ptr<viewMessages> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class PremiumFeature;

class ok;

class viewPremiumFeature final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  object_ptr<PremiumFeature> feature_;
  static jfieldID feature_fieldID;

  viewPremiumFeature();

  explicit viewPremiumFeature(object_ptr<PremiumFeature> &&feature_);

  static const std::int32_t ID = 192950706;

  using ReturnType = object_ptr<ok>;

  static object_ptr<viewPremiumFeature> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class viewSponsoredChat final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int53 sponsored_chat_unique_id_;
  static jfieldID sponsored_chat_unique_id_fieldID;

  viewSponsoredChat();

  explicit viewSponsoredChat(int53 sponsored_chat_unique_id_);

  static const std::int32_t ID = 1722644778;

  using ReturnType = object_ptr<ok>;

  static object_ptr<viewSponsoredChat> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class viewTrendingStickerSets final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  array<int64> sticker_set_ids_;
  static jfieldID sticker_set_ids_fieldID;

  viewTrendingStickerSets();

  explicit viewTrendingStickerSets(array<int64> &&sticker_set_ids_);

  static const std::int32_t ID = -952416520;

  using ReturnType = object_ptr<ok>;

  static object_ptr<viewTrendingStickerSets> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

class ok;

class writeGeneratedFilePart final : public Function {
  std::int32_t get_id() const final {
    return ID;
  }

 public:
  static jclass Class;
  int64 generation_id_;
  static jfieldID generation_id_fieldID;
  int53 offset_;
  static jfieldID offset_fieldID;
  bytes data_;
  static jfieldID data_fieldID;

  writeGeneratedFilePart();

  writeGeneratedFilePart(int64 generation_id_, int53 offset_, bytes const &data_);

  static const std::int32_t ID = 214474389;

  using ReturnType = object_ptr<ok>;

  static object_ptr<writeGeneratedFilePart> fetch(JNIEnv *env, jobject &p);

  void store(TlStorerToString &s, const char *field_name) const final;

  static ReturnType fetch_result(JNIEnv *env, jobject &p);

  static void init_jni_vars(JNIEnv *env);
};

}  // namespace td_api
}  // namespace td
